const axios = require('axios');
const ms = require('ms');
const { getMetadata } = require('./vault/metadata-store');
const { MOUNT_TARGET_SUFFIX, MOUNT_SOURCE_SUFFIX } = require('./consts');
const getLogger = require('./logger/logger');

const logger = getLogger();

const PORT = 8080;
const TIMEOUT = ms('10s');

async function sendAccessListRequest(source, target, aclServer, relationshipId) {
    try {
        const sourceCredentials = await getMetadata(`${relationshipId}.${MOUNT_SOURCE_SUFFIX}.cifs`);
        const targetCredentials = await getMetadata(`${relationshipId}.${MOUNT_TARGET_SUFFIX}.cifs`);
        const sourceToSend = { ...source.cifs, ...sourceCredentials };
        const targetToSend = { ...target.cifs, ...targetCredentials };

        logger.info(`Sending command to SMB ACL server: ${aclServer}, src:`, JSON.stringify(sourceToSend), 'trg:', JSON.stringify(targetToSend));

        await axios.post(`http://${aclServer}:${PORT}/api/copy-acl`, {
            source: sourceToSend,
            target: targetToSend
        }, {
            timeout: TIMEOUT
        });

        logger.info('Command to SMB ACL server successfully sent');
    } catch (err) {
        logger.error(`Failed to send command to SMB ACL server. Error:${err.response.status} - ${err.response.statusText}`);
    }
}

module.exports = sendAccessListRequest;
