const AWS = require('aws-sdk');
const getLogger = require('../logger/logger');

const logger = getLogger();

function getKms(region) {
    return new AWS.KMS({ region });
}

async function listKmsAliases(region) {
    try {
        const kms = getKms(region);
        const data = await kms.listAliases({}).promise();
        return data.Aliases;
    } catch (err) {
        logger.error('Failed to receive kms keys list', err);
        return [];
    }
}

async function listAllKmsAliases(region, nextMarker, keys = []) {
    try {
        const kms = getKms(region);
        const data = await kms.listAliases({ Marker: nextMarker }).promise();
        const aliases = keys.concat(data.Aliases);
        if (data.Truncated) {
            return listAllKmsAliases(region, data.NextMarker, aliases);
        }
        return aliases;
    } catch (err) {
        logger.error('Failed to receive kms keys list', err);
        return [];
    }
}

async function describeKmsKey(region, keyId) {
    const kms = getKms(region);
    const data = await kms.describeKey({ KeyId: keyId }).promise();
    return data.KeyMetadata;
}

module.exports = { listKmsAliases, describeKmsKey, listAllKmsAliases };
