const AWS = require('aws-sdk');
const getLogger = require('../logger/logger');

const logger = getLogger();

const metadataService = new AWS.MetadataService();

async function getMetadata(url) {
    return new Promise((resolve, reject) => metadataService.request(url, (err, data) => (err ? reject(err) : resolve(data))));
}

async function getNetworkMetadata() {
    const mac = await getMetadata('/latest/meta-data/mac');

    const [subnetId, subnetCidrBlock, vpcId, vpcCidrBlock] = await Promise.all([
        getMetadata(`/latest/meta-data/network/interfaces/macs/${mac}/subnet-id`),
        getMetadata(`/latest/meta-data/network/interfaces/macs/${mac}/subnet-ipv4-cidr-block`),
        getMetadata(`/latest/meta-data/network/interfaces/macs/${mac}/vpc-id`),
        getMetadata(`/latest/meta-data/network/interfaces/macs/${mac}/vpc-ipv4-cidr-block`)
    ]);

    return {
        subnetId, subnetCidrBlock, vpcId, vpcCidrBlock
    };
}

async function getAwsMetadata() {
    const [document, instanceId, {
        subnetId, subnetCidrBlock, vpcId, vpcCidrBlock
    }] = await Promise.all([
        getMetadata('/latest/dynamic/instance-identity/document'),
        getMetadata('/latest/meta-data/instance-id'),
        getNetworkMetadata()
    ]);

    const { region, availabilityZone } = JSON.parse(document);

    const awsMetadataObj = {
        region,
        availabilityZone,
        instanceId,
        subnetId,
        subnetCidrBlock,
        vpcId,
        vpcCidrBlock
    };

    let publicIp;
    try {
        publicIp = await getMetadata('/latest/meta-data/public-ipv4');
        awsMetadataObj.publicIp = publicIp;
    } catch (err) {
        logger.info('Couldn\'t get public IP of AWS databroker');
    }

    return awsMetadataObj;
}

module.exports = getAwsMetadata;
