const { rmdir, mkdir } = require('fs/promises');
const level = require('level');

async function putIntoDB(DB, objects) {
    // Manual JSON.stringify (optimization)
    return DB.batch(objects.map(object => ({ type: 'put', key: object.path, value: object.etag ? `{ "path": "${object.path}", "etag": "${object.etag}", "size": ${object.size} }` : `{ "path": "${object.path}", "size": ${object.size}, "mtimeSec": "${object.mtimeSec}" }` })));
}

async function removeFromDB(DB, objects) {
    return DB.batch(objects.map(object => ({ type: 'del', key: object.path })));
}

async function readFromDB(DB, minimumValue, limit = 1000) {
    return new Promise((resolve, reject) => {
        const objects = [];
        DB.createValueStream({ limit, gt: minimumValue })
            .on('data', data => objects.push(JSON.parse(data)))
            .on('error', err => reject(err))
            .on('end', () => resolve(objects));
    });
}

async function openDB(dbPath) {
    await mkdir(dbPath, { recursive: true });
    return level(dbPath);
}

async function closeDB(DB, dbPath) {
    await DB.close();
    return rmdir(dbPath, { maxRetries: 3, recursive: true });
}

module.exports = {
    putIntoDB, removeFromDB, readFromDB, closeDB, openDB
};
