const { get } = require('lodash');
const { getRuntimeConfiguration, getRuntimeAbout } = require('./phoenix/metadata');

const TYPE_FILE = 'FILE';
const TYPE_DIRECTORY = 'DIRECTORY';
const TYPE_SYMBOLIC = 'SYMBOLIC';
const TYPE_OTHER = 'OTHER';

const SCAN_MODE_SOURCE_ONLY = 'SCAN_MODE_SOURCE_ONLY';
const SCAN_MODE_TARGET_ONLY = 'SCAN_MODE_TARGET_ONLY';
const SCAN_MODE_SOURCE_AND_TARGET = 'SCAN_MODE_SOURCE_AND_TARGET';
const SCAN_MODE_CLOUD_TO_CLOUD = 'SCAN_MODE_CLOUD_TO_CLOUD';

const ACTION_COPY = 'COPY';
const ACTION_DELETE_SOURCE = 'DELETE_SOURCE';
const ACTION_DELETE_TARGET = 'DELETE_TARGET';
const ACTION_GRACE = 'GRACE';
const ACTION_IGNORE = 'IGNORE';
const ACTION_ABORT = 'ABORT';
const ACTION_LIST = 'LIST';

const ACTION_DELETE_FAILED = 'DELETE_FAILED';
const ACTION_COPY_FAILED = 'COPY_FAILED';

const ABORT_ERROR = 'RequestAbortedError';

const IPC_TYPE_COPY = 'copy';
const IPC_TYPE_REMOVE = 'remove';
const IPC_TYPE_ABORT = 'abort';
const IPC_TYPE_DONE = 'done';
const IPC_TYPE_UPDATE = 'update';

const SQS_DEFAULT_MESSAGE_VISIBILITY_TIMEOUT = '5m';
const SQS_CHANGE_MESSAGE_VISIBILITY_TIMEOUT_INTERVAL = '1m';
const SQS_MESSAGE_UNCHANGED_VISIBILITY_TIMEOUT_LIMIT = '3m';

const SQS_DEFAULT_WAIT_TIMEOUT = 20; // In seconds
const SQS_MAX_NUMBER_OF_MESSAGES = 10;
const SQS_MAX_CHARS_PER_MESSAGE = 131077;
const SQS_DEFAULT_REGION = 'us-east-1';
const OSS_DEFAULT_REGION = 'oss-me-east-1';

const WORKER_TYPE_SCANNER = 'SCANNER';
const WORKER_TYPE_TRANSFERRER = 'TRANSFERRER';
const WORKER_TYPE_SERVER = 'SERVER';
const WORKER_TYPE_ACL_SERVER = 'ACL-SERVER';

const STATUS_SUCCEEDED = 'SUCCEEDED';
const STATUS_FAILED = 'FAILED';
const STATUS_IGNORED = 'IGNORED';

const SCAN_STEP = 'SCAN';
const LIST_STEP = 'LIST';
const TRANSFER_STEP = 'TRANSFER';
const INITIALIZE_STEP = 'INITIALIZE';

const MOUNT_SOURCE_SUFFIX = 'src';
const MOUNT_TARGET_SUFFIX = 'trg';
const MOUNT_EXPORTS_FOLDER = 'exports';
const MOUNT_SHARES_FOLDER = 'shares';
const MOUNT_FOLDER = 'mnt';

const DATA_BROKER_RUNTIME_CONFIG = process.env.NODE_ENV === 'test' ? '' : getRuntimeConfiguration();
const DATA_BROKER_RUNTIME_ABOUT = getRuntimeAbout();

const CLEAN_RELATIONSHIP_RESOURCES_TIMEOUT = 30000;

const NFS_VERSIONS = [4.2, 4.1, 4.0, 3];
const CIFS_VERSIONS = [1.0, 2.0, 2.1, 3.0, 3.11];

const AWS_SQS_CREDENTIALS = {
    ACCESS_KEY_ID: get(DATA_BROKER_RUNTIME_CONFIG, 'aws.sqs.accessKeyId'),
    SECRET_ACCESS_KEY: get(DATA_BROKER_RUNTIME_CONFIG, 'aws.sqs.secretAccessKey')
};

const LOGS_BUCKET_NAME = 'metadata.datafabric.io';

const PUT_LOGS_ARN = 'arn:aws:iam::167338112540:role/cs_data_broker_put_logs';

const DEPLOYMENT_TYPES = {
    ONPREM: 'ONPREM',
    AWS: 'AWS',
    AZURE: 'AZURE',
    GCP: 'GCP'
};

const CERT_AND_KEY_FOLDER = 'certification';
const OPER_LIST_WITH_STATS = 'list-with-stats';
const OPER_GET_FILE = 'get-file';
const OPER_GET_SYMLINK = 'get-symlink';
const OPER_MOUNT = 'mount-endpoint';
const OPER_UMOUNT = 'umount-endpoint';
const OPER_SECURE_MKDIRS = 'secure-mkdirs';

const OPER_UPLOAD_FILE = 'upload-file';
const OPER_UPLOAD_SYMLINK = 'upload-symlink';
const OPER_UPLOAD_DIR = 'upload-dir';
const OPER_DELETE_FILE = 'delete-file';

const NETAPP_ENTRY_SUFFIX = '_netapp';
const NETAPP_ENTRY_RANDOM_SIZE = 7;
const NETAPP_ENTRY_RANDOM_PATTERN = 'Aa0';

const LOGS_DIRECTORY_RELATIVE_PATH = './logs';
const LOCAL_CONFIG_FILE = './config/local.json';

const S3_PROVIDER = 's3';
const ALIBABA_PROVIDER = 'alibaba';
const SGWS_PROVIDER = 'sgws';
const IBM_PROVIDER = 'ibm';
const ONTAP_S3_PROVIDER = 'ontap';

module.exports = {
    CERT_AND_KEY_FOLDER,
    OPER_SECURE_MKDIRS,
    TYPE_FILE,
    TYPE_DIRECTORY,
    TYPE_SYMBOLIC,
    TYPE_OTHER,
    SCAN_MODE_SOURCE_ONLY,
    SCAN_MODE_TARGET_ONLY,
    SCAN_MODE_SOURCE_AND_TARGET,
    SCAN_MODE_CLOUD_TO_CLOUD,
    IPC_TYPE_COPY,
    IPC_TYPE_REMOVE,
    IPC_TYPE_ABORT,
    IPC_TYPE_UPDATE,
    IPC_TYPE_DONE,
    SQS_DEFAULT_MESSAGE_VISIBILITY_TIMEOUT,
    SQS_CHANGE_MESSAGE_VISIBILITY_TIMEOUT_INTERVAL,
    SQS_MESSAGE_UNCHANGED_VISIBILITY_TIMEOUT_LIMIT,
    SQS_DEFAULT_WAIT_TIMEOUT,
    SQS_MAX_CHARS_PER_MESSAGE,
    SQS_MAX_NUMBER_OF_MESSAGES,
    SQS_DEFAULT_REGION,
    WORKER_TYPE_SCANNER,
    WORKER_TYPE_TRANSFERRER,
    WORKER_TYPE_ACL_SERVER,
    AWS_SQS_CREDENTIALS,
    DATA_BROKER_RUNTIME_CONFIG,
    DATA_BROKER_RUNTIME_ABOUT,
    ACTION_LIST,
    ACTION_COPY,
    ACTION_DELETE_SOURCE,
    ACTION_DELETE_TARGET,
    ACTION_IGNORE,
    ACTION_GRACE,
    ACTION_ABORT,
    ACTION_DELETE_FAILED,
    ACTION_COPY_FAILED,
    ABORT_ERROR,
    SCAN_STEP,
    LIST_STEP,
    TRANSFER_STEP,
    INITIALIZE_STEP,
    STATUS_FAILED,
    STATUS_SUCCEEDED,
    STATUS_IGNORED,
    DEPLOYMENT_TYPES,
    CLEAN_RELATIONSHIP_RESOURCES_TIMEOUT,
    MOUNT_SOURCE_SUFFIX,
    MOUNT_TARGET_SUFFIX,
    MOUNT_EXPORTS_FOLDER,
    MOUNT_SHARES_FOLDER,
    MOUNT_FOLDER,
    NFS_VERSIONS,
    CIFS_VERSIONS,
    OPER_UMOUNT,
    OPER_GET_SYMLINK,
    OPER_MOUNT,
    OPER_LIST_WITH_STATS,
    OPER_GET_FILE,
    OPER_DELETE_FILE,
    OPER_UPLOAD_DIR,
    OPER_UPLOAD_SYMLINK,
    OPER_UPLOAD_FILE,
    WORKER_TYPE_SERVER,
    NETAPP_ENTRY_SUFFIX,
    NETAPP_ENTRY_RANDOM_SIZE,
    NETAPP_ENTRY_RANDOM_PATTERN,
    OSS_DEFAULT_REGION,
    IBM_PROVIDER,
    SGWS_PROVIDER,
    ALIBABA_PROVIDER,
    ONTAP_S3_PROVIDER,
    S3_PROVIDER,
    LOGS_DIRECTORY_RELATIVE_PATH,
    LOGS_BUCKET_NAME,
    PUT_LOGS_ARN,
    LOCAL_CONFIG_FILE
};
