const Joi = require('@hapi/joi');
const {
    IPC_TYPE_COPY, IPC_TYPE_DONE, IPC_TYPE_REMOVE, IPC_TYPE_ABORT, IPC_TYPE_UPDATE
} = require('../../lib/consts');

const transferSettingsSchema = Joi.object().keys({
    retries: Joi.number().min(0).max(5).required()
});

const transferrerIncomingIPCCopySchema = Joi.object().keys({
    transferrerQueue: Joi.string().required(),
    relationshipId: Joi.string().required(),
    requestId: Joi.string().required(),
    type: Joi.string().valid(IPC_TYPE_COPY).required(),
    settings: transferSettingsSchema.required(),
    serverName: Joi.string(),
    certificate: Joi.string()
});

const transferrerIncomingIPCDoneRemoveAbortSchema = Joi.object().keys({
    type: Joi.string().valid(IPC_TYPE_DONE, IPC_TYPE_REMOVE, IPC_TYPE_ABORT, IPC_TYPE_UPDATE).required(),
    relationshipId: Joi.string().required(),
    source: Joi.string().required(),
    target: Joi.string().required()
});

const transferrerIncomingIPCMessagesSchemes = {
    copy: transferrerIncomingIPCCopySchema,
    remove: transferrerIncomingIPCDoneRemoveAbortSchema,
    abort: transferrerIncomingIPCDoneRemoveAbortSchema,
    done: transferrerIncomingIPCDoneRemoveAbortSchema,
    update: transferrerIncomingIPCDoneRemoveAbortSchema
};

module.exports = transferrerIncomingIPCMessagesSchemes;
