const AWS = require('aws-sdk-mock');
const { listAllKmsAliases } = require('../../../src/lib/aws/kms');

describe('list all kms keys - s3', () => {
    const listAliasFunction = (params, callback) => {
        let result;
        if (params.Marker) {
            result = {
                Truncated: false,
                Aliases: [{
                    TargetKeyId: 'id1',
                    AliasName: 'alias name',
                    AliasArn: 'alias arn'
                },
                {
                    TargetKeyId: 'id2',
                    AliasName: 'alias name',
                    AliasArn: 'alias arn1'
                }]
            };
        } else {
            result = {
                NextMarker: '1',
                Truncated: true,
                Aliases: [{
                    TargetKeyId: 'id4',
                    AliasName: 'alias name',
                    AliasArn: 'alias arn'
                },
                {
                    TargetKeyId: 'id3',
                    AliasName: 'alias name',
                    AliasArn: 'alias arn1'
                }]
            };
        }
        callback(null, result);
    };

    afterEach(() => {
        AWS.restore('KMS', 'listAliases');
    });

    test('list kms keys with recursive calls', async () => {
        AWS.mock('KMS', 'listAliases', listAliasFunction);
        const listKmsResult = await listAllKmsAliases('us-east-1');

        expect(listKmsResult).toEqual(expect.arrayContaining([
            {
                TargetKeyId: 'id2',
                AliasName: 'alias name',
                AliasArn: 'alias arn1'
            },
            {
                TargetKeyId: 'id1',
                AliasName: 'alias name',
                AliasArn: 'alias arn'
            },
            {
                TargetKeyId: 'id3',
                AliasName: 'alias name',
                AliasArn: 'alias arn1'
            },
            {
                TargetKeyId: 'id4',
                AliasName: 'alias name',
                AliasArn: 'alias arn'
            }]));
    });
});
