const AWS = require('aws-sdk-mock');
const { receiveParsedMessages } = require('../../../src/lib/aws/sqs');

describe('receiveMessage', () => {
    afterEach(() => AWS.restore('SQS', 'receiveMessage'));

    test('receiveMessage - no messages', async () => {
        AWS.mock('SQS', 'receiveMessage', (params, callback) => callback(null, {}));
        const messages = await receiveParsedMessages('');
        expect(messages).toHaveLength(0);
    });

    test('receiveMessage - no JSON message', async () => {
        AWS.mock('SQS', 'receiveMessage', (params, callback) => callback(null, {
            Messages: [
                {
                    Body: 'I am not JSON',
                    Attributes: {
                        ApproximateReceiveCount: '1'
                    }
                }
            ]
        }));
        const messages = await receiveParsedMessages('');
        expect(messages).toHaveLength(0);
    });

    test('receiveMessage - mix', async () => {
        AWS.mock('SQS', 'receiveMessage', (params, callback) => callback(null, {
            Messages: [
                {
                    Body: 'I am not JSON',
                    Attributes: {
                        ApproximateReceiveCount: '1'
                    }
                },
                {
                    Body: JSON.stringify({ name: 'Donald' }),
                    ReceiptHandle: '1',
                    MessageAttributes: {
                        first: {
                            DataType: 'String',
                            StringValue: 'sasha'
                        },
                        last: {
                            DataType: 'String',
                            StringValue: 'korman'
                        }
                    },
                    Attributes: {
                        ApproximateReceiveCount: '1'
                    }
                }
            ]
        }));
        const messages = await receiveParsedMessages('');
        expect(messages).toHaveLength(1);
        expect(Object.keys(messages[0])).toContain('lastVisibilityTimeoutChange');
        expect(Object.keys(messages[0])).toContain('receiveTimeStamp');
        delete messages[0].lastVisibilityTimeoutChange;
        delete messages[0].receiveTimeStamp;
        expect(messages[0]).toEqual({
            body: { name: 'Donald' },
            receiptHandle: '1',
            attributes: { approximateReceiveCount: '1', first: 'sasha', last: 'korman' }
        });
    });
});
