const { cloneDeep } = require('lodash');
const sqsMasterOutgoingMessagesSchemes = require('../../../src/messages/master/outgoing-messages');

describe('masterStatsSchema validations ', () => {
    const masterStatsSchema = sqsMasterOutgoingMessagesSchemes.statistics;
    const validMasterStats = {
        relationshipId: '1',
        SCAN: {
            FAILED: {
                FILE: {
                    bytes: 0,
                    items: 0
                },
                DIRECTORY: {
                    bytes: 0,
                    items: 0
                },
                SYMBOLIC: {
                    bytes: 0,
                    items: 0
                }
            },
            DELETE_TARGET: {
                FILE: {
                    bytes: 0,
                    items: 0
                },
                DIRECTORY: {
                    bytes: 0,
                    items: 0
                },
                SYMBOLIC: {
                    bytes: 0,
                    items: 0
                }
            },
            COPY: {
                FILE: {
                    bytes: 3041752,
                    items: 256
                },
                DIRECTORY: {
                    bytes: 0,
                    items: 130
                },
                SYMBOLIC: {
                    bytes: 0,
                    items: 0
                }
            }
        },
        TRANSFER: {
            DELETE_TARGET: {
                FILE: {
                    bytes: 0,
                    items: 0
                },
                DIRECTORY: {
                    bytes: 0,
                    items: 0
                },
                SYMBOLIC: {
                    bytes: 0,
                    items: 0
                }
            },
            COPY: {
                FILE: {
                    bytes: 3010232,
                    items: 260
                },
                DIRECTORY: {
                    bytes: 0,
                    items: 123
                },
                SYMBOLIC: {
                    bytes: 0,
                    items: 0
                }
            },
            DELETE_FAILED: {
                FILE: {
                    bytes: 0,
                    items: 0
                },
                DIRECTORY: {
                    bytes: 0,
                    items: 0
                },
                SYMBOLIC: {
                    bytes: 0,
                    items: 0
                }
            },
            COPY_FAILED: {
                FILE: {
                    bytes: 3010232,
                    items: 260
                },
                DIRECTORY: {
                    bytes: 0,
                    items: 123
                },
                SYMBOLIC: {
                    bytes: 0,
                    items: 0
                }
            }
        }
    };

    test('masterStatsSchema valid command', () => expect(masterStatsSchema.validate(validMasterStats).error).toBeUndefined());

    test('masterStatsSchema FILE" must be an object', () => {
        const invalidMasterStats = cloneDeep(validMasterStats);
        invalidMasterStats.SCAN.DELETE_TARGET.FILE = -1;
        expect(masterStatsSchema.validate(invalidMasterStats).error.details[0])
            .toEqual(expect.objectContaining({ message: '"SCAN.DELETE_TARGET.FILE" must be of type object' }));
    });

    test('masterStatsSchema bytes must be larger than or equal to 0', () => {
        const invalidMasterStats = cloneDeep(validMasterStats);
        invalidMasterStats.SCAN.DELETE_TARGET.FILE.bytes = -1;
        expect(masterStatsSchema.validate(invalidMasterStats).error.details[0])
            .toEqual(expect.objectContaining({ message: '"SCAN.DELETE_TARGET.FILE.bytes" must be larger than or equal to 0' }));
    });
    test('masterStatsSchema bytes" is required', () => {
        const invalidMasterStats = cloneDeep(validMasterStats);
        delete invalidMasterStats.SCAN.DELETE_TARGET.FILE.bytes;
        expect(masterStatsSchema.validate(invalidMasterStats).error.details[0])
            .toEqual(expect.objectContaining({ message: '"SCAN.DELETE_TARGET.FILE.bytes" is required' }));
    });

    test('masterStatsSchema items" must be larger than or equal to 0', () => {
        const invalidMasterStats = cloneDeep(validMasterStats);
        invalidMasterStats.SCAN.DELETE_TARGET.FILE.items = -1;
        expect(masterStatsSchema.validate(invalidMasterStats).error.details[0])
            .toEqual(expect.objectContaining({ message: '"SCAN.DELETE_TARGET.FILE.items" must be larger than or equal to 0' }));
    });

    test('masterStatsSchema one of [FILE, DIRECTORY, SYMBOLIC, OTHER]', () => {
        const invalidMasterStats = cloneDeep(validMasterStats);
        delete invalidMasterStats.SCAN.DELETE_TARGET.FILE;
        delete invalidMasterStats.SCAN.DELETE_TARGET.DIRECTORY;
        delete invalidMasterStats.SCAN.DELETE_TARGET.SYMBOLIC;
        expect(masterStatsSchema.validate(invalidMasterStats).error.details[0])
            .toEqual(expect.objectContaining({ message: '"SCAN.DELETE_TARGET" must contain at least one of [FILE, DIRECTORY, SYMBOLIC, OTHER]' }));
    });

    test('masterStatsSchema  one of [SCAN, TRANSFER]', () => {
        const invalidMasterStats = cloneDeep(validMasterStats);
        delete invalidMasterStats.SCAN;
        delete invalidMasterStats.TRANSFER;
        expect(masterStatsSchema.validate(invalidMasterStats).error.details[0])
            .toEqual(expect.objectContaining({ message: '"value" must contain at least one of [SCAN, TRANSFER, LIST]' }));
    });

    test('masterStatsSchema relationshipId is required', () => {
        const invalidMasterStats = cloneDeep(validMasterStats);
        delete invalidMasterStats.relationshipId;
        expect(masterStatsSchema.validate(invalidMasterStats).error.details[0])
            .toEqual(expect.objectContaining({ message: '"relationshipId" is required' }));
    });
});
