/**
 * Make any changes you need to make to the database here
 * and return promise
 * 'this' is the mongoose object
 */
const randomize = require('randomatic');
const crypto = require('crypto');
const IPA_SERIAL_NUMBER_PREFIX = '915';
const IPA_UNREGISTERED_STATUS = 'unregistered';
const notEncryptedEncoding = 'utf8';
const encryptedEncoding = 'hex';
const key = Buffer.from('5ebe2294ecd0e0f08eab7690d2a6ee69', encryptedEncoding);
const iv = Buffer.from('26ae5cc854e36b6bdfca366848dea6bb', encryptedEncoding);
const JWT_SECRET = 'E3%c_mA~8^779}u';


function generateSerialNumber() {
    return `${IPA_SERIAL_NUMBER_PREFIX}${randomize('0', 17)}`;
}

function decryptNssPassword(password) {
    const decipher = crypto.createDecipher('aes-256-ctr', JWT_SECRET);
    let dec = decipher.update(password, encryptedEncoding, notEncryptedEncoding);
    dec += decipher.final(notEncryptedEncoding);
    return dec;
}

function encryptNssPassword(password) {
    const cipher = crypto.createCipheriv('aes-128-cbc', key, iv);
    let crypted = cipher.update(password, notEncryptedEncoding, encryptedEncoding);
    crypted += cipher.final(encryptedEncoding);
    return crypted;
}

async function up() {
    const {db} = this.connection;
    const usersDb = db.collection('users');

    const usersWithoutSupport = await usersDb.find({}).toArray();

    const promises = usersWithoutSupport.map(async user => {
        try {
            const byolNssCredentials = user.byol.nssCredentials;

            const support = {
                serialNumber: generateSerialNumber(),
                status: IPA_UNREGISTERED_STATUS
            };

            if (byolNssCredentials) {
                const decryptedNssPassword = decryptNssPassword(byolNssCredentials.nssPassword);
                const encryptedNssPassword = encryptNssPassword(decryptedNssPassword);

                const nssCredentials = {
                    nssUsername: byolNssCredentials.nssUsername,
                    nssPassword: encryptedNssPassword
                };

                await usersDb.update({_id: user._id}, {
                    $set: {nssCredentials},
                    $unset: {'byol.nssCredentials': undefined}
                });
            }

            await usersDb.update({_id: user._id}, {
                $set: {support}
            });
        } catch (err) {
            console.log(err); // eslint-disable-line no-console
        }
    });

    return Promise.all(promises);
}

/**
 * Make any changes that UNDO the up function side effects here (if possible)
 * and return promise
 * 'this' is the mongoose object
 */
async function down() {
    return this.model('User').countDocuments();
}

module.exports = {up, down};

