
/**
 * Make any changes you need to make to the database here
 * and return promise
 * 'this' is the mongoose object
 */
async function up() {
    const { db } = this.connection;
    const relationshipsDb = db.collection('relationships');

    // finding encrypted relationships with listener data broker on the target side
    const encryptedRelationshipsTargetListener = await relationshipsDb.find({'target.nfs.dataBrokerId': {$exists: true}}).toArray();
    await Promise.all(encryptedRelationshipsTargetListener.map(async relationship => {
        const { _id, dataBroker, target: { nfs: { dataBrokerId } } } = relationship;
        const encryption = {
            listener: {
                dataBrokerId,
                endpointType: 'trg'
            },
            initiator: {
                dataBrokerId: dataBroker,
                endpointType: 'src'
            }
        };
        // set encryption object in the relationship
        await relationshipsDb.update({ _id }, {$set: {encryption}});
        // change provider to nfs instead of snfs.
        await relationshipsDb.update({_id}, {$set: {'target.nfs.provider': 'nfs'}});
        await relationshipsDb.update({_id}, {$set: {'source.nfs.provider': 'nfs'}});
        // remove data broker id from target
        return relationshipsDb.update({ _id }, {$unset: {'target.nfs.dataBrokerId': ''}});
    }));

    // finding encrypted relationships with listener data broker on the source side
    const encryptedRelationshipsSourceListener = await relationshipsDb.find({'source.nfs.dataBrokerId': {$exists: true}}).toArray();

    await Promise.all(encryptedRelationshipsSourceListener.map(async relationship => {
        const { _id, dataBroker, source: { nfs: { dataBrokerId } } } = relationship;
        const encryption = {
            listener: {
                dataBrokerId,
                endpointType: 'src'
            },
            initiator: {
                dataBrokerId: dataBroker,
                endpointType: 'trg'
            }
        };
        // set encryption object in the relationship
        await relationshipsDb.update({ _id }, {$set: {encryption}});
        // change provider to nfs instead of snfs.
        await relationshipsDb.update({_id}, {$set: {'source.nfs.provider': 'nfs'}});
        await relationshipsDb.update({_id}, {$set: {'target.nfs.provider': 'nfs'}});
        // remove data broker id from source
        return relationshipsDb.update({ _id }, {$unset: {'source.nfs.dataBrokerId': ''}});
    }));
}

/**
 * Make any changes that UNDO the up function side effects here (if possible)
 * and return promise
 * 'this' is the mongoose object
 */
async function down() {
    return this.model('User').countDocuments();
}

module.exports = { up, down };

