
/**
 * Make any changes you need to make to the database here
 * and return promise
 * 'this' is the mongoose object
 */
async function up() {
    const { db } = this.connection;
    const timelineDb = db.collection('timelinev2');

    // finding timelines with relationship, that the listener data broker was at the target
    const encryptedTimelineRelationshipsTarget = await timelineDb.find({'relationship.target.nfs.dataBrokerId': {$exists: true}}).toArray();
    await Promise.all(encryptedTimelineRelationshipsTarget.map(async timeline => {
        const { _id, relationship: { dataBrokerId: initiatorDataBrokerId, target: { nfs: { dataBrokerId } } } } = timeline;
        const encryption = {
            listener: {
                dataBrokerId,
                endpointType: 'trg'
            },
            initiator: {
                dataBrokerId: initiatorDataBrokerId,
                endpointType: 'src'
            }
        };
        // set encryption in the relationship of the timeline
        await timelineDb.update({ _id }, {$set: {'relationship.encryption': encryption}});
        // change provider to nfs instead of snfs.
        await timelineDb.update({_id}, {$set: {'relationship.target.nfs.provider': 'nfs'}});
        await timelineDb.update({_id}, {$set: {'relationship.source.nfs.provider': 'nfs'}});
        // remove data broker id from target
        return timelineDb.update({ _id }, {$unset: {'relationship.target.nfs.dataBrokerId': ''}});
    }));

    // finding encrypted relationships with listener data broker on the source side
    const encryptedTimelineRelationshipsSource = await timelineDb.find({'relationship.source.nfs.dataBrokerId': {$exists: true}}).toArray();

    await Promise.all(encryptedTimelineRelationshipsSource.map(async timeline => {
        const { _id, relationship: { dataBrokerId: initiatorDataBrokerId, source: { nfs: { dataBrokerId } } } } = timeline;
        const encryption = {
            listener: {
                dataBrokerId,
                endpointType: 'src'
            },
            initiator: {
                dataBrokerId: initiatorDataBrokerId,
                endpointType: 'trg'
            }
        };
        // set encryption in the relationship of the timeline
        await timelineDb.update({ _id }, {$set: {'relationship.encryption': encryption}});
        // change provider to nfs instead of snfs.
        await timelineDb.update({_id}, {$set: {'relationship.source.nfs.provider': 'nfs'}});
        await timelineDb.update({_id}, {$set: {'relationship.target.nfs.provider': 'nfs'}});
        // remove data broker id from source
        return timelineDb.update({ _id }, {$unset: {'relationship.source.nfs.dataBrokerId': ''}});
    }));
}

/**
 * Make any changes that UNDO the up function side effects here (if possible)
 * and return promise
 * 'this' is the mongoose object
 */
async function down() {
    return this.model('User').countDocuments();
}

module.exports = { up, down };

