const mongoose = require('mongoose');
const config = require('config');
const getLogger = require('./utils/logger');
const migration = require('./utils/migration');

const logger = getLogger();

async function connectToMongo() {
    try {
        logger.info('Connecting to DB:', config.get('db'));
        await mongoose.connect(config.get('db'),
            {
                useUnifiedTopology:true,
                poolSize: 10000,
                autoIndex: true,
                useNewUrlParser: true,
                useFindAndModify: false,
                useCreateIndex: true
            });
        logger.info('Connected to db');
        mongoose.connection.on('error', err => logger.error('Database connection failure:', err));
    } catch (err) {
        logger.error('Failed connecting to mongo', err);
        process.exit(-1);
    }
}

async function connectAndMigrate() {
    await connectToMongo();
    try {
        logger.debug('Migrating DB');
        await migration.run('up', null);
    } catch (err) {
        logger.error('Failed migrating MongoDB:', err);
        process.exit(-1);
    }
}

module.exports = connectAndMigrate;
