const {SERVICE_REGION} = require('../../utils/consts');

function getSns(accessKeyId, secretAccessKey) {
    return new AWS.SNS({accessKeyId, secretAccessKey, region: SERVICE_REGION});
}

async function subscribe(topicArn, queueArn, accessKeyId, secretAccessKey) {
    const sns = getSns(accessKeyId, secretAccessKey);

    const data = await sns.subscribe({
        Protocol: 'sqs',
        TopicArn: topicArn,
        Endpoint: queueArn
    }).promise();

    return data.SubscriptionArn;
}

async function confirmSubscription(topicArn, token, accessKeyId, secretAccessKey) {
    const sns = getSns(accessKeyId, secretAccessKey);

    const data = await sns.confirmSubscription({
        Token: token,
        TopicArn: topicArn
    }).promise();

    return data.SubscriptionArn;
}

module.exports = {subscribe, confirmSubscription};
