const axios = require('axios');
const ms = require('ms');
const {get} = require('lodash');
const {AXIOS_RETRY} = require('../utils/consts');
const axiosRetry = require('axios-retry');

const getLogger = require('../utils/logger');
const logger = getLogger('axios');

axiosRetry(axios, {retries: AXIOS_RETRY, retryDelay: () => ms('5s')});

axios.default.interceptors.request.use(request => {
    logger.info(`AXIOS request: ${request.method.toUpperCase()} ${request.url}`, request.data ? JSON.stringify(request.data) : '');
    return request;
});

axios.default.interceptors.response.use(response => {
    logger.info(`AXIOS response: ${response.status} (${response.statusText}) - ${response.config.method.toUpperCase()} ${response.config.url}`, JSON.stringify(response.data));
    return response;
}, error => {
    // logger.error('Axios error', error); //DEBUG delete
    // 504 - received in case of service connector is not available
    if (get(error, 'response.status') !== 504) {
        logger.error(`AXIOS error :status ${error.code} ${error.response ? error.response.status : error.response} error : ${error.response ? JSON.stringify(error.response.data) : error.Error}`);
    }

    return Promise.reject(error);
});