const onFinished = require('on-finished');
const {isEmpty} = require('lodash');
const {sendAuditGroup} = require('../operations/audit-operations');
const TimelineV2 = require('../models/timeline-v2');

const getLogger = require('../utils/logger');

const logger = getLogger();

async function timelineMiddleware(req, res, next) {
    onFinished(res, async(err, res) => {
        const {req: {timelineV2}} = res;
        if (timelineV2) {
            if (!timelineV2.isModified()) {
                logger.warn(`Make sure you haven't forgot to update the timelineV2 for the action ${timelineV2.summary}`);
            }
            if (timelineV2.failureMessage) {
                timelineV2.status = 'FAILED';
            } else if (res.statusCode !== 202) { // when status is 202 it is an async request. wait for the message to update the status to done
                timelineV2.status = 'DONE';
            }

            timelineV2.markModified('status');
            const timeline = await timelineV2.save();
            await sendAuditGroup(timeline,  res.req.account.tenancyAccountId,
                undefined,
                (timelineV2.response || (timeline.relationship || timeline.dataBroker)));
        }
    });

    const {method, account} = req;
    try {
        const accountExists = !isEmpty(account);
        if (method !== 'GET' && accountExists) {
            const {swagger: {operation: {summary}}, account:{tenancyAccountId}, id: requestId, auth0Id, userName} = req;
            if (summary) {
                //saving on timelinev2
                req.timelineV2 = new TimelineV2({account:account.id, requestId, summary, auth0Id, userName});
                const timeline = await req.timelineV2.save();
                await sendAuditGroup(timeline, tenancyAccountId, req.body);
            }
        }
    } catch (err) {
        logger.error(`Unable to create a timeline for account: ${account.tenancyAccountId}, ${err}`);
    } finally {
        next();
    }
}

module.exports = timelineMiddleware;
