const mongoose = require('mongoose');
const {defaultTransform} = require('../utils/utils');

const ByolSchema = new mongoose.Schema({
    reservedRelationships: { type: Number, required: true, default: 0 },
    usedRelationships: { type: Number, required: true, default: 0 }
}, { _id: false });

const costSchema = new mongoose.Schema({
    account: {
        type: mongoose.Schema.Types.ObjectId, ref: 'Account', index: true
    },
    tier1: { type: Number, required: true, default: 0 },
    tier2: { type: Number, required: true, default: 0 },
    tier3: { type: Number, required: true, default: 0 },
    byol: { type: ByolSchema, required: true, default: { enabledRelationship: 0, usedRelationship: 0 } },
    timestamp: { type: Date, required: true }
}, {
    timestamps: true,
    toJSON: {
        transform: defaultTransform((doc, ret) => ret)
    }
});

const Cost = mongoose.model('Cost', costSchema);

// Returns the latest cost records from the table
async function lastCostRecords() {
    return Cost.aggregate([
        {$sort: {timestamp: -1}},
        {
            $group: {
                _id: '$account',
                tier1: {$first: '$tier1'},
                tier2: {$first: '$tier2'},
                tier3: {$first: '$tier3'},
                byol: {$first: '$byol'},
                lastDate: {$max: '$timestamp'}
            }
        }]);
}


module.exports = {Cost, lastCostRecords};

