const {escape} = require('querystring');
const express = require('express');
const {
    API_PATH_MARKETPLACE_AZURE_LANDING, API_PATH_MARKETPLACE_AZURE_EVENTS, SAAS_SUBSCRIPTION_PROVIDER_AZURE, SAAS_REGISTRATION_STATUS_UNSUBSCRIBE_SUCCESS
} = require('../utils/consts');
const getLogger = require('../utils/logger');
const {Account} = require('../models/account');
const {createMessage} = require('../models/message-v2');

const router = express.Router();
const logger = getLogger();


router.get(`${API_PATH_MARKETPLACE_AZURE_LANDING}`, (req, res) => {
    logger.info('Raw token:', req.query.token);
    const token = escape(req.query.token);
    logger.info('Escaped token:', token);

    if (token) {
        res.redirect(`/?rt=${token}&provider=${SAAS_SUBSCRIPTION_PROVIDER_AZURE}`);
    } else {
        logger.warn('Landed on landing page URL without a token');
        res.redirect('/');
    }
});

router.post(`${API_PATH_MARKETPLACE_AZURE_EVENTS}`, async(req, res) => {
    const {body: {status, action, subscriptionId}} = req;
    logger.info('Got AZURE marketplace event:', req.body);

    // TODO-SK we should check for status === 'Succeeded' as well, but currently only 'InProgress' is received
    if (action === 'Unsubscribe') {
        try {
            const accounts = await Account.find({'subscription.customerIdentifier': subscriptionId}).exec();

            await Account.updateMany({
                'subscription.customerIdentifier': subscriptionId
            },
            {
                $set: {
                    'subscription.status': SAAS_REGISTRATION_STATUS_UNSUBSCRIBE_SUCCESS,
                    'subscription.updatedAt': new Date()
                },
                $unset: {
                    'subscription.customerIdentifier': ''
                }
            }).exec();

            accounts.forEach(account => {
                const {id:accountId, tenancyAccountId} = account;
                logger.info('Sending update subscription status to account:', accountId);
                return createMessage(accountId, undefined, undefined, {'update-subscription-status': {
                    accountId: tenancyAccountId,
                    subscriptionStatus: SAAS_REGISTRATION_STATUS_UNSUBSCRIBE_SUCCESS
                }});
            });

        } catch (err) {
            logger.error(`Failed to unsubscribe AZURE subscription ${subscriptionId}`, err);
        }
    } else {
        logger.warn('Ignoring AZURE marketplace event, status:', status);
    }

    res.status(200).end();
});


module.exports = router;
