const Joi = require('@hapi/joi');
const {IPA_SERVICE_PWD} = require('../utils/consts');

const verifyNssCredentialsRequestSchema = Joi.object().keys({
    licenseRequest: Joi.object().keys({
        version: Joi.string().required().allow(''),
        serialNumber: Joi.string().required().valid(''),
        op: Joi.string().required(),
        package: Joi.string().required().valid('')
    })
});

const statusResponseSchema = Joi.object().keys({
    statusResp: Joi.object().keys({
        statusCode: Joi.string().required(),
        message: Joi.string().required()
    })
});

const registrationUserRequestSchema = Joi.object().keys({
    serviceUserID: Joi.string().required(),
    servicePwd: Joi.string().required().valid(IPA_SERVICE_PWD),
    nssUserID: Joi.string().required(),
    awsAccountID: Joi.string().required(),
    awsVSAEC2InstanceID: Joi.string().required(),
    vsaSerialNumber: Joi.string().required(),
    vsaStartDate: Joi.number()
});

const registrationUserResponseSchema = Joi.object().keys({
    regResp: Joi.object().keys({
        statusCode: Joi.string().required(),
        message: Joi.string().required()
    })
});

const registrationUserStatusRequestSchema = Joi.object().keys({
    serviceUserID: Joi.string().required(),
    servicePwd: Joi.string().required().valid(IPA_SERVICE_PWD),
    vsaSerialNumber: Joi.string().required()
});

const deRegistrationUserRequestSchema = Joi.object().keys({
    serviceUserID: Joi.string().required(),
    servicePwd: Joi.string().required().valid(IPA_SERVICE_PWD),
    vsaSerialNumber: Joi.string().required(),
    cpAccountID: Joi.string().required()
});

const deRegistrationUserResponseSchema = Joi.object().keys({
    deResp: Joi.object().keys({
        statusCode: Joi.string().required(),
        message: Joi.string().required()
    })
});

module.exports = {
    verifyNssCredentialsRequestSchema,
    statusResponseSchema,
    registrationUserRequestSchema,
    registrationUserResponseSchema,
    registrationUserStatusRequestSchema,
    deRegistrationUserRequestSchema,
    deRegistrationUserResponseSchema
};
