const Agenda = require('agenda');
const mongoose = require('mongoose');
const os = require('os');
const {
    AGENDA_TASK_INTERVAL_NAME_UPDATE_CUSTOMER_IDENTIFIER_ON_AWS_ACCOUNTS,
    AGENDA_TASK_INTERVAL_UPDATE_DATABROKER, AGENDA_TASK_NAME_UPDATE_CUSTOMER_IDENTIFIER_ON_AWS_ACCOUNTS,
    AGENDA_TASK_NAME_UPDATE_DATABROKER, AGENDA_TASK_NAME_SYNC_SCHEDULER, AGENDA_TASK_INTERVAL_BACKUP_DB, AGENDA_TASK_NAME_BACKUP_DB,
    AGENDA_TASK_INTERVAL_SAAS_METERING, AGENDA_TASK_NAME_AWS_SAAS_METERING, AGENDA_TASK_INTERVAL_SYNC_SCHEDULER,
    AGENDA_TASK_NAME_AZURE_SAAS_METERING, AGENDA_TASK_NAME_DELETE_NOT_ACTIVE_DATA_BROKERS, AGENDA_TASK_INTERVAL_DELETE_NOT_ACTIVE_DATABROKERS
} = require('../utils/consts');
const getLogger = require('../utils/logger');

const logger = getLogger();

const AGENDA_COLLECTION_NAME = 'jobs';
const AGENDA_PROCESS_EVERY = '10 seconds';

const agenda = new Agenda({
    name: `${os.hostname()}-${process.pid}`,
    processEvery: AGENDA_PROCESS_EVERY
});

agenda.on('ready', () => {
    agenda.every(AGENDA_TASK_INTERVAL_BACKUP_DB, AGENDA_TASK_NAME_BACKUP_DB);
    agenda.every(AGENDA_TASK_INTERVAL_SAAS_METERING, AGENDA_TASK_NAME_AWS_SAAS_METERING);
    agenda.every(AGENDA_TASK_INTERVAL_SAAS_METERING, AGENDA_TASK_NAME_AZURE_SAAS_METERING);
    agenda.every(AGENDA_TASK_INTERVAL_SYNC_SCHEDULER, AGENDA_TASK_NAME_SYNC_SCHEDULER);
    agenda.every(AGENDA_TASK_INTERVAL_UPDATE_DATABROKER, AGENDA_TASK_NAME_UPDATE_DATABROKER);
    agenda.every(AGENDA_TASK_INTERVAL_DELETE_NOT_ACTIVE_DATABROKERS, AGENDA_TASK_NAME_DELETE_NOT_ACTIVE_DATA_BROKERS);
    agenda.every(AGENDA_TASK_INTERVAL_NAME_UPDATE_CUSTOMER_IDENTIFIER_ON_AWS_ACCOUNTS, AGENDA_TASK_NAME_UPDATE_CUSTOMER_IDENTIFIER_ON_AWS_ACCOUNTS);
    agenda.start();
});

agenda.on('error', err => logger.error('Got agenda error', err));

async function initAgenda() {
    await new Promise((resolve, reject) => {
        mongoose.connection.db.collection(AGENDA_COLLECTION_NAME, (err, agendaCollection) => {
            if (err) {
                return reject(err);
            }
            resolve(agendaCollection.deleteMany({}));
        });
    });

    await agenda.mongo(mongoose.connection.db, AGENDA_COLLECTION_NAME, err => {
        if (err) {
            throw err;
        }
    });
}


module.exports = { agenda, initAgenda };
