const config = require('config');
const fs = require('fs');
const path = require('path');
const log4js = require('log4js');
const moment = require('moment');
const { first } = require('lodash');
const stackTrace = require('stack-trace');
const {getRequestId, getAccountId, getAction} = require('./async-local-storage');

const logsDirectory = 'logs';
if (!fs.existsSync(logsDirectory)) {
    fs.mkdirSync(logsDirectory);
}

const loggerConfig = JSON.parse(fs.readFileSync(path.resolve(__dirname, config.get('paths.log4js')), 'utf8'));

const tokens = {
    date: event => moment(event.startTime).format('Y-MM-DDTHH:mm:ss.SSSZZ'),
    requestId: () => getRequestId() || 'headless',
    who: () => {
        const accountId = getAccountId();
        return accountId || 'system';
    },
    action: () => getAction() || 'system'
};

Object.keys(loggerConfig.appenders).forEach(appenderType => {
    const { layout } = loggerConfig.appenders[appenderType];
    if (layout) {
        const { pattern } = layout;
        if (pattern) {
            layout.tokens = tokens;
        }
    }
});


log4js.configure(loggerConfig);

function getLogger(category) {
    const logger = log4js.getLogger(category);
    const trace = stackTrace.parse(new Error('something went wrong'));
    logger.addContext('module', trace[1].getFileName().replace(/^.*[\\\/]src[\\\/]/, '').split(/[\\\/]/).join('.'));
    logger.addContext('service', first(config.get('server.host').split('.')));
    return logger;
}

module.exports = getLogger;
