const ms = require('ms');
const {first} = require('lodash');
const {Message2} = require('../models/message-v2');
const getLogger = require('./logger');

const logger = getLogger();
logger.addContext('module', 'messages-poller-2');


async function poll2(account, lastMessageId, pollTimeout) {

    if (lastMessageId) {
        let interval;
        let timeout;

        const setIntervalTime = '3s';
        const setTimeoutTime = pollTimeout;

        const p1 = new Promise((resolve) => {
            timeout = setTimeout(async() => {
                clearInterval(interval);
                resolve([]);
            }, ms(setTimeoutTime));
        });

        const p2 = new Promise((resolve, reject) => {
            interval = setInterval(async() => {
                try {
                    const messages = await Message2.find({account, milliSeconds: {$gt: lastMessageId}}).exec();
                    if (messages.length) {
                        clearInterval(interval);
                        clearTimeout(timeout);
                        resolve(messages.map(message => {
                            const {payload} = message;
                            payload.id = message.milliSeconds;
                            return payload;
                        }));
                    }
                } catch (err) {
                    logger.error('Received error when trying to get messages', err);
                    clearInterval(interval);
                    clearTimeout(timeout);
                    reject(err);
                }
            }, ms(setIntervalTime));
        });

        return Promise.race([p1, p2]);

    } else {
        const messages = await Message2.find().sort({milliSeconds: -1}).limit(1).exec();
        const id = messages.length ? first(messages).milliSeconds : Date.now();
        return [{
            header: {
                requestId: 'init',
                clientId: 'init',
                agentId: 'init'
            },
            payload: {
                init: {}
            },
            id
        }];
    }
}

module.exports = poll2;
