const hyperid = require('hyperid');
const { isPlainObject, isString, isArray, isEmpty, trim, set } = require('lodash');
const {SECRET_KEYS} = require('../utils/consts');

function generateRequestId() {
    return hyperid().uuid;
}

function isEmptyString(str) {
    return isEmpty(trim(str));
}


function timeout(ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
}

const defaultTransform = f => (doc, ret, options) => {
    if (ret.createdAt) {
        ret.createdAt = ret.createdAt.getTime();
    }
    delete ret.updatedAt;
    delete ret.__v;
    delete ret._id;
    if (ret.id) {
        ret.id = doc.id;
    }
    return f(doc, ret, options);
};

function removeFirstSlash(path) {
    return (path) ? (path.startsWith('/') ? path.substr(1) : path) : '';
}

function hideSecretsHelper(obj, cache = []) {
    cache.push(obj); // We use cache in order to stop iterating when we find circular object
    if (isArray(obj)) {
        obj.forEach(arrayObj => hideSecretsHelper(arrayObj, cache));
    } else if (isPlainObject(obj)) {
        Object.keys(obj).forEach(key => {
            if ((isPlainObject(obj[key]) || isArray(obj[key])) && !cache.includes(obj[key])) {
                hideSecretsHelper(obj[key], cache);
            } else if (SECRET_KEYS.includes(key)) {
                set(obj, `${key}`, '******');
            } else if (key === 'azure' && isString(obj[key])) { // because on metadata message we have azure and not storageAccountConnectionString and we dont want to hide azure endPoint
                set(obj, `${key}`, '******');
            }
        });
    }
}


module.exports = {
    hideSecretsHelper, removeFirstSlash, defaultTransform, timeout, isEmptyString, generateRequestId
};
