import {AUTH_REFRESH_SSO} from "../store/global/action_creators_consts";
import * as actionCreators from '../store/global/action_creators';
import _ from "lodash";
import {decodeAll} from "../utils/encoders.js";
import {setAuthHeader} from "../sdk";

export default ({dispatch, getState}) => next => action => {
    if (action.type === AUTH_REFRESH_SSO) {
        const {location: {query, pathname}} = getState().router;
        const {_ui: {isCmIntegration}} = getState().global;
        const decodedQueryRt = _.get(query, 'rt') ? decodeAll(query.rt) : undefined;
        const cmIntegration = isCmIntegration || pathname === '/cm';

        const loginFail = (err) => {
            /* eslint-disable no-console */
            console.log("SSO return error");
            console.log(err);
            /* eslint-enable no-console */
            dispatch(actionCreators.setUserInfoFailed(err));
            if (!cmIntegration && err.needLogin) {
                //in case of cmIntegration we don't want to redirect to login, just to show a reload message (handled by render)
                //for any other error than "needLogin" we don't want to login again, we want to show the oops (handled later in the render)
                const rtQueryParam = decodedQueryRt;
                const providerQueryParam = query.provider;
                const cameFromMarketPlace = !_.isEmpty(rtQueryParam);
                const displayEvent = cameFromMarketPlace ? {
                    level: "message",
                    title: "Thank you for subscribing to Cloud Sync service",
                    message: "To complete the subscription, you must log in. New users should sign up."
                } : undefined;
                const paramsForUI = displayEvent ? {rt: rtQueryParam, provider: providerQueryParam} : undefined;
                window.auth.loginRedirect({}, {
                    startOnSignup: (query && query.su && query.su),
                    service: "cloud-sync",
                    displayEvent,
                }, paramsForUI);
            }
        };

        const loginSuccess = (authResult) => {
            /* eslint-disable no-console */
            console.log("SSO return success");
            console.log(authResult);
            /* eslint-disable no-console */
            const loginToken = `Bearer ${authResult.accessToken}`;
            const marketPlaceToken = authResult.paramsForUI.rt ? authResult.paramsForUI.rt : decodedQueryRt;
            const marketPlaceProvider = authResult.paramsForUI.provider ? authResult.paramsForUI.provider : query.provider;
            // eslint-disable-next-line no-console
            console.log(`>>>>>>>>>>>>>>> token:\n${loginToken} token payload:\n${JSON.stringify(authResult.idTokenPayload)}\nrtFromParamsForUI: ${marketPlaceToken}`);
            window.localStorage.auth0LoginToken = loginToken;
            setAuthHeader({authorization: loginToken});
            dispatch(actionCreators.setUserInfoSuccess({...authResult, accessToken: loginToken}));
            dispatch(actionCreators.getAccounts(loginToken, marketPlaceToken ? marketPlaceToken : undefined, marketPlaceProvider ? marketPlaceProvider : undefined)); //this needs the user token so must be after login
            dispatch(actionCreators.bootAnalytics({isCmIntegration: cmIntegration}));

            setTimeout(() => {
                window.auth.refreshSso().then(loginSuccess, loginFail);
            }, authResult.expiresIn * 1000 - 10000);
        };

        window.auth.refreshSso().then(loginSuccess, loginFail);
    }
    return next(action);
}

