import React, {useEffect} from 'react';
import {connect} from 'react-redux';
import * as actionCreators from '../store/wizard/action_creators';
import {removeNotification} from '../store/global/action_creators';
import {routesContainer} from '../store/global/action_creators_consts';
import Header from './header/Header';
import Footer from './footer/Footer';
import {buildUrlParams} from "../utils/encoders";
import {getPropFromState, buildErrorsList} from "../utils/mapStateToPropsUtils";
import NotificationPanel from './notifications/NotificationPanel';
import useRunOnce from '../hooks/useRunOnce';

const ContentWrapper = (props) => {
    const {_routing, match, location, updateUrlParams, _ui} = props;
    const {blocked: isUiBlocked} = _ui;

    useRunOnce(() => {
        const urlParams = buildUrlParams(_routing, match, location, window.location.href);
        updateUrlParams(urlParams);
    });

    useEffect(() => {
        if (_routing?.href !== window.location.href) {
            const urlParams = buildUrlParams(_routing, match, location, window.location.href);
            updateUrlParams(urlParams);
        }
    }, [_routing, match, location, updateUrlParams]);

    useEffect(() => {
        //enforcement - when UI is blocked - don't allow any path but License Settings
        if (!isUiBlocked) return;
        const currentPath = window.location.pathname;
        const isRoutedToLicenseScreen = currentPath === "/license";
        if (isUiBlocked && !isRoutedToLicenseScreen) {
            routesContainer.routes.replace("/license");
        }
    }, [isUiBlocked, _routing]);


    const { ContentHeader, Content, PrimaryHeader, contentKey, contentUiState, hideMainHeader, _notifications, _generalErrors} = props;
    const {removeError, removeNotification} = props;
    const {isCmIntegration} = _ui;
    const {_errors} = _generalErrors;

    let uiState = contentUiState;
    const component = {
        content: <Content {...props}/>,
        primaryHeader: PrimaryHeader ? <PrimaryHeader {...props}/> : undefined,
        header: ContentHeader ? <ContentHeader {...props}/> : undefined,
        key: contentKey
    };

    if (props._ui.states && props._ui.states.length > 0) {
        uiState = props._ui.states[props._ui.states.length - 1];
    }

    if (hideMainHeader || isCmIntegration) {
        uiState = uiState + " no-portal-widget";
    }

    if (isCmIntegration) {
        uiState = uiState + " cm-integration";
    }

    //TODO improve
    document.body.className = uiState;

    return (
        <div className="content-wrapper">

            {!hideMainHeader && !isCmIntegration &&
            <Header {...props}>
                {component.primaryHeader && !_ui.blocked && <PrimaryHeader/>}
            </Header>}

            {hideMainHeader && !isCmIntegration &&
                (component.primaryHeader && !_ui.blocked && <PrimaryHeader/>)
            }

            <section id="primary">
                <header className="secondary">
                    {component.header && <ContentHeader/>}
                </header>
                <section className="content">
                    {component.content}
                </section>
            </section>

            {!isCmIntegration && <div className="footer-container">
                <Footer {...props}/>
            </div>}

            <NotificationPanel
                errors={_errors}
                notifications={_notifications}
                removeError={removeError}
                removeNotification={removeNotification}/>
        </div>
    )
};

const mapStateToProps = (state, ownProps) => ({
    auth: state.global.auth || {succeeded: false, inProgress: true, failed: false}, //for the footer
    _notifications: state.global._notifications,
    _generalErrors: buildErrorsList(state),
    _routing: getPropFromState(state, "_routing", {}),
    _ui: {...state.syncReducer._ui, ...state.global._ui},
    _licenseSettings: state.global._licenseSettings, //for the footer
    ...ownProps
});

export default connect(mapStateToProps, {...actionCreators, removeNotification})(ContentWrapper);