import React, {Component} from 'react';
import classNames from "classnames";
import Popover from "react-popover";
import "./popover.scss";

export default class ReactPopover extends Component {
    constructor(props, context) {
        super(props, context);
        this.state = {isOpen: false};
        this.setWrapperRef = this.setWrapperRef.bind(this);
        this.handleClickOutside = this.handleClickOutside.bind(this);
    }

    toggle(toState = null) {
        this.setState({isOpen: toState === null ? !this.state.isOpen : toState})
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutside);
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutside);
    }

    /**
     * Set the wrapper ref
     */
    setWrapperRef(node) {
        this.wrapperRef = node;
    }

    /**
     * Alert if clicked on outside of element
     */
    handleClickOutside(event) {
        if (this.wrapperRef && !this.wrapperRef.contains(event.target)) {
            this.setState({isOpen: false})
        }
    }


    render() {
        const {isOpen} = this.state;
        const {container, popoverClass, children, preferPlace, containerClassName, trigger, interactive} = this.props;
        const triggerProps = trigger === "mouseover" ? {
            onMouseOver: () => this.toggle(true),
            onMouseOut: () => this.toggle(false)
        } : {
            onClick: () => this.toggle()
        };

        const triggerElement = React.cloneElement(container, triggerProps);
        const tooltipProps = interactive ? triggerProps : {};
        const popoverBody = (
            <div className="popover-container" ref={this.setWrapperRef} {...tooltipProps}>
                <div className={classNames("react-popover", popoverClass)}>
                    <div className="content">
                        {children}
                    </div>
                </div>
            </div>
        );

        return (
            <Popover isOpen={isOpen}
                     body={popoverBody}
                     preferPlace={preferPlace ? preferPlace : "below"}
                     enterExitTransitionDurationMs={0}
                     className={popoverClass} {...triggerProps} >
            <span className={classNames("popover-trigger-container", {active: isOpen}, containerClassName)}>
                {triggerElement}
                </span>
            </Popover>

        );
    }
}

