import React, {Component} from "react";
import classNames from "classnames";
import ReactDOM from "react-dom"
import Hamburger from "./../widgets/Hamburger";
import {CSSTransition} from "react-transition-group"
import _ from "lodash";

export default class extends Component{
    constructor(props, context){
        super(props, context);
        this.state = {};
    }

    UNSAFE_componentWillUpdate(newProps){
        if(!newProps.menuopen && this.props.menuopen){
            this._menuNode.style.right = "-" + this._menuNode.getBoundingClientRect().width+"px";
        }
    }

    drawMenuCallback(el) {
        if(!el) return;

        const node = ReactDOM.findDOMNode(el);
        node.style.right = "-" + node.getBoundingClientRect().width+"px";
        this._menuNode = node;
        setTimeout(()=>{
            node.style.transition = "right 0.4s cubic-bezier(.25,.8,.25,1)";
            node.style.right = "";
        })
    }

    render(){
        const {Card, data, selected, selector, menuItems, isMenuDisabled, menuopen, triggerMenu, additionalData} = this.props;
        const isSelectionDisabled = !selector || (_.isFunction(Card.isSelectionDisabled) ? Card.isSelectionDisabled(data) : false);

        const disableMenu = !menuItems || (_.isFunction(isMenuDisabled) ? isMenuDisabled(data) : false);
        const cardClass = classNames({
            selectable: !isSelectionDisabled,
            selected,
            menuable: !!menuItems,
            "menu-open": menuopen
        }, _.isFunction(Card.className)? Card.className(data) : Card.className);

        return (
            <div className={cardClass}>
                {!disableMenu && <div className="card-menu">
                    <Hamburger className={classNames({open: menuopen})} onClick={triggerMenu}></Hamburger>
                    <CSSTransition classNames="menu" timeout={400}>
                        {menuopen ?
                            <ul ref={this.drawMenuCallback.bind(this)}>
                                {menuItems(data, triggerMenu).map( (item, index) => (<li key={index}>{item}</li>))}
                            </ul>
                            :
                            <div></div>
                        }
                    </CSSTransition>
                </div>}
                {selector && !isSelectionDisabled && <button className="card-body" onClick={() => selector(data.id)}>{Card.renderContent(data, additionalData)}</button>}
                {(!selector || isSelectionDisabled) && <div className="card-body" disabled={isSelectionDisabled}>{Card.renderContent(data, additionalData)}</div>}
            </div>);
    }
}
