import React, {useState, useMemo, useCallback} from "react";
import classNames from "classnames";
import SearchWidget from "./../widgets/SearchWidget"
import ViewToggleWidget from "./../widgets/ViewToggleWidget"
import ItemCountWidget from "./../widgets/ItemCountWidget"
import {defaultFilterWithoutKeys} from "./../../utils/helpers"
import {parentHasClass} from "./../../utils/dom"
import CardItem from "./CardItem";
import useEvent from '../../hooks/useEvent';

require("./cardsContainer.scss");

const CardsContainer = (props) => {
    const {className, data, Card, wizardType, containerOptions, selectionOptions = {}, menuItems, isMenuDisabled, additionalData} = props;
    const {singularTitle, pluralTitle, viewToggle, noWidgets = false, keysToExcludeFromSearch} = containerOptions;

    const [filter, setFilter] = useState(null);
    const [openedCardId, setOpenedCardId] = useState(null);

    const eventListener = useCallback((el) => {
        if (parentHasClass(el.target, "card-container")) return;
        setOpenedCardId(null);
    }, [setOpenedCardId]);

    useEvent("click", eventListener);

    const triggerMenu = (id) => {
        if (openedCardId === id) setOpenedCardId(null);
        else setOpenedCardId(id);
    };

    const filteredItems = useMemo(() => filter
        ? data.filter(item => defaultFilterWithoutKeys(item, filter, keysToExcludeFromSearch))
        : data, [filter, data, keysToExcludeFromSearch]);

    return (
        <div className={classNames(className, "widgets-cards-container")}>
            {!noWidgets && <div className="clearfix widgets-container">
                <ItemCountWidget className="float-left" singularTitle={singularTitle} pluralTitle={pluralTitle}
                                 itemCount={data.length} filteredItemCount={filteredItems.length}/>
                {viewToggle && <ViewToggleWidget className="float-right" toggle={viewToggle}
                                                 icon={require("./../../assets/svgs/table-view-icon.svg")}/>}
                <SearchWidget className="float-right" setFilter={setFilter}/>
            </div>}
            <div className="card-container">{filteredItems.map(entry =>
                (
                    <CardItem Card={Card}
                              data={entry}
                              additionalData={additionalData}
                              wizardType={wizardType}
                              key={entry.id} selector={selectionOptions.selector}
                              selected={selectionOptions.selectedId === entry.id}
                              menuopen={openedCardId === entry.id} menuItems={menuItems}
                              isMenuDisabled={isMenuDisabled} triggerMenu={() => triggerMenu(entry.id)}
                              relationshipRecommendations={entry.recommendations}
                    />
                )
            )
            }</div>
        </div>
    );
};

export default CardsContainer;

