import React, {useState, useCallback} from "react";
import RightPanelLayout from '../rightPanel/RightPanelLayout';
import DataBrokerDetails from "./DataBrokerDetails";
import {DATA_BROKER_WIZARD_STEPS as STEPS, DOCUMENTATION_DATA_BROKER, FM_AGENT_TYPE_ONPREM} from '../../consts';
import ProviderLogin from './ProviderLogin';
import InstanceDetails from "./InstanceDetails";
import PopoverInfo from '../_common_/reactPopover/PopoverInfo';
import "./dataBrokerWizard.scss";

const typeNamesMap = {
    "AWS": "AWS",
    "AZURE": "Azure",
    "GCP": "Google Cloud"
};

const DataBrokerWizard = (props) => {
    const {closePanel, isGcpAccessRequired, isAwsAccessRequired, shouldDisplayPortBool, existingGroupId, existingAgentType} = props;

    const [step, setStep] = useState(STEPS.DATA_BROKER_DETAILS);
    const [type, setType] = useState(null);
    const [dataBroker, setDataBroker] = useState({});

    const titleAddition = (step > STEPS.PROVIDER_LOGIN && type !== FM_AGENT_TYPE_ONPREM.name) ? `in ${typeNamesMap[type]}` : "";

    const rightPanelTitle = existingGroupId
        ? <>
            <div>Accelerate relationship by adding a Data Broker {titleAddition}</div>
            <PopoverInfo>If the existing data brokers in this relationship are also used in other sync relationships,
                then we’ll automatically add this new data broker to those relationships, as well.</PopoverInfo></>
        : `Create Data Broker ${titleAddition}`;

    const goToNextStep = useCallback(() => {
        setStep(step => (step + 1 >= STEPS.length) ?  step : step + 1);
    }, []);

    //goToPrevStep will be needed when we add "proxy" step

    // const goToPrevStep = useCallback(() => {
    //     setStep(step => (step - 1 < 0) ?  step : step - 1);
    // }, []);

    return <RightPanelLayout
        title={rightPanelTitle}
        closePanel={closePanel} customClass="data-broker-wizard"
        helpUrl={DOCUMENTATION_DATA_BROKER}
    >

        {step === STEPS.DATA_BROKER_DETAILS &&
        <DataBrokerDetails
            goToNextStep={goToNextStep}
            type={type}
            setType={setType}
            isGcpAccessRequired={isGcpAccessRequired}
            isAwsAccessRequired={isAwsAccessRequired}
            shouldDisplayPortBool={shouldDisplayPortBool}
            defaultProvider={existingAgentType}
            setDataBroker={setDataBroker}
            existingGroupId={existingGroupId}
        />}

        {step === STEPS.PROVIDER_LOGIN &&
        <ProviderLogin
            type={type}
            dataBroker={dataBroker}
            goToNextStep={goToNextStep}
        />}

        {step === STEPS.INSTANCE_DETAILS &&
        <InstanceDetails
            type={type}
            dataBroker={dataBroker}
            closePanel={closePanel}
            isGcpAccessRequired={isGcpAccessRequired}
            isAwsAccessRequired={isAwsAccessRequired}
            isAccelerate={!!existingGroupId}
        />}
    </RightPanelLayout>
};

export default DataBrokerWizard;
