import React, {useCallback} from "react";
import InstanceDetailsGcpForm from './InstanceDetailsGcpForm';
import {FM_AGENT_TYPE_GCP} from "../../consts";
import * as deployService from '../../services/gcp.deploy.service';
import {extractErrorGcp} from "../../utils/providers-utils";
import {randomString} from '../../utils/helpers';

const InstanceDetailsGcp = ({addDeployment, dataBroker, onFinish, setStepError}) => {
    const {id, name, template} = dataBroker;

    const handleError = useCallback((error) => setStepError(extractErrorGcp(error)), [setStepError]);

    const handleDeployClick = useCallback(async (values) => {
        const deploymentName = getDeployName(name);
        try {
            const deployDetails = {
                ...getDeployDetails(values),
                dataBrokerName: name,
                deploymentName
            };
            const deployResponse = await deployService.startDeployProcess(
                deployDetails,
                template
            );
            const operationName = deployResponse.name;

            addDeployment({
                dataBrokerId: id,
                dataBrokerName: name,
                providerType: FM_AGENT_TYPE_GCP.name,
                deployDetails: {
                    projectId: deployDetails.projectId,
                    operationName,
                    deploymentName
                }
            });

            onFinish();
        }

        catch (err) {
            handleError(err);
        }

    }, [addDeployment, id, name, template, onFinish, handleError]);

    return <InstanceDetailsGcpForm handleDeployClick={handleDeployClick} handleError={handleError}/>;
};

export default InstanceDetailsGcp;

const getDeployDetails = ({region, zone, VPC, subnet, serviceAccount, project}) => ({
    region: region.value,
    zone: zone.value,
    network: VPC.value,
    subnetwork: subnet.value,
    serviceAccount: serviceAccount.value,
    projectId: project.value
});

const getDeployName = (dataBrokerName) => {
    return `deploy-${dataBrokerName.toLowerCase()}-${randomString(3).toLowerCase()}`;
};
