import React, {useCallback} from "react";
import useGcpForm from './useGcpForm';
import SelectReduxField from '../_common_/forms/SelectReduxField';
import {DOCUMENTATION_URL} from '../../consts';
import ButtonWithError from '../widgets/ButtonWithError';
import Validator from 'validatorjs';
import {formValueSelector, reduxForm} from 'redux-form';
import {connect} from 'react-redux';
import _ from "lodash";
import "./instanceDetailsGcpForm.scss";
import InstanceDetailsColumnHeader from './InstanceDetailsColumnHeader';

const InstanceDetailsGcpForm = ({handleDeployClick, handleError, change, selectedProjectId, selectedVpc, selectedRegion, handleSubmit, submitting}) => {

    const {projects, serviceAccounts, regions, vpcs, zones, subnets} = useGcpForm(selectedProjectId, selectedVpc, selectedRegion, handleError);

    const clearFields = useCallback((fieldArr) => fieldArr.forEach(fieldName => change(fieldName, '')), [change]);

    return (<form className="data-broker-wizard-step instance-details-gcp" onSubmit={handleSubmit(handleDeployClick)} >
        <div className="title">Basic Settings</div>
        <div className="content-area">
            <div className="form-content">
                <div className="col left">
                    <InstanceDetailsColumnHeader text={"Project"}/>
                    <SelectReduxField fieldName="project"
                                      data={projects}
                                      propertyToDisplay="name"
                                      placeholder="Select a project"
                                      label="Project"
                                      displayName="Project"
                                      isDisabled={submitting}
                                      handleChangeFunction={() => clearFields(['serviceAccount', 'region', 'zone', 'VPC', 'subnet'])}/>

                    <SelectReduxField fieldName="serviceAccount"
                                      data={serviceAccounts}
                                      propertyToDisplay="displayName"
                                      isDisabled={!selectedProjectId || submitting}
                                      disabledTitle="Waiting for a project"
                                      placeholder="Select a service account"
                                      label="Service Account"
                                      displayName="Service Account"/>
                    <span className="permissions-label">Select a Service Account that includes
                                  <a href={DOCUMENTATION_URL + '/task_installing_gcp.html#permissions-required-to-deploy-the-data-broker-in-gcp'}
                                     target="_blank" rel="noopener noreferrer"> these permissions</a> </span>

                </div>
                <div className="col right">
                    <InstanceDetailsColumnHeader text={"Location"}/>
                    <SelectReduxField fieldName="region"
                                      data={regions}
                                      propertyToDisplay="name"
                                      isDisabled={!selectedProjectId || submitting}
                                      disabledTitle="Waiting for a project"
                                      placeholder="Select a region"
                                      label="Region"
                                      displayName="Region"
                                      handleChangeFunction={() => clearFields(['zone', 'subnet'])}/>

                    <SelectReduxField fieldName="zone"
                                      data={zones}
                                      propertyToDisplay="displayName"
                                      isDisabled={!selectedRegion || submitting}
                                      disabledTitle="Waiting for a region"
                                      placeholder="Select a zone"
                                      label="Zone"
                                      displayName="Zone"/>

                    <SelectReduxField fieldName="VPC"
                                      data={vpcs}
                                      propertyToDisplay="displayName"
                                      isDisabled={!selectedProjectId || !selectedRegion || submitting}
                                      disabledTitle="Waiting for a project and a region"
                                      placeholder="Select a VPC"
                                      label="VPC"
                                      displayName="VPC"
                                      handleChangeFunction={() => clearFields(['subnet'])}/>

                    <SelectReduxField fieldName="subnet"
                                      data={subnets}
                                      propertyToDisplay="displayName"
                                      isDisabled={!selectedVpc || submitting}
                                      disabledTitle="Waiting for a VPC"
                                      placeholder="Select a subnet"
                                      label="Subnet"
                                      displayName="subnet"/>
                </div>
            </div>
        </div>
        <div className="button-footer">
            <ButtonWithError className="button primary" type="submit" disable={submitting}>
                Continue
            </ButtonWithError>
        </div>
    </form>);
};

const validate = (values) => {
    const rules = {
        project: ['required'],
        serviceAccount: ['required'],
        region: ['required'],
        zone: ['required'],
        VPC: ['required'],
        subnet: ['required'],
    };

    const validator = new Validator(values, rules);
    validator.passes();
    return validator.errors.all();
};

const connectedToForm = reduxForm({form: 'gcpCreateDb', validate})(InstanceDetailsGcpForm);

const selector = formValueSelector('gcpCreateDb');

export default connect(state => ({
    selectedProjectId: _.get(selector(state, 'project'), 'value'),
    selectedVpc: selector(state, 'VPC'),
    selectedRegion: selector(state, 'region')
}))(connectedToForm);
