import React, {useState} from 'react';
import ProviderLoginLayout from './ProviderLoginLayout';
import {AWS_CREATE_BROKER_POLICY_REQUIREMENTS, FM_AGENT_TYPE_AWS} from '../../consts';
import {InputField} from '../_common_/forms/forms';
import {reduxForm, Field} from 'redux-form';
import Validator from 'validatorjs';
import {getVpcs, DEFAULT_REGION} from '../../services/aws.api.service';
import "./providerLoginAws.scss";
import {connect} from 'react-redux';
import {setAwsCredentials} from "../../store/deploy/action_creators";
import useRunOnce from '../../hooks/useRunOnce';
import {extractErrorAws} from "../../utils/providers-utils";

const ProviderLoginAws = ({goToNextStep, handleSubmit, submitting, credentials, setAwsCredentials, dataBroker}) => {
    const {fileLink} = dataBroker;
    const templateUrl = fileLink && fileLink.substr(fileLink.indexOf("https%3"));
    const cloudFormationLink = templateUrl && `${fileLink}&templateUrl=${templateUrl}`;

    const [error, setError] = useState(null);

    const onSubmit = async (values) => {
        const credentials = {
            accessKeyId: values.accessKeyId,
            secretAccessKey: values.secretAccessKey
        };
        try {
            //try to fetch data just in order to validate the credentials
            await getVpcs(credentials, DEFAULT_REGION);
            setAwsCredentials(credentials);
            goToNextStep();
        } catch (err) {
            setError(extractErrorAws(err));
        }
    };

    useRunOnce(() => {
        //in case credentials already exist from previous wizard use, can skip this step
        if (credentials?.accessKeyId) {
            goToNextStep();
        }
    });

    return (
        <ProviderLoginLayout
            providerDetails={FM_AGENT_TYPE_AWS}
            handleClick={handleSubmit(onSubmit)}
            isContinueDisabled={submitting}
            error={error}
            setError={setError}
        >
            <div className="form-content">
                <p>We need an AWS access key to create the data broker in AWS on your behalf.</p>
                <p>{`The keys aren't saved or used for any other purposes.`}</p>

                <form className="aws-login-form">
                    <Field component={InputField}
                           label="AWS Access Key"
                           name="accessKeyId"
                           placeholder="Enter AWS Access Key"/>
                    <Field component={InputField}
                           label="AWS Secret Key"
                           name="secretAccessKey"
                           placeholder="Enter AWS Secret Key"/>

                </form>
                <p>Make sure that your AWS account has the permission defined in the <a href={AWS_CREATE_BROKER_POLICY_REQUIREMENTS} target="_blank" rel="noopener noreferrer"> Policy Requirements</a>.
                </p>
                <p>If you’d rather not provide access keys, <a href={cloudFormationLink} target="_blank"
                                                                                       rel="noopener noreferrer">use a
                    CloudFormation template instead.</a>
                </p>
            </div>
        </ProviderLoginLayout>);
};

const rules = {
    accessKeyId: ['required'],
    secretAccessKey: ['required']
};

const validate = (values) => {
    const validator = new Validator(values, rules);
    validator.setAttributeNames({accessKeyId: 'AWS Access Key', secretAccessKey: 'AWS Secret Key'});
    validator.passes();
    return validator.errors.all();
};

const ProviderLoginAwsForm = reduxForm({form: 'providerLoginAws', validate})(ProviderLoginAws);

const mapStateToProps = state => ({
    credentials: state.deploy.credentials.aws
});

export default connect(mapStateToProps, {setAwsCredentials})(ProviderLoginAwsForm);