import {useMemo} from 'react';
import useFetchWhenChange from '../../hooks/useFetchWhenChange';
import * as awsService from '../../services/aws.api.service';
import _ from "lodash";

const useAwsForm = (credentials, region, vpc, handleError) => {

    const vpcs = useFetchWhenChange(awsService.getVpcs, 'Vpcs', [credentials, region], null, null, handleError);
    const subnets = useFetchWhenChange(awsService.getSubnets, 'Subnets', [credentials, region],null, null, handleError);
    const keyPairs = useFetchWhenChange(awsService.getKeyPairs, 'KeyPairs', [credentials, region], null, null, handleError);

    const vpcsOptions = useMemo(() => {
       return vpcs?.map(vpc => {
           const nameTag = vpc.Tags && vpc.Tags.find(tag => tag.Key === 'Name');
           return {
               value: vpc.VpcId,
               label: `${vpc.VpcId} - ${vpc.CidrBlock}`,
               additionalLabel: `${nameTag ? nameTag.Value + " | " : ""}${subnets?.filter(subnet => subnet.VpcId === vpc.VpcId).length} Subnets`
           }
       })
    }, [vpcs, subnets]);

    const subnetsOptions = useMemo(() => {
        return vpc && _
                .chain(subnets)
                .filter(subnet => subnet.VpcId === vpc)
                .map(subnet => {
                    const nameTag = subnet.Tags && subnet.Tags.find(tag => tag.Key === 'Name');
                    return {
                        value: subnet.SubnetId,
                        label: subnet.CidrBlock,
                        additionalLabel: nameTag && nameTag.Value
                    }
                })
                .value()
    }, [vpc, subnets]);

    const keyPairsOptions = useMemo(() => {
        return keyPairs?.map(keyPair => ({value: keyPair.KeyName, label:keyPair.KeyName}));
    }, [keyPairs]);

    return {vpcsOptions, subnetsOptions, keyPairsOptions}
};

export default useAwsForm;