import React, {Component} from 'react';
import classNames from "classnames";
import {connect} from 'react-redux';
import _ from "lodash";
import {getPropFromState} from '../../utils/mapStateToPropsUtils';

class DialogContainer extends Component{
    constructor(props, context){
        super(props, context);
        this.state = {modalClasses: []};
        this.closeDialogHandler = (e) => {
            if(e.keyCode === 8 && e.target.nodeName !== "INPUT" && e.target.nodeName !== "TEXTAREA"){
                e.preventDefault();
            }

            if (e.keyCode === 27) {
                this.closeDialog();
            }
        }
    }

    componentDidMount(){
        dialogContainer = this;
    }

    setDialog(dialog, parent, disableClose) {
        if(this.state.dialog === dialog) {
            return;
        }

        this.setState({dialog, style: (parent ? {opacity: 0} : {}), disableClose});

        if (!disableClose) {
            document.addEventListener('keydown', this.closeDialogHandler);
        }

        if(parent) {
            const parentBr = parent.getBoundingClientRect();
            setTimeout(() => {

                const modalBr = this._modal.getBoundingClientRect();

                const x = (parentBr.left + parentBr.width/2) - modalBr.left;
                const y = (parentBr.top + parentBr.height/2) - (modalBr.top + modalBr.height);
                this.setState(Object.assign({}, this.state, {style: {opacity: 0, transform: `translate(${x}px, ${y}px) scale(0)`}}));

                setTimeout(() => {
                    this.setState(Object.assign({}, this.state, {style: {transition: "transform 0.5s cubic-bezier(0.25, 0.8, 0.25, 1)", transformOrigin: "0 100%", opacity: 1, transform: "translate(0, 0) scale(1)"}}))
                }, 100)
            });
        }
    }

    closeDialog(){
        this.setState({dialog: null, modalClasses: []});

        document.removeEventListener('keydown', this.closeDialogHandler);
    }

    addClassToModal(className) {
        if (!this.state.modalClasses.includes(className)) {
            this.setState({modalClasses: this.state.modalClasses.concat(className)});
        }
    }

    render(){
        const content = this.state.dialog ?
            React.cloneElement(this.state.dialog, {closeDialog: this.closeDialog.bind(this), addClassToModal: this.addClassToModal.bind(this), ...this.props}) : null;
        return (
            content ? (
                    <div className={classNames("dialog", content.props.className)}>
                        <div className="mask" onClick={this.state.disableClose ? _.noop : this.closeDialog.bind(this)}/>
                        <div className={classNames("modal", this.state.modalClasses)} ref={(el) => this._modal = el} style={this.state.style}>
                            {content}
                        </div>
                    </div>) : <div className="dialog-placeholder"/>

        )
    }
}

let dialogContainer;

const mapStateToProps = state => {
  return {
      _relationships: getPropFromState(state, "_relationships", {inProgress: true}),
      _timeline: state.global._timeline,
  }
};

//The dialog needs to access the state directly -in order to get updates correctly (when the content is dynamic)
//This is because the dialog is a separate node in the DOM - it is not a child component that gets props updates from parent
//React.Portal could work but it creates undesired behavior in case of menu that triggers the dialog -
// The menu should be closed when dialog opens, but because it is a parent of the dialog it stays open in the background.
export const ConnectedDialogContainer = connect(mapStateToProps)(DialogContainer);

export const openDialog = (dialog, parent, disableClose = false) => {
    dialogContainer.setDialog(dialog, parent, disableClose);
};