import React, {Component} from 'react';
import Isvg from 'react-inlinesvg';
import classNames from "classnames";
import * as actionCreatorsConsts from "../../store/global/action_creators_consts";
import * as licenseUtils from "../../utils/license-utils";
import {openDialog} from '../dialog/Dialog';
import EulaDialog from '../eula/EulaDialog';
import {DOCUMENTATION_URL} from '../../consts';
import ReactPopover from '../_common_/reactPopover/ReactPopover';
import "./footer.scss";


export default class extends Component {
    toggleErrorsState() {
        this.props.toggleErrorsState();
    }

    renderLicenseStatusFooter() {
        const {licenseStatusText, warning, error ,title} = licenseUtils.calculateLicenseStatusFooter(this.props);

        return (
            <div className={classNames("license-status-footer", {warning, error})}>
                <button onClick={() => {actionCreatorsConsts.routesContainer.routes.push('/license');} } title={title}>
                    <span><Isvg src={require("../../assets/svgs/exclamation-mark-triangle-icon.svg")}/></span>
                    {licenseStatusText}
                </button>
            </div>
        );
    }

    render() {
        const {auth, _licenseSettings, _ui} = this.props;

        return (
            <>
                {!_ui.blocked &&
                <footer className="primary">
                    {_licenseSettings && _licenseSettings.succeeded &&
                     this.renderLicenseStatusFooter()}


                    {auth.data &&
                    <div className="footer-item api">
                      <a href={process.env.REACT_APP_API_BASE_URL + "/docs"} target="_blank" rel="noopener noreferrer" >
                          <Isvg src={require("./../../assets/svgs/api.svg")}/>
                          <div>API Documentation</div>
                      </a>
                    </div>
                    }
                    <div className="footer-item help">
                        <ReactPopover popoverClass="help-menu" container={
                            <button type="button">
                                <Isvg src={require("./../../assets/svgs/help.svg")}/>
                                <div>Help</div>
                            </button>
                        }>
                            <ul className="no-bullet">
                                <li><button type="button" onClick={(el) => openDialog(<EulaDialog className="eula"/>, el.target)}>EULA <span className="float-right"><Isvg src={require("./../../assets/svgs/top-right-arrow-icon.svg")}/></span></button></li>
                                <li><a href={DOCUMENTATION_URL} target="_blank" rel="noopener noreferrer" >Documentation<span className="float-right"><Isvg src={require("./../../assets/svgs/top-right-arrow-icon.svg")}/></span></a></li>
                            </ul>
                        </ReactPopover>
                    </div>
                    {process.env.REACT_APP_VERSION && <div className="version">
                        <div>{`Version: ${process.env.REACT_APP_VERSION}`}</div>
                    </div>}
                    {/*<div className={classNames("errors", {"is-visible":shouldShowErrorsIcon })} onClick={this.toggleErrorsState.bind(this)}>*/}
                    {/*    <span><Isvg src={require("./../../assets/svgs/errors-footer.svg")}/></span>*/}
                    {/*    {errorsMessage}*/}
                    {/*</div>*/}
                </footer>}
            </>
        );
    }
}