import React, {useState, useCallback} from "react"
import Isvg from "react-inlinesvg";
import {connect} from 'react-redux';
import {routesContainer} from '../../store/global/action_creators_consts';
import useEvent from '../../hooks/useEvent';
import Select from 'react-select';
import _ from 'lodash';
import {setActiveAccount} from "../../utils/messages-utils";

require ("./userInfoContent.scss");

const UserInfoContent =  (props) => {
    const {auth: {userName, email, company, accountId, uid}, _accounts, closeSlider} = props;
    const accounts = _.get(_accounts, 'data');
    const activeAccount = _.find(accounts, account => account.accountId === accountId);
    const [selectedAccount, setSelectedAccount] = useState(activeAccount);
    const [menuIsOpen, setMenuIsOpen] = useState(false);
    const accountOptions = _.map(accounts, account => {return {label: account.name, value: account.accountId}});

    const handleKeydown = useCallback((e) => {
        if (e.target.className === "switch-account") return;
        setMenuIsOpen(false);
    }, [setMenuIsOpen]);

    useEvent("mousedown", handleKeydown);

    const InfoLine = ({text, title}) => {
        return (
            <div className="info-line">
                <div className="text no-wrap" title={text}>{text}</div>
                <div className="title">{title}</div>
            </div>
        )
    };

    const switchAccount = (e) => {
        setMenuIsOpen(false);
        const switchToAccountId = e.value;
        if (switchToAccountId === selectedAccount.accountId) return;
        const switchToAccount = _.find(accounts, account => account.accountId === switchToAccountId);
        setSelectedAccount(switchToAccount);
        closeSlider();

        //timeout in order to allow the closeSlider animation
        setTimeout(()=>{
            props.setActiveAccount({accountId: switchToAccountId, uid});
            routesContainer.routes.push("/dashboard");
        }, 500)
    };

    return (<div className="user-info">
                <header>
                    <div><span><Isvg src={require("./../../assets/svgs/user.svg")} /></span>User Settings</div>
                    <button type="button" className="button card-blue" onClick={async () => {
                        window.auth.logout()
                    }}>Logout
                    </button>
                </header>
                <div className="info-line-container">
                    <InfoLine text={userName} title="Name"/>
                    <InfoLine text={email} title="Email"/>
                    <InfoLine text={company} title="Company"/>

                    <div className="info-line">
                        <div className="select-wrapper">
                            <Select className="Select account-options"
                                    classNamePrefix="Select"
                                    onChange={switchAccount}
                                    options={accountOptions}
                                    isSearchable={false}
                                    isClearable={false}
                                    value={_.find(accountOptions, (option) => option.value === _.get(selectedAccount, 'accountId'))}
                                    isLoading={!accounts}  menuIsOpen={menuIsOpen}/>
                            {(accountOptions.length > 1) && <div className="switch-account" onClick={()=>{setMenuIsOpen(!menuIsOpen)}}>Switch Account</div>}
                        </div>
                        <div className="title">Account</div>

                    </div>
                </div>
            <div className="edit-user-info-line">
                To edit user info or password go to <a href={process.env.REACT_APP_PORTAL_BASE_URL + "?userMenuOpen"} target="blank">NetApp Cloud Central</a>
            </div>
        </div>
    )
};

const mapStateToProps = state => {
    return {
        auth: state.global.auth.data,
        _accounts: state.global._accounts
    }
};

export default connect(mapStateToProps, {setActiveAccount})(UserInfoContent);
