import React, {useState} from "react";
import Isvg from "react-inlinesvg";
import {NOTIFICATION_TYPES} from "../../consts";
import capitalize from "lodash/capitalize";
import "./notificationItems.scss";

//if notification text is more than this, should use variant "long"
const MAX_CHARS_COUNT = 130; //for general page notifications
const MAX_CHARS_COUNT_NARROW = 105;  // for notifications in right panel

const NOTIFICATION_VARIANTS = {
    LONG: "long",
    SHORT: "short"
};

const {SUCCESS, ERROR, INFO} = NOTIFICATION_TYPES;

const NotificationItem = ({type, text, shortDescription, isNarrow = false, onClose}) => {
    const [expanded, setExpanded] = useState(false);
    const typeLowerCase = type.toLowerCase();

    const {variant, description} = getNotificationDetails(type, text, shortDescription, isNarrow);

    const toggleExpanded = () => {
        setExpanded(!expanded);
    };

    return (
        <div className={`notification-item ${typeLowerCase} ${variant} ${isNarrow ? "narrow" : ""}`}>

            <div className={`notification-border ${typeLowerCase}`}/>

            <div className="notification-icon">
                <Isvg src={require(`../../assets/svgs/notification-icon-${typeLowerCase}.svg`)}/>
            </div>

            <div className="notification-text-container">

                {variant === NOTIFICATION_VARIANTS.SHORT &&
                    <div className="notification-text">{text}</div>}

                {variant === NOTIFICATION_VARIANTS.LONG && <>
                    <div className="notification-text">
                        <div className="notification-text-description">{description}</div>
                        <div className="notification-toggle">
                            <button onClick={toggleExpanded}>Show {expanded ? "less" : "more"}</button>
                            <div className={`caret-icon ${expanded ? "" : "down"}`}>
                                <Isvg src={require("../../assets/svgs/caret-up.svg")}/>
                            </div>
                        </div>
                    </div>
                    {expanded && <div className="notification-text-long">{text}</div>}
                </>}

            </div>

            {onClose && (<div className="notification-close-icon">
                    <Isvg src={require("../../assets/svgs/notification-close.svg")} onClick={onClose}/>
                </div>)}
        </div>
    );
};

const getNotificationDetails = (type, text, shortDescription, isNarrow) => {
    const textCharCount = text.length;
    const maxCharCount = isNarrow ? MAX_CHARS_COUNT_NARROW : MAX_CHARS_COUNT;
    const variant = textCharCount > maxCharCount ? NOTIFICATION_VARIANTS.LONG : NOTIFICATION_VARIANTS.SHORT;
    const defaultShortDescription = (type === ERROR) ? "An error occurred." : `${capitalize(type)} notification.`;
    const description = (shortDescription && shortDescription.length < MAX_CHARS_COUNT) ? shortDescription : defaultShortDescription;
    return {variant, description};
};

const NotificationInfo = (props) => <NotificationItem type={INFO} {...props}/>;
const NotificationError = (props) => <NotificationItem type={ERROR} {...props}/>;
const NotificationSuccess = (props) => <NotificationItem type={SUCCESS} {...props}/>;

export {
    NotificationItem,
    NotificationInfo,
    NotificationError,
    NotificationSuccess
}