import React from 'react';
import classNames from 'classnames';
import DataBrokerProperties from "./../tabs/DataBrokerProperties";
import {FM_AGENT_TYPE_AWS, FM_AGENT_TYPE_AZURE, FM_AGENT_TYPE_ONPREM, FM_AGENT_TYPE_GCP} from "../../consts";
import ReactLoading from 'react-loading';
import {DEPLOY_STATUS_PROGRESS, DEPLOY_STATUS_SUCCESS, DEPLOY_STATUS_ERROR} from "../../consts";
import Isvg from "react-inlinesvg";
import PopoverInfo from '../_common_/reactPopover/PopoverInfo';
import {openDialog} from '../dialog/Dialog';
import OnPremInstallationDialog from "./OnPremInstallationDialog";
import "./dataBrokerCard.scss";

export default {
    isSelectionDisabled: ({isReady, isDisabled}) => {return(!isReady || isDisabled)},
    className: ({isReady, isFailed, isWaiting, isDisabled}) => classNames("data-broker-card", {
            waiting: isWaiting,
            ready: isReady,
            failed: isFailed || isDisabled
        }),
    renderContent: (data, additionalData) => {
        const {id, isFailed, isWaiting, isDisabled, placement, type, ui, port, name, deployment, agentFailureMessage, disabledErrorMessage} = data;
        const isReady =  !isDisabled && !isWaiting && !isFailed;

        const isAWS = type === FM_AGENT_TYPE_AWS.name;
        const isAzure = type === FM_AGENT_TYPE_AZURE.name;
        const isGcp = type === FM_AGENT_TYPE_GCP.name;
        const isOnprem = type === FM_AGENT_TYPE_ONPREM.name;
        const {shouldDisplayPort, isAwsAccessRequired, isGcpAccessRequired} = additionalData;


        return (<>
                <div className="checkmark"><Isvg src={require('../../assets/svgs/checkmark-fill-circle-icon-color.svg')}/></div>
                <div className="card-header">
                     <div className="card-header-name">
                         <div className="icon">{isReady ? ui.type.circledIcon: ui.type.disabledCircledIcon}</div>
                         <div className={classNames("name", {aws: isAWS, azure: isAzure, onprem: isOnprem, gcp: isGcp})} title={data.name} >{name}</div>
                    </div>

                   <div className="card-header-status">

                       {(isFailed || isDisabled) &&  <div className="status-error">
                           <div className="icon"><div className="circled"><span>!</span></div></div>
                           <div>{isFailed ? "Connection Failed" : "Missing credentials"}</div>
                           <PopoverInfo>{isFailed ? agentFailureMessage : disabledErrorMessage}</PopoverInfo>
                       </div>}

                       {isReady && <div className="status-active">
                           <div className="icon"><Isvg src={require("../../assets/svgs/checkmark-green.svg")}/></div>
                           <div>Active</div>
                       </div>}

                       {isWaiting && isOnprem && <div className="instructions">
                           <button className="text-button" onClick={(el) => {
                               openDialog(<OnPremInstallationDialog
                                   isAwsAccessRequired={isAwsAccessRequired}
                                   isGcpAccessRequired={isGcpAccessRequired}
                                   dataBroker={data}
                               />, el.target);
                           }}>View installation instructions
                           </button>
                           <Isvg src={require("../../assets/svgs/external-link-icon.svg")}/>
                       </div>}

                   </div>

                </div>
                <div className="card-content">
                    {!isWaiting && <DataBrokerProperties dataBroker={{id, type, placement, port}} shouldDisplayPort={shouldDisplayPort} />}
                    {isWaiting && <DeploymentStatus deployment={deployment} isOnprem={isOnprem} providerName={ui.type.providerName}/>}
                </div>
            </>
        )
    }
};

const DeploymentStatus = ({deployment, isOnprem, providerName}) => {

    return (<div className="card-content-waiting">
          {deployment && (deployment.deployStatus === DEPLOY_STATUS_PROGRESS || deployment.deployStatus === DEPLOY_STATUS_SUCCESS) && <div className="deploy-progress">
              <div className="deploy-progress-title">
                  <ReactLoading type="spin" color="#0891d8" height={0} width={20} delay={50} />
                  <div className="deploy-progress-title-text">Deploying Data Broker</div>
              </div>
              <div>This process usually takes around 10 minutes</div>
          </div>}

          {deployment && deployment.deployStatus === DEPLOY_STATUS_ERROR && <div className="deploy-error">
              <div className="deploy-error-title">
                  <div className="circled"><span>!</span></div>
                  <div>An error occurred while deploying the Data Broker</div>
              </div>
              <div className="deploy-error-description">
                  <div>An error occurred while deploying the Data Broker</div>
                  <PopoverInfo popoverClass="deploy-error-description">{deployment.errorMessage}</PopoverInfo>
              </div>
          </div>}
          {!deployment && <div className="deploy-unknown">
              <div className="deploy-unknown-title">
                  <Isvg src={require("../../assets/svgs/clock-icon.svg")}/>
                  <div>Waiting for Data Broker creation{isOnprem ? "" : ` in ${providerName}`}</div>
              </div>
              <div>Details about the data broker will appear as soon as the deployment is complete.</div>
              <div>We can’t follow the progress because we don’t have information about the deployment.</div>
          </div>}
      </div>
  );
};
