import React, {useState} from 'react';
import {connect} from 'react-redux';
import stepinize from "./../wizard/stepinize"
import {FM_AGENT} from '../../consts';
import {Route, Switch} from 'react-router-dom';
import classNames from "classnames";
import Isvg from "react-inlinesvg";
import _ from "lodash";
import HowDoesItWorkButton from '../systemFlow/HowDoesItWorkButton';
import {FlowEncrypted} from '../systemFlow/SystemFlow';
import ButtonWithNotification from '../widgets/ButtonWithNotification';
import useRunOnce from '../../hooks/useRunOnce';
import {getWizardParamsFromUrlParams} from '../../utils/mapStateToPropsUtils';
import {updateUrlParams, wizardStepExited, wizardStepStarted} from '../../store/wizard/action_creators';

require("./dataBrokerFunctionStep.scss");

const DataBrokerFunctionStep = (props) => {
    const {_wizard: {wizardTypeForSystemFlow, isSecure, selectedWizardParams: {selectedIsTargetDataBrokerInitiator}}, up} = props;

    const [isTargetDataBrokerInit, setIsTargetDataBrokerInit] = useState("false");

    useRunOnce(() => {
        if (!_.isUndefined(selectedIsTargetDataBrokerInitiator)) {
            setIsTargetDataBrokerInit(selectedIsTargetDataBrokerInitiator);
        }
    });

    const onContinue = () => {
        const nextStepPath = props.nextStep.path;
        props._wizard.nextStep(`/${isTargetDataBrokerInit}/${nextStepPath}`, props.history);
    };

    return (
        <div className="data-broker-function wizard-step">
            <div className={classNames("scrollable-area", {up, down: !up})}>
                <div className="title-and-actions">
                    <div className="actions-left">
                        <HowDoesItWorkButton wizardType={wizardTypeForSystemFlow} isSecure={isSecure}/>
                    </div>
                    <div className="title-middle">
                        <h3>Define Data Brokers Functionality</h3>
                    </div>
                    <div className="actions-right"/>
                </div>

                <p className="description">Define which data broker listens for connection requests on a given port and which one initiates the connection. <br/>
                    After you create the relationship, the initiator starts the connection between the two data brokers and data transfers from the source to the target.</p>

                <div className="diagram">
                    <FlowEncrypted wizardType={wizardTypeForSystemFlow}/>
                </div>
                <div className="buttons-container">
                    <div className="databrokers-function-pair">
                        <div className="databroker-function source"><div>{(isTargetDataBrokerInit === "true")? "LISTENER" : "INITIATOR"}</div></div>
                        <div className="databroker-function target"><div>{(isTargetDataBrokerInit === "true")? "INITIATOR" : "LISTENER"}</div></div>
                    </div>

                    <div className="button-wrapper">
                        <Isvg className="dots" src={require("./../../assets/svgs/dots-connecting.svg")}/>
                        <button className="switch-initiator button primary" type="button" onClick={() => {setIsTargetDataBrokerInit(isTargetDataBrokerInit === "true" ? "false" : "true")}}>
                            Switch Functionality</button>
                        <Isvg className="dots translated" src={require("./../../assets/svgs/dots-connecting.svg")}/>
                    </div>
                </div>
            </div>
            <ButtonWithNotification onClick={onContinue} text="Continue"/>
        </div>)
};

const willMount = () => true;
const isFetching = () => false;
const stepId = () => {
    return 'dataBrokerFunction';
};
const canShowNextStep = ({_wizard}) => {
    return !_.isUndefined(_wizard.selectedWizardParams.selectedIsTargetDataBrokerInitiator);
};

const nextRoutes = ({match, nextStep, NextStepComp}) => {
    return (
        <Switch>
            <Route path={`${match.url}/:isTargetDataBrokerInitiator/${nextStep.path}`} render={(props) =>
                <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
            }/>
        </Switch>
    )
};

const mapStateToProps = (state, ownProps) => {
    return {
        _wizard: getWizardParamsFromUrlParams(state, ownProps),
        routing: state.router,
        ...ownProps
    }
};

const mapDispatchToProps = {
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
};

export default connect(mapStateToProps, mapDispatchToProps)(stepinize(DataBrokerFunctionStep, {
    stepId,
    willMount,
    isFetching,
    canShowNextStep,
    nextRoutes,
    loading: () => {return {text: FM_AGENT + "s", icon: require("./../../assets/svgs/data-broker-default.svg")}}
}));
