import React, {useState} from 'react';
import {DndProvider} from 'react-dnd';
import {HTML5Backend} from 'react-dnd-html5-backend';
import {connect} from "react-redux";
import Isvg from 'react-inlinesvg';
import DropEndPoint from "./DropEndPoint";
import DraggableEndPoint from "./DraggableEndPoint";
import _ from "lodash";
import {MISSING_ANF_NOTIFICATION} from "../../consts";
import {isSecureAllowed, isSourceTargetSupported} from "./../../utils/sync-utils";
import {openDialog} from '../dialog/Dialog';
import HowDoesItWorkDialog from '../systemFlow/HowDoesItWorkDialog';
import {removeSettingsFromSessionStorage} from './../../utils/settings-utils';
import useRunOnce from '../../hooks/useRunOnce';
import {getCvsWorkingEnvironments} from "../../store/wizard/action_creators";
import useRunOnceWhenTruthy from '../../hooks/useRunOnceWhenTruthy';
import {getAnfWorkingEnvironments} from "../../utils/cvsUtils";
import ButtonWithNotification from '../widgets/ButtonWithNotification';
import "./dragSourceAndTargetStep.scss";

const titlesToEndPoint = {
    "On Prem": {
        icon: require('./../../assets/svgs/onprem-default.svg'),
        disabledIcon: require('./../../assets/svgs/onprem-disabled.svg'),
        dropIcon: require('./../../assets/svgs/onprem-selected.svg'),
        protocolSelectorIcon: require('./../../assets/svgs/onprem-table.svg'),
        title: "On Prem", group: "fileSystem", protocolOptions: ["nfs", "cifs"]
    },
    "Cloud Volumes ONTAP": {
        icon: require('./../../assets/svgs/cvo-default.svg'),
        disabledIcon: require('./../../assets/svgs/cvo-disabled.svg'),
        dropIcon: require('./../../assets/svgs/cvo-selected.svg'),
        protocolSelectorIcon: require('./../../assets/svgs/cvo-table.svg'),
        title: "Cloud Volumes ONTAP", provider: "cvo", group: "fileSystem", protocolOptions: ["nfs", "cifs"]
    },
    "Cloud Volumes Service": {
        icon: require('./../../assets/svgs/cvs-default.svg'),
        disabledIcon: require('./../../assets/svgs/cvs-disabled.svg'),
        dropIcon: require('./../../assets/svgs/cvs-selected.svg'),
        protocolSelectorIcon: require('./../../assets/svgs/cvs-table.svg'),
        title: "Cloud Volumes Service", provider: "cvs", group: "fileSystem", protocolOptions: ["nfs", "cifs"]
    },
    "Azure NetApp Files": {
        icon: require('./../../assets/svgs/anf-default.svg'),
        disabledIcon: require('./../../assets/svgs/anf-disabled.svg'),
        dropIcon: require('./../../assets/svgs/anf-selected.svg'),
        protocolSelectorIcon: require('./../../assets/svgs/anf-table.svg'),
        title: "Azure NetApp Files", provider: "anf", group: "fileSystem", protocolOptions: ["nfs", "cifs"]
    },
    "SMB Server": {
        icon: require('./../../assets/svgs/cifs-default.svg'),
        disabledIcon: require('./../../assets/svgs/cifs-disabled.svg'),
        dropIcon: require('./../../assets/svgs/cifs-selected.svg'),
        title: "SMB Server", protocol: "cifs", group: "fileSystem", firstRoute: "cifs"
    },
    "NFS Server": {
        icon: require('./../../assets/svgs/nfs-default.svg'),
        disabledIcon: require('./../../assets/svgs/nfs-disabled.svg'),
        dropIcon: require('./../../assets/svgs/nfs-selected.svg'),
        title: "NFS Server", protocol: "nfs", group: "fileSystem", firstRoute: "nfs"
    },
    "AWS EFS": {
        icon: require('./../../assets/svgs/efs-default.svg'),
        disabledIcon: require('./../../assets/svgs/efs-disabled.svg'),
        dropIcon: require('./../../assets/svgs/efs-selected.svg'),
        title: "AWS EFS", protocol: "nfs", provider: "efs", group: "fileSystem", firstRoute: "nfs"
    },
    "AWS S3": {
        icon: require('./../../assets/svgs/s3-default.svg'),
        disabledIcon: require('./../../assets/svgs/s3-disabled.svg'),
        dropIcon: require('./../../assets/svgs/s3-selected.svg'),
        title: "AWS S3", protocol: "s3", group: "storage", firstRoute: "agents"
    },
    "StorageGRID": {
        icon: require('./../../assets/svgs/sgws-default.svg'),
        disabledIcon: require('./../../assets/svgs/sgws-disabled.svg'),
        dropIcon: require('./../../assets/svgs/sgws-selected.svg'),
        title: "StorageGRID", protocol: "s3", provider: "sgws", group: "storage", firstRoute: "s3based"
    },
    "Google Cloud Storage": {
        icon: require('./../../assets/svgs/gcp-default.svg'),
        disabledIcon: require('./../../assets/svgs/gcp-disabled.svg'),
        dropIcon: require('./../../assets/svgs/gcp-selected.svg'),
        title: "Google Cloud Storage", protocol: "gcp", group: "storage", firstRoute: "agents"
    },
    "IBM Cloud Object Storage": {
        icon: require('./../../assets/svgs/ibm-default.svg'),
        disabledIcon: require('./../../assets/svgs/ibm-disabled.svg'),
        dropIcon: require('./../../assets/svgs/ibm-selected.svg'),
        title: "IBM Cloud Object Storage", protocol: "s3", provider: "ibm", group: "storage", firstRoute: "s3based"
    },
    "ONTAP S3 Storage": {
        icon: require('./../../assets/svgs/ontap-default.svg'),
        disabledIcon: require('./../../assets/svgs/ontap-disabled.svg'),
        dropIcon: require('./../../assets/svgs/ontap-selected.svg'),
        title: "ONTAP S3 Storage", protocol: "s3", provider: "ontap", group: "storage", firstRoute: "s3based"
    },
    "Azure Blob": {
        icon: require('./../../assets/svgs/azure-default.svg'),
        disabledIcon: require('./../../assets/svgs/azure-disabled.svg'),
        dropIcon: require('./../../assets/svgs/azure-selected.svg'),
        title: "Azure Blob", protocol: "azure", group: "storage", firstRoute: "azure"
    }
};

const endPoints = [
    titlesToEndPoint["On Prem"],
    titlesToEndPoint["Cloud Volumes ONTAP"],
    titlesToEndPoint["Azure NetApp Files"],
    titlesToEndPoint["Cloud Volumes Service"],
    titlesToEndPoint["NFS Server"],
    titlesToEndPoint["SMB Server"],
    titlesToEndPoint["AWS EFS"],
    titlesToEndPoint["AWS S3"],
    titlesToEndPoint["StorageGRID"],
    titlesToEndPoint["IBM Cloud Object Storage"],
    titlesToEndPoint["ONTAP S3 Storage"],
    titlesToEndPoint["Google Cloud Storage"],
    titlesToEndPoint["Azure Blob"]
];

const cmIntegrationTypesMap = {
    "ANF": "Azure NetApp Files",
    "CVO": "Cloud Volumes ONTAP",
    "CVS": "Cloud Volumes Service",
    "ONPREM": "OnPrem",
    "S3": "AWS S3"
};

const DragSourceAndTargetStepComp = ({history, _ui: {isCmIntegration}, getCvsWorkingEnvironments, accountInfo, anfWorkingEnvironments, isFetchingWorkingEnvironments, inputDetails}) => {
    const [error, setError] = useState(null);
    const [source, setSource] = useState(null);
    const [target, setTarget] = useState(null);
    const [isSecure, setIsSecure] = useState(false);

    //For case user was in the middle of wizard and used browser's "back"
    useRunOnce(() => removeSettingsFromSessionStorage());

    //initialize cmIntegration inputs, in case of coming from right panel
    useRunOnceWhenTruthy(() => {
        const {workingEnvironmentName, workingEnvironmentType, viewDashboard, isSource} = inputDetails;
        if (viewDashboard || !workingEnvironmentType) return;
        const type = cmIntegrationTypesMap[workingEnvironmentType];
        const endpoint = {...titlesToEndPoint[type], workingEnvironmentName};
        if (isSource) {
            setSource(endpoint);
        }
        else {
            setTarget(endpoint);
        }
    }, inputDetails);

    useRunOnceWhenTruthy(() => {
        getCvsWorkingEnvironments(accountInfo.data.accountId);
    }, isCmIntegration && accountInfo?.data?.accountId);

    const handleDrop = (draggableEndPoint, dropEndPoint) => {
        setError("");
        const endPoint = titlesToEndPoint[draggableEndPoint];

        if (isCmIntegration && !isFetchingWorkingEnvironments && isMissingAnfWe(endPoint)) {
            setError(MISSING_ANF_NOTIFICATION);
            return;
        }

        if (dropEndPoint.includes("Source")) {
            setSource(endPoint);
            //align source with target (in case of dropping a source with options, when target is defined)
            if (endPoint.protocolOptions && _.includes(endPoint.protocolOptions, target?.protocol)) {
                setSource({...endPoint, protocol: target.protocol})
            }
        }
        if (dropEndPoint.includes("Target")) {
            setTarget(endPoint);
            //align target with source (in case of dropping a target with options, when source is defined)
            if (endPoint.protocolOptions && _.includes(endPoint.protocolOptions, source?.protocol)) {
                setTarget({...endPoint, protocol: source.protocol})
            }
        }
    };

    const isMissingAnfWe = (firstEndpoint, secondEndpoint) => {
        return (firstEndpoint?.provider === "anf" || secondEndpoint?.provider === "anf") && anfWorkingEnvironments.length === 0;
    };

    const handleContinue = () => {

        if (_.isEmpty(source) || _.isEmpty(target)) {
            setError("You must select both source and target");
            return;
        }

        if (isCmIntegration && isMissingAnfWe(source, target)) {
            setError(MISSING_ANF_NOTIFICATION);
            return;
        }

        const protocols = (source.protocol) + "-" + (target.protocol);
        const sourceProvider = source.provider || 'NA';
        const targetProvider = target.provider || 'NA';
        const providers = `${sourceProvider}-${targetProvider}`;
        //This is a safety net - recheck isSourceTargetSupported
        if (isSourceTargetSupported(source, target)) {
            history.push(`/sync/${protocols}/${providers}/${isSecure}/start-wizard`);
        }
        else {
            setError("Unsupported selection");
        }
    };

    const removeTargetEndPoint = () => {
        setTarget(null);
        setError("");
    };

    const removeSourceEndPoint = () => {
        setSource(null);
        setError("");
    };

    const changeSecureSelection = (index) => {
        setIsSecure(index === 1);
    };

    const isAllowed = (entry) => {
        return source && target
            ? false
            : !source && !target
                ? true
                : isSourceTargetSupported(source || entry, target || entry);
    };

    const showLoaderCondition = (endpoint)=> endpoint.provider === "anf" && isFetchingWorkingEnvironments;
    const renderEncryptedCheckbox = isSecureAllowed(source, target) && !isFetchingWorkingEnvironments;
    const getWizardTypeForSystemFlow = source && target ? `${source.provider || source.protocol}-${target.provider || target.protocol}` : undefined;

    return (
        <DndProvider backend={HTML5Backend}>
            <div className="select-direction-step">
                <div className="scrollable-area">
                    <h3>Drag & Drop your selection into the source and target components</h3>
                    <div className="end-points-container">{endPoints.map((entry) =>
                        (<div className="end-point-wrapper" key={entry.title}>
                            <DraggableEndPoint icon={entry.icon} disabledIcon={entry.disabledIcon} title={entry.title}
                                               onDrop={handleDrop}
                                               isAllowed={isAllowed(entry)}>
                            </DraggableEndPoint>
                            <div className={`text ${isAllowed(entry) ? "" : "is-disabled"}`}>{entry.title}</div>
                        </div>)
                    )}
                    </div>
                    <div className="select-container">
                        <DropEndPoint isSource={true} title="Drag Source Here"
                                      handleEndPointSelection={removeSourceEndPoint}
                                      endPoint={source}
                                      setEndPoint={setSource}
                                      showLoaderCondition={showLoaderCondition}
                                      oppositeEndPoint={target}>
                        </DropEndPoint>

                        <div className="arrow">
                            <Isvg src={require('./../../assets/svgs/arrow-right.svg')}/>
                        </div>

                        <DropEndPoint isSource={false} title="Drag Target Here"
                                      handleEndPointSelection={removeTargetEndPoint}
                                      endPoint={target}
                                      setEndPoint={setTarget}
                                      showLoaderCondition={showLoaderCondition}
                                      oppositeEndPoint={source}>
                        </DropEndPoint>

                    </div>
                    { renderEncryptedCheckbox &&
                    <div className="encrypted-checkbox-wrapper">
                        <div className="title">
                            <span className="lock-icon"><Isvg src={require('./../../assets/svgs/lock.svg')}/></span>
                            <span className="title-text">Data-In-Flight Encryption</span>
                            <span className="title-right" onClick={(el) => openDialog(<HowDoesItWorkDialog wizardType={getWizardTypeForSystemFlow} isSecure={true} className="how-it-works encrypted"/>, el.target)}>
                                How Does it Work ?</span>
                        </div>
                        <div className="radio-buttons-line">
                            <span className="radio-buttons-label">Enable data-in-flight encryption when syncing data?</span>
                            <span className="radio-buttons-wrapper">
                                <label className="radio-button">
                                    <input name="secureNfs" type="radio" value="Yes"
                                           checked={isSecure}
                                           onChange={changeSecureSelection.bind(this, 1)} /> Yes
                                </label>
                                <label className="radio-button">
                                <input name="secureNfs" type="radio" value="No"
                                       checked={!isSecure}
                                       onChange={changeSecureSelection.bind(this, 0)} /> No
                                </label>
                            </span>
                        </div>
                    </div>
                    }
                </div>

                <ButtonWithNotification
                        onClick={handleContinue}
                        submitting={isFetchingWorkingEnvironments}
                        error={error}
                        clearError={() => {setError(null)}}
                        text="Continue"/>

            </div>
        </DndProvider>
    )
};

const mapStateToProps = state => ({
    _ui: state.global._ui,
    accountInfo: state.global._accountInfo,
    inputDetails: state.global._cmIntegration.inputDetails,
    anfWorkingEnvironments: getAnfWorkingEnvironments(state.syncReducer._cvsWorkingEnvironments),
    isFetchingWorkingEnvironments: state.syncReducer._cvsWorkingEnvironments?.inProgress
});

export default connect(mapStateToProps, {getCvsWorkingEnvironments})(DragSourceAndTargetStepComp);
