import React from 'react';
import {
    serverCredentialsStep,
    getServersByType,
    serverIPStep,
    getAnfVolumes,
    clearUrlParamsIfChanged,
    clearShowAction,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
} from '../../store/wizard/action_creators';
import {buildServersList, getWizardParamsFromUrlParams, getPropFromState} from "../../utils/mapStateToPropsUtils";
import {connect} from 'react-redux';
import stepinize from "./../wizard/stepinize";
import {reduxForm} from 'redux-form';
import {
    DEFAULT_NFS_VERSION,
    END_POINT_NAMES_MAP,
    HOST_IPV4_IPV6_REG_EXP,
    HOST_NAME_REG_EXP,
    IP_REG_EXP,
    NFS_ANF_VERSIONS,
    NFS_CVS_VERSIONS,
    NFS_VERSIONS,
    SELECTED_PARAMS_NFS_SERVER,
    SELECTED_PARAMS_NFS_VERSION,
    SELECTED_PARAMS_TARGET_NFS_SERVER,
    SELECTED_PARAMS_TARGET_NFS_VERSION
} from '../../consts';
import {Route, Switch} from 'react-router-dom';
import ServerStepTemplate from "./ServerStepTemplate";
import {calculateStepId} from '../../utils/sync-utils';
import {fetchAnfVolumes, getIsIntegratedStep} from '../../utils/wizard-utils';
import _ from "lodash";
const indefinite = require('indefinite');

const validate = ({serverName}) => {
    let errors = {};
    if (serverName && serverName.length > 256) {
        errors.serverName = "NFS hostname or IP address must be less than 256 characters";
    } else if (!IP_REG_EXP.test(serverName) && !HOST_NAME_REG_EXP.test(serverName) && !HOST_IPV4_IPV6_REG_EXP.test(serverName)) {
        errors.serverName = "Invalid hostname or IP address";
    }
    return errors;
};

const NFSServerStep = (props) => {
    if (!props._wizard) return <div/>;
    const {_servers, _anfVolumes, endpointObj, protocolAndProviderKey, isTargetStepFromSameSourceType, _wizard, _ui: {isCmIntegration}} = props;
    const {isSourceStep, selectedWizardParams} = _wizard;
    const {selectedNFS, selectedNFSTarget, nfsVersion, targetNfsVersion, selectedAnfWe, selectedAnfWeTarget, selectedAnfVolTarget, selectedAnfVol} = selectedWizardParams;

    const isFullAnfStep = getIsIntegratedStep(isCmIntegration, endpointObj);
    const relevantAnfWe = isTargetStepFromSameSourceType ? selectedAnfWeTarget : selectedAnfWe;
    const selectedVolume = isTargetStepFromSameSourceType ? selectedAnfVolTarget : selectedAnfVol;

    const serverType = endpointObj.provider || endpointObj.protocol;
    const defaultNFSVersion = DEFAULT_NFS_VERSION[serverType.toUpperCase()];
    const selectedServer = isTargetStepFromSameSourceType ? selectedNFSTarget : selectedNFS;

    const stepOptions = {
        storage: _servers[protocolAndProviderKey],
        server: selectedServer,
        serverSVG: `${endpointObj.provider || "nfs"}-default.svg`,
        step: "ip",
        showVolumesTable: isFullAnfStep,
        selectedVolumeId: selectedVolume, //relevant for case of "showVolumesTable"
        volumes:  _anfVolumes[relevantAnfWe], //relevant for case of "showVolumesTable"
        serverTitle: serverType.toUpperCase(),
        serverType: "NFS",
        selectType: "Server",
        serverPlaceholder: "For example: 217.70.21.18 or netapp.com",
        title: `Select ${indefinite(serverType.toUpperCase())} ${!isSourceStep || isTargetStepFromSameSourceType ? "Target" : "Source"}`,
        selectedVersion: (isTargetStepFromSameSourceType ? targetNfsVersion : nfsVersion) || defaultNFSVersion,
        versionsList: isFullAnfStep
            ? null
            : _.includes(["nfs", "cvo"], serverType)
                ? NFS_VERSIONS
                : serverType === "anf"
                    ? NFS_ANF_VERSIONS
                    : serverType === "cvs"
                        ? NFS_CVS_VERSIONS
                        : null,
        clearURLOptions: [
            {
                changedParam: isTargetStepFromSameSourceType ? SELECTED_PARAMS_TARGET_NFS_SERVER : SELECTED_PARAMS_NFS_SERVER,
                newValueFromState: "selectedIp",
                urlParams: isTargetStepFromSameSourceType ? ["exportPathTarget", "rootExportTarget"] : ["exportPath", "rootExport"]
            },
            {
                changedParam: isTargetStepFromSameSourceType ? SELECTED_PARAMS_TARGET_NFS_VERSION : SELECTED_PARAMS_NFS_VERSION,
                newValueFromState: "version",
                urlParams: isTargetStepFromSameSourceType ? ["exportPathTarget", "rootExportTarget"] : ["exportPath", "rootExport"]
            }],
        validate
    };

    return <ServerStepTemplate stepOptions={stepOptions} {...props}/>;
};

const stepId = ({_wizard: {wizardInfo}, isTargetStepFromSameSourceType}) => {
    return calculateStepId(wizardInfo, isTargetStepFromSameSourceType, "nfs");
};

const willMount = (props, currentThis) => {
    const {isTargetStepFromSameSourceType, endpointObj, protocolAndProviderKey, _ui: {isCmIntegration}, _wizard: {selectedWizardParams}} = props;

    if (getIsIntegratedStep(isCmIntegration, endpointObj)) {
        const {selectedAnfWe, selectedAnfWeTarget} =  selectedWizardParams;
        const relevantAnfWe = isTargetStepFromSameSourceType ? selectedAnfWeTarget : selectedAnfWe;
        const isNeedForTimeout = !relevantAnfWe;
        if (isNeedForTimeout) setTimeout(fetchAnfVolumes.bind(this, currentThis, props), 300);
        else fetchAnfVolumes.bind(this, currentThis, props)();
    }
    else {
        if (!props._servers || !props._servers[protocolAndProviderKey] || !props._servers[protocolAndProviderKey].succeeded) {
            return props.getServersByType(endpointObj.protocol, endpointObj.provider, protocolAndProviderKey);
        }
    }
};

const isFetching = (props) => {
    const {_ui: {isCmIntegration}, endpointObj} = props;
    if (getIsIntegratedStep(isCmIntegration, endpointObj)) {
        const {_anfVolumes, _wizard: {selectedWizardParams}, isTargetStepFromSameSourceType} = props;
        const {selectedAnfWe, selectedAnfWeTarget} =  selectedWizardParams;
        const relevantAnfWe = isTargetStepFromSameSourceType ? selectedAnfWeTarget : selectedAnfWe;
        return _anfVolumes[relevantAnfWe] && _anfVolumes[relevantAnfWe].inProgress;
    }
    else {
        const {_servers, protocolAndProviderKey} = props;
        return _servers[protocolAndProviderKey].inProgress;
    }
};

const canShowNextStep = ({_wizard, isTargetStepFromSameSourceType}) => {
    if (!_wizard) return false;
    return isTargetStepFromSameSourceType ? _wizard.selectedWizardParams.selectedNFSTarget : _wizard.selectedWizardParams.selectedNFS;
};
const nextRoutes = ({match, isTargetStepFromSameSourceType, nextStep, NextStepComp, _ui: {isCmIntegration}, endpointObj}) => {
    const isFullAnfStep = getIsIntegratedStep(isCmIntegration, endpointObj);

    return (
        <Switch>
            {isTargetStepFromSameSourceType &&
            <Route path={`${match.url}/:nfsServerTarget/:targetNfsVersion${isFullAnfStep ? "/:anfVolTarget/:rootExportTarget" : ""}/${nextStep.path}`} render={(props) =>
                <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
            }/>}
            {!isTargetStepFromSameSourceType &&
            <Route path={`${match.url}/:nfsServer/:nfsVersion${isFullAnfStep ? "/:anfVol/:rootExport" : ""}/${nextStep.path}`} render={(props) =>
                <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
            }/>}
        </Switch>
    )
};

const NFSServerStepForm = reduxForm({
    form: 'nfsServer',
    validate
})(NFSServerStep);

const mapStateToProps = (state, ownProps) => {
    return {
        _servers: buildServersList(state),
        _wizard: getWizardParamsFromUrlParams(state, ownProps),
        _anfVolumes:  getPropFromState(state, "_anfVolumes", {}),
        _accountInfo: state.global._accountInfo,
        _relationships: getPropFromState(state, "_relationships", {}),
        _ui: {...state.syncReducer._ui, ...state.global._ui},
        routing: state.router,
        ...ownProps
    }
};

const mapDispatchToProps = {
    getServersByType,
    getAnfVolumes,
    serverIPStep,
    serverCredentialsStep,
    clearUrlParamsIfChanged,
    clearShowAction,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
};

export default connect(mapStateToProps, mapDispatchToProps)(stepinize(NFSServerStepForm, {
    stepId,
    willMount,
    isFetching,
    canShowNextStep,
    nextRoutes,
    loading: ({endpointObj}) => {
        const serverType = endpointObj.provider || endpointObj.protocol;
        return {
            text: `${END_POINT_NAMES_MAP[serverType.toUpperCase()]}`,
            icon: require(`../../assets/svgs/${serverType}-default.svg`)
        }
    }
}));
