import React, {Component} from 'react';
import {
    showS3BasedBuckets,
    listS3BasedBucketFolders,
    clearFoldersCache,
    closeSearchWidget,
    clearCloseSearchWidget,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
} from '../../store/wizard/action_creators';
import {connect} from 'react-redux';
import stepinize from "./../wizard/stepinize";
import TagsPopover from "./../widgets/TagsPopover";
import {reduxForm} from 'redux-form';
import {Route, Switch} from 'react-router-dom';
import ShowObjectStorageStepTemplate from "./ShowObjectStorageStepTemplate"
import {END_POINT_NAMES_MAP} from '../../consts';
import {calculateStepId} from '../../utils/sync-utils';
import _ from "lodash";
import {getPropFromState, getWizardParamsFromUrlParams} from '../../utils/mapStateToPropsUtils';

require("./showObjectStorageStep.scss");

class S3BasedBucketsStep extends Component{
    static innerColumns(row) {
        const {region, tags} = row;
        return region && tags ? [
            <td className="path" key="path">
                <div className="text" title={row.path}>{row.path}</div>
                <div className="title">Name</div>
            </td>,
            <td className="region" key="region">
                <div className="text">{region.displayName}|{region.name}</div>
                <div className="title">Region</div>
            </td>,
            <TagsPopover tags={tags} inTable={true} leftAdjust={-470} key="tags"/>,
        ] : [<td className="path" title={row.path} key="path">{row.path}</td>];
    }

    callListS3BasedBucketFoldersAction(dataBrokerId, bucket, path) {
        const {listS3BasedBucketFolders, endpointObj, isTargetStepFromSameSourceType, _wizard: {selectedWizardParams: {selectedS3Based, selectedS3BasedTarget}}} = this.props;
        const relevantSelected = isTargetStepFromSameSourceType ? selectedS3BasedTarget : selectedS3Based;
        listS3BasedBucketFolders(dataBrokerId, relevantSelected, bucket, path, isTargetStepFromSameSourceType, endpointObj.provider);
    }

    render(){
        const {clearFoldersCache, _showS3BasedBuckets, endpointObj, isTargetStepFromSameSourceType, _wizard: {selectedWizardParams: {selectedS3BasedBucketPath, selectedS3BasedBucketPathTarget, selectedS3Based, selectedS3BasedTarget}}} = this.props;
        const providerUpper = endpointObj.provider ? _.toUpper(endpointObj.provider) : _.toUpper(endpointObj.protocol);
        const storageType = END_POINT_NAMES_MAP[providerUpper];
        const containerOptions = {
            singularTitle: "Bucket",
            pluralTitle: "Buckets",
            itemIdentifier: "actualPath",
            rootName: "All Buckets",
            targetName: "Define Target Folder",
            directoriesName: "Folders",
            directoryName: "folder",
            icon: require("./../../assets/svgs/bucket-icon.svg")
        };

        const relevantSelected = isTargetStepFromSameSourceType ? selectedS3BasedTarget : selectedS3Based;
        const relevantBuckets = _showS3BasedBuckets[relevantSelected];

        const stepOptions = {
            selectedBaseObject: isTargetStepFromSameSourceType ? selectedS3BasedBucketPathTarget : selectedS3BasedBucketPath,
            title: "Select a Bucket",
            failedMessage: "Unable to retrieve buckets",
            notFoundMessage: "No buckets were found",
            bottomMessage: `in your ${storageType}`,
            storage: relevantBuckets,
            getStorageByBrokerId: false,
            listSubFoldersAction: this.callListS3BasedBucketFoldersAction.bind(this),
            storageObjectName: "Bucket",
            storageObjectType: storageType,
            innerColumns: S3BasedBucketsStep.innerColumns,
            stepFormName: "s3BasedBucketsForm",
            containerOptions,
            supportAddStorageObject: false,
            innerPathBeginWithSlash: true,
            refreshStep:() => {clearFoldersCache({cacheName:'_showS3BasedBuckets',storageKey:relevantSelected}); fetchS3BasedBuckets(this, this.props)}
        };

        return <ShowObjectStorageStepTemplate stepOptions={stepOptions} {...this.props}/>;
    }
}

const stepId = ({_wizard: {wizardInfo}, isTargetStepFromSameSourceType}) => {
    return calculateStepId(wizardInfo, isTargetStepFromSameSourceType, "s3", "Buckets");
};

const fetchS3BasedBuckets = (localThis, props) => {
    const localSelected = props.isTargetStepFromSameSourceType ? localThis.props._wizard.selectedWizardParams.selectedS3BasedTarget : localThis.props._wizard.selectedWizardParams.selectedS3Based;
    const provider = props.endpointObj.provider;
    return props.showS3BasedBuckets(localThis.props._wizard.selectedWizardParams.selectedDataBroker.id, localSelected, props.isTargetStepFromSameSourceType, provider);
};

const willMount = (props, currentThis) => {
    const {_showS3BasedBuckets, isTargetStepFromSameSourceType,
        _wizard : {selectedWizardParams: {selectedS3BasedTarget, selectedS3Based, selectedDataBroker}}} = props;

    const relevantSelected = isTargetStepFromSameSourceType ? selectedS3BasedTarget : selectedS3Based;
    const buckets = (_showS3BasedBuckets[relevantSelected] && _showS3BasedBuckets[relevantSelected].dataBrokerId ===  _.get(selectedDataBroker, 'id')) ?
        _showS3BasedBuckets[relevantSelected] : undefined;

    if (!buckets || buckets.failedRetrieve) {
        const isNeedForTimeout = !props._wizard.selectedWizardParams.selectedDataBroker || !relevantSelected;
        if (isNeedForTimeout)
            setTimeout(fetchS3BasedBuckets.bind(this, currentThis, props), 200);
        else fetchS3BasedBuckets.bind(this, currentThis, props)();
    }
};

const isFetching = (props) => {
    const {_showS3BasedBuckets} = props;
    const relevantSelected = props.isTargetStepFromSameSourceType ? props._wizard.selectedWizardParams.selectedS3BasedTarget : props._wizard.selectedWizardParams.selectedS3Based;
    return !_showS3BasedBuckets[relevantSelected] || (_showS3BasedBuckets[relevantSelected].inProgress  ||  ((!_showS3BasedBuckets[relevantSelected].data || _showS3BasedBuckets[relevantSelected].data.length === 0) && !_showS3BasedBuckets[relevantSelected].retrievedAll));
};

const canShowNextStep = (props) => {
    const {_showS3BasedBuckets} = props;
    const relevantSelected = props.isTargetStepFromSameSourceType ? props._wizard.selectedWizardParams.selectedS3BasedTarget : props._wizard.selectedWizardParams.selectedS3Based;
    const relevantSelectedPath = props.isTargetStepFromSameSourceType ? props._wizard.selectedWizardParams.selectedS3BasedBucketPathTarget : props._wizard.selectedWizardParams.selectedS3BasedBucketPath;
    return _showS3BasedBuckets[relevantSelected] && (_showS3BasedBuckets[relevantSelected].succeeded && _showS3BasedBuckets[relevantSelected].data && _showS3BasedBuckets[relevantSelected].data.length > 0 && relevantSelectedPath);
};

const nextRoutes = ({match, isTargetStepFromSameSourceType, nextStep, NextStepComp}) => {
    return (
        <Switch>
            {isTargetStepFromSameSourceType &&
                <Route path={`${match.url}/:s3BasedBucketPathTarget/${nextStep.path}`} render={(props) =>
                    <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
                }/>}
            {!isTargetStepFromSameSourceType &&
                <Route path={`${match.url}/:s3BasedBucketPath/${nextStep.path}`} render={(props) =>
                    <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
                }/>}
        </Switch>
    )
};

const S3BasedBucketsForm = reduxForm({form: 's3BasedBucketsForm'})(S3BasedBucketsStep);

const mapStateToProps = (state, ownProps) => {
    return {
        _showS3BasedBuckets:  getPropFromState(state, "_showS3BasedBuckets", {}),
        _wizard: getWizardParamsFromUrlParams(state, ownProps),
        _ui: {...state.syncReducer._ui, ...state.global._ui},
        routing: state.router,
        ...ownProps
    }
};

const mapDispatchToProps = {
    listS3BasedBucketFolders,
    showS3BasedBuckets,
    clearFoldersCache,
    closeSearchWidget,
    clearCloseSearchWidget,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
};

export default connect(mapStateToProps, mapDispatchToProps)(stepinize(S3BasedBucketsForm, {stepId, willMount, isFetching, canShowNextStep, nextRoutes,
    loading: ({endpointObj}) => {
        const serverType = endpointObj.provider || endpointObj.protocol;
        return {
            text: "Buckets",
            icon: require(`./../../assets/svgs/${serverType}-default.svg`)
        }
    }
}));
