import React, {Component} from 'react';
import {
    showExports,
    addNewExport,
    listFolders,
    showEFSExports,
    clearFoldersCache,
    closeSearchWidget,
    clearCloseSearchWidget,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
} from '../../store/wizard/action_creators'
import {connect} from 'react-redux';
import {encodeAll} from "../../utils/encoders"
import stepinize from "./../wizard/stepinize"
import {reduxForm} from 'redux-form';
import _ from "lodash";
import {Route, Switch} from 'react-router-dom';
import ShowDirectoriesStepTemplate from "./ShowDirectoriesStepTemplate";
import {ManualDirectoryForm} from "./../systemFlow/ManualDirectoryDialog";
import {getIsIntegratedStep} from '../../utils/wizard-utils';
import {
    getPropFromState,
    getWizardParamsFromUrlParams
} from '../../utils/mapStateToPropsUtils';

class ShowExportsStep extends Component {

    addManualExport(values) {
        const {_wizard: {isSecure, selectedWizardParams: {selectedNFS, selectedNFSTarget, selectedDataBroker, selectedDataBrokerTarget}}, isTargetStepFromSameSourceType} = this.props;
        const isUsingDataBrokerTarget = isSecure && isTargetStepFromSameSourceType;
        const actualDataBrokerId = isUsingDataBrokerTarget ? selectedDataBrokerTarget.id : selectedDataBroker.id;
        const currentNFS = isTargetStepFromSameSourceType ? selectedNFSTarget : selectedNFS;
        this.props.addNewExport(values.directoryName, encodeAll(currentNFS), actualDataBrokerId);
        this.setState({selectedId: values.directoryName, rootValue: values.directoryName, error: null});
    }

    static innerColumns(row) {
        //SER-306 SER- Wrong details in select directory screen - don't show information at all
        return [<td className="path" title={row.path} key="path">{row.path}</td>];
    }

    render() {
        const {
            clearFoldersCache, _showExports, listFolders, isTargetStepFromSameSourceType, endpointObj, _ui: {isCmIntegration},
            _wizard: {isSecure, selectedWizardParams}} = this.props;
        const {selectedExport, selectedRootExport, selectedExportTarget, selectedRootExportTarget,
            selectedNFSTarget, selectedNFS, nfsVersion, targetNfsVersion, selectedDataBroker, selectedDataBrokerTarget} = selectedWizardParams;
        const isUsingDataBrokerTarget = isSecure && isTargetStepFromSameSourceType;
        const actualDataBrokerId = isUsingDataBrokerTarget ? selectedDataBrokerTarget.id : selectedDataBroker.id;
        const actualServer = isTargetStepFromSameSourceType ? selectedNFSTarget : selectedNFS;
        const actualNfsVersion = isTargetStepFromSameSourceType ? targetNfsVersion : nfsVersion;
        const storageKey = `${encodeAll(actualServer)}-${actualDataBrokerId}`;
        const stepOptions = {
            containerOptions: {
                singularTitle: "Directory",
                pluralTitle: "Directories",
                itemIdentifier: "actualPath",
                rootName: "All Directories",
                targetName: "Define Target Directory",
                directoriesName: "Directories",
                directoryName: "Directory",
                icon: require(`./../../assets/svgs/directory-no-circle.svg`)
            },
            failedMessage: "Unable to retrieve directories",
            notFoundMessage: "No directories were found",
            title: `Select ${isTargetStepFromSameSourceType ? "Target" : "Source"} Directory`,
            selectedDirectory: selectedExport,
            selectedRootDirectory: selectedRootExport,
            selectedDirectoryTarget: selectedExportTarget,
            selectedRootDirectoryTarget: selectedRootExportTarget,
            isStaticRoot: getIsIntegratedStep(isCmIntegration, endpointObj),
            storage: _showExports,
            storageKey,
            listSubDirectoriesAction: listFolders,
            selectedServer: selectedNFS,
            selectedServerTarget: selectedNFSTarget,
            additionalParam: actualNfsVersion,
            stepFormName: "exportsForm",
            innerColumns: ShowExportsStep.innerColumns,
            addManualOptions: {
                manualForm: ManualDirectoryForm,
                directoryType: "export",
                addManualAction: this.addManualExport,
                addManualText: "Add Export Manually"
            },
            refreshStep: () => {clearFoldersCache({cacheName:'_showExports',storageKey}); fetchExports(this, this.props)}
        };
        return <ShowDirectoriesStepTemplate stepOptions={stepOptions} {...this.props}/>
    }
}

//This functions needs to use the updated "this" of stepinize in order to get access to the updated props (calculated by "mapStateToProps")
//otherwise selectedNFSTarget is not defined (jira CS-4168)
const fetchExports = (localThis, props) => {
    const { endpointObj, isTargetStepFromSameSourceType } = props;
    const localSelectedNFSProp = isTargetStepFromSameSourceType ? localThis.props._wizard.selectedWizardParams.selectedNFSTarget : localThis.props._wizard.selectedWizardParams.selectedNFS;

    if (endpointObj.provider === "efs")
        return props.showEFSExports(localThis.props._wizard.selectedWizardParams.selectedDataBroker.id, localSelectedNFSProp);
    else if (isTargetStepFromSameSourceType && localThis.props._wizard.selectedWizardParams.selectedDataBrokerTarget) {
        return props.showExports(localThis.props._wizard.selectedWizardParams.selectedDataBrokerTarget.id, localSelectedNFSProp);
    } else return props.showExports(localThis.props._wizard.selectedWizardParams.selectedDataBroker.id, localSelectedNFSProp);
};

const willMount = (props, currentThis) => {
    const {
        _showExports, isTargetStepFromSameSourceType,
        _wizard: {isSecure, selectedWizardParams: {selectedNFSTarget, selectedNFS, selectedDataBroker, selectedDataBrokerTarget}}
    } = props;
    const encodedNFS = isTargetStepFromSameSourceType ? encodeAll(selectedNFSTarget) : encodeAll(selectedNFS);
    const isUsingDataBrokerTarget = isSecure && isTargetStepFromSameSourceType;
    const dataBroker = isUsingDataBrokerTarget ? selectedDataBrokerTarget : selectedDataBroker;
    const showExportsKey = `${encodedNFS}-${dataBroker.id}`;
    const showExportsForNFS = _showExports[showExportsKey];

    if (!showExportsForNFS || showExportsForNFS.failedRetrieve ) {

        const isMissingDataBrokerId = (!isUsingDataBrokerTarget && !_.get(props, "_wizard.selectedWizardParams.selectedDataBroker.id")) ||
            (isUsingDataBrokerTarget && !_.get(props, "_wizard.selectedWizardParams.selectedDataBrokerTarget.id"));
        const selectedNFSProp = isTargetStepFromSameSourceType ? selectedNFSTarget : selectedNFS;
        const isNeedForTimeout = isMissingDataBrokerId || !selectedNFSProp;

        if (isNeedForTimeout) setTimeout(fetchExports.bind(this, currentThis, props), 300);
        else fetchExports.bind(this, currentThis, props)();
    }
};

const stepId = ({isTargetStepFromSameSourceType}) => {
    return isTargetStepFromSameSourceType ? 'targetDirectories' : 'directories';
};

const isFetching = (props) => {
    const {_showExports, _wizard: {isSecure, selectedWizardParams: {selectedNFS, selectedNFSTarget, selectedDataBroker, selectedDataBrokerTarget}}, isTargetStepFromSameSourceType} = props;
    const encodedNFS = isTargetStepFromSameSourceType ? encodeAll(selectedNFSTarget) : encodeAll(selectedNFS);
    const isUsingDataBrokerTarget = isSecure && isTargetStepFromSameSourceType;
    const dataBroker = isUsingDataBrokerTarget ? selectedDataBrokerTarget : selectedDataBroker;
    const showExportsKey = `${encodedNFS}-${dataBroker.id}`;
    const showExportsForNFS = _showExports[showExportsKey];
    if (!showExportsForNFS) return true;
    return (showExportsForNFS.inProgress) || ((!showExportsForNFS.data || showExportsForNFS.data.length === 0) && !showExportsForNFS.retrievedAll);
};
const canShowNextStep = (props) => {
    const {_showExports, _wizard: {isSecure, selectedWizardParams: {selectedNFS, selectedNFSTarget, selectedDataBrokerTarget, selectedDataBroker, selectedExport, selectedRootExport, selectedExportTarget, selectedRootExportTarget}, isTargetStepFromSameSourceType}} = props;
    const encodedNFS = isTargetStepFromSameSourceType ? encodeAll(selectedNFSTarget) : encodeAll(selectedNFS);
    const isUsingDataBrokerTarget = isSecure && isTargetStepFromSameSourceType;
    const dataBroker = isUsingDataBrokerTarget ? selectedDataBrokerTarget : selectedDataBroker;
    const showExportsKey = `${encodedNFS}-${dataBroker.id}`;
    const showExportsForNFS = _showExports[encodeAll(showExportsKey)];
    if (!showExportsForNFS) return false;
    const actualExport = isTargetStepFromSameSourceType ? selectedExportTarget : selectedExport;
    const actualRootExport = isTargetStepFromSameSourceType ? selectedRootExportTarget : selectedRootExport;
    return showExportsForNFS.succeeded && showExportsForNFS.data && showExportsForNFS.data.length > 0 && actualExport && actualRootExport;
};


const nextRoutes = ({match, isTargetStepFromSameSourceType, nextStep, NextStepComp}) => {
    return (
        <Switch>
            {isTargetStepFromSameSourceType &&
            <Route path={`${match.url}/:exportPathTarget/:rootExportTarget/${nextStep.path}`} render={(props) =>
                <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
            }/>}
            {!isTargetStepFromSameSourceType &&
            <Route path={`${match.url}/:exportPath/:rootExport/${nextStep.path}`} render={(props) =>
                <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
            }/>}
        </Switch>
    )
};

const mapStateToProps = (state, ownProps) => {
    return {
        _wizard: getWizardParamsFromUrlParams(state, ownProps),
        _showExports:  getPropFromState(state, "_showExports", {}),
        _ui: {...state.syncReducer._ui, ...state.global._ui},
        routing: state.router,
        ...ownProps
    }
};

const mapDispatchToProps = {
    showExports,
    showEFSExports,
    addNewExport,
    listFolders,
    clearFoldersCache,
    closeSearchWidget,
    clearCloseSearchWidget,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
};

const ExportsForm = reduxForm({form: 'exportsForm'})(ShowExportsStep);
export default connect(mapStateToProps, mapDispatchToProps)(stepinize(ExportsForm, {
    stepId,
    willMount,
    isFetching,
    canShowNextStep,
    nextRoutes,
    loading: () => {
        return {text: "Directories", icon: require("./../../assets/svgs/directory-default.svg")}
    }
}));
