import React, {Component} from 'react';
import {
    showShares,
    addNewShare,
    listShareFolders,
    clearFoldersCache,
    closeSearchWidget,
    clearCloseSearchWidget,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
} from '../../store/wizard/action_creators'
import {connect} from 'react-redux';
import {encodeAll} from "../../utils/encoders"
import stepinize from "./../wizard/stepinize"
import {reduxForm} from 'redux-form';
import {Route, Switch} from 'react-router-dom';
import ShowDirectoriesStepTemplate from "./ShowDirectoriesStepTemplate";
import {ManualDirectoryForm} from '../systemFlow/ManualDirectoryDialog';
import {getPropFromState, getWizardParamsFromUrlParams} from '../../utils/mapStateToPropsUtils';

class ShowSharesStep extends Component {

    addManualShare(closeDialog,values){
        const {_wizard: {selectedWizardParams: {selectedCIFS, selectedCIFSTarget, selectedDataBroker}}, isTargetStepFromSameSourceType} = this.props;
        const currentCIFS = isTargetStepFromSameSourceType ? selectedCIFSTarget: selectedCIFS;
        this.props.addNewShare(values.directoryName, encodeAll(currentCIFS), selectedDataBroker.id);
        closeDialog();
        this.setState({selectedId: values.directoryName, rootValue: values.directoryName, error: null});
    }

    static innerColumns(row) {
        //SER-306 SER- Wrong details in select directory screen - don't show information at all
        return [<td className="path" title={row.path} key="path">{row.path}</td>];
    }
    render() {
        const {clearFoldersCache, _showShares, listShareFolders, isTargetStepFromSameSourceType,
            _wizard: {selectedWizardParams: {selectedCIFS, selectedCIFSTarget, selectedDataBroker, selectedShare, selectedShareTarget, selectedRootShare, selectedRootShareTarget, cifsVersion, targetCifsVersion}}} = this.props;
        const actualCifsVersion = isTargetStepFromSameSourceType ? targetCifsVersion : cifsVersion;
        const actualCifsServer = isTargetStepFromSameSourceType ? selectedCIFSTarget : selectedCIFS;
        const storageKey = `${encodeAll(actualCifsServer)}-${selectedDataBroker.id}`;

        const stepOptions = {
            containerOptions: {
                singularTitle: "Share",
                pluralTitle: "Shares",
                itemIdentifier: "actualPath",
                rootName: "All Shares",
                targetName: "Define Target Share",
                directoriesName: "Directories",
                directoryName: "directory",
                icon: require(`./../../assets/svgs/directory-no-circle.svg`)
            },
            failedMessage: "Unable to retrieve shares",
            notFoundMessage: "No shares were found",
            title: `Select ${isTargetStepFromSameSourceType ? "Target" : "Source"} Share`,
            selectedDirectory: selectedShare,
            selectedRootDirectory: selectedRootShare,
            selectedDirectoryTarget: selectedShareTarget,
            selectedRootDirectoryTarget: selectedRootShareTarget,
            additionalParam: actualCifsVersion,
            storage: _showShares,
            storageKey,
            listSubDirectoriesAction: listShareFolders,
            selectedServer: selectedCIFS,
            selectedServerTarget: selectedCIFSTarget,
            stepFormName: "sharesForm",
            innerColumns: ShowSharesStep.innerColumns,
            addManualOptions: {
                manualForm: ManualDirectoryForm,
                directoryType: "share",
                addManualAction: this.addManualShare,
                addManualText: "Add Share Manually"
            },
            refreshStep: () => {clearFoldersCache({cacheName:"_showShares",storageKey}); fetchShares(this, this.props)}
        };
        return <ShowDirectoriesStepTemplate stepOptions={stepOptions} {...this.props}/>
    }
}

const fetchShares = (localThis, props) => {
    const {isTargetStepFromSameSourceType} = props;
    const localSelectedCIFS = isTargetStepFromSameSourceType ? localThis.props._wizard.selectedWizardParams.selectedCIFSTarget : localThis.props._wizard.selectedWizardParams.selectedCIFS;
    const localCifsVersion = isTargetStepFromSameSourceType ? localThis.props._wizard.selectedWizardParams.targetCifsVersion : localThis.props._wizard.selectedWizardParams.cifsVersion;
    return props.showShares(localThis.props._wizard.selectedWizardParams.selectedDataBroker.id, localSelectedCIFS, localCifsVersion, isTargetStepFromSameSourceType);
};

const willMount = (props, currentThis) => {
    const {_showShares, isTargetStepFromSameSourceType,
        _wizard: {selectedWizardParams: {selectedCIFSTarget, selectedCIFS, selectedDataBroker}}} = props;
    const encodedCIFS = isTargetStepFromSameSourceType ? encodeAll(selectedCIFSTarget) : encodeAll(selectedCIFS);
    const showSharesKey = `${encodedCIFS}-${selectedDataBroker.id}`;
    const showSharesForCIFS = _showShares[showSharesKey];

    //if the exports are already there, don't bring them again, otherwise the earlier step of ShowExports will think it should be mounted again, causing an endless loop
    if (!showSharesForCIFS || showSharesForCIFS.failedRetrieve) {
        const isNeedForTimeout = !props._wizard.selectedWizardParams.selectedDataBroker || !encodedCIFS;
        if (isNeedForTimeout) setTimeout(fetchShares.bind(this, currentThis,props), 200);
        else fetchShares.bind(this, currentThis, props)();
    }
};
const isFetching = (props) => {
    const {_showShares, _wizard: {selectedWizardParams: {selectedCIFS, selectedCIFSTarget, selectedDataBroker}}, isTargetStepFromSameSourceType} = props;
    const encodedCIFS = isTargetStepFromSameSourceType ? encodeAll(selectedCIFSTarget) : encodeAll(selectedCIFS);
    const showSharesKey = `${encodedCIFS}-${selectedDataBroker.id}`;
    const showSharesForCIFS = _showShares[showSharesKey];
    if (!showSharesForCIFS) return true;
    return (showSharesForCIFS.inProgress) || ((!showSharesForCIFS.data || showSharesForCIFS.data.length === 0) && !showSharesForCIFS.retrievedAll);
};

const stepId = ({isTargetStepFromSameSourceType}) => {
    return isTargetStepFromSameSourceType ? 'targetShares' : 'shares';
};

const canShowNextStep = (props) => {
    const {_showShares, _wizard: {selectedWizardParams: {selectedCIFSTarget, selectedCIFS, selectedDataBroker, selectedShare, selectedRootShare,selectedShareTarget, selectedRootShareTarget}}, isTargetStepFromSameSourceType} = props;
    const encodedCIFS = isTargetStepFromSameSourceType ? encodeAll(selectedCIFS) : encodeAll(selectedCIFSTarget);
    const showSharesKey = `${encodedCIFS}-${selectedDataBroker.id}`;
    const showSharesForCIFS = _showShares[showSharesKey];
    if (!showSharesForCIFS) return false;
    const actualShare = isTargetStepFromSameSourceType ? selectedShareTarget : selectedShare;
    const actualRootShare = isTargetStepFromSameSourceType ? selectedRootShareTarget : selectedRootShare;
    return showSharesForCIFS.succeeded && showSharesForCIFS.data && showSharesForCIFS.data.length > 0 && actualShare && actualRootShare;
};


const nextRoutes = ({match, isTargetStepFromSameSourceType, nextStep, NextStepComp}) => {

    return (
        <Switch>
            {isTargetStepFromSameSourceType &&
                <Route path={`${match.url}/:sharePathTarget/:rootShareTarget/${nextStep.path}`} render={(props) =>
                    <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
                }/> }
            {!isTargetStepFromSameSourceType &&
                <Route path={`${match.url}/:sharePath/:rootShare/${nextStep.path}`} render={(props) =>
                    <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
                }/>}
        </Switch>
    )
};


const SharesForm = reduxForm({form: 'sharesForm'})(ShowSharesStep);

const mapStateToProps = (state, ownProps) => {
    return {
        _wizard: getWizardParamsFromUrlParams(state, ownProps),
        _showShares:  getPropFromState(state, "_showShares", {}),
        _ui: {...state.syncReducer._ui, ...state.global._ui},
        routing: state.router,
        ...ownProps
    }
};

const mapDispatchToProps = {
    showShares,
    addNewShare,
    listShareFolders,
    clearFoldersCache,
    closeSearchWidget,
    clearCloseSearchWidget,
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
};

export default connect(mapStateToProps, mapDispatchToProps)(stepinize(SharesForm, {
    stepId,
    willMount,
    isFetching,
    canShowNextStep,
    nextRoutes,
    loading: () => {return {text: "Shares", icon: require("./../../assets/svgs/directory-default.svg")}}
}));
