import React, {useState} from 'react';
import {connect} from 'react-redux';
import stepinize from "./../wizard/stepinize";
import HowDoesItWorkButton from "./../systemFlow/HowDoesItWorkButton";
import classNames from "classnames";
import {Route, Switch} from 'react-router-dom';
import {
    getFormValuesFromSettings,
    getSettingsFromSessionStorage,
    saveSettingsToSessionStorage
} from '../../utils/settings-utils';
import {getCredentialsByType} from '../../utils/sync-utils';
import SyncSettingsForm from '../tabs/syncSettings/SyncSettingsForm';
import ButtonWithNotification from '../widgets/ButtonWithNotification';
import useRunOnce from '../../hooks/useRunOnce';
import _ from "lodash";
import {getWizardParamsFromUrlParams} from '../../utils/mapStateToPropsUtils';
import {updateUrlParams, wizardStepExited, wizardStepStarted} from '../../store/wizard/action_creators';

const SyncSettingsStep = (props) => {
    const { _wizard: {wizardTypeForSystemFlow, isSecure, wizardInfo}, up} = props;

    const [initialValues, setInitialValues] = useState();

    useRunOnce(() => {
        const initialSettings = getSettingsFromSessionStorage();
        const initialFormValues = getFormValuesFromSettings(initialSettings, false);
        setInitialValues(initialFormValues);
    });

    const onContinue = () => {
        const nextStep = props.nextStep;
        props._wizard.nextStep(`/${nextStep.path}`, props.history, true);
    };

    const doSubmit = (settings) => {
        //In the wizard, copyAcl is defined in the source step for cifs-cifs relationship
        const { selectedCIFS, selectedCIFSTarget } = props._wizard.selectedWizardParams;
        let copyAcl = {copyAcl: false};
        if (selectedCIFS && selectedCIFSTarget) {
            copyAcl = getCredentialsByType('cifs', selectedCIFS, false, ['serverName', 'username', 'password'])
        }
        const settingsWithAcl = Object.assign({}, settings, copyAcl);

        saveSettingsToSessionStorage(settingsWithAcl);
        onContinue();
    };

    const showObjectTaggingSection = wizardInfo.target.protocol === "s3" && _.isNil(wizardInfo.target.provider);

    return (<div className="wizard-step settings-step">
        <div className={classNames("scrollable-area", {up, down: !up})}>
            <div className="title-and-actions">
                <div className="actions-left">
                    <HowDoesItWorkButton wizardType={wizardTypeForSystemFlow} isSecure={isSecure}/>
                </div>
                <div className="title-middle">
                    <h3>Sync Relationship Settings</h3>
                </div>
                <div className="actions-right"/>
            </div>
            <div className="header-text">You can modify settings that define how source files and folders are synced and maintained
                in the target location</div>

            <SyncSettingsForm doSubmit={doSubmit}
                              initialValues={initialValues}
                              showObjectTaggingSection={showObjectTaggingSection}/>
        </div>
        <ButtonWithNotification type="submit" form="syncSettings" text="Continue"/>
    </div>)
};

const willMount = () => {};
const isFetching = () => false;
const canShowNextStep = () => true;

const nextRoutes = ({match, nextStep, NextStepComp}) => {
    return (
        <Switch>
            <Route path={`${match.url}/${nextStep.path}`} render={(props) =>
                <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
            }/>
        </Switch>
    )
};

const mapStateToProps = (state, ownProps) => {
    return {
        _wizard: getWizardParamsFromUrlParams(state, ownProps),
        _ui: {...state.syncReducer._ui, ...state.global._ui},
        routing: state.router,
        ...ownProps
    }
};

const mapDispatchToProps = {
    updateUrlParams,
    wizardStepStarted,
    wizardStepExited
};

export default connect(mapStateToProps, mapDispatchToProps)(stepinize(SyncSettingsStep, {
    stepId: () => 'settings',
    willMount,
    isFetching,
    canShowNextStep,
    nextRoutes
}));
