import React from 'react';
import {reduxForm, Field } from 'redux-form';
import {InputField} from "../_common_/forms/forms";
import Validator from 'validatorjs';
import {BUCKET_PERMISSIONS_DOCUMENTATION} from '../../consts'
import DialogLayout from '../dialog/DialogLayout';
import "./newBucketDialog.scss";


const rules = {
    bucketName: ['required', 'min:3', 'max:63', 'regex:/^((?!/).)*$/'  /*'regex:/^([a-z]|(\d(?!\d{0,2}\.\d{1,3}\.\d{1,3}\.\d{1,3})))([a-z\d]|(\.(?!(\.|-)))|(-(?!\.))){1,61}[a-z\d\.]$/'*/]
    //regex from http://info.easydynamics.com/blog/aws-s3-bucket-name-validation-regex
};

const validate = (values) => {
    const validator = new Validator(values, rules);
    validator.setAttributeNames({bucketName : 'Bucket Name' });
    validator.passes();
    return validator.errors.all();
};


const NewBucketDialog = ({handleSubmit, onSubmitFunction, closeDialog}) => {
    const placeholder = "Bucket Name";
    return (
        <DialogLayout
            title="Add A Bucket Manually"
            closeDialog={closeDialog}
            onConfirm={handleSubmit(onSubmitFunction)}
            confirmButtonText="Add"
        >
            <div className="row">
                <div className="medium-12 columns">
                    <div>Enter the name of a bucket that is not associated with your AWS account.</div>
                    <div>Specific permissions must be applied to the S3 bucket. </div>
                    <div>View the <a href={BUCKET_PERMISSIONS_DOCUMENTATION} rel="noopener noreferrer" target="_blank">permission requirements</a>.</div>
                </div>
            </div>
            <div className={"row bucket-name-filed"}>
                <div className={"medium-6 columns"}>
                    <Field component={InputField} className="buttomBorder" name="bucketName" placeholder={placeholder} label={placeholder}/>
                </div>
            </div>
        </DialogLayout>
    )
};

export const NewBucketForm = reduxForm({
    form: 'new-bucket',
    validate
})(NewBucketDialog);
