import React from 'react';
import Isvg from 'react-inlinesvg';
import classNames from "classnames";
import {
    FM_AGENT, END_POINT_NAMES_MAP
} from '../../consts';
import {sourceAndTargetDisplayName} from "./../../utils/sync-utils";
const indefinite = require('indefinite');
require("./systemFlow.scss");

const protocolInfo = {
    "nfs": {
        svg: <Isvg src={require("../../assets/svgs/nfs-default.svg")}/>,
        title: END_POINT_NAMES_MAP.NFS
    },
    "cifs": {
        svg: <Isvg src={require("./../../assets/svgs/cifs-default.svg")}/>,
        title: END_POINT_NAMES_MAP.CIFS
    },
    "efs": {
        svg: <Isvg src={require("./../../assets/svgs/efs-default.svg")}/>,
        title: END_POINT_NAMES_MAP.EFS
    },
    "cvs": {
        svg: <Isvg src={require("./../../assets/svgs/cvs-default.svg")}/>,
        title: END_POINT_NAMES_MAP.CVS
    },
    "anf": {
        svg: <Isvg src={require("./../../assets/svgs/anf-default.svg")}/>,
        title: END_POINT_NAMES_MAP.ANF
    },
    "s3": {
        svg: <Isvg src={require("./../../assets/svgs/s3-default.svg")}/>,
        title: END_POINT_NAMES_MAP.S3
    },
    "sgws": {
        svg: <Isvg src={require("./../../assets/svgs/sgws-default.svg")}/>,
        title: END_POINT_NAMES_MAP.SGWS
    },
    "gcp": {
        svg: <Isvg src={require("./../../assets/svgs/gcp-default.svg")}/>,
        title: `${END_POINT_NAMES_MAP.GCP}`
    },
    "ibm": {
        svg: <Isvg src={require("./../../assets/svgs/ibm-default.svg")}/>,
        title: END_POINT_NAMES_MAP.IBM
    },
    "azure": {
        svg: <Isvg src={require("./../../assets/svgs/azure-default.svg")}/>,
        title: END_POINT_NAMES_MAP.AZURE
    },
    "ontap": {
        svg: <Isvg src={require("./../../assets/svgs/ontap-default.svg")}/>,
        title: END_POINT_NAMES_MAP.ONTAP
    },
    "cvo": {
        svg: <Isvg src={require("./../../assets/svgs/cvo-default.svg")}/>,
        title: END_POINT_NAMES_MAP.CVO
    }
};

export const SystemFlow = ({children, wizardType, isSecure, isCmIntegration}) => {
    return ( <div>
            {!isSecure && <HowDoesItWork wizardType={wizardType} isCmIntegration={isCmIntegration}>{children}</HowDoesItWork>}
            {isSecure && <HowDoesItWorkSecure wizardType={wizardType}>{children}</HowDoesItWorkSecure>}
        </div>
    )
};

const HowDoesItWork = ({children, wizardType}) => {
    const splitted = wizardType.split('-');
    const source = splitted[0];
    const target = splitted[1];

    const dottedArrows = wizardType === "azure-azure" || wizardType === "s3-s3" || wizardType === "gcp-gcp";

    const sourceDisplayName = sourceAndTargetDisplayName[source];
    const targetDisplayName = sourceAndTargetDisplayName[target];
    return (

        <div className="system-flow">
            <div className="divTable">
                <div className="divTableBody">
                    <div className={classNames("divTableRow", {"bi-directional": dottedArrows})}>
                        <div className="table-cell-container">
                            {protocolInfo[source].svg}
                            <div className="title">{protocolInfo[source].title}</div>
                        </div>
                        {dottedArrows && <div className="table-cell-container arrows">
                            <div className="image">
                                <Isvg src={require("./../../assets/svgs/system-flow-bi-direction-arrows.svg")}/>
                            </div>
                        </div>}
                        <div className="table-cell-container">
                            {protocolInfo[target].svg}
                            <div className="title">{protocolInfo[target].title}</div>
                        </div>
                    </div>
                    <div className="divTableRow bottom">
                        <div className={`divTableCellBig ${dottedArrows ? "dotted" : ""}`}><span><Isvg src={dottedArrows
                                ? require("../../assets/svgs/data-broker-with-dotted-arrows.svg")
                                : require("../../assets/svgs/data-broker-with-arrows-white.svg")}/></span>
                        </div>
                    </div>
                </div>
            </div>

            <p>Create a {FM_AGENT} that has a connection to the source and target.</p>
            {!dottedArrows && <p>The {FM_AGENT} will be used as a mediator between {sourceDisplayName} and {targetDisplayName}.</p>}
            {dottedArrows && <p>In {indefinite(sourceDisplayName)} to {targetDisplayName} sync relationship, the {FM_AGENT} initiates and monitors the relationship,
                but the data copy itself is done directly by {END_POINT_NAMES_MAP[target.toUpperCase()]}.</p>}
            {children}
        </div>
    )
};

const HowDoesItWorkSecure = ({wizardType}) => {
    const split = wizardType.split('-');
    const target = split[1];
    return (
        <div className="system-flow encrypted">
            <div>
                <FlowEncrypted wizardType={wizardType}/>
            </div>

            <div className="encryption-text">
                <p>When you enable data-in-flight encryption, a data broker encrypts your data before sending it over the
                    network to a second data broker.</p>
                <p>The second data broker decrypts the data before sending it
                    to the target {protocolInfo[target].title}.</p>
                <p>The data brokers communicate over the network using an encrypted connection.</p>
            </div>
        </div>
    )
};

export const FlowEncrypted = ({wizardType}) => {
    const split = wizardType.split('-');
    const source = split[0];
    const target = split[1];
    return (
        <div className="flow-encrypted">
            <div className="end-point source">
                {protocolInfo[source].svg}
                <div className="end-point-title">Source {protocolInfo[source].title}</div>
            </div>
            <div className="end-point target">
                {protocolInfo[target].svg}
                <div className="end-point-title">Target {protocolInfo[target].title}</div>
            </div>
            <Isvg className="diagram" src={require('./../../assets/svgs/how-it-work-encrypted.svg')}/>
        </div>
    )
};
