import React, {Component} from "react";
import SelectableTable from "./SelectableTable";
import Isvg from 'react-inlinesvg';
import { NEW_TARGET_FOLDER_ID } from "../../consts";
import {Field, initialize} from 'redux-form';
require("./createFolderRow.scss");

export default class CreateFolderRow extends Component{

    componentDidMount() {
        const {initialTargetFolder, dispatch, selectionOptions, currentPath, formName} = this.props;
        if (initialTargetFolder) {
            dispatch(initialize(formName, {targetFolder: initialTargetFolder}));
            selectionOptions.selector(currentPath + "/" + NEW_TARGET_FOLDER_ID);
        }
    }

    renderCreateFolderRow(){
        const {selectionOptions, resetForm, targetPlaceholder} = this.props;
        const isSelected = selectionOptions.selectedId && selectionOptions.selectedId.endsWith(NEW_TARGET_FOLDER_ID);

        if (!isSelected) {
            return (
                <div className="create-folder-row">
                    <td key="plus-icon" className="check plus-button">
                        <span><Isvg src={require("../../assets/svgs/plus-fill-circle-icon.svg")} /></span>
                    </td>
                    <td className="path">{targetPlaceholder}</td>
                </div>
            );
        }
        else {
            return (
                <div className="create-folder-row">
                    <td key="check-icon" className="check" onClick={resetForm}>
                        <span><Isvg src={require("../../assets/svgs/checkmark-fill-circle-icon.svg")} /></span>
                    </td>
                    <td className="path target-folder-name">
                        <Field name="targetFolder" component="input"
                               autoFocus
                               placeholder={targetPlaceholder}
                               autoComplete="off"
                                /* don't un-toggle selection when clicking inside of the input */
                               onClick={ (e) => {e.stopPropagation();} }/>
                    </td>
                    <td className="dive clear-button" onClick={resetForm}>
                        {/* any click will outside of the input will clear the target folder selection and text */}
                        <button type="button">
                            <span><Isvg src={require("./../../assets/svgs/x.svg")} /></span>
                        </button>
                    </td>
                </div>
            )
        }
    }

    render() {
        const {selectionOptions, itemIdentifier, currentPath, targetName} = this.props;

        const data = [{path: currentPath, [itemIdentifier]: currentPath + '/' + NEW_TARGET_FOLDER_ID}];

        return <SelectableTable noAnimate={true} Table={{renderRow: this.renderCreateFolderRow.bind(this, selectionOptions, targetName)}}
                         data={data} itemIdentifier={itemIdentifier} selectionOptions={selectionOptions}/>

    }
}