import React, {useState, useMemo} from "react"
import SearchWidget from "./../widgets/SearchWidget"
import ViewToggleWidget from "./../widgets/ViewToggleWidget"
import ItemCountWidget from "./../widgets/ItemCountWidget"
import {defaultFilterWithoutKeys} from "./../../utils/helpers"
import SelectableTable from "./SelectableTable";

const SearchableSelectableTable = (props) => {
    const {containerOptions, Table, data, menuItems, isMenuDisabled, selectionOptions, stillLoading} = props;
    const {singularTitle, pluralTitle, viewToggle, itemIdentifier, keysToExcludeFromSearch} = containerOptions;

    const [filter, setFilter] = useState(null);

    const filteredItems = useMemo(() => filter
        ? data.filter(item => defaultFilterWithoutKeys(item, filter, keysToExcludeFromSearch))
        : data, [filter, data, keysToExcludeFromSearch]);

    return (
        <div className="searchable-selectable-table-container">
            <div>
                <ItemCountWidget className="float-left" singularTitle={singularTitle} pluralTitle={pluralTitle} itemCount={data.length} filteredItemCount={filteredItems.length}/>
                {viewToggle ? <ViewToggleWidget className="float-right" toggle={viewToggle} icon={require("./../../assets/svgs/component-view-icon.svg")}/> : null}
                <SearchWidget className="float-right" items={data} setFilter={setFilter}/>
            </div>
            <SelectableTable Table={Table} menuItems={menuItems} isMenuDisabled={isMenuDisabled} data={filteredItems} selectionOptions={selectionOptions} itemIdentifier={itemIdentifier} noAnimate={data.length > 20}/>
            {stillLoading && <div className="table-loader">
                <img src={require("./../../assets/images/table-loader.gif")} alt=""/>
            </div>}
        </div>
    )
};

export default SearchableSelectableTable;