import React, {Component} from "react"
import classNames from 'classnames';
import {MatchChildWidth} from "./../../utils/directives";
import {CSSTransition, TransitionGroup} from "react-transition-group"
import Hamburger from "./../widgets/Hamburger"
import {parentHasClass} from "./../../utils/dom"
import ReactDOM from "react-dom"
import _ from "lodash";
import Isvg from 'react-inlinesvg';
import FilterWidget from '../widgets/FilterWidget';
import PopoverInfo from '../_common_/reactPopover/PopoverInfo';

export const HEADER_TYPES = {
    "ALL": "all",
    "FILTER": "filter",
    "TD": "td"
};

export default class extends Component{
    constructor(props, context){
        super(props, context);
        this.state = {openedRowId: null, mask: false};
        this.eventListener = (el) => {
            if (el.target.nodeName === "INPUT") return; //otherwise clicking on the filter's checkbox will not work
            if(parentHasClass(el.target, "table-menu")) return;
            this.setState(Object.assign({}, this.state, {openedRowId: null}));
        }
    }

    applyFilter(filterMethod, filterName, filters) {
        this.setState({[filterName]: {filterOpen: false}, mask: false});
        filterMethod(filterName, filters, this.state.sort);
    }

    handleSort(sortMethod, column) {
        //if columns was not sorted yet, we start with ascending
        const ascending = this.state.sort && this.state.sort.column === column ? !this.state.sort.ascending : true;
        this.setState({sort: {column, ascending}});
        sortMethod(column, ascending);
    }

    UNSAFE_componentWillUpdate(newProps, newState){
        if(this.state.openedRowId && newState.openedRowId !== this.state.openedRowId && this._menuNode && this._menuNode.parentNode && this._menuNode.parentNode.style){
            this._menuNode.parentNode.style.right = "-" + this._menuNode.style.width;
        }
    }

    triggerMenu(rowId){
        if(this.state.openedRowId === rowId) this.setState(Object.assign({}, this.state, {openedRowId: null}));
        else this.setState(Object.assign({}, this.state, {openedRowId: rowId}));
    }

    componentDidMount(){
        document.body.addEventListener('click', this.eventListener);
    }

    componentWillUnmount(){
        document.body.removeEventListener('click', this.eventListener);
    }

    drawMenuCallback(el){
        if(!el) return;

        const node = ReactDOM.findDOMNode(el);
        const parent = node.parentNode;
        this._menuNode = node;

        setTimeout(()=>{
            parent.style.right = "-" + node.style.width;

            setTimeout(()=>{
                parent.style.right = "100%";
            })
        })
    }

    render(){
        const {Table, menuItems, isMenuDisabled, data, selectionOptions, itemIdentifier, noAnimate} = this.props;
        const rowIdentifier = itemIdentifier || "id";

        const headerElement = Table.getTableHeader ? Table.getTableHeader().map((cell, index) => {
            const type = cell.type ? cell.type : HEADER_TYPES.TD;
            const isFilterActive = cell.filterOptions && _.some(cell.filterOptions.filterValues, filterValue => !filterValue.checked);
            const sorted = this.state.sort && this.state.sort.column === cell.name;
            const sortedAscending = sorted && this.state.sort.ascending;
            return (
                <React.Fragment key={index}>
                    {type === HEADER_TYPES.ALL &&
                    <td key={index}
                        style={{width: cell.width, maxWidth: cell.width}}
                        className={classNames(cell.className, "all-selector", {selected: cell.allOptions.allSelected})}
                        onClick={() => cell.allOptions.toggleAll()}>
                        <div/>
                    </td>}
                    {type === HEADER_TYPES.FILTER &&
                    <td key={index} className={`${cell.className} filter-header`} style={{width: cell.width, maxWidth: cell.width}}>
                        {cell.text}
                        <button type="button" className={`filter-button ${isFilterActive ? "active" : ""}`} onClick={() => this.setState({[cell.filterOptions.filterName] : {filterOpen: true}, mask: true})}>
                            <Isvg  src={require("./../../assets/svgs/filter.svg")}/>
                        </button>
                        {cell.filterOptions && this.state[cell.filterOptions.filterName] && this.state[cell.filterOptions.filterName].filterOpen &&
                        <FilterWidget filterValues={cell.filterOptions.filterValues} allCount={cell.filterOptions.allCount}
                                      close={() => this.setState({[cell.filterOptions.filterName]: {filterOpen: false}, mask: false})}
                                      apply={this.applyFilter.bind(this, cell.filterOptions.filter, cell.filterOptions.filterName)}
                                      fixed={this.state.headerFixed}/>}
                    </td>}
                    {type === HEADER_TYPES.TD &&
                    <td key={index} className={cell.className} style={{width: cell.width, maxWidth: cell.width}}>

                        {cell.info &&
                        <PopoverInfo>{cell.info.text}</PopoverInfo>}

                        <span className={classNames(cell.sort ? "sort" : "", cell.className, this.state.sort && this.state.sort.column === cell.name ? "sorted" : "")}>{cell.text}
                            {cell.sort && <button type="button" className="sort-button" onClick={() => this.handleSort(cell.sort, cell.name)}>
                                {!sorted && <Isvg src={require("./../../assets/svgs/sort.svg")}/>}
                                {sorted && sortedAscending && <Isvg src={require("./../../assets/svgs/sort-ascending.svg")}/>}
                                {sorted && !sortedAscending && <Isvg src={require("./../../assets/svgs/sort-descending.svg")}/>}
                            </button>}</span>
                    </td>}
                </React.Fragment>
            )
        }) : null;

        const d = data.map((row, index) =>
            {
                const el = Table.renderRow(row, index);
                const disableMenu = _.isFunction(isMenuDisabled) ? isMenuDisabled(row) : false;
                const id = row[rowIdentifier];
                const inProccess = (row.activity && row.activity.isBeingDeleted);
                return (
                    <CSSTransition classNames="tr" timeout={200} key={id}>
                        <tr key={id} onClick={() => !row.isDisabled && selectionOptions ? selectionOptions.selector(id) : ""} className={classNames(el.props.className, {"menu-open": this.state.openedRowId === id, selected: selectionOptions && selectionOptions.selectedId === id, disabled: inProccess || row.isDisabled})}>
                    {el.props.children}
                    {menuItems ?
                        <td className={classNames("table-menu", {"disabled" : disableMenu || inProccess})}>
                            <Hamburger className={classNames({open: this.state.openedRowId === id})} onClick={this.triggerMenu.bind(this, id)}/>
                            <CSSTransition classNames="menu" timeout={400}>
                                {this.state.openedRowId === id ?
                                    <div className="open-menu">
                                        <MatchChildWidth component="ul" ref={this.drawMenuCallback.bind(this)} >
                                            {menuItems(row, () => this.triggerMenu(this, null)).map( (item, index) => (<li key={index}>{item}</li>))}
                                        </MatchChildWidth>
                                    </div>
                                    :
                                    <div/>
                                }
                            </CSSTransition>
                        </td>
                        :
                        null
                    }
                </tr></CSSTransition>)
            }
        );

        const classes = classNames("standard", {menuable: !!menuItems, selectable: !!selectionOptions}, Table.className);
        return (
            <div>
                <table className={classes}>
                    {Table.renderHeader && Table.renderHeader()}

                    {headerElement &&
                        <thead>
                        <tr>
                            {headerElement}
                        </tr>
                        </thead>}

                    {!noAnimate && <TransitionGroup component="tbody">{d}</TransitionGroup>}
                    {noAnimate && <tbody>{d}</tbody>}
                </table>
            </div>
        )
    }
}