import React, {useState} from "react";
import "./dataBrokerGroupDialog.scss";
import _ from "lodash";
import DataBrokerDialogCard from "./DataBrokerDialogCard";
import DialogLayout from '../dialog/DialogLayout';

const DataBrokerGroupDialog = (props) => {
    const {relationshipId, closeDialog, copyDoneByCloudProvider} = props;

    //The dialog needs to access the state directly -in order to get updates correctly - hence the relationshipId/index hack
    //This is because the dialog is a separate node in the DOM - it is not a child component that gets props updates from parent
    const index = props._relationships.data.findIndex(relationship => relationship.id === relationshipId);
    const dataBrokers = props._relationships.data[index].ui.dataBrokers;
    const defaultSelectedId = dataBrokers && dataBrokers[0].id;
    const isSingleDb = dataBrokers && dataBrokers.length === 1;

    const {_deployments} = props;
    const dataBrokersWithDeployInfo = _.map(dataBrokers, dataBroker => {
        if (dataBroker.placement) return dataBroker;
        const deployment = _.find(_deployments, {dataBrokerId: dataBroker.id});
        return {...dataBroker, deployment}
    });

    const [selectedId, setSelectedId] = useState(defaultSelectedId);

    const handleSectionClick = id => {
        if (isSingleDb) return;
        if (selectedId === id) {
            setSelectedId(null);
        } else {
            setSelectedId(id);
        }
    };

    return (
        <DialogLayout
            title="Data Broker Information"
            closeDialog={closeDialog}
        >
            {dataBrokersWithDeployInfo.map((dataBroker) => {
                    return <DataBrokerDialogCard
                        isActive={selectedId === dataBroker.id}
                        key={dataBroker.id}
                        dataBroker={dataBroker}
                        copyDoneByCloudProvider={copyDoneByCloudProvider}
                        displayOnlyHeader={selectedId !== dataBroker.id }
                        isSingleDb={isSingleDb}
                        handleSectionClick={handleSectionClick}/>
                }
            )}
        </DialogLayout>
    )
};

export default DataBrokerGroupDialog;
