import Isvg from 'react-inlinesvg';
import React from "react";
import {openDialog} from '../dialog/Dialog';
import {RESPONSE_STATUS_RUNNING} from '../../consts';
import {calcTransferRate} from '../../utils/dashboard-utils';
import DataBrokerGroupDialog from "./DataBrokerGroupDialog";
import _ from 'lodash';
import "./dataBrokerGroupInfo.scss";

const DataBrokerGroupInfo = ({dataBrokers, relationshipId, copyDoneByCloudProvider, relationshipStatus}) =>  {
    const isSingleDb = dataBrokers.length === 1;
    const summaryLine = isSingleDb ? dataBrokers[0].name : `${dataBrokers.length} data brokers`;
    const transferRateSum = _.reduce(dataBrokers, (sum, dataBroker) => {
        return dataBroker.isReady ? sum + dataBroker.transferRate : sum;
    }, 0);
    return (
        <div className="data-broker-row">
            <div className="data-broker-summary" title={summaryLine}>
                <button className="info"
                        onClick={(el) => {openDialog(<DataBrokerGroupDialog className="data-broker-group-info"
                                                                            relationshipId={relationshipId} copyDoneByCloudProvider={copyDoneByCloudProvider}>
                                                    </DataBrokerGroupDialog>, el.target);}} key="dataBrokerGroup">
                       <Isvg src={require("./../../assets/svgs/info-circle-icon.svg")}/>
                </button>
                <div className="data-broker-name">{summaryLine}</div>
            </div>
            {!copyDoneByCloudProvider && relationshipStatus === RESPONSE_STATUS_RUNNING &&
                <div className="data-broker-transfer-rate">{calcTransferRate(transferRateSum)}</div>
            }
        </div>
)};

export default DataBrokerGroupInfo;