import React from "react";
import {FM_AGENT_TYPE_AWS, FM_AGENT_TYPE_ONPREM, FM_AGENT_TYPE_AZURE, FM_AGENT_TYPE_GCP} from "../../consts";
import "./dataBrokerProperties.scss";

export default ({dataBroker: {id, placement, type, port}, shouldDisplayPort}) => {
    const renderAWSPropertiesTable = (placement, port, dataBrokerId) => (
        <table>
            <tbody>
            <tr>
                <CellContent name="Region" value={placement.region} withSeparator={true}/>
                <CellContent name="VPC" value={`${placement.vpcCidrBlock} | ${placement.vpcId}`} withSeparator={true}/>
                <CellContent name="Private IP" value={placement.privateIp} withSeparator={true}/>
                <CellContent name="Broker ID" value={dataBrokerId} withSeparator={true}/>
            </tr>
            <tr>
                <CellContent name="Availability Zone" value={placement.availabilityZone}/>
                <CellContent name="Subnet" value={`${placement.subnetCidrBlock} | ${placement.subnetId}`}/>
                <CellContent name="Instance ID" value={placement.instanceId}/>
                { shouldDisplayPort && <CellContent name="Port" value={port ? port : 0 }/>}
            </tr>
            </tbody>
        </table>
    );

    const renderAzurePropertiesTable = (placement, port, dataBrokerId) => (
        <table>
            <tbody>
            <tr>
                <CellContent name="Region" value={placement.region} withSeparator={true}/>
                <CellContent name="Resource Group" value={placement.resourceGroupName} withSeparator={true}/>
                <CellContent name="Broker ID" value={dataBrokerId} withSeparator={true}/>
            </tr>
            <tr>
                <CellContent name="Private IP" value={placement.privateIp}/>
                <CellContent name="Subnet" value={placement.subnetCidrBlock}/>
                { shouldDisplayPort && <CellContent name="Port" value={port ? port : 0 }/>}
            </tr>
            </tbody>
        </table>
    );

    const renderGcpPropertiesTable = (placement, port, dataBrokerId) => (
        <table>
            <tbody>
            <tr>
                <CellContent name="Project ID" value={placement.gcpProjectId} withSeparator={true}/>
                <CellContent name="Network" value={placement.network || '-'} withSeparator={true}/>
                <CellContent name="Public IP" value={placement.publicIp || '-'} withSeparator={true}/>
                <CellContent name="Broker ID" value={dataBrokerId} withSeparator={true}/>
            </tr>
            <tr>
                <CellContent name="Zone" value={placement.zone || '-'}/>
                <CellContent name="Subnet" value={placement.subnetCidrBlock || '-'}/>
                <CellContent name="Instance ID" value={placement.instanceId || '-'}/>
                { shouldDisplayPort && <CellContent name="Port" value={port ? port : 0 }/>}
            </tr>
            </tbody>
        </table>
    );

    const renderOnPremPropertiesTable = (placement, port, dataBrokerId) => (
        <table>
            <tbody>
                <tr>
                    <CellContent name="Hostname" value={placement.hostname} withSeparator={true}/>
                    <CellContent name="Platform" value={placement.platform} withSeparator={true}/>
                    <CellContent name="Broker ID" value={dataBrokerId} withSeparator={true}/>
                </tr>
                <tr>
                    <CellContent name="Private IPs" value={placement.privateIp}/>
                    { shouldDisplayPort && <CellContent name="Port" value={port ? port : 0 }/>}
                </tr>
            </tbody>
        </table>
    );

    return (
        <div className="data-broker-properties">
            {type === FM_AGENT_TYPE_AWS.name && renderAWSPropertiesTable(placement, port, id)}
            {type === FM_AGENT_TYPE_AZURE.name && renderAzurePropertiesTable(placement, port, id)}
            {type=== FM_AGENT_TYPE_GCP.name && renderGcpPropertiesTable(placement, port, id)}
            {type === FM_AGENT_TYPE_ONPREM.name && renderOnPremPropertiesTable(placement, port, id)}
        </div>
    );
};

const CellContent = ({name, value, withSeparator}) => (
    <td>
        <div className="property-value" title={value}>{value}</div>
        <div className="property-name">{name}</div>
        {withSeparator && <div className="property-separator"/>}
    </td>
);

