import React from 'react';
import {FM_AGENT_TYPE_AWS, FM_AGENT_TYPE_AZURE, FM_AGENT_TYPE_GCP, FM_AGENT_TYPE_ONPREM} from '../../consts';
import './dataBrokerPropertiesForDialog.scss';

const DataBrokerPropertiesForDialog = ({dataBroker}) => {
    const {type} = dataBroker;
    return (
        <div>
            {type === FM_AGENT_TYPE_AWS.name
            && renderAWSPropertiesTable(dataBroker)}

            {type === FM_AGENT_TYPE_AZURE.name
            && renderAzurePropertiesTable(dataBroker)}

            {type === FM_AGENT_TYPE_GCP.name
            && renderGcpPropertiesTable(dataBroker)}

            {(type === FM_AGENT_TYPE_ONPREM.name)
            && renderOnPremPropertiesTable(dataBroker)}
        </div>
    );
};

export default DataBrokerPropertiesForDialog;

const renderAWSPropertiesTable = ({placement, id}) => (
        <div className="data-broker-info-table">
            <div className="table-col">
                <CellContent name="Region" value={placement.region}/>
                <CellContent name="Availability Zone" value={placement.availabilityZone}/>
                <CellContent name="Instance ID" value={placement.instanceId}/>
                <CellContent name="Broker ID" value={id}/>
            </div>
            <div className="table-col">
                <CellContent name="VPC" value={`${placement.vpcCidrBlock} | ${placement.vpcId}`}/>
                <CellContent name="Subnet" value={`${placement.subnetCidrBlock} | ${placement.subnetId}`}/>
                <CellContent name="Private IP" value={placement.privateIp}/>
            </div>
        </div>
    );

const renderAzurePropertiesTable = ({placement, id}) => (
    <div className="data-broker-info-table">
        <div className="table-col">
            <CellContent name="Region" value={placement.region}/>
            <CellContent name="Private IP" value={placement.privateIp}/>
            <CellContent name="Broker ID" value={id}/>
        </div>
        <div className="table-col">
            <CellContent name="Resource Group" value={placement.resourceGroupName}/>
            <CellContent name="Subnet" value={placement.subnetCidrBlock}/>
        </div>
    </div>
);

const renderOnPremPropertiesTable = ({placement, id}) => (
    <div className="data-broker-info-table">
        <div className="table-col">
            <CellContent name="Hostname" value={placement.hostname}/>
            <CellContent name="Private IPs" value={placement.privateIp}/>
        </div>
        <div className="table-col">
            <CellContent name="Platform" value={placement.platform}/>
            <CellContent name="Broker ID" value={id}/>
        </div>
    </div>
);

const renderGcpPropertiesTable = ({placement, id}) => (
    <div className="data-broker-info-table">
        <div className="table-col">
            <CellContent name="Project ID" value={placement.gcpProjectId}/>
            <CellContent name="Public IP" value={placement.publicIp}/>
            <CellContent name="Subnet" value={placement.subnetCidrBlock}/>
            <CellContent name="Broker ID" value={id}/>
        </div>
        <div className="table-col">
            <CellContent name="Network" value={placement.network}/>
            <CellContent name="Zone" value={placement.zone}/>
            <CellContent name="Instance ID" value={placement.instanceId}/>
        </div>
    </div>
);

const CellContent = ({name, value}) => (<div className="cell-content">
        <div className="property-name">{name}:</div>
        <div className="property-value" title={value}>{value}</div>
    </div>
);
