import React, {Component} from "react";
import {connect} from 'react-redux';
import TimelineTable from './TimelineTable';
import {SWITCH} from "./../../utils/directives"
import LoadingBar from "../_common_/loadingBar/LoadingBar";
import LoadingIcon from "../_common_/loadingIcon/LoadingIcon";
import SearchableTableContainer from "./../table/SearchableSelectableTable";
import {createStringFromEndPointObject} from "../../utils/helpers";
import {getTimeline} from "../../store/global/action_creators";

require("./timeline.scss");

const createStringFromDataBrokersArray = (dataBrokers) => {
    return dataBrokers.map( broker => broker.name).join(', ');
};

class Timeline extends Component {
    componentDidMount(){
        this.props.getTimeline();
    }

    render() {
        const {_timeline, _timeline: {data}} = this.props;
        const containerOptions = {
            singularTitle: "Timeline action",
            pluralTitle: "Timeline actions"
        };

        const dataProcessed = data ? data.map((item) => {
            return {
                ...item,
                source: item.relationship ? createStringFromEndPointObject(item.relationship.source) : '',
                target: item.relationship ? createStringFromEndPointObject(item.relationship.target) : '',
                dataBrokers: item.group ? createStringFromDataBrokersArray(item.group.dataBrokers) :
                    (item.dataBroker ?  item.dataBroker.name : '')
            };
        }) : [];

        return(<div className="timeline">
            <div className="scrollable-area">
                <div className="timeline-content">
                    <h1>Timeline</h1>
                    <SWITCH on={_timeline.inProgress || !data}>
                        <div data-when={true} className="timeline-actions-loading">
                            <LoadingBar className="top"/>
                            <LoadingIcon text="Timeline actions" icon={require("./../../assets/svgs/timeline-actions.svg")} className="timeline-actions"/>
                        </div>
                        <div data-when={false}>
                            <div className="table-row">
                                <SearchableTableContainer containerOptions={containerOptions} Table={TimelineTable} data={dataProcessed}/>
                            </div>
                        </div>
                    </SWITCH>
                </div>
            </div>
        </div>)
    }
}
const mapStateToProps = (state) => {
    return {
        _timeline: state.global._timeline,
        _relationships: state.syncReducer._relationships
    }
}
export default connect(mapStateToProps, {getTimeline})(Timeline)