import React, {Component} from 'react';
import SelectableTable from "./../../table/SelectableTable";
import moment from "moment";
import classNames from "classnames";
require("./costTable.scss");


export default class CostTable extends Component{
    static getClassName(row, isTotalCost, className) {
        //for all the empty fields on the last row of total cost, put no-empty to avoid automatic -- insertion.
        // mark the total-cost field of lastRow so it will be painted with blue color
        return row.isLast? (isTotalCost? className : "no-empty") : (isTotalCost? `${className}-val` : "");
    }
    renderRowTimes(row) {
        const fromText = row.isLast? "" : moment(row.from).utc().format('MM/DD/YY, h:mm a');
        const toText = row.isLast? "" : moment(row.to).utc().format('MM/DD/YY, h:mm a');
        return (
            <div>
                <td className={CostTable.getClassName(row)} title={fromText}>{fromText}</td>
                <td className={CostTable.getClassName(row)} title={toText}>{toText}</td>
                <td className={CostTable.getClassName(row)}>{row.isLast? "" : row.hours}</td>
            </div>
        )
    }

    renderHeaderTimes() {
        return (
            <thead>
            <tr>
                <td>From</td>
                <td>To</td>
                <td>Hours</td>
            </tr>
            </thead>
        )
    }

    renderRowTotal(row){
        return (
            <div>
                <td className={classNames("small", CostTable.getClassName(row, true, "total-relationships"))}>{row.totalRelationships}</td>
                <td className={classNames("large", CostTable.getClassName(row, true, "total-relationships"))}>{row.totalRelationships} {row.isLast? "" : "Relationships"}</td>
                <td className={CostTable.getClassName(row, true, "total-cost")}>${row.totalCost}</td>
            </div>
        )
    }

    renderHeaderTotal() {
        return (
            <thead>
            <tr>
                <td>Total Relationships</td>
                <td className="total-cost">Total Cost</td>
            </tr>
            </thead>
        )
    }

    renderRowBYOL(row){
        return (
            <div>
                <td className={classNames("small","byol-row", CostTable.getClassName(row))}>{row.isLast? "" : (row.byol.usedRelationships ? `${row.byol.usedRelationships}/${row.byol.reservedRelationships}` : "")}</td>
                <td className={classNames("large", "byol-row",CostTable.getClassName(row))}>{row.isLast? "" : (row.byol.usedRelationships ? `${row.byol.usedRelationships}/${row.byol.reservedRelationships} Relationships` : "")}</td>
            </div>
        )
    }

    renderHeaderBYOL() {
        return (
            <thead>
            <tr>
                <td className="byol">BYOL License</td>
            </tr>
            </thead>
        )
    }

    renderTierRow(key, row) {
        return (
            <div>
                <td className={`small tier-row ${CostTable.getClassName(row)}`}><span className={CostTable.getClassName(row)}>{row.isLast? "" : `${row[key].numOfRelationships}`} </span><span className={`${CostTable.getClassName(row)} right`}>{row.isLast? "" : row[key].sumOfBilling ? `$${row[key].sumOfBilling}` : ""}</span></td>
                <td className={`large tier-row ${CostTable.getClassName(row)}`}><span className={CostTable.getClassName(row)}>{row.isLast? "" : `${row[key].numOfRelationships} Relationships`} </span><span className={`${CostTable.getClassName(row)} right`}>{row.isLast? "" : row[key].sumOfBilling ? `$${row[key].sumOfBilling}` : ""}</span></td>
            </div>
        )
    }

    renderTierHeader(className, textHeader) {
        return (
            <thead>
            <tr>
                <td className={classNames("small",className)}><div>{textHeader[0]}</div><div>{textHeader[1]}</div></td>
                <td className={classNames("large",className)}>{textHeader[0]} {textHeader[1]}</td>
            </tr>
            </thead>
        )
    }

    render(){
        const {tableData} = this.props;
        return (
            <div>
                <div className="tables-container">
                    {tableData && <SelectableTable noAnimate={true} Table={{renderRow: this.renderRowTimes.bind(this), renderHeader: this.renderHeaderTimes.bind(this), className:"cost-table"}} data={tableData.costArr} selectionOptions={null} itemIdentifier="from">
                    </SelectableTable>}
                    {tableData && tableData.costArr && tableData.isBYOL &&<SelectableTable noAnimate={true} Table={{renderRow: this.renderRowBYOL.bind(this), renderHeader: this.renderHeaderBYOL.bind(this), className:"cost-table"}} data={tableData.costArr} selectionOptions={null} itemIdentifier="from">
                    </SelectableTable>}
                    {tableData && tableData.costArr && tableData.isTier1 && <SelectableTable noAnimate={true} Table={{renderRow: this.renderTierRow.bind(this, "tier1"), renderHeader: this.renderTierHeader.bind(this, "tier1", ["First","1-5 Sync Relationships"]), className:"cost-table"}} data={tableData.costArr} selectionOptions={null} itemIdentifier="from">
                    </SelectableTable>}
                    {tableData && tableData.costArr && tableData.isTier2 && <SelectableTable noAnimate={true} Table={{renderRow: this.renderTierRow.bind(this, "tier2"), renderHeader: this.renderTierHeader.bind(this, "tier2", ["Next","6-20 Relationships"]), className:"cost-table"}} data={tableData.costArr} selectionOptions={null} itemIdentifier="from">
                    </SelectableTable>}
                    {tableData && tableData.costArr && tableData.isTier3 && <SelectableTable noAnimate={true} Table={{renderRow: this.renderTierRow.bind(this, "tier3"), renderHeader: this.renderTierHeader.bind(this, "tier3", ["More than","20 Relationships"]), className:"cost-table"}} data={tableData.costArr} selectionOptions={null} itemIdentifier="from">
                    </SelectableTable>}
                    {tableData && <SelectableTable noAnimate={true} Table={{renderRow: this.renderRowTotal.bind(this), renderHeader: this.renderHeaderTotal.bind(this), className:"cost-table"}} data={tableData.costArr} selectionOptions={null} itemIdentifier="from">
                    </SelectableTable>}
                </div>
                <div className="cost-text">{"This is an estimated cost for the Cloud Sync relationships you have created. This cost doesn't include the resources you have created in AWS"}</div>

            </div>
        )
    }
}
