import React, {Component} from 'react';
import {AddBYOLLicenseForm} from './AddBYOLLicense';
import BYOLLicenseLine from './BYOLLicenseLine';
import {connect} from 'react-redux';
import classNames from 'classnames';
import {purchaseInfo, SUPPORT_REGISTERED} from '../../../consts';
import Isvg from 'react-inlinesvg';
import {getLicenseSettings} from "../../../store/global/action_creators";

require("./byolLicenses.scss");

class BYOLLicenses extends Component {
    constructor(props, context) {
        super(props, context);
        this.state = {adding: false};
    }

    switchToAddState() {
        this.setState({adding: true});
    }

    finishedAdding() {
        this.setState({adding: false});
        this.props.getLicenseSettings(); // update license page
    }


    render() {
        const {licenses, auth: {data: {supportRegistrationStatus}}} = this.props;
        const isFirstLicense = licenses.length === 0;
        const isSupportRegistered = supportRegistrationStatus === SUPPORT_REGISTERED;
        const addBYOLLicenseForm = this.state.adding &&
            <AddBYOLLicenseForm isFirstLicense={isFirstLicense}
                                needNssRegistration={isFirstLicense && !isSupportRegistered}
                                onComplete={this.finishedAdding.bind(this)}/>;

        return (
            <div>
                <div className="subscription-card byol-licenses payment-option row medium">
                    <div className="payment-icon">
                        <span><Isvg src={require('../../../assets/svgs/netapp-icon.svg')}/></span>
                    </div>
                    {isFirstLicense ?
                        <div className="medium-8 columns payment-text">
                            <h5>Add BYOL License</h5>
                            <div>You can pay for sync relationships by purchasing licenses from
                                NetApp.<br/>{purchaseInfo}</div>
                            {addBYOLLicenseForm}
                        </div>
                        :
                        <div className={classNames("columns licenses-table", {'adding': this.state.adding})}>
                            <h5>BYOL List</h5>
                            {licenses.map((license) => (
                                <BYOLLicenseLine key={license.serialNumber} license={license}/>))}
                            {addBYOLLicenseForm}
                        </div>
                    }

                    {!this.state.adding &&
                    <div className={classNames({
                        "payment-action": isFirstLicense,
                        "medium-offset-10 under-table-button": !isFirstLicense
                    }, "medium-1 columns")}>
                        <button className="button" onClick={this.switchToAddState.bind(this)}>Add License</button>
                    </div>}
                </div>
            </div>
        )
    }
}

const mapStateToProps = (state) => ({
    _licenses: state.global._licenses,
    auth: state.global.auth,
})

export default connect(mapStateToProps, {getLicenseSettings})(BYOLLicenses)