import React from "react";
import Isvg from 'react-inlinesvg';
import classNames from "classnames";
import {routesContainer, STATUS_PROGRESS} from '../../../store/global/action_creators_consts';
import ReactPopover from '../../_common_/reactPopover/ReactPopover';
import PopoverInfo from '../../_common_/reactPopover/PopoverInfo';
import {checkIfFailExist, noConnectionToAllBrokers} from '../../../utils/dashboard-utils';
import {openDialog} from '../../dialog/Dialog';
import DownloadLogsDialog from '../DownloadLogsDialog';
import _ from "lodash";
import RelationshipRecommendations from "../../RelationshipRecommendations/RelationshipRecommendations";

require("./syncStatistics.scss");


export default (props) => {
    const {relationship, isCardView, closeDialog, calledFrom, downloadDataBrokerLogs} = props;
    const {ui: {activityValues}} = relationship;

    if (!activityValues) return <div/>;

    const isFailExist = checkIfFailExist(activityValues);

    const {_downloadLogsRequests} = props;
    const downloadLogsInProgress = _.get(_downloadLogsRequests, [relationship.relationshipId, 'status']) === STATUS_PROGRESS;
    const noConnectionToBrokers = noConnectionToAllBrokers(relationship.group.dataBrokers);
    const disableDownloadLogs = noConnectionToBrokers || downloadLogsInProgress;
    const downloadLogsTitle = noConnectionToBrokers ? 'No available data brokers' : '';

    const viewFailedClicked = () => {
        routesContainer.routes.push(`/failed/${relationship.relationshipId}?calledFrom=${calledFrom}`);
        if (closeDialog) closeDialog();
    };

    const syncTimingElement = isCardView ?
        (<div>{relationship.ui.lastTransferredHeaderText}</div>) :
        (<div className="duration-popover">
            <PopoverInfo>{relationship.ui.lastTransferredHeaderText}</PopoverInfo>
            <div>Sync Timing</div>
        </div>);

    return (<div className="sync-info">
        <div className={classNames({"card-view": isCardView}, "sync-duration")}>
            {syncTimingElement}
        </div>
        <div className={classNames({"card-view": isCardView}, "phases-summary")}>
            <PhaseInfo title="Scan" data={activityValues.scan}/>
            <PhaseInfo title="Copy" data={activityValues.copy}/>
        </div>
        <div className="bottom-line">
            <div className="recommendations">
                {relationship.ui.recommendations && relationship.ui.recommendations[0] && <RelationshipRecommendations recommendations={relationship.ui.recommendations} variant='cardView' />}
            </div>
            <div className="links">
        <div className={classNames({"card-view": isCardView}, "info-links download-logs")}>
            <ReactPopover popoverClass={classNames("download-logs info-tooltip", {'hidden': !downloadLogsInProgress})}
                          key="dbLogs"
                          trigger="mouseover"
                          container={
                              <button disabled={disableDownloadLogs} title={downloadLogsTitle} className="download-logs"
                                      onClick={(el) => {
                                          openDialog(<DownloadLogsDialog className="default-dialog" relationship={relationship}
                                                                         downloadDataBrokerLogs={downloadDataBrokerLogs}/>, el.target);
                                      }}>
                                  <span className="icon"><Isvg src={require("./../../../assets/svgs/download-no-fill.svg")}/></span>
                                  <span className="text">Download Data Broker Logs</span>
                              </button>}>
                {`Request was sent. This process usually takes a couple of minutes. The logs will be saved to your web browser's download location.`}
            </ReactPopover>
        </div>

        {isFailExist &&
        <div onClick={viewFailedClicked} className={classNames({"card-view": isCardView}, "info-links view-failed-files")}>
            <Isvg src={require("./../../../assets/svgs/top-right-arrow-icon.svg")}/>View details about sync failures
        </div>}
            </div>
        </div>
    </div>);
};

const DataDisplay = ({value, label, isFailed}) => (
    <div className={classNames("data-display", {"hidden": !value, "failed": isFailed})}>
        <div className="value">{value}</div>
        <div className="text">{label}</div>
</div>);

const StatusLine = ({data: {name, data}}) => {
    const sumOriginalValues = Object.values(data).reduce((sum, {original})=> sum + original, 0);
    const isFailExists = (name === "Failed") &&  sumOriginalValues > 0;

    return (<div className={classNames({failed: isFailExists}, "status-line")}>
        <div className="line-title">{name}</div>
        <div className="details">
            <DataDisplay value={data.dirs.formatted} label={data.dirs.label} isFailed={name === "Failed" && data.dirs.original > 0}/>
            <div className="separator"/>
            <DataDisplay value={data.files.formatted} label={data.files.label} isFailed={name === "Failed" && data.files.original > 0}/>
            <div className="separator"/>
            <DataDisplay value={data.bytes.formatted} label={data.bytes.label} isFailed={name === "Failed" && data.bytes.original > 0}/>
        </div>
    </div>)
};

const PhaseInfo = ({title, data}) => (<div className={`phase-info ${title}`}>
    <div className="phase-title">{title}</div>
    {data.map((statusData)=>(<StatusLine key={`${title}-${statusData.name}`} data={statusData}/>))}
</div>);
