import React, {Component} from 'react'
import classNames from 'classnames';
import _ from 'lodash'

require("./filterWidget.scss");

export default class FilterWidget extends Component {
    constructor(props, context) {
        super(props, context);
        const {filterValues} = this.props;
        const all = this.calculateAllFilter(filterValues);
        const clonedFilterValues = _.cloneDeep(filterValues);
        const filterList = [...clonedFilterValues];
        const originalFilterList = _.cloneDeep(filterList);
        this.state = {all, filterList, originalFilterList};
    }

    calculateAllFilter(filterValues) {
        const all = {
            className: "all",
            name: "All",
            checked: _.reduce(filterValues, (checked, value) => checked && value.checked, true)
        };
        return all;
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutside.bind(this));
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutside.bind(this));
    }

    setWrapperRef(node) {
        this.wrapperRef = node;
    }

    handleClickOutside(event) {
        if (this.wrapperRef && !this.wrapperRef.contains(event.target)) {
            this.setState({filterList: _.cloneDeep(this.state.originalFilterList), all: this.calculateAllFilter(this.state.originalFilterList)}); //return to original list
            this.props.close();
        }
    }

    apply() {
        const { apply } = this.props;
        apply(this.state.filterList);
    }

    clear() {
        this.handleClick("All", {target: {checked: true}});
        this.apply();
    }

    handleClick(name, event) {
        const filterList = this.state.filterList.map(filter => {
            if (name === "All"|| filter.name === name) {
                filter.checked = event.target.checked;
            }
            return filter;
        });

        const allFilter = this.state.all;
        allFilter.checked = _.reduce(filterList, (checked, value) => checked && value.checked, true);
        this.setState({all: allFilter, filterList})
    }

    render() {
        const {all, filterList} = this.state;
        const {fixed} = this.props;
        const buildFieldElement = (value) => {
            return(
                <label title={value.name} key={value.name}>
                <input className={`value ${value.className}`}
                       onChange={this.handleClick.bind(this, value.name)}
                       name={value.name} type="checkbox"
                       checked={value.checked}
                />{value.name}</label>
            );
        };

        const noneChecked = () => {
            const checked = _.reduce(filterList, (checked, value) => checked || value.checked, false);
            return !checked;
        };

        const allChecked = () => {
            return _.every(filterList, filter => filter.checked);
        };

        return (
            <div className={classNames("filter-widget", {fixed})} ref={this.setWrapperRef.bind(this)}>
                <div className="filters-list">
                    {buildFieldElement(all)}
                    {filterList.map(value => {
                        return buildFieldElement(value);
                    })}
                </div>
                <div className={"buttons-row"}>
                    <button type="button" disabled={allChecked()} onClick={this.clear.bind(this)}>Clear</button>
                    <button type="button" disabled={noneChecked()} onClick={this.apply.bind(this)}>Apply</button>
                </div>
            </div>
        );
    }
}
