import React from 'react';
import {startWizard} from '../../store/wizard/action_creators'
import {connect} from 'react-redux';
import {Route, Switch} from 'react-router-dom';
import {calculateNextStepObj, getStepsOrder, getWizardInfo, stepInfoById} from './../../utils/sync-utils';
import {getNextStepComponent, weSelectionNeeded} from './../../utils/wizard-utils';
import useRunOnceWhenTruthy from '../../hooks/useRunOnceWhenTruthy';
import {getPropFromState} from '../../utils/mapStateToPropsUtils';

const WizardContent = (props) => {
    const {match, location, history, _wizard, _ui} = props;

    useRunOnceWhenTruthy(() => {
        const protocols = match.params.protocols;
        const providers = match.params.providers;
        const isSecure = match.params.isSecure === 'true';
        const wizardInfo = getWizardInfo(protocols, providers);
        const {isCmIntegration} = _ui;
        const stepsOrder = getStepsOrder(wizardInfo, isSecure, weSelectionNeeded(wizardInfo, isCmIntegration));
        props.startWizard(protocols, wizardInfo, isSecure, stepsOrder, stepInfoById);
        const firstStepPath = stepInfoById[stepsOrder[0]].path;
        //this condition is needed in order to support page refresh
        if (location.pathname.endsWith("/start-wizard")) {
            history.push(`/sync/${protocols}/${providers}/${isSecure}/${firstStepPath}`);
        }
    }, match.isExact || !_wizard);

    if (!_wizard) return <div/>;
    const nextStep = calculateNextStepObj(_wizard);
    const NextStepComp = getNextStepComponent(nextStep.path);

    return (
        <Switch>
            <Route path={`/sync/:protocols/:providers/:isSecure/${nextStep.path}`} render={(props) =>
                <NextStepComp {...props} isTargetStepFromSameSourceType={nextStep.isTargetStepFromSameSourceType}/>
            }/>
        </Switch>
    )
};

const mapStateToProps = (state) => ({
    _wizard: getPropFromState(state, "_wizard", null),
    _ui: {...state.syncReducer._ui, ...state.global._ui}
});

export default connect(mapStateToProps, {startWizard})(WizardContent);