import React from 'react';
import Isvg from 'react-inlinesvg';
import classNames from 'classnames';
import {getPropFromState} from '../../utils/mapStateToPropsUtils'
import {connect} from 'react-redux';
import {withRouter} from 'react-router-dom';
import _ from "lodash";
import {decodeAll} from "../../utils/encoders";
import ReactPopover from '../_common_/reactPopover/ReactPopover';
import "./wizardHeader.scss";

const WizardHeader = (props) => {
    const {_wizard} = props;
    return (<div className={classNames("wizard-header", {hidden: _wizard && _wizard.hideHeader})}>
            <WizardSteps {...props}/>
        </div>)
};

const WizardSteps = (props) => {
    const {_wizard} = props;
    if(!_wizard || !_wizard.currentStep) return (<div/>);
    const {_wizard: {stepsOrder, stepInfoById, currentStep}, _routing: {location}, _ui: {isCmIntegration}} = props;
    const currentIndex = stepsOrder.indexOf(currentStep);
    let currentPathSegments = _.dropWhile(location.pathname.split("/"), _.isEmpty);
    currentPathSegments = _.each(currentPathSegments, segment => {
        return decodeAll(segment);
    });

    //cm integration has less room in the wizard header
    const isManySteps = stepsOrder.length > (isCmIntegration ? 6 : 8);


    return (<div className="steps">
                {stepsOrder.map((stepId) => {
                    const history = props.history;
                    const pathName = stepInfoById[stepId].path;
                    const pathIndex = currentPathSegments.indexOf(pathName);
                    const stepIndex = stepsOrder.indexOf(stepId);
                    const goBack = stepIndex < currentIndex ? () => {
                        history.push("/" + _.take(currentPathSegments, pathIndex + 1).join("/") + "?down=true")
                    } : undefined;
                    return (<WizardStep goBack={goBack} key={stepId} current={stepIndex === currentIndex}
                                        future={currentIndex < stepIndex} stepName={stepInfoById[stepId].name}
                                        stepNumber={stepIndex + 1} last={stepIndex === (stepsOrder.length-1)}
                                        smallButtons={isManySteps}
                    />)
                })}
            </div>
    )
};

const WizardStep = (props) => {
    const {stepName, current, future, goBack, stepNumber, smallButtons} = props;
    const classes = classNames({
        step: true,
        current,
        future,
    });

    const stepNumberElement = (<div className="step-number">
        {future && <Isvg src={require(`./../../assets/svgs/circle-empty.svg`)}/>}
        <div className="number-text"><div>{stepNumber}</div></div>
    </div>);

    return (
        <div className={classes} title={stepName}>
             <button type="button" className={classNames({disabled: !goBack})} onClick={goBack ? goBack : () => {}}>
                 {smallButtons &&
                     <ReactPopover trigger="mouseover"
                                   container={stepNumberElement}                     >
                         {stepName}
                     </ReactPopover>
                 }
                 {!smallButtons && <>
                     {(current || future)
                         ? stepNumberElement
                         : <Isvg src={require("./../../assets/svgs/checkmark-blue.svg")}/>}
                     <ReactPopover trigger="mouseover"
                                   container={<div className="step-name">{stepName}</div>}                     >
                         {stepName}
                     </ReactPopover>
                 </>}
            </button>
        </div>
    )
};

const mapStateToProps = state => ({
    _wizard: getPropFromState(state, "_wizard", null),
    _routing: getPropFromState(state, "_routing", {}),
    _ui: {...state.syncReducer._ui, ...state.global._ui}
});
export default withRouter(connect(mapStateToProps)(WizardHeader));