import React from 'react';
import ReactDOM from 'react-dom';
import {Provider} from 'react-redux';
import * as actionCreatorsConsts from './store/global/action_creators_consts';
import * as actionCreators from './store/global/action_creators';
import {history, store} from './store/store';
import sdk from "./sdk";
import {Route, Switch} from 'react-router-dom';
import _ from "lodash";
import {bindActionCreators} from "redux";
import {ConnectedRouter, push, replace} from "connected-react-router";
import numeral from "numeral";
import Auth from './auth/auth';
import AppWithRouter from './App';
import { RightPanelProvider } from './context/right-panel-context';
import "./index.scss";

const opts = {
    clientID: process.env.REACT_APP_AUTH_CLIENT,
    domain: process.env.REACT_APP_AUTH_DOMAIN,
    audience: process.env.REACT_APP_AUTH_AUDIENCE
};
// eslint-disable-next-line
console.log(`%c Creating auth with opts ${JSON.stringify(opts)}`, 'font-weight:bold;color:green');
window.auth = new Auth(opts);
store.dispatch({type: actionCreatorsConsts.AUTH_REFRESH_SSO}); //start login flow

Object.assign(numeral.locales["en"].abbreviations, {thousand: "K", million: "M", billion: "B", trillion: "T"});

if (window.navigator.userAgent.indexOf("MSIE 10") > -1 || window.navigator.userAgent.match(/Trident\//)) {
    document.getElementById('noIE').className = "show";
    document.getElementById('app').className = "hide";
} else {
    actionCreatorsConsts.routesContainer.routes = {
        push: bindActionCreators(push, store.dispatch),
        replace: bindActionCreators(replace, store.dispatch)
    };

    window.__$$expose = {
        actionCreators,
        store,
        sdk
    };

    sdk.interceptors().response.use(null, function (error) {
        const {isCmIntegration} = store.getState().global._ui;
        if (error.response && (error.response.status === 401)) {
            const errorMessage = error.response.data ? error.response.data.message : error.response.message;
            if (_.isEqual(errorMessage, "unauthorized") || _.isEqual(errorMessage, "TokenExpiredError: jwt expired") || _.isEqual(error.response.statusText, "Unauthorized")) {
                //if we are in sso login, and there is an error other than unauthorized, we don't want to logout, to get to the oops page
                if (!isCmIntegration) {
                    store.dispatch(actionCreators.logout());
                }
            }
        }
        return Promise.reject(error.response);
    });


    history.listen(location => {
        window.ga && window.ga('send', 'pageview', location.pathname);
    });



    ReactDOM.render(
        <Provider store={store}>
            <RightPanelProvider>
                <ConnectedRouter history={history}>
                    <Switch>
                        <Route path="/" component={AppWithRouter}/>
                    </Switch>
                </ConnectedRouter>
            </RightPanelProvider>
        </Provider>,
        document.getElementById('root')
    );
}
