import Immutable from 'seamless-immutable';
import * as consts from './action_creators_consts';
import _ from "lodash";
import {DEPLOY_STATUS_PROGRESS} from "../../consts";

export const getInitialState = () => Immutable({
    credentials: {},
    deployments: []
});

export const deployReducer = (state = getInitialState(), action = {}) => {
    switch (action.type) {
        case consts.SET_AWS_CREDENTIALS: {
            return state.setIn(['credentials', 'aws'], action.payload);
        }
        case consts.SET_AZURE_CREDENTIALS: {
            return state.setIn(['credentials', 'azure'], action.payload);
        }
        case consts.ADD_DEPLOYMENT: {
            const {dataBrokerId, dataBrokerName, providerType, deployDetails} = action.payload;
            return state.setIn(['deployments'], [...state.deployments, {
                dataBrokerId,
                dataBrokerName,
                providerType,
                deployDetails,
                deployStatus: DEPLOY_STATUS_PROGRESS
            }]);
        }
        case consts.SET_DEPLOYMENT_STATUS: {
            const {dataBrokerId, deployStatus, errorMessage} = action.payload;
            return state.setIn(["deployments"], _.map(state.deployments, (deployment) => {
                return (deployment.dataBrokerId === dataBrokerId
                    ? {...deployment, deployStatus, errorMessage}
                    : deployment);
            }));
        }
        default:
            return state;
    }
};