import {applyMiddleware, createStore, combineReducers} from 'redux';
import {connectRouter, routerMiddleware} from 'connected-react-router';
import {createBrowserHistory} from 'history';
import thunk from 'redux-thunk'
import {filteringFunc, syncReducer} from './wizard/reducer';
import {createLogger} from 'redux-logger';
import * as actions from "./wizard/action_creators_consts";
import isFunction from "lodash/isFunction";
import RavenMiddleware from 'redux-raven-middleware';
import {uuid} from "../utils/helpers";
import authMiddleware from '../auth/authMiddleware';
import {reducer as formReducer} from 'redux-form';
import {ignoreActions} from "redux-ignore";
import {globalReducer} from "./global/reducer";
import {deployReducer} from "./deploy/reducer";

export const history = createBrowserHistory();

const reducer = combineReducers({
    global: globalReducer,
    syncReducer: ignoreActions(syncReducer, filteringFunc),
    router: connectRouter(history),
    form: formReducer,
    deploy: deployReducer
});

const predicate = (getState, action) => {
    if ((action.type === actions.UPDATE_RELATIONSHIP) && !(window.location.href.includes("dashboard") || window.location.href.includes("review") || window.location.href.includes("timeline"))) return false;
    if ((action.type === actions.UPDATE_DATABROKER_TRANSFER_RATE) && !window.location.href.includes("dashboard")) return false;
    return true;
};

const logger = createLogger({predicate});

const router = routerMiddleware(history);

const promise = ({dispatch}) => {
    return next => action => {
        if(action.payload && action.payload.then && isFunction(action.payload.then)){
            const id = uuid();
            action.payload.then(
                    r => dispatch(Object.assign({}, action, {payload: r, id, requestId: r ? r.requestId : undefined, status: actions.STATUS_SUCCESS})),
                    e => dispatch(Object.assign({}, action, {payload: e, id, status: actions.STATUS_ERROR})));

            next(Object.assign({}, action, {id, payload: null, status: actions.STATUS_PROGRESS}));
        }
        else{
            next(action);
        }
    }
};

function configureStore(initialState) {
    const store = createStore(
        reducer,
        initialState,
        applyMiddleware(authMiddleware, RavenMiddleware('https://32cc596b1c1345f983855e1932b33582@sentry.io/148201'), router, thunk, promise, logger)
    );

/*
    if (module.hot) {
        // Enable Webpack hot module replacement for reducers
        module.hot.accept('../reducer', () => {
            const nextRootReducer = require('../reducers').default
            store.replaceReducer(nextRootReducer)
        })
    }

*/
    return store
}

export const store = configureStore();
