import React from "react";
import Isvg from 'react-inlinesvg';
import ReactLoading from 'react-loading';
import classNames from "classnames";
import {RESPONSE_STATUS_ABORTED, RESPONSE_STATUS_INTERRUPTED, RESPONSE_STATUS_FAILED, RESPONSE_STATUS_ABORTING,
    RESPONSE_STATUS_RUNNING, RESPONSE_STATUS_PENDING, RESPONSE_STATUS_DONE, STATUS_STARTING} from "../consts";
import _ from 'lodash';
import {formatBytes} from './helpers';

export const getSVGByProtocol =(protocol, isSecureRelationship) => {
    return <div className="icon"><Isvg src={require(`../assets/svgs/${protocol}-${isSecureRelationship ? "table-secure" : "table"}.svg`)}/></div>;
};

export const getClassBySyncStatus = (status, dataBrokers) => {
    return classNames({
        "sync-status" : true,
        "syncing" : (status === RESPONSE_STATUS_RUNNING || status === RESPONSE_STATUS_ABORTING || status === STATUS_STARTING),
        "unknown" : (noConnectionToAllBrokers(dataBrokers)) && status === RESPONSE_STATUS_RUNNING,
        "synced-done" : (status === RESPONSE_STATUS_DONE),
        "failed" : (status === RESPONSE_STATUS_FAILED || status === RESPONSE_STATUS_INTERRUPTED || status === RESPONSE_STATUS_ABORTED),
        "pending": status === RESPONSE_STATUS_PENDING
    });
};

export const getSVGBySyncStatus =(status, dataBrokers) => {
    switch (status) {
        case "FAILED":
        case "ABORTED":
        case "INTERRUPTED": return <Isvg src={require("./../assets/svgs/x-circle-icon.svg")}/>;
        case "RUNNING":
        case "STARTING": {
            if (noConnectionToAllBrokers(dataBrokers))
                return <Isvg src={require("./../assets/svgs/x-circle-icon.svg")}/>;
            else return <div className="loading"><ReactLoading type="spin" color="#0891d8" height={0} width={20} delay={50} /></div>;

        }
        case "ABORTING": return <div className="loading"><ReactLoading type="spin" color="#0891d8" height={0} width={20} delay={50} /></div>;
        case "PENDING": return <Isvg src={require("./../assets/svgs/pending.svg")}/>;
        default: return <Isvg src={require("./../assets/svgs/checkmark-circle-icon.svg")}/>;
    }
};

export const noConnectionToAllBrokers = (dataBrokers) => {
    return _.every(dataBrokers, (dataBroker) => (dataBroker.isFailed || dataBroker.isWaiting));
};

export const getFixedProtocol = (endPoint) => {
    const {protocol} = endPoint;
    const details = endPoint[protocol];

    switch (protocol) {
        case "nfs":
        case "cifs":
        case "s3": {
            return details.provider || protocol;
        }
        default: {
            //do nothing;
        }
    }

    return protocol;
};

export const getCopyDoneByCloudProvider = (source, target) => {
    return (source.protocol === 's3' && target.protocol === 's3' && source['s3'].provider === 's3' && target['s3'].provider === 's3')
        || (source.protocol === 'azure' && target.protocol === 'azure')
        || (source.protocol === 'gcp' && target.protocol === 'gcp');
};


export const checkIfFailExist = (activityValues) => {
    const scanFailedObj = _.find(activityValues.scan, {name: "Failed"});
    const copyFailedObj = _.find(activityValues.copy, {name: "Failed"});
    const scanFailedCount = Object.values(scanFailedObj.data).reduce((sum, {original})=> sum + original, 0);
    const copyFailedCount = Object.values(copyFailedObj.data).reduce((sum, {original})=> sum + original, 0);
    const totalFailed = scanFailedCount + copyFailedCount;
    return totalFailed > 0;
};

export const calcTransferRate = (transferRate) => {
    return _.isNil(transferRate) ? "" : `(${formatBytes(transferRate).size}${formatBytes(transferRate).suffix}/s)`;
};