import sdk from "../sdk";
import {
    startup,
    updateDataBrokerStatus,
    updateBuckets,
    updateBucketsFolders,
    updateContainers,
    updateContainersFolders,
    updateDataBrokerLogs,
    updateDatabrokerTransferRate,
    updateExports,
    updateFolders,
    updateGCPBucketFolders,
    updateGCPBuckets,
    updateRelationship,
    updateS3BasedBuckets,
    updateS3BasedBucketsFolders,
    updateShareFolders,
    updateShares,
    updateKmsKeys
} from "../store/wizard/action_creators";
import * as actionCreatorsConsts from "../store/global/action_creators_consts";
import {saveAccountLastUsedPerUser} from "./account-utils";
import * as consts from "../consts";
import {
    clearCurrentAccountData,
    getAccountInfo,
    updateAccountInfo,
    updateSubscriptionStatus,
    updateSupportInfo, updateTimeline
} from "../store/global/action_creators";

let lastMessage;

export function messagesPoller() {
    return (dispatch) => {
        return sdk.messages(lastMessage)
            .then(r => {
                handleMessages(dispatch, r);
                dispatch(messagesPoller());
            })
            .catch(e => {
                //don't send error message for unauthorized response on get messages, to avoid showing it in the general error panel
                if (e && e.status !== 401) {
                    dispatch({
                        type: actionCreatorsConsts.MESSAGES,
                        status: actionCreatorsConsts.STATUS_ERROR,
                        payload: e
                    });
                }
                sleep(5000).then(()=>{dispatch(messagesPoller())});
            })
    }
}

export const setActiveAccount = ({accountId, uid}) => {
    return dispatch => {
        dispatch(clearCurrentAccountData()); //In order to prevent "old" data from being rendered until the new data arrives
        dispatch(getAccountInfo(accountId))
            .then(() => {
                saveAccountLastUsedPerUser(accountId, uid);
                startup(dispatch);
                if (!lastMessage) dispatch(messagesPoller()); //start polling only the first time an account is selected
            })
            .catch(e => {
                //don't send error message for unauthorized response on get messages, to avoid showing it in the general error panel
                if (e.status !== 401) {
                    dispatch({
                        type: actionCreatorsConsts.MESSAGES,
                        status: actionCreatorsConsts.STATUS_ERROR,
                        payload: e
                    });
                }
            });
    }
};

export function handleMessages(dispatch, messages) {
    messages.forEach(message => {
        lastMessage = message.id;
        message.payload.requestId = message.header.requestId;
        if (message.payload[consts.PAYLOAD_DATA_BROKER_STATUS]) {
            dispatch(updateDataBrokerStatus(message.payload));
        } else if (message.payload[consts.PAYLOAD_SHOW_EXPORTS]) { //list exports
            dispatch(updateExports({...message.payload, dataBrokerId: message.header.agentId}));
        } else if (message.payload[consts.PAYLOAD_LIST_CIFS_SHARES]) { //list cifs share
            dispatch(updateShares({...message.payload, dataBrokerId: message.header.agentId}));
        } else if (message.payload[consts.PAYLOAD_LIST_FOLDERS]) { //list sub folders of export
            dispatch(updateFolders({...message.payload, dataBrokerId: message.header.agentId}));
        } else if (message.payload[consts.PAYLOAD_LIST_CIFS_SHARE_FOLDERS]) { //list sub folders of shares
            dispatch(updateShareFolders({...message.payload, dataBrokerId: message.header.agentId}));
        } else if (message.payload[consts.PAYLOAD_LIST_BUCKETS]) { //buckets
            message.payload.dataBrokerId  = message.header.agentId;
            dispatch(updateBuckets(message.payload)); //sub directories of buckets
        } else if (message.payload[consts.PAYLOAD_LIST_SUB_DIRECTORIES]) {
            message.payload.dataBrokerId  = message.header.agentId;
            dispatch(updateBucketsFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_CONTAINERS]) { //azure blob containers
            message.payload.dataBrokerId  = message.header.agentId;
            dispatch(updateContainers(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_CONTAINER_SUB_DIRECTORIES]) { //sub directories of containers
            message.payload.dataBrokerId  = message.header.agentId;
            dispatch(updateContainersFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_SGWS_BUCKETS]) { //sgws buckets
            dispatch(updateS3BasedBuckets(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_IBM_BUCKETS]) { //ibm buckets
            dispatch(updateS3BasedBuckets(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_ONTAP_BUCKETS]) { //sgws buckets
            dispatch(updateS3BasedBuckets(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_SGWS_BUCKETS_FOLDERS]) { //sub directories of sgws buckets
            dispatch(updateS3BasedBucketsFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_IBM_BUCKETS_FOLDERS]) { //sub directories of sgws buckets
            dispatch(updateS3BasedBucketsFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_ONTAP_BUCKETS_FOLDERS]) { //sub directories of ibm buckets
            dispatch(updateS3BasedBucketsFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_GCP_BUCKETS]) { //gcp buckets
            message.payload.dataBrokerId  = message.header.agentId;
            dispatch(updateGCPBuckets(message.payload));
        } else if (message.payload[consts.PAYLOAD_LIST_GCP_BUCKET_FOLDERS]) { //sub directories of gcp buckets
            message.payload.dataBrokerId  = message.header.agentId;
            dispatch(updateGCPBucketFolders(message.payload));
        } else if (message.payload[consts.PAYLOAD_TIMELINE]) {
            dispatch(updateTimeline(message.payload));
        } else if (message.payload[consts.PAYLOAD_UPDATE_RELATIONSHIP]) {
            dispatch(updateRelationship(message.payload))
        } else if (message.payload[consts.PAYLOAD_UPDATE_SUBSCRIPTION_STATUS]) {
            dispatch(updateSubscriptionStatus(message.payload))
        } else if (message.payload[consts.PAYLOAD_DATABROKER_TRANSFER_RATE]) {
            dispatch(updateDatabrokerTransferRate(message.payload))
        } else if (message.payload[consts.PAYLOAD_UPDATE_SUPPORT_INFO]) {
            dispatch(updateSupportInfo(message.payload))
        } else if (message.payload[consts.PAYLOAD_UPDATE_ACCOUNT_INFO]) {
            dispatch(updateAccountInfo(message.payload))
        } else if (message.payload[consts.PAYLOAD_DATA_BROKER_LOGS]) {
            dispatch(updateDataBrokerLogs(message.payload))
        } else if (message.payload[consts.PAYLOAD_LIST_KMS_KEYS]) {
            dispatch(updateKmsKeys({...message.payload, dataBrokerId: message.header.agentId}))
        }
    })
}

const sleep = (period) =>  {
    return new Promise(resolve => setTimeout(() => resolve(), period));
};