import _ from "lodash";

const extractErrorAzure = (error, fallbackMsg = "Unknown error") => {
    if (!error) return fallbackMsg;
    if (typeof error === "string") return error;
    if (error.data && error.data.message && typeof error.data.message === "string") return error.data.message;
    if (error.data && error.data.error && error.data.error.message && typeof error.data.error.message === "string") return error.data.error.message;
    if (error.details && error.details[0]) {
        const parsed = JSON.parse(error.details[0].message);
        const msg = _.get(parsed, "error.details[0].message") ||  _.get(parsed, "error.message");
        if (msg && typeof msg === "string") return msg;
    }
    if (error.message && typeof error.message === "string") return error.message;
    return fallbackMsg;
};

const extractErrorGcp = (err, fallbackMsg = "Unknown error") => {
    if (!err) return fallbackMsg;
    const error = err.error ? err.error : err;
    if (typeof error === "string") return error;
    if (error.message && typeof error.message === "string") return error.message;
    let errorMessage = JSON.stringify(error.errors);
    if (error.errors[0] && error.errors[0].message) {
        const errorObj = JSON.parse(error.errors[0].message);
        if (errorObj.ResourceErrorMessage) {
            errorMessage = errorObj.ResourceErrorMessage.message;
        }
    }
    if (errorMessage && typeof errorMessage === "string") return errorMessage;
    return fallbackMsg;
};

const extractErrorAws = (error, fallbackMsg = "Unknown error") => {
    let errorMsg = error && error.message ? error.message.toString() : error.toString();
    if (errorMsg === '[object Object]') {
        errorMsg = fallbackMsg;
    }
    return errorMsg;
};

export {
    extractErrorAzure,
    extractErrorGcp,
    extractErrorAws
}