import _ from "lodash";
import moment from 'moment'
import isNil from 'lodash/isNil'
import get from 'lodash/get'
import * as actions from '../store/wizard/action_creators_consts.js';
import {PAYLOAD_UPDATE_RELATIONSHIP} from './../consts';
import numeral from "numeral";
import {mapProtocolToSourceSteps} from "./../utils/sync-utils"
import * as consts from "../consts";
import {RESPONSE_STATUS_DONE, RESPONSE_STATUS_RUNNING} from "../consts";
import {formatActivityValues, statusConvert} from "./helpers";
import {routesContainer} from "../store/global/action_creators_consts";
import * as actionCreatorsConsts from "../store/wizard/action_creators_consts";
import {fromNowNoFuture, stringDuration} from "./time";
import {calcTransferRate, getCopyDoneByCloudProvider, noConnectionToAllBrokers} from "./dashboard-utils";
import {getNextTimeHour} from "./settings-utils";
import Immutable from "seamless-immutable";


var FifoArray = require('fifo-array');


export const formatCount = (count) => {
    if(!count) return {size: "0", suffix: ""};
    let formattedCount = numeral(count).format('0.[0] a');
    return {
        size :  (formattedCount.split(" ")[0]) ,
        suffix : formattedCount.split(" ")[1]
    };
};

export const isItSourceStep = (wizardInfo, stepId) => {
    return _.includes(mapProtocolToSourceSteps[wizardInfo.source.protocol], stepId);
};

export const updateCurrentStep = (state) => {
    const identifyCurrentStep = (stepsOrder, steps) => {
        return stepsOrder.find((stepName, index) => {
            return index+2 > stepsOrder.length || !steps[stepsOrder[index+1]]
        })
    };
    const identifiedStepId = identifyCurrentStep(state._wizard.stepsOrder, state._wizard.steps);
    //console.log(`@@@@@@@@@@@@@@@@@@@@@@@\n stepsOrder: ${state._wizard.stepsOrder}\nsteps:${JSON.stringify(state._wizard.steps)}\identifiedStepId: ${identifiedStepId}\n@@@@@@@@@@@@@@@@@@@@@@@@`);
    return state.update("_wizard", _wizard => _wizard.set("currentStep", identifiedStepId).set("isSourceStep", isItSourceStep(_wizard.wizardInfo, identifiedStepId)));
};

export const updateErrorsPanelStatus = (state) => {
    if (_.isEmpty(state._generalErrors._errorsMap)) return state.setIn(["_generalErrors","_errorsPanelOpen"], false).setIn(["_generalErrors","_errorsPanelSize"], {height: 0});
    return state.setIn(["_generalErrors","_errorsPanelOpen"], true);
};


export const buildStateFromAction = (action, postProcessor = buildState => buildState) => {
    return postProcessor({
        id: action.id,
        failed: action.status === actions.STATUS_ERROR,
        inProgress: action.status === actions.STATUS_PROGRESS,
        succeeded: action.status === actions.STATUS_SUCCESS,
        data: action.status === actions.STATUS_SUCCESS ? action.payload : undefined,
        error: action.status === actions.STATUS_ERROR ? action.payload : undefined,
        //requestId: action.requestId
    });
};

export const updateLastRequestsForRelationshipsData = (state, action) => {
    const activityStatus = action.payload[PAYLOAD_UPDATE_RELATIONSHIP].activity.status;
    if(!state._relationships || !state._relationships.data) return state;
   return state.updateIn(["_relationships", "data"], relationships => {
        return relationships.map(relationship => {
            if(relationship.id === action.payload[PAYLOAD_UPDATE_RELATIONSHIP].id){
                relationship = _.cloneDeep(relationship);
                if(relationship && relationship.lastRequests){
                    const lastRequests = new FifoArray(3, relationship.lastRequests);
                    const curruntRequest = _.find(lastRequests , function(req) {
                        return req.id === action.payload.requestId;
                    });
                     if(curruntRequest){
                        curruntRequest.isDone = (activityStatus === RESPONSE_STATUS_DONE);
                    }
                    //new requestId
                    else{
                        lastRequests.push({
                            id : action.payload.requestId,
                            isDone : (activityStatus === RESPONSE_STATUS_DONE)
                        });
                    }
                    relationship.lastRequests = lastRequests;
                }
            }
            return relationship;
        })
    });
};

export const resetActivityValues = () => {
    const newAct = {
        bytesCopied: 0,
        bytesFailed: 0,
        bytesMarkedForCopy: 0,
        bytesMarkedForRemove: 0,
        bytesRemoved: 0,
        bytesRemovedFailed: 0,
        bytesScanned: 0,
        dirsCopied: 0,
        dirsFailed: 0,
        dirsMarkedForCopy: 0,
        dirsMarkedForRemove: 0,
        dirsRemoved: 0,
        dirsScanned: 0,
        filesCopied: 0,
        filesFailed: 0,
        filesMarkedForCopy: 0,
        filesMarkedforRemove: 0,
        filesRemoved: 0,
        filesRemovedFailed: 0,
        filesScanned: 0,
        progress: 0,
        type: "Sync",
        executionTime: 1,
        status: RESPONSE_STATUS_RUNNING
    };
    return formatActivityValues(newAct);
};

export const updateKeyWithBuiltState = (state, action, key, postProcessor) => {
    const originalState = Immutable.getIn(state, key);
    if(originalState && action.status !== actions.STATUS_PROGRESS && originalState.id !== action.id){
        // eslint-disable-next-line no-console
        console.log(`%c handleAction discarding action @ ${action.type} since the id doesn't match the id of the same action in the store`, "font-weight: bold;");
        return state;
    }

    return state.setIn(key, buildStateFromAction(action, postProcessor));
};

export const addUnhandledMessages = ((state, action) => {
    action = _.cloneDeep(action);
    action.unhandledCounter = action.unhandledCounter ? action.unhandledCounter + 1 : 1;
    if (action.unhandledCounter >= 5) {
        // eslint-disable-next-line no-console
        console.log(`%c addUnhandledMessages action @ ${action.type} wasn't handled for at least 5 messages, discarding`, "font-weight: bold;");
        return state;
    }
    else return state.update("unhandledMessages", unhandledMessages => unhandledMessages.concat(action));
});

export const isProtocolIncludedWithoutProvider = (protocol, wizardInfo) => {
    return (wizardInfo.source.protocol === protocol && !wizardInfo.source.provider) || (wizardInfo.target.protocol === protocol && !wizardInfo.target.provider);
};

export const handleUnhandledMessages = (state, handleAction, metaData) => {
    return state.unhandledMessages.reduce((state, action) => {
        // eslint-disable-next-line no-console
        console.log(`%c Handling unhandled message @ ${action.type} (${action.unhandledCounter})`, "font-weight: bold;");
        return handleAction(state, action, metaData)
    }, state.set("unhandledMessages", []));
};

export const handleErrors = (action, state) => {
    if (!routesContainer.routes) return state;

    if (action.status === actionCreatorsConsts.STATUS_ERROR && !action.selfErrorHandling) {
        const thePayload = action.payload;
        const errorTime = moment();
        let errorMessage = "Unknown error";
        let errorKey = action.type;
        if (!_.isUndefined(thePayload)) {
            const gotErrorResponse = !_.isEmpty(thePayload);
            const isNetworkError = gotErrorResponse && thePayload.message === "Network Error";
            const isClientError = !gotErrorResponse;
            errorKey = thePayload.status === 504 || thePayload.status === 502 || thePayload.status === 503 || isNetworkError ? "NETWORK" : action.type;
            errorMessage = thePayload.status === 504 || thePayload.status === 502 || thePayload.status === 503 || isNetworkError ? "Please check your network connectivity and refresh the browser." : !isClientError ? (thePayload.data?.message || thePayload.data?.reason || thePayload.data) : `Client Error: ${thePayload.message}`;
            if (!errorMessage) errorMessage = thePayload?.statusText || thePayload?.response?.data?.message || "Unknown error";


            //ugly patch to fix the firefox bug SER-394 During refresh of browser a connectivity error pops up in the client without a need, on messages.cancel the firefox throws it's own weired network error, we don't want to show it
            if (isNetworkError && thePayload.fileName && thePayload.fileName.indexOf("bundle.js") !== -1) {
                return state;
            }

            /* eslint-disable no-console */
            try {
                if (isClientError && console.error) {
                    console.error(action.payload)
                }
            } catch (e) {
                if (console.log) console.log(`Suppressed error ${action.payload.message}`);
            }
            /* eslint-enable no-console */
        }
        const error = {
            id: errorKey,
            message: errorMessage,
            time: errorTime
        };

        return updateErrorsPanelStatus(state.updateIn(["_generalErrors", "_errorsMap"], _errorsMap => _errorsMap.set(errorKey, error)));
    }

    if (action.status && action.status !== actionCreatorsConsts.STATUS_ERROR) {
        const updateErrorsMap = (errorsMap) => {
            const updatedErrorsMap = errorsMap.without(action.type).without(actionCreatorsConsts.action_update_map[action.type]);
            if (action.status === actionCreatorsConsts.STATUS_SUCCESS) return updatedErrorsMap.without("NETWORK");
            else return updatedErrorsMap;
        };

        return updateErrorsPanelStatus(state.updateIn(["_generalErrors", "_errorsMap"], updateErrorsMap));
    } else return state;
};

export const processAgent = (entry, wizardInfo, isRunning) => {

    const addErrorMessage = (entry, cloudCredentialsSource, cloudProvider, cloudStorageName, article) => {
        entry.isDisabled = entry.placement && !entry.placement[cloudCredentialsSource];
        entry.disabledErrorMessage = `A data broker that doesn’t have ${cloudProvider} credentials can’t be used in a relationship that includes ${article} ${cloudStorageName} bucket.
            Select ${article} ${cloudProvider} data broker, or select an on-prem data broker that has ${cloudProvider} credentials.`;
    };

    entry.isFailed = entry.status === consts.BROKER_STATUS_FAILED;
    entry.isWaiting = _.isEmpty(entry.placement);
    entry.isReady = !entry.isWaiting && !entry.isFailed;
    entry.isDisabled = false;

    if (wizardInfo) {
        if (isProtocolIncludedWithoutProvider('s3', wizardInfo)) {
            addErrorMessage(entry, 'awsAccountId', 'AWS', 'S3', 'an');
        }
        if (!entry.isDisabled && isProtocolIncludedWithoutProvider('gcp', wizardInfo)) {
            addErrorMessage(entry, 'gcpProjectId', 'GCP', 'GCP', 'a');
        }
    }

    entry.creationFrom = fromNowNoFuture(entry.createdAt);
    entry = attachDataBrokerUI(entry, isRunning);

    if (entry.isFailed && entry.message) entry.agentFailureMessage = entry.message.message ? entry.message.message : entry.message;

    return entry;
};

const attachDataBrokerUI = (dataBroker, isRunning) => {
    dataBroker = _.cloneDeep(dataBroker);
    _.set(dataBroker, 'ui.transferRate', isRunning && dataBroker.isReady ? calcTransferRate(dataBroker.transferRate) : "");
    switch (dataBroker.type) {
        case consts.FM_AGENT_TYPE_AZURE.name:
            _.set(dataBroker, 'ui.type', consts.FM_AGENT_TYPE_AZURE);
            break;
        case consts.FM_AGENT_TYPE_GCP.name:
            _.set(dataBroker, 'ui.type', consts.FM_AGENT_TYPE_GCP);
            break;
        case consts.FM_AGENT_TYPE_ONPREM.name:
            _.set(dataBroker, 'ui.type', consts.FM_AGENT_TYPE_ONPREM);
            break;
        case consts.FM_AGENT_TYPE_AWS.name:
        default:
            _.set(dataBroker, 'ui.type', consts.FM_AGENT_TYPE_AWS);
            break;
    }
    return dataBroker;
};

export const processRelationship = (relationship) => {
    const isRunning = relationship.activity.status === consts.RESPONSE_STATUS_RUNNING;
    const uploadScanning = relationship.activity.scanning;
    const calculatedProgress = uploadScanning ?
        `(preparing ${numeral(uploadScanning.objectsCompleted + uploadScanning.filesCompleted + uploadScanning.objectsIgnored).format('0.[0]a')})` :
        (isRunning && !isNil(relationship.activity.progress) ? `(${relationship.activity.progress}%)` : "");
    const statusConverted = relationship.activity.status === consts.RESPONSE_STATUS_DONE ?
        (relationship.activity.filesFailed === 0 && relationship.activity.dirsFailed === 0 && relationship.activity.dirsFailedToScan === 0 ? 'SUCCESS' : 'COMPLETE') : relationship.activity.status;
    const tags = relationship.target[relationship.target.protocol].tags;
    const dataBrokers = relationship.group.dataBrokers.map((dataBroker) => processAgent(dataBroker, undefined, isRunning));
    //if all the agents are in failed/waiting status, we want to show unknown status for a running copy/sync, or the last sync/copy status for a non currently running relationship.
    const showUnknown = isRunning && noConnectionToAllBrokers(dataBrokers);
    const estimatedTime = isRunning && !isNil(relationship.activity.timeEstimation) ? stringDuration(relationship.activity.timeEstimation * 1000) : undefined;
    const estimatedTimeText = estimatedTime ? `${estimatedTime === consts.TEXT_DURATION_FEW_SECONDS ? '' : 'About '}${estimatedTime} remaining` : '';
    const copyDoneByCloudProvider = getCopyDoneByCloudProvider(relationship.source, relationship.target);

    relationship.ui = Object.assign({}, formatActivityValues(relationship.activity), {
        showUnknown,
        statusText: showUnknown ? consts.UNKNOWN_SYNC_STATUS : statusConvert[statusConverted],
        progress: calculatedProgress,
        status: relationship.activity.status,
        pendingSyncStartTime: relationship.activity.status === consts.RESPONSE_STATUS_PENDING ? getNextTimeHour(get(relationship.settings, 'schedule.nextTime')) : '',
        isScheduleOn: get(relationship.settings, 'schedule.isEnabled') ? "ON" : "OFF",
        failureMessage: relationship.activity.failureMessage,
        actualTarget: relationship.actualTarget,
        source: relationship.source,
        target: relationship.target,
        host: relationship.host,
        type: relationship.activity.type,
        tags,
        dataBrokers,
        estimatedTimeText,
        copyDoneByCloudProvider,
        recommendations:[]
    });
};

export const processTimeline = (timeline) => {
    const convertTimelineStatus = {
        PROGRESS: "In Progress",
        DONE: "Success",
        FAILED: "Failed",
        INTERRUPTED: "Interrupted",
        ABORTING: "Aborting",
        ABORTED: "Aborted",
        PENDING: "Pending"
    };
    timeline.createdAt = moment(timeline.createdAt).format('MMMM Do YYYY, h:mm a');
    timeline.status = convertTimelineStatus[timeline.status];
};