import DataBrokerStep from '../components/syncWizard/DataBrokerStep';
import DataBrokerFunctionStep from '../components/syncWizard/DataBrokerFunctionStep';
import ShowBucketsStep from '../components/syncWizard/ShowBucketsStep';
import ShowGCPBucketsStep from '../components/syncWizard/ShowGCPBucketsStep';
import S3BasedBucketsStep from '../components/syncWizard/S3BasedBucketsStep';
import NFSServerStep from '../components/syncWizard/NFSServerStep';
import CIFSServerStep from '../components/syncWizard/CIFSServerStep';
import AzureBlobStep from '../components/syncWizard/AzureBlobStep';
import S3BasedServerStep from '../components/syncWizard/S3BasedServerStep';
import ShowExportsStep from '../components/syncWizard/ShowExportsStep';
import ShowSharesStep from '../components/syncWizard/ShowSharesStep';
import ShowAzureBlobContainersStep from '../components/syncWizard/ShowAzureBlobContainersStep';
import RelationshipTagsStep from '../components/syncWizard/RelationshipTagsStep';
import SyncSettingsStep from '../components/syncWizard/SyncSettingsStep';
import RelationshipStep from '../components/syncWizard/RelationshipStep';
import AnfWeStep from '../components/syncWizard/AnfWeStep';

export const getNextStepComponent = (nextStepPath) => {
    switch (nextStepPath) {
        case 'agents':
        case 'targetagents':
            return DataBrokerStep;
        case 'brokerfunction':
            return DataBrokerFunctionStep;
        case 'buckets':
        case 'targetbuckets':
            return ShowBucketsStep;
        case 'gcpbuckets':
        case 'targetgcpbuckets':
            return ShowGCPBucketsStep;
        case 's3basedbuckets':
        case 'targets3basedbuckets':
            return S3BasedBucketsStep;
        case 'nfs':
        case 'targetnfs':
            return NFSServerStep;
        case 'targetcifs':
        case 'cifs':
            return CIFSServerStep;
        case 'azure':
        case 'targetazure':
            return AzureBlobStep;
        case 's3based':
        case 'targets3based':
            return S3BasedServerStep;
        case 'targetexports':
        case 'exports':
            return ShowExportsStep;
        case 'shares':
        case 'targetshares':
            return ShowSharesStep;
        case 'blobcontainers':
        case 'targetblobcontainers':
            return ShowAzureBlobContainersStep;
        case 'tags':
            return RelationshipTagsStep;
        case 'settings':
            return SyncSettingsStep;
        case 'review':
            return RelationshipStep;
        case 'anfwe':
        case 'targetanfwe':
            return AnfWeStep;
        default: {
            throw new Error(`Unknown step ${nextStepPath}`);
        }
    }
};

//Currently the only step that differs in cmIntegration mode is "anf"
export const getIsIntegratedStep = (isCmIntegration, endpointObj) => isCmIntegration && endpointObj.provider === "anf";

export const fetchAnfVolumes = (localThis, props) => {
    const { isTargetStepFromSameSourceType, _accountInfo, _anfVolumes} = props;
    const localSelectedAnf = isTargetStepFromSameSourceType
        ? localThis.props._wizard.selectedWizardParams.selectedAnfWeTarget
        : localThis.props._wizard.selectedWizardParams.selectedAnfWe;
    const relevantAnfVols = _anfVolumes[localSelectedAnf];
    if (!relevantAnfVols) {
        const accountId = _accountInfo?.data?.accountId;
        return props.getAnfVolumes(accountId, localSelectedAnf);
    }
};

export const scrollToTop = (element) => {
    if (!element) return;
    element.scrollTo({
        top: 0,
        behavior: 'smooth',
    });
};

export const weSelectionNeeded = (wizardInfo, isCmIntegration) => {
    if (!isCmIntegration) return {
        source: false,
        target: false
    };
    const isSourceAnf = wizardInfo.source.provider === "anf";
    const isTargetAnf = wizardInfo.target.provider === "anf";
    return {
        source: isSourceAnf,
        target: isTargetAnf
    };
};