import * as settings from "./../../src/utils/settings-utils";

describe("createSettingsObjectFromFormValues", () => {
    test("should build defaults object", () => {
        const scheduleDefaultForTest = {...settings.SETTINGS_DEFAULTS.schedule, nextTime: settings.MOCK_DATE_FOR_TESTS};
        expect(settings.createSettingsObjectFromFormValues({
            graceValue: settings.SETTINGS_DEFAULTS.gracePeriod,
            graceResolution: {value: "Seconds"},
            gracePeriodOn: settings.EXCLUDE_FILES_OPTION,
            deleteOnTarget: settings.NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT,
            deleteOnSource: settings.NOT_DELETE_FILES_FROM_SOURCE_OPTION_TEXT,
            objectTagging: settings.OBJECT_TAGGING_USE_OPTION_TEXT,
            fileTypes: [settings.fileTypesMap.files, settings.fileTypesMap.directories, settings.fileTypesMap.symlinks],
            retries: {value: settings.SETTINGS_DEFAULTS.retries},
            filesExtensions: [],
            fileSizeOption: settings.FILE_SIZE_OPTION_DISABLED,
            fileDateModifiedOption: settings.DATE_MODIFIED_OPTIONS.ALL,
            fileDateModifiedMin: null,
            fileDateModifiedMax: null,
            scheduleStatus: settings.SCHEDULE_OPTIONS.NOW,
            scheduleInterval: 1,
            scheduleResolution: {value: 'Days'},
            copyAcl: false
        })).toEqual({...settings.SETTINGS_DEFAULTS, schedule: scheduleDefaultForTest});
    });
});

describe("getFormValuesFromSettings", () => {
    test("should build object from default values", () => {
        expect(settings.getFormValuesFromSettings(settings.SETTINGS_DEFAULTS, false)).toEqual({
            graceValue: settings.SETTINGS_DEFAULTS.gracePeriod,
            graceResolution: {label: "Seconds", value: "Seconds"},
            gracePeriodOn: settings.EXCLUDE_FILES_OPTION,
            deleteOnTarget: settings.NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT,
            deleteOnSource: settings.NOT_DELETE_FILES_FROM_SOURCE_OPTION_TEXT,
            objectTagging: settings.OBJECT_TAGGING_USE_OPTION_TEXT,
            fileTypes: [settings.fileTypesMap.files, settings.fileTypesMap.directories, settings.fileTypesMap.symlinks],
            retries: {label: `${settings.SETTINGS_DEFAULTS.retries}`, value: settings.SETTINGS_DEFAULTS.retries},
            filesExtensions: [],
            fileSizeOption: settings.FILE_SIZE_OPTION_DISABLED,
            fileMinSizeValue: '0',
            fileMinSizeResolution: {label: "B", value: "B"},
            fileMaxSizeValue: '500',
            fileMaxSizeResolution: {label: "TB", value: "TB"},
            fileDateModifiedOption: {label: settings.DATE_MODIFIED_OPTIONS.ALL, value: settings.DATE_MODIFIED_OPTIONS.ALL},
            fileDateModifiedMin: null,
            fileDateModifiedMax: null,
            scheduleStatus: settings.SCHEDULE_OPTIONS.NOW,
            scheduleNextTime: {label: '10:00', value: '10:00'},
            scheduleInterval: 1,
            scheduleResolution: {label: 'Days', value: 'Days'},
            copyAcl: false
        });
    });
});

describe("getGracePeriod", () => {
    test("should return 30", () => {
        expect(settings.getGracePeriod(30, {value: "Seconds"}, true)).toEqual(30);
    });
    test("should return 0 for 0 input", () => {
        expect(settings.getGracePeriod(0, {value: "Seconds"}, true)).toEqual(0);
    });
    test("should return 120", () => {
        expect(settings.getGracePeriod(2, {value: "Minutes"}, true)).toEqual(120);
    });
    test("should return 18000", () => {
        expect(settings.getGracePeriod(5, {value: "Hours"}, true)).toEqual(18000);
    });
    test("should return 0 for disabled", () => {
        expect(settings.getGracePeriod(30, {value: "Hours"}, false)).toEqual(0);
    });
});

describe("getFormGrace", () => {
    test("should return 30 Seconds", () => {
        expect(settings.getFormGrace(30)).toEqual({count: 30, resolution: {label: "Seconds", value: "Seconds"}});
    });
    test("should return 5 Hours", () => {
        expect(settings.getFormGrace(18000)).toEqual({count: 5, resolution: {label: "Hours", value: "Hours"}});
    });
    test("should return 0", () => {
        expect(settings.getFormGrace(0)).toEqual({count: 0, resolution: {label: "Seconds", value: "Seconds"}});
    });
});

describe("getDeleteOnTargetBool", () => {
    test("should return truth", () => {
        expect(settings.getDeleteOnTargetBool(settings.DELETE_FILES_FROM_TARGET_OPTION_TEXT)).toEqual(true);
    });
    test("should return false", () => {
        expect(settings.getDeleteOnTargetBool(settings.NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT)).toEqual(false);
    });
});

describe("getFormDeleteOnTarget", () => {
    test("should return DELETE_FILES_FROM_TARGET_OPTION_TEXT", () => {
        expect(settings.getFormDeleteOnTarget(true)).toEqual(settings.DELETE_FILES_FROM_TARGET_OPTION_TEXT);
    });
    test("should return NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT", () => {
        expect(settings.getFormDeleteOnTarget(false)).toEqual(settings.NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT);
    });
});

describe("getDeleteOnSourceBool", () => {
    test("should return truth", () => {
        expect(settings.getDeleteOnSourceBool(settings.DELETE_FILES_FROM_SOURCE_OPTION_TEXT)).toEqual(true);
    });
    test("should return false", () => {
        expect(settings.getDeleteOnSourceBool(settings.NOT_DELETE_FILES_FROM_SOURCE_OPTION_TEXT)).toEqual(false);
    });
});

describe("getFormDeleteOnSource", () => {
    test("should return DELETE_FILES_FROM_SOURCE_OPTION_TEXT", () => {
        expect(settings.getFormDeleteOnSource(true)).toEqual(settings.DELETE_FILES_FROM_SOURCE_OPTION_TEXT);
    });
    test("should return NOT_DELETE_FILES_FROM_SOURCE_OPTION_TEXT", () => {
        expect(settings.getFormDeleteOnSource(false)).toEqual(settings.NOT_DELETE_FILES_FROM_SOURCE_OPTION_TEXT);
    });
});

describe("getObjectTaggingBool", () => {
    test("should return truth", () => {
        expect(settings.getObjectTaggingBool(settings.OBJECT_TAGGING_USE_OPTION_TEXT)).toEqual(true);
    });
    test("should return false", () => {
        expect(settings.getObjectTaggingBool(settings.OBJECT_TAGGING_DISABLE_OPTION_TEXT)).toEqual(false);
    });
});

describe("getFormObjectTagging", () => {
    test("should return OBJECT_TAGGING_USE_OPTION_TEXT", () => {
        expect(settings.getFormObjectTagging(true)).toEqual(settings.OBJECT_TAGGING_USE_OPTION_TEXT);
    });
    test("should return OBJECT_TAGGING_DISABLE_OPTION_TEXT", () => {
        expect(settings.getFormObjectTagging(false)).toEqual(settings.OBJECT_TAGGING_DISABLE_OPTION_TEXT);
    });
});

describe("getFileTypesObj", () => {
    test("should return only files", () => {
        expect(settings.getFileTypesObj([settings.fileTypesMap.files])).toEqual({files: true, directories: false, symlinks: false});
    });
    test("should return all types", () => {
        expect(settings.getFileTypesObj([settings.fileTypesMap.files, settings.fileTypesMap.directories, settings.fileTypesMap.symlinks])).toEqual({files: true, directories: true, symlinks: true});
    });
});

describe("getFormFileTypes", () => {
    test("should return only files", () => {
        expect(settings.getFormFileTypes({files: true, directories: false, symlinks: false})).toEqual([settings.fileTypesMap.files]);
    });
    test("should return all types", () => {
        expect(settings.getFormFileTypes({files: true, directories: true, symlinks: true})).toEqual([settings.fileTypesMap.files, settings.fileTypesMap.directories, settings.fileTypesMap.symlinks]);
    });
});

describe("getScheduleData", () => {
    test("should return 5 days", () => {
        expect(settings.getScheduleData({
            scheduleInterval: 5,
            scheduleResolution: {value: "Days"},
            scheduleStatus: settings.SCHEDULE_OPTIONS.NOW
        })).toEqual({
            syncInDays: 5,
            syncInHours: 0,
            syncInMinutes: 0,
            isEnabled: true,
            syncWhenCreated: true,
            nextTime: settings.MOCK_DATE_FOR_TESTS
        });
    });
    test("should return isEnabled - false", () => {
        expect(settings.getScheduleData({
            scheduleStatus: settings.SCHEDULE_OPTIONS.ONCE,
            scheduleInterval: 5,
            scheduleResolution: {value: "Days"}
        })).toMatchObject({isEnabled: false});
    });

});

describe("getFileTypesSummary", () => {
    test("should return summary in case all types are selected", () => {
        expect(settings.getFileTypesSummary([settings.fileTypesMap.symlinks, settings.fileTypesMap.directories, settings.fileTypesMap.files]))
            .toEqual(`${settings.fileTypesMap.files}, ${settings.fileTypesMap.directories}, ${settings.fileTypesMap.symlinks}`);
    });
    test("should return summary in case 2 types are selected", () => {
        expect(settings.getFileTypesSummary([settings.fileTypesMap.symlinks, settings.fileTypesMap.files]))
            .toEqual(`${settings.fileTypesMap.files}, ${settings.fileTypesMap.symlinks}`);
    });
});

describe("getSummaryLines", () => {
    test("should build summary lines object", () => {
        const input = {
            gracePeriodOn: settings.NOT_EXCLUDE_FILES_OPTION,
            deleteOnTarget: settings.NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT,
            fileTypes: [settings.fileTypesMap.files],
            retries: {value: 4},
            filesExtensions: [],
            fileSizeOption: settings.FILE_SIZE_OPTION_DISABLED,
            fileDateModifiedOption: {value: settings.DATE_MODIFIED_OPTIONS.ALL},
            scheduleStatus: settings.SCHEDULE_OPTIONS.NOW,
            scheduleResolution: {value: 'Days'},
            scheduleInterval: 1
        };
        expect(settings.getSummaryLines(input)).toEqual({
            gracePeriod: settings.NOT_EXCLUDE_FILES_OPTION,
            deleteOnTarget: settings.NOT_DELETE_FILES_FROM_TARGET_OPTION_TEXT,
            fileTypes: `Include Only: ${settings.fileTypesMap.files}`,
            retries: `Retry 4 times before skipping file`,
            filesExtensions: 'None',
            fileSize: 'All',
            dateModified: settings.DATE_MODIFIED_OPTIONS.ALL,
            schedule: 'ON | Every 1 Day',
            acl: "Inactive"
        })
    });
});

describe("extensionsInputTransformation", () => {
    test(`should return the input with a "." prefix if needed`, () => {
        expect(settings.extensionsInputTransformation('abc')).toEqual('.abc');
        expect(settings.extensionsInputTransformation('.abc')).toEqual('.abc');
        expect(settings.extensionsInputTransformation('.a.bc')).toEqual('.a.bc');
        expect(settings.extensionsInputTransformation('a.bc')).toEqual('.a.bc');
    });
});

describe("convertDateToString", () => {
    test(`should convert a date to string`, () => {
        expect(settings.convertDateToString(new Date('08/10/1980'))).toEqual('1980-08-10');
    });
    test(`should return null`, () => {
        expect(settings.convertDateToString(null)).toBeNull();
    });
});

describe("convertStringToDate", () => {
    test(`should return null in case of default`, () => {
        expect(settings.convertStringToDate(settings.SETTINGS_DEFAULTS.minDate)).toBeNull();
    });
});



