import * as sync from "./../../src/utils/sync-utils";
import * as settingsUtils from "./../../src/utils/settings-utils";
import {initTests} from "./../globals";
import {
    DEFAULT_NFS_VERSION
} from './../../src/consts';

beforeEach(() => {
    initTests();
});

describe("getWizardInfo", () => {
    test("should build wizardInfo object for nfsAnf-s3", () => {
        expect(sync.getWizardInfo('nfs-s3', 'anf-NA')).toEqual({source: {protocol: 'nfs', provider: 'anf'}, target: {protocol: 's3', provider: undefined}});
    });
    test("should build wizardInfo object for nfs-azure", () => {
        expect(sync.getWizardInfo('nfs-azure', 'NA-NA')).toEqual({source: {protocol: 'nfs', provider: undefined}, target: {protocol: 'azure', provider: undefined}});
    });
});

describe("getStepsOrder", () => {

    test("should return steps order per relationship nfs-s3", () => {
        const wizardInfo = {source: {protocol: 'nfs', provider: undefined}, target: {protocol: 's3', provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(['nfs', 'dataBroker', "directories", "s3Buckets", "settings", "relationshipTags", "review"]);
    });

    test("should return steps order per relationship nfsEfs-nfs", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: "efs"}, target: {protocol: "nfs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(['nfsEfs', 'dataBroker', "directories", "targetNfs", "targetDirectories", "settings", "review"]);
    });

    test("should return steps order per relationship s3-nfs", () => {
        const wizardInfo = {source: {protocol: "s3", provider: undefined}, target: {protocol: "nfs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["dataBroker", "s3Buckets", "nfs" , "directories", "settings", "review"]);
    });

    test("should return steps order per relationship s3-s3", () => {
        const wizardInfo = {source: {protocol: "s3", provider: undefined}, target: {protocol: "s3", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["dataBroker", "s3Buckets", "targetS3Buckets" , "settings", "relationshipTags", "review"]);
    });

    test("should return steps order per relationship s3-cifs", () => {
        const wizardInfo = {source: {protocol: "s3", provider: undefined}, target: {protocol: "cifs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["dataBroker", "s3Buckets", "cifs" , "shares", "settings", "review"]);
    });

    test("should return steps order per relationship s3-nfsEfs", () => {
        const wizardInfo = {source: {protocol: "s3", provider: undefined}, target: {protocol: "nfs", provider: "efs"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["dataBroker", "s3Buckets", "nfsEfs" , "directories", "settings", "review"]);
    });

    test("should return steps order per relationship s3-nfsCvs", () => {
        const wizardInfo = {source: {protocol: "s3", provider: undefined}, target: {protocol: "nfs", provider: "cvs"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["dataBroker", "s3Buckets", "nfsCvs" , "directories", "settings", "review"]);
    });

    test("should return steps order per relationship s3-s3Sgws", () => {
        const wizardInfo = {source: {protocol: "s3", provider: undefined}, target: {protocol: "s3", provider: "sgws"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["dataBroker", "s3Buckets", "targetS3Sgws" , "targetS3SgwsBuckets", "settings", "review"]);
    });

    test("should return steps order per relationship nfs-nfs", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfs", "dataBroker", "directories", "targetNfs", "targetDirectories" , "settings", "review"]);
    });

    test("should return steps order per relationship secure nfs-nfs", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo, true)).toEqual(["nfs", "dataBrokerFunction", "dataBroker", "directories", "targetNfs", "targetDataBroker", "targetDirectories" , "settings", "review"]);
    });7

    test("should return steps order per relationship nfs-s3Sgws", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: undefined}, target: {protocol: "s3", provider: "sgws"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfs", "dataBroker", "directories", "s3Sgws", "s3SgwsBuckets" , "settings", "review"]);
    });

    test("should return steps order per relationship nfs-gcp", () => {
        const wizardInfo = { source: {protocol: "nfs", provider: undefined}, target: {protocol: "gcp", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfs", "dataBroker", "directories", "gcpBuckets", "settings", "review"]);
    });

    test("should return steps order per relationship nfs-s3Ibm", () => {
        const wizardInfo = { source: {protocol: "nfs", provider: undefined}, target: {protocol: "s3", provider: "ibm"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfs", "dataBroker", "directories", "s3Ibm", "s3IbmBuckets", "settings", "review"]);
    });

    test("should return steps order per relationship nfs-nfsEfs", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: "efs"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfs", "dataBroker", "directories", "targetNfsEfs", "targetDirectories", "settings", "review"]);
    });

    test("should return steps order per relationship nfs-nfsCvs", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: "cvs"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfs", "dataBroker", "directories", "targetNfsCvs", "targetDirectories", "settings", "review"]);
    });

    test("should return steps order per relationship nfs-nfsAnf", () => {
        const wizardInfo = { source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: "anf"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfs", "dataBroker", "directories", "targetNfsAnf", "targetDirectories", "settings", "review"]);
    });


    test("should return steps order per relationship nfs-azure", () => {
        const wizardInfo = { source: {protocol: "nfs", provider: undefined}, target: {protocol: "azure", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfs", "dataBroker", "directories", "azureBlob", "blobContainers", "settings", "relationshipTags", "review"]);
    });

    test("should return steps order per relationship nfsEfs-s3", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: "efs"}, target: {protocol: "s3", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfsEfs", "dataBroker", "directories", "s3Buckets", "settings", "relationshipTags", "review"]);
    });

    test("should return steps order per relationship cifs-cifs", () => {
        const wizardInfo = { source: {protocol: "cifs", provider: undefined}, target: {protocol: "cifs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["cifs", "dataBroker", "shares", "targetCifs", "targetShares", "settings", "review"]);
    });

    test("should return steps order per relationship cifs-s3", () => {
        const wizardInfo = {source: {protocol: "cifs", provider: undefined}, target: {protocol: "s3", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["cifs", "dataBroker", "shares", "s3Buckets", "settings", "relationshipTags", "review"]);
    });

    test("should return steps order per relationship cifs-s3Sgws", () => {
        const wizardInfo = { source: {protocol: "cifs", provider: undefined}, target: {protocol: "s3", provider: "sgws"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["cifs", "dataBroker", "shares", "s3Sgws", "s3SgwsBuckets", "settings", "review"]);
    });


    test("should return steps order per relationship cifs-s3Ibm", () => {
        const wizardInfo = { source: {protocol: "cifs", provider: undefined}, target: {protocol: "s3", provider: "ibm"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["cifs", "dataBroker", "shares", "s3Ibm", "s3IbmBuckets", "settings", "review"]);
    });

    test("should return steps order per relationship cifs-azure", () => {
        const wizardInfo = {source: {protocol: "cifs", provider: undefined}, target: {protocol: "azure", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["cifs", "dataBroker", "shares", "azureBlob", "blobContainers", "settings", "relationshipTags", "review"]);
    });

    test("should return steps order per relationship cifs-gcp", () => {
        const wizardInfo = { source: {protocol: "cifs", provider: undefined}, target: {protocol: "gcp", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["cifs", "dataBroker", "shares", "gcpBuckets", "settings", "review"]);
    });

    test("should return steps order per relationship azure-nfs", () => {
        const wizardInfo = { source: {protocol: "azure", provider: undefined}, target: {protocol: "nfs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["azureBlob", "dataBroker", "blobContainers", "nfs", "directories", "settings", "review"]);
    });


    test("should return steps order per relationship azure-cifs", () => {
        const wizardInfo = { source: {protocol: "azure", provider: undefined}, target: {protocol: "cifs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["azureBlob", "dataBroker", "blobContainers", "cifs", "shares", "settings", "review"]);
    });

    test("should return steps order per relationship azure-azure", () => {
        const wizardInfo = { source: {protocol: "azure", provider: undefined}, target: {protocol: "azure", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["azureBlob", "dataBroker", "blobContainers", "targetAzureBlob", "targetBlobContainers", "settings", "relationshipTags", "review"]);
    });

    test("should return steps order per relationship gcp-nfs", () => {
        const wizardInfo = {source: {protocol: "gcp", provider: undefined},target: {protocol: "nfs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["dataBroker", "gcpBuckets", "nfs", "directories", "settings", "review"]);
    });

    test("should return steps order per relationship gcp-cifs", () => {
        const wizardInfo = {source: {protocol: "gcp", provider: undefined}, target: {protocol: "cifs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["dataBroker", "gcpBuckets", "cifs", "shares", "settings", "review"]);
    });

    test("should return steps order per relationship s3Sgws-nfs", () => {
        const wizardInfo = {source: {protocol: "s3", provider: "sgws"}, target: {protocol: "nfs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["s3Sgws", "dataBroker", "s3SgwsBuckets", "nfs", "directories", "settings", "review"]);
    });

    test("should return steps order per relationship s3Sgws-cifs", () => {
        const wizardInfo = {source: {protocol: "s3", provider: "sgws"}, target: {protocol: "cifs", provider: undefined} };
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["s3Sgws", "dataBroker", "s3SgwsBuckets", "cifs", "shares", "settings", "review"]);
    });

    test("should return steps order per relationship s3Sgws-s3", () => {
        const wizardInfo = {source: {protocol: "s3", provider: "sgws"}, target: {protocol: "s3", provider: undefined} };
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["s3Sgws", "dataBroker", "s3SgwsBuckets", "targetS3Buckets", "settings", "relationshipTags", "review"]);
    });

    test("should return steps order per relationship s3Sgws-s3Sgws", () => {
        const wizardInfo = { source: {protocol: "s3", provider: "sgws"}, target: {protocol: "s3", provider: "sgws"} };
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["s3Sgws", "dataBroker", "s3SgwsBuckets", "targetS3Sgws", "targetS3SgwsBuckets", "settings", "review"]);
    });

    test("should return steps order per relationship s3Ibm-s3Ibm", () => {
        const wizardInfo = { source: {protocol: "s3", provider: "ibm"}, target: {protocol: "s3", provider: "ibm"} };
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["s3Ibm", "dataBroker", "s3IbmBuckets", "targetS3Ibm", "targetS3IbmBuckets", "settings", "review"]);
    });

    test("should return steps order per relationship s3Ibm-cifs", () => {
        const wizardInfo = { source: {protocol: "s3", provider: "ibm"}, target: {protocol: "cifs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["s3Ibm", "dataBroker", "s3IbmBuckets", "cifs", "shares", "settings", "review"]);
    });

    test("should return steps order per relationship s3Ibm-nfs", () => {
        const wizardInfo = { source: {protocol: "s3", provider: "ibm"}, target: {protocol: "nfs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["s3Ibm", "dataBroker", "s3IbmBuckets", "nfs", "directories", "settings", "review"]);
    });


    test("should return steps order per relationship nfsCvs-s3", () => {
        const wizardInfo = { source: {protocol: "nfs", provider: "cvs"}, target: {protocol: "s3", provider: undefined} };
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfsCvs", "dataBroker", "directories", "s3Buckets", "settings", "relationshipTags", "review"]);
    });

    test("should return steps order per relationship nfsCvs-nfs", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: "cvs"}, target: {protocol: "nfs", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfsCvs", "dataBroker", "directories", "targetNfs", "targetDirectories", "settings", "review"]);
    });

    test("should return steps order per relationship nfsAnf-s3", () => {
        const wizardInfo = { source: {protocol: "nfs", provider: "anf"},target: {protocol: "s3", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfsAnf", "dataBroker", "directories", "s3Buckets", "settings", "relationshipTags", "review"]);
    });

    test("should return steps order per relationship nfsAnf-nfsAnf", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: "anf"},target: {protocol: "nfs", provider: "anf"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfsAnf", "dataBroker", "directories", "targetNfsAnf", "targetDirectories", "settings", "review"]);
    });

    test("should return steps order per relationship nfsEfs-nfsAnf", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: "efs"}, target: {protocol: "nfs", provider: "anf"} };
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfsEfs", "dataBroker", "directories", "targetNfsAnf", "targetDirectories", "settings", "review"]);
    });

    test("should return steps order per relationship nfsAnf-azure", () => {
        const wizardInfo = { source: {protocol: "nfs", provider: "anf"},  target: {protocol: "azure", provider: undefined}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["nfsAnf", "dataBroker", "directories", "azureBlob", "blobContainers", "settings", "relationshipTags", "review"]);
    });

    test("should return steps order per relationship azure-nfsAnf", () => {
        const wizardInfo = { source: {protocol: "azure", provider: undefined},target: {protocol: "nfs", provider: "anf"}};
        expect(sync.getStepsOrder(wizardInfo)).toEqual(["azureBlob", "dataBroker", "blobContainers", "nfsAnf", "directories", "settings", "review"]);
    });

    test("should return steps order per relationship nfs-nfsAnf in cmIntegration mode", () => {
        const wizardInfo = { source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: "anf"}};
        const weSelectionNeeded = {source: false, target: true};
        expect(sync.getStepsOrder(wizardInfo, false, weSelectionNeeded)).toEqual(["nfs", "dataBroker", "directories", "targetAnfWe", "targetNfsAnf", "targetDirectories", "settings", "review"]);
    });

    test("should return steps order per secure relationship nfsAnf-nfsAnf in cmIntegration mode", () => {
        const wizardInfo = { source: {protocol: "nfs", provider: "anf"}, target: {protocol: "nfs", provider: "anf"}};
        const weSelectionNeeded = {source: true, target: true};
        expect(sync.getStepsOrder(wizardInfo, true, weSelectionNeeded)).toEqual(["anfWe", "nfsAnf", "dataBrokerFunction", "dataBroker", "directories", "targetAnfWe", "targetNfsAnf", "targetDataBroker", "targetDirectories", "settings", "review"]);
    });
});

describe("getProtocolAndProviderKey", () => {
    test("should build protocolAndProviderKey for protocol and provider", () => {
        expect(sync.getProtocolAndProviderKey('nfs', 'anf')).toEqual('nfsAnf');
    });
    test("should build protocolAndProviderKey for protocol without provider", () => {
        expect(sync.getProtocolAndProviderKey('s3', undefined)).toEqual('s3');
    });
});

describe("getEndPointObjectForStep", () => {
    test("should return the correct EndPoint object when nfs is source", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: undefined}, target: {protocol: "s3", provider: undefined}};
        expect(sync.getEndPointObjectForStep(wizardInfo, false, "nfs")).toEqual({protocol: "nfs", provider: undefined});
    });
    test("should return the correct EndPoint object when nfs is target", () => {
        const wizardInfo = {source: {protocol: "s3", provider: undefined}, target: {protocol: "nfs", provider: "anf"}};
        expect(sync.getEndPointObjectForStep(wizardInfo, undefined, "nfsAnf")).toEqual({protocol: "nfs", provider: "anf"});
    });
    test("should return the correct EndPoint object when nfs is target and isTargetOfSameSource is true", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: "anf"}};
        expect(sync.getEndPointObjectForStep(wizardInfo, true, "nfsAnf")).toEqual({protocol: "nfs", provider: "anf"});
    });
    test("should return the correct EndPoint object when s3Sgws is target", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: undefined}, target: {protocol: "s3", provider: "ibm"}};
        expect(sync.getEndPointObjectForStep(wizardInfo, false, "s3IbmBuckets")).toEqual({protocol: "s3", provider: "ibm"});
    });
    test("should return error when stepProtocol is not found", () => {
        const wizardInfo = {source: {protocol: "cifs", provider: undefined}, target: {protocol: "s3", provider: undefined}};
        expect(()=> {sync.getEndPointObjectForStep(wizardInfo, undefined, "nfs")}).toThrow('wizardInfo does not include nfs');
    });
});

describe("calculateStepId", () => {
    test("should return stepId in case of nfsEfs as source", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: "efs"}, target: {protocol: "s3", provider: undefined}};
        expect(sync.calculateStepId(wizardInfo, undefined, "nfs")).toEqual("nfsEfs");
    });
    test("should return stepId in case of nfsEfs as target", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: "efs"}};
        expect(sync.calculateStepId(wizardInfo, true, "nfs")).toEqual("targetNfsEfs");
    });
    test("should return stepId in case of sgwsBuckets as source", () => {
        const wizardInfo = {source: {protocol: "sgws", provider: undefined}, target: {protocol: "sgws", provider: undefined}};
        expect(sync.calculateStepId(wizardInfo, undefined, "sgws", "Buckets")).toEqual("sgwsBuckets");
    });
    test("should return stepId in case of sgwbIbmBuckets as target", () => {
        const wizardInfo = {source: {protocol: "sgws", provider: "ibm"}, target: {protocol: "sgws", provider: "ibm"}};
        expect(sync.calculateStepId(wizardInfo, true, "sgws", "Buckets")).toEqual("targetSgwsIbmBuckets");
    });
});

describe("getWizardTypeForSystemFlow", () => {
    test("should return wizardType for nfs-efs", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: "efs"}};
        expect(sync.getWizardTypeForSystemFlow(wizardInfo)).toEqual("nfs-efs");
    });
    test("should return wizardType for anf-s3", () => {
        const wizardInfo = {source: {protocol: "nfs", provider: "anf"}, target: {protocol: "s3", provider: undefined}};
        expect(sync.getWizardTypeForSystemFlow(wizardInfo)).toEqual("anf-s3");
    });
    test("should return wizardType for cifs-cifs", () => {
        const wizardInfo = {source: {protocol: "cifs", provider: undefined}, target: {protocol: "cifs", provider: undefined}};
        expect(sync.getWizardTypeForSystemFlow(wizardInfo)).toEqual("cifs-cifs");
    });
});

describe("splitCombinationToSourceTarget", () => {
    test("should return object for cifs-s3", () => {
        expect(sync.splitCombinationToSourceTarget("cifs-s3")).toEqual({source: 'cifs', target: 's3'});
    });
    test("should return object for s3-nfs", () => {
        expect(sync.splitCombinationToSourceTarget("s3-nfs")).toEqual({source: "s3", target: "nfs"});
    });
    test("should return object for anf-nfs", () => {
        expect(sync.splitCombinationToSourceTarget("anf-nfs")).toEqual({source: 'anf', target: 'nfs'});
    });
});

describe("breakFullPathToRootAndPath", () => {
    const paths = "folder1/folder2/folder3/folder4";
    const root = "/folder1/folder2/folder3";
    test("should return the same paths case1", () => {
        expect(sync.breakFullPathToRootAndPath(paths, "folder2")).toEqual({
            "pathWithoutRoot": "folder1/folder2/folder3/folder4",
            "rootOfPath": "folder2"
        });
    });
    test("should return the same paths case2", () => {
        expect(sync.breakFullPathToRootAndPath(paths, "")).toEqual({
            "pathWithoutRoot": "folder1/folder2/folder3/folder4",
            "rootOfPath": ""
        });
    });
    test("should return the same paths case3", () => {
        expect(sync.breakFullPathToRootAndPath("", root)).toEqual({
            "pathWithoutRoot": "",
            "rootOfPath": "/folder1/folder2/folder3"
        });
    });
    test("should return [/folder1/folder2/folder3]", () => {
        expect(sync.breakFullPathToRootAndPath("folder1/folder2/folder3", root)).toEqual({
            "pathWithoutRoot": "",
            "rootOfPath": "/folder1/folder2/folder3"
        });
    });
    test("should return [folder1/folder2/folder3, folder4]", () => {
        expect(sync.breakFullPathToRootAndPath(paths, root)).toEqual({
            "pathWithoutRoot": "folder4",
            "rootOfPath": "/folder1/folder2/folder3"
        });
    });


});

describe("appendItemToLocal", () => {
    const domain = "guy", username = "test1", password = "123456", ip = "1.1.1.1";
    let cred = {domain, username, password};
    let retCred = {domain, username, password};
    test("should return the cred", () => {
        sync.appendItemToLocalStorage("cifs", ip, cred);
        expect(sync.getCredentialsByType("cifs", ip)).toEqual(retCred);
    });
    test("should overwrite the username", () => {
        cred = {...cred, username: "test2"};
        retCred = {...retCred, username: "test2"};
        sync.appendItemToLocalStorage("cifs", ip, cred);
        expect(sync.getCredentialsByType("cifs", ip)).toEqual(retCred);
    });
    test("should return both of the cred", () => {
        const cred2 = {...cred, username: "test4"};
        const retCred2 = {...retCred, username: "test4"};
        sync.appendItemToLocalStorage("cifs", ip, cred);
        sync.appendItemToLocalStorage("cifs", "2.2.2.2", cred2);
        expect(sync.getCredentialsByType("cifs", ip)).toEqual(retCred);
        expect(sync.getCredentialsByType("cifs", "2.2.2.2")).toEqual(retCred2);
    });
    test("should return both cifs and sg", () => {
        const cred2 = {...cred, username: "test4"};
        const retCred2 = {...retCred, username: "test4"};
        const sgCred = {accessKey: "accessKey", secretKey: "secretKey", port: "80"};
        const sgCred2 = {accessKey: "accessKey2", secretKey: "secretKey2", port: "88"};
        sync.appendItemToLocalStorage("cifs", ip, cred);
        sync.appendItemToLocalStorage("cifs", "2.2.2.2", cred2);
        expect(sync.getCredentialsByType("cifs", ip)).toEqual(retCred);
        expect(sync.getCredentialsByType("cifs", "2.2.2.2")).toEqual(retCred2);
        sync.appendItemToLocalStorage("objectStorage", ip, sgCred);
        sync.appendItemToLocalStorage("objectStorage", "2.2.2.2", sgCred2);
        expect(sync.getCredentialsByType("objectStorage", ip)).toEqual(sgCred);
        expect(sync.getCredentialsByType("objectStorage", "2.2.2.2")).toEqual(sgCred2);
    });
});

describe('isSourceTargetSupported', () => {
    test('should return true on supported combination', function () {
        expect(sync.isSourceTargetSupported({protocol: "nfs"}, {protocol: "s3"})).toEqual(true);
    });
    test('should return false on unsupported combination', function () {
        expect(sync.isSourceTargetSupported({protocol: "nfs"}, {protocol: "cifs"})).toEqual(false);
    });
});

describe("splitCombinationToSourceTarget", () => {
    test("should return the right protocols for the requested combination", () => {
        expect(sync.splitCombinationToSourceTarget("s3-nfs")).toEqual({source: "s3", target: "nfs"});
    });

    test("should fix the sg to the right protocol", () => {
        expect(sync.splitCombinationToSourceTarget("nfs-sgws")).toEqual({source: "nfs", target: "sgws"});
    });
});

describe("getCredentials", () => {
    const saveCredentials = (key, innerKey, value) => {
        window.localStorage.setItem(key, JSON.stringify({[innerKey]: value}));
    };
    test("should return the right CIFS credentials when exist", () => {
        const innerKey = "1.1.1.1";
        const item = {domain: "domain", username: "user", password: "password"};
        saveCredentials("cifs", innerKey, item);
        expect(sync.getCredentialsByType("cifs", "1.1.1.1")).toEqual({
            domain: "domain",
            username: "user",
            password: "password"
        });
    });

    test("should return the right SG credentials when exist", () => {
        const item = {port: "80", secretKey: "secretKey", accessKey: "accessKey"};
        const key = "1.1.1.1";
        saveCredentials("storageGrid", key, item);
        const serverCredentials = sync.getCredentialsByType("storageGrid", key);
        expect(serverCredentials).toEqual(item);
    });

});

describe("buildSyncObject", () => {
    const fillOSCredentials = (host) => {
        const secretKey = "secretKey2";
        const accessKey = "accessKey2";
        const port = "88";
        sync.appendItemToLocalStorage("objectStorage", host, {port, secretKey, accessKey});
    };

    const fillCIFSCredentials = (cifs, isTargetOfSameSource) => {
        const username = "username";
        const password = "password";
        const domain = "domain";
        sync.appendItemToLocalStorage("cifs", cifs, {domain, username, password}, isTargetOfSameSource);
    };

    const fillAzureCredentials = (storageAccountName) => {
        const storageAccountConnectionString = "storageAccountConnectionString";
        sync.appendItemToLocalStorage("azure", storageAccountName, {storageAccountName, connectionString: storageAccountConnectionString});
    };

    const fillSettings = (settings) => {
        settingsUtils.saveSettingsToSessionStorage(settings);
    };

    test("should build object for nfs to nfs relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: undefined}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/data2"
                }
            }, _ui: {}
        })).toEqual({
                settings: settingsUtils.SETTINGS_DEFAULTS,
                dataBrokerId: 1,
                source: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.NFS}, protocol: "nfs"},
                sourceCredentials: undefined,
                target: {nfs: {export: "/data2", host: "2.2.2.2", path: "", version: DEFAULT_NFS_VERSION.NFS}, protocol: "nfs"},
                targetCredentials: undefined
            }
        );
    });

    test("should build object for nfs to nfsCvs relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: "cvs"}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/data2"
                }
            }, _ui: {}
        })).toEqual({
                settings: settingsUtils.SETTINGS_DEFAULTS,
                dataBrokerId: 1,
                source: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.NFS}, protocol: "nfs"},
                sourceCredentials: undefined,
                target: {nfs: {export: "/data2", host: "2.2.2.2", path: "", version: DEFAULT_NFS_VERSION.CVS, "provider": "cvs"}, protocol: "nfs"},
                targetCredentials: undefined
            }
        );
    });
    test("should build object for nfs to anf relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: "anf"}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/data2"
                }
            }, _ui: {}
        })).toEqual({
                settings: settingsUtils.SETTINGS_DEFAULTS,
                dataBrokerId: 1,
                source: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.NFS}, protocol: "nfs"},
                sourceCredentials: undefined,

                target: {nfs: {export: "/data2", host: "2.2.2.2", path: "", version: DEFAULT_NFS_VERSION.ANF, "provider": "anf"}, protocol: "nfs"},
                targetCredentials: undefined
            }
        );
    });
    test("should build object for cvs to nfs relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: "cvs"}, target: {protocol: "nfs", provider: undefined}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/data2"
                }
            }, _ui: {}
        })).toEqual({
                settings: settingsUtils.SETTINGS_DEFAULTS,
                dataBrokerId: 1,
                source: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.CVS, "provider": "cvs"}, protocol: "nfs"},
                sourceCredentials: undefined,
                target: {nfs: {export: "/data2", host: "2.2.2.2", path: "", version: DEFAULT_NFS_VERSION.NFS}, protocol: "nfs"},
                targetCredentials: undefined
            }
        );
    });
    test("should build object for anf to nfs relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: "anf"}, target: {protocol: "nfs", provider: undefined}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/data2"
                }
            }, _ui: {}
        })).toEqual({
                settings: settingsUtils.SETTINGS_DEFAULTS,
                dataBrokerId: 1,
                source: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.ANF, "provider": "anf"}, protocol: "nfs"},
                sourceCredentials: undefined,
                target: {nfs: {export: "/data2", host: "2.2.2.2", path: "", version: DEFAULT_NFS_VERSION.NFS}, protocol: "nfs"},
                targetCredentials: undefined
            }
        );
    });

    test("should build object for nfs to efs relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: "efs"}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1/folder1",
                    selectedExportTarget: "/folder2",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                    selectedRootExport: "/data1",
                    selectedRootExportTarget: "/"
                }
            }, _ui: {}
        })).toEqual({
                "settings": settingsUtils.SETTINGS_DEFAULTS,
                "dataBrokerId": 1,
                "source": {"nfs": {"export": "/data1", "host": "1.1.1.1", "path": "folder1", version: DEFAULT_NFS_VERSION.NFS}, "protocol": "nfs"},
                "sourceCredentials": undefined,
                "target": {"nfs": {"export": "/", "host": "2.2.2.2", "path": "folder2", "provider": "efs", "version": DEFAULT_NFS_VERSION.EFS}, "protocol": "nfs"},
                "targetCredentials": undefined
            }
        );
    });
    test("should build object for nfs to s3 relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "s3", provider: undefined}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    relationshipTags: "a=b",
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExport: "/data1",
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/data1",
                    selectedEncryptionType: "none",
                    selectedStorageClass: "STANDARD"
                }
            }, _ui: {}
        })).toEqual({
                "settings": settingsUtils.SETTINGS_DEFAULTS,
                "dataBrokerId": 1,
                "source": {"nfs": {"export": "/data1", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.NFS}, "protocol": "nfs"},
                "sourceCredentials": undefined,
                "target": {"protocol": "s3", "s3": {"bucket": "chandler", "prefix": "centralPark1", "tags": [{"key": "a", "value": "b"}], "storageClass": "STANDARD"}},
                "targetCredentials": undefined
            }
        );
    });
    test("should build object for nfs to s3Sgws relationship", () => {
        const host = "2.2.2.2";
        fillOSCredentials(host);
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);

        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "s3", provider: "sgws"}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/data1",
                    selectedS3Based: host,
                    selectedS3BasedBucket: {name: "large", path: "large", actualPath: "large"},
                    selectedS3BasedBucketPath: "large/targetBucket"
                }
            }, _ui: {}
        })).toEqual({
                "settings": settingsUtils.SETTINGS_DEFAULTS,
                "dataBrokerId": 1,
                "source": {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.NFS, provider: undefined}, protocol: "nfs"},
                "sourceCredentials": undefined,
                "target": {
                    "protocol": "s3",
                    "s3": {"bucket": "large", "host": "2.2.2.2", "port": "88", "prefix": "targetBucket", "provider": "sgws"}
                },
                "targetCredentials": {"s3": {"accessKey": "accessKey2", "secretKey": "secretKey2"}}
            }
        );
    });
    test("should build object for efs to nfs relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: "efs"}, target: {protocol: "nfs", provider: undefined}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/folder1",
                    selectedExportTarget: "/data1",
                    selectedNFS: "2.2.2.2",
                    selectedNFSTarget: "1.1.1.1",
                    selectedRootExport: "/",
                    selectedRootExportTarget: "/data1"
                }
            }, _ui: {}
        })).toEqual({
                "settings": settingsUtils.SETTINGS_DEFAULTS,
                "dataBrokerId": 1,
                "source": {"nfs": {"export": "/", "host": "2.2.2.2", "path": "folder1", "provider": "efs", "version": DEFAULT_NFS_VERSION.EFS}, "protocol": "nfs"},
                "sourceCredentials": undefined,
                "target": {"nfs": {"export": "/data1", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.NFS}, "protocol": "nfs"},
                "targetCredentials": undefined
            }
        );
    });
    test("should build object for cifs to cifs relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        fillCIFSCredentials("1.1.1.1");
        fillCIFSCredentials("2.2.2.2", true);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "cifs", provider: undefined}, target: {protocol: "cifs", provider: undefined}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedCIFS: "1.1.1.1",
                    selectedCIFSTarget: "2.2.2.2",
                    selectedRootShare: "/data1",
                    selectedRootShareTarget: "/data2",
                    selectedShare: "/data1/folder1",
                    selectedShareTarget: "/data2/folder1/targetFolder1"
                }
            }, _ui: {}
        })).toEqual({
                "settings": settingsUtils.SETTINGS_DEFAULTS,
                "dataBrokerId": 1,
                "source": {"cifs": {"host": "1.1.1.1", "path": "folder1", "share": "/data1", version: "2.1"}, "protocol": "cifs"},
                "sourceCredentials": {"cifs": {username: "username", password: "password", domain: "domain"}},
                "target": {
                    "cifs": {"host": "2.2.2.2", "path": "folder1/targetFolder1", "share": "/data2", version: "2.1"}, "protocol": "cifs"
                },
                "targetCredentials": {"cifs": {username: "username", password: "password", domain: "domain"}}
            }
        );
    });
    test("should build object for cifs to s3 relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        fillCIFSCredentials("1.1.1.1");
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "cifs", provider: undefined}, target: {protocol: "s3", provider: undefined}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedBucketPath: "large/targetBucket",
                    selectedCIFS: "1.1.1.1",
                    selectedRootShare: "/data1",
                    selectedShare: "/data1",
                    selectedEncryptionType: "aes256"
                }
            }, _ui: {}
        })).toEqual({
                "settings": settingsUtils.SETTINGS_DEFAULTS,
                "dataBrokerId": 1,
                "source": {"cifs": {"host": "1.1.1.1", "path": "", "share": "/data1", version: "2.1"}, "protocol": "cifs"},
                "sourceCredentials": {"cifs": {domain: "domain", username: "username", password: "password"}},
                "target": {"protocol": "s3", "s3": {"bucket": "large", "prefix": "targetBucket", encryption: {protocol: "aes256"}}},
                "targetCredentials": undefined
            }
        );
    });
    test("should build object for cifs to s3Sgws relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        const s3host = "2.2.2.2";
        fillOSCredentials(s3host);
        const cifs = "1.1.1.1";
        fillCIFSCredentials(cifs);

        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "cifs", provider: undefined}, target: {protocol: "s3", provider: "sgws"}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedCIFS: cifs,
                    selectedRootShare: "/data1",
                    selectedS3Based: s3host,
                    selectedS3BasedBucket: {name: "large", path: "large", actualPath: "large"},
                    selectedS3BasedBucketPath: "large",
                    selectedShare: "/data1/folder1"
                }
            }, _ui: {}
        })).toEqual({
                "settings": settingsUtils.SETTINGS_DEFAULTS,
                "dataBrokerId": 1,
                "source": {"cifs": {"host": "1.1.1.1", "path": "folder1", "share": "/data1", version: "2.1"}, "protocol": "cifs"},
                "sourceCredentials": {"cifs": {domain: "domain", username: "username", password: "password"}},
                "target": {"protocol": "s3", "s3": {"bucket": "large", "host": "2.2.2.2", "port": "88", "prefix": "", "provider": "sgws"}},
                "targetCredentials": {"s3": {"accessKey": "accessKey2", "secretKey": "secretKey2"}}
            }
        );
    });
    test("should build object for s3 to nfs relationship", () => {
        const settings = Object.assign({}, settingsUtils.SETTINGS_DEFAULTS, {deleteOnTarget: true});
        fillSettings(settings);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "s3", provider: undefined}, target: {protocol: "nfs", provider: undefined}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExport: "/complex/junction/path",
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/complex/junction/path",
                    selectedEncryptionType: "none",
                    selectedStorageClass: "STANDARD"
                }
            }, _ui: {}
        })).toEqual({
                "dataBrokerId": 1,
                "source": {"protocol": "s3", "s3": {"bucket": "chandler", "prefix": "centralPark1", "storageClass": "STANDARD"}},
                "sourceCredentials": undefined,
                "target": {"nfs": {"export": "/complex/junction/path", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.NFS}, "protocol": "nfs"},
                "targetCredentials": undefined,
                "settings": settings
            }
        );
    });

    test("should build object for s3 to cvs relationship", () => {
        const settings = Object.assign({}, settingsUtils.SETTINGS_DEFAULTS, {deleteOnTarget: true});
        fillSettings(settings);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "s3", provider: undefined}, target: {protocol: "nfs", provider: "cvs"}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExport: "/complex/junction/path",
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/complex/junction/path"
                }
            }, _ui: {}
        })).toEqual({
                "dataBrokerId": 1,
                "source": {"protocol": "s3", "s3": {"bucket": "chandler", "prefix": "centralPark1"}},
                "sourceCredentials": undefined,
                "target": {"nfs": {"export": "/complex/junction/path", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.CVS, "provider": "cvs"}, "protocol": "nfs"},
                "targetCredentials": undefined,
                "settings": settings
            }
        );
    });
    test("should build object for s3 to anf relationship", () => {
        const settings = Object.assign({}, settingsUtils.SETTINGS_DEFAULTS, {deleteOnTarget: true});
        fillSettings(settings);
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "s3", provider: undefined}, target: {protocol: "nfs", provider: "anf"}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExport: "/complex/junction/path",
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/complex/junction/path"
                }
            }, _ui: {}
        })).toEqual({
                "dataBrokerId": 1,
                "source": {"protocol": "s3", "s3": {"bucket": "chandler", "prefix": "centralPark1"}},
                "sourceCredentials": undefined,
                "target": {"nfs": {"export": "/complex/junction/path", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.ANF, "provider": "anf"}, "protocol": "nfs"},
                "targetCredentials": undefined,
                "settings": settings
            }
        );
    });

    test("should build object for s3 to cifs relationship", () => {
        const settings = Object.assign({}, settingsUtils.SETTINGS_DEFAULTS, {deleteOnTarget: true});
        fillSettings(settings);
        const cifs = "1.1.1.1";
        fillCIFSCredentials(cifs);

        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "s3", provider: undefined}, target: {protocol: "cifs", provider: undefined}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedBucketPath: "chandler",
                    selectedCIFS: cifs,
                    selectedRootShare: "/data1",
                    selectedShare: "/data1"
                }
            }, _ui: {}
        })).toEqual({
                "dataBrokerId": 1,
                "source": {"protocol": "s3", "s3": {"bucket": "chandler", "prefix": ""}},
                "sourceCredentials": undefined,
                "target": {"cifs": {"host": "1.1.1.1", "path": "", "share": "/data1", version: "2.1"}, "protocol": "cifs"},
                "targetCredentials": {"cifs": {domain: "domain", username: "username", password: "password"}},
                "settings": settings,
            }
        );
    });
    test("should build object for azure to nfs relationship", () => {
        const settings = Object.assign({}, settingsUtils.SETTINGS_DEFAULTS, {deleteOnTarget: true});
        fillSettings(settings);
        fillAzureCredentials("storageAccountName");
        expect(sync.buildSyncObject({
            _wizard: {
                wizardInfo: {source: {protocol: "azure", provider: undefined}, target: {protocol: "nfs", provider: undefined}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedExport: "/data1",
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/data1",
                    selectedStorageAccountName: "storageAccountName",
                    selectedAzureBlobContainerPath: "container/test",
                    selectedBlobTier: "COOL"
                }
            }, _ui: {}
        })).toEqual({
                dataBrokerId: 1,
                source: {
                    protocol: "azure",
                    azure: {storageAccountName: "storageAccountName", container: "container", prefix: "test","tags": [], blobTier: "COOL"}
                },
                sourceCredentials: {azure: {storageAccountConnectionString: "storageAccountConnectionString"}},
                target: {nfs: {export: "/data1", host: "1.1.1.1", path: "", version: DEFAULT_NFS_VERSION.NFS}, protocol: "nfs"},
                settings: settings,
            }
        );
    });
    test("should build object for secure nfs to nfs relationship", () => {
        fillSettings(settingsUtils.SETTINGS_DEFAULTS);
        expect(sync.buildSyncObject({
            _wizard: {
                isSecure: true,
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: undefined}},
                selectedWizardParams: {
                    selectedDataBroker: {id: 1},
                    selectedDataBrokerTarget: {id: 2},
                    selectedExport: "/folder1",
                    selectedExportTarget: "/data1",
                    selectedNFS: "2.2.2.2",
                    selectedNFSTarget: "1.1.1.1",
                    selectedRootExport: "/",
                    selectedRootExportTarget: "/data1",
                    selectedIsTargetDataBrokerInitiator: "false"
                }
            }, _ui: {}
        })).toEqual({
                "settings": settingsUtils.SETTINGS_DEFAULTS,
                "dataBrokerId": 1,
                "source": {"nfs": {"export": "/", "host": "2.2.2.2", "path": "folder1", "provider": undefined, "version": DEFAULT_NFS_VERSION.NFS}, "protocol": "nfs"},
                "sourceCredentials": undefined,
                "target": {"nfs": {"export": "/data1", "host": "1.1.1.1", "path": "", version: DEFAULT_NFS_VERSION.NFS, "provider": undefined}, "protocol": "nfs"},
                "targetCredentials": undefined,
                "encryption": {initiator: {dataBrokerId: 1, endpointType: "src"}, listener: {dataBrokerId: 2, endpointType: "trg"}}
            }
        );
    });

});

describe("getSourceAndTargetInfo", () => {
    const fillOSCredentials = (host) => {
        sync.appendItemToLocalStorage("objectStorage", host, {port: "88"});
    };

    test("should build object for nfs to nfs relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: undefined}},
                selectedWizardParams: {
                    selectedExport: "/data1",
                    selectedExportTarget: "/data2",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2"
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data1",
                    "title": "NFS Server",
                },
                "targetInfo": {
                    "description": "2.2.2.2",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data2",
                    "title": "NFS Server",
                }
            }
        );
    });
    test("should build object for nfs to efs relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "nfs", provider: "efs"}},
                selectedWizardParams: {
                    selectedExport: "/data1/folder1",
                    selectedExportTarget: "/folder2",
                    selectedNFS: "1.1.1.1",
                    selectedNFSTarget: "2.2.2.2",
                }
            }, _ui: {}
        })).toEqual(
            {
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data1/folder1",
                    "title": "NFS Server"
                },
                "targetInfo": {
                    "description": "2.2.2.2",
                    "subtitle": "Directory",
                    "subtitleDescription": "/folder2",
                    "title": "AWS EFS"
                }
            }
        );
    });
    test("should build object for nfs to s3 relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "s3", provider: undefined}},
                selectedWizardParams: {
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExport: "/data1",
                    selectedNFS: "1.1.1.1",
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data1",
                    "title": "NFS Server"
                },
                "targetInfo": {
                    "description": "chandler",
                    "subtitle": "Folder",
                    "subtitleDescription": "centralPark1",
                    "title": "AWS S3 Bucket"
                }
            }
        );
    });
    test("should build object for nfs to s3Sgws relationship", () => {
        const host = "2.2.2.2";
        fillOSCredentials(host);
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "s3", provider: "sgws"}},
                selectedWizardParams: {
                    selectedExport: "/data1",
                    selectedNFS: "1.1.1.1",
                    selectedRootExport: "/data1",
                    selectedS3Based: host,
                    selectedS3BasedBucket: {name: "large", path: "large", actualPath: "large"},
                    selectedS3BasedBucketPath: "large/targetBucket"
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data1",
                    "title": "NFS Server"
                },
                "targetInfo": {
                    "description": "2.2.2.2:88/large",
                    "subtitle": "Folder",
                    "subtitleDescription": "targetBucket",
                    "title": "StorageGRID Bucket"
                }
            }
        );
    });
    test("should build object for efs to nfs relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: "efs"}, target: {protocol: "nfs", provider: undefined}},
                selectedWizardParams: {
                    selectedExport: "/folder1",
                    selectedExportTarget: "/data1",
                    selectedNFS: "2.2.2.2",
                    selectedNFSTarget: "1.1.1.1"
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "2.2.2.2",
                    "subtitle": "Directory",
                    "subtitleDescription": "/folder1",
                    "title": "AWS EFS"
                },
                "targetInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/data1",
                    "title": "NFS Server"
                }
            }
        );
    });
    test("should build object for cifs to cifs relationship", () => {

        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "cifs", provider: undefined}, target: {protocol: "cifs", provider: undefined}},
                selectedWizardParams: {
                    selectedCIFS: "1.1.1.1",
                    selectedCIFSTarget: "2.2.2.2",
                    selectedShare: "/data1/folder1",
                    selectedShareTarget: "/data2/folder1/targetFolder1"
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Share",
                    "subtitleDescription": "/data1/folder1",
                    "title": "SMB Server"
                },
                "targetInfo": {
                    "description": "2.2.2.2",
                    "subtitle": "Share",
                    "subtitleDescription": "/data2/folder1/targetFolder1",
                    "title": "SMB Server"
                }
            }
        );
    });
    test("should build object for cifs to s3 relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "cifs", provider: undefined}, target: {protocol: "s3", provider: undefined}},
                selectedWizardParams: {
                    selectedBucketPath: "large/targetBucket",
                    selectedCIFS: "1.1.1.1",
                    selectedShare: "/data1"
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Share",
                    "subtitleDescription": "/data1",
                    "title": "SMB Server"
                },
                "targetInfo": {
                    "description": "large",
                    "subtitle": "Folder",
                    "subtitleDescription": "targetBucket",
                    "title": "AWS S3 Bucket"
                }
            }
        );
    });
    test("should build object for cifs to s3Sgws relationship", () => {
        const host = "2.2.2.2";
        fillOSCredentials(host);
        const cifs = "1.1.1.1";

        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "cifs", provider: undefined}, target: {protocol: "s3", provider: "sgws"}},
                selectedWizardParams: {
                    selectedCIFS: cifs,
                    selectedRootShare: "/data1",
                    selectedS3Based: host,
                    selectedS3BasedBucket: {name: "large", path: "large", actualPath: "large"},
                    selectedS3BasedBucketPath: "large",
                    selectedShare: "/data1/folder1"
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Share",
                    "subtitleDescription": "/data1/folder1",
                    "title": "SMB Server"
                },
                "targetInfo": {
                    "description": "2.2.2.2:88/large",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "StorageGRID Bucket"
                }
            }
        );
    });
    test("should build object for s3 to nfs relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "s3", provider: undefined}, target: {protocol: "nfs", provider: undefined}},
                selectedWizardParams: {
                    selectedBucketPath: "chandler/centralPark1",
                    selectedExportTarget: "/complex/junction/path",
                    selectedNFSTarget: "1.1.1.1",
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "chandler",
                    "subtitle": "Folder",
                    "subtitleDescription": "centralPark1",
                    "title": "AWS S3 Bucket"
                },
                "targetInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Directory",
                    "subtitleDescription": "/complex/junction/path",
                    "title": "NFS Server"
                }
            }
        );
    });
    test("should build object for s3 to s3Sgws relationship", () => {
        const host = "123.123.123.123";
        fillOSCredentials(host);
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "s3", provider: undefined}, target: {protocol: "s3", provider: "sgws"}},
                selectedWizardParams: {
                    selectedS3Based: host,
                    selectedS3BasedBucketPath: "small",
                    selectedBucketPath: "tiny"
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "tiny",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "AWS S3 Bucket"
                },
                "targetInfo": {
                    "description": "123.123.123.123:88/small",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "StorageGRID Bucket"
                }
            }
        );
    });
    test("should build object for s3Sgws to s3 relationship", () => {
        const host = "123.123.123.123";
        fillOSCredentials(host);
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "s3", provider: "sgws"}, target: {protocol: "s3", provider: undefined}},
                selectedWizardParams: {
                    selectedS3Based: host,
                    selectedS3BasedBucketPath: "small",
                    selectedBucketPath: "tiny",
                    selectedStorageClass: "STANDARD",
                    relationshipTags: "noTags"
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "123.123.123.123:88/small",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "StorageGRID Bucket"
                },
                "targetInfo": {
                    "description": "tiny",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "AWS S3 Bucket"
                }
            }
        );
    });
    test("should build object for cifs to gcp relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "cifs", provider: undefined}, target: {protocol: "gcp", provider: undefined}},
                selectedWizardParams: {
                    selectedCIFS: "123.123.123.123",
                    selectedShare: "/data2",
                    cifsVersion: "2.1",
                    selectedRootShare: "/data2",
                    selectedGCPBucketPath: "large"
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "123.123.123.123",
                    "subtitle": "Share",
                    "subtitleDescription": "/data2",
                    "title": "SMB Server"
                },
                "targetInfo": {
                    "description": "large",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "Google Cloud Storage Bucket"
                }
            }
        );
    });
    test("should build object for gcp to cifs relationship", () => {
        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "gcp", provider: undefined}, target: {protocol: "cifs", provider: undefined}},
                selectedWizardParams: {
                    selectedCIFS: "123.123.123.123",
                    selectedShare: "/data2",
                    cifsVersion: "2.1",
                    selectedRootShare: "/data2",
                    selectedGCPBucketPath: "large"
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "large",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "Google Cloud Storage Bucket"
                },
                "targetInfo": {
                    "description": "123.123.123.123",
                    "subtitle": "Share",
                    "subtitleDescription": "/data2",
                    "title": "SMB Server"
                }
            }
        );
    });
    test("should build object for s3 to cifs relationship", () => {
        const cifs = "1.1.1.1";

        expect(sync.getSourceAndTargetInfo({
            _wizard: {
                wizardInfo: {source: {protocol: "s3", provider: undefined}, target: {protocol: "cifs", provider: undefined}},
                selectedWizardParams: {
                    selectedBucketPath: "chandler",
                    selectedCIFSTarget: cifs,
                    selectedShareTarget: "/data1"
                }
            }, _ui: {}
        })).toEqual({
                "sourceInfo": {
                    "description": "chandler",
                    "subtitle": "Folder",
                    "subtitleDescription": "",
                    "title": "AWS S3 Bucket"
                },
                "targetInfo": {
                    "description": "1.1.1.1",
                    "subtitle": "Share",
                    "subtitleDescription": "/data1",
                    "title": "SMB Server"
                }
            }
        );
    });
});

describe("getWizardSelectedParams", () => {
    test("should build the right selected params for nfs-s3", () => {

        const nfsServer = "1---252E1---252E1---252E1";
        const exportPath = "---252Fdata1---252Ffolder1";
        const rootExport = "---252Fdata1";
        const dataBrokerId = 1;
        const bucketPath = "chandler---252FcentralPark1";
        const relationshipTags = "test---3D1";
        const storageClass = "DEEP_ARCHIVE";
        const encryptionType = "kms";
        const wizardParams = {
            nfsServer,
            exportPath,
            rootExport,
            dataBrokerId,
            bucketPath,
            relationshipTags,
            storageClass,
            encryptionType
        };

        const props = {
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: undefined}, target: {protocol: "s3", provider: undefined}},
            },
            _dataBrokers: {
                succeeded: true,
                data: [{id: dataBrokerId}],
            },
            _showExports: {
                [`${nfsServer}-${dataBrokerId}`]: {
                    succeeded: true,
                    data: []
                }
            },
            _showBuckets: {
                [dataBrokerId]: {
                    succeeded: true,
                    data: [{name: "chandler"}]
                }

            }

        };

        expect(sync.getWizardSelectedParams(props, wizardParams)).toEqual(
            {
                selectedDataBroker: {id: dataBrokerId},
                selectedIsTargetDataBrokerInitiator: "false",
                relationshipTags: "test=1",
                selectedNFS: "1.1.1.1",
                selectedExport: "/data1/folder1",
                selectedRootExport: "/data1",
                selectedBucketPath: "chandler/centralPark1",
                selectedStorageClass: "DEEP_ARCHIVE",
                selectedEncryptionType: "kms"
            }
        );
    });
    test("should build the right selected params for cifs-s3", () => {

        const cifsServer = "1---252E1---252E1---252E1";
        const sharePath = "---252Fdata1---252Ffolder1";
        const rootShare = "---252Fdata1";
        const dataBrokerId = 1;
        const bucketPath = "chandler---252FcentralPark1";
        const wizardParams = {cifsServer, sharePath, rootShare, dataBrokerId, bucketPath};

        const props = {
            _wizard: {
                wizardInfo: {source: {protocol: "cifs", provider: undefined}, target: {protocol: "s3", provider: undefined}},            },
            _dataBrokers: {
                succeeded: true,
                data: [{id: dataBrokerId}],
            },
            _showShares: {
                [`${cifsServer}-${dataBrokerId}`]: {
                    succeeded: true,
                    data: []
                }
            },
            _showBuckets: {
                [dataBrokerId]: {
                    succeeded: true,
                    data: [{name: "chandler"}]
                }
            }

        };

        expect(sync.getWizardSelectedParams(props, wizardParams)).toEqual(
            {
                selectedDataBroker: {id: dataBrokerId},
                selectedIsTargetDataBrokerInitiator: "false",
                selectedCIFS: "1.1.1.1",
                selectedRootShare: "/data1",
                selectedShare: "/data1/folder1",
                selectedBucketPath: "chandler/centralPark1"
            }
        );
    });
    test("should build the right selected params for cifs-s3Sgws", () => {

        const cifsServer = "1---252E1---252E1---252E1";
        const dataBrokerId = "1";
        const wizardParams = {
            cifsServer,
            dataBrokerId,
            rootShare: "---252Fdata1",
            s3BasedBucketPath: "large",
            s3Based: "2---252E2---252E2---252E2",
            shareExport: "---252Fdata1",
            sharePath: "---252Fdata1---252Ffolder1"
        };

        const props = {
            _wizard: {
                wizardInfo: {source: {protocol: "cifs", provider: undefined}, target: {protocol: "s3", provider: "sgws"}},
            },
            _dataBrokers: {
                succeeded: true,
                data: [{id: dataBrokerId}],
            },
            _showShares: {
                [`${cifsServer}-${dataBrokerId}`]: {
                    succeeded: true,
                    data: []
                }
            },
            _showS3BasedBuckets: { '2.2.2.2': {
                succeeded: true,
                data: [{name: "large"}]
            }}

        };

        expect(sync.getWizardSelectedParams(props, wizardParams)).toEqual(
            {
                selectedDataBroker: {id: dataBrokerId},
                selectedIsTargetDataBrokerInitiator: "false",
                selectedCIFS: "1.1.1.1",
                selectedRootShare: "/data1",
                selectedShare: "/data1/folder1",
                selectedS3Based: "2.2.2.2",
                selectedS3BasedBucket: {name: "large"},
                selectedS3BasedBucketPath: "large"
            }
        );
    });
    test("should build the right selected params for efs-nfs", () => {

        const exportPath = "---252Ffolder1---252Ffolder0";
        const exportPathTarget = "---252Fcomplex---252Fjunction---252Fpath---252Ftest";
        const nfsServer = "1---252E1---252E1---252E1";
        const nfsServerTarget = "2---252E2---252E2---252E2";
        const rootExport = "---252F";
        const rootExportTarget = "---252Fcomplex---252Fjunction---252Fpath";
        const dataBrokerId = 1;
        const wizardParams = {
            exportPath,
            exportPathTarget,
            nfsServer,
            nfsServerTarget,
            dataBrokerId,
            rootExport,
            rootExportTarget
        };

        const props = {
            _wizard: {
                wizardInfo: {source: {protocol: "nfs", provider: "efs"}, target: {protocol: "nfs", provider: undefined}},
            },
            _dataBrokers: {
                succeeded: true,
                data: [{id: dataBrokerId}],
            },
            _showExports: {
                [`${nfsServer}-${dataBrokerId}`]: {
                    succeeded: true,
                    data: []
                },
                [`${nfsServerTarget}-${dataBrokerId}`]: {
                    succeeded: true,
                    data: []
                }
            }

        };

        expect(sync.getWizardSelectedParams(props, wizardParams)).toEqual(
            {
                selectedDataBroker: {id: dataBrokerId},
                selectedIsTargetDataBrokerInitiator: "false",
                selectedNFS: "1.1.1.1",
                selectedNFSTarget: "2.2.2.2",
                selectedRootExport: "/",
                selectedExport: "/folder1/folder0",
                selectedExportTarget: "/complex/junction/path/test",
                selectedRootExportTarget: "/complex/junction/path"
            }
        );
    });
});