/**
 * <p>Contains the parameters for accepting the quote.</p>
 */
export interface AcceptReservedInstancesExchangeQuoteRequest {
    __type?: "AcceptReservedInstancesExchangeQuoteRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *       and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *       Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the Convertible Reserved Instances to exchange for another Convertible
     *             Reserved Instance of the same or higher value.</p>
     */
    ReservedInstanceIds: string[] | undefined;
    /**
     * <p>The configuration of the target Convertible Reserved Instance to exchange for your
     *             current Convertible Reserved Instances.</p>
     */
    TargetConfigurations?: TargetConfigurationRequest[];
}
export declare namespace AcceptReservedInstancesExchangeQuoteRequest {
    const filterSensitiveLog: (obj: AcceptReservedInstancesExchangeQuoteRequest) => any;
    const isa: (o: any) => o is AcceptReservedInstancesExchangeQuoteRequest;
}
/**
 * <p>The result of the exchange and whether it was <code>successful</code>.</p>
 */
export interface AcceptReservedInstancesExchangeQuoteResult {
    __type?: "AcceptReservedInstancesExchangeQuoteResult";
    /**
     * <p>The ID of the successful exchange.</p>
     */
    ExchangeId?: string;
}
export declare namespace AcceptReservedInstancesExchangeQuoteResult {
    const filterSensitiveLog: (obj: AcceptReservedInstancesExchangeQuoteResult) => any;
    const isa: (o: any) => o is AcceptReservedInstancesExchangeQuoteResult;
}
export interface AcceptTransitGatewayPeeringAttachmentRequest {
    __type?: "AcceptTransitGatewayPeeringAttachmentRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the transit gateway attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
}
export declare namespace AcceptTransitGatewayPeeringAttachmentRequest {
    const filterSensitiveLog: (obj: AcceptTransitGatewayPeeringAttachmentRequest) => any;
    const isa: (o: any) => o is AcceptTransitGatewayPeeringAttachmentRequest;
}
export interface AcceptTransitGatewayPeeringAttachmentResult {
    __type?: "AcceptTransitGatewayPeeringAttachmentResult";
    /**
     * <p>The transit gateway peering attachment.</p>
     */
    TransitGatewayPeeringAttachment?: TransitGatewayPeeringAttachment;
}
export declare namespace AcceptTransitGatewayPeeringAttachmentResult {
    const filterSensitiveLog: (obj: AcceptTransitGatewayPeeringAttachmentResult) => any;
    const isa: (o: any) => o is AcceptTransitGatewayPeeringAttachmentResult;
}
export interface AcceptTransitGatewayVpcAttachmentRequest {
    __type?: "AcceptTransitGatewayVpcAttachmentRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
}
export declare namespace AcceptTransitGatewayVpcAttachmentRequest {
    const filterSensitiveLog: (obj: AcceptTransitGatewayVpcAttachmentRequest) => any;
    const isa: (o: any) => o is AcceptTransitGatewayVpcAttachmentRequest;
}
export interface AcceptTransitGatewayVpcAttachmentResult {
    __type?: "AcceptTransitGatewayVpcAttachmentResult";
    /**
     * <p>The VPC attachment.</p>
     */
    TransitGatewayVpcAttachment?: TransitGatewayVpcAttachment;
}
export declare namespace AcceptTransitGatewayVpcAttachmentResult {
    const filterSensitiveLog: (obj: AcceptTransitGatewayVpcAttachmentResult) => any;
    const isa: (o: any) => o is AcceptTransitGatewayVpcAttachmentResult;
}
export interface AcceptVpcEndpointConnectionsRequest {
    __type?: "AcceptVpcEndpointConnectionsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC endpoint service.</p>
     */
    ServiceId: string | undefined;
    /**
     * <p>The IDs of one or more interface VPC endpoints.</p>
     */
    VpcEndpointIds: string[] | undefined;
}
export declare namespace AcceptVpcEndpointConnectionsRequest {
    const filterSensitiveLog: (obj: AcceptVpcEndpointConnectionsRequest) => any;
    const isa: (o: any) => o is AcceptVpcEndpointConnectionsRequest;
}
export interface AcceptVpcEndpointConnectionsResult {
    __type?: "AcceptVpcEndpointConnectionsResult";
    /**
     * <p>Information about the interface endpoints that were not accepted, if
     *             applicable.</p>
     */
    Unsuccessful?: UnsuccessfulItem[];
}
export declare namespace AcceptVpcEndpointConnectionsResult {
    const filterSensitiveLog: (obj: AcceptVpcEndpointConnectionsResult) => any;
    const isa: (o: any) => o is AcceptVpcEndpointConnectionsResult;
}
export interface AcceptVpcPeeringConnectionRequest {
    __type?: "AcceptVpcPeeringConnectionRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC peering connection. You must specify this parameter in the
     * 			request.</p>
     */
    VpcPeeringConnectionId?: string;
}
export declare namespace AcceptVpcPeeringConnectionRequest {
    const filterSensitiveLog: (obj: AcceptVpcPeeringConnectionRequest) => any;
    const isa: (o: any) => o is AcceptVpcPeeringConnectionRequest;
}
export interface AcceptVpcPeeringConnectionResult {
    __type?: "AcceptVpcPeeringConnectionResult";
    /**
     * <p>Information about the VPC peering connection.</p>
     */
    VpcPeeringConnection?: VpcPeeringConnection;
}
export declare namespace AcceptVpcPeeringConnectionResult {
    const filterSensitiveLog: (obj: AcceptVpcPeeringConnectionResult) => any;
    const isa: (o: any) => o is AcceptVpcPeeringConnectionResult;
}
/**
 * <p>Describes an account attribute.</p>
 */
export interface AccountAttribute {
    __type?: "AccountAttribute";
    /**
     * <p>The name of the account attribute.</p>
     */
    AttributeName?: string;
    /**
     * <p>The values for the account attribute.</p>
     */
    AttributeValues?: AccountAttributeValue[];
}
export declare namespace AccountAttribute {
    const filterSensitiveLog: (obj: AccountAttribute) => any;
    const isa: (o: any) => o is AccountAttribute;
}
export declare type AccountAttributeName = "default-vpc" | "supported-platforms";
/**
 * <p>Describes a value of an account attribute.</p>
 */
export interface AccountAttributeValue {
    __type?: "AccountAttributeValue";
    /**
     * <p>The value of the attribute.</p>
     */
    AttributeValue?: string;
}
export declare namespace AccountAttributeValue {
    const filterSensitiveLog: (obj: AccountAttributeValue) => any;
    const isa: (o: any) => o is AccountAttributeValue;
}
/**
 * <p>Describes a running instance in a Spot Fleet.</p>
 */
export interface ActiveInstance {
    __type?: "ActiveInstance";
    /**
     * <p>The health status of the instance. If the status of either the instance status check
     *           or the system status check is <code>impaired</code>, the health status of the instance
     *           is <code>unhealthy</code>. Otherwise, the health status is <code>healthy</code>.</p>
     */
    InstanceHealth?: InstanceHealthStatus | string;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: string;
    /**
     * <p>The ID of the Spot Instance request.</p>
     */
    SpotInstanceRequestId?: string;
}
export declare namespace ActiveInstance {
    const filterSensitiveLog: (obj: ActiveInstance) => any;
    const isa: (o: any) => o is ActiveInstance;
}
export declare enum ActivityStatus {
    ERROR = "error",
    FULFILLED = "fulfilled",
    PENDING_FULFILLMENT = "pending_fulfillment",
    PENDING_TERMINATION = "pending_termination"
}
/**
 * <p>Describes an Elastic IP address.</p>
 */
export interface Address {
    __type?: "Address";
    /**
     * <p>The ID representing the allocation of the address for use with EC2-VPC.</p>
     */
    AllocationId?: string;
    /**
     * <p>The ID representing the association of the address with an instance in a VPC.</p>
     */
    AssociationId?: string;
    /**
     * <p>The customer-owned IP address.</p>
     */
    CustomerOwnedIp?: string;
    /**
     * <p>The ID of the customer-owned address pool.</p>
     */
    CustomerOwnedIpv4Pool?: string;
    /**
     * <p>Indicates whether this Elastic IP address is for use with instances
     * 				in EC2-Classic (<code>standard</code>) or instances in a VPC (<code>vpc</code>).</p>
     */
    Domain?: DomainType | string;
    /**
     * <p>The ID of the instance that the address is associated with (if any).</p>
     */
    InstanceId?: string;
    /**
     * <p>The name of the location from which the IP address is advertised.</p>
     */
    NetworkBorderGroup?: string;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The ID of the AWS account that owns the network interface.</p>
     */
    NetworkInterfaceOwnerId?: string;
    /**
     * <p>The private IP address associated with the Elastic IP address.</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>The Elastic IP address.</p>
     */
    PublicIp?: string;
    /**
     * <p>The ID of an address pool.</p>
     */
    PublicIpv4Pool?: string;
    /**
     * <p>Any tags assigned to the Elastic IP address.</p>
     */
    Tags?: Tag[];
}
export declare namespace Address {
    const filterSensitiveLog: (obj: Address) => any;
    const isa: (o: any) => o is Address;
}
export interface AdvertiseByoipCidrRequest {
    __type?: "AdvertiseByoipCidrRequest";
    /**
     * <p>The address range, in CIDR notation. This must be the exact range that you provisioned.
     *          You can't advertise only a portion of the provisioned range.</p>
     */
    Cidr: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace AdvertiseByoipCidrRequest {
    const filterSensitiveLog: (obj: AdvertiseByoipCidrRequest) => any;
    const isa: (o: any) => o is AdvertiseByoipCidrRequest;
}
export interface AdvertiseByoipCidrResult {
    __type?: "AdvertiseByoipCidrResult";
    /**
     * <p>Information about the address range.</p>
     */
    ByoipCidr?: ByoipCidr;
}
export declare namespace AdvertiseByoipCidrResult {
    const filterSensitiveLog: (obj: AdvertiseByoipCidrResult) => any;
    const isa: (o: any) => o is AdvertiseByoipCidrResult;
}
export declare type Affinity = "default" | "host";
export interface AllocateAddressRequest {
    __type?: "AllocateAddressRequest";
    /**
     * <p>[EC2-VPC] The Elastic IP address to recover or an IPv4 address from an address pool.</p>
     */
    Address?: string;
    /**
     * <p>The ID of a customer-owned address pool. Use this parameter to let Amazon EC2
     *         select an address from the address pool. Alternatively, specify a specific
     *         address from the address pool.</p>
     */
    CustomerOwnedIpv4Pool?: string;
    /**
     * <p>Set to <code>vpc</code> to allocate the address for use with instances in a VPC.</p>
     *          <p>Default: The address is for use with instances in EC2-Classic.</p>
     */
    Domain?: DomainType | string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The location from which the IP address is advertised. Use this parameter to limit the address to this location.</p>
     *          <p>Use <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeVpcs.html">DescribeVpcs</a> to view the network border groups.</p>
     *
     *          <note>
     *             <p>You cannot use a network border group with EC2 Classic. If you attempt this operation on EC2 classic, you will receive an <code>InvalidParameterCombination</code> error. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/errors-overview.html">Error Codes</a>.</p>
     *          </note>
     */
    NetworkBorderGroup?: string;
    /**
     * <p>The ID of an address pool that you own. Use this parameter to let Amazon EC2 select an address from the address pool.
     *        To specify a specific address from the address pool, use the <code>Address</code> parameter instead.</p>
     */
    PublicIpv4Pool?: string;
}
export declare namespace AllocateAddressRequest {
    const filterSensitiveLog: (obj: AllocateAddressRequest) => any;
    const isa: (o: any) => o is AllocateAddressRequest;
}
export interface AllocateAddressResult {
    __type?: "AllocateAddressResult";
    /**
     * <p>[EC2-VPC] The ID that AWS assigns to represent the allocation of the Elastic IP address for use with instances in a VPC.</p>
     */
    AllocationId?: string;
    /**
     * <p>The customer-owned IP address.</p>
     */
    CustomerOwnedIp?: string;
    /**
     * <p>The ID of the customer-owned address pool.</p>
     */
    CustomerOwnedIpv4Pool?: string;
    /**
     * <p>Indicates whether this Elastic IP address is for use with instances
     * 				in EC2-Classic (<code>standard</code>) or instances in a VPC (<code>vpc</code>).</p>
     */
    Domain?: DomainType | string;
    /**
     * <p>The location from which the IP address is advertised.</p>
     */
    NetworkBorderGroup?: string;
    /**
     * <p>The Elastic IP address.</p>
     */
    PublicIp?: string;
    /**
     * <p>The ID of an address pool.</p>
     */
    PublicIpv4Pool?: string;
}
export declare namespace AllocateAddressResult {
    const filterSensitiveLog: (obj: AllocateAddressResult) => any;
    const isa: (o: any) => o is AllocateAddressResult;
}
export interface AllocateHostsRequest {
    __type?: "AllocateHostsRequest";
    /**
     * <p>Indicates whether the host accepts any untargeted instance launches that
     *     		match its instance type configuration, or if it only accepts Host tenancy
     *     		instance launches that specify its unique host ID. For more information,
     *     		see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/how-dedicated-hosts-work.html#dedicated-hosts-understanding">
     *             	Understanding Instance Placement and Host Affinity</a> in the
     *         	<i>Amazon EC2 User Guide for Linux Instances</i>.</p>
     *
     *         <p>Default: <code>on</code>
     *          </p>
     */
    AutoPlacement?: AutoPlacement | string;
    /**
     * <p>The Availability Zone in which to allocate the Dedicated Host.</p>
     */
    AvailabilityZone: string | undefined;
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Indicates whether to enable or disable host recovery for the Dedicated Host.
     * 			Host recovery is disabled by default. For more information, see
     * 			<a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-recovery.html">
     * 				Host Recovery</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     * 		       <p>Default: <code>off</code>
     *          </p>
     */
    HostRecovery?: HostRecovery | string;
    /**
     * <p>Specifies the instance family to be supported by the Dedicated Hosts. If you specify
     * 			an instance family, the Dedicated Hosts support multiple instance types within that
     * 			instance family.</p>
     *
     * 		       <p>If you want the Dedicated Hosts to support a specific instance type only, omit this
     * 			parameter and specify <b>InstanceType</b>
     * 			instead. You cannot specify <b>InstanceFamily</b> and
     * 			<b>InstanceType</b> in the same request.</p>
     */
    InstanceFamily?: string;
    /**
     * <p>Specifies the instance type to be supported by the Dedicated Hosts. If you
     * 		specify an instance type, the Dedicated Hosts support instances of the
     * 		specified instance type only.</p>
     *
     *     	    <p>If you want the Dedicated Hosts to support multiple instance types in a specific
     *     		instance family, omit this parameter and specify <b>InstanceFamily</b>
     *     		instead. You cannot specify <b>InstanceType</b> and
     *     		<b>InstanceFamily</b> in the same request.</p>
     */
    InstanceType?: string;
    /**
     * <p>The number of Dedicated Hosts to allocate to your account with these parameters.</p>
     */
    Quantity: number | undefined;
    /**
     * <p>The tags to apply to the Dedicated Host during creation.</p>
     */
    TagSpecifications?: TagSpecification[];
}
export declare namespace AllocateHostsRequest {
    const filterSensitiveLog: (obj: AllocateHostsRequest) => any;
    const isa: (o: any) => o is AllocateHostsRequest;
}
/**
 * <p>Contains the output of AllocateHosts.</p>
 */
export interface AllocateHostsResult {
    __type?: "AllocateHostsResult";
    /**
     * <p>The ID of the allocated Dedicated Host. This is used to launch an instance onto a
     * 			specific host.</p>
     */
    HostIds?: string[];
}
export declare namespace AllocateHostsResult {
    const filterSensitiveLog: (obj: AllocateHostsResult) => any;
    const isa: (o: any) => o is AllocateHostsResult;
}
export declare type AllocationState = "available" | "pending" | "permanent-failure" | "released" | "released-permanent-failure" | "under-assessment";
export declare enum AllocationStrategy {
    CAPACITY_OPTIMIZED = "capacityOptimized",
    DIVERSIFIED = "diversified",
    LOWEST_PRICE = "lowestPrice"
}
/**
 * <p>Describes a principal.</p>
 */
export interface AllowedPrincipal {
    __type?: "AllowedPrincipal";
    /**
     * <p>The Amazon Resource Name (ARN) of the principal.</p>
     */
    Principal?: string;
    /**
     * <p>The type of principal.</p>
     */
    PrincipalType?: PrincipalType | string;
}
export declare namespace AllowedPrincipal {
    const filterSensitiveLog: (obj: AllowedPrincipal) => any;
    const isa: (o: any) => o is AllowedPrincipal;
}
export declare type AllowsMultipleInstanceTypes = "off" | "on";
export interface ApplySecurityGroupsToClientVpnTargetNetworkRequest {
    __type?: "ApplySecurityGroupsToClientVpnTargetNetworkRequest";
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the security groups to apply to the associated target network. Up to 5 security groups can
     * 			be applied to an associated target network.</p>
     */
    SecurityGroupIds: string[] | undefined;
    /**
     * <p>The ID of the VPC in which the associated target network is located.</p>
     */
    VpcId: string | undefined;
}
export declare namespace ApplySecurityGroupsToClientVpnTargetNetworkRequest {
    const filterSensitiveLog: (obj: ApplySecurityGroupsToClientVpnTargetNetworkRequest) => any;
    const isa: (o: any) => o is ApplySecurityGroupsToClientVpnTargetNetworkRequest;
}
export interface ApplySecurityGroupsToClientVpnTargetNetworkResult {
    __type?: "ApplySecurityGroupsToClientVpnTargetNetworkResult";
    /**
     * <p>The IDs of the applied security groups.</p>
     */
    SecurityGroupIds?: string[];
}
export declare namespace ApplySecurityGroupsToClientVpnTargetNetworkResult {
    const filterSensitiveLog: (obj: ApplySecurityGroupsToClientVpnTargetNetworkResult) => any;
    const isa: (o: any) => o is ApplySecurityGroupsToClientVpnTargetNetworkResult;
}
export declare type ArchitectureType = "arm64" | "i386" | "x86_64";
export declare type ArchitectureValues = "arm64" | "i386" | "x86_64";
/**
 * <p>Describes the private IP addresses assigned to a network interface.</p>
 */
export interface AssignedPrivateIpAddress {
    __type?: "AssignedPrivateIpAddress";
    /**
     * <p>The private IP address assigned to the network interface.</p>
     */
    PrivateIpAddress?: string;
}
export declare namespace AssignedPrivateIpAddress {
    const filterSensitiveLog: (obj: AssignedPrivateIpAddress) => any;
    const isa: (o: any) => o is AssignedPrivateIpAddress;
}
export interface AssignIpv6AddressesRequest {
    __type?: "AssignIpv6AddressesRequest";
    /**
     * <p>The number of IPv6 addresses to assign to the network interface. Amazon EC2
     *             automatically selects the IPv6 addresses from the subnet range. You can't use this
     *             option if specifying specific IPv6 addresses.</p>
     */
    Ipv6AddressCount?: number;
    /**
     * <p>One or more specific IPv6 addresses to be assigned to the network interface. You can't use this option if you're specifying a number of IPv6 addresses.</p>
     */
    Ipv6Addresses?: string[];
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId: string | undefined;
}
export declare namespace AssignIpv6AddressesRequest {
    const filterSensitiveLog: (obj: AssignIpv6AddressesRequest) => any;
    const isa: (o: any) => o is AssignIpv6AddressesRequest;
}
export interface AssignIpv6AddressesResult {
    __type?: "AssignIpv6AddressesResult";
    /**
     * <p>The IPv6 addresses assigned to the network interface.</p>
     */
    AssignedIpv6Addresses?: string[];
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
}
export declare namespace AssignIpv6AddressesResult {
    const filterSensitiveLog: (obj: AssignIpv6AddressesResult) => any;
    const isa: (o: any) => o is AssignIpv6AddressesResult;
}
/**
 * <p>Contains the parameters for AssignPrivateIpAddresses.</p>
 */
export interface AssignPrivateIpAddressesRequest {
    __type?: "AssignPrivateIpAddressesRequest";
    /**
     * <p>Indicates whether to allow an IP address that is already assigned to another network interface or instance to be reassigned to the specified network interface.</p>
     */
    AllowReassignment?: boolean;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId: string | undefined;
    /**
     * <p>One or more IP addresses to be assigned as a secondary private IP address to the network interface. You can't specify this parameter when also specifying a number of secondary IP addresses.</p>
     *         <p>If you don't specify an IP address, Amazon EC2 automatically selects an IP address within the subnet range.</p>
     */
    PrivateIpAddresses?: string[];
    /**
     * <p>The number of secondary IP addresses to assign to the network interface. You can't specify this parameter when also specifying private IP addresses.</p>
     */
    SecondaryPrivateIpAddressCount?: number;
}
export declare namespace AssignPrivateIpAddressesRequest {
    const filterSensitiveLog: (obj: AssignPrivateIpAddressesRequest) => any;
    const isa: (o: any) => o is AssignPrivateIpAddressesRequest;
}
export interface AssignPrivateIpAddressesResult {
    __type?: "AssignPrivateIpAddressesResult";
    /**
     * <p>The private IP addresses assigned to the network interface.</p>
     */
    AssignedPrivateIpAddresses?: AssignedPrivateIpAddress[];
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
}
export declare namespace AssignPrivateIpAddressesResult {
    const filterSensitiveLog: (obj: AssignPrivateIpAddressesResult) => any;
    const isa: (o: any) => o is AssignPrivateIpAddressesResult;
}
export interface AssociateAddressRequest {
    __type?: "AssociateAddressRequest";
    /**
     * <p>[EC2-VPC] The allocation ID. This is required for EC2-VPC.</p>
     */
    AllocationId?: string;
    /**
     * <p>[EC2-VPC] For a VPC in an EC2-Classic account, specify true to allow an Elastic IP address that is already associated with an instance or network interface to be reassociated with the specified instance or network interface. Otherwise, the operation fails. In a VPC in an EC2-VPC-only account, reassociation is automatic, therefore you can specify false to ensure the operation fails if the Elastic IP address is already associated with another resource.</p>
     */
    AllowReassociation?: boolean;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance. This is required for EC2-Classic. For EC2-VPC, you can specify either the instance ID or the network interface ID, but not both. The operation fails if you specify an instance ID unless exactly one network interface is attached.</p>
     */
    InstanceId?: string;
    /**
     * <p>[EC2-VPC] The ID of the network interface. If the instance has more than one network interface, you must specify a network interface ID.</p>
     *    	     <p>For EC2-VPC, you can specify either the instance ID or the network interface ID, but not both. </p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>[EC2-VPC] The primary or secondary private IP address to associate with the Elastic IP address. If no private IP address is specified, the Elastic IP address is associated with the primary private IP address.</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>The Elastic IP address to associate with the instance. This is required for EC2-Classic.</p>
     */
    PublicIp?: string;
}
export declare namespace AssociateAddressRequest {
    const filterSensitiveLog: (obj: AssociateAddressRequest) => any;
    const isa: (o: any) => o is AssociateAddressRequest;
}
export interface AssociateAddressResult {
    __type?: "AssociateAddressResult";
    /**
     * <p>[EC2-VPC] The ID that represents the association of the Elastic IP address with an instance.</p>
     */
    AssociationId?: string;
}
export declare namespace AssociateAddressResult {
    const filterSensitiveLog: (obj: AssociateAddressResult) => any;
    const isa: (o: any) => o is AssociateAddressResult;
}
export interface AssociateClientVpnTargetNetworkRequest {
    __type?: "AssociateClientVpnTargetNetworkRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the subnet to associate with the Client VPN endpoint.</p>
     */
    SubnetId: string | undefined;
}
export declare namespace AssociateClientVpnTargetNetworkRequest {
    const filterSensitiveLog: (obj: AssociateClientVpnTargetNetworkRequest) => any;
    const isa: (o: any) => o is AssociateClientVpnTargetNetworkRequest;
}
export interface AssociateClientVpnTargetNetworkResult {
    __type?: "AssociateClientVpnTargetNetworkResult";
    /**
     * <p>The unique ID of the target network association.</p>
     */
    AssociationId?: string;
    /**
     * <p>The current state of the target network association.</p>
     */
    Status?: AssociationStatus;
}
export declare namespace AssociateClientVpnTargetNetworkResult {
    const filterSensitiveLog: (obj: AssociateClientVpnTargetNetworkResult) => any;
    const isa: (o: any) => o is AssociateClientVpnTargetNetworkResult;
}
export interface AssociateDhcpOptionsRequest {
    __type?: "AssociateDhcpOptionsRequest";
    /**
     * <p>The ID of the DHCP options set, or <code>default</code> to associate
     *         no DHCP options with the VPC.</p>
     */
    DhcpOptionsId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace AssociateDhcpOptionsRequest {
    const filterSensitiveLog: (obj: AssociateDhcpOptionsRequest) => any;
    const isa: (o: any) => o is AssociateDhcpOptionsRequest;
}
export declare type AssociatedNetworkType = "vpc";
/**
 * <p>Describes a target network that is associated with a Client VPN endpoint. A target network is a subnet in a VPC.</p>
 */
export interface AssociatedTargetNetwork {
    __type?: "AssociatedTargetNetwork";
    /**
     * <p>The ID of the subnet.</p>
     */
    NetworkId?: string;
    /**
     * <p>The target network type.</p>
     */
    NetworkType?: AssociatedNetworkType | string;
}
export declare namespace AssociatedTargetNetwork {
    const filterSensitiveLog: (obj: AssociatedTargetNetwork) => any;
    const isa: (o: any) => o is AssociatedTargetNetwork;
}
export interface AssociateIamInstanceProfileRequest {
    __type?: "AssociateIamInstanceProfileRequest";
    /**
     * <p>The IAM instance profile.</p>
     */
    IamInstanceProfile: IamInstanceProfileSpecification | undefined;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
}
export declare namespace AssociateIamInstanceProfileRequest {
    const filterSensitiveLog: (obj: AssociateIamInstanceProfileRequest) => any;
    const isa: (o: any) => o is AssociateIamInstanceProfileRequest;
}
export interface AssociateIamInstanceProfileResult {
    __type?: "AssociateIamInstanceProfileResult";
    /**
     * <p>Information about the IAM instance profile association.</p>
     */
    IamInstanceProfileAssociation?: IamInstanceProfileAssociation;
}
export declare namespace AssociateIamInstanceProfileResult {
    const filterSensitiveLog: (obj: AssociateIamInstanceProfileResult) => any;
    const isa: (o: any) => o is AssociateIamInstanceProfileResult;
}
export interface AssociateRouteTableRequest {
    __type?: "AssociateRouteTableRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the internet gateway or virtual private gateway.</p>
     */
    GatewayId?: string;
    /**
     * <p>The ID of the route table.</p>
     */
    RouteTableId: string | undefined;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId?: string;
}
export declare namespace AssociateRouteTableRequest {
    const filterSensitiveLog: (obj: AssociateRouteTableRequest) => any;
    const isa: (o: any) => o is AssociateRouteTableRequest;
}
export interface AssociateRouteTableResult {
    __type?: "AssociateRouteTableResult";
    /**
     * <p>The route table association ID. This ID is required for disassociating the route
     * 			table.</p>
     */
    AssociationId?: string;
    /**
     * <p>The state of the association.</p>
     */
    AssociationState?: RouteTableAssociationState;
}
export declare namespace AssociateRouteTableResult {
    const filterSensitiveLog: (obj: AssociateRouteTableResult) => any;
    const isa: (o: any) => o is AssociateRouteTableResult;
}
export interface AssociateSubnetCidrBlockRequest {
    __type?: "AssociateSubnetCidrBlockRequest";
    /**
     * <p>The IPv6 CIDR block for your subnet. The subnet must have a /64 prefix
     *             length.</p>
     */
    Ipv6CidrBlock: string | undefined;
    /**
     * <p>The ID of your subnet.</p>
     */
    SubnetId: string | undefined;
}
export declare namespace AssociateSubnetCidrBlockRequest {
    const filterSensitiveLog: (obj: AssociateSubnetCidrBlockRequest) => any;
    const isa: (o: any) => o is AssociateSubnetCidrBlockRequest;
}
export interface AssociateSubnetCidrBlockResult {
    __type?: "AssociateSubnetCidrBlockResult";
    /**
     * <p>Information about the IPv6 CIDR block association.</p>
     */
    Ipv6CidrBlockAssociation?: SubnetIpv6CidrBlockAssociation;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId?: string;
}
export declare namespace AssociateSubnetCidrBlockResult {
    const filterSensitiveLog: (obj: AssociateSubnetCidrBlockResult) => any;
    const isa: (o: any) => o is AssociateSubnetCidrBlockResult;
}
export interface AssociateTransitGatewayMulticastDomainRequest {
    __type?: "AssociateTransitGatewayMulticastDomainRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the subnets to associate with the transit gateway multicast domain.</p>
     */
    SubnetIds?: string[];
    /**
     * <p>The ID of the transit gateway attachment to associate with the transit gateway multicast domain.</p>
     */
    TransitGatewayAttachmentId?: string;
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace AssociateTransitGatewayMulticastDomainRequest {
    const filterSensitiveLog: (obj: AssociateTransitGatewayMulticastDomainRequest) => any;
    const isa: (o: any) => o is AssociateTransitGatewayMulticastDomainRequest;
}
export interface AssociateTransitGatewayMulticastDomainResult {
    __type?: "AssociateTransitGatewayMulticastDomainResult";
    /**
     * <p>Information about the transit gateway multicast domain associations.</p>
     */
    Associations?: TransitGatewayMulticastDomainAssociations;
}
export declare namespace AssociateTransitGatewayMulticastDomainResult {
    const filterSensitiveLog: (obj: AssociateTransitGatewayMulticastDomainResult) => any;
    const isa: (o: any) => o is AssociateTransitGatewayMulticastDomainResult;
}
export interface AssociateTransitGatewayRouteTableRequest {
    __type?: "AssociateTransitGatewayRouteTableRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
    /**
     * <p>The ID of the transit gateway route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace AssociateTransitGatewayRouteTableRequest {
    const filterSensitiveLog: (obj: AssociateTransitGatewayRouteTableRequest) => any;
    const isa: (o: any) => o is AssociateTransitGatewayRouteTableRequest;
}
export interface AssociateTransitGatewayRouteTableResult {
    __type?: "AssociateTransitGatewayRouteTableResult";
    /**
     * <p>The ID of the association.</p>
     */
    Association?: TransitGatewayAssociation;
}
export declare namespace AssociateTransitGatewayRouteTableResult {
    const filterSensitiveLog: (obj: AssociateTransitGatewayRouteTableResult) => any;
    const isa: (o: any) => o is AssociateTransitGatewayRouteTableResult;
}
export interface AssociateVpcCidrBlockRequest {
    __type?: "AssociateVpcCidrBlockRequest";
    /**
     * <p>Requests an Amazon-provided IPv6 CIDR block with a /56 prefix length for the VPC. You cannot specify the range of IPv6 addresses, or the size of the CIDR block.</p>
     */
    AmazonProvidedIpv6CidrBlock?: boolean;
    /**
     * <p>An IPv4 CIDR block to associate with the VPC.</p>
     */
    CidrBlock?: string;
    /**
     * <p>An IPv6 CIDR block from the IPv6 address pool. You must also specify <code>Ipv6Pool</code> in the request.</p>
     *         <p>To let Amazon choose the IPv6 CIDR block for you, omit this parameter.</p>
     */
    Ipv6CidrBlock?: string;
    /**
     * <p>The name of the location from which we advertise the IPV6 CIDR block. Use this parameter to limit the CiDR block to this location.</p>
     *          <p> You must set <code>AmazonProvidedIpv6CidrBlock</code> to <code>true</code> to use this parameter.</p>
     *          <p> You can have one IPv6 CIDR block association per network border group.</p>
     */
    Ipv6CidrBlockNetworkBorderGroup?: string;
    /**
     * <p>The ID of an IPv6 address pool from which to allocate the IPv6 CIDR block.</p>
     */
    Ipv6Pool?: string;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace AssociateVpcCidrBlockRequest {
    const filterSensitiveLog: (obj: AssociateVpcCidrBlockRequest) => any;
    const isa: (o: any) => o is AssociateVpcCidrBlockRequest;
}
export interface AssociateVpcCidrBlockResult {
    __type?: "AssociateVpcCidrBlockResult";
    /**
     * <p>Information about the IPv4 CIDR block association.</p>
     */
    CidrBlockAssociation?: VpcCidrBlockAssociation;
    /**
     * <p>Information about the IPv6 CIDR block association.</p>
     */
    Ipv6CidrBlockAssociation?: VpcIpv6CidrBlockAssociation;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace AssociateVpcCidrBlockResult {
    const filterSensitiveLog: (obj: AssociateVpcCidrBlockResult) => any;
    const isa: (o: any) => o is AssociateVpcCidrBlockResult;
}
/**
 * <p>Describes the state of a target network association.</p>
 */
export interface AssociationStatus {
    __type?: "AssociationStatus";
    /**
     * <p>The state of the target network association.</p>
     */
    Code?: AssociationStatusCode | string;
    /**
     * <p>A message about the status of the target network association, if applicable.</p>
     */
    Message?: string;
}
export declare namespace AssociationStatus {
    const filterSensitiveLog: (obj: AssociationStatus) => any;
    const isa: (o: any) => o is AssociationStatus;
}
export declare type AssociationStatusCode = "associated" | "associating" | "association-failed" | "disassociated" | "disassociating";
export interface AttachClassicLinkVpcRequest {
    __type?: "AttachClassicLinkVpcRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of one or more of the VPC's security groups. You cannot specify security groups from a different VPC.</p>
     */
    Groups: string[] | undefined;
    /**
     * <p>The ID of an EC2-Classic instance to link to the ClassicLink-enabled VPC.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>The ID of a ClassicLink-enabled VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace AttachClassicLinkVpcRequest {
    const filterSensitiveLog: (obj: AttachClassicLinkVpcRequest) => any;
    const isa: (o: any) => o is AttachClassicLinkVpcRequest;
}
export interface AttachClassicLinkVpcResult {
    __type?: "AttachClassicLinkVpcResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace AttachClassicLinkVpcResult {
    const filterSensitiveLog: (obj: AttachClassicLinkVpcResult) => any;
    const isa: (o: any) => o is AttachClassicLinkVpcResult;
}
export interface AttachInternetGatewayRequest {
    __type?: "AttachInternetGatewayRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the internet gateway.</p>
     */
    InternetGatewayId: string | undefined;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace AttachInternetGatewayRequest {
    const filterSensitiveLog: (obj: AttachInternetGatewayRequest) => any;
    const isa: (o: any) => o is AttachInternetGatewayRequest;
}
export declare type AttachmentStatus = "attached" | "attaching" | "detached" | "detaching";
/**
 * <p>Contains the parameters for AttachNetworkInterface.</p>
 */
export interface AttachNetworkInterfaceRequest {
    __type?: "AttachNetworkInterfaceRequest";
    /**
     * <p>The index of the device for the network interface attachment.</p>
     */
    DeviceIndex: number | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId: string | undefined;
}
export declare namespace AttachNetworkInterfaceRequest {
    const filterSensitiveLog: (obj: AttachNetworkInterfaceRequest) => any;
    const isa: (o: any) => o is AttachNetworkInterfaceRequest;
}
/**
 * <p>Contains the output of AttachNetworkInterface.</p>
 */
export interface AttachNetworkInterfaceResult {
    __type?: "AttachNetworkInterfaceResult";
    /**
     * <p>The ID of the network interface attachment.</p>
     */
    AttachmentId?: string;
}
export declare namespace AttachNetworkInterfaceResult {
    const filterSensitiveLog: (obj: AttachNetworkInterfaceResult) => any;
    const isa: (o: any) => o is AttachNetworkInterfaceResult;
}
export interface AttachVolumeRequest {
    __type?: "AttachVolumeRequest";
    /**
     * <p>The device name (for example, <code>/dev/sdh</code> or <code>xvdh</code>).</p>
     */
    Device: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>The ID of the EBS volume. The volume and instance must be within the same Availability
     *       Zone.</p>
     */
    VolumeId: string | undefined;
}
export declare namespace AttachVolumeRequest {
    const filterSensitiveLog: (obj: AttachVolumeRequest) => any;
    const isa: (o: any) => o is AttachVolumeRequest;
}
/**
 * <p>Contains the parameters for AttachVpnGateway.</p>
 */
export interface AttachVpnGatewayRequest {
    __type?: "AttachVpnGatewayRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
    /**
     * <p>The ID of the virtual private gateway.</p>
     */
    VpnGatewayId: string | undefined;
}
export declare namespace AttachVpnGatewayRequest {
    const filterSensitiveLog: (obj: AttachVpnGatewayRequest) => any;
    const isa: (o: any) => o is AttachVpnGatewayRequest;
}
/**
 * <p>Contains the output of AttachVpnGateway.</p>
 */
export interface AttachVpnGatewayResult {
    __type?: "AttachVpnGatewayResult";
    /**
     * <p>Information about the attachment.</p>
     */
    VpcAttachment?: VpcAttachment;
}
export declare namespace AttachVpnGatewayResult {
    const filterSensitiveLog: (obj: AttachVpnGatewayResult) => any;
    const isa: (o: any) => o is AttachVpnGatewayResult;
}
/**
 * <p>Describes a value for a resource attribute that is a Boolean value.</p>
 */
export interface AttributeBooleanValue {
    __type?: "AttributeBooleanValue";
    /**
     * <p>The attribute value. The valid values are <code>true</code> or <code>false</code>.</p>
     */
    Value?: boolean;
}
export declare namespace AttributeBooleanValue {
    const filterSensitiveLog: (obj: AttributeBooleanValue) => any;
    const isa: (o: any) => o is AttributeBooleanValue;
}
/**
 * <p>Describes a value for a resource attribute that is a String.</p>
 */
export interface AttributeValue {
    __type?: "AttributeValue";
    /**
     * <p>The attribute value. The value is case-sensitive.</p>
     */
    Value?: string;
}
export declare namespace AttributeValue {
    const filterSensitiveLog: (obj: AttributeValue) => any;
    const isa: (o: any) => o is AttributeValue;
}
/**
 * <p>Information about an authorization rule.</p>
 */
export interface AuthorizationRule {
    __type?: "AuthorizationRule";
    /**
     * <p>Indicates whether the authorization rule grants access to all clients.</p>
     */
    AccessAll?: boolean;
    /**
     * <p>The ID of the Client VPN endpoint with which the authorization rule is associated.</p>
     */
    ClientVpnEndpointId?: string;
    /**
     * <p>A brief description of the authorization rule.</p>
     */
    Description?: string;
    /**
     * <p>The IPv4 address range, in CIDR notation, of the network to which the authorization rule applies.</p>
     */
    DestinationCidr?: string;
    /**
     * <p>The ID of the Active Directory group to which the authorization rule grants access.</p>
     */
    GroupId?: string;
    /**
     * <p>The current state of the authorization rule.</p>
     */
    Status?: ClientVpnAuthorizationRuleStatus;
}
export declare namespace AuthorizationRule {
    const filterSensitiveLog: (obj: AuthorizationRule) => any;
    const isa: (o: any) => o is AuthorizationRule;
}
export interface AuthorizeClientVpnIngressRequest {
    __type?: "AuthorizeClientVpnIngressRequest";
    /**
     * <p>The ID of the Active Directory group to grant access.</p>
     */
    AccessGroupId?: string;
    /**
     * <p>Indicates whether to grant access to all clients. Use <code>true</code> to grant all clients
     * 			who successfully establish a VPN connection access to the network.</p>
     */
    AuthorizeAllGroups?: boolean;
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>A brief description of the authorization rule.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IPv4 address range, in CIDR notation, of the network for which access is being authorized.</p>
     */
    TargetNetworkCidr: string | undefined;
}
export declare namespace AuthorizeClientVpnIngressRequest {
    const filterSensitiveLog: (obj: AuthorizeClientVpnIngressRequest) => any;
    const isa: (o: any) => o is AuthorizeClientVpnIngressRequest;
}
export interface AuthorizeClientVpnIngressResult {
    __type?: "AuthorizeClientVpnIngressResult";
    /**
     * <p>The current state of the authorization rule.</p>
     */
    Status?: ClientVpnAuthorizationRuleStatus;
}
export declare namespace AuthorizeClientVpnIngressResult {
    const filterSensitiveLog: (obj: AuthorizeClientVpnIngressResult) => any;
    const isa: (o: any) => o is AuthorizeClientVpnIngressResult;
}
export interface AuthorizeSecurityGroupEgressRequest {
    __type?: "AuthorizeSecurityGroupEgressRequest";
    /**
     * <p>Not supported. Use a set of IP permissions to specify the CIDR.</p>
     */
    CidrIp?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Not supported. Use a set of IP permissions to specify the port.</p>
     */
    FromPort?: number;
    /**
     * <p>The ID of the security group.</p>
     */
    GroupId: string | undefined;
    /**
     * <p>The sets of IP permissions. You can't specify a destination security group and a CIDR IP
     *             address range in the same set of permissions.</p>
     */
    IpPermissions?: IpPermission[];
    /**
     * <p>Not supported. Use a set of IP permissions to specify the protocol name or
     *             number.</p>
     */
    IpProtocol?: string;
    /**
     * <p>Not supported. Use a set of IP permissions to specify a
     *             destination security group.</p>
     */
    SourceSecurityGroupName?: string;
    /**
     * <p>Not supported. Use a set of IP permissions to specify a
     *             destination security group.</p>
     */
    SourceSecurityGroupOwnerId?: string;
    /**
     * <p>Not supported. Use a set of IP permissions to specify the port.</p>
     */
    ToPort?: number;
}
export declare namespace AuthorizeSecurityGroupEgressRequest {
    const filterSensitiveLog: (obj: AuthorizeSecurityGroupEgressRequest) => any;
    const isa: (o: any) => o is AuthorizeSecurityGroupEgressRequest;
}
export interface AuthorizeSecurityGroupIngressRequest {
    __type?: "AuthorizeSecurityGroupIngressRequest";
    /**
     * <p>The IPv4 address range, in CIDR format. You can't specify this parameter when specifying a source
     *       security group. To specify an IPv6 address range, use a set of IP permissions.</p>
     *          <p>Alternatively, use a set of IP permissions to specify multiple rules and a description for the rule.</p>
     */
    CidrIp?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The start of port range for the TCP and UDP protocols, or an ICMP type number.
     * 			For the ICMP type number, use <code>-1</code> to specify all types. If you
     * 			specify all ICMP types, you must specify all codes.</p>
     *          <p>Alternatively, use a set of IP permissions to specify multiple rules and a description for the rule.</p>
     */
    FromPort?: number;
    /**
     * <p>The ID of the security group. You must specify either the security group ID or the
     * 			security group name in the request. For security groups in a nondefault VPC, you must
     * 			specify the security group ID.</p>
     */
    GroupId?: string;
    /**
     * <p>[EC2-Classic, default VPC] The name of the security group. You must specify either the
     * 			security group ID or the security group name in the request.</p>
     */
    GroupName?: string;
    /**
     * <p>The sets of IP permissions.</p>
     */
    IpPermissions?: IpPermission[];
    /**
     * <p>The IP protocol name (<code>tcp</code>, <code>udp</code>, <code>icmp</code>) or number
     *       (see <a href="http://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml">Protocol Numbers</a>). To specify <code>icmpv6</code>, use a set of IP permissions.</p>
     *          <p>[VPC only] Use <code>-1</code> to specify all protocols. If you specify <code>-1</code> or a
     *          protocol other than <code>tcp</code>, <code>udp</code>, or <code>icmp</code>, traffic on all ports
     *          is allowed, regardless of any ports you specify.</p>
     *          <p>Alternatively, use a set of IP permissions to specify multiple rules and a description for the rule.</p>
     */
    IpProtocol?: string;
    /**
     * <p>[EC2-Classic, default VPC] The name of the source security group. You can't specify this parameter
     *          in combination with the following parameters: the CIDR IP address range, the start of the port range,
     *          the IP protocol, and the end of the port range. Creates rules that grant full ICMP, UDP, and TCP access.
     *          To create a rule with a specific IP protocol and port range, use a set of IP permissions instead. For
     *          EC2-VPC, the source security group must be in the same VPC.</p>
     */
    SourceSecurityGroupName?: string;
    /**
     * <p>[nondefault VPC] The AWS account ID for the source security group, if the source security group is
     *          in a different account. You can't specify this parameter in combination with the following parameters:
     *          the CIDR IP address range, the IP protocol, the start of the port range, and the end of the port range.
     *          Creates rules that grant full ICMP, UDP, and TCP access. To create a rule with a specific IP protocol
     *          and port range, use a set of IP permissions instead.</p>
     */
    SourceSecurityGroupOwnerId?: string;
    /**
     * <p>The end of port range for the TCP and UDP protocols, or an ICMP code number.
     * 			For the ICMP code number, use <code>-1</code> to specify all codes. If you
     * 			specify all ICMP types, you must specify all codes.</p>
     *          <p>Alternatively, use a set of IP permissions to specify multiple rules and a description for the rule.</p>
     */
    ToPort?: number;
}
export declare namespace AuthorizeSecurityGroupIngressRequest {
    const filterSensitiveLog: (obj: AuthorizeSecurityGroupIngressRequest) => any;
    const isa: (o: any) => o is AuthorizeSecurityGroupIngressRequest;
}
export declare type AutoAcceptSharedAttachmentsValue = "disable" | "enable";
export declare type AutoPlacement = "off" | "on";
/**
 * <p>Describes an Availability Zone or Local Zone.</p>
 */
export interface AvailabilityZone {
    __type?: "AvailabilityZone";
    /**
     * <p> For Availability Zones, this parameter has the same value as the Region name.</p>
     *          <p>For Local Zones, the name of the associated group, for example <code>us-west-2-lax-1</code>.</p>
     */
    GroupName?: string;
    /**
     * <p>Any messages about the Availability Zone or Local Zone.</p>
     */
    Messages?: AvailabilityZoneMessage[];
    /**
     * <p>The name of the location from which the address is advertised.</p>
     */
    NetworkBorderGroup?: string;
    /**
     * <p> For Availability Zones, this parameter always has the value of
     *         <code>opt-in-not-required</code>.</p>
     *          <p>For Local Zones, this parameter is the opt in status. The possible values are <code>opted-in</code>, and
     *       <code>not-opted-in</code>.</p>
     */
    OptInStatus?: AvailabilityZoneOptInStatus | string;
    /**
     * <p>The name of the Region.</p>
     */
    RegionName?: string;
    /**
     * <p>The state of the Availability Zone or Local Zone.</p>
     */
    State?: AvailabilityZoneState | string;
    /**
     * <p>The ID of the Availability Zone or Local Zone.</p>
     */
    ZoneId?: string;
    /**
     * <p>The name of the Availability Zone or Local Zone.</p>
     */
    ZoneName?: string;
}
export declare namespace AvailabilityZone {
    const filterSensitiveLog: (obj: AvailabilityZone) => any;
    const isa: (o: any) => o is AvailabilityZone;
}
/**
 * <p>Describes a message about an Availability Zone or Local Zone.</p>
 */
export interface AvailabilityZoneMessage {
    __type?: "AvailabilityZoneMessage";
    /**
     * <p>The message about the Availability Zone or Local Zone.</p>
     */
    Message?: string;
}
export declare namespace AvailabilityZoneMessage {
    const filterSensitiveLog: (obj: AvailabilityZoneMessage) => any;
    const isa: (o: any) => o is AvailabilityZoneMessage;
}
export declare type AvailabilityZoneOptInStatus = "not-opted-in" | "opt-in-not-required" | "opted-in";
export declare type AvailabilityZoneState = "available" | "impaired" | "information" | "unavailable";
/**
 * <p>The capacity information for instances that can be launched onto the Dedicated Host. </p>
 */
export interface AvailableCapacity {
    __type?: "AvailableCapacity";
    /**
     * <p>The number of instances that can be launched onto the Dedicated Host depending on
     *     		the host's available capacity. For Dedicated Hosts that support multiple instance types,
     *     		this parameter represents the number of instances for each instance size that is
     *     		supported on the host.</p>
     */
    AvailableInstanceCapacity?: InstanceCapacity[];
    /**
     * <p>The number of vCPUs available for launching instances onto the Dedicated Host.</p>
     */
    AvailableVCpus?: number;
}
export declare namespace AvailableCapacity {
    const filterSensitiveLog: (obj: AvailableCapacity) => any;
    const isa: (o: any) => o is AvailableCapacity;
}
export declare enum BatchState {
    ACTIVE = "active",
    CANCELLED = "cancelled",
    CANCELLED_RUNNING = "cancelled_running",
    CANCELLED_TERMINATING_INSTANCES = "cancelled_terminating",
    FAILED = "failed",
    MODIFYING = "modifying",
    SUBMITTED = "submitted"
}
export interface BlobAttributeValue {
    __type?: "BlobAttributeValue";
    Value?: Uint8Array;
}
export declare namespace BlobAttributeValue {
    const filterSensitiveLog: (obj: BlobAttributeValue) => any;
    const isa: (o: any) => o is BlobAttributeValue;
}
/**
 * <p>Describes a block device mapping.</p>
 */
export interface BlockDeviceMapping {
    __type?: "BlockDeviceMapping";
    /**
     * <p>The device name (for example, <code>/dev/sdh</code> or <code>xvdh</code>).</p>
     */
    DeviceName?: string;
    /**
     * <p>Parameters used to automatically set up EBS volumes when the instance is
     *             launched.</p>
     */
    Ebs?: EbsBlockDevice;
    /**
     * <p>Suppresses the specified device included in the block device mapping of the
     *             AMI.</p>
     */
    NoDevice?: string;
    /**
     * <p>The virtual device name (<code>ephemeral</code>N). Instance store volumes are numbered
     *             starting from 0. An instance type with 2 available instance store volumes can specify
     *             mappings for <code>ephemeral0</code> and <code>ephemeral1</code>. The number of
     *             available instance store volumes depends on the instance type. After you connect to the
     *             instance, you must mount the volume.</p>
     *         <p>NVMe instance store volumes are automatically enumerated and assigned a device name.
     *             Including them in your block device mapping has no effect.</p>
     *         <p>Constraints: For M3 instances, you must specify instance store volumes in the block
     *             device mapping for the instance. When you launch an M3 instance, we ignore any instance
     *             store volumes specified in the block device mapping for the AMI.</p>
     */
    VirtualName?: string;
}
export declare namespace BlockDeviceMapping {
    const filterSensitiveLog: (obj: BlockDeviceMapping) => any;
    const isa: (o: any) => o is BlockDeviceMapping;
}
/**
 * <p>Contains the parameters for BundleInstance.</p>
 */
export interface BundleInstanceRequest {
    __type?: "BundleInstanceRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance to bundle.</p>
     *          <p>Type: String</p>
     *          <p>Default: None</p>
     *          <p>Required: Yes</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>The bucket in which to store the AMI. You can specify a bucket that you already own or a new bucket that Amazon EC2 creates on your behalf. If you specify a bucket that belongs to someone else, Amazon EC2 returns an error.</p>
     */
    Storage: Storage | undefined;
}
export declare namespace BundleInstanceRequest {
    const filterSensitiveLog: (obj: BundleInstanceRequest) => any;
    const isa: (o: any) => o is BundleInstanceRequest;
}
/**
 * <p>Contains the output of BundleInstance.</p>
 */
export interface BundleInstanceResult {
    __type?: "BundleInstanceResult";
    /**
     * <p>Information about the bundle task.</p>
     */
    BundleTask?: BundleTask;
}
export declare namespace BundleInstanceResult {
    const filterSensitiveLog: (obj: BundleInstanceResult) => any;
    const isa: (o: any) => o is BundleInstanceResult;
}
/**
 * <p>Describes a bundle task.</p>
 */
export interface BundleTask {
    __type?: "BundleTask";
    /**
     * <p>The ID of the bundle task.</p>
     */
    BundleId?: string;
    /**
     * <p>If the task fails, a description of the error.</p>
     */
    BundleTaskError?: BundleTaskError;
    /**
     * <p>The ID of the instance associated with this bundle task.</p>
     */
    InstanceId?: string;
    /**
     * <p>The level of task completion, as a percent (for example, 20%).</p>
     */
    Progress?: string;
    /**
     * <p>The time this task started.</p>
     */
    StartTime?: Date;
    /**
     * <p>The state of the task.</p>
     */
    State?: BundleTaskState | string;
    /**
     * <p>The Amazon S3 storage locations.</p>
     */
    Storage?: Storage;
    /**
     * <p>The time of the most recent update for the task.</p>
     */
    UpdateTime?: Date;
}
export declare namespace BundleTask {
    const filterSensitiveLog: (obj: BundleTask) => any;
    const isa: (o: any) => o is BundleTask;
}
/**
 * <p>Describes an error for <a>BundleInstance</a>.</p>
 */
export interface BundleTaskError {
    __type?: "BundleTaskError";
    /**
     * <p>The error code.</p>
     */
    Code?: string;
    /**
     * <p>The error message.</p>
     */
    Message?: string;
}
export declare namespace BundleTaskError {
    const filterSensitiveLog: (obj: BundleTaskError) => any;
    const isa: (o: any) => o is BundleTaskError;
}
export declare type BundleTaskState = "bundling" | "cancelling" | "complete" | "failed" | "pending" | "storing" | "waiting-for-shutdown";
/**
 * <p>Information about an address range that is provisioned for use with your AWS resources
 *          through bring your own IP addresses (BYOIP).</p>
 */
export interface ByoipCidr {
    __type?: "ByoipCidr";
    /**
     * <p>The address range, in CIDR notation.</p>
     */
    Cidr?: string;
    /**
     * <p>The description of the address range.</p>
     */
    Description?: string;
    /**
     * <p>The state of the address pool.</p>
     */
    State?: ByoipCidrState | string;
    /**
     * <p>Upon success, contains the ID of the address pool. Otherwise, contains an error message.</p>
     */
    StatusMessage?: string;
}
export declare namespace ByoipCidr {
    const filterSensitiveLog: (obj: ByoipCidr) => any;
    const isa: (o: any) => o is ByoipCidr;
}
export declare type ByoipCidrState = "advertised" | "deprovisioned" | "failed-deprovision" | "failed-provision" | "pending-deprovision" | "pending-provision" | "provisioned" | "provisioned-not-publicly-advertisable";
export declare enum CancelBatchErrorCode {
    FLEET_REQUEST_ID_DOES_NOT_EXIST = "fleetRequestIdDoesNotExist",
    FLEET_REQUEST_ID_MALFORMED = "fleetRequestIdMalformed",
    FLEET_REQUEST_NOT_IN_CANCELLABLE_STATE = "fleetRequestNotInCancellableState",
    UNEXPECTED_ERROR = "unexpectedError"
}
/**
 * <p>Contains the parameters for CancelBundleTask.</p>
 */
export interface CancelBundleTaskRequest {
    __type?: "CancelBundleTaskRequest";
    /**
     * <p>The ID of the bundle task.</p>
     */
    BundleId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace CancelBundleTaskRequest {
    const filterSensitiveLog: (obj: CancelBundleTaskRequest) => any;
    const isa: (o: any) => o is CancelBundleTaskRequest;
}
/**
 * <p>Contains the output of CancelBundleTask.</p>
 */
export interface CancelBundleTaskResult {
    __type?: "CancelBundleTaskResult";
    /**
     * <p>Information about the bundle task.</p>
     */
    BundleTask?: BundleTask;
}
export declare namespace CancelBundleTaskResult {
    const filterSensitiveLog: (obj: CancelBundleTaskResult) => any;
    const isa: (o: any) => o is CancelBundleTaskResult;
}
export interface CancelCapacityReservationRequest {
    __type?: "CancelCapacityReservationRequest";
    /**
     * <p>The ID of the Capacity Reservation to be cancelled.</p>
     */
    CapacityReservationId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace CancelCapacityReservationRequest {
    const filterSensitiveLog: (obj: CancelCapacityReservationRequest) => any;
    const isa: (o: any) => o is CancelCapacityReservationRequest;
}
export interface CancelCapacityReservationResult {
    __type?: "CancelCapacityReservationResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace CancelCapacityReservationResult {
    const filterSensitiveLog: (obj: CancelCapacityReservationResult) => any;
    const isa: (o: any) => o is CancelCapacityReservationResult;
}
export interface CancelConversionRequest {
    __type?: "CancelConversionRequest";
    /**
     * <p>The ID of the conversion task.</p>
     */
    ConversionTaskId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The reason for canceling the conversion task.</p>
     */
    ReasonMessage?: string;
}
export declare namespace CancelConversionRequest {
    const filterSensitiveLog: (obj: CancelConversionRequest) => any;
    const isa: (o: any) => o is CancelConversionRequest;
}
export interface CancelExportTaskRequest {
    __type?: "CancelExportTaskRequest";
    /**
     * <p>The ID of the export task. This is the ID returned by <code>CreateInstanceExportTask</code>.</p>
     */
    ExportTaskId: string | undefined;
}
export declare namespace CancelExportTaskRequest {
    const filterSensitiveLog: (obj: CancelExportTaskRequest) => any;
    const isa: (o: any) => o is CancelExportTaskRequest;
}
export interface CancelImportTaskRequest {
    __type?: "CancelImportTaskRequest";
    /**
     * <p>The reason for canceling the task.</p>
     */
    CancelReason?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the import image or import snapshot task to be canceled.</p>
     */
    ImportTaskId?: string;
}
export declare namespace CancelImportTaskRequest {
    const filterSensitiveLog: (obj: CancelImportTaskRequest) => any;
    const isa: (o: any) => o is CancelImportTaskRequest;
}
export interface CancelImportTaskResult {
    __type?: "CancelImportTaskResult";
    /**
     * <p>The ID of the task being canceled.</p>
     */
    ImportTaskId?: string;
    /**
     * <p>The current state of the task being canceled.</p>
     */
    PreviousState?: string;
    /**
     * <p>The current state of the task being canceled.</p>
     */
    State?: string;
}
export declare namespace CancelImportTaskResult {
    const filterSensitiveLog: (obj: CancelImportTaskResult) => any;
    const isa: (o: any) => o is CancelImportTaskResult;
}
/**
 * <p>Describes a request to cancel a Spot Instance.</p>
 */
export interface CancelledSpotInstanceRequest {
    __type?: "CancelledSpotInstanceRequest";
    /**
     * <p>The ID of the Spot Instance request.</p>
     */
    SpotInstanceRequestId?: string;
    /**
     * <p>The state of the Spot Instance request.</p>
     */
    State?: CancelSpotInstanceRequestState | string;
}
export declare namespace CancelledSpotInstanceRequest {
    const filterSensitiveLog: (obj: CancelledSpotInstanceRequest) => any;
    const isa: (o: any) => o is CancelledSpotInstanceRequest;
}
/**
 * <p>Contains the parameters for CancelReservedInstancesListing.</p>
 */
export interface CancelReservedInstancesListingRequest {
    __type?: "CancelReservedInstancesListingRequest";
    /**
     * <p>The ID of the Reserved Instance listing.</p>
     */
    ReservedInstancesListingId: string | undefined;
}
export declare namespace CancelReservedInstancesListingRequest {
    const filterSensitiveLog: (obj: CancelReservedInstancesListingRequest) => any;
    const isa: (o: any) => o is CancelReservedInstancesListingRequest;
}
/**
 * <p>Contains the output of CancelReservedInstancesListing.</p>
 */
export interface CancelReservedInstancesListingResult {
    __type?: "CancelReservedInstancesListingResult";
    /**
     * <p>The Reserved Instance listing.</p>
     */
    ReservedInstancesListings?: ReservedInstancesListing[];
}
export declare namespace CancelReservedInstancesListingResult {
    const filterSensitiveLog: (obj: CancelReservedInstancesListingResult) => any;
    const isa: (o: any) => o is CancelReservedInstancesListingResult;
}
/**
 * <p>Describes a Spot Fleet error.</p>
 */
export interface CancelSpotFleetRequestsError {
    __type?: "CancelSpotFleetRequestsError";
    /**
     * <p>The error code.</p>
     */
    Code?: CancelBatchErrorCode | string;
    /**
     * <p>The description for the error code.</p>
     */
    Message?: string;
}
export declare namespace CancelSpotFleetRequestsError {
    const filterSensitiveLog: (obj: CancelSpotFleetRequestsError) => any;
    const isa: (o: any) => o is CancelSpotFleetRequestsError;
}
/**
 * <p>Describes a Spot Fleet request that was not successfully canceled.</p>
 */
export interface CancelSpotFleetRequestsErrorItem {
    __type?: "CancelSpotFleetRequestsErrorItem";
    /**
     * <p>The error.</p>
     */
    Error?: CancelSpotFleetRequestsError;
    /**
     * <p>The ID of the Spot Fleet request.</p>
     */
    SpotFleetRequestId?: string;
}
export declare namespace CancelSpotFleetRequestsErrorItem {
    const filterSensitiveLog: (obj: CancelSpotFleetRequestsErrorItem) => any;
    const isa: (o: any) => o is CancelSpotFleetRequestsErrorItem;
}
/**
 * <p>Contains the parameters for CancelSpotFleetRequests.</p>
 */
export interface CancelSpotFleetRequestsRequest {
    __type?: "CancelSpotFleetRequestsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *       and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *       Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the Spot Fleet requests.</p>
     */
    SpotFleetRequestIds: string[] | undefined;
    /**
     * <p>Indicates whether to terminate instances for a Spot Fleet request if it is canceled successfully.</p>
     */
    TerminateInstances: boolean | undefined;
}
export declare namespace CancelSpotFleetRequestsRequest {
    const filterSensitiveLog: (obj: CancelSpotFleetRequestsRequest) => any;
    const isa: (o: any) => o is CancelSpotFleetRequestsRequest;
}
/**
 * <p>Contains the output of CancelSpotFleetRequests.</p>
 */
export interface CancelSpotFleetRequestsResponse {
    __type?: "CancelSpotFleetRequestsResponse";
    /**
     * <p>Information about the Spot Fleet requests that are successfully canceled.</p>
     */
    SuccessfulFleetRequests?: CancelSpotFleetRequestsSuccessItem[];
    /**
     * <p>Information about the Spot Fleet requests that are not successfully canceled.</p>
     */
    UnsuccessfulFleetRequests?: CancelSpotFleetRequestsErrorItem[];
}
export declare namespace CancelSpotFleetRequestsResponse {
    const filterSensitiveLog: (obj: CancelSpotFleetRequestsResponse) => any;
    const isa: (o: any) => o is CancelSpotFleetRequestsResponse;
}
/**
 * <p>Describes a Spot Fleet request that was successfully canceled.</p>
 */
export interface CancelSpotFleetRequestsSuccessItem {
    __type?: "CancelSpotFleetRequestsSuccessItem";
    /**
     * <p>The current state of the Spot Fleet request.</p>
     */
    CurrentSpotFleetRequestState?: BatchState | string;
    /**
     * <p>The previous state of the Spot Fleet request.</p>
     */
    PreviousSpotFleetRequestState?: BatchState | string;
    /**
     * <p>The ID of the Spot Fleet request.</p>
     */
    SpotFleetRequestId?: string;
}
export declare namespace CancelSpotFleetRequestsSuccessItem {
    const filterSensitiveLog: (obj: CancelSpotFleetRequestsSuccessItem) => any;
    const isa: (o: any) => o is CancelSpotFleetRequestsSuccessItem;
}
/**
 * <p>Contains the parameters for CancelSpotInstanceRequests.</p>
 */
export interface CancelSpotInstanceRequestsRequest {
    __type?: "CancelSpotInstanceRequestsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more Spot Instance request IDs.</p>
     */
    SpotInstanceRequestIds: string[] | undefined;
}
export declare namespace CancelSpotInstanceRequestsRequest {
    const filterSensitiveLog: (obj: CancelSpotInstanceRequestsRequest) => any;
    const isa: (o: any) => o is CancelSpotInstanceRequestsRequest;
}
/**
 * <p>Contains the output of CancelSpotInstanceRequests.</p>
 */
export interface CancelSpotInstanceRequestsResult {
    __type?: "CancelSpotInstanceRequestsResult";
    /**
     * <p>One or more Spot Instance requests.</p>
     */
    CancelledSpotInstanceRequests?: CancelledSpotInstanceRequest[];
}
export declare namespace CancelSpotInstanceRequestsResult {
    const filterSensitiveLog: (obj: CancelSpotInstanceRequestsResult) => any;
    const isa: (o: any) => o is CancelSpotInstanceRequestsResult;
}
export declare type CancelSpotInstanceRequestState = "active" | "cancelled" | "closed" | "completed" | "open";
/**
 * <p>Describes a Capacity Reservation.</p>
 */
export interface CapacityReservation {
    __type?: "CapacityReservation";
    /**
     * <p>The Availability Zone in which the capacity is reserved.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The Availability Zone ID of the Capacity Reservation.</p>
     */
    AvailabilityZoneId?: string;
    /**
     * <p>The remaining capacity. Indicates the number of instances that can be launched in the Capacity Reservation.</p>
     */
    AvailableInstanceCount?: number;
    /**
     * <p>The Amazon Resource Name (ARN) of the Capacity Reservation.</p>
     */
    CapacityReservationArn?: string;
    /**
     * <p>The ID of the Capacity Reservation.</p>
     */
    CapacityReservationId?: string;
    /**
     * <p>The date and time at which the Capacity Reservation was created.</p>
     */
    CreateDate?: Date;
    /**
     * <p>Indicates whether the Capacity Reservation supports EBS-optimized instances. This optimization provides
     * 			dedicated throughput to Amazon EBS and an optimized configuration stack to provide
     * 			optimal I/O performance. This optimization isn't available with all instance types.
     * 			Additional usage charges apply when using an EBS- optimized instance.</p>
     */
    EbsOptimized?: boolean;
    /**
     * <p>The date and time at which the Capacity Reservation expires. When a Capacity Reservation expires, the reserved capacity
     * 			is released and you can no longer launch instances into it. The Capacity Reservation's state changes to
     * 				<code>expired</code> when it reaches its end date and time.</p>
     */
    EndDate?: Date;
    /**
     * <p>Indicates the way in which the Capacity Reservation ends. A Capacity Reservation can have one of the following end
     * 			types:</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>unlimited</code> - The Capacity Reservation remains active until you explicitly cancel it.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>limited</code> - The Capacity Reservation expires automatically at a specified date and time.</p>
     *             </li>
     *          </ul>
     */
    EndDateType?: EndDateType | string;
    /**
     * <p>Indicates whether the Capacity Reservation supports instances with temporary, block-level
     * 			storage.</p>
     */
    EphemeralStorage?: boolean;
    /**
     * <p>Indicates the type of instance launches that the Capacity Reservation accepts. The options
     * 			include:</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>open</code> - The Capacity Reservation accepts all instances that have matching attributes (instance type, platform,
     * 				and Availability Zone). Instances that have matching attributes launch into the Capacity Reservation automatically without specifying
     * 				any additional parameters.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>targeted</code> - The Capacity Reservation only accepts instances that have matching attributes
     * 					(instance type, platform, and Availability Zone), and explicitly target the
     * 					Capacity Reservation. This ensures that only permitted instances can use the reserved capacity. </p>
     *             </li>
     *          </ul>
     */
    InstanceMatchCriteria?: InstanceMatchCriteria | string;
    /**
     * <p>The type of operating system for which the Capacity Reservation reserves capacity.</p>
     */
    InstancePlatform?: CapacityReservationInstancePlatform | string;
    /**
     * <p>The type of instance for which the Capacity Reservation reserves capacity.</p>
     */
    InstanceType?: string;
    /**
     * <p>The ID of the AWS account that owns the Capacity Reservation.</p>
     */
    OwnerId?: string;
    /**
     * <p>The current state of the Capacity Reservation. A Capacity Reservation can be in one of the following states:</p>
     * 			      <ul>
     *             <li>
     *                <p>
     *                   <code>active</code> - The Capacity Reservation is active and the capacity is available for your use.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>expired</code> - The Capacity Reservation expired automatically at the date and time specified
     * 					in your request. The reserved capacity is no longer available for your use.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>cancelled</code> - The Capacity Reservation was manually cancelled. The reserved capacity is no
     * 					longer available for your use.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>pending</code> - The Capacity Reservation request was successful but the capacity
     * 					provisioning is still pending.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>failed</code> - The Capacity Reservation request has failed. A request might fail
     * 					due to invalid request parameters, capacity constraints, or instance limit constraints.
     * 					Failed requests are retained for 60 minutes.</p>
     *             </li>
     *          </ul>
     */
    State?: CapacityReservationState | string;
    /**
     * <p>Any tags assigned to the Capacity Reservation.</p>
     */
    Tags?: Tag[];
    /**
     * <p>Indicates the tenancy of the Capacity Reservation. A Capacity Reservation can have one of the following tenancy settings:</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>default</code> - The Capacity Reservation is created on hardware that is shared with other AWS accounts.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>dedicated</code> - The Capacity Reservation is created on single-tenant hardware that is dedicated to a single AWS account.</p>
     *             </li>
     *          </ul>
     */
    Tenancy?: CapacityReservationTenancy | string;
    /**
     * <p>The total number of instances for which the Capacity Reservation reserves capacity.</p>
     */
    TotalInstanceCount?: number;
}
export declare namespace CapacityReservation {
    const filterSensitiveLog: (obj: CapacityReservation) => any;
    const isa: (o: any) => o is CapacityReservation;
}
export declare enum CapacityReservationInstancePlatform {
    LINUX_UNIX = "Linux/UNIX",
    LINUX_WITH_SQL_SERVER_ENTERPRISE = "Linux with SQL Server Enterprise",
    LINUX_WITH_SQL_SERVER_STANDARD = "Linux with SQL Server Standard",
    LINUX_WITH_SQL_SERVER_WEB = "Linux with SQL Server Web",
    RED_HAT_ENTERPRISE_LINUX = "Red Hat Enterprise Linux",
    SUSE_LINUX = "SUSE Linux",
    WINDOWS = "Windows",
    WINDOWS_WITH_SQL_SERVER = "Windows with SQL Server",
    WINDOWS_WITH_SQL_SERVER_ENTERPRISE = "Windows with SQL Server Enterprise",
    WINDOWS_WITH_SQL_SERVER_STANDARD = "Windows with SQL Server Standard",
    WINDOWS_WITH_SQL_SERVER_WEB = "Windows with SQL Server Web"
}
/**
 * <p>Describes the strategy for using unused Capacity Reservations for fulfilling On-Demand
 *          capacity.</p>
 *          <note>
 *             <p>This strategy can only be used if the EC2 Fleet is of type
 *             <code>instant</code>.</p>
 *          </note>
 *          <p>For more information about Capacity Reservations, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-capacity-reservations.html">On-Demand Capacity
 *             Reservations</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. For examples of using
 *          Capacity Reservations in an EC2 Fleet, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-fleet-examples.html">EC2 Fleet Example
 *             Configurations</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
 */
export interface CapacityReservationOptions {
    __type?: "CapacityReservationOptions";
    /**
     * <p>Indicates whether to use unused Capacity Reservations for fulfilling On-Demand capacity.</p>
     *          <p>If you specify <code>use-capacity-reservations-first</code>, the fleet uses unused
     *          Capacity Reservations to fulfill On-Demand capacity up to the target On-Demand capacity. If
     *          multiple instance pools have unused Capacity Reservations, the On-Demand allocation
     *          strategy (<code>lowest-price</code> or <code>prioritized</code>) is applied. If the number
     *          of unused Capacity Reservations is less than the On-Demand target capacity, the remaining
     *          On-Demand target capacity is launched according to the On-Demand allocation strategy
     *             (<code>lowest-price</code> or <code>prioritized</code>).</p>
     *          <p>If you do not specify a value, the fleet fulfils the On-Demand capacity according to the
     *          chosen On-Demand allocation strategy.</p>
     */
    UsageStrategy?: FleetCapacityReservationUsageStrategy | string;
}
export declare namespace CapacityReservationOptions {
    const filterSensitiveLog: (obj: CapacityReservationOptions) => any;
    const isa: (o: any) => o is CapacityReservationOptions;
}
/**
 * <p>Describes the strategy for using unused Capacity Reservations for fulfilling On-Demand
 *          capacity.</p>
 *          <note>
 *             <p>This strategy can only be used if the EC2 Fleet is of type <code>instant</code>.</p>
 *          </note>
 *          <p>For more information about Capacity Reservations, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-capacity-reservations.html">On-Demand Capacity
 *             Reservations</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>. For examples of using
 *          Capacity Reservations in an EC2 Fleet, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-fleet-examples.html">EC2 Fleet Example
 *             Configurations</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
 */
export interface CapacityReservationOptionsRequest {
    __type?: "CapacityReservationOptionsRequest";
    /**
     * <p>Indicates whether to use unused Capacity Reservations for fulfilling On-Demand capacity.</p>
     *          <p>If you specify <code>use-capacity-reservations-first</code>, the fleet uses unused
     *          Capacity Reservations to fulfill On-Demand capacity up to the target On-Demand capacity. If
     *          multiple instance pools have unused Capacity Reservations, the On-Demand allocation
     *          strategy (<code>lowest-price</code> or <code>prioritized</code>) is applied. If the number
     *          of unused Capacity Reservations is less than the On-Demand target capacity, the remaining
     *          On-Demand target capacity is launched according to the On-Demand allocation strategy
     *             (<code>lowest-price</code> or <code>prioritized</code>).</p>
     *          <p>If you do not specify a value, the fleet fulfils the On-Demand capacity according to the
     *          chosen On-Demand allocation strategy.</p>
     */
    UsageStrategy?: FleetCapacityReservationUsageStrategy | string;
}
export declare namespace CapacityReservationOptionsRequest {
    const filterSensitiveLog: (obj: CapacityReservationOptionsRequest) => any;
    const isa: (o: any) => o is CapacityReservationOptionsRequest;
}
export declare type CapacityReservationPreference = "none" | "open";
/**
 * <p>Describes an instance's Capacity Reservation targeting option. You can specify only one parameter
 * 			at a time. If you specify <code>CapacityReservationPreference</code> and
 * 			<code>CapacityReservationTarget</code>, the request fails.</p>
 * 			      <p>Use the <code>CapacityReservationPreference</code> parameter to configure the instance
 * 			to run as an On-Demand Instance or to run in any <code>open</code> Capacity Reservation that has
 * 			matching attributes (instance type, platform, Availability Zone). Use the
 * 			<code>CapacityReservationTarget</code> parameter to explicitly target a specific
 * 			Capacity Reservation.</p>
 */
export interface CapacityReservationSpecification {
    __type?: "CapacityReservationSpecification";
    /**
     * <p>Indicates the instance's Capacity Reservation preferences. Possible preferences include:</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>open</code> - The instance can run in any <code>open</code> Capacity Reservation that has matching attributes
     * 				(instance type, platform, Availability Zone).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>none</code> - The instance avoids running in a Capacity Reservation even if one is available. The
     * 					instance runs as an On-Demand Instance.</p>
     *             </li>
     *          </ul>
     */
    CapacityReservationPreference?: CapacityReservationPreference | string;
    /**
     * <p>Information about the target Capacity Reservation.</p>
     */
    CapacityReservationTarget?: CapacityReservationTarget;
}
export declare namespace CapacityReservationSpecification {
    const filterSensitiveLog: (obj: CapacityReservationSpecification) => any;
    const isa: (o: any) => o is CapacityReservationSpecification;
}
/**
 * <p>Describes the instance's Capacity Reservation targeting preferences. The action returns the
 *                 <code>capacityReservationPreference</code> response element if the instance is
 *             configured to run in On-Demand capacity, or if it is configured in run in any
 *                 <code>open</code> Capacity Reservation that has matching attributes (instance type, platform,
 *             Availability Zone). The action returns the <code>capacityReservationTarget</code>
 *             response element if the instance explicily targets a specific Capacity Reservation.</p>
 */
export interface CapacityReservationSpecificationResponse {
    __type?: "CapacityReservationSpecificationResponse";
    /**
     * <p>Describes the instance's Capacity Reservation preferences. Possible preferences include:</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>open</code> - The instance can run in any <code>open</code> Capacity Reservation that
     *                     has matching attributes (instance type, platform, Availability Zone).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>none</code> - The instance avoids running in a Capacity Reservation even if one is
     *                     available. The instance runs in On-Demand capacity.</p>
     *             </li>
     *          </ul>
     */
    CapacityReservationPreference?: CapacityReservationPreference | string;
    /**
     * <p>Information about the targeted Capacity Reservation.</p>
     */
    CapacityReservationTarget?: CapacityReservationTargetResponse;
}
export declare namespace CapacityReservationSpecificationResponse {
    const filterSensitiveLog: (obj: CapacityReservationSpecificationResponse) => any;
    const isa: (o: any) => o is CapacityReservationSpecificationResponse;
}
export declare type CapacityReservationState = "active" | "cancelled" | "expired" | "failed" | "pending";
/**
 * <p>Describes a target Capacity Reservation.</p>
 */
export interface CapacityReservationTarget {
    __type?: "CapacityReservationTarget";
    /**
     * <p>The ID of the Capacity Reservation.</p>
     */
    CapacityReservationId?: string;
}
export declare namespace CapacityReservationTarget {
    const filterSensitiveLog: (obj: CapacityReservationTarget) => any;
    const isa: (o: any) => o is CapacityReservationTarget;
}
/**
 * <p>Describes a target Capacity Reservation.</p>
 */
export interface CapacityReservationTargetResponse {
    __type?: "CapacityReservationTargetResponse";
    /**
     * <p>The ID of the Capacity Reservation.</p>
     */
    CapacityReservationId?: string;
}
export declare namespace CapacityReservationTargetResponse {
    const filterSensitiveLog: (obj: CapacityReservationTargetResponse) => any;
    const isa: (o: any) => o is CapacityReservationTargetResponse;
}
export declare type CapacityReservationTenancy = "dedicated" | "default";
/**
 * <p>Information about the client certificate used for authentication.</p>
 */
export interface CertificateAuthentication {
    __type?: "CertificateAuthentication";
    /**
     * <p>The ARN of the client certificate. </p>
     */
    ClientRootCertificateChain?: string;
}
export declare namespace CertificateAuthentication {
    const filterSensitiveLog: (obj: CertificateAuthentication) => any;
    const isa: (o: any) => o is CertificateAuthentication;
}
/**
 * <p>Information about the client certificate to be used for authentication.</p>
 */
export interface CertificateAuthenticationRequest {
    __type?: "CertificateAuthenticationRequest";
    /**
     * <p>The ARN of the client certificate. The certificate must be signed by a certificate
     * 			authority (CA) and it must be provisioned in AWS Certificate Manager (ACM).</p>
     */
    ClientRootCertificateChainArn?: string;
}
export declare namespace CertificateAuthenticationRequest {
    const filterSensitiveLog: (obj: CertificateAuthenticationRequest) => any;
    const isa: (o: any) => o is CertificateAuthenticationRequest;
}
/**
 * <p>Provides authorization for Amazon to bring a specific IP address range to a specific
 *          AWS account using bring your own IP addresses (BYOIP). For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-byoip.html#prepare-for-byoip">Prepare to Bring Your Address Range to Your AWS Account</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
 */
export interface CidrAuthorizationContext {
    __type?: "CidrAuthorizationContext";
    /**
     * <p>The plain-text authorization message for the prefix and account.</p>
     */
    Message: string | undefined;
    /**
     * <p>The signed authorization message for the prefix and account.</p>
     */
    Signature: string | undefined;
}
export declare namespace CidrAuthorizationContext {
    const filterSensitiveLog: (obj: CidrAuthorizationContext) => any;
    const isa: (o: any) => o is CidrAuthorizationContext;
}
/**
 * <p>Describes an IPv4 CIDR block.</p>
 */
export interface CidrBlock {
    __type?: "CidrBlock";
    /**
     * <p>The IPv4 CIDR block.</p>
     */
    CidrBlock?: string;
}
export declare namespace CidrBlock {
    const filterSensitiveLog: (obj: CidrBlock) => any;
    const isa: (o: any) => o is CidrBlock;
}
/**
 * <p>Describes the ClassicLink DNS support status of a VPC.</p>
 */
export interface ClassicLinkDnsSupport {
    __type?: "ClassicLinkDnsSupport";
    /**
     * <p>Indicates whether ClassicLink DNS support is enabled for the VPC.</p>
     */
    ClassicLinkDnsSupported?: boolean;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace ClassicLinkDnsSupport {
    const filterSensitiveLog: (obj: ClassicLinkDnsSupport) => any;
    const isa: (o: any) => o is ClassicLinkDnsSupport;
}
/**
 * <p>Describes a linked EC2-Classic instance.</p>
 */
export interface ClassicLinkInstance {
    __type?: "ClassicLinkInstance";
    /**
     * <p>A list of security groups.</p>
     */
    Groups?: GroupIdentifier[];
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>Any tags assigned to the instance.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace ClassicLinkInstance {
    const filterSensitiveLog: (obj: ClassicLinkInstance) => any;
    const isa: (o: any) => o is ClassicLinkInstance;
}
/**
 * <p>Describes a Classic Load Balancer.</p>
 */
export interface ClassicLoadBalancer {
    __type?: "ClassicLoadBalancer";
    /**
     * <p>The name of the load balancer.</p>
     */
    Name?: string;
}
export declare namespace ClassicLoadBalancer {
    const filterSensitiveLog: (obj: ClassicLoadBalancer) => any;
    const isa: (o: any) => o is ClassicLoadBalancer;
}
/**
 * <p>Describes the Classic Load Balancers to attach to a Spot Fleet. Spot Fleet registers the running Spot Instances
 *             with these Classic Load Balancers.</p>
 */
export interface ClassicLoadBalancersConfig {
    __type?: "ClassicLoadBalancersConfig";
    /**
     * <p>One or more Classic Load Balancers.</p>
     */
    ClassicLoadBalancers?: ClassicLoadBalancer[];
}
export declare namespace ClassicLoadBalancersConfig {
    const filterSensitiveLog: (obj: ClassicLoadBalancersConfig) => any;
    const isa: (o: any) => o is ClassicLoadBalancersConfig;
}
/**
 * <p>Describes the state of a client certificate revocation list.</p>
 */
export interface ClientCertificateRevocationListStatus {
    __type?: "ClientCertificateRevocationListStatus";
    /**
     * <p>The state of the client certificate revocation list.</p>
     */
    Code?: ClientCertificateRevocationListStatusCode | string;
    /**
     * <p>A message about the status of the client certificate revocation list, if applicable.</p>
     */
    Message?: string;
}
export declare namespace ClientCertificateRevocationListStatus {
    const filterSensitiveLog: (obj: ClientCertificateRevocationListStatus) => any;
    const isa: (o: any) => o is ClientCertificateRevocationListStatus;
}
export declare type ClientCertificateRevocationListStatusCode = "active" | "pending";
/**
 * <p>Describes the client-specific data.</p>
 */
export interface ClientData {
    __type?: "ClientData";
    /**
     * <p>A user-defined comment about the disk upload.</p>
     */
    Comment?: string;
    /**
     * <p>The time that the disk upload ends.</p>
     */
    UploadEnd?: Date;
    /**
     * <p>The size of the uploaded disk image, in GiB.</p>
     */
    UploadSize?: number;
    /**
     * <p>The time that the disk upload starts.</p>
     */
    UploadStart?: Date;
}
export declare namespace ClientData {
    const filterSensitiveLog: (obj: ClientData) => any;
    const isa: (o: any) => o is ClientData;
}
/**
 * <p>Describes the authentication methods used by a Client VPN endpoint. Client VPN supports
 * 			Active Directory and mutual authentication. For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/clientvpn-admin/authentication-authrization.html#client-authentication">Authentication</a>
 * 			in the <i>AWS Client VPN Administrator Guide</i>.</p>
 */
export interface ClientVpnAuthentication {
    __type?: "ClientVpnAuthentication";
    /**
     * <p>Information about the Active Directory, if applicable.</p>
     */
    ActiveDirectory?: DirectoryServiceAuthentication;
    /**
     * <p>Information about the authentication certificates, if applicable.</p>
     */
    MutualAuthentication?: CertificateAuthentication;
    /**
     * <p>The authentication type used.</p>
     */
    Type?: ClientVpnAuthenticationType | string;
}
export declare namespace ClientVpnAuthentication {
    const filterSensitiveLog: (obj: ClientVpnAuthentication) => any;
    const isa: (o: any) => o is ClientVpnAuthentication;
}
/**
 * <p>Describes the authentication method to be used by a Client VPN endpoint. Client VPN supports
 * 			Active Directory and mutual authentication. For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/clientvpn-admin/authentication-authrization.html#client-authentication">Authentication</a>
 * 			in the <i>AWS Client VPN Administrator Guide</i>.</p>
 */
export interface ClientVpnAuthenticationRequest {
    __type?: "ClientVpnAuthenticationRequest";
    /**
     * <p>Information about the Active Directory to be used, if applicable. You must provide this information if <b>Type</b> is <code>directory-service-authentication</code>.</p>
     */
    ActiveDirectory?: DirectoryServiceAuthenticationRequest;
    /**
     * <p>Information about the authentication certificates to be used, if applicable. You must provide this information if <b>Type</b> is <code>certificate-authentication</code>.</p>
     */
    MutualAuthentication?: CertificateAuthenticationRequest;
    /**
     * <p>The type of client authentication to be used. Specify <code>certificate-authentication</code> to use certificate-based authentication, or
     * 			<code>directory-service-authentication</code> to use Active Directory authentication.</p>
     */
    Type?: ClientVpnAuthenticationType | string;
}
export declare namespace ClientVpnAuthenticationRequest {
    const filterSensitiveLog: (obj: ClientVpnAuthenticationRequest) => any;
    const isa: (o: any) => o is ClientVpnAuthenticationRequest;
}
export declare type ClientVpnAuthenticationType = "certificate-authentication" | "directory-service-authentication";
/**
 * <p>Describes the state of an authorization rule.</p>
 */
export interface ClientVpnAuthorizationRuleStatus {
    __type?: "ClientVpnAuthorizationRuleStatus";
    /**
     * <p>The state of the authorization rule.</p>
     */
    Code?: ClientVpnAuthorizationRuleStatusCode | string;
    /**
     * <p>A message about the status of the authorization rule, if applicable.</p>
     */
    Message?: string;
}
export declare namespace ClientVpnAuthorizationRuleStatus {
    const filterSensitiveLog: (obj: ClientVpnAuthorizationRuleStatus) => any;
    const isa: (o: any) => o is ClientVpnAuthorizationRuleStatus;
}
export declare type ClientVpnAuthorizationRuleStatusCode = "active" | "authorizing" | "failed" | "revoking";
/**
 * <p>Describes a client connection.</p>
 */
export interface ClientVpnConnection {
    __type?: "ClientVpnConnection";
    /**
     * <p>The IP address of the client.</p>
     */
    ClientIp?: string;
    /**
     * <p>The ID of the Client VPN endpoint to which the client is connected.</p>
     */
    ClientVpnEndpointId?: string;
    /**
     * <p>The common name associated with the client. This is either the name of the client certificate,
     * 			or the Active Directory user name.</p>
     */
    CommonName?: string;
    /**
     * <p>The date and time the client connection was terminated.</p>
     */
    ConnectionEndTime?: string;
    /**
     * <p>The date and time the client connection was established.</p>
     */
    ConnectionEstablishedTime?: string;
    /**
     * <p>The ID of the client connection.</p>
     */
    ConnectionId?: string;
    /**
     * <p>The number of bytes received by the client.</p>
     */
    EgressBytes?: string;
    /**
     * <p>The number of packets received by the client.</p>
     */
    EgressPackets?: string;
    /**
     * <p>The number of bytes sent by the client.</p>
     */
    IngressBytes?: string;
    /**
     * <p>The number of packets sent by the client.</p>
     */
    IngressPackets?: string;
    /**
     * <p>The current state of the client connection.</p>
     */
    Status?: ClientVpnConnectionStatus;
    /**
     * <p>The current date and time.</p>
     */
    Timestamp?: string;
    /**
     * <p>The username of the client who established the client connection. This information is only provided
     * 			if Active Directory client authentication is used.</p>
     */
    Username?: string;
}
export declare namespace ClientVpnConnection {
    const filterSensitiveLog: (obj: ClientVpnConnection) => any;
    const isa: (o: any) => o is ClientVpnConnection;
}
/**
 * <p>Describes the status of a client connection.</p>
 */
export interface ClientVpnConnectionStatus {
    __type?: "ClientVpnConnectionStatus";
    /**
     * <p>The state of the client connection.</p>
     */
    Code?: ClientVpnConnectionStatusCode | string;
    /**
     * <p>A message about the status of the client connection, if applicable.</p>
     */
    Message?: string;
}
export declare namespace ClientVpnConnectionStatus {
    const filterSensitiveLog: (obj: ClientVpnConnectionStatus) => any;
    const isa: (o: any) => o is ClientVpnConnectionStatus;
}
export declare type ClientVpnConnectionStatusCode = "active" | "failed-to-terminate" | "terminated" | "terminating";
/**
 * <p>Describes a Client VPN endpoint.</p>
 */
export interface ClientVpnEndpoint {
    __type?: "ClientVpnEndpoint";
    /**
     * <p>Information about the associated target networks. A target network is a subnet in a VPC.</p>
     */
    AssociatedTargetNetworks?: AssociatedTargetNetwork[];
    /**
     * <p>Information about the authentication method used by the Client VPN endpoint.</p>
     */
    AuthenticationOptions?: ClientVpnAuthentication[];
    /**
     * <p>The IPv4 address range, in CIDR notation, from which client IP addresses are assigned.</p>
     */
    ClientCidrBlock?: string;
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId?: string;
    /**
     * <p>Information about the client connection logging options for the Client VPN endpoint.</p>
     */
    ConnectionLogOptions?: ConnectionLogResponseOptions;
    /**
     * <p>The date and time the Client VPN endpoint was created.</p>
     */
    CreationTime?: string;
    /**
     * <p>The date and time the Client VPN endpoint was deleted, if applicable.</p>
     */
    DeletionTime?: string;
    /**
     * <p>A brief description of the endpoint.</p>
     */
    Description?: string;
    /**
     * <p>The DNS name to be used by clients when connecting to the Client VPN endpoint.</p>
     */
    DnsName?: string;
    /**
     * <p>Information about the DNS servers to be used for DNS resolution. </p>
     */
    DnsServers?: string[];
    /**
     * <p>The ARN of the server certificate.</p>
     */
    ServerCertificateArn?: string;
    /**
     * <p>Indicates whether split-tunnel is enabled in the AWS Client VPN endpoint.</p>
     * 		       <p>For information about split-tunnel VPN endpoints, see <a href="https://docs.aws.amazon.com/vpn/latest/clientvpn-admin/split-tunnel-vpn.html">Split-Tunnel AWS Client VPN Endpoint</a> in the <i>AWS
     * 				Client VPN Administrator Guide</i>.</p>
     */
    SplitTunnel?: boolean;
    /**
     * <p>The current state of the Client VPN endpoint.</p>
     */
    Status?: ClientVpnEndpointStatus;
    /**
     * <p>Any tags assigned to the Client VPN endpoint.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The transport protocol used by the Client VPN endpoint.</p>
     */
    TransportProtocol?: TransportProtocol | string;
    /**
     * <p>The port number for the  Client VPN endpoint.</p>
     */
    VpnPort?: number;
    /**
     * <p>The protocol used by the VPN session.</p>
     */
    VpnProtocol?: VpnProtocol | string;
}
export declare namespace ClientVpnEndpoint {
    const filterSensitiveLog: (obj: ClientVpnEndpoint) => any;
    const isa: (o: any) => o is ClientVpnEndpoint;
}
/**
 * <p>Describes the state of a Client VPN endpoint.</p>
 */
export interface ClientVpnEndpointStatus {
    __type?: "ClientVpnEndpointStatus";
    /**
     * <p>The state of the Client VPN endpoint. Possible states include:</p>
     * 		       <ul>
     *             <li>
     * 				           <p>
     *                   <code>pending-associate</code> - The Client VPN endpoint has been created but no target networks
     * 					have been associated. The Client VPN endpoint cannot accept connections.</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>available</code> - The Client VPN endpoint has been created and a target network has been
     * 					associated. The Client VPN endpoint can accept connections.</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>deleting</code> - The Client VPN endpoint is being deleted. The Client VPN endpoint cannot accept
     * 					connections.</p>
     *             </li>
     *             <li>
     * 				           <p>
     *                   <code>deleted</code> - The Client VPN endpoint has been deleted. The Client VPN endpoint cannot accept
     * 					connections.</p>
     * 			         </li>
     *          </ul>
     */
    Code?: ClientVpnEndpointStatusCode | string;
    /**
     * <p>A message about the status of the Client VPN endpoint.</p>
     */
    Message?: string;
}
export declare namespace ClientVpnEndpointStatus {
    const filterSensitiveLog: (obj: ClientVpnEndpointStatus) => any;
    const isa: (o: any) => o is ClientVpnEndpointStatus;
}
export declare type ClientVpnEndpointStatusCode = "available" | "deleted" | "deleting" | "pending-associate";
/**
 * <p>Information about a Client VPN endpoint route.</p>
 */
export interface ClientVpnRoute {
    __type?: "ClientVpnRoute";
    /**
     * <p>The ID of the Client VPN endpoint with which the route is associated.</p>
     */
    ClientVpnEndpointId?: string;
    /**
     * <p>A brief description of the route.</p>
     */
    Description?: string;
    /**
     * <p>The IPv4 address range, in CIDR notation, of the route destination.</p>
     */
    DestinationCidr?: string;
    /**
     * <p>Indicates how the route was associated with the Client VPN endpoint.
     * 			<code>associate</code> indicates that the route was automatically added when the target network
     * 			was associated with the Client VPN endpoint. <code>add-route</code> indicates that the route
     * 			was manually added using the <b>CreateClientVpnRoute</b> action.</p>
     */
    Origin?: string;
    /**
     * <p>The current state of the route.</p>
     */
    Status?: ClientVpnRouteStatus;
    /**
     * <p>The ID of the subnet through which traffic is routed.</p>
     */
    TargetSubnet?: string;
    /**
     * <p>The route type.</p>
     */
    Type?: string;
}
export declare namespace ClientVpnRoute {
    const filterSensitiveLog: (obj: ClientVpnRoute) => any;
    const isa: (o: any) => o is ClientVpnRoute;
}
/**
 * <p>Describes the state of a Client VPN endpoint route.</p>
 */
export interface ClientVpnRouteStatus {
    __type?: "ClientVpnRouteStatus";
    /**
     * <p>The state of the Client VPN endpoint route.</p>
     */
    Code?: ClientVpnRouteStatusCode | string;
    /**
     * <p>A message about the status of the Client VPN endpoint route, if applicable.</p>
     */
    Message?: string;
}
export declare namespace ClientVpnRouteStatus {
    const filterSensitiveLog: (obj: ClientVpnRouteStatus) => any;
    const isa: (o: any) => o is ClientVpnRouteStatus;
}
export declare type ClientVpnRouteStatusCode = "active" | "creating" | "deleting" | "failed";
/**
 * <p>Describes address usage for a customer-owned address pool.</p>
 */
export interface CoipAddressUsage {
    __type?: "CoipAddressUsage";
    /**
     * <p>The allocation ID of the address.</p>
     */
    AllocationId?: string;
    /**
     * <p>The AWS account ID.</p>
     */
    AwsAccountId?: string;
    /**
     * <p>The AWS service.</p>
     */
    AwsService?: string;
    /**
     * <p>The customer-owned IP address.</p>
     */
    CoIp?: string;
}
export declare namespace CoipAddressUsage {
    const filterSensitiveLog: (obj: CoipAddressUsage) => any;
    const isa: (o: any) => o is CoipAddressUsage;
}
/**
 * <p>Describes a customer-owned address pool.</p>
 */
export interface CoipPool {
    __type?: "CoipPool";
    /**
     * <p>The ID of the local gateway route table.</p>
     */
    LocalGatewayRouteTableId?: string;
    /**
     * <p>The address ranges of the address pool.</p>
     */
    PoolCidrs?: string[];
    /**
     * <p>The ID of the address pool.</p>
     */
    PoolId?: string;
    /**
     * <p>The tags.</p>
     */
    Tags?: Tag[];
}
export declare namespace CoipPool {
    const filterSensitiveLog: (obj: CoipPool) => any;
    const isa: (o: any) => o is CoipPool;
}
export interface ConfirmProductInstanceRequest {
    __type?: "ConfirmProductInstanceRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>The product code. This must be a product code that you own.</p>
     */
    ProductCode: string | undefined;
}
export declare namespace ConfirmProductInstanceRequest {
    const filterSensitiveLog: (obj: ConfirmProductInstanceRequest) => any;
    const isa: (o: any) => o is ConfirmProductInstanceRequest;
}
export interface ConfirmProductInstanceResult {
    __type?: "ConfirmProductInstanceResult";
    /**
     * <p>The AWS account ID of the instance owner. This is only present if the product code is
     *             attached to the instance.</p>
     */
    OwnerId?: string;
    /**
     * <p>The return value of the request. Returns <code>true</code> if the specified product
     *             code is owned by the requester and associated with the specified instance.</p>
     */
    Return?: boolean;
}
export declare namespace ConfirmProductInstanceResult {
    const filterSensitiveLog: (obj: ConfirmProductInstanceResult) => any;
    const isa: (o: any) => o is ConfirmProductInstanceResult;
}
/**
 * <p>Describes the client connection logging options for the Client VPN endpoint.</p>
 */
export interface ConnectionLogOptions {
    __type?: "ConnectionLogOptions";
    /**
     * <p>The name of the CloudWatch Logs log group.</p>
     */
    CloudwatchLogGroup?: string;
    /**
     * <p>The name of the CloudWatch Logs log stream to which the connection data is published.</p>
     */
    CloudwatchLogStream?: string;
    /**
     * <p>Indicates whether connection logging is enabled.</p>
     */
    Enabled?: boolean;
}
export declare namespace ConnectionLogOptions {
    const filterSensitiveLog: (obj: ConnectionLogOptions) => any;
    const isa: (o: any) => o is ConnectionLogOptions;
}
/**
 * <p>Information about the client connection logging options for a Client VPN endpoint.</p>
 */
export interface ConnectionLogResponseOptions {
    __type?: "ConnectionLogResponseOptions";
    /**
     * <p>The name of the Amazon CloudWatch Logs log group to which connection logging data is published.</p>
     */
    CloudwatchLogGroup?: string;
    /**
     * <p>The name of the Amazon CloudWatch Logs log stream to which connection logging data is published.</p>
     */
    CloudwatchLogStream?: string;
    /**
     * <p>Indicates whether client connection logging is enabled for the Client VPN endpoint.</p>
     */
    Enabled?: boolean;
}
export declare namespace ConnectionLogResponseOptions {
    const filterSensitiveLog: (obj: ConnectionLogResponseOptions) => any;
    const isa: (o: any) => o is ConnectionLogResponseOptions;
}
/**
 * <p>Describes a connection notification for a VPC endpoint or VPC endpoint
 *             service.</p>
 */
export interface ConnectionNotification {
    __type?: "ConnectionNotification";
    /**
     * <p>The events for the notification. Valid values are <code>Accept</code>,
     *                 <code>Connect</code>, <code>Delete</code>, and <code>Reject</code>.</p>
     */
    ConnectionEvents?: string[];
    /**
     * <p>The ARN of the SNS topic for the notification.</p>
     */
    ConnectionNotificationArn?: string;
    /**
     * <p>The ID of the notification.</p>
     */
    ConnectionNotificationId?: string;
    /**
     * <p>The state of the notification.</p>
     */
    ConnectionNotificationState?: ConnectionNotificationState | string;
    /**
     * <p>The type of notification.</p>
     */
    ConnectionNotificationType?: ConnectionNotificationType | string;
    /**
     * <p>The ID of the endpoint service.</p>
     */
    ServiceId?: string;
    /**
     * <p>The ID of the VPC endpoint.</p>
     */
    VpcEndpointId?: string;
}
export declare namespace ConnectionNotification {
    const filterSensitiveLog: (obj: ConnectionNotification) => any;
    const isa: (o: any) => o is ConnectionNotification;
}
export declare enum ConnectionNotificationState {
    Disabled = "Disabled",
    Enabled = "Enabled"
}
export declare enum ConnectionNotificationType {
    Topic = "Topic"
}
export declare type ContainerFormat = "ova";
/**
 * <p>Describes a conversion task.</p>
 */
export interface ConversionTask {
    __type?: "ConversionTask";
    /**
     * <p>The ID of the conversion task.</p>
     */
    ConversionTaskId?: string;
    /**
     * <p>The time when the task expires. If the upload isn't complete before the expiration time, we automatically cancel
     *    the task.</p>
     */
    ExpirationTime?: string;
    /**
     * <p>If the task is for importing an instance, this contains information about the import instance task.</p>
     */
    ImportInstance?: ImportInstanceTaskDetails;
    /**
     * <p>If the task is for importing a volume, this contains information about the import volume task.</p>
     */
    ImportVolume?: ImportVolumeTaskDetails;
    /**
     * <p>The state of the conversion task.</p>
     */
    State?: ConversionTaskState | string;
    /**
     * <p>The status message related to the conversion task.</p>
     */
    StatusMessage?: string;
    /**
     * <p>Any tags assigned to the task.</p>
     */
    Tags?: Tag[];
}
export declare namespace ConversionTask {
    const filterSensitiveLog: (obj: ConversionTask) => any;
    const isa: (o: any) => o is ConversionTask;
}
export declare type ConversionTaskState = "active" | "cancelled" | "cancelling" | "completed";
export interface CopyFpgaImageRequest {
    __type?: "CopyFpgaImageRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.
     *          For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The description for the new AFI.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The name for the new AFI. The default is the name of the source AFI.</p>
     */
    Name?: string;
    /**
     * <p>The ID of the source AFI.</p>
     */
    SourceFpgaImageId: string | undefined;
    /**
     * <p>The Region that contains the source AFI.</p>
     */
    SourceRegion: string | undefined;
}
export declare namespace CopyFpgaImageRequest {
    const filterSensitiveLog: (obj: CopyFpgaImageRequest) => any;
    const isa: (o: any) => o is CopyFpgaImageRequest;
}
export interface CopyFpgaImageResult {
    __type?: "CopyFpgaImageResult";
    /**
     * <p>The ID of the new AFI.</p>
     */
    FpgaImageId?: string;
}
export declare namespace CopyFpgaImageResult {
    const filterSensitiveLog: (obj: CopyFpgaImageResult) => any;
    const isa: (o: any) => o is CopyFpgaImageResult;
}
/**
 * <p>Contains the parameters for CopyImage.</p>
 */
export interface CopyImageRequest {
    __type?: "CopyImageRequest";
    /**
     * <p>Unique, case-sensitive identifier you provide to ensure
     * 				idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">How to Ensure Idempotency</a> in the
     * 			  <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    ClientToken?: string;
    /**
     * <p>A description for the new AMI in the destination Region.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Specifies whether the destination snapshots of the copied image should be encrypted.
     *        You can encrypt a copy of an unencrypted snapshot, but you cannot create an unencrypted
     *        copy of an encrypted snapshot. The default CMK for EBS is used unless you specify a non-default
     *        AWS Key Management Service (AWS KMS) CMK using <code>KmsKeyId</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>An identifier for the symmetric AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating
     *       the encrypted volume. This parameter is only required if you want to use a non-default CMK; if
     *       this parameter is not specified, the default CMK for EBS is used. If a <code>KmsKeyId</code>
     *       is specified, the <code>Encrypted</code> flag must also be set. </p>
     *    	     <p>To specify a CMK, use its key ID, Amazon Resource Name (ARN), alias name, or alias ARN.
     *    		When using an alias name, prefix it with "alias/". For example:</p>
     *    	     <ul>
     *             <li>
     *    			         <p>Key ID: <code>1234abcd-12ab-34cd-56ef-1234567890ab</code>
     *                </p>
     *    		       </li>
     *             <li>
     *    			         <p>Key ARN: <code>arn:aws:kms:us-east-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab</code>
     *                </p>
     *    		       </li>
     *             <li>
     *    			         <p>Alias name: <code>alias/ExampleAlias</code>
     *                </p>
     *    		       </li>
     *             <li>
     *    			         <p>Alias ARN: <code>arn:aws:kms:us-east-2:111122223333:alias/ExampleAlias</code>
     *                </p>
     *    		       </li>
     *          </ul>
     *          <p>AWS parses <code>KmsKeyId</code> asynchronously, meaning that the action you call may
     *       appear to complete even though you provided an invalid identifier. This action will eventually
     *       report failure. </p>
     *          <p>The specified CMK must exist in the Region that the snapshot is being copied to. </p>
     *          <p>Amazon EBS does not support asymmetric CMKs.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>The name of the new AMI in the destination Region.</p>
     */
    Name: string | undefined;
    /**
     * <p>The ID of the AMI to copy.</p>
     */
    SourceImageId: string | undefined;
    /**
     * <p>The name of the Region that contains the AMI to copy.</p>
     */
    SourceRegion: string | undefined;
}
export declare namespace CopyImageRequest {
    const filterSensitiveLog: (obj: CopyImageRequest) => any;
    const isa: (o: any) => o is CopyImageRequest;
}
/**
 * <p>Contains the output of CopyImage.</p>
 */
export interface CopyImageResult {
    __type?: "CopyImageResult";
    /**
     * <p>The ID of the new AMI.</p>
     */
    ImageId?: string;
}
export declare namespace CopyImageResult {
    const filterSensitiveLog: (obj: CopyImageResult) => any;
    const isa: (o: any) => o is CopyImageResult;
}
export interface CopySnapshotRequest {
    __type?: "CopySnapshotRequest";
    /**
     * <p>A description for the EBS snapshot.</p>
     */
    Description?: string;
    /**
     * <p>The destination Region to use in the <code>PresignedUrl</code> parameter of a snapshot
     *       copy operation. This parameter is only valid for specifying the destination Region in a
     *         <code>PresignedUrl</code> parameter, where it is required.</p>
     *
     *          <p>The snapshot copy is sent to the regional endpoint that you sent the HTTP
     *       request to (for example, <code>ec2.us-east-1.amazonaws.com</code>). With the AWS CLI, this is
     *       specified using the <code>--region</code> parameter or the default Region in your AWS
     *       configuration file.</p>
     */
    DestinationRegion?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>To encrypt a copy of an unencrypted snapshot if encryption by default is not enabled,
     *       enable encryption using this parameter. Otherwise, omit this parameter. Encrypted snapshots
     *       are encrypted, even if you omit this parameter and encryption by default is not enabled. You
     *       cannot set this parameter to false. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the
     *       <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>The identifier of the AWS Key Management Service (AWS KMS) customer master key (CMK) to use for Amazon EBS encryption.
     *       If this parameter is not specified, your AWS managed CMK for EBS is used. If <code>KmsKeyId</code> is
     *       specified, the encrypted state must be <code>true</code>.</p>
     *          <p>You can specify the CMK using any of the following:</p>
     *          <ul>
     *             <li>
     *                <p>Key ID. For example, key/1234abcd-12ab-34cd-56ef-1234567890ab.</p>
     *             </li>
     *             <li>
     *                <p>Key alias. For example, alias/ExampleAlias.</p>
     *             </li>
     *             <li>
     *                <p>Key ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:key/<i>abcd1234-a123-456a-a12b-a123b4cd56ef</i>.</p>
     *             </li>
     *             <li>
     *                <p>Alias ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>.</p>
     *             </li>
     *          </ul>
     *          <p>AWS authenticates the CMK asynchronously. Therefore, if you specify an ID, alias, or ARN that is not valid,
     *       the action can appear to complete, but eventually fails.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>When you copy an encrypted source snapshot using the Amazon EC2 Query API, you must supply a
     *       pre-signed URL. This parameter is optional for unencrypted snapshots. For more information,
     *       see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html">Query
     *         Requests</a>.</p>
     *          <p>The <code>PresignedUrl</code> should use the snapshot source endpoint, the
     *         <code>CopySnapshot</code> action, and include the <code>SourceRegion</code>,
     *         <code>SourceSnapshotId</code>, and <code>DestinationRegion</code> parameters. The
     *         <code>PresignedUrl</code> must be signed using AWS Signature Version 4. Because EBS
     *       snapshots are stored in Amazon S3, the signing algorithm for this parameter uses the same logic
     *       that is described in <a href="https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-query-string-auth.html">Authenticating Requests by Using Query
     *         Parameters (AWS Signature Version 4)</a> in the <i>Amazon Simple Storage Service API Reference</i>. An
     *       invalid or improperly signed <code>PresignedUrl</code> will cause the copy operation to fail
     *       asynchronously, and the snapshot will move to an <code>error</code> state.</p>
     */
    PresignedUrl?: string;
    /**
     * <p>The ID of the Region that contains the snapshot to be copied.</p>
     */
    SourceRegion: string | undefined;
    /**
     * <p>The ID of the EBS snapshot to copy.</p>
     */
    SourceSnapshotId: string | undefined;
    /**
     * <p>The tags to apply to the new snapshot.</p>
     */
    TagSpecifications?: TagSpecification[];
}
export declare namespace CopySnapshotRequest {
    const filterSensitiveLog: (obj: CopySnapshotRequest) => any;
    const isa: (o: any) => o is CopySnapshotRequest;
}
export interface CopySnapshotResult {
    __type?: "CopySnapshotResult";
    /**
     * <p>The ID of the new snapshot.</p>
     */
    SnapshotId?: string;
    /**
     * <p>Any tags applied to the new snapshot.</p>
     */
    Tags?: Tag[];
}
export declare namespace CopySnapshotResult {
    const filterSensitiveLog: (obj: CopySnapshotResult) => any;
    const isa: (o: any) => o is CopySnapshotResult;
}
export declare type CopyTagsFromSource = "volume";
/**
 * <p>The CPU options for the instance.</p>
 */
export interface CpuOptions {
    __type?: "CpuOptions";
    /**
     * <p>The number of CPU cores for the instance.</p>
     */
    CoreCount?: number;
    /**
     * <p>The number of threads per CPU core.</p>
     */
    ThreadsPerCore?: number;
}
export declare namespace CpuOptions {
    const filterSensitiveLog: (obj: CpuOptions) => any;
    const isa: (o: any) => o is CpuOptions;
}
/**
 * <p>The CPU options for the instance. Both the core count and threads per core must be
 *             specified in the request.</p>
 */
export interface CpuOptionsRequest {
    __type?: "CpuOptionsRequest";
    /**
     * <p>The number of CPU cores for the instance.</p>
     */
    CoreCount?: number;
    /**
     * <p>The number of threads per CPU core. To disable multithreading for
     *             the instance, specify a value of <code>1</code>. Otherwise, specify the default value of
     *                 <code>2</code>.</p>
     */
    ThreadsPerCore?: number;
}
export declare namespace CpuOptionsRequest {
    const filterSensitiveLog: (obj: CpuOptionsRequest) => any;
    const isa: (o: any) => o is CpuOptionsRequest;
}
export interface CreateCapacityReservationRequest {
    __type?: "CreateCapacityReservationRequest";
    /**
     * <p>The Availability Zone in which to create the Capacity Reservation.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The ID of the Availability Zone in which to create the Capacity Reservation.</p>
     */
    AvailabilityZoneId?: string;
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     * 			request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure
     * 				Idempotency</a>.</p>
     * 		       <p>Constraint: Maximum 64 ASCII characters.</p>
     */
    ClientToken?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Indicates whether the Capacity Reservation supports EBS-optimized instances. This optimization provides
     * 			dedicated throughput to Amazon EBS and an optimized configuration stack to provide
     * 			optimal I/O performance. This optimization isn't available with all instance types.
     * 			Additional usage charges apply when using an EBS- optimized instance.</p>
     */
    EbsOptimized?: boolean;
    /**
     * <p>The date and time at which the Capacity Reservation expires. When a Capacity Reservation expires, the reserved capacity
     * 			is released and you can no longer launch instances into it. The Capacity Reservation's state changes to
     * 				<code>expired</code> when it reaches its end date and time.</p>
     * 		       <p>You must provide an <code>EndDate</code> value if <code>EndDateType</code> is
     * 				<code>limited</code>. Omit <code>EndDate</code> if <code>EndDateType</code> is
     * 				<code>unlimited</code>.</p>
     *
     * 		       <p>If the <code>EndDateType</code> is <code>limited</code>, the Capacity Reservation is cancelled within an hour from the specified time. For example, if you specify
     * 			5/31/2019, 13:30:55, the Capacity Reservation is guaranteed to end between 13:30:55 and 14:30:55 on 5/31/2019.</p>
     */
    EndDate?: Date;
    /**
     * <p>Indicates the way in which the Capacity Reservation ends. A Capacity Reservation can have one of the following end
     * 			types:</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>unlimited</code> - The Capacity Reservation remains active until you explicitly cancel it. Do not
     * 					provide an <code>EndDate</code> if the <code>EndDateType</code> is
     * 						<code>unlimited</code>.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>limited</code> - The Capacity Reservation expires automatically at a specified date and time. You must
     * 					provide an <code>EndDate</code> value if the <code>EndDateType</code> value is
     * 						<code>limited</code>.</p>
     *             </li>
     *          </ul>
     */
    EndDateType?: EndDateType | string;
    /**
     * <p>Indicates whether the Capacity Reservation supports instances with temporary, block-level
     * 			storage.</p>
     */
    EphemeralStorage?: boolean;
    /**
     * <p>The number of instances for which to reserve capacity.</p>
     */
    InstanceCount: number | undefined;
    /**
     * <p>Indicates the type of instance launches that the Capacity Reservation accepts. The options
     * 			include:</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>open</code> - The Capacity Reservation automatically matches all instances that have matching attributes (instance type, platform,
     * 				and Availability Zone). Instances that have matching attributes run in the Capacity Reservation automatically without specifying
     * 				any additional parameters.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>targeted</code> - The Capacity Reservation only accepts instances that have matching attributes
     * 					(instance type, platform, and Availability Zone), and explicitly target the
     * 					Capacity Reservation. This ensures that only permitted instances can use the reserved capacity. </p>
     *             </li>
     *          </ul>
     * 		       <p>Default: <code>open</code>
     *          </p>
     */
    InstanceMatchCriteria?: InstanceMatchCriteria | string;
    /**
     * <p>The type of operating system for which to reserve capacity.</p>
     */
    InstancePlatform: CapacityReservationInstancePlatform | string | undefined;
    /**
     * <p>The instance type for which to reserve capacity. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance
     * 				Types</a> in the <i>Amazon Elastic Compute Cloud User
     * 			Guide</i>.</p>
     */
    InstanceType: string | undefined;
    /**
     * <p>The tags to apply to the Capacity Reservation during launch.</p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>Indicates the tenancy of the Capacity Reservation. A Capacity Reservation can have one of the following tenancy settings:</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>default</code> - The Capacity Reservation is created on hardware that is shared with other AWS accounts.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>dedicated</code> - The Capacity Reservation is created on single-tenant hardware that is dedicated to a single AWS account.</p>
     *             </li>
     *          </ul>
     */
    Tenancy?: CapacityReservationTenancy | string;
}
export declare namespace CreateCapacityReservationRequest {
    const filterSensitiveLog: (obj: CreateCapacityReservationRequest) => any;
    const isa: (o: any) => o is CreateCapacityReservationRequest;
}
export interface CreateCapacityReservationResult {
    __type?: "CreateCapacityReservationResult";
    /**
     * <p>Information about the Capacity Reservation.</p>
     */
    CapacityReservation?: CapacityReservation;
}
export declare namespace CreateCapacityReservationResult {
    const filterSensitiveLog: (obj: CreateCapacityReservationResult) => any;
    const isa: (o: any) => o is CreateCapacityReservationResult;
}
export interface CreateClientVpnEndpointRequest {
    __type?: "CreateClientVpnEndpointRequest";
    /**
     * <p>Information about the authentication method to be used to authenticate clients.</p>
     */
    AuthenticationOptions: ClientVpnAuthenticationRequest[] | undefined;
    /**
     * <p>The IPv4 address range, in CIDR notation, from which to assign client IP addresses. The address range cannot overlap with the local CIDR of the VPC in which the associated subnet is located, or the routes that you add manually. The address range cannot be changed after the Client VPN endpoint has been created. The CIDR block should be /22 or greater.</p>
     */
    ClientCidrBlock: string | undefined;
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Information about the client connection logging options.</p>
     * 		       <p>If you enable client connection logging, data about client connections is sent to a
     * 			Cloudwatch Logs log stream. The following information is logged:</p>
     * 		       <ul>
     *             <li>
     *                <p>Client connection requests</p>
     *             </li>
     *             <li>
     *                <p>Client connection results (successful and unsuccessful)</p>
     *             </li>
     *             <li>
     *                <p>Reasons for unsuccessful client connection requests</p>
     *             </li>
     *             <li>
     *                <p>Client connection termination time</p>
     *             </li>
     *          </ul>
     */
    ConnectionLogOptions: ConnectionLogOptions | undefined;
    /**
     * <p>A brief description of the Client VPN endpoint.</p>
     */
    Description?: string;
    /**
     * <p>Information about the DNS servers to be used for DNS resolution. A Client VPN endpoint can
     * 			have up to two DNS servers. If no DNS server is specified, the DNS address configured on the device is used for the DNS server.</p>
     */
    DnsServers?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ARN of the server certificate. For more information, see
     * 			the <a href="https://docs.aws.amazon.com/acm/latest/userguide/">AWS Certificate Manager User Guide</a>.</p>
     */
    ServerCertificateArn: string | undefined;
    /**
     * <p>Indicates whether split-tunnel is enabled on the AWS Client VPN endpoint.</p>
     * 		       <p>By default, split-tunnel on a VPN endpoint is disabled.</p>
     * 		       <p>For information about split-tunnel VPN endpoints, see <a href="https://docs.aws.amazon.com/vpn/latest/clientvpn-admin/split-tunnel-vpn.html">Split-Tunnel AWS Client VPN Endpoint</a> in the <i>AWS
     * 				Client VPN Administrator Guide</i>.</p>
     */
    SplitTunnel?: boolean;
    /**
     * <p>The tags to apply to the Client VPN endpoint during creation.</p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>The transport protocol to be used by the VPN session.</p>
     * 		       <p>Default value: <code>udp</code>
     *          </p>
     */
    TransportProtocol?: TransportProtocol | string;
    /**
     * <p>The port number to assign to the Client VPN endpoint for TCP and UDP traffic.</p>
     * 	        <p>Valid Values: <code>443</code> | <code>1194</code>
     *          </p>
     * 	        <p>Default Value: <code>443</code>
     *          </p>
     */
    VpnPort?: number;
}
export declare namespace CreateClientVpnEndpointRequest {
    const filterSensitiveLog: (obj: CreateClientVpnEndpointRequest) => any;
    const isa: (o: any) => o is CreateClientVpnEndpointRequest;
}
export interface CreateClientVpnEndpointResult {
    __type?: "CreateClientVpnEndpointResult";
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId?: string;
    /**
     * <p>The DNS name to be used by clients when establishing their VPN session.</p>
     */
    DnsName?: string;
    /**
     * <p>The current state of the Client VPN endpoint.</p>
     */
    Status?: ClientVpnEndpointStatus;
}
export declare namespace CreateClientVpnEndpointResult {
    const filterSensitiveLog: (obj: CreateClientVpnEndpointResult) => any;
    const isa: (o: any) => o is CreateClientVpnEndpointResult;
}
export interface CreateClientVpnRouteRequest {
    __type?: "CreateClientVpnRouteRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The ID of the Client VPN endpoint to which to add the route.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>A brief description of the route.</p>
     */
    Description?: string;
    /**
     * <p>The IPv4 address range, in CIDR notation, of the route destination. For example:</p>
     * 		       <ul>
     *             <li>
     *                <p>To add a route for Internet access, enter <code>0.0.0.0/0</code>
     *                </p>
     *             </li>
     *             <li>
     *                <p>To add a route for a peered VPC, enter the peered VPC's IPv4 CIDR range</p>
     *             </li>
     *             <li>
     *                <p>To add a route for an on-premises network, enter the AWS Site-to-Site VPN connection's IPv4 CIDR range</p>
     *             </li>
     *          </ul>
     * 		       <p>Route address ranges cannot overlap with the CIDR range specified for client allocation.</p>
     */
    DestinationCidrBlock: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the subnet through which you want to route traffic. The specified subnet must be
     * 			an existing target network of the Client VPN endpoint.</p>
     */
    TargetVpcSubnetId: string | undefined;
}
export declare namespace CreateClientVpnRouteRequest {
    const filterSensitiveLog: (obj: CreateClientVpnRouteRequest) => any;
    const isa: (o: any) => o is CreateClientVpnRouteRequest;
}
export interface CreateClientVpnRouteResult {
    __type?: "CreateClientVpnRouteResult";
    /**
     * <p>The current state of the route.</p>
     */
    Status?: ClientVpnRouteStatus;
}
export declare namespace CreateClientVpnRouteResult {
    const filterSensitiveLog: (obj: CreateClientVpnRouteResult) => any;
    const isa: (o: any) => o is CreateClientVpnRouteResult;
}
/**
 * <p>Contains the parameters for CreateCustomerGateway.</p>
 */
export interface CreateCustomerGatewayRequest {
    __type?: "CreateCustomerGatewayRequest";
    /**
     * <p>For devices that support BGP, the customer gateway's BGP ASN.</p>
     *         <p>Default: 65000</p>
     */
    BgpAsn: number | undefined;
    /**
     * <p>The Amazon Resource Name (ARN) for the customer gateway certificate.</p>
     */
    CertificateArn?: string;
    /**
     * <p>A name for the customer gateway device.</p>
     *         <p>Length Constraints: Up to 255 characters.</p>
     */
    DeviceName?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The Internet-routable IP address for the customer gateway's outside interface. The address must be static.</p>
     */
    PublicIp?: string;
    /**
     * <p>The type of VPN connection that this customer gateway supports (<code>ipsec.1</code>).</p>
     */
    Type: GatewayType | string | undefined;
}
export declare namespace CreateCustomerGatewayRequest {
    const filterSensitiveLog: (obj: CreateCustomerGatewayRequest) => any;
    const isa: (o: any) => o is CreateCustomerGatewayRequest;
}
/**
 * <p>Contains the output of CreateCustomerGateway.</p>
 */
export interface CreateCustomerGatewayResult {
    __type?: "CreateCustomerGatewayResult";
    /**
     * <p>Information about the customer gateway.</p>
     */
    CustomerGateway?: CustomerGateway;
}
export declare namespace CreateCustomerGatewayResult {
    const filterSensitiveLog: (obj: CreateCustomerGatewayResult) => any;
    const isa: (o: any) => o is CreateCustomerGatewayResult;
}
export interface CreateDefaultSubnetRequest {
    __type?: "CreateDefaultSubnetRequest";
    /**
     * <p>The Availability Zone in which to create the default subnet.</p>
     */
    AvailabilityZone: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace CreateDefaultSubnetRequest {
    const filterSensitiveLog: (obj: CreateDefaultSubnetRequest) => any;
    const isa: (o: any) => o is CreateDefaultSubnetRequest;
}
export interface CreateDefaultSubnetResult {
    __type?: "CreateDefaultSubnetResult";
    /**
     * <p>Information about the subnet.</p>
     */
    Subnet?: Subnet;
}
export declare namespace CreateDefaultSubnetResult {
    const filterSensitiveLog: (obj: CreateDefaultSubnetResult) => any;
    const isa: (o: any) => o is CreateDefaultSubnetResult;
}
export interface CreateDefaultVpcRequest {
    __type?: "CreateDefaultVpcRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace CreateDefaultVpcRequest {
    const filterSensitiveLog: (obj: CreateDefaultVpcRequest) => any;
    const isa: (o: any) => o is CreateDefaultVpcRequest;
}
export interface CreateDefaultVpcResult {
    __type?: "CreateDefaultVpcResult";
    /**
     * <p>Information about the VPC.</p>
     */
    Vpc?: Vpc;
}
export declare namespace CreateDefaultVpcResult {
    const filterSensitiveLog: (obj: CreateDefaultVpcResult) => any;
    const isa: (o: any) => o is CreateDefaultVpcResult;
}
export interface CreateDhcpOptionsRequest {
    __type?: "CreateDhcpOptionsRequest";
    /**
     * <p>A DHCP configuration option.</p>
     */
    DhcpConfigurations: NewDhcpConfiguration[] | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace CreateDhcpOptionsRequest {
    const filterSensitiveLog: (obj: CreateDhcpOptionsRequest) => any;
    const isa: (o: any) => o is CreateDhcpOptionsRequest;
}
export interface CreateDhcpOptionsResult {
    __type?: "CreateDhcpOptionsResult";
    /**
     * <p>A set of DHCP options.</p>
     */
    DhcpOptions?: DhcpOptions;
}
export declare namespace CreateDhcpOptionsResult {
    const filterSensitiveLog: (obj: CreateDhcpOptionsResult) => any;
    const isa: (o: any) => o is CreateDhcpOptionsResult;
}
export interface CreateEgressOnlyInternetGatewayRequest {
    __type?: "CreateEgressOnlyInternetGatewayRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     * 			request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">How to Ensure
     * 				Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC for which to create the egress-only internet gateway.</p>
     */
    VpcId: string | undefined;
}
export declare namespace CreateEgressOnlyInternetGatewayRequest {
    const filterSensitiveLog: (obj: CreateEgressOnlyInternetGatewayRequest) => any;
    const isa: (o: any) => o is CreateEgressOnlyInternetGatewayRequest;
}
export interface CreateEgressOnlyInternetGatewayResult {
    __type?: "CreateEgressOnlyInternetGatewayResult";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     * 			request.</p>
     */
    ClientToken?: string;
    /**
     * <p>Information about the egress-only internet gateway.</p>
     */
    EgressOnlyInternetGateway?: EgressOnlyInternetGateway;
}
export declare namespace CreateEgressOnlyInternetGatewayResult {
    const filterSensitiveLog: (obj: CreateEgressOnlyInternetGatewayResult) => any;
    const isa: (o: any) => o is CreateEgressOnlyInternetGatewayResult;
}
/**
 * <p>Describes the instances that could not be launched by the fleet.</p>
 */
export interface CreateFleetError {
    __type?: "CreateFleetError";
    /**
     * <p>The error code that indicates why the instance could not be launched. For more
     *          information about error codes, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/errors-overview.html.html">Error Codes</a>.</p>
     */
    ErrorCode?: string;
    /**
     * <p>The error message that describes why the instance could not be launched. For more
     *          information about error messages, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/errors-overview.html.html">Error Codes</a>.</p>
     */
    ErrorMessage?: string;
    /**
     * <p>The launch templates and overrides that were used for launching the instances. The
     *          values that you specify in the Overrides replace the values in the launch template.</p>
     */
    LaunchTemplateAndOverrides?: LaunchTemplateAndOverridesResponse;
    /**
     * <p>Indicates if the instance that could not be launched was a Spot Instance or On-Demand Instance.</p>
     */
    Lifecycle?: InstanceLifecycle | string;
}
export declare namespace CreateFleetError {
    const filterSensitiveLog: (obj: CreateFleetError) => any;
    const isa: (o: any) => o is CreateFleetError;
}
/**
 * <p>Describes the instances that were launched by the fleet.</p>
 */
export interface CreateFleetInstance {
    __type?: "CreateFleetInstance";
    /**
     * <p>The IDs of the instances.</p>
     */
    InstanceIds?: string[];
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The launch templates and overrides that were used for launching the instances. The
     *          values that you specify in the Overrides replace the values in the launch template.</p>
     */
    LaunchTemplateAndOverrides?: LaunchTemplateAndOverridesResponse;
    /**
     * <p>Indicates if the instance that was launched is a Spot Instance or On-Demand Instance.</p>
     */
    Lifecycle?: InstanceLifecycle | string;
    /**
     * <p>The value is <code>Windows</code> for Windows instances. Otherwise, the value is
     *          blank.</p>
     */
    Platform?: PlatformValues | string;
}
export declare namespace CreateFleetInstance {
    const filterSensitiveLog: (obj: CreateFleetInstance) => any;
    const isa: (o: any) => o is CreateFleetInstance;
}
export interface CreateFleetRequest {
    __type?: "CreateFleetRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     *          request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring
     *             Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Indicates whether running instances should be terminated if the total target capacity of
     *          the EC2 Fleet is decreased below the current size of the EC2 Fleet.</p>
     */
    ExcessCapacityTerminationPolicy?: FleetExcessCapacityTerminationPolicy | string;
    /**
     * <p>The configuration for the EC2 Fleet.</p>
     */
    LaunchTemplateConfigs: FleetLaunchTemplateConfigRequest[] | undefined;
    /**
     * <p>Describes the configuration of On-Demand Instances in an EC2 Fleet.</p>
     */
    OnDemandOptions?: OnDemandOptionsRequest;
    /**
     * <p>Indicates whether EC2 Fleet should replace unhealthy instances.</p>
     */
    ReplaceUnhealthyInstances?: boolean;
    /**
     * <p>Describes the configuration of Spot Instances in an EC2 Fleet.</p>
     */
    SpotOptions?: SpotOptionsRequest;
    /**
     * <p>The key-value pair for tagging the EC2 Fleet request on creation. The value for
     *             <code>ResourceType</code> must be <code>fleet</code>, otherwise the fleet request fails.
     *          To tag instances at launch, specify the tags in the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-launch-templates.html#create-launch-template">launch
     *             template</a>. For information about tagging after launch, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html#tag-resources">Tagging Your
     *             Resources</a>. </p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>The number of units to request.</p>
     */
    TargetCapacitySpecification: TargetCapacitySpecificationRequest | undefined;
    /**
     * <p>Indicates whether running instances should be terminated when the EC2 Fleet expires.</p>
     */
    TerminateInstancesWithExpiration?: boolean;
    /**
     * <p>The type of the request. By default, the EC2 Fleet places an asynchronous request for your
     *          desired capacity, and maintains it by replenishing interrupted Spot Instances
     *          (<code>maintain</code>). A value of <code>instant</code> places a synchronous one-time
     *          request, and returns errors for any instances that could not be launched. A value of
     *             <code>request</code> places an asynchronous one-time request without maintaining
     *          capacity or submitting requests in alternative capacity pools if capacity is unavailable.
     *          For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-fleet-configuration-strategies.html#ec2-fleet-request-type">EC2 Fleet
     *             Request Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    Type?: FleetType | string;
    /**
     * <p>The start date and time of the request, in UTC format (for example,
     *             <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).
     *          The default is to start fulfilling the request immediately.</p>
     */
    ValidFrom?: Date;
    /**
     * <p>The end date and time of the request, in UTC format (for example,
     *             <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).
     *          At this point, no new EC2 Fleet requests are placed or able to fulfill the request. If no value is specified, the request remains until you cancel it.</p>
     */
    ValidUntil?: Date;
}
export declare namespace CreateFleetRequest {
    const filterSensitiveLog: (obj: CreateFleetRequest) => any;
    const isa: (o: any) => o is CreateFleetRequest;
}
export interface CreateFleetResult {
    __type?: "CreateFleetResult";
    /**
     * <p>Information about the instances that could not be launched by the fleet. Valid only when
     *             <b>Type</b> is set to <code>instant</code>.</p>
     */
    Errors?: CreateFleetError[];
    /**
     * <p>The ID of the EC2 Fleet.</p>
     */
    FleetId?: string;
    /**
     * <p>Information about the instances that were launched by the fleet. Valid only when
     *             <b>Type</b> is set to <code>instant</code>.</p>
     */
    Instances?: CreateFleetInstance[];
}
export declare namespace CreateFleetResult {
    const filterSensitiveLog: (obj: CreateFleetResult) => any;
    const isa: (o: any) => o is CreateFleetResult;
}
export interface CreateFlowLogsRequest {
    __type?: "CreateFlowLogsRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     *             request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">How to Ensure
     *                 Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The ARN for the IAM role that permits Amazon EC2 to publish flow logs to a CloudWatch Logs log group
     *             in your account.</p>
     *         <p>If you specify <code>LogDestinationType</code> as <code>s3</code>, do not specify
     *             <code>DeliverLogsPermissionArn</code> or <code>LogGroupName</code>.</p>
     */
    DeliverLogsPermissionArn?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Specifies the destination to which the flow log data is to be published. Flow log data can be published
     *             to a CloudWatch Logs log group or an Amazon S3 bucket. The value specified for this parameter depends on the value specified
     *             for <code>LogDestinationType</code>.</p>
     *         <p>If <code>LogDestinationType</code> is not specified or <code>cloud-watch-logs</code>,
     *             specify the Amazon Resource Name (ARN) of the CloudWatch Logs log group. For example, to publish
     *             to a log group called <code>my-logs</code>, specify
     *             <code>arn:aws:logs:us-east-1:123456789012:log-group:my-logs</code>. Alternatively,
     *             use <code>LogGroupName</code> instead.</p>
     *         <p>If LogDestinationType is <code>s3</code>, specify the ARN of the Amazon S3 bucket. You can also specify a
     *             subfolder in the bucket. To specify a subfolder in the bucket, use the following  ARN format:
     *             <code>bucket_ARN/subfolder_name/</code>. For example, to specify a subfolder named <code>my-logs</code> in a
     *             bucket named <code>my-bucket</code>, use the following ARN: <code>arn:aws:s3:::my-bucket/my-logs/</code>. You
     *             cannot use <code>AWSLogs</code> as a subfolder name. This is a reserved term.</p>
     */
    LogDestination?: string;
    /**
     * <p>Specifies the type of destination to which the flow log data is to be published. Flow log data can be
     *             published to CloudWatch Logs or Amazon S3. To publish flow log data to CloudWatch Logs, specify <code>cloud-watch-logs</code>. To
     *             publish flow log data to Amazon S3, specify <code>s3</code>.</p>
     *         <p>If you specify <code>LogDestinationType</code> as <code>s3</code>, do not specify
     *             <code>DeliverLogsPermissionArn</code> or <code>LogGroupName</code>.</p>
     *         <p>Default: <code>cloud-watch-logs</code>
     *          </p>
     */
    LogDestinationType?: LogDestinationType | string;
    /**
     * <p>The fields to include in the flow log record, in the order in which they should
     *             appear. For a list of available fields, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/flow-logs.html#flow-log-records">Flow Log Records</a>. If you
     *             omit this parameter, the flow log is created using the default format. If you specify this parameter, you must specify at least one field.</p>
     *         <p>Specify the fields using the <code>${field-id}</code> format, separated by spaces. For
     *             the AWS CLI, use single quotation marks (' ') to surround the parameter value.</p>
     *         <p>Only applicable to flow logs that are published to an Amazon S3 bucket.</p>
     */
    LogFormat?: string;
    /**
     * <p>The name of a new or existing CloudWatch Logs log group where Amazon EC2 publishes your flow logs.</p>
     *         <p>If you specify <code>LogDestinationType</code> as <code>s3</code>, do not specify
     *             <code>DeliverLogsPermissionArn</code> or <code>LogGroupName</code>.</p>
     */
    LogGroupName?: string;
    /**
     * <p>The ID of the subnet, network interface, or VPC for which you want to create a flow log.</p>
     *         <p>Constraints: Maximum of 1000 resources</p>
     */
    ResourceIds: string[] | undefined;
    /**
     * <p>The type of resource for which to create the flow log. For example, if you specified a VPC ID for
     *             the <code>ResourceId</code> property, specify <code>VPC</code> for this property.</p>
     */
    ResourceType: FlowLogsResourceType | string | undefined;
    /**
     * <p>The type of traffic to log. You can log traffic that the resource accepts or rejects, or all traffic.</p>
     */
    TrafficType: TrafficType | string | undefined;
}
export declare namespace CreateFlowLogsRequest {
    const filterSensitiveLog: (obj: CreateFlowLogsRequest) => any;
    const isa: (o: any) => o is CreateFlowLogsRequest;
}
export interface CreateFlowLogsResult {
    __type?: "CreateFlowLogsResult";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     *             request.</p>
     */
    ClientToken?: string;
    /**
     * <p>The IDs of the flow logs.</p>
     */
    FlowLogIds?: string[];
    /**
     * <p>Information about the flow logs that could not be created successfully.</p>
     */
    Unsuccessful?: UnsuccessfulItem[];
}
export declare namespace CreateFlowLogsResult {
    const filterSensitiveLog: (obj: CreateFlowLogsResult) => any;
    const isa: (o: any) => o is CreateFlowLogsResult;
}
export interface CreateFpgaImageRequest {
    __type?: "CreateFpgaImageRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.
     *          For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>A description for the AFI.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The location of the encrypted design checkpoint in Amazon S3. The input must be a tarball.</p>
     */
    InputStorageLocation: StorageLocation | undefined;
    /**
     * <p>The location in Amazon S3 for the output logs.</p>
     */
    LogsStorageLocation?: StorageLocation;
    /**
     * <p>A name for the AFI.</p>
     */
    Name?: string;
    /**
     * <p>The tags to apply to the FPGA image during creation.</p>
     */
    TagSpecifications?: TagSpecification[];
}
export declare namespace CreateFpgaImageRequest {
    const filterSensitiveLog: (obj: CreateFpgaImageRequest) => any;
    const isa: (o: any) => o is CreateFpgaImageRequest;
}
export interface CreateFpgaImageResult {
    __type?: "CreateFpgaImageResult";
    /**
     * <p>The global FPGA image identifier (AGFI ID).</p>
     */
    FpgaImageGlobalId?: string;
    /**
     * <p>The FPGA image identifier (AFI ID).</p>
     */
    FpgaImageId?: string;
}
export declare namespace CreateFpgaImageResult {
    const filterSensitiveLog: (obj: CreateFpgaImageResult) => any;
    const isa: (o: any) => o is CreateFpgaImageResult;
}
export interface CreateImageRequest {
    __type?: "CreateImageRequest";
    /**
     * <p>The block device mappings. This parameter cannot be used to modify the encryption status of existing volumes or snapshots. To create an AMI with encrypted snapshots, use the <a>CopyImage</a> action.</p>
     */
    BlockDeviceMappings?: BlockDeviceMapping[];
    /**
     * <p>A description for the new image.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>A name for the new image.</p>
     *          <p>Constraints: 3-128 alphanumeric characters, parentheses (()), square brackets ([]), spaces ( ), periods (.), slashes (/), dashes (-), single quotes ('), at-signs (@), or underscores(_)</p>
     */
    Name: string | undefined;
    /**
     * <p>By default, Amazon EC2 attempts to shut down and reboot the instance before creating the image. If the 'No Reboot' option is set, Amazon EC2 doesn't shut down the instance before creating the image. When this option is used, file system integrity on the created image can't be guaranteed.</p>
     */
    NoReboot?: boolean;
}
export declare namespace CreateImageRequest {
    const filterSensitiveLog: (obj: CreateImageRequest) => any;
    const isa: (o: any) => o is CreateImageRequest;
}
export interface CreateImageResult {
    __type?: "CreateImageResult";
    /**
     * <p>The ID of the new AMI.</p>
     */
    ImageId?: string;
}
export declare namespace CreateImageResult {
    const filterSensitiveLog: (obj: CreateImageResult) => any;
    const isa: (o: any) => o is CreateImageResult;
}
export interface CreateInstanceExportTaskRequest {
    __type?: "CreateInstanceExportTaskRequest";
    /**
     * <p>A description for the conversion task or the resource being exported. The maximum length is 255 bytes.</p>
     */
    Description?: string;
    /**
     * <p>The format and location for an instance export task.</p>
     */
    ExportToS3Task?: ExportToS3TaskSpecification;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>The target virtualization environment.</p>
     */
    TargetEnvironment?: ExportEnvironment | string;
}
export declare namespace CreateInstanceExportTaskRequest {
    const filterSensitiveLog: (obj: CreateInstanceExportTaskRequest) => any;
    const isa: (o: any) => o is CreateInstanceExportTaskRequest;
}
export interface CreateInstanceExportTaskResult {
    __type?: "CreateInstanceExportTaskResult";
    /**
     * <p>Information about the instance export task.</p>
     */
    ExportTask?: ExportTask;
}
export declare namespace CreateInstanceExportTaskResult {
    const filterSensitiveLog: (obj: CreateInstanceExportTaskResult) => any;
    const isa: (o: any) => o is CreateInstanceExportTaskResult;
}
export interface CreateInternetGatewayRequest {
    __type?: "CreateInternetGatewayRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace CreateInternetGatewayRequest {
    const filterSensitiveLog: (obj: CreateInternetGatewayRequest) => any;
    const isa: (o: any) => o is CreateInternetGatewayRequest;
}
export interface CreateInternetGatewayResult {
    __type?: "CreateInternetGatewayResult";
    /**
     * <p>Information about the internet gateway.</p>
     */
    InternetGateway?: InternetGateway;
}
export declare namespace CreateInternetGatewayResult {
    const filterSensitiveLog: (obj: CreateInternetGatewayResult) => any;
    const isa: (o: any) => o is CreateInternetGatewayResult;
}
export interface CreateKeyPairRequest {
    __type?: "CreateKeyPairRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>A unique name for the key pair.</p>
     * 	        <p>Constraints: Up to 255 ASCII characters</p>
     */
    KeyName: string | undefined;
}
export declare namespace CreateKeyPairRequest {
    const filterSensitiveLog: (obj: CreateKeyPairRequest) => any;
    const isa: (o: any) => o is CreateKeyPairRequest;
}
export interface CreateLaunchTemplateRequest {
    __type?: "CreateLaunchTemplateRequest";
    /**
     * <p>Unique, case-sensitive identifier you provide to ensure the idempotency of the
     *             request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring
     *                 Idempotency</a>.</p>
     *         <p>Constraint: Maximum 128 ASCII characters.</p>
     */
    ClientToken?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually
     *             making the request, and provides an error response. If you have the required
     *             permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is
     *                 <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The information for the launch template.</p>
     */
    LaunchTemplateData: RequestLaunchTemplateData | undefined;
    /**
     * <p>A name for the launch template.</p>
     */
    LaunchTemplateName: string | undefined;
    /**
     * <p>The tags to apply to the launch template during creation.</p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>A description for the first version of the launch template.</p>
     */
    VersionDescription?: string;
}
export declare namespace CreateLaunchTemplateRequest {
    const filterSensitiveLog: (obj: CreateLaunchTemplateRequest) => any;
    const isa: (o: any) => o is CreateLaunchTemplateRequest;
}
export interface CreateLaunchTemplateResult {
    __type?: "CreateLaunchTemplateResult";
    /**
     * <p>Information about the launch template.</p>
     */
    LaunchTemplate?: LaunchTemplate;
}
export declare namespace CreateLaunchTemplateResult {
    const filterSensitiveLog: (obj: CreateLaunchTemplateResult) => any;
    const isa: (o: any) => o is CreateLaunchTemplateResult;
}
export interface CreateLaunchTemplateVersionRequest {
    __type?: "CreateLaunchTemplateVersionRequest";
    /**
     * <p>Unique, case-sensitive identifier you provide to ensure the idempotency of the
     *             request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring
     *                 Idempotency</a>.</p>
     *         <p>Constraint: Maximum 128 ASCII characters.</p>
     */
    ClientToken?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually
     *             making the request, and provides an error response. If you have the required
     *             permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is
     *                 <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The information for the launch template.</p>
     */
    LaunchTemplateData: RequestLaunchTemplateData | undefined;
    /**
     * <p>The ID of the launch template. You must specify either the launch template ID or
     *             launch template name in the request.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template. You must specify either the launch template ID or
     *             launch template name in the request.</p>
     */
    LaunchTemplateName?: string;
    /**
     * <p>The version number of the launch template version on which to base the new version.
     *             The new version inherits the same launch parameters as the source version, except for
     *             parameters that you specify in <code>LaunchTemplateData</code>. Snapshots applied
     *             to the block device mapping are ignored when creating a new version unless they
     *             are explicitly included.</p>
     */
    SourceVersion?: string;
    /**
     * <p>A description for the version of the launch template.</p>
     */
    VersionDescription?: string;
}
export declare namespace CreateLaunchTemplateVersionRequest {
    const filterSensitiveLog: (obj: CreateLaunchTemplateVersionRequest) => any;
    const isa: (o: any) => o is CreateLaunchTemplateVersionRequest;
}
export interface CreateLaunchTemplateVersionResult {
    __type?: "CreateLaunchTemplateVersionResult";
    /**
     * <p>Information about the launch template version.</p>
     */
    LaunchTemplateVersion?: LaunchTemplateVersion;
}
export declare namespace CreateLaunchTemplateVersionResult {
    const filterSensitiveLog: (obj: CreateLaunchTemplateVersionResult) => any;
    const isa: (o: any) => o is CreateLaunchTemplateVersionResult;
}
export interface CreateLocalGatewayRouteRequest {
    __type?: "CreateLocalGatewayRouteRequest";
    /**
     * <p>The CIDR range used for destination matches. Routing decisions are based on
     *         the most specific match.</p>
     */
    DestinationCidrBlock: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the local gateway route table.</p>
     */
    LocalGatewayRouteTableId: string | undefined;
    /**
     * <p>The ID of the virtual interface group.</p>
     */
    LocalGatewayVirtualInterfaceGroupId: string | undefined;
}
export declare namespace CreateLocalGatewayRouteRequest {
    const filterSensitiveLog: (obj: CreateLocalGatewayRouteRequest) => any;
    const isa: (o: any) => o is CreateLocalGatewayRouteRequest;
}
export interface CreateLocalGatewayRouteResult {
    __type?: "CreateLocalGatewayRouteResult";
    /**
     * <p>Information about the route.</p>
     */
    Route?: LocalGatewayRoute;
}
export declare namespace CreateLocalGatewayRouteResult {
    const filterSensitiveLog: (obj: CreateLocalGatewayRouteResult) => any;
    const isa: (o: any) => o is CreateLocalGatewayRouteResult;
}
export interface CreateLocalGatewayRouteTableVpcAssociationRequest {
    __type?: "CreateLocalGatewayRouteTableVpcAssociationRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the local gateway route table.</p>
     */
    LocalGatewayRouteTableId: string | undefined;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace CreateLocalGatewayRouteTableVpcAssociationRequest {
    const filterSensitiveLog: (obj: CreateLocalGatewayRouteTableVpcAssociationRequest) => any;
    const isa: (o: any) => o is CreateLocalGatewayRouteTableVpcAssociationRequest;
}
export interface CreateLocalGatewayRouteTableVpcAssociationResult {
    __type?: "CreateLocalGatewayRouteTableVpcAssociationResult";
    /**
     * <p>Information about the association.</p>
     */
    LocalGatewayRouteTableVpcAssociation?: LocalGatewayRouteTableVpcAssociation;
}
export declare namespace CreateLocalGatewayRouteTableVpcAssociationResult {
    const filterSensitiveLog: (obj: CreateLocalGatewayRouteTableVpcAssociationResult) => any;
    const isa: (o: any) => o is CreateLocalGatewayRouteTableVpcAssociationResult;
}
export interface CreateNatGatewayRequest {
    __type?: "CreateNatGatewayRequest";
    /**
     * <p>The allocation ID of an Elastic IP address to associate with the NAT gateway. If the Elastic IP address is associated with another resource, you must first disassociate it.</p>
     */
    AllocationId: string | undefined;
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     * 			request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure
     * 				Idempotency</a>.</p>
     *          <p>Constraint: Maximum 64 ASCII characters.</p>
     */
    ClientToken?: string;
    /**
     * <p>The subnet in which to create the NAT gateway.</p>
     */
    SubnetId: string | undefined;
}
export declare namespace CreateNatGatewayRequest {
    const filterSensitiveLog: (obj: CreateNatGatewayRequest) => any;
    const isa: (o: any) => o is CreateNatGatewayRequest;
}
export interface CreateNatGatewayResult {
    __type?: "CreateNatGatewayResult";
    /**
     * <p>Unique, case-sensitive identifier to ensure the idempotency of the request. Only returned if a client token was provided in the request.</p>
     */
    ClientToken?: string;
    /**
     * <p>Information about the NAT gateway.</p>
     */
    NatGateway?: NatGateway;
}
export declare namespace CreateNatGatewayResult {
    const filterSensitiveLog: (obj: CreateNatGatewayResult) => any;
    const isa: (o: any) => o is CreateNatGatewayResult;
}
export interface CreateNetworkAclEntryRequest {
    __type?: "CreateNetworkAclEntryRequest";
    /**
     * <p>The IPv4 network range to allow or deny, in CIDR notation (for example
     *                 <code>172.16.0.0/24</code>).</p>
     */
    CidrBlock?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Indicates whether this is an egress rule (rule is applied to traffic leaving the subnet).</p>
     */
    Egress: boolean | undefined;
    /**
     * <p>ICMP protocol: The ICMP or ICMPv6 type and code. Required if specifying protocol
     * 		        1 (ICMP) or protocol 58 (ICMPv6) with an IPv6 CIDR block.</p>
     */
    IcmpTypeCode?: IcmpTypeCode;
    /**
     * <p>The IPv6 network range to allow or deny, in CIDR notation (for example
     *                 <code>2001:db8:1234:1a00::/64</code>).</p>
     */
    Ipv6CidrBlock?: string;
    /**
     * <p>The ID of the network ACL.</p>
     */
    NetworkAclId: string | undefined;
    /**
     * <p>TCP or UDP protocols: The range of ports the rule applies to.
     * 		        Required if specifying protocol 6 (TCP) or 17 (UDP).</p>
     */
    PortRange?: PortRange;
    /**
     * <p>The protocol number. A value of "-1" means all protocols. If you specify "-1" or a
     *   			protocol number other than "6" (TCP), "17" (UDP), or "1" (ICMP), traffic on all ports is
     * 			allowed, regardless of any ports or ICMP types or codes that you specify. If you specify
     * 			protocol "58" (ICMPv6) and specify an IPv4 CIDR block, traffic for all ICMP types and
     * 			codes allowed, regardless of any that you specify. If you specify protocol "58" (ICMPv6)
     * 			and specify an IPv6 CIDR block, you must specify an ICMP type and code.</p>
     */
    Protocol: string | undefined;
    /**
     * <p>Indicates whether to allow or deny the traffic that matches the rule.</p>
     */
    RuleAction: RuleAction | string | undefined;
    /**
     * <p>The rule number for the entry (for example, 100). ACL entries are processed in ascending order by rule number.</p>
     *          <p>Constraints: Positive integer from 1 to 32766. The range 32767 to 65535 is reserved for internal use.</p>
     */
    RuleNumber: number | undefined;
}
export declare namespace CreateNetworkAclEntryRequest {
    const filterSensitiveLog: (obj: CreateNetworkAclEntryRequest) => any;
    const isa: (o: any) => o is CreateNetworkAclEntryRequest;
}
export interface CreateNetworkAclRequest {
    __type?: "CreateNetworkAclRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace CreateNetworkAclRequest {
    const filterSensitiveLog: (obj: CreateNetworkAclRequest) => any;
    const isa: (o: any) => o is CreateNetworkAclRequest;
}
export interface CreateNetworkAclResult {
    __type?: "CreateNetworkAclResult";
    /**
     * <p>Information about the network ACL.</p>
     */
    NetworkAcl?: NetworkAcl;
}
export declare namespace CreateNetworkAclResult {
    const filterSensitiveLog: (obj: CreateNetworkAclResult) => any;
    const isa: (o: any) => o is CreateNetworkAclResult;
}
/**
 * <p>Contains the parameters for CreateNetworkInterfacePermission.</p>
 */
export interface CreateNetworkInterfacePermissionRequest {
    __type?: "CreateNetworkInterfacePermissionRequest";
    /**
     * <p>The AWS account ID.</p>
     */
    AwsAccountId?: string;
    /**
     * <p>The AWS service. Currently not supported.</p>
     */
    AwsService?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     * 			and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     * 			Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId: string | undefined;
    /**
     * <p>The type of permission to grant.</p>
     */
    Permission: InterfacePermissionType | string | undefined;
}
export declare namespace CreateNetworkInterfacePermissionRequest {
    const filterSensitiveLog: (obj: CreateNetworkInterfacePermissionRequest) => any;
    const isa: (o: any) => o is CreateNetworkInterfacePermissionRequest;
}
/**
 * <p>Contains the output of CreateNetworkInterfacePermission.</p>
 */
export interface CreateNetworkInterfacePermissionResult {
    __type?: "CreateNetworkInterfacePermissionResult";
    /**
     * <p>Information about the permission for the network interface.</p>
     */
    InterfacePermission?: NetworkInterfacePermission;
}
export declare namespace CreateNetworkInterfacePermissionResult {
    const filterSensitiveLog: (obj: CreateNetworkInterfacePermissionResult) => any;
    const isa: (o: any) => o is CreateNetworkInterfacePermissionResult;
}
/**
 * <p>Contains the parameters for CreateNetworkInterface.</p>
 */
export interface CreateNetworkInterfaceRequest {
    __type?: "CreateNetworkInterfaceRequest";
    /**
     * <p>A description for the network interface.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of one or more security groups.</p>
     */
    Groups?: string[];
    /**
     * <p>Indicates the type of network interface. To create an Elastic Fabric Adapter (EFA), specify
     * 			<code>efa</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/efa.html">
     * 		    Elastic Fabric Adapter</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    InterfaceType?: NetworkInterfaceCreationType | string;
    /**
     * <p>The number of IPv6 addresses to assign to a network interface. Amazon EC2
     *             automatically selects the IPv6 addresses from the subnet range. You can't use this
     *             option if specifying specific IPv6 addresses. If your subnet has the <code>AssignIpv6AddressOnCreation</code> attribute set
     *             to <code>true</code>, you can specify <code>0</code> to override this setting.</p>
     */
    Ipv6AddressCount?: number;
    /**
     * <p>One or more specific IPv6 addresses from the IPv6 CIDR block range of your subnet.
     *             You can't use this option if you're specifying a number of IPv6 addresses.</p>
     */
    Ipv6Addresses?: InstanceIpv6Address[];
    /**
     * <p>The primary private IPv4 address of the network interface. If you don't specify an
     *             IPv4 address, Amazon EC2 selects one for you from the subnet's IPv4 CIDR range. If you
     *             specify an IP address, you cannot indicate any IP addresses specified in
     *             <code>privateIpAddresses</code> as primary (only one IP address can be designated as
     *             primary).</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>One or more private IPv4 addresses.</p>
     */
    PrivateIpAddresses?: PrivateIpAddressSpecification[];
    /**
     * <p>The number of secondary private IPv4 addresses to assign to a network interface. When
     *             you specify a number of secondary IPv4 addresses, Amazon EC2 selects these IP addresses
     *             within the subnet's IPv4 CIDR range. You can't specify this option and specify more than
     *             one private IP address using <code>privateIpAddresses</code>.</p>
     *         <p>The number of IP addresses you can assign to a network interface varies by instance
     *             type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-eni.html#AvailableIpPerENI">IP Addresses Per ENI Per
     *                 Instance Type</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.</p>
     */
    SecondaryPrivateIpAddressCount?: number;
    /**
     * <p>The ID of the subnet to associate with the network interface.</p>
     */
    SubnetId: string | undefined;
}
export declare namespace CreateNetworkInterfaceRequest {
    const filterSensitiveLog: (obj: CreateNetworkInterfaceRequest) => any;
    const isa: (o: any) => o is CreateNetworkInterfaceRequest;
}
/**
 * <p>Contains the output of CreateNetworkInterface.</p>
 */
export interface CreateNetworkInterfaceResult {
    __type?: "CreateNetworkInterfaceResult";
    /**
     * <p>Information about the network interface.</p>
     */
    NetworkInterface?: NetworkInterface;
}
export declare namespace CreateNetworkInterfaceResult {
    const filterSensitiveLog: (obj: CreateNetworkInterfaceResult) => any;
    const isa: (o: any) => o is CreateNetworkInterfaceResult;
}
export interface CreatePlacementGroupRequest {
    __type?: "CreatePlacementGroupRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>A name for the placement group. Must be unique within the scope of your account for
     *             the Region.</p>
     *         <p>Constraints: Up to 255 ASCII characters</p>
     */
    GroupName?: string;
    /**
     * <p>The number of partitions. Valid only when <b>Strategy</b> is
     *             set to <code>partition</code>.</p>
     */
    PartitionCount?: number;
    /**
     * <p>The placement strategy.</p>
     */
    Strategy?: PlacementStrategy | string;
}
export declare namespace CreatePlacementGroupRequest {
    const filterSensitiveLog: (obj: CreatePlacementGroupRequest) => any;
    const isa: (o: any) => o is CreatePlacementGroupRequest;
}
/**
 * <p>Contains the parameters for CreateReservedInstancesListing.</p>
 */
export interface CreateReservedInstancesListingRequest {
    __type?: "CreateReservedInstancesListingRequest";
    /**
     * <p>Unique, case-sensitive identifier you provide to ensure idempotency of your
     * 				listings. This helps avoid duplicate listings. For more information, see
     * 				<a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.</p>
     */
    ClientToken: string | undefined;
    /**
     * <p>The number of instances that are a part of a Reserved Instance account to be listed in the Reserved Instance Marketplace. This number should be less than or equal to the instance count associated with the Reserved Instance ID specified in this call.</p>
     */
    InstanceCount: number | undefined;
    /**
     * <p>A list specifying the price of the Standard Reserved Instance for each month remaining in the Reserved Instance term.</p>
     */
    PriceSchedules: PriceScheduleSpecification[] | undefined;
    /**
     * <p>The ID of the active Standard Reserved Instance.</p>
     */
    ReservedInstancesId: string | undefined;
}
export declare namespace CreateReservedInstancesListingRequest {
    const filterSensitiveLog: (obj: CreateReservedInstancesListingRequest) => any;
    const isa: (o: any) => o is CreateReservedInstancesListingRequest;
}
/**
 * <p>Contains the output of CreateReservedInstancesListing.</p>
 */
export interface CreateReservedInstancesListingResult {
    __type?: "CreateReservedInstancesListingResult";
    /**
     * <p>Information about the Standard Reserved Instance listing.</p>
     */
    ReservedInstancesListings?: ReservedInstancesListing[];
}
export declare namespace CreateReservedInstancesListingResult {
    const filterSensitiveLog: (obj: CreateReservedInstancesListingResult) => any;
    const isa: (o: any) => o is CreateReservedInstancesListingResult;
}
export interface CreateRouteRequest {
    __type?: "CreateRouteRequest";
    /**
     * <p>The IPv4 CIDR address block used for the destination match. Routing decisions are based on the most specific match.</p>
     */
    DestinationCidrBlock?: string;
    /**
     * <p>The IPv6 CIDR block used for the destination match. Routing decisions are based on the most specific match.</p>
     */
    DestinationIpv6CidrBlock?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>[IPv6 traffic only] The ID of an egress-only internet gateway.</p>
     */
    EgressOnlyInternetGatewayId?: string;
    /**
     * <p>The ID of an internet gateway or virtual private gateway attached to your
     * 			VPC.</p>
     */
    GatewayId?: string;
    /**
     * <p>The ID of a NAT instance in your VPC. The operation fails if you specify an instance ID unless exactly one network interface is attached.</p>
     */
    InstanceId?: string;
    /**
     * <p>The ID of the local gateway.</p>
     */
    LocalGatewayId?: string;
    /**
     * <p>[IPv4 traffic only] The ID of a NAT gateway.</p>
     */
    NatGatewayId?: string;
    /**
     * <p>The ID of a network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The ID of the route table for the route.</p>
     */
    RouteTableId: string | undefined;
    /**
     * <p>The ID of a transit gateway.</p>
     */
    TransitGatewayId?: string;
    /**
     * <p>The ID of a VPC peering connection.</p>
     */
    VpcPeeringConnectionId?: string;
}
export declare namespace CreateRouteRequest {
    const filterSensitiveLog: (obj: CreateRouteRequest) => any;
    const isa: (o: any) => o is CreateRouteRequest;
}
export interface CreateRouteResult {
    __type?: "CreateRouteResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace CreateRouteResult {
    const filterSensitiveLog: (obj: CreateRouteResult) => any;
    const isa: (o: any) => o is CreateRouteResult;
}
export interface CreateRouteTableRequest {
    __type?: "CreateRouteTableRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace CreateRouteTableRequest {
    const filterSensitiveLog: (obj: CreateRouteTableRequest) => any;
    const isa: (o: any) => o is CreateRouteTableRequest;
}
export interface CreateRouteTableResult {
    __type?: "CreateRouteTableResult";
    /**
     * <p>Information about the route table.</p>
     */
    RouteTable?: RouteTable;
}
export declare namespace CreateRouteTableResult {
    const filterSensitiveLog: (obj: CreateRouteTableResult) => any;
    const isa: (o: any) => o is CreateRouteTableResult;
}
export interface CreateSecurityGroupRequest {
    __type?: "CreateSecurityGroupRequest";
    /**
     * <p>A description for the security group. This is informational only.</p>
     * 		       <p>Constraints: Up to 255 characters in length</p>
     * 		       <p>Constraints for EC2-Classic: ASCII characters</p>
     * 		       <p>Constraints for EC2-VPC: a-z, A-Z, 0-9, spaces, and ._-:/()#,@[]+=&;{}!$*</p>
     */
    Description: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The name of the security group.</p>
     *          <p>Constraints: Up to 255 characters in length. Cannot start with
     *             <code>sg-</code>.</p>
     *          <p>Constraints for EC2-Classic: ASCII characters</p>
     *          <p>Constraints for EC2-VPC: a-z, A-Z, 0-9, spaces, and ._-:/()#,@[]+=&;{}!$*</p>
     */
    GroupName: string | undefined;
    /**
     * <p>[EC2-VPC] The ID of the VPC. Required for EC2-VPC.</p>
     */
    VpcId?: string;
}
export declare namespace CreateSecurityGroupRequest {
    const filterSensitiveLog: (obj: CreateSecurityGroupRequest) => any;
    const isa: (o: any) => o is CreateSecurityGroupRequest;
}
export interface CreateSecurityGroupResult {
    __type?: "CreateSecurityGroupResult";
    /**
     * <p>The ID of the security group.</p>
     */
    GroupId?: string;
}
export declare namespace CreateSecurityGroupResult {
    const filterSensitiveLog: (obj: CreateSecurityGroupResult) => any;
    const isa: (o: any) => o is CreateSecurityGroupResult;
}
export interface CreateSnapshotRequest {
    __type?: "CreateSnapshotRequest";
    /**
     * <p>A description for the snapshot.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The tags to apply to the snapshot during creation.</p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>The ID of the EBS volume.</p>
     */
    VolumeId: string | undefined;
}
export declare namespace CreateSnapshotRequest {
    const filterSensitiveLog: (obj: CreateSnapshotRequest) => any;
    const isa: (o: any) => o is CreateSnapshotRequest;
}
export interface CreateSnapshotsRequest {
    __type?: "CreateSnapshotsRequest";
    /**
     * <p>Copies the tags from the specified volume to corresponding snapshot.</p>
     */
    CopyTagsFromSource?: CopyTagsFromSource | string;
    /**
     * <p> A description propagated to every snapshot specified by the instance.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The instance to specify which volumes should be included in the snapshots.</p>
     */
    InstanceSpecification: InstanceSpecification | undefined;
    /**
     * <p>Tags to apply to every snapshot specified by the instance.</p>
     */
    TagSpecifications?: TagSpecification[];
}
export declare namespace CreateSnapshotsRequest {
    const filterSensitiveLog: (obj: CreateSnapshotsRequest) => any;
    const isa: (o: any) => o is CreateSnapshotsRequest;
}
export interface CreateSnapshotsResult {
    __type?: "CreateSnapshotsResult";
    /**
     * <p>List of snapshots.</p>
     */
    Snapshots?: SnapshotInfo[];
}
export declare namespace CreateSnapshotsResult {
    const filterSensitiveLog: (obj: CreateSnapshotsResult) => any;
    const isa: (o: any) => o is CreateSnapshotsResult;
}
/**
 * <p>Contains the parameters for CreateSpotDatafeedSubscription.</p>
 */
export interface CreateSpotDatafeedSubscriptionRequest {
    __type?: "CreateSpotDatafeedSubscriptionRequest";
    /**
     * <p>The Amazon S3 bucket in which to store the Spot Instance data feed.</p>
     */
    Bucket: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>A prefix for the data feed file names.</p>
     */
    Prefix?: string;
}
export declare namespace CreateSpotDatafeedSubscriptionRequest {
    const filterSensitiveLog: (obj: CreateSpotDatafeedSubscriptionRequest) => any;
    const isa: (o: any) => o is CreateSpotDatafeedSubscriptionRequest;
}
/**
 * <p>Contains the output of CreateSpotDatafeedSubscription.</p>
 */
export interface CreateSpotDatafeedSubscriptionResult {
    __type?: "CreateSpotDatafeedSubscriptionResult";
    /**
     * <p>The Spot Instance data feed subscription.</p>
     */
    SpotDatafeedSubscription?: SpotDatafeedSubscription;
}
export declare namespace CreateSpotDatafeedSubscriptionResult {
    const filterSensitiveLog: (obj: CreateSpotDatafeedSubscriptionResult) => any;
    const isa: (o: any) => o is CreateSpotDatafeedSubscriptionResult;
}
export interface CreateSubnetRequest {
    __type?: "CreateSubnetRequest";
    /**
     * <p>The Availability Zone or Local Zone for the subnet.</p>
     * 		       <p>Default: AWS selects one for you. If you create more than one subnet in your VPC, we do not necessarily select a different zone for each subnet.</p>
     *          <p>To create a subnet in a Local Zone, set this value to the Local Zone ID, for example
     *                 <code>us-west-2-lax-1a</code>. For information about the Regions that support Local
     *             Zones, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-regions-availability-zones.html#concepts-available-regions">Available Regions</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The AZ ID or the Local Zone ID of the subnet.</p>
     */
    AvailabilityZoneId?: string;
    /**
     * <p>The IPv4 network range for the subnet, in CIDR notation. For example, <code>10.0.0.0/24</code>.</p>
     */
    CidrBlock: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IPv6 network range for the subnet, in CIDR notation. The subnet size must use a
     *             /64 prefix length.</p>
     */
    Ipv6CidrBlock?: string;
    /**
     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
     */
    OutpostArn?: string;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace CreateSubnetRequest {
    const filterSensitiveLog: (obj: CreateSubnetRequest) => any;
    const isa: (o: any) => o is CreateSubnetRequest;
}
export interface CreateSubnetResult {
    __type?: "CreateSubnetResult";
    /**
     * <p>Information about the subnet.</p>
     */
    Subnet?: Subnet;
}
export declare namespace CreateSubnetResult {
    const filterSensitiveLog: (obj: CreateSubnetResult) => any;
    const isa: (o: any) => o is CreateSubnetResult;
}
export interface CreateTagsRequest {
    __type?: "CreateTagsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the resources, separated by spaces.</p>
     *    	     <p>Constraints: Up to 1000 resource IDs. We recommend breaking up this request into smaller batches.</p>
     */
    Resources: string[] | undefined;
    /**
     * <p>The tags. The <code>value</code> parameter is required, but if you don't want the tag to have a value,
     *         specify the parameter with no value, and we set the value to an empty string.</p>
     */
    Tags: Tag[] | undefined;
}
export declare namespace CreateTagsRequest {
    const filterSensitiveLog: (obj: CreateTagsRequest) => any;
    const isa: (o: any) => o is CreateTagsRequest;
}
export interface CreateTrafficMirrorFilterRequest {
    __type?: "CreateTrafficMirrorFilterRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The description of the Traffic Mirror filter.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The tags to assign to a Traffic Mirror filter.</p>
     */
    TagSpecifications?: TagSpecification[];
}
export declare namespace CreateTrafficMirrorFilterRequest {
    const filterSensitiveLog: (obj: CreateTrafficMirrorFilterRequest) => any;
    const isa: (o: any) => o is CreateTrafficMirrorFilterRequest;
}
export interface CreateTrafficMirrorFilterResult {
    __type?: "CreateTrafficMirrorFilterResult";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Information about the Traffic Mirror filter.</p>
     */
    TrafficMirrorFilter?: TrafficMirrorFilter;
}
export declare namespace CreateTrafficMirrorFilterResult {
    const filterSensitiveLog: (obj: CreateTrafficMirrorFilterResult) => any;
    const isa: (o: any) => o is CreateTrafficMirrorFilterResult;
}
export interface CreateTrafficMirrorFilterRuleRequest {
    __type?: "CreateTrafficMirrorFilterRuleRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The description of the Traffic Mirror rule.</p>
     */
    Description?: string;
    /**
     * <p>The destination CIDR block to assign to the Traffic Mirror rule.</p>
     */
    DestinationCidrBlock: string | undefined;
    /**
     * <p>The destination port range.</p>
     */
    DestinationPortRange?: TrafficMirrorPortRangeRequest;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The protocol, for example UDP, to assign to the Traffic Mirror rule.</p>
     *          <p>For information about the protocol value, see <a href="https://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml">Protocol Numbers</a> on the  Internet Assigned Numbers Authority (IANA) website.</p>
     */
    Protocol?: number;
    /**
     * <p>The action to take (<code>accept</code> | <code>reject</code>) on the filtered traffic.</p>
     */
    RuleAction: TrafficMirrorRuleAction | string | undefined;
    /**
     * <p>The number of the Traffic Mirror rule. This number must be unique for each Traffic Mirror rule in a given
     *          direction. The rules are processed in ascending order by rule number.</p>
     */
    RuleNumber: number | undefined;
    /**
     * <p>The source CIDR block to assign to the Traffic Mirror rule.</p>
     */
    SourceCidrBlock: string | undefined;
    /**
     * <p>The source port range.</p>
     */
    SourcePortRange?: TrafficMirrorPortRangeRequest;
    /**
     * <p>The type of traffic (<code>ingress</code> | <code>egress</code>).</p>
     */
    TrafficDirection: TrafficDirection | string | undefined;
    /**
     * <p>The ID of the filter that this rule is associated with.</p>
     */
    TrafficMirrorFilterId: string | undefined;
}
export declare namespace CreateTrafficMirrorFilterRuleRequest {
    const filterSensitiveLog: (obj: CreateTrafficMirrorFilterRuleRequest) => any;
    const isa: (o: any) => o is CreateTrafficMirrorFilterRuleRequest;
}
export interface CreateTrafficMirrorFilterRuleResult {
    __type?: "CreateTrafficMirrorFilterRuleResult";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The Traffic Mirror rule.</p>
     */
    TrafficMirrorFilterRule?: TrafficMirrorFilterRule;
}
export declare namespace CreateTrafficMirrorFilterRuleResult {
    const filterSensitiveLog: (obj: CreateTrafficMirrorFilterRuleResult) => any;
    const isa: (o: any) => o is CreateTrafficMirrorFilterRuleResult;
}
export interface CreateTrafficMirrorSessionRequest {
    __type?: "CreateTrafficMirrorSessionRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The description of the Traffic Mirror session.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the source network interface.</p>
     */
    NetworkInterfaceId: string | undefined;
    /**
     * <p>The number of bytes in each packet to mirror. These are bytes after the VXLAN header. Do
     *          not specify this parameter when you want to mirror the entire packet. To mirror a subset of
     *          the packet, set this to the length (in bytes) that you want to mirror. For example, if you
     *          set this value to 100, then the first 100 bytes that meet the filter criteria are copied to
     *          the target.</p>
     *          <p>If you do not want to mirror the entire packet, use the <code>PacketLength</code> parameter to specify the number of bytes in each packet to mirror.</p>
     */
    PacketLength?: number;
    /**
     * <p>The session number determines the order in which sessions are evaluated when an interface is used by multiple sessions. The first session with a matching filter is the one that mirrors the packets.</p>
     *          <p>Valid values are 1-32766.</p>
     */
    SessionNumber: number | undefined;
    /**
     * <p>The tags to assign to a Traffic Mirror session.</p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>The ID of the Traffic Mirror filter.</p>
     */
    TrafficMirrorFilterId: string | undefined;
    /**
     * <p>The ID of the Traffic Mirror target.</p>
     */
    TrafficMirrorTargetId: string | undefined;
    /**
     * <p>The VXLAN ID for the Traffic Mirror session. For more information about the VXLAN
     *          protocol, see <a href="https://tools.ietf.org/html/rfc7348">RFC 7348</a>. If you do
     *          not specify a <code>VirtualNetworkId</code>, an account-wide unique id is chosen at
     *          random.</p>
     */
    VirtualNetworkId?: number;
}
export declare namespace CreateTrafficMirrorSessionRequest {
    const filterSensitiveLog: (obj: CreateTrafficMirrorSessionRequest) => any;
    const isa: (o: any) => o is CreateTrafficMirrorSessionRequest;
}
export interface CreateTrafficMirrorSessionResult {
    __type?: "CreateTrafficMirrorSessionResult";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Information about the Traffic Mirror session.</p>
     */
    TrafficMirrorSession?: TrafficMirrorSession;
}
export declare namespace CreateTrafficMirrorSessionResult {
    const filterSensitiveLog: (obj: CreateTrafficMirrorSessionResult) => any;
    const isa: (o: any) => o is CreateTrafficMirrorSessionResult;
}
export interface CreateTrafficMirrorTargetRequest {
    __type?: "CreateTrafficMirrorTargetRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The description of the Traffic Mirror target.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The network interface ID that is associated with the target.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The Amazon Resource Name (ARN) of the Network Load Balancer that is associated with the target.</p>
     */
    NetworkLoadBalancerArn?: string;
    /**
     * <p>The tags to assign to the Traffic Mirror target.</p>
     */
    TagSpecifications?: TagSpecification[];
}
export declare namespace CreateTrafficMirrorTargetRequest {
    const filterSensitiveLog: (obj: CreateTrafficMirrorTargetRequest) => any;
    const isa: (o: any) => o is CreateTrafficMirrorTargetRequest;
}
export interface CreateTrafficMirrorTargetResult {
    __type?: "CreateTrafficMirrorTargetResult";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Information about the Traffic Mirror target.</p>
     */
    TrafficMirrorTarget?: TrafficMirrorTarget;
}
export declare namespace CreateTrafficMirrorTargetResult {
    const filterSensitiveLog: (obj: CreateTrafficMirrorTargetResult) => any;
    const isa: (o: any) => o is CreateTrafficMirrorTargetResult;
}
export interface CreateTransitGatewayMulticastDomainRequest {
    __type?: "CreateTransitGatewayMulticastDomainRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The tags for the transit gateway multicast domain.</p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId: string | undefined;
}
export declare namespace CreateTransitGatewayMulticastDomainRequest {
    const filterSensitiveLog: (obj: CreateTransitGatewayMulticastDomainRequest) => any;
    const isa: (o: any) => o is CreateTransitGatewayMulticastDomainRequest;
}
export interface CreateTransitGatewayMulticastDomainResult {
    __type?: "CreateTransitGatewayMulticastDomainResult";
    /**
     * <p>Information about the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomain?: TransitGatewayMulticastDomain;
}
export declare namespace CreateTransitGatewayMulticastDomainResult {
    const filterSensitiveLog: (obj: CreateTransitGatewayMulticastDomainResult) => any;
    const isa: (o: any) => o is CreateTransitGatewayMulticastDomainResult;
}
export interface CreateTransitGatewayPeeringAttachmentRequest {
    __type?: "CreateTransitGatewayPeeringAttachmentRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The AWS account ID of the owner of the peer transit gateway.</p>
     */
    PeerAccountId: string | undefined;
    /**
     * <p>The Region where the peer transit gateway is located.</p>
     */
    PeerRegion: string | undefined;
    /**
     * <p>The ID of the peer transit gateway with which to create the peering attachment.</p>
     */
    PeerTransitGatewayId: string | undefined;
    /**
     * <p>The tags to apply to the transit gateway peering attachment.</p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId: string | undefined;
}
export declare namespace CreateTransitGatewayPeeringAttachmentRequest {
    const filterSensitiveLog: (obj: CreateTransitGatewayPeeringAttachmentRequest) => any;
    const isa: (o: any) => o is CreateTransitGatewayPeeringAttachmentRequest;
}
export interface CreateTransitGatewayPeeringAttachmentResult {
    __type?: "CreateTransitGatewayPeeringAttachmentResult";
    /**
     * <p>The transit gateway peering attachment.</p>
     */
    TransitGatewayPeeringAttachment?: TransitGatewayPeeringAttachment;
}
export declare namespace CreateTransitGatewayPeeringAttachmentResult {
    const filterSensitiveLog: (obj: CreateTransitGatewayPeeringAttachmentResult) => any;
    const isa: (o: any) => o is CreateTransitGatewayPeeringAttachmentResult;
}
export interface CreateTransitGatewayRequest {
    __type?: "CreateTransitGatewayRequest";
    /**
     * <p>A description of the transit gateway.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The transit gateway options.</p>
     */
    Options?: TransitGatewayRequestOptions;
    /**
     * <p>The tags to apply to the transit gateway.</p>
     */
    TagSpecifications?: TagSpecification[];
}
export declare namespace CreateTransitGatewayRequest {
    const filterSensitiveLog: (obj: CreateTransitGatewayRequest) => any;
    const isa: (o: any) => o is CreateTransitGatewayRequest;
}
export interface CreateTransitGatewayResult {
    __type?: "CreateTransitGatewayResult";
    /**
     * <p>Information about the transit gateway.</p>
     */
    TransitGateway?: TransitGateway;
}
export declare namespace CreateTransitGatewayResult {
    const filterSensitiveLog: (obj: CreateTransitGatewayResult) => any;
    const isa: (o: any) => o is CreateTransitGatewayResult;
}
export interface CreateTransitGatewayRouteRequest {
    __type?: "CreateTransitGatewayRouteRequest";
    /**
     * <p>Indicates whether to drop traffic that matches this route.</p>
     */
    Blackhole?: boolean;
    /**
     * <p>The CIDR range used for destination matches. Routing decisions are based on the
     *          most specific match.</p>
     */
    DestinationCidrBlock: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
    /**
     * <p>The ID of the transit gateway route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace CreateTransitGatewayRouteRequest {
    const filterSensitiveLog: (obj: CreateTransitGatewayRouteRequest) => any;
    const isa: (o: any) => o is CreateTransitGatewayRouteRequest;
}
export interface CreateTransitGatewayRouteResult {
    __type?: "CreateTransitGatewayRouteResult";
    /**
     * <p>Information about the route.</p>
     */
    Route?: TransitGatewayRoute;
}
export declare namespace CreateTransitGatewayRouteResult {
    const filterSensitiveLog: (obj: CreateTransitGatewayRouteResult) => any;
    const isa: (o: any) => o is CreateTransitGatewayRouteResult;
}
export interface CreateTransitGatewayRouteTableRequest {
    __type?: "CreateTransitGatewayRouteTableRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The tags to apply to the transit gateway route table.</p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId: string | undefined;
}
export declare namespace CreateTransitGatewayRouteTableRequest {
    const filterSensitiveLog: (obj: CreateTransitGatewayRouteTableRequest) => any;
    const isa: (o: any) => o is CreateTransitGatewayRouteTableRequest;
}
export interface CreateTransitGatewayRouteTableResult {
    __type?: "CreateTransitGatewayRouteTableResult";
    /**
     * <p>Information about the transit gateway route table.</p>
     */
    TransitGatewayRouteTable?: TransitGatewayRouteTable;
}
export declare namespace CreateTransitGatewayRouteTableResult {
    const filterSensitiveLog: (obj: CreateTransitGatewayRouteTableResult) => any;
    const isa: (o: any) => o is CreateTransitGatewayRouteTableResult;
}
export interface CreateTransitGatewayVpcAttachmentRequest {
    __type?: "CreateTransitGatewayVpcAttachmentRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The VPC attachment options.</p>
     */
    Options?: CreateTransitGatewayVpcAttachmentRequestOptions;
    /**
     * <p>The IDs of one or more subnets. You can specify only one subnet per Availability Zone.
     *          You must specify at least one subnet, but we recommend that you specify two subnets for better availability.
     *          The transit gateway uses one IP address from each specified subnet.</p>
     */
    SubnetIds: string[] | undefined;
    /**
     * <p>The tags to apply to the VPC attachment.</p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId: string | undefined;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace CreateTransitGatewayVpcAttachmentRequest {
    const filterSensitiveLog: (obj: CreateTransitGatewayVpcAttachmentRequest) => any;
    const isa: (o: any) => o is CreateTransitGatewayVpcAttachmentRequest;
}
/**
 * <p>Describes the options for a VPC attachment.</p>
 */
export interface CreateTransitGatewayVpcAttachmentRequestOptions {
    __type?: "CreateTransitGatewayVpcAttachmentRequestOptions";
    /**
     * <p>Enable or disable DNS support. The default is <code>enable</code>.</p>
     */
    DnsSupport?: DnsSupportValue | string;
    /**
     * <p>Enable or disable IPv6 support. The default is <code>enable</code>.</p>
     */
    Ipv6Support?: Ipv6SupportValue | string;
}
export declare namespace CreateTransitGatewayVpcAttachmentRequestOptions {
    const filterSensitiveLog: (obj: CreateTransitGatewayVpcAttachmentRequestOptions) => any;
    const isa: (o: any) => o is CreateTransitGatewayVpcAttachmentRequestOptions;
}
export interface CreateTransitGatewayVpcAttachmentResult {
    __type?: "CreateTransitGatewayVpcAttachmentResult";
    /**
     * <p>Information about the VPC attachment.</p>
     */
    TransitGatewayVpcAttachment?: TransitGatewayVpcAttachment;
}
export declare namespace CreateTransitGatewayVpcAttachmentResult {
    const filterSensitiveLog: (obj: CreateTransitGatewayVpcAttachmentResult) => any;
    const isa: (o: any) => o is CreateTransitGatewayVpcAttachmentResult;
}
/**
 * <p>Describes the user or group to be added or removed from the list of create volume
 *       permissions for a volume.</p>
 */
export interface CreateVolumePermission {
    __type?: "CreateVolumePermission";
    /**
     * <p>The group to be added or removed. The possible value is <code>all</code>.</p>
     */
    Group?: PermissionGroup | string;
    /**
     * <p>The AWS account ID to be added or removed.</p>
     */
    UserId?: string;
}
export declare namespace CreateVolumePermission {
    const filterSensitiveLog: (obj: CreateVolumePermission) => any;
    const isa: (o: any) => o is CreateVolumePermission;
}
/**
 * <p>Describes modifications to the list of create volume permissions for a volume.</p>
 */
export interface CreateVolumePermissionModifications {
    __type?: "CreateVolumePermissionModifications";
    /**
     * <p>Adds the specified AWS account ID or group to the list.</p>
     */
    Add?: CreateVolumePermission[];
    /**
     * <p>Removes the specified AWS account ID or group from the list.</p>
     */
    Remove?: CreateVolumePermission[];
}
export declare namespace CreateVolumePermissionModifications {
    const filterSensitiveLog: (obj: CreateVolumePermissionModifications) => any;
    const isa: (o: any) => o is CreateVolumePermissionModifications;
}
export interface CreateVolumeRequest {
    __type?: "CreateVolumeRequest";
    /**
     * <p>The Availability Zone in which to create the volume.</p>
     */
    AvailabilityZone: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Specifies whether the volume should be encrypted.
     *       The effect of setting the encryption state to <code>true</code> depends on
     * the volume origin (new or from a snapshot), starting encryption state, ownership, and whether encryption by default is enabled.
     *       For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html#encryption-by-default">Encryption by Default</a>
     *       in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *          <p>Encrypted Amazon EBS volumes must be attached to instances that support Amazon EBS encryption.
     *       For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html#EBSEncryption_supported_instances">Supported
     *         Instance Types</a>.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>The number of I/O operations per second (IOPS) to provision for the volume, with a maximum
     *       ratio of 50 IOPS/GiB. Range is 100 to 64,000 IOPS for
     *       volumes in most Regions. Maximum IOPS of 64,000 is guaranteed only on
     *         <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html#ec2-nitro-instances">Nitro-based instances</a>. Other instance families guarantee performance up to
     *       32,000 IOPS. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html">Amazon EBS Volume Types</a> in the
     *         <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *          <p>This parameter is valid only for Provisioned IOPS SSD (io1) volumes.</p>
     */
    Iops?: number;
    /**
     * <p>The identifier of the AWS Key Management Service (AWS KMS) customer master key (CMK) to use for Amazon EBS encryption.
     *       If this parameter is not specified, your AWS managed CMK for EBS is used. If <code>KmsKeyId</code> is
     *       specified, the encrypted state must be <code>true</code>.</p>
     *          <p>You can specify the CMK using any of the following:</p>
     *          <ul>
     *             <li>
     *                <p>Key ID. For example, key/1234abcd-12ab-34cd-56ef-1234567890ab.</p>
     *             </li>
     *             <li>
     *                <p>Key alias. For example, alias/ExampleAlias.</p>
     *             </li>
     *             <li>
     *                <p>Key ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:key/<i>abcd1234-a123-456a-a12b-a123b4cd56ef</i>.</p>
     *             </li>
     *             <li>
     *                <p>Alias ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>.</p>
     *             </li>
     *          </ul>
     *          <p>AWS authenticates the CMK asynchronously. Therefore, if you specify an ID, alias, or ARN that is not valid,
     *       the action can appear to complete, but eventually fails.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
     */
    OutpostArn?: string;
    /**
     * <p>The size of the volume, in GiBs.</p>
     *          <p>Constraints: 1-16,384 for <code>gp2</code>,
     *       4-16,384 for <code>io1</code>, 500-16,384 for
     *         <code>st1</code>, 500-16,384 for <code>sc1</code>, and
     *       1-1,024 for <code>standard</code>. If you specify a
     *       snapshot, the volume size must be equal to or larger than the snapshot size.</p>
     *          <p>Default: If you're creating the volume from a snapshot and don't specify a volume size,
     *       the default is the snapshot size.</p>
     *          <note>
     *             <p>At least one of Size or SnapshotId is required.</p>
     *          </note>
     */
    Size?: number;
    /**
     * <p>The snapshot from which to create the volume.</p>
     *          <note>
     *             <p>At least one of Size or SnapshotId are required.</p>
     *          </note>
     */
    SnapshotId?: string;
    /**
     * <p>The tags to apply to the volume during creation.</p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>The volume type. This can be <code>gp2</code> for General Purpose SSD, <code>io1</code> for Provisioned IOPS SSD,
     *         <code>st1</code> for Throughput Optimized HDD, <code>sc1</code> for Cold HDD, or
     *         <code>standard</code> for Magnetic volumes.</p>
     *          <p>Default: <code>gp2</code>
     *          </p>
     */
    VolumeType?: VolumeType | string;
}
export declare namespace CreateVolumeRequest {
    const filterSensitiveLog: (obj: CreateVolumeRequest) => any;
    const isa: (o: any) => o is CreateVolumeRequest;
}
export interface CreateVpcEndpointConnectionNotificationRequest {
    __type?: "CreateVpcEndpointConnectionNotificationRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     *             request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure
     *                 Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>One or more endpoint events for which to receive notifications. Valid values are
     *                 <code>Accept</code>, <code>Connect</code>, <code>Delete</code>, and
     *                 <code>Reject</code>.</p>
     */
    ConnectionEvents: string[] | undefined;
    /**
     * <p>The ARN of the SNS topic for the notifications.</p>
     */
    ConnectionNotificationArn: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the endpoint service.</p>
     */
    ServiceId?: string;
    /**
     * <p>The ID of the endpoint.</p>
     */
    VpcEndpointId?: string;
}
export declare namespace CreateVpcEndpointConnectionNotificationRequest {
    const filterSensitiveLog: (obj: CreateVpcEndpointConnectionNotificationRequest) => any;
    const isa: (o: any) => o is CreateVpcEndpointConnectionNotificationRequest;
}
export interface CreateVpcEndpointConnectionNotificationResult {
    __type?: "CreateVpcEndpointConnectionNotificationResult";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     *             request.</p>
     */
    ClientToken?: string;
    /**
     * <p>Information about the notification.</p>
     */
    ConnectionNotification?: ConnectionNotification;
}
export declare namespace CreateVpcEndpointConnectionNotificationResult {
    const filterSensitiveLog: (obj: CreateVpcEndpointConnectionNotificationResult) => any;
    const isa: (o: any) => o is CreateVpcEndpointConnectionNotificationResult;
}
/**
 * <p>Contains the parameters for CreateVpcEndpoint.</p>
 */
export interface CreateVpcEndpointRequest {
    __type?: "CreateVpcEndpointRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     *             request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure
     *                 Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>A policy to attach to the endpoint that controls access to the
     *             service. The policy must be in valid JSON format. If this parameter is not specified, we
     *             attach a default policy that allows full access to the service.</p>
     */
    PolicyDocument?: string;
    /**
     * <p>(Interface endpoint) Indicates whether to associate a private hosted zone with the
     *             specified VPC. The private hosted zone contains a record set for the default public DNS
     *             name for the service for the Region (for example,
     *                 <code>kinesis.us-east-1.amazonaws.com</code>), which resolves to the private IP
     *             addresses of the endpoint network interfaces in the VPC. This enables you to make
     *             requests to the default public DNS name for the service instead of the public DNS names
     *             that are automatically generated by the VPC endpoint service.</p>
     *         <p>To use a private hosted zone, you must set the following VPC attributes to
     *             <code>true</code>: <code>enableDnsHostnames</code> and
     *             <code>enableDnsSupport</code>. Use <a>ModifyVpcAttribute</a> to set the VPC
     *             attributes.</p>
     *         <p>Default: <code>true</code>
     *          </p>
     */
    PrivateDnsEnabled?: boolean;
    /**
     * <p>(Gateway endpoint) One or more route table IDs.</p>
     */
    RouteTableIds?: string[];
    /**
     * <p>(Interface endpoint) The ID of one or more security groups to associate with the
     *             endpoint network interface.</p>
     */
    SecurityGroupIds?: string[];
    /**
     * <p>The service name. To get a list of available services, use the <a>DescribeVpcEndpointServices</a> request, or get the name from the service
     *             provider.</p>
     */
    ServiceName: string | undefined;
    /**
     * <p>(Interface endpoint) The ID of one or more subnets in which to create an endpoint
     *             network interface.</p>
     */
    SubnetIds?: string[];
    /**
     * <p>The type of endpoint.</p>
     *         <p>Default: Gateway</p>
     */
    VpcEndpointType?: VpcEndpointType | string;
    /**
     * <p>The ID of the VPC in which the endpoint will be used.</p>
     */
    VpcId: string | undefined;
}
export declare namespace CreateVpcEndpointRequest {
    const filterSensitiveLog: (obj: CreateVpcEndpointRequest) => any;
    const isa: (o: any) => o is CreateVpcEndpointRequest;
}
/**
 * <p>Contains the output of CreateVpcEndpoint.</p>
 */
export interface CreateVpcEndpointResult {
    __type?: "CreateVpcEndpointResult";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     *             request.</p>
     */
    ClientToken?: string;
    /**
     * <p>Information about the endpoint.</p>
     */
    VpcEndpoint?: VpcEndpoint;
}
export declare namespace CreateVpcEndpointResult {
    const filterSensitiveLog: (obj: CreateVpcEndpointResult) => any;
    const isa: (o: any) => o is CreateVpcEndpointResult;
}
export interface CreateVpcEndpointServiceConfigurationRequest {
    __type?: "CreateVpcEndpointServiceConfigurationRequest";
    /**
     * <p>Indicates whether requests from service consumers to create an endpoint to your service must
     *             be accepted. To accept a request, use <a>AcceptVpcEndpointConnections</a>.</p>
     */
    AcceptanceRequired?: boolean;
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.
     *             For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">How to Ensure
     *                 Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The Amazon Resource Names (ARNs) of one or more Network Load Balancers for your
     *             service.</p>
     */
    NetworkLoadBalancerArns: string[] | undefined;
    /**
     * <p>The private DNS name to assign to the VPC endpoint service.</p>
     */
    PrivateDnsName?: string;
}
export declare namespace CreateVpcEndpointServiceConfigurationRequest {
    const filterSensitiveLog: (obj: CreateVpcEndpointServiceConfigurationRequest) => any;
    const isa: (o: any) => o is CreateVpcEndpointServiceConfigurationRequest;
}
export interface CreateVpcEndpointServiceConfigurationResult {
    __type?: "CreateVpcEndpointServiceConfigurationResult";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     *             request.</p>
     */
    ClientToken?: string;
    /**
     * <p>Information about the service configuration.</p>
     */
    ServiceConfiguration?: ServiceConfiguration;
}
export declare namespace CreateVpcEndpointServiceConfigurationResult {
    const filterSensitiveLog: (obj: CreateVpcEndpointServiceConfigurationResult) => any;
    const isa: (o: any) => o is CreateVpcEndpointServiceConfigurationResult;
}
export interface CreateVpcPeeringConnectionRequest {
    __type?: "CreateVpcPeeringConnectionRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The AWS account ID of the owner of the accepter VPC.</p>
     *          <p>Default: Your AWS account ID</p>
     */
    PeerOwnerId?: string;
    /**
     * <p>The Region code for the accepter VPC, if the accepter VPC is located in a Region
     *             other than the Region in which you make the request.</p>
     * 		       <p>Default: The Region in which you make the request.</p>
     */
    PeerRegion?: string;
    /**
     * <p>The ID of the VPC with which you are creating the VPC peering connection. You must
     * 			specify this parameter in the request.</p>
     */
    PeerVpcId?: string;
    /**
     * <p>The ID of the requester VPC. You must specify this parameter in the
     * 			request.</p>
     */
    VpcId?: string;
}
export declare namespace CreateVpcPeeringConnectionRequest {
    const filterSensitiveLog: (obj: CreateVpcPeeringConnectionRequest) => any;
    const isa: (o: any) => o is CreateVpcPeeringConnectionRequest;
}
export interface CreateVpcPeeringConnectionResult {
    __type?: "CreateVpcPeeringConnectionResult";
    /**
     * <p>Information about the VPC peering connection.</p>
     */
    VpcPeeringConnection?: VpcPeeringConnection;
}
export declare namespace CreateVpcPeeringConnectionResult {
    const filterSensitiveLog: (obj: CreateVpcPeeringConnectionResult) => any;
    const isa: (o: any) => o is CreateVpcPeeringConnectionResult;
}
export interface CreateVpcRequest {
    __type?: "CreateVpcRequest";
    /**
     * <p>Requests an Amazon-provided IPv6 CIDR block with a /56 prefix length for the VPC.
     *             You cannot specify the range of IP addresses, or the size of the CIDR block.</p>
     */
    AmazonProvidedIpv6CidrBlock?: boolean;
    /**
     * <p>The IPv4 network range for the VPC, in CIDR notation. For example,
     *                 <code>10.0.0.0/16</code>.</p>
     */
    CidrBlock: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The tenancy options for instances launched into the VPC. For <code>default</code>, instances
     *       are launched with shared tenancy by default. You can launch instances with any tenancy into a
     *       shared tenancy VPC. For <code>dedicated</code>, instances are launched as dedicated tenancy
     *       instances by default. You can only launch instances with a tenancy of <code>dedicated</code>
     *       or <code>host</code> into a dedicated tenancy VPC. </p>
     *          <p>
     *             <b>Important:</b> The <code>host</code> value cannot be used with this parameter. Use the <code>default</code> or <code>dedicated</code> values only.</p>
     *          <p>Default: <code>default</code>
     *          </p>
     */
    InstanceTenancy?: Tenancy | string;
    /**
     * <p>The IPv6 CIDR block from the IPv6 address pool. You must also specify <code>Ipv6Pool</code> in the request.</p>
     *         <p>To let Amazon choose the IPv6 CIDR block for you, omit this parameter.</p>
     */
    Ipv6CidrBlock?: string;
    /**
     * <p>The name of the location from which we advertise the IPV6 CIDR block. Use this parameter to limit the address to this location.</p>
     *          <p> You must set <code>AmazonProvidedIpv6CidrBlock</code> to <code>true</code> to use this parameter.</p>
     */
    Ipv6CidrBlockNetworkBorderGroup?: string;
    /**
     * <p>The ID of an IPv6 address pool from which to allocate the IPv6 CIDR block.</p>
     */
    Ipv6Pool?: string;
}
export declare namespace CreateVpcRequest {
    const filterSensitiveLog: (obj: CreateVpcRequest) => any;
    const isa: (o: any) => o is CreateVpcRequest;
}
export interface CreateVpcResult {
    __type?: "CreateVpcResult";
    /**
     * <p>Information about the VPC.</p>
     */
    Vpc?: Vpc;
}
export declare namespace CreateVpcResult {
    const filterSensitiveLog: (obj: CreateVpcResult) => any;
    const isa: (o: any) => o is CreateVpcResult;
}
/**
 * <p>Contains the parameters for CreateVpnConnection.</p>
 */
export interface CreateVpnConnectionRequest {
    __type?: "CreateVpnConnectionRequest";
    /**
     * <p>The ID of the customer gateway.</p>
     */
    CustomerGatewayId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The options for the VPN connection.</p>
     */
    Options?: VpnConnectionOptionsSpecification;
    /**
     * <p>The ID of the transit gateway. If you specify a transit gateway, you cannot specify a virtual private gateway.</p>
     */
    TransitGatewayId?: string;
    /**
     * <p>The type of VPN connection (<code>ipsec.1</code>).</p>
     */
    Type: string | undefined;
    /**
     * <p>The ID of the virtual private gateway. If you specify a virtual private gateway, you cannot specify a transit gateway.</p>
     */
    VpnGatewayId?: string;
}
export declare namespace CreateVpnConnectionRequest {
    const filterSensitiveLog: (obj: CreateVpnConnectionRequest) => any;
    const isa: (o: any) => o is CreateVpnConnectionRequest;
}
/**
 * <p>Contains the output of CreateVpnConnection.</p>
 */
export interface CreateVpnConnectionResult {
    __type?: "CreateVpnConnectionResult";
    /**
     * <p>Information about the VPN connection.</p>
     */
    VpnConnection?: VpnConnection;
}
export declare namespace CreateVpnConnectionResult {
    const filterSensitiveLog: (obj: CreateVpnConnectionResult) => any;
    const isa: (o: any) => o is CreateVpnConnectionResult;
}
/**
 * <p>Contains the parameters for CreateVpnConnectionRoute.</p>
 */
export interface CreateVpnConnectionRouteRequest {
    __type?: "CreateVpnConnectionRouteRequest";
    /**
     * <p>The CIDR block associated with the local subnet of the customer network.</p>
     */
    DestinationCidrBlock: string | undefined;
    /**
     * <p>The ID of the VPN connection.</p>
     */
    VpnConnectionId: string | undefined;
}
export declare namespace CreateVpnConnectionRouteRequest {
    const filterSensitiveLog: (obj: CreateVpnConnectionRouteRequest) => any;
    const isa: (o: any) => o is CreateVpnConnectionRouteRequest;
}
/**
 * <p>Contains the parameters for CreateVpnGateway.</p>
 */
export interface CreateVpnGatewayRequest {
    __type?: "CreateVpnGatewayRequest";
    /**
     * <p>A private Autonomous System Number (ASN) for the Amazon side of a BGP session. If you're using a 16-bit ASN, it must be in the 64512 to 65534 range. If you're using a 32-bit ASN, it must be in the 4200000000 to 4294967294 range.</p>
     * 		       <p>Default: 64512</p>
     */
    AmazonSideAsn?: number;
    /**
     * <p>The Availability Zone for the virtual private gateway.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The type of VPN connection this virtual private gateway supports.</p>
     */
    Type: GatewayType | string | undefined;
}
export declare namespace CreateVpnGatewayRequest {
    const filterSensitiveLog: (obj: CreateVpnGatewayRequest) => any;
    const isa: (o: any) => o is CreateVpnGatewayRequest;
}
/**
 * <p>Contains the output of CreateVpnGateway.</p>
 */
export interface CreateVpnGatewayResult {
    __type?: "CreateVpnGatewayResult";
    /**
     * <p>Information about the virtual private gateway.</p>
     */
    VpnGateway?: VpnGateway;
}
export declare namespace CreateVpnGatewayResult {
    const filterSensitiveLog: (obj: CreateVpnGatewayResult) => any;
    const isa: (o: any) => o is CreateVpnGatewayResult;
}
/**
 * <p>Describes the credit option for CPU usage of a T2 or T3 instance.</p>
 */
export interface CreditSpecification {
    __type?: "CreditSpecification";
    /**
     * <p>The credit option for CPU usage of a T2 or T3 instance. Valid values are
     *                 <code>standard</code> and <code>unlimited</code>.</p>
     */
    CpuCredits?: string;
}
export declare namespace CreditSpecification {
    const filterSensitiveLog: (obj: CreditSpecification) => any;
    const isa: (o: any) => o is CreditSpecification;
}
/**
 * <p>The credit option for CPU usage of a T2 or T3 instance.</p>
 */
export interface CreditSpecificationRequest {
    __type?: "CreditSpecificationRequest";
    /**
     * <p>The credit option for CPU usage of a T2 or T3 instance. Valid values are
     *                 <code>standard</code> and <code>unlimited</code>.</p>
     */
    CpuCredits: string | undefined;
}
export declare namespace CreditSpecificationRequest {
    const filterSensitiveLog: (obj: CreditSpecificationRequest) => any;
    const isa: (o: any) => o is CreditSpecificationRequest;
}
export declare type CurrencyCodeValues = "USD";
/**
 * <p>Describes a customer gateway.</p>
 */
export interface CustomerGateway {
    __type?: "CustomerGateway";
    /**
     * <p>The customer gateway's Border Gateway Protocol (BGP) Autonomous System Number (ASN).</p>
     */
    BgpAsn?: string;
    /**
     * <p>The Amazon Resource Name (ARN) for the customer gateway certificate.</p>
     */
    CertificateArn?: string;
    /**
     * <p>The ID of the customer gateway.</p>
     */
    CustomerGatewayId?: string;
    /**
     * <p>The name of customer gateway device.</p>
     */
    DeviceName?: string;
    /**
     * <p>The Internet-routable IP address of the customer gateway's outside interface.</p>
     */
    IpAddress?: string;
    /**
     * <p>The current state of the customer gateway (<code>pending | available | deleting |
     *             deleted</code>).</p>
     */
    State?: string;
    /**
     * <p>Any tags assigned to the customer gateway.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The type of VPN connection the customer gateway supports (<code>ipsec.1</code>).</p>
     */
    Type?: string;
}
export declare namespace CustomerGateway {
    const filterSensitiveLog: (obj: CustomerGateway) => any;
    const isa: (o: any) => o is CustomerGateway;
}
export declare type DatafeedSubscriptionState = "Active" | "Inactive";
export declare type DefaultRouteTableAssociationValue = "disable" | "enable";
export declare type DefaultRouteTablePropagationValue = "disable" | "enable";
export declare enum DefaultTargetCapacityType {
    ON_DEMAND = "on-demand",
    SPOT = "spot"
}
export interface DeleteClientVpnEndpointRequest {
    __type?: "DeleteClientVpnEndpointRequest";
    /**
     * <p>The ID of the Client VPN to be deleted.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DeleteClientVpnEndpointRequest {
    const filterSensitiveLog: (obj: DeleteClientVpnEndpointRequest) => any;
    const isa: (o: any) => o is DeleteClientVpnEndpointRequest;
}
export interface DeleteClientVpnEndpointResult {
    __type?: "DeleteClientVpnEndpointResult";
    /**
     * <p>The current state of the Client VPN endpoint.</p>
     */
    Status?: ClientVpnEndpointStatus;
}
export declare namespace DeleteClientVpnEndpointResult {
    const filterSensitiveLog: (obj: DeleteClientVpnEndpointResult) => any;
    const isa: (o: any) => o is DeleteClientVpnEndpointResult;
}
export interface DeleteClientVpnRouteRequest {
    __type?: "DeleteClientVpnRouteRequest";
    /**
     * <p>The ID of the Client VPN endpoint from which the route is to be deleted.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>The IPv4 address range, in CIDR notation, of the route to be deleted.</p>
     */
    DestinationCidrBlock: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the target subnet used by the route.</p>
     */
    TargetVpcSubnetId?: string;
}
export declare namespace DeleteClientVpnRouteRequest {
    const filterSensitiveLog: (obj: DeleteClientVpnRouteRequest) => any;
    const isa: (o: any) => o is DeleteClientVpnRouteRequest;
}
export interface DeleteClientVpnRouteResult {
    __type?: "DeleteClientVpnRouteResult";
    /**
     * <p>The current state of the route.</p>
     */
    Status?: ClientVpnRouteStatus;
}
export declare namespace DeleteClientVpnRouteResult {
    const filterSensitiveLog: (obj: DeleteClientVpnRouteResult) => any;
    const isa: (o: any) => o is DeleteClientVpnRouteResult;
}
/**
 * <p>Contains the parameters for DeleteCustomerGateway.</p>
 */
export interface DeleteCustomerGatewayRequest {
    __type?: "DeleteCustomerGatewayRequest";
    /**
     * <p>The ID of the customer gateway.</p>
     */
    CustomerGatewayId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DeleteCustomerGatewayRequest {
    const filterSensitiveLog: (obj: DeleteCustomerGatewayRequest) => any;
    const isa: (o: any) => o is DeleteCustomerGatewayRequest;
}
export interface DeleteDhcpOptionsRequest {
    __type?: "DeleteDhcpOptionsRequest";
    /**
     * <p>The ID of the DHCP options set.</p>
     */
    DhcpOptionsId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DeleteDhcpOptionsRequest {
    const filterSensitiveLog: (obj: DeleteDhcpOptionsRequest) => any;
    const isa: (o: any) => o is DeleteDhcpOptionsRequest;
}
export interface DeleteEgressOnlyInternetGatewayRequest {
    __type?: "DeleteEgressOnlyInternetGatewayRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the egress-only internet gateway.</p>
     */
    EgressOnlyInternetGatewayId: string | undefined;
}
export declare namespace DeleteEgressOnlyInternetGatewayRequest {
    const filterSensitiveLog: (obj: DeleteEgressOnlyInternetGatewayRequest) => any;
    const isa: (o: any) => o is DeleteEgressOnlyInternetGatewayRequest;
}
export interface DeleteEgressOnlyInternetGatewayResult {
    __type?: "DeleteEgressOnlyInternetGatewayResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    ReturnCode?: boolean;
}
export declare namespace DeleteEgressOnlyInternetGatewayResult {
    const filterSensitiveLog: (obj: DeleteEgressOnlyInternetGatewayResult) => any;
    const isa: (o: any) => o is DeleteEgressOnlyInternetGatewayResult;
}
/**
 * <p>Describes an EC2 Fleet error.</p>
 */
export interface DeleteFleetError {
    __type?: "DeleteFleetError";
    /**
     * <p>The error code.</p>
     */
    Code?: DeleteFleetErrorCode | string;
    /**
     * <p>The description for the error code.</p>
     */
    Message?: string;
}
export declare namespace DeleteFleetError {
    const filterSensitiveLog: (obj: DeleteFleetError) => any;
    const isa: (o: any) => o is DeleteFleetError;
}
export declare enum DeleteFleetErrorCode {
    FLEET_ID_DOES_NOT_EXIST = "fleetIdDoesNotExist",
    FLEET_ID_MALFORMED = "fleetIdMalformed",
    FLEET_NOT_IN_DELETABLE_STATE = "fleetNotInDeletableState",
    UNEXPECTED_ERROR = "unexpectedError"
}
/**
 * <p>Describes an EC2 Fleet that was not successfully deleted.</p>
 */
export interface DeleteFleetErrorItem {
    __type?: "DeleteFleetErrorItem";
    /**
     * <p>The error.</p>
     */
    Error?: DeleteFleetError;
    /**
     * <p>The ID of the EC2 Fleet.</p>
     */
    FleetId?: string;
}
export declare namespace DeleteFleetErrorItem {
    const filterSensitiveLog: (obj: DeleteFleetErrorItem) => any;
    const isa: (o: any) => o is DeleteFleetErrorItem;
}
export interface DeleteFleetsRequest {
    __type?: "DeleteFleetsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the EC2 Fleets.</p>
     */
    FleetIds: string[] | undefined;
    /**
     * <p>Indicates whether to terminate instances for an EC2 Fleet if it is deleted
     *          successfully.</p>
     */
    TerminateInstances: boolean | undefined;
}
export declare namespace DeleteFleetsRequest {
    const filterSensitiveLog: (obj: DeleteFleetsRequest) => any;
    const isa: (o: any) => o is DeleteFleetsRequest;
}
export interface DeleteFleetsResult {
    __type?: "DeleteFleetsResult";
    /**
     * <p>Information about the EC2 Fleets that are successfully deleted.</p>
     */
    SuccessfulFleetDeletions?: DeleteFleetSuccessItem[];
    /**
     * <p>Information about the EC2 Fleets that are not successfully deleted.</p>
     */
    UnsuccessfulFleetDeletions?: DeleteFleetErrorItem[];
}
export declare namespace DeleteFleetsResult {
    const filterSensitiveLog: (obj: DeleteFleetsResult) => any;
    const isa: (o: any) => o is DeleteFleetsResult;
}
/**
 * <p>Describes an EC2 Fleet that was successfully deleted.</p>
 */
export interface DeleteFleetSuccessItem {
    __type?: "DeleteFleetSuccessItem";
    /**
     * <p>The current state of the EC2 Fleet.</p>
     */
    CurrentFleetState?: FleetStateCode | string;
    /**
     * <p>The ID of the EC2 Fleet.</p>
     */
    FleetId?: string;
    /**
     * <p>The previous state of the EC2 Fleet.</p>
     */
    PreviousFleetState?: FleetStateCode | string;
}
export declare namespace DeleteFleetSuccessItem {
    const filterSensitiveLog: (obj: DeleteFleetSuccessItem) => any;
    const isa: (o: any) => o is DeleteFleetSuccessItem;
}
export interface DeleteFlowLogsRequest {
    __type?: "DeleteFlowLogsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more flow log IDs.</p>
     *         <p>Constraint: Maximum of 1000 flow log IDs.</p>
     */
    FlowLogIds: string[] | undefined;
}
export declare namespace DeleteFlowLogsRequest {
    const filterSensitiveLog: (obj: DeleteFlowLogsRequest) => any;
    const isa: (o: any) => o is DeleteFlowLogsRequest;
}
export interface DeleteFlowLogsResult {
    __type?: "DeleteFlowLogsResult";
    /**
     * <p>Information about the flow logs that could not be deleted successfully.</p>
     */
    Unsuccessful?: UnsuccessfulItem[];
}
export declare namespace DeleteFlowLogsResult {
    const filterSensitiveLog: (obj: DeleteFlowLogsResult) => any;
    const isa: (o: any) => o is DeleteFlowLogsResult;
}
export interface DeleteFpgaImageRequest {
    __type?: "DeleteFpgaImageRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the AFI.</p>
     */
    FpgaImageId: string | undefined;
}
export declare namespace DeleteFpgaImageRequest {
    const filterSensitiveLog: (obj: DeleteFpgaImageRequest) => any;
    const isa: (o: any) => o is DeleteFpgaImageRequest;
}
export interface DeleteFpgaImageResult {
    __type?: "DeleteFpgaImageResult";
    /**
     * <p>Is <code>true</code> if the request succeeds, and an error otherwise.</p>
     */
    Return?: boolean;
}
export declare namespace DeleteFpgaImageResult {
    const filterSensitiveLog: (obj: DeleteFpgaImageResult) => any;
    const isa: (o: any) => o is DeleteFpgaImageResult;
}
export interface DeleteInternetGatewayRequest {
    __type?: "DeleteInternetGatewayRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the internet gateway.</p>
     */
    InternetGatewayId: string | undefined;
}
export declare namespace DeleteInternetGatewayRequest {
    const filterSensitiveLog: (obj: DeleteInternetGatewayRequest) => any;
    const isa: (o: any) => o is DeleteInternetGatewayRequest;
}
export interface DeleteKeyPairRequest {
    __type?: "DeleteKeyPairRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The name of the key pair.</p>
     */
    KeyName: string | undefined;
}
export declare namespace DeleteKeyPairRequest {
    const filterSensitiveLog: (obj: DeleteKeyPairRequest) => any;
    const isa: (o: any) => o is DeleteKeyPairRequest;
}
export interface DeleteLaunchTemplateRequest {
    __type?: "DeleteLaunchTemplateRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually
     *             making the request, and provides an error response. If you have the required
     *             permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is
     *                 <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the launch template. You must specify either the launch template ID or
     *             launch template name in the request.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template. You must specify either the launch template ID or
     *             launch template name in the request.</p>
     */
    LaunchTemplateName?: string;
}
export declare namespace DeleteLaunchTemplateRequest {
    const filterSensitiveLog: (obj: DeleteLaunchTemplateRequest) => any;
    const isa: (o: any) => o is DeleteLaunchTemplateRequest;
}
export interface DeleteLaunchTemplateResult {
    __type?: "DeleteLaunchTemplateResult";
    /**
     * <p>Information about the launch template.</p>
     */
    LaunchTemplate?: LaunchTemplate;
}
export declare namespace DeleteLaunchTemplateResult {
    const filterSensitiveLog: (obj: DeleteLaunchTemplateResult) => any;
    const isa: (o: any) => o is DeleteLaunchTemplateResult;
}
export interface DeleteLaunchTemplateVersionsRequest {
    __type?: "DeleteLaunchTemplateVersionsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually
     *             making the request, and provides an error response. If you have the required
     *             permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is
     *                 <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the launch template. You must specify either the launch template ID or
     *             launch template name in the request.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template. You must specify either the launch template ID or
     *             launch template name in the request.</p>
     */
    LaunchTemplateName?: string;
    /**
     * <p>The version numbers of one or more launch template versions to delete.</p>
     */
    Versions: string[] | undefined;
}
export declare namespace DeleteLaunchTemplateVersionsRequest {
    const filterSensitiveLog: (obj: DeleteLaunchTemplateVersionsRequest) => any;
    const isa: (o: any) => o is DeleteLaunchTemplateVersionsRequest;
}
/**
 * <p>Describes a launch template version that could not be deleted.</p>
 */
export interface DeleteLaunchTemplateVersionsResponseErrorItem {
    __type?: "DeleteLaunchTemplateVersionsResponseErrorItem";
    /**
     * <p>The ID of the launch template.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template.</p>
     */
    LaunchTemplateName?: string;
    /**
     * <p>Information about the error.</p>
     */
    ResponseError?: ResponseError;
    /**
     * <p>The version number of the launch template.</p>
     */
    VersionNumber?: number;
}
export declare namespace DeleteLaunchTemplateVersionsResponseErrorItem {
    const filterSensitiveLog: (obj: DeleteLaunchTemplateVersionsResponseErrorItem) => any;
    const isa: (o: any) => o is DeleteLaunchTemplateVersionsResponseErrorItem;
}
/**
 * <p>Describes a launch template version that was successfully deleted.</p>
 */
export interface DeleteLaunchTemplateVersionsResponseSuccessItem {
    __type?: "DeleteLaunchTemplateVersionsResponseSuccessItem";
    /**
     * <p>The ID of the launch template.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template.</p>
     */
    LaunchTemplateName?: string;
    /**
     * <p>The version number of the launch template.</p>
     */
    VersionNumber?: number;
}
export declare namespace DeleteLaunchTemplateVersionsResponseSuccessItem {
    const filterSensitiveLog: (obj: DeleteLaunchTemplateVersionsResponseSuccessItem) => any;
    const isa: (o: any) => o is DeleteLaunchTemplateVersionsResponseSuccessItem;
}
export interface DeleteLaunchTemplateVersionsResult {
    __type?: "DeleteLaunchTemplateVersionsResult";
    /**
     * <p>Information about the launch template versions that were successfully
     *             deleted.</p>
     */
    SuccessfullyDeletedLaunchTemplateVersions?: DeleteLaunchTemplateVersionsResponseSuccessItem[];
    /**
     * <p>Information about the launch template versions that could not be deleted.</p>
     */
    UnsuccessfullyDeletedLaunchTemplateVersions?: DeleteLaunchTemplateVersionsResponseErrorItem[];
}
export declare namespace DeleteLaunchTemplateVersionsResult {
    const filterSensitiveLog: (obj: DeleteLaunchTemplateVersionsResult) => any;
    const isa: (o: any) => o is DeleteLaunchTemplateVersionsResult;
}
export interface DeleteLocalGatewayRouteRequest {
    __type?: "DeleteLocalGatewayRouteRequest";
    /**
     * <p>The CIDR range for the route. This must match the CIDR for the route exactly.</p>
     */
    DestinationCidrBlock: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the local gateway route table.</p>
     */
    LocalGatewayRouteTableId: string | undefined;
}
export declare namespace DeleteLocalGatewayRouteRequest {
    const filterSensitiveLog: (obj: DeleteLocalGatewayRouteRequest) => any;
    const isa: (o: any) => o is DeleteLocalGatewayRouteRequest;
}
export interface DeleteLocalGatewayRouteResult {
    __type?: "DeleteLocalGatewayRouteResult";
    /**
     * <p>Information about the route.</p>
     */
    Route?: LocalGatewayRoute;
}
export declare namespace DeleteLocalGatewayRouteResult {
    const filterSensitiveLog: (obj: DeleteLocalGatewayRouteResult) => any;
    const isa: (o: any) => o is DeleteLocalGatewayRouteResult;
}
export interface DeleteLocalGatewayRouteTableVpcAssociationRequest {
    __type?: "DeleteLocalGatewayRouteTableVpcAssociationRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the association.</p>
     */
    LocalGatewayRouteTableVpcAssociationId: string | undefined;
}
export declare namespace DeleteLocalGatewayRouteTableVpcAssociationRequest {
    const filterSensitiveLog: (obj: DeleteLocalGatewayRouteTableVpcAssociationRequest) => any;
    const isa: (o: any) => o is DeleteLocalGatewayRouteTableVpcAssociationRequest;
}
export interface DeleteLocalGatewayRouteTableVpcAssociationResult {
    __type?: "DeleteLocalGatewayRouteTableVpcAssociationResult";
    /**
     * <p>Information about the association.</p>
     */
    LocalGatewayRouteTableVpcAssociation?: LocalGatewayRouteTableVpcAssociation;
}
export declare namespace DeleteLocalGatewayRouteTableVpcAssociationResult {
    const filterSensitiveLog: (obj: DeleteLocalGatewayRouteTableVpcAssociationResult) => any;
    const isa: (o: any) => o is DeleteLocalGatewayRouteTableVpcAssociationResult;
}
export interface DeleteNatGatewayRequest {
    __type?: "DeleteNatGatewayRequest";
    /**
     * <p>The ID of the NAT gateway.</p>
     */
    NatGatewayId: string | undefined;
}
export declare namespace DeleteNatGatewayRequest {
    const filterSensitiveLog: (obj: DeleteNatGatewayRequest) => any;
    const isa: (o: any) => o is DeleteNatGatewayRequest;
}
export interface DeleteNatGatewayResult {
    __type?: "DeleteNatGatewayResult";
    /**
     * <p>The ID of the NAT gateway.</p>
     */
    NatGatewayId?: string;
}
export declare namespace DeleteNatGatewayResult {
    const filterSensitiveLog: (obj: DeleteNatGatewayResult) => any;
    const isa: (o: any) => o is DeleteNatGatewayResult;
}
export interface DeleteNetworkAclEntryRequest {
    __type?: "DeleteNetworkAclEntryRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Indicates whether the rule is an egress rule.</p>
     */
    Egress: boolean | undefined;
    /**
     * <p>The ID of the network ACL.</p>
     */
    NetworkAclId: string | undefined;
    /**
     * <p>The rule number of the entry to delete.</p>
     */
    RuleNumber: number | undefined;
}
export declare namespace DeleteNetworkAclEntryRequest {
    const filterSensitiveLog: (obj: DeleteNetworkAclEntryRequest) => any;
    const isa: (o: any) => o is DeleteNetworkAclEntryRequest;
}
export interface DeleteNetworkAclRequest {
    __type?: "DeleteNetworkAclRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the network ACL.</p>
     */
    NetworkAclId: string | undefined;
}
export declare namespace DeleteNetworkAclRequest {
    const filterSensitiveLog: (obj: DeleteNetworkAclRequest) => any;
    const isa: (o: any) => o is DeleteNetworkAclRequest;
}
/**
 * <p>Contains the parameters for DeleteNetworkInterfacePermission.</p>
 */
export interface DeleteNetworkInterfacePermissionRequest {
    __type?: "DeleteNetworkInterfacePermissionRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     * 			and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     * 			Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Specify <code>true</code> to remove the permission even if the network interface is
     * 			attached to an instance.</p>
     */
    Force?: boolean;
    /**
     * <p>The ID of the network interface permission.</p>
     */
    NetworkInterfacePermissionId: string | undefined;
}
export declare namespace DeleteNetworkInterfacePermissionRequest {
    const filterSensitiveLog: (obj: DeleteNetworkInterfacePermissionRequest) => any;
    const isa: (o: any) => o is DeleteNetworkInterfacePermissionRequest;
}
/**
 * <p>Contains the output for DeleteNetworkInterfacePermission.</p>
 */
export interface DeleteNetworkInterfacePermissionResult {
    __type?: "DeleteNetworkInterfacePermissionResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds, otherwise returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace DeleteNetworkInterfacePermissionResult {
    const filterSensitiveLog: (obj: DeleteNetworkInterfacePermissionResult) => any;
    const isa: (o: any) => o is DeleteNetworkInterfacePermissionResult;
}
/**
 * <p>Contains the parameters for DeleteNetworkInterface.</p>
 */
export interface DeleteNetworkInterfaceRequest {
    __type?: "DeleteNetworkInterfaceRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId: string | undefined;
}
export declare namespace DeleteNetworkInterfaceRequest {
    const filterSensitiveLog: (obj: DeleteNetworkInterfaceRequest) => any;
    const isa: (o: any) => o is DeleteNetworkInterfaceRequest;
}
export interface DeletePlacementGroupRequest {
    __type?: "DeletePlacementGroupRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The name of the placement group.</p>
     */
    GroupName: string | undefined;
}
export declare namespace DeletePlacementGroupRequest {
    const filterSensitiveLog: (obj: DeletePlacementGroupRequest) => any;
    const isa: (o: any) => o is DeletePlacementGroupRequest;
}
/**
 * <p>Describes the error for a Reserved Instance whose queued purchase could not be deleted.</p>
 */
export interface DeleteQueuedReservedInstancesError {
    __type?: "DeleteQueuedReservedInstancesError";
    /**
     * <p>The error code.</p>
     */
    Code?: DeleteQueuedReservedInstancesErrorCode | string;
    /**
     * <p>The error message.</p>
     */
    Message?: string;
}
export declare namespace DeleteQueuedReservedInstancesError {
    const filterSensitiveLog: (obj: DeleteQueuedReservedInstancesError) => any;
    const isa: (o: any) => o is DeleteQueuedReservedInstancesError;
}
export declare enum DeleteQueuedReservedInstancesErrorCode {
    RESERVED_INSTANCES_ID_INVALID = "reserved-instances-id-invalid",
    RESERVED_INSTANCES_NOT_IN_QUEUED_STATE = "reserved-instances-not-in-queued-state",
    UNEXPECTED_ERROR = "unexpected-error"
}
export interface DeleteQueuedReservedInstancesRequest {
    __type?: "DeleteQueuedReservedInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *       and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *       Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the Reserved Instances.</p>
     */
    ReservedInstancesIds: string[] | undefined;
}
export declare namespace DeleteQueuedReservedInstancesRequest {
    const filterSensitiveLog: (obj: DeleteQueuedReservedInstancesRequest) => any;
    const isa: (o: any) => o is DeleteQueuedReservedInstancesRequest;
}
export interface DeleteQueuedReservedInstancesResult {
    __type?: "DeleteQueuedReservedInstancesResult";
    /**
     * <p>Information about the queued purchases that could not be deleted.</p>
     */
    FailedQueuedPurchaseDeletions?: FailedQueuedPurchaseDeletion[];
    /**
     * <p>Information about the queued purchases that were successfully deleted.</p>
     */
    SuccessfulQueuedPurchaseDeletions?: SuccessfulQueuedPurchaseDeletion[];
}
export declare namespace DeleteQueuedReservedInstancesResult {
    const filterSensitiveLog: (obj: DeleteQueuedReservedInstancesResult) => any;
    const isa: (o: any) => o is DeleteQueuedReservedInstancesResult;
}
export interface DeleteRouteRequest {
    __type?: "DeleteRouteRequest";
    /**
     * <p>The IPv4 CIDR range for the route. The value you specify must match the CIDR for the route exactly.</p>
     */
    DestinationCidrBlock?: string;
    /**
     * <p>The IPv6 CIDR range for the route. The value you specify must match the CIDR for the route exactly.</p>
     */
    DestinationIpv6CidrBlock?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the route table.</p>
     */
    RouteTableId: string | undefined;
}
export declare namespace DeleteRouteRequest {
    const filterSensitiveLog: (obj: DeleteRouteRequest) => any;
    const isa: (o: any) => o is DeleteRouteRequest;
}
export interface DeleteRouteTableRequest {
    __type?: "DeleteRouteTableRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the route table.</p>
     */
    RouteTableId: string | undefined;
}
export declare namespace DeleteRouteTableRequest {
    const filterSensitiveLog: (obj: DeleteRouteTableRequest) => any;
    const isa: (o: any) => o is DeleteRouteTableRequest;
}
export interface DeleteSecurityGroupRequest {
    __type?: "DeleteSecurityGroupRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the security group. Required for a nondefault VPC.</p>
     */
    GroupId?: string;
    /**
     * <p>[EC2-Classic, default VPC] The name of the security group. You can specify either the security group name or the security group ID.</p>
     */
    GroupName?: string;
}
export declare namespace DeleteSecurityGroupRequest {
    const filterSensitiveLog: (obj: DeleteSecurityGroupRequest) => any;
    const isa: (o: any) => o is DeleteSecurityGroupRequest;
}
export interface DeleteSnapshotRequest {
    __type?: "DeleteSnapshotRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the EBS snapshot.</p>
     */
    SnapshotId: string | undefined;
}
export declare namespace DeleteSnapshotRequest {
    const filterSensitiveLog: (obj: DeleteSnapshotRequest) => any;
    const isa: (o: any) => o is DeleteSnapshotRequest;
}
/**
 * <p>Contains the parameters for DeleteSpotDatafeedSubscription.</p>
 */
export interface DeleteSpotDatafeedSubscriptionRequest {
    __type?: "DeleteSpotDatafeedSubscriptionRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DeleteSpotDatafeedSubscriptionRequest {
    const filterSensitiveLog: (obj: DeleteSpotDatafeedSubscriptionRequest) => any;
    const isa: (o: any) => o is DeleteSpotDatafeedSubscriptionRequest;
}
export interface DeleteSubnetRequest {
    __type?: "DeleteSubnetRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId: string | undefined;
}
export declare namespace DeleteSubnetRequest {
    const filterSensitiveLog: (obj: DeleteSubnetRequest) => any;
    const isa: (o: any) => o is DeleteSubnetRequest;
}
export interface DeleteTagsRequest {
    __type?: "DeleteTagsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the resources, separated by spaces.</p>
     *    	     <p>Constraints: Up to 1000 resource IDs. We recommend breaking up this request into smaller batches.</p>
     */
    Resources: string[] | undefined;
    /**
     * <p>The tags to delete. Specify a tag key and an optional tag value to delete
     *             specific tags. If you specify a tag key without a tag value, we delete any tag with this
     *             key regardless of its value. If you specify a tag key with an empty string as the tag
     *             value, we delete the tag only if its value is an empty string.</p>
     *         <p>If you omit this parameter, we delete all user-defined tags for the specified
     *             resources. We do not delete AWS-generated tags (tags that have the <code>aws:</code>
     *             prefix).</p>
     */
    Tags?: Tag[];
}
export declare namespace DeleteTagsRequest {
    const filterSensitiveLog: (obj: DeleteTagsRequest) => any;
    const isa: (o: any) => o is DeleteTagsRequest;
}
export interface DeleteTrafficMirrorFilterRequest {
    __type?: "DeleteTrafficMirrorFilterRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the Traffic Mirror filter.</p>
     */
    TrafficMirrorFilterId: string | undefined;
}
export declare namespace DeleteTrafficMirrorFilterRequest {
    const filterSensitiveLog: (obj: DeleteTrafficMirrorFilterRequest) => any;
    const isa: (o: any) => o is DeleteTrafficMirrorFilterRequest;
}
export interface DeleteTrafficMirrorFilterResult {
    __type?: "DeleteTrafficMirrorFilterResult";
    /**
     * <p>The ID of the Traffic Mirror filter.</p>
     */
    TrafficMirrorFilterId?: string;
}
export declare namespace DeleteTrafficMirrorFilterResult {
    const filterSensitiveLog: (obj: DeleteTrafficMirrorFilterResult) => any;
    const isa: (o: any) => o is DeleteTrafficMirrorFilterResult;
}
export interface DeleteTrafficMirrorFilterRuleRequest {
    __type?: "DeleteTrafficMirrorFilterRuleRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the Traffic Mirror rule.</p>
     */
    TrafficMirrorFilterRuleId: string | undefined;
}
export declare namespace DeleteTrafficMirrorFilterRuleRequest {
    const filterSensitiveLog: (obj: DeleteTrafficMirrorFilterRuleRequest) => any;
    const isa: (o: any) => o is DeleteTrafficMirrorFilterRuleRequest;
}
export interface DeleteTrafficMirrorFilterRuleResult {
    __type?: "DeleteTrafficMirrorFilterRuleResult";
    /**
     * <p>The ID of the deleted Traffic Mirror rule.</p>
     */
    TrafficMirrorFilterRuleId?: string;
}
export declare namespace DeleteTrafficMirrorFilterRuleResult {
    const filterSensitiveLog: (obj: DeleteTrafficMirrorFilterRuleResult) => any;
    const isa: (o: any) => o is DeleteTrafficMirrorFilterRuleResult;
}
export interface DeleteTrafficMirrorSessionRequest {
    __type?: "DeleteTrafficMirrorSessionRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the Traffic Mirror session.</p>
     */
    TrafficMirrorSessionId: string | undefined;
}
export declare namespace DeleteTrafficMirrorSessionRequest {
    const filterSensitiveLog: (obj: DeleteTrafficMirrorSessionRequest) => any;
    const isa: (o: any) => o is DeleteTrafficMirrorSessionRequest;
}
export interface DeleteTrafficMirrorSessionResult {
    __type?: "DeleteTrafficMirrorSessionResult";
    /**
     * <p>The ID of the deleted Traffic Mirror session.</p>
     */
    TrafficMirrorSessionId?: string;
}
export declare namespace DeleteTrafficMirrorSessionResult {
    const filterSensitiveLog: (obj: DeleteTrafficMirrorSessionResult) => any;
    const isa: (o: any) => o is DeleteTrafficMirrorSessionResult;
}
export interface DeleteTrafficMirrorTargetRequest {
    __type?: "DeleteTrafficMirrorTargetRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the Traffic Mirror target.</p>
     */
    TrafficMirrorTargetId: string | undefined;
}
export declare namespace DeleteTrafficMirrorTargetRequest {
    const filterSensitiveLog: (obj: DeleteTrafficMirrorTargetRequest) => any;
    const isa: (o: any) => o is DeleteTrafficMirrorTargetRequest;
}
export interface DeleteTrafficMirrorTargetResult {
    __type?: "DeleteTrafficMirrorTargetResult";
    /**
     * <p>The ID of the deleted Traffic Mirror target.</p>
     */
    TrafficMirrorTargetId?: string;
}
export declare namespace DeleteTrafficMirrorTargetResult {
    const filterSensitiveLog: (obj: DeleteTrafficMirrorTargetResult) => any;
    const isa: (o: any) => o is DeleteTrafficMirrorTargetResult;
}
export interface DeleteTransitGatewayMulticastDomainRequest {
    __type?: "DeleteTransitGatewayMulticastDomainRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId: string | undefined;
}
export declare namespace DeleteTransitGatewayMulticastDomainRequest {
    const filterSensitiveLog: (obj: DeleteTransitGatewayMulticastDomainRequest) => any;
    const isa: (o: any) => o is DeleteTransitGatewayMulticastDomainRequest;
}
export interface DeleteTransitGatewayMulticastDomainResult {
    __type?: "DeleteTransitGatewayMulticastDomainResult";
    /**
     * <p>Information about the deleted transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomain?: TransitGatewayMulticastDomain;
}
export declare namespace DeleteTransitGatewayMulticastDomainResult {
    const filterSensitiveLog: (obj: DeleteTransitGatewayMulticastDomainResult) => any;
    const isa: (o: any) => o is DeleteTransitGatewayMulticastDomainResult;
}
export interface DeleteTransitGatewayPeeringAttachmentRequest {
    __type?: "DeleteTransitGatewayPeeringAttachmentRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the transit gateway peering attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
}
export declare namespace DeleteTransitGatewayPeeringAttachmentRequest {
    const filterSensitiveLog: (obj: DeleteTransitGatewayPeeringAttachmentRequest) => any;
    const isa: (o: any) => o is DeleteTransitGatewayPeeringAttachmentRequest;
}
export interface DeleteTransitGatewayPeeringAttachmentResult {
    __type?: "DeleteTransitGatewayPeeringAttachmentResult";
    /**
     * <p>The transit gateway peering attachment.</p>
     */
    TransitGatewayPeeringAttachment?: TransitGatewayPeeringAttachment;
}
export declare namespace DeleteTransitGatewayPeeringAttachmentResult {
    const filterSensitiveLog: (obj: DeleteTransitGatewayPeeringAttachmentResult) => any;
    const isa: (o: any) => o is DeleteTransitGatewayPeeringAttachmentResult;
}
export interface DeleteTransitGatewayRequest {
    __type?: "DeleteTransitGatewayRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId: string | undefined;
}
export declare namespace DeleteTransitGatewayRequest {
    const filterSensitiveLog: (obj: DeleteTransitGatewayRequest) => any;
    const isa: (o: any) => o is DeleteTransitGatewayRequest;
}
export interface DeleteTransitGatewayResult {
    __type?: "DeleteTransitGatewayResult";
    /**
     * <p>Information about the deleted transit gateway.</p>
     */
    TransitGateway?: TransitGateway;
}
export declare namespace DeleteTransitGatewayResult {
    const filterSensitiveLog: (obj: DeleteTransitGatewayResult) => any;
    const isa: (o: any) => o is DeleteTransitGatewayResult;
}
export interface DeleteTransitGatewayRouteRequest {
    __type?: "DeleteTransitGatewayRouteRequest";
    /**
     * <p>The CIDR range for the route. This must match the CIDR for the route exactly.</p>
     */
    DestinationCidrBlock: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the transit gateway route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace DeleteTransitGatewayRouteRequest {
    const filterSensitiveLog: (obj: DeleteTransitGatewayRouteRequest) => any;
    const isa: (o: any) => o is DeleteTransitGatewayRouteRequest;
}
export interface DeleteTransitGatewayRouteResult {
    __type?: "DeleteTransitGatewayRouteResult";
    /**
     * <p>Information about the route.</p>
     */
    Route?: TransitGatewayRoute;
}
export declare namespace DeleteTransitGatewayRouteResult {
    const filterSensitiveLog: (obj: DeleteTransitGatewayRouteResult) => any;
    const isa: (o: any) => o is DeleteTransitGatewayRouteResult;
}
export interface DeleteTransitGatewayRouteTableRequest {
    __type?: "DeleteTransitGatewayRouteTableRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the transit gateway route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace DeleteTransitGatewayRouteTableRequest {
    const filterSensitiveLog: (obj: DeleteTransitGatewayRouteTableRequest) => any;
    const isa: (o: any) => o is DeleteTransitGatewayRouteTableRequest;
}
export interface DeleteTransitGatewayRouteTableResult {
    __type?: "DeleteTransitGatewayRouteTableResult";
    /**
     * <p>Information about the deleted transit gateway route table.</p>
     */
    TransitGatewayRouteTable?: TransitGatewayRouteTable;
}
export declare namespace DeleteTransitGatewayRouteTableResult {
    const filterSensitiveLog: (obj: DeleteTransitGatewayRouteTableResult) => any;
    const isa: (o: any) => o is DeleteTransitGatewayRouteTableResult;
}
export interface DeleteTransitGatewayVpcAttachmentRequest {
    __type?: "DeleteTransitGatewayVpcAttachmentRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
}
export declare namespace DeleteTransitGatewayVpcAttachmentRequest {
    const filterSensitiveLog: (obj: DeleteTransitGatewayVpcAttachmentRequest) => any;
    const isa: (o: any) => o is DeleteTransitGatewayVpcAttachmentRequest;
}
export interface DeleteTransitGatewayVpcAttachmentResult {
    __type?: "DeleteTransitGatewayVpcAttachmentResult";
    /**
     * <p>Information about the deleted VPC attachment.</p>
     */
    TransitGatewayVpcAttachment?: TransitGatewayVpcAttachment;
}
export declare namespace DeleteTransitGatewayVpcAttachmentResult {
    const filterSensitiveLog: (obj: DeleteTransitGatewayVpcAttachmentResult) => any;
    const isa: (o: any) => o is DeleteTransitGatewayVpcAttachmentResult;
}
export interface DeleteVolumeRequest {
    __type?: "DeleteVolumeRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the volume.</p>
     */
    VolumeId: string | undefined;
}
export declare namespace DeleteVolumeRequest {
    const filterSensitiveLog: (obj: DeleteVolumeRequest) => any;
    const isa: (o: any) => o is DeleteVolumeRequest;
}
export interface DeleteVpcEndpointConnectionNotificationsRequest {
    __type?: "DeleteVpcEndpointConnectionNotificationsRequest";
    /**
     * <p>One or more notification IDs.</p>
     */
    ConnectionNotificationIds: string[] | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DeleteVpcEndpointConnectionNotificationsRequest {
    const filterSensitiveLog: (obj: DeleteVpcEndpointConnectionNotificationsRequest) => any;
    const isa: (o: any) => o is DeleteVpcEndpointConnectionNotificationsRequest;
}
export interface DeleteVpcEndpointConnectionNotificationsResult {
    __type?: "DeleteVpcEndpointConnectionNotificationsResult";
    /**
     * <p>Information about the notifications that could not be deleted
     *             successfully.</p>
     */
    Unsuccessful?: UnsuccessfulItem[];
}
export declare namespace DeleteVpcEndpointConnectionNotificationsResult {
    const filterSensitiveLog: (obj: DeleteVpcEndpointConnectionNotificationsResult) => any;
    const isa: (o: any) => o is DeleteVpcEndpointConnectionNotificationsResult;
}
export interface DeleteVpcEndpointServiceConfigurationsRequest {
    __type?: "DeleteVpcEndpointServiceConfigurationsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of one or more services.</p>
     */
    ServiceIds: string[] | undefined;
}
export declare namespace DeleteVpcEndpointServiceConfigurationsRequest {
    const filterSensitiveLog: (obj: DeleteVpcEndpointServiceConfigurationsRequest) => any;
    const isa: (o: any) => o is DeleteVpcEndpointServiceConfigurationsRequest;
}
export interface DeleteVpcEndpointServiceConfigurationsResult {
    __type?: "DeleteVpcEndpointServiceConfigurationsResult";
    /**
     * <p>Information about the service configurations that were not deleted, if
     *             applicable.</p>
     */
    Unsuccessful?: UnsuccessfulItem[];
}
export declare namespace DeleteVpcEndpointServiceConfigurationsResult {
    const filterSensitiveLog: (obj: DeleteVpcEndpointServiceConfigurationsResult) => any;
    const isa: (o: any) => o is DeleteVpcEndpointServiceConfigurationsResult;
}
/**
 * <p>Contains the parameters for DeleteVpcEndpoints.</p>
 */
export interface DeleteVpcEndpointsRequest {
    __type?: "DeleteVpcEndpointsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more VPC endpoint IDs.</p>
     */
    VpcEndpointIds: string[] | undefined;
}
export declare namespace DeleteVpcEndpointsRequest {
    const filterSensitiveLog: (obj: DeleteVpcEndpointsRequest) => any;
    const isa: (o: any) => o is DeleteVpcEndpointsRequest;
}
/**
 * <p>Contains the output of DeleteVpcEndpoints.</p>
 */
export interface DeleteVpcEndpointsResult {
    __type?: "DeleteVpcEndpointsResult";
    /**
     * <p>Information about the VPC endpoints that were not successfully deleted.</p>
     */
    Unsuccessful?: UnsuccessfulItem[];
}
export declare namespace DeleteVpcEndpointsResult {
    const filterSensitiveLog: (obj: DeleteVpcEndpointsResult) => any;
    const isa: (o: any) => o is DeleteVpcEndpointsResult;
}
export interface DeleteVpcPeeringConnectionRequest {
    __type?: "DeleteVpcPeeringConnectionRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC peering connection.</p>
     */
    VpcPeeringConnectionId: string | undefined;
}
export declare namespace DeleteVpcPeeringConnectionRequest {
    const filterSensitiveLog: (obj: DeleteVpcPeeringConnectionRequest) => any;
    const isa: (o: any) => o is DeleteVpcPeeringConnectionRequest;
}
export interface DeleteVpcPeeringConnectionResult {
    __type?: "DeleteVpcPeeringConnectionResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace DeleteVpcPeeringConnectionResult {
    const filterSensitiveLog: (obj: DeleteVpcPeeringConnectionResult) => any;
    const isa: (o: any) => o is DeleteVpcPeeringConnectionResult;
}
export interface DeleteVpcRequest {
    __type?: "DeleteVpcRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace DeleteVpcRequest {
    const filterSensitiveLog: (obj: DeleteVpcRequest) => any;
    const isa: (o: any) => o is DeleteVpcRequest;
}
/**
 * <p>Contains the parameters for DeleteVpnConnection.</p>
 */
export interface DeleteVpnConnectionRequest {
    __type?: "DeleteVpnConnectionRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPN connection.</p>
     */
    VpnConnectionId: string | undefined;
}
export declare namespace DeleteVpnConnectionRequest {
    const filterSensitiveLog: (obj: DeleteVpnConnectionRequest) => any;
    const isa: (o: any) => o is DeleteVpnConnectionRequest;
}
/**
 * <p>Contains the parameters for DeleteVpnConnectionRoute.</p>
 */
export interface DeleteVpnConnectionRouteRequest {
    __type?: "DeleteVpnConnectionRouteRequest";
    /**
     * <p>The CIDR block associated with the local subnet of the customer network.</p>
     */
    DestinationCidrBlock: string | undefined;
    /**
     * <p>The ID of the VPN connection.</p>
     */
    VpnConnectionId: string | undefined;
}
export declare namespace DeleteVpnConnectionRouteRequest {
    const filterSensitiveLog: (obj: DeleteVpnConnectionRouteRequest) => any;
    const isa: (o: any) => o is DeleteVpnConnectionRouteRequest;
}
/**
 * <p>Contains the parameters for DeleteVpnGateway.</p>
 */
export interface DeleteVpnGatewayRequest {
    __type?: "DeleteVpnGatewayRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the virtual private gateway.</p>
     */
    VpnGatewayId: string | undefined;
}
export declare namespace DeleteVpnGatewayRequest {
    const filterSensitiveLog: (obj: DeleteVpnGatewayRequest) => any;
    const isa: (o: any) => o is DeleteVpnGatewayRequest;
}
export interface DeprovisionByoipCidrRequest {
    __type?: "DeprovisionByoipCidrRequest";
    /**
     * <p>The address range, in CIDR notation. The prefix must be the same prefix
     *          that you specified when you provisioned the address range.</p>
     */
    Cidr: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DeprovisionByoipCidrRequest {
    const filterSensitiveLog: (obj: DeprovisionByoipCidrRequest) => any;
    const isa: (o: any) => o is DeprovisionByoipCidrRequest;
}
export interface DeprovisionByoipCidrResult {
    __type?: "DeprovisionByoipCidrResult";
    /**
     * <p>Information about the address range.</p>
     */
    ByoipCidr?: ByoipCidr;
}
export declare namespace DeprovisionByoipCidrResult {
    const filterSensitiveLog: (obj: DeprovisionByoipCidrResult) => any;
    const isa: (o: any) => o is DeprovisionByoipCidrResult;
}
/**
 * <p>Contains the parameters for DeregisterImage.</p>
 */
export interface DeregisterImageRequest {
    __type?: "DeregisterImageRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the AMI.</p>
     */
    ImageId: string | undefined;
}
export declare namespace DeregisterImageRequest {
    const filterSensitiveLog: (obj: DeregisterImageRequest) => any;
    const isa: (o: any) => o is DeregisterImageRequest;
}
export interface DeregisterTransitGatewayMulticastGroupMembersRequest {
    __type?: "DeregisterTransitGatewayMulticastGroupMembersRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IP address assigned to the  transit gateway multicast group.</p>
     */
    GroupIpAddress?: string;
    /**
     * <p>The IDs of the group members' network interfaces.</p>
     */
    NetworkInterfaceIds?: string[];
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace DeregisterTransitGatewayMulticastGroupMembersRequest {
    const filterSensitiveLog: (obj: DeregisterTransitGatewayMulticastGroupMembersRequest) => any;
    const isa: (o: any) => o is DeregisterTransitGatewayMulticastGroupMembersRequest;
}
export interface DeregisterTransitGatewayMulticastGroupMembersResult {
    __type?: "DeregisterTransitGatewayMulticastGroupMembersResult";
    /**
     * <p>Information about the deregistered members.</p>
     */
    DeregisteredMulticastGroupMembers?: TransitGatewayMulticastDeregisteredGroupMembers;
}
export declare namespace DeregisterTransitGatewayMulticastGroupMembersResult {
    const filterSensitiveLog: (obj: DeregisterTransitGatewayMulticastGroupMembersResult) => any;
    const isa: (o: any) => o is DeregisterTransitGatewayMulticastGroupMembersResult;
}
export interface DeregisterTransitGatewayMulticastGroupSourcesRequest {
    __type?: "DeregisterTransitGatewayMulticastGroupSourcesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IP address assigned to the  transit gateway multicast group.</p>
     */
    GroupIpAddress?: string;
    /**
     * <p>The IDs of the group sources' network interfaces.</p>
     */
    NetworkInterfaceIds?: string[];
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace DeregisterTransitGatewayMulticastGroupSourcesRequest {
    const filterSensitiveLog: (obj: DeregisterTransitGatewayMulticastGroupSourcesRequest) => any;
    const isa: (o: any) => o is DeregisterTransitGatewayMulticastGroupSourcesRequest;
}
export interface DeregisterTransitGatewayMulticastGroupSourcesResult {
    __type?: "DeregisterTransitGatewayMulticastGroupSourcesResult";
    /**
     * <p>Information about the deregistered group sources.</p>
     */
    DeregisteredMulticastGroupSources?: TransitGatewayMulticastDeregisteredGroupSources;
}
export declare namespace DeregisterTransitGatewayMulticastGroupSourcesResult {
    const filterSensitiveLog: (obj: DeregisterTransitGatewayMulticastGroupSourcesResult) => any;
    const isa: (o: any) => o is DeregisterTransitGatewayMulticastGroupSourcesResult;
}
export interface DescribeAccountAttributesRequest {
    __type?: "DescribeAccountAttributesRequest";
    /**
     * <p>The account attribute names.</p>
     */
    AttributeNames?: (AccountAttributeName | string)[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *            and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *            Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DescribeAccountAttributesRequest {
    const filterSensitiveLog: (obj: DescribeAccountAttributesRequest) => any;
    const isa: (o: any) => o is DescribeAccountAttributesRequest;
}
export interface DescribeAccountAttributesResult {
    __type?: "DescribeAccountAttributesResult";
    /**
     * <p>Information about the account attributes.</p>
     */
    AccountAttributes?: AccountAttribute[];
}
export declare namespace DescribeAccountAttributesResult {
    const filterSensitiveLog: (obj: DescribeAccountAttributesResult) => any;
    const isa: (o: any) => o is DescribeAccountAttributesResult;
}
export interface DescribeAddressesRequest {
    __type?: "DescribeAddressesRequest";
    /**
     * <p>[EC2-VPC] Information about the allocation IDs.</p>
     */
    AllocationIds?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. Filter names and values are case-sensitive.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>allocation-id</code> - [EC2-VPC] The allocation ID for the address.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>association-id</code> - [EC2-VPC] The association ID for the address.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>domain</code> - Indicates whether the address is for use in EC2-Classic (<code>standard</code>)
     *               or in a VPC (<code>vpc</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-id</code> - The ID of the instance the address is associated with, if any.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-border-group</code> - The location from where the IP address is advertised.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface-id</code> - [EC2-VPC] The ID of the network interface that the address is associated with, if any.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface-owner-id</code> - The AWS account ID of the owner.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>private-ip-address</code> - [EC2-VPC] The private IP address associated with the Elastic IP address.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>public-ip</code> - The Elastic IP address.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>One or more Elastic IP addresses.</p>
     *          <p>Default: Describes all your Elastic IP addresses.</p>
     */
    PublicIps?: string[];
}
export declare namespace DescribeAddressesRequest {
    const filterSensitiveLog: (obj: DescribeAddressesRequest) => any;
    const isa: (o: any) => o is DescribeAddressesRequest;
}
export interface DescribeAddressesResult {
    __type?: "DescribeAddressesResult";
    /**
     * <p>Information about the Elastic IP addresses.</p>
     */
    Addresses?: Address[];
}
export declare namespace DescribeAddressesResult {
    const filterSensitiveLog: (obj: DescribeAddressesResult) => any;
    const isa: (o: any) => o is DescribeAddressesResult;
}
export interface DescribeAggregateIdFormatRequest {
    __type?: "DescribeAggregateIdFormatRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DescribeAggregateIdFormatRequest {
    const filterSensitiveLog: (obj: DescribeAggregateIdFormatRequest) => any;
    const isa: (o: any) => o is DescribeAggregateIdFormatRequest;
}
export interface DescribeAggregateIdFormatResult {
    __type?: "DescribeAggregateIdFormatResult";
    /**
     * <p>Information about each resource's ID format.</p>
     */
    Statuses?: IdFormat[];
    /**
     * <p>Indicates whether all resource types in the Region are configured to use longer IDs.
     *             This value is only <code>true</code> if all users are configured to use longer IDs for
     *             all resources types in the Region.</p>
     */
    UseLongIdsAggregated?: boolean;
}
export declare namespace DescribeAggregateIdFormatResult {
    const filterSensitiveLog: (obj: DescribeAggregateIdFormatResult) => any;
    const isa: (o: any) => o is DescribeAggregateIdFormatResult;
}
export interface DescribeAvailabilityZonesRequest {
    __type?: "DescribeAvailabilityZonesRequest";
    /**
     * <p>Include all Availability Zones and Local Zones regardless of your opt in status.</p>
     *          <p>If you do not use this parameter, the results include only the zones for the Regions where you have chosen the option to opt in.</p>
     */
    AllAvailabilityZones?: boolean;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>group-name</code> - For Availability Zones, use the Region name. For Local Zones,
     *           use the name of the group associated with the Local Zone (for example,
     *             <code>us-west-2-lax-1</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>message</code> - The Availability Zone or Local Zone message.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>opt-in-status</code> - The opt in status (<code>opted-in</code>, and
     *           <code>not-opted-in</code> | <code>opt-in-not-required</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>region-name</code> - The name of the Region for the Availability Zone or Local Zone (for example, <code>us-east-1</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the Availability Zone or Local Zone (<code>available</code> |
     *           <code>information</code> | <code>impaired</code> | <code>unavailable</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>zone-id</code> - The ID of the Availability Zone (for example,
     *             <code>use1-az1</code>) or the Local Zone (for example, use
     *           <code>usw2-lax1-az1</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>zone-name</code> - The name of the Availability Zone (for example,
     *             <code>us-east-1a</code>) or the Local Zone (for example, use
     *           <code>us-west-2-lax-1a</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The IDs of the Availability Zones and Local Zones.</p>
     */
    ZoneIds?: string[];
    /**
     * <p>The names of the Availability Zones and Local Zones.</p>
     */
    ZoneNames?: string[];
}
export declare namespace DescribeAvailabilityZonesRequest {
    const filterSensitiveLog: (obj: DescribeAvailabilityZonesRequest) => any;
    const isa: (o: any) => o is DescribeAvailabilityZonesRequest;
}
export interface DescribeAvailabilityZonesResult {
    __type?: "DescribeAvailabilityZonesResult";
    /**
     * <p>Information about the Availability Zones and Local Zones.</p>
     */
    AvailabilityZones?: AvailabilityZone[];
}
export declare namespace DescribeAvailabilityZonesResult {
    const filterSensitiveLog: (obj: DescribeAvailabilityZonesResult) => any;
    const isa: (o: any) => o is DescribeAvailabilityZonesResult;
}
export interface DescribeBundleTasksRequest {
    __type?: "DescribeBundleTasksRequest";
    /**
     * <p>The bundle task IDs.</p>
     *          <p>Default: Describes all your bundle tasks.</p>
     */
    BundleIds?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>bundle-id</code> - The ID of the bundle task.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>error-code</code> - If the task failed, the error code returned.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>error-message</code> - If the task failed, the error message returned.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-id</code> - The ID of the instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>progress</code> - The level of task completion, as a percentage (for example, 20%).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>s3-bucket</code> - The Amazon S3 bucket to store the AMI.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>s3-prefix</code> - The beginning of the AMI name.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>start-time</code> - The time the task started (for example, 2013-09-15T17:15:20.000Z).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the task (<code>pending</code> | <code>waiting-for-shutdown</code> | <code>bundling</code> |
     *            <code>storing</code> | <code>cancelling</code> | <code>complete</code> | <code>failed</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>update-time</code> - The time of the most recent update for the task.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
}
export declare namespace DescribeBundleTasksRequest {
    const filterSensitiveLog: (obj: DescribeBundleTasksRequest) => any;
    const isa: (o: any) => o is DescribeBundleTasksRequest;
}
export interface DescribeBundleTasksResult {
    __type?: "DescribeBundleTasksResult";
    /**
     * <p>Information about the bundle tasks.</p>
     */
    BundleTasks?: BundleTask[];
}
export declare namespace DescribeBundleTasksResult {
    const filterSensitiveLog: (obj: DescribeBundleTasksResult) => any;
    const isa: (o: any) => o is DescribeBundleTasksResult;
}
export interface DescribeByoipCidrsRequest {
    __type?: "DescribeByoipCidrsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults: number | undefined;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeByoipCidrsRequest {
    const filterSensitiveLog: (obj: DescribeByoipCidrsRequest) => any;
    const isa: (o: any) => o is DescribeByoipCidrsRequest;
}
export interface DescribeByoipCidrsResult {
    __type?: "DescribeByoipCidrsResult";
    /**
     * <p>Information about your address ranges.</p>
     */
    ByoipCidrs?: ByoipCidr[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeByoipCidrsResult {
    const filterSensitiveLog: (obj: DescribeByoipCidrsResult) => any;
    const isa: (o: any) => o is DescribeByoipCidrsResult;
}
export interface DescribeCapacityReservationsRequest {
    __type?: "DescribeCapacityReservationsRequest";
    /**
     * <p>The ID of the Capacity Reservation.</p>
     */
    CapacityReservationIds?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned nextToken value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeCapacityReservationsRequest {
    const filterSensitiveLog: (obj: DescribeCapacityReservationsRequest) => any;
    const isa: (o: any) => o is DescribeCapacityReservationsRequest;
}
export interface DescribeCapacityReservationsResult {
    __type?: "DescribeCapacityReservationsResult";
    /**
     * <p>Information about the Capacity Reservations.</p>
     */
    CapacityReservations?: CapacityReservation[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeCapacityReservationsResult {
    const filterSensitiveLog: (obj: DescribeCapacityReservationsResult) => any;
    const isa: (o: any) => o is DescribeCapacityReservationsResult;
}
export interface DescribeClassicLinkInstancesRequest {
    __type?: "DescribeClassicLinkInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     * 		       <ul>
     *             <li>
     *                       <p>
     *                   <code>group-id</code> - The ID of a VPC security group that's associated with the instance.</p>
     *                   </li>
     *             <li>
     * 				           <p>
     *                   <code>instance-id</code> - The ID of the instance.</p>
     * 			         </li>
     *             <li>
     * 			            <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     * 			         </li>
     *             <li>
     *
     * 				           <p>
     * 					             <code>vpc-id</code> - The ID of the VPC to which the instance is
     * 					linked.</p>
     *
     * 				           <p>
     *                   <code>vpc-id</code> - The ID of the VPC that the instance is linked to.</p>
     *
     * 			         </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>One or more instance IDs. Must be instances linked to a VPC through ClassicLink.</p>
     */
    InstanceIds?: string[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     * 		       <p>Constraint: If the value is greater than 1000, we return only 1000 items.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeClassicLinkInstancesRequest {
    const filterSensitiveLog: (obj: DescribeClassicLinkInstancesRequest) => any;
    const isa: (o: any) => o is DescribeClassicLinkInstancesRequest;
}
export interface DescribeClassicLinkInstancesResult {
    __type?: "DescribeClassicLinkInstancesResult";
    /**
     * <p>Information about one or more linked EC2-Classic instances.</p>
     */
    Instances?: ClassicLinkInstance[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeClassicLinkInstancesResult {
    const filterSensitiveLog: (obj: DescribeClassicLinkInstancesResult) => any;
    const isa: (o: any) => o is DescribeClassicLinkInstancesResult;
}
export interface DescribeClientVpnAuthorizationRulesRequest {
    __type?: "DescribeClientVpnAuthorizationRulesRequest";
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. Filter names and values are case-sensitive.</p>
     * 	        <ul>
     *             <li>
     *                <p>
     *                   <code>description</code> - The description of the authorization rule.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>destination-cidr</code> - The CIDR of the network to which the authorization rule
     *                     applies.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>group-id</code> - The ID of the Active Directory group to which the authorization rule grants access.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeClientVpnAuthorizationRulesRequest {
    const filterSensitiveLog: (obj: DescribeClientVpnAuthorizationRulesRequest) => any;
    const isa: (o: any) => o is DescribeClientVpnAuthorizationRulesRequest;
}
export interface DescribeClientVpnAuthorizationRulesResult {
    __type?: "DescribeClientVpnAuthorizationRulesResult";
    /**
     * <p>Information about the authorization rules.</p>
     */
    AuthorizationRules?: AuthorizationRule[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeClientVpnAuthorizationRulesResult {
    const filterSensitiveLog: (obj: DescribeClientVpnAuthorizationRulesResult) => any;
    const isa: (o: any) => o is DescribeClientVpnAuthorizationRulesResult;
}
export interface DescribeClientVpnConnectionsRequest {
    __type?: "DescribeClientVpnConnectionsRequest";
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. Filter names and values are case-sensitive.</p>
     * 	        <ul>
     *             <li>
     *                <p>
     *                   <code>connection-id</code> - The ID of the connection.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>username</code> - For Active Directory client authentication, the user name of the
     *                     client who established the client connection.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeClientVpnConnectionsRequest {
    const filterSensitiveLog: (obj: DescribeClientVpnConnectionsRequest) => any;
    const isa: (o: any) => o is DescribeClientVpnConnectionsRequest;
}
export interface DescribeClientVpnConnectionsResult {
    __type?: "DescribeClientVpnConnectionsResult";
    /**
     * <p>Information about the active and terminated client connections.</p>
     */
    Connections?: ClientVpnConnection[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeClientVpnConnectionsResult {
    const filterSensitiveLog: (obj: DescribeClientVpnConnectionsResult) => any;
    const isa: (o: any) => o is DescribeClientVpnConnectionsResult;
}
export interface DescribeClientVpnEndpointsRequest {
    __type?: "DescribeClientVpnEndpointsRequest";
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointIds?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. Filter names and values are case-sensitive.</p>
     * 	        <ul>
     *             <li>
     *                <p>
     *                   <code>endpoint-id</code> - The ID of the Client VPN endpoint.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transport-protocol</code> - The transport protocol (<code>tcp</code> |
     *                     <code>udp</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeClientVpnEndpointsRequest {
    const filterSensitiveLog: (obj: DescribeClientVpnEndpointsRequest) => any;
    const isa: (o: any) => o is DescribeClientVpnEndpointsRequest;
}
export interface DescribeClientVpnEndpointsResult {
    __type?: "DescribeClientVpnEndpointsResult";
    /**
     * <p>Information about the Client VPN endpoints.</p>
     */
    ClientVpnEndpoints?: ClientVpnEndpoint[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeClientVpnEndpointsResult {
    const filterSensitiveLog: (obj: DescribeClientVpnEndpointsResult) => any;
    const isa: (o: any) => o is DescribeClientVpnEndpointsResult;
}
export interface DescribeClientVpnRoutesRequest {
    __type?: "DescribeClientVpnRoutesRequest";
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. Filter names and values are case-sensitive.</p>
     * 	        <ul>
     *             <li>
     *                <p>
     *                   <code>destination-cidr</code> - The CIDR of the route destination.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>origin</code> - How the route was associated with the Client VPN endpoint (<code>associate</code> | <code>add-route</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>target-subnet</code> - The ID of the subnet through which traffic is routed.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeClientVpnRoutesRequest {
    const filterSensitiveLog: (obj: DescribeClientVpnRoutesRequest) => any;
    const isa: (o: any) => o is DescribeClientVpnRoutesRequest;
}
export interface DescribeClientVpnRoutesResult {
    __type?: "DescribeClientVpnRoutesResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the Client VPN endpoint routes.</p>
     */
    Routes?: ClientVpnRoute[];
}
export declare namespace DescribeClientVpnRoutesResult {
    const filterSensitiveLog: (obj: DescribeClientVpnRoutesResult) => any;
    const isa: (o: any) => o is DescribeClientVpnRoutesResult;
}
export interface DescribeClientVpnTargetNetworksRequest {
    __type?: "DescribeClientVpnTargetNetworksRequest";
    /**
     * <p>The IDs of the target network associations.</p>
     */
    AssociationIds?: string[];
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. Filter names and values are case-sensitive.</p>
     * 	        <ul>
     *             <li>
     *                <p>
     *                   <code>association-id</code> - The ID of the association.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>target-network-id</code> - The ID of the subnet specified as the target network.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>vpc-id</code> - The ID of the VPC in which the target network is located.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the nextToken value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeClientVpnTargetNetworksRequest {
    const filterSensitiveLog: (obj: DescribeClientVpnTargetNetworksRequest) => any;
    const isa: (o: any) => o is DescribeClientVpnTargetNetworksRequest;
}
export interface DescribeClientVpnTargetNetworksResult {
    __type?: "DescribeClientVpnTargetNetworksResult";
    /**
     * <p>Information about the associated target networks.</p>
     */
    ClientVpnTargetNetworks?: TargetNetwork[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeClientVpnTargetNetworksResult {
    const filterSensitiveLog: (obj: DescribeClientVpnTargetNetworksResult) => any;
    const isa: (o: any) => o is DescribeClientVpnTargetNetworksResult;
}
export interface DescribeCoipPoolsRequest {
    __type?: "DescribeCoipPoolsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters. The following are the possible values:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>coip-pool.pool-id</code>
     *                </p>
     *             </li>
     *          </ul>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>coip-pool.local-gateway-route-table-id</code>
     *                </p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The IDs of the address pools.</p>
     */
    PoolIds?: string[];
}
export declare namespace DescribeCoipPoolsRequest {
    const filterSensitiveLog: (obj: DescribeCoipPoolsRequest) => any;
    const isa: (o: any) => o is DescribeCoipPoolsRequest;
}
export interface DescribeCoipPoolsResult {
    __type?: "DescribeCoipPoolsResult";
    /**
     * <p>Information about the address pools.</p>
     */
    CoipPools?: CoipPool[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeCoipPoolsResult {
    const filterSensitiveLog: (obj: DescribeCoipPoolsResult) => any;
    const isa: (o: any) => o is DescribeCoipPoolsResult;
}
export interface DescribeConversionTasksRequest {
    __type?: "DescribeConversionTasksRequest";
    /**
     * <p>The conversion task IDs.</p>
     */
    ConversionTaskIds?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DescribeConversionTasksRequest {
    const filterSensitiveLog: (obj: DescribeConversionTasksRequest) => any;
    const isa: (o: any) => o is DescribeConversionTasksRequest;
}
export interface DescribeConversionTasksResult {
    __type?: "DescribeConversionTasksResult";
    /**
     * <p>Information about the conversion tasks.</p>
     */
    ConversionTasks?: ConversionTask[];
}
export declare namespace DescribeConversionTasksResult {
    const filterSensitiveLog: (obj: DescribeConversionTasksResult) => any;
    const isa: (o: any) => o is DescribeConversionTasksResult;
}
/**
 * <p>Contains the parameters for DescribeCustomerGateways.</p>
 */
export interface DescribeCustomerGatewaysRequest {
    __type?: "DescribeCustomerGatewaysRequest";
    /**
     * <p>One or more customer gateway IDs.</p>
     *         <p>Default: Describes all your customer gateways.</p>
     */
    CustomerGatewayIds?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>bgp-asn</code> - The customer gateway's Border Gateway Protocol (BGP) Autonomous System Number (ASN).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>customer-gateway-id</code> - The ID of the customer gateway.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ip-address</code> - The IP address of the customer gateway's Internet-routable external interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>state</code> - The state of the customer gateway (<code>pending</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>type</code> - The type of customer gateway. Currently, the only supported type is <code>ipsec.1</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
}
export declare namespace DescribeCustomerGatewaysRequest {
    const filterSensitiveLog: (obj: DescribeCustomerGatewaysRequest) => any;
    const isa: (o: any) => o is DescribeCustomerGatewaysRequest;
}
/**
 * <p>Contains the output of DescribeCustomerGateways.</p>
 */
export interface DescribeCustomerGatewaysResult {
    __type?: "DescribeCustomerGatewaysResult";
    /**
     * <p>Information about one or more customer gateways.</p>
     */
    CustomerGateways?: CustomerGateway[];
}
export declare namespace DescribeCustomerGatewaysResult {
    const filterSensitiveLog: (obj: DescribeCustomerGatewaysResult) => any;
    const isa: (o: any) => o is DescribeCustomerGatewaysResult;
}
export interface DescribeDhcpOptionsRequest {
    __type?: "DescribeDhcpOptionsRequest";
    /**
     * <p>The IDs of one or more DHCP options sets.</p>
     * 		       <p>Default: Describes all your DHCP options sets.</p>
     */
    DhcpOptionsIds?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     * 		       <ul>
     *             <li>
     * 		             <p>
     *                   <code>dhcp-options-id</code> - The ID of a DHCP options set.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>key</code> - The key for one of the options (for example, <code>domain-name</code>).</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>value</code> - The value for one of the options.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>owner-id</code> - The ID of the AWS account that owns the DHCP options set.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     * 		          </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeDhcpOptionsRequest {
    const filterSensitiveLog: (obj: DescribeDhcpOptionsRequest) => any;
    const isa: (o: any) => o is DescribeDhcpOptionsRequest;
}
export interface DescribeDhcpOptionsResult {
    __type?: "DescribeDhcpOptionsResult";
    /**
     * <p>Information about one or more DHCP options sets.</p>
     */
    DhcpOptions?: DhcpOptions[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeDhcpOptionsResult {
    const filterSensitiveLog: (obj: DescribeDhcpOptionsResult) => any;
    const isa: (o: any) => o is DescribeDhcpOptionsResult;
}
export interface DescribeEgressOnlyInternetGatewaysRequest {
    __type?: "DescribeEgressOnlyInternetGatewaysRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more egress-only internet gateway IDs.</p>
     */
    EgressOnlyInternetGatewayIds?: string[];
    /**
     * <p>One or more filters.</p>
     *     	    <ul>
     *             <li>
     *     			        <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *     		      </li>
     *             <li>
     *     			        <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *     		      </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeEgressOnlyInternetGatewaysRequest {
    const filterSensitiveLog: (obj: DescribeEgressOnlyInternetGatewaysRequest) => any;
    const isa: (o: any) => o is DescribeEgressOnlyInternetGatewaysRequest;
}
export interface DescribeEgressOnlyInternetGatewaysResult {
    __type?: "DescribeEgressOnlyInternetGatewaysResult";
    /**
     * <p>Information about the egress-only internet gateways.</p>
     */
    EgressOnlyInternetGateways?: EgressOnlyInternetGateway[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeEgressOnlyInternetGatewaysResult {
    const filterSensitiveLog: (obj: DescribeEgressOnlyInternetGatewaysResult) => any;
    const isa: (o: any) => o is DescribeEgressOnlyInternetGatewaysResult;
}
export interface DescribeElasticGpusRequest {
    __type?: "DescribeElasticGpusRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The Elastic Graphics accelerator IDs.</p>
     */
    ElasticGpuIds?: string[];
    /**
     * <p>The filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>availability-zone</code> - The Availability Zone in which the
     *                     Elastic Graphics accelerator resides.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>elastic-gpu-health</code> - The status of the Elastic Graphics accelerator
     *                         (<code>OK</code> | <code>IMPAIRED</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>elastic-gpu-state</code> - The state of the Elastic Graphics accelerator
     *                         (<code>ATTACHED</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>elastic-gpu-type</code> - The type of Elastic Graphics accelerator; for example,
     *                         <code>eg1.medium</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-id</code> - The ID of the instance to which the
     *                     Elastic Graphics accelerator is associated.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining
     *             results, make another call with the returned <code>NextToken</code> value. This value
     *             can be between 5 and 1000.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to request the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeElasticGpusRequest {
    const filterSensitiveLog: (obj: DescribeElasticGpusRequest) => any;
    const isa: (o: any) => o is DescribeElasticGpusRequest;
}
export interface DescribeElasticGpusResult {
    __type?: "DescribeElasticGpusResult";
    /**
     * <p>Information about the Elastic Graphics accelerators.</p>
     */
    ElasticGpuSet?: ElasticGpus[];
    /**
     * <p>The total number of items to return. If the total number of items available is more
     *             than the value specified in max-items then a Next-Token will be provided in the output
     *             that you can use to resume pagination.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to use to retrieve the next page of results. This value is
     *                 <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeElasticGpusResult {
    const filterSensitiveLog: (obj: DescribeElasticGpusResult) => any;
    const isa: (o: any) => o is DescribeElasticGpusResult;
}
export interface DescribeExportImageTasksRequest {
    __type?: "DescribeExportImageTasksRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the export image tasks.</p>
     */
    ExportImageTaskIds?: string[];
    /**
     * <p>Filter tasks using the <code>task-state</code> filter and one of the following values: <code>active</code>,
     *     <code>completed</code>, <code>deleting</code>, or <code>deleted</code>.</p>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return in a single call.</p>
     */
    MaxResults?: number;
    /**
     * <p>A token that indicates the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeExportImageTasksRequest {
    const filterSensitiveLog: (obj: DescribeExportImageTasksRequest) => any;
    const isa: (o: any) => o is DescribeExportImageTasksRequest;
}
export interface DescribeExportImageTasksResult {
    __type?: "DescribeExportImageTasksResult";
    /**
     * <p>Information about the export image tasks.</p>
     */
    ExportImageTasks?: ExportImageTask[];
    /**
     * <p>The token to use to get the next page of results. This value is <code>null</code> when there are no more results
     *    to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeExportImageTasksResult {
    const filterSensitiveLog: (obj: DescribeExportImageTasksResult) => any;
    const isa: (o: any) => o is DescribeExportImageTasksResult;
}
export interface DescribeExportTasksRequest {
    __type?: "DescribeExportTasksRequest";
    /**
     * <p>The export task IDs.</p>
     */
    ExportTaskIds?: string[];
    /**
     * <p>the filters for the export tasks.</p>
     */
    Filters?: Filter[];
}
export declare namespace DescribeExportTasksRequest {
    const filterSensitiveLog: (obj: DescribeExportTasksRequest) => any;
    const isa: (o: any) => o is DescribeExportTasksRequest;
}
export interface DescribeExportTasksResult {
    __type?: "DescribeExportTasksResult";
    /**
     * <p>Information about the export tasks.</p>
     */
    ExportTasks?: ExportTask[];
}
export declare namespace DescribeExportTasksResult {
    const filterSensitiveLog: (obj: DescribeExportTasksResult) => any;
    const isa: (o: any) => o is DescribeExportTasksResult;
}
export interface DescribeFastSnapshotRestoresRequest {
    __type?: "DescribeFastSnapshotRestoresRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>availability-zone</code>: The Availability Zone of the snapshot.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>owner-id</code>: The ID of the AWS account that owns the snapshot.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>snapshot-id</code>: The ID of the snapshot.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code>: The state of fast snapshot restores for the snapshot
     *          (<code>enabling</code> |
     *           <code>optimizing</code> |
     *           <code>enabled</code> |
     *           <code>disabling</code> |
     *           <code>disabled</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeFastSnapshotRestoresRequest {
    const filterSensitiveLog: (obj: DescribeFastSnapshotRestoresRequest) => any;
    const isa: (o: any) => o is DescribeFastSnapshotRestoresRequest;
}
export interface DescribeFastSnapshotRestoresResult {
    __type?: "DescribeFastSnapshotRestoresResult";
    /**
     * <p>Information about the state of fast snapshot restores.</p>
     */
    FastSnapshotRestores?: DescribeFastSnapshotRestoreSuccessItem[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeFastSnapshotRestoresResult {
    const filterSensitiveLog: (obj: DescribeFastSnapshotRestoresResult) => any;
    const isa: (o: any) => o is DescribeFastSnapshotRestoresResult;
}
/**
 * <p>Describes fast snapshot restores for a snapshot.</p>
 */
export interface DescribeFastSnapshotRestoreSuccessItem {
    __type?: "DescribeFastSnapshotRestoreSuccessItem";
    /**
     * <p>The Availability Zone.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The time at which fast snapshot restores entered the <code>disabled</code> state.</p>
     */
    DisabledTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>disabling</code> state.</p>
     */
    DisablingTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>enabled</code> state.</p>
     */
    EnabledTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>enabling</code> state.</p>
     */
    EnablingTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>optimizing</code> state.</p>
     */
    OptimizingTime?: Date;
    /**
     * <p>The alias of the snapshot owner.</p>
     */
    OwnerAlias?: string;
    /**
     * <p>The ID of the AWS account that owns the snapshot.</p>
     */
    OwnerId?: string;
    /**
     * <p>The ID of the snapshot.</p>
     */
    SnapshotId?: string;
    /**
     * <p>The state of fast snapshot restores.</p>
     */
    State?: FastSnapshotRestoreStateCode | string;
    /**
     * <p>The reason for the state transition. The possible values are as follows:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>Client.UserInitiated</code> - The state successfully transitioned to <code>enabling</code> or
     *           <code>disabling</code>.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>Client.UserInitiated - Lifecycle state transition</code> - The state successfully transitioned
     *           to <code>optimizing</code>, <code>enabled</code>, or <code>disabled</code>.</p>
     *             </li>
     *          </ul>
     */
    StateTransitionReason?: string;
}
export declare namespace DescribeFastSnapshotRestoreSuccessItem {
    const filterSensitiveLog: (obj: DescribeFastSnapshotRestoreSuccessItem) => any;
    const isa: (o: any) => o is DescribeFastSnapshotRestoreSuccessItem;
}
/**
 * <p>Describes the instances that could not be launched by the fleet.</p>
 */
export interface DescribeFleetError {
    __type?: "DescribeFleetError";
    /**
     * <p>The error code that indicates why the instance could not be launched. For more
     *          information about error codes, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/errors-overview.html.html">Error Codes</a>.</p>
     */
    ErrorCode?: string;
    /**
     * <p>The error message that describes why the instance could not be launched. For more
     *          information about error messages, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/errors-overview.html.html">Error Codes</a>.</p>
     */
    ErrorMessage?: string;
    /**
     * <p>The launch templates and overrides that were used for launching the instances. The
     *          values that you specify in the Overrides replace the values in the launch template.</p>
     */
    LaunchTemplateAndOverrides?: LaunchTemplateAndOverridesResponse;
    /**
     * <p>Indicates if the instance that could not be launched was a Spot Instance or On-Demand Instance.</p>
     */
    Lifecycle?: InstanceLifecycle | string;
}
export declare namespace DescribeFleetError {
    const filterSensitiveLog: (obj: DescribeFleetError) => any;
    const isa: (o: any) => o is DescribeFleetError;
}
export interface DescribeFleetHistoryRequest {
    __type?: "DescribeFleetHistoryRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The type of events to describe. By default, all events are described.</p>
     */
    EventType?: FleetEventType | string;
    /**
     * <p>The ID of the EC2 Fleet.</p>
     */
    FleetId: string | undefined;
    /**
     * <p>The maximum number of results to return in a single call. Specify a value between 1 and
     *          1000. The default value is 1000. To retrieve the remaining results, make another call with
     *          the returned <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The start date and time for the events, in UTC format (for example,
     *             <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    StartTime: Date | undefined;
}
export declare namespace DescribeFleetHistoryRequest {
    const filterSensitiveLog: (obj: DescribeFleetHistoryRequest) => any;
    const isa: (o: any) => o is DescribeFleetHistoryRequest;
}
export interface DescribeFleetHistoryResult {
    __type?: "DescribeFleetHistoryResult";
    /**
     * <p>The ID of the EC Fleet.</p>
     */
    FleetId?: string;
    /**
     * <p>Information about the events in the history of the EC2 Fleet.</p>
     */
    HistoryRecords?: HistoryRecordEntry[];
    /**
     * <p>The last date and time for the events, in UTC format (for example,
     *             <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).
     *          All records up to this time were retrieved.</p>
     *          <p>If <code>nextToken</code> indicates that there are more results, this value is not
     *          present.</p>
     */
    LastEvaluatedTime?: Date;
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The start date and time for the events, in UTC format (for example,
     *             <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    StartTime?: Date;
}
export declare namespace DescribeFleetHistoryResult {
    const filterSensitiveLog: (obj: DescribeFleetHistoryResult) => any;
    const isa: (o: any) => o is DescribeFleetHistoryResult;
}
export interface DescribeFleetInstancesRequest {
    __type?: "DescribeFleetInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>instance-type</code> - The instance type.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The ID of the EC2 Fleet.</p>
     */
    FleetId: string | undefined;
    /**
     * <p>The maximum number of results to return in a single call. Specify a value between 1 and
     *          1000. The default value is 1000. To retrieve the remaining results, make another call with
     *          the returned <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeFleetInstancesRequest {
    const filterSensitiveLog: (obj: DescribeFleetInstancesRequest) => any;
    const isa: (o: any) => o is DescribeFleetInstancesRequest;
}
export interface DescribeFleetInstancesResult {
    __type?: "DescribeFleetInstancesResult";
    /**
     * <p>The running instances. This list is refreshed periodically and might be out of
     *          date.</p>
     */
    ActiveInstances?: ActiveInstance[];
    /**
     * <p>The ID of the EC2 Fleet.</p>
     */
    FleetId?: string;
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeFleetInstancesResult {
    const filterSensitiveLog: (obj: DescribeFleetInstancesResult) => any;
    const isa: (o: any) => o is DescribeFleetInstancesResult;
}
/**
 * <p>Describes the instances that were launched by the fleet.</p>
 */
export interface DescribeFleetsInstances {
    __type?: "DescribeFleetsInstances";
    /**
     * <p>The IDs of the instances.</p>
     */
    InstanceIds?: string[];
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The launch templates and overrides that were used for launching the instances. The
     *          values that you specify in the Overrides replace the values in the launch template.</p>
     */
    LaunchTemplateAndOverrides?: LaunchTemplateAndOverridesResponse;
    /**
     * <p>Indicates if the instance that was launched is a Spot Instance or On-Demand Instance.</p>
     */
    Lifecycle?: InstanceLifecycle | string;
    /**
     * <p>The value is <code>Windows</code> for Windows instances. Otherwise, the value is
     *          blank.</p>
     */
    Platform?: PlatformValues | string;
}
export declare namespace DescribeFleetsInstances {
    const filterSensitiveLog: (obj: DescribeFleetsInstances) => any;
    const isa: (o: any) => o is DescribeFleetsInstances;
}
export interface DescribeFleetsRequest {
    __type?: "DescribeFleetsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>activity-status</code> - The progress of the EC2 Fleet ( <code>error</code> |
     *                   <code>pending-fulfillment</code> | <code>pending-termination</code> |
     *                   <code>fulfilled</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>excess-capacity-termination-policy</code> - Indicates whether to terminate
     *                running instances if the target capacity is decreased below the current EC2 Fleet size
     *                   (<code>true</code> | <code>false</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>fleet-state</code> - The state of the EC2 Fleet (<code>submitted</code> |
     *                   <code>active</code> | <code>deleted</code> | <code>failed</code> |
     *                   <code>deleted-running</code> | <code>deleted-terminating</code> |
     *                   <code>modifying</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>replace-unhealthy-instances</code> - Indicates whether EC2 Fleet should replace
     *                unhealthy instances (<code>true</code> | <code>false</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>type</code> - The type of request (<code>instant</code> |
     *                   <code>request</code> | <code>maintain</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The ID of the EC2 Fleets.</p>
     */
    FleetIds?: string[];
    /**
     * <p>The maximum number of results to return in a single call. Specify a value between 1 and
     *          1000. The default value is 1000. To retrieve the remaining results, make another call with
     *          the returned <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeFleetsRequest {
    const filterSensitiveLog: (obj: DescribeFleetsRequest) => any;
    const isa: (o: any) => o is DescribeFleetsRequest;
}
export interface DescribeFleetsResult {
    __type?: "DescribeFleetsResult";
    /**
     * <p>Information about the EC2 Fleets.</p>
     */
    Fleets?: FleetData[];
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeFleetsResult {
    const filterSensitiveLog: (obj: DescribeFleetsResult) => any;
    const isa: (o: any) => o is DescribeFleetsResult;
}
export interface DescribeFlowLogsRequest {
    __type?: "DescribeFlowLogsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                     <code>deliver-log-status</code> - The status of the logs delivery (<code>SUCCESS</code> |
     *                     <code>FAILED</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>log-destination-type</code> - The type of destination to which the flow
     *                     log publishes data. Possible destination types include
     *                     <code>cloud-watch-logs</code> and <code>S3</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>flow-log-id</code> - The ID of the flow log.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>log-group-name</code> - The name of the log group.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>resource-id</code> - The ID of the VPC, subnet, or network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>traffic-type</code> - The type of traffic (<code>ACCEPT</code> |
     *                     <code>REJECT</code> | <code>ALL</code>).</p>
     *             </li>
     *          </ul>
     */
    Filter?: Filter[];
    /**
     * <p>One or more flow log IDs.</p>
     *         <p>Constraint: Maximum of 1000 flow log IDs.</p>
     */
    FlowLogIds?: string[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeFlowLogsRequest {
    const filterSensitiveLog: (obj: DescribeFlowLogsRequest) => any;
    const isa: (o: any) => o is DescribeFlowLogsRequest;
}
export interface DescribeFlowLogsResult {
    __type?: "DescribeFlowLogsResult";
    /**
     * <p>Information about the flow logs.</p>
     */
    FlowLogs?: FlowLog[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeFlowLogsResult {
    const filterSensitiveLog: (obj: DescribeFlowLogsResult) => any;
    const isa: (o: any) => o is DescribeFlowLogsResult;
}
export interface DescribeFpgaImageAttributeRequest {
    __type?: "DescribeFpgaImageAttributeRequest";
    /**
     * <p>The AFI attribute.</p>
     */
    Attribute: FpgaImageAttributeName | string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the AFI.</p>
     */
    FpgaImageId: string | undefined;
}
export declare namespace DescribeFpgaImageAttributeRequest {
    const filterSensitiveLog: (obj: DescribeFpgaImageAttributeRequest) => any;
    const isa: (o: any) => o is DescribeFpgaImageAttributeRequest;
}
export interface DescribeFpgaImageAttributeResult {
    __type?: "DescribeFpgaImageAttributeResult";
    /**
     * <p>Information about the attribute.</p>
     */
    FpgaImageAttribute?: FpgaImageAttribute;
}
export declare namespace DescribeFpgaImageAttributeResult {
    const filterSensitiveLog: (obj: DescribeFpgaImageAttributeResult) => any;
    const isa: (o: any) => o is DescribeFpgaImageAttributeResult;
}
export interface DescribeFpgaImagesRequest {
    __type?: "DescribeFpgaImagesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>create-time</code> - The creation time of the AFI.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>fpga-image-id</code> - The FPGA image identifier (AFI ID).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>fpga-image-global-id</code> - The global FPGA image identifier (AGFI ID).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>name</code> - The name of the AFI.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>owner-id</code> - The AWS account ID of the AFI owner.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>product-code</code> - The product code.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>shell-version</code> - The version of the AWS Shell that was used to create the bitstream.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the AFI (<code>pending</code> | <code>failed</code> | <code>available</code> | <code>unavailable</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>update-time</code> - The time of the most recent update.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The AFI IDs.</p>
     */
    FpgaImageIds?: string[];
    /**
     * <p>The maximum number of results to return in a single call.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>Filters the AFI by owner. Specify an AWS account ID, <code>self</code> (owner is the sender of the request), or an AWS owner alias (valid values are <code>amazon</code> | <code>aws-marketplace</code>).</p>
     */
    Owners?: string[];
}
export declare namespace DescribeFpgaImagesRequest {
    const filterSensitiveLog: (obj: DescribeFpgaImagesRequest) => any;
    const isa: (o: any) => o is DescribeFpgaImagesRequest;
}
export interface DescribeFpgaImagesResult {
    __type?: "DescribeFpgaImagesResult";
    /**
     * <p>Information about the FPGA images.</p>
     */
    FpgaImages?: FpgaImage[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeFpgaImagesResult {
    const filterSensitiveLog: (obj: DescribeFpgaImagesResult) => any;
    const isa: (o: any) => o is DescribeFpgaImagesResult;
}
export interface DescribeHostReservationOfferingsRequest {
    __type?: "DescribeHostReservationOfferingsRequest";
    /**
     * <p>The filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                     <code>instance-family</code> - The instance family of the offering (for example,
     *                         <code>m4</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>payment-option</code> - The payment option (<code>NoUpfront</code> |
     *                         <code>PartialUpfront</code> | <code>AllUpfront</code>).</p>
     *             </li>
     *          </ul>
     */
    Filter?: Filter[];
    /**
     * <p>This is the maximum duration of the reservation to purchase, specified in seconds.
     *             Reservations are available in one-year and three-year terms. The number of seconds
     *             specified must be the number of seconds in a year (365x24x60x60) times one of the
     *             supported durations (1 or 3). For example, specify 94608000 for three years.</p>
     */
    MaxDuration?: number;
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned <code>nextToken</code> value. This value can be between 5 and 500. If <code>maxResults</code> is given a larger value than 500, you receive an error.</p>
     */
    MaxResults?: number;
    /**
     * <p>This is the minimum duration of the reservation you'd like to purchase, specified
     *             in seconds. Reservations are available in one-year and three-year terms. The number of
     *             seconds specified must be the number of seconds in a year (365x24x60x60) times one of
     *             the supported durations (1 or 3). For example, specify 31536000 for one year.</p>
     */
    MinDuration?: number;
    /**
     * <p>The token to use to retrieve the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the reservation offering.</p>
     */
    OfferingId?: string;
}
export declare namespace DescribeHostReservationOfferingsRequest {
    const filterSensitiveLog: (obj: DescribeHostReservationOfferingsRequest) => any;
    const isa: (o: any) => o is DescribeHostReservationOfferingsRequest;
}
export interface DescribeHostReservationOfferingsResult {
    __type?: "DescribeHostReservationOfferingsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the offerings.</p>
     */
    OfferingSet?: HostOffering[];
}
export declare namespace DescribeHostReservationOfferingsResult {
    const filterSensitiveLog: (obj: DescribeHostReservationOfferingsResult) => any;
    const isa: (o: any) => o is DescribeHostReservationOfferingsResult;
}
export interface DescribeHostReservationsRequest {
    __type?: "DescribeHostReservationsRequest";
    /**
     * <p>The filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                     <code>instance-family</code> - The instance family (for example,
     *                     <code>m4</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>payment-option</code> - The payment option (<code>NoUpfront</code> |
     *                         <code>PartialUpfront</code> | <code>AllUpfront</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>state</code> - The state of the reservation (<code>payment-pending</code>
     *                     | <code>payment-failed</code> | <code>active</code> |
     *                     <code>retired</code>).</p>
     *             </li>
     *             <li>
     *         		     <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *         	   </li>
     *             <li>
     *         		     <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *         	   </li>
     *          </ul>
     */
    Filter?: Filter[];
    /**
     * <p>The host reservation IDs.</p>
     */
    HostReservationIdSet?: string[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned <code>nextToken</code> value. This value can be between 5 and 500. If <code>maxResults</code> is given a larger value than 500, you receive an error.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to use to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeHostReservationsRequest {
    const filterSensitiveLog: (obj: DescribeHostReservationsRequest) => any;
    const isa: (o: any) => o is DescribeHostReservationsRequest;
}
export interface DescribeHostReservationsResult {
    __type?: "DescribeHostReservationsResult";
    /**
     * <p>Details about the reservation's configuration.</p>
     */
    HostReservationSet?: HostReservation[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeHostReservationsResult {
    const filterSensitiveLog: (obj: DescribeHostReservationsResult) => any;
    const isa: (o: any) => o is DescribeHostReservationsResult;
}
export interface DescribeHostsRequest {
    __type?: "DescribeHostsRequest";
    /**
     * <p>The filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>auto-placement</code> - Whether auto-placement is enabled or disabled
     *                         (<code>on</code> | <code>off</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>availability-zone</code> - The Availability Zone of the
     *                     host.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>client-token</code> - The idempotency token that you provided when you
     *                     allocated the host.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>host-reservation-id</code> - The ID of the reservation assigned to
     *                     this host.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-type</code> - The instance type size that the Dedicated Host
     *                     is configured to support.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>state</code> - The allocation state of the Dedicated Host
     *                         (<code>available</code> | <code>under-assessment</code> |
     *                         <code>permanent-failure</code> | <code>released</code> |
     *                         <code>released-permanent-failure</code>).</p>
     *             </li>
     *             <li>
     *         		     <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *         	   </li>
     *          </ul>
     */
    Filter?: Filter[];
    /**
     * <p>The IDs of the Dedicated Hosts. The IDs are used for targeted instance
     *             launches.</p>
     */
    HostIds?: string[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned <code>nextToken</code> value. This value can be between 5 and 500. If <code>maxResults</code> is given a larger value than 500, you receive an error.</p>
     *          <p>You cannot specify this parameter and the host IDs parameter in the same request.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to use to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeHostsRequest {
    const filterSensitiveLog: (obj: DescribeHostsRequest) => any;
    const isa: (o: any) => o is DescribeHostsRequest;
}
export interface DescribeHostsResult {
    __type?: "DescribeHostsResult";
    /**
     * <p>Information about the Dedicated Hosts.</p>
     */
    Hosts?: Host[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeHostsResult {
    const filterSensitiveLog: (obj: DescribeHostsResult) => any;
    const isa: (o: any) => o is DescribeHostsResult;
}
export interface DescribeIamInstanceProfileAssociationsRequest {
    __type?: "DescribeIamInstanceProfileAssociationsRequest";
    /**
     * <p>The IAM instance profile associations.</p>
     */
    AssociationIds?: string[];
    /**
     * <p>The filters.</p>
     *         <ul>
     *             <li>
     *                <p>
     *                   <code>instance-id</code> - The ID of the instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the association (<code>associating</code> |
     *                 <code>associated</code> | <code>disassociating</code> |
     *                 <code>disassociated</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining
     *             results, make another call with the returned <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to request the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeIamInstanceProfileAssociationsRequest {
    const filterSensitiveLog: (obj: DescribeIamInstanceProfileAssociationsRequest) => any;
    const isa: (o: any) => o is DescribeIamInstanceProfileAssociationsRequest;
}
export interface DescribeIamInstanceProfileAssociationsResult {
    __type?: "DescribeIamInstanceProfileAssociationsResult";
    /**
     * <p>Information about the IAM instance profile associations.</p>
     */
    IamInstanceProfileAssociations?: IamInstanceProfileAssociation[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeIamInstanceProfileAssociationsResult {
    const filterSensitiveLog: (obj: DescribeIamInstanceProfileAssociationsResult) => any;
    const isa: (o: any) => o is DescribeIamInstanceProfileAssociationsResult;
}
export interface DescribeIdentityIdFormatRequest {
    __type?: "DescribeIdentityIdFormatRequest";
    /**
     * <p>The ARN of the principal, which can be an IAM role, IAM user, or the root user.</p>
     */
    PrincipalArn: string | undefined;
    /**
     * <p>The type of resource: <code>bundle</code> |
     *           <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> |
     *           <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> |
     *           <code>export-task</code> | <code>flow-log</code> | <code>image</code> |
     *           <code>import-task</code> | <code>instance</code> | <code>internet-gateway</code> |
     *           <code>network-acl</code> | <code>network-acl-association</code> |
     *           <code>network-interface</code> | <code>network-interface-attachment</code> |
     *           <code>prefix-list</code> | <code>reservation</code> | <code>route-table</code> |
     *           <code>route-table-association</code> | <code>security-group</code> |
     *           <code>snapshot</code> | <code>subnet</code> |
     *           <code>subnet-cidr-block-association</code> | <code>volume</code> | <code>vpc</code>
     *           | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> |
     *           <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>
     *          </p>
     */
    Resource?: string;
}
export declare namespace DescribeIdentityIdFormatRequest {
    const filterSensitiveLog: (obj: DescribeIdentityIdFormatRequest) => any;
    const isa: (o: any) => o is DescribeIdentityIdFormatRequest;
}
export interface DescribeIdentityIdFormatResult {
    __type?: "DescribeIdentityIdFormatResult";
    /**
     * <p>Information about the ID format for the resources.</p>
     */
    Statuses?: IdFormat[];
}
export declare namespace DescribeIdentityIdFormatResult {
    const filterSensitiveLog: (obj: DescribeIdentityIdFormatResult) => any;
    const isa: (o: any) => o is DescribeIdentityIdFormatResult;
}
export interface DescribeIdFormatRequest {
    __type?: "DescribeIdFormatRequest";
    /**
     * <p>The type of resource: <code>bundle</code> |
     *            <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> |
     *            <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> |
     *            <code>export-task</code> | <code>flow-log</code> | <code>image</code> |
     *            <code>import-task</code> | <code>instance</code> | <code>internet-gateway</code> |
     *            <code>network-acl</code> | <code>network-acl-association</code> |
     *            <code>network-interface</code> | <code>network-interface-attachment</code> |
     *            <code>prefix-list</code> | <code>reservation</code> | <code>route-table</code> |
     *            <code>route-table-association</code> | <code>security-group</code> |
     *            <code>snapshot</code> | <code>subnet</code> |
     *            <code>subnet-cidr-block-association</code> | <code>volume</code> | <code>vpc</code>
     *            | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> |
     *            <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>
     *          </p>
     */
    Resource?: string;
}
export declare namespace DescribeIdFormatRequest {
    const filterSensitiveLog: (obj: DescribeIdFormatRequest) => any;
    const isa: (o: any) => o is DescribeIdFormatRequest;
}
export interface DescribeIdFormatResult {
    __type?: "DescribeIdFormatResult";
    /**
     * <p>Information about the ID format for the resource.</p>
     */
    Statuses?: IdFormat[];
}
export declare namespace DescribeIdFormatResult {
    const filterSensitiveLog: (obj: DescribeIdFormatResult) => any;
    const isa: (o: any) => o is DescribeIdFormatResult;
}
/**
 * <p>Contains the parameters for DescribeImageAttribute.</p>
 */
export interface DescribeImageAttributeRequest {
    __type?: "DescribeImageAttributeRequest";
    /**
     * <p>The AMI attribute.</p>
     *    	     <p>
     *             <b>Note</b>: Depending on your account privileges, the <code>blockDeviceMapping</code>
     *    		attribute may return a <code>Client.AuthFailure</code> error. If this happens, use
     *    		<a>DescribeImages</a> to get information about the block device mapping for the AMI.</p>
     */
    Attribute: ImageAttributeName | string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the AMI.</p>
     */
    ImageId: string | undefined;
}
export declare namespace DescribeImageAttributeRequest {
    const filterSensitiveLog: (obj: DescribeImageAttributeRequest) => any;
    const isa: (o: any) => o is DescribeImageAttributeRequest;
}
export interface DescribeImagesRequest {
    __type?: "DescribeImagesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Scopes the images by users with explicit launch permissions.
     * 				Specify an AWS account ID, <code>self</code> (the sender of the request),
     * 				or <code>all</code> (public AMIs).</p>
     */
    ExecutableUsers?: string[];
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>architecture</code> - The image architecture (<code>i386</code> |
     *             <code>x86_64</code> | <code>arm64</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>block-device-mapping.delete-on-termination</code> - A Boolean value that indicates
     *           whether the Amazon EBS volume is deleted on instance termination.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>block-device-mapping.device-name</code> - The device name specified in the block device mapping (for
     *           example, <code>/dev/sdh</code> or <code>xvdh</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>block-device-mapping.snapshot-id</code> - The ID of the snapshot used for the EBS
     *           volume.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>block-device-mapping.volume-size</code> - The volume size of the EBS volume, in GiB.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>block-device-mapping.volume-type</code> - The volume type of the EBS volume
     *             (<code>gp2</code> | <code>io1</code> | <code>st1 </code>| <code>sc1</code> |
     *             <code>standard</code>).</p>
     *             </li>
     *             <li>
     *     		         <p>
     *     			           <code>block-device-mapping.encrypted</code> - A Boolean that indicates whether the EBS volume is encrypted.</p>
     *     	       </li>
     *             <li>
     *                <p>
     *                   <code>description</code> - The description of the image (provided during image
     *           creation).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>ena-support</code> - A Boolean that indicates whether enhanced networking
     *           with ENA is enabled.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>hypervisor</code> - The hypervisor type (<code>ovm</code> |
     *           <code>xen</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>image-id</code> - The ID of the image.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>image-type</code> - The image type (<code>machine</code> | <code>kernel</code> |
     *             <code>ramdisk</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>is-public</code> - A Boolean that indicates whether the image is public.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>kernel-id</code> - The kernel ID.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>manifest-location</code> - The location of the image manifest.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>name</code> - The name of the AMI (provided during image creation).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>owner-alias</code> - String value from an
     *           Amazon-maintained list (<code>amazon</code> | <code>aws-marketplace</code> |
     *           <code>microsoft</code>) of snapshot owners. Not to be confused with the
     *           user-configured AWS account alias, which is set from the IAM
     *           console.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>owner-id</code> - The AWS account ID of the image owner.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>platform</code> - The platform. To only list Windows-based AMIs, use
     *             <code>windows</code>.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>product-code</code> - The product code.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>product-code.type</code> - The type of the product code (<code>devpay</code> |
     *             <code>marketplace</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>ramdisk-id</code> - The RAM disk ID.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>root-device-name</code> - The device name of the root device volume (for example, <code>/dev/sda1</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>root-device-type</code> - The type of the root device volume (<code>ebs</code> |
     *             <code>instance-store</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the image (<code>available</code> | <code>pending</code>
     *           | <code>failed</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state-reason-code</code> - The reason code for the state change.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state-reason-message</code> - The message for the state change.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>sriov-net-support</code> - A value of <code>simple</code> indicates
     *                     that enhanced networking with the Intel 82599 VF interface is enabled.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>virtualization-type</code> - The virtualization type (<code>paravirtual</code> |
     *             <code>hvm</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The image IDs.</p>
     *          <p>Default: Describes all images available to you.</p>
     */
    ImageIds?: string[];
    /**
     * <p>Filters the images by the owner. Specify an AWS account ID, <code>self</code> (owner is
     *       the sender of the request), or an AWS owner alias (valid values are <code>amazon</code> |
     *         <code>aws-marketplace</code> | <code>microsoft</code>). Omitting this option returns all
     *       images for which you have launch permissions, regardless of ownership.</p>
     */
    Owners?: string[];
}
export declare namespace DescribeImagesRequest {
    const filterSensitiveLog: (obj: DescribeImagesRequest) => any;
    const isa: (o: any) => o is DescribeImagesRequest;
}
export interface DescribeImagesResult {
    __type?: "DescribeImagesResult";
    /**
     * <p>Information about the images.</p>
     */
    Images?: Image[];
}
export declare namespace DescribeImagesResult {
    const filterSensitiveLog: (obj: DescribeImagesResult) => any;
    const isa: (o: any) => o is DescribeImagesResult;
}
export interface DescribeImportImageTasksRequest {
    __type?: "DescribeImportImageTasksRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Filter tasks using the <code>task-state</code> filter and one of the following values: <code>active</code>,
     *     <code>completed</code>, <code>deleting</code>, or <code>deleted</code>.</p>
     */
    Filters?: Filter[];
    /**
     * <p>The IDs of the import image tasks.</p>
     */
    ImportTaskIds?: string[];
    /**
     * <p>The maximum number of results to return in a single call.</p>
     */
    MaxResults?: number;
    /**
     * <p>A token that indicates the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeImportImageTasksRequest {
    const filterSensitiveLog: (obj: DescribeImportImageTasksRequest) => any;
    const isa: (o: any) => o is DescribeImportImageTasksRequest;
}
export interface DescribeImportImageTasksResult {
    __type?: "DescribeImportImageTasksResult";
    /**
     * <p>A list of zero or more import image tasks that are currently active or were completed or canceled in the
     *    previous 7 days.</p>
     */
    ImportImageTasks?: ImportImageTask[];
    /**
     * <p>The token to use to get the next page of results. This value is <code>null</code> when there are no more results
     *    to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeImportImageTasksResult {
    const filterSensitiveLog: (obj: DescribeImportImageTasksResult) => any;
    const isa: (o: any) => o is DescribeImportImageTasksResult;
}
export interface DescribeImportSnapshotTasksRequest {
    __type?: "DescribeImportSnapshotTasksRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     */
    Filters?: Filter[];
    /**
     * <p>A list of import snapshot task IDs.</p>
     */
    ImportTaskIds?: string[];
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining results, make another call
     *    with the returned <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>A token that indicates the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeImportSnapshotTasksRequest {
    const filterSensitiveLog: (obj: DescribeImportSnapshotTasksRequest) => any;
    const isa: (o: any) => o is DescribeImportSnapshotTasksRequest;
}
export interface DescribeImportSnapshotTasksResult {
    __type?: "DescribeImportSnapshotTasksResult";
    /**
     * <p>A list of zero or more import snapshot tasks that are currently active or were completed or canceled in the
     *    previous 7 days.</p>
     */
    ImportSnapshotTasks?: ImportSnapshotTask[];
    /**
     * <p>The token to use to get the next page of results. This value is <code>null</code> when there are no more results
     *    to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeImportSnapshotTasksResult {
    const filterSensitiveLog: (obj: DescribeImportSnapshotTasksResult) => any;
    const isa: (o: any) => o is DescribeImportSnapshotTasksResult;
}
export interface DescribeInstanceAttributeRequest {
    __type?: "DescribeInstanceAttributeRequest";
    /**
     * <p>The instance attribute.</p>
     *         <p>Note: The <code>enaSupport</code> attribute is not supported at this time.</p>
     */
    Attribute: InstanceAttributeName | string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
}
export declare namespace DescribeInstanceAttributeRequest {
    const filterSensitiveLog: (obj: DescribeInstanceAttributeRequest) => any;
    const isa: (o: any) => o is DescribeInstanceAttributeRequest;
}
export interface DescribeInstanceCreditSpecificationsRequest {
    __type?: "DescribeInstanceCreditSpecificationsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>instance-id</code> - The ID of the instance.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The instance IDs.</p>
     *         <p>Default: Describes all your instances.</p>
     *         <p>Constraints: Maximum 1000 explicitly specified instance IDs.</p>
     */
    InstanceIds?: string[];
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining
     *             results, make another call with the returned <code>NextToken</code> value. This value
     *             can be between 5 and 1000. You cannot specify this parameter and the instance IDs
     *             parameter in the same call.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeInstanceCreditSpecificationsRequest {
    const filterSensitiveLog: (obj: DescribeInstanceCreditSpecificationsRequest) => any;
    const isa: (o: any) => o is DescribeInstanceCreditSpecificationsRequest;
}
export interface DescribeInstanceCreditSpecificationsResult {
    __type?: "DescribeInstanceCreditSpecificationsResult";
    /**
     * <p>Information about the credit option for CPU usage of an instance.</p>
     */
    InstanceCreditSpecifications?: InstanceCreditSpecification[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code>
     *             when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeInstanceCreditSpecificationsResult {
    const filterSensitiveLog: (obj: DescribeInstanceCreditSpecificationsResult) => any;
    const isa: (o: any) => o is DescribeInstanceCreditSpecificationsResult;
}
export interface DescribeInstancesRequest {
    __type?: "DescribeInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                     <code>affinity</code> - The affinity setting for an instance running on a
     *                     Dedicated Host (<code>default</code> | <code>host</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>architecture</code> - The instance architecture (<code>i386</code> |
     *                         <code>x86_64</code> | <code>arm64</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>availability-zone</code> - The Availability Zone of the instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>block-device-mapping.attach-time</code> - The attach time for an EBS
     *                     volume mapped to the instance, for example,
     *                         <code>2010-09-15T17:15:20.000Z</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>block-device-mapping.delete-on-termination</code> - A Boolean that
     *                     indicates whether the EBS volume is deleted on instance termination.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>block-device-mapping.device-name</code> - The device name specified in the
     *                     block device mapping (for example, <code>/dev/sdh</code> or
     *                     <code>xvdh</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>block-device-mapping.status</code> - The status for the EBS volume
     *                         (<code>attaching</code> | <code>attached</code> | <code>detaching</code> |
     *                         <code>detached</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>block-device-mapping.volume-id</code> - The volume ID of the EBS
     *                     volume.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>client-token</code> - The idempotency token you provided when you launched
     *                     the instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>dns-name</code> - The public DNS name of the instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>group-id</code> - The ID of the security group for the instance.
     *                     EC2-Classic only.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>group-name</code> - The name of the security group for the instance.
     *                     EC2-Classic only.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>hibernation-options.configured</code> - A Boolean that indicates whether
     *                     the instance is enabled for hibernation. A value of <code>true</code> means that
     *                     the instance is enabled for hibernation. </p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>host-id</code> - The ID of the Dedicated Host on which the instance is
     *                     running, if applicable.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>hypervisor</code> - The hypervisor type of the instance
     *                         (<code>ovm</code> | <code>xen</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>iam-instance-profile.arn</code> - The instance profile associated with
     *                     the instance. Specified as an ARN.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>image-id</code> - The ID of the image used to launch the
     *                     instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-id</code> - The ID of the instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-lifecycle</code> - Indicates whether this is a Spot Instance or
     *                     a Scheduled Instance (<code>spot</code> | <code>scheduled</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-state-code</code> - The state of the instance, as a 16-bit
     *                     unsigned integer. The high byte is used for internal purposes and should be
     *                     ignored. The low byte is set based on the state represented. The valid values
     *                     are: 0 (pending), 16 (running), 32 (shutting-down), 48 (terminated), 64
     *                     (stopping), and 80 (stopped).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-state-name</code> - The state of the instance
     *                         (<code>pending</code> | <code>running</code> | <code>shutting-down</code> |
     *                         <code>terminated</code> | <code>stopping</code> |
     *                     <code>stopped</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-type</code> - The type of instance (for example,
     *                         <code>t2.micro</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance.group-id</code> - The ID of the security group for the
     *                     instance. </p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance.group-name</code> - The name of the security group for the
     *                     instance. </p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ip-address</code> - The public IPv4 address of the instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>kernel-id</code> - The kernel ID.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>key-name</code> - The name of the key pair used when the instance was
     *                     launched.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>launch-index</code> - When launching multiple instances, this is the
     *                     index for the instance in the launch group (for example, 0, 1, 2, and so on). </p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>launch-time</code> - The time when the instance was launched.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>metadata-options.http-tokens</code> - The metadata request authorization
     *                     state (<code>optional</code> | <code>required</code>)</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>metadata-options.http-put-response-hop-limit</code> - The http metadata
     *                     request put response hop limit (integer, possible values <code>1</code> to
     *                         <code>64</code>)</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>metadata-options.http-endpoint</code> - Enable or disable metadata
     *                     access on http endpoint (<code>enabled</code> | <code>disabled</code>)</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>monitoring-state</code> - Indicates whether detailed monitoring is
     *                     enabled (<code>disabled</code> | <code>enabled</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.addresses.private-ip-address</code> - The private IPv4
     *                     address associated with the network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.addresses.primary</code> - Specifies whether the IPv4
     *                     address of the network interface is the primary private IPv4 address.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.addresses.association.public-ip</code> - The ID of the
     *                     association of an Elastic IP address (IPv4) with a network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.addresses.association.ip-owner-id</code> - The owner
     *                     ID of the private IPv4 address associated with the network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.association.public-ip</code> - The address of the
     *                     Elastic IP address (IPv4) bound to the network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.association.ip-owner-id</code> - The owner of the
     *                     Elastic IP address (IPv4) associated with the network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.association.allocation-id</code> - The allocation ID
     *                     returned when you allocated the Elastic IP address (IPv4) for your network
     *                     interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.association.association-id</code> - The association ID
     *                     returned when the network interface was associated with an IPv4 address.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.attachment.attachment-id</code> - The ID of the
     *                     interface attachment.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.attachment.instance-id</code> - The ID of the instance
     *                     to which the network interface is attached.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.attachment.instance-owner-id</code> - The owner ID of
     *                     the instance to which the network interface is attached.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.attachment.device-index</code> - The device index to
     *                     which the network interface is attached.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.attachment.status</code> - The status of the
     *                     attachment (<code>attaching</code> | <code>attached</code> |
     *                         <code>detaching</code> | <code>detached</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.attachment.attach-time</code> - The time that the
     *                     network interface was attached to an instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.attachment.delete-on-termination</code> - Specifies
     *                     whether the attachment is deleted when an instance is terminated.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.availability-zone</code> - The Availability Zone for
     *                     the network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.description</code> - The description of the network
     *                     interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.group-id</code> - The ID of a security group
     *                     associated with the network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.group-name</code> - The name of a security group
     *                     associated with the network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.ipv6-addresses.ipv6-address</code> - The IPv6 address
     *                     associated with the network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.mac-address</code> - The MAC address of the network
     *                     interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.network-interface-id</code> - The ID of the network
     *                     interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.owner-id</code> - The ID of the owner of the network
     *                     interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.private-dns-name</code> - The private DNS name of the
     *                     network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.requester-id</code> - The requester ID for the network
     *                     interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.requester-managed</code> - Indicates whether the
     *                     network interface is being managed by AWS.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.status</code> - The status of the network interface
     *                         (<code>available</code>) | <code>in-use</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.source-dest-check</code> - Whether the network
     *                     interface performs source/destination checking. A value of <code>true</code>
     *                     means that checking is enabled, and <code>false</code> means that checking is
     *                     disabled. The value must be <code>false</code> for the network interface to
     *                     perform network address translation (NAT) in your VPC.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.subnet-id</code> - The ID of the subnet for the
     *                     network interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>network-interface.vpc-id</code> - The ID of the VPC for the network
     *                     interface.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>owner-id</code> - The AWS account ID of the instance owner.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>placement-group-name</code> - The name of the placement group for the
     *                     instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>placement-partition-number</code> - The partition in which the instance is
     *                     located.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>platform</code> - The platform. To list only Windows instances, use
     *                         <code>windows</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>private-dns-name</code> - The private IPv4 DNS name of the
     *                     instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>private-ip-address</code> - The private IPv4 address of the
     *                     instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>product-code</code> - The product code associated with the AMI used to
     *                     launch the instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>product-code.type</code> - The type of product code (<code>devpay</code> |
     *                         <code>marketplace</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>ramdisk-id</code> - The RAM disk ID.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>reason</code> - The reason for the current state of the instance (for
     *                     example, shows "User Initiated [date]" when you stop or terminate the instance).
     *                     Similar to the state-reason-code filter.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>requester-id</code> - The ID of the entity that launched the instance on
     *                     your behalf (for example, AWS Management Console, Auto Scaling, and so
     *                     on).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>reservation-id</code> - The ID of the instance's reservation. A
     *                     reservation ID is created any time you launch an instance. A reservation ID has
     *                     a one-to-one relationship with an instance launch request, but can be associated
     *                     with more than one instance if you launch multiple instances using the same
     *                     launch request. For example, if you launch one instance, you get one reservation
     *                     ID. If you launch ten instances using the same launch request, you also get one
     *                     reservation ID.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>root-device-name</code> - The device name of the root device volume (for
     *                     example, <code>/dev/sda1</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>root-device-type</code> - The type of the root device volume
     *                         (<code>ebs</code> | <code>instance-store</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>source-dest-check</code> - Indicates whether the instance performs
     *                     source/destination checking. A value of <code>true</code> means that checking is
     *                     enabled, and <code>false</code> means that checking is disabled. The value must
     *                     be <code>false</code> for the instance to perform network address translation
     *                     (NAT) in your VPC. </p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>spot-instance-request-id</code> - The ID of the Spot Instance
     *                     request.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>state-reason-code</code> - The reason code for the state change.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>state-reason-message</code> - A message that describes the state
     *                     change.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>subnet-id</code> - The ID of the subnet for the instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources that have a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>tenancy</code> - The tenancy of an instance (<code>dedicated</code> |
     *                         <code>default</code> | <code>host</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>virtualization-type</code> - The virtualization type of the instance
     *                         (<code>paravirtual</code> | <code>hvm</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>vpc-id</code> - The ID of the VPC that the instance is running in.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The instance IDs.</p>
     *         <p>Default: Describes all your instances.</p>
     */
    InstanceIds?: string[];
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining
     *             results, make another call with the returned <code>NextToken</code> value. This value
     *             can be between 5 and 1000. You cannot specify this parameter and the instance IDs
     *             parameter in the same call.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to request the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeInstancesRequest {
    const filterSensitiveLog: (obj: DescribeInstancesRequest) => any;
    const isa: (o: any) => o is DescribeInstancesRequest;
}
export interface DescribeInstancesResult {
    __type?: "DescribeInstancesResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code>
     *             when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the reservations.</p>
     */
    Reservations?: Reservation[];
}
export declare namespace DescribeInstancesResult {
    const filterSensitiveLog: (obj: DescribeInstancesResult) => any;
    const isa: (o: any) => o is DescribeInstancesResult;
}
export interface DescribeInstanceStatusRequest {
    __type?: "DescribeInstanceStatusRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>availability-zone</code> - The Availability Zone of the instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>event.code</code> - The code for the scheduled event
     *                         (<code>instance-reboot</code> | <code>system-reboot</code> |
     *                         <code>system-maintenance</code> | <code>instance-retirement</code> |
     *                         <code>instance-stop</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>event.description</code> - A description of the event.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>event.instance-event-id</code> - The ID of the event whose date and time
     *                     you are modifying.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>event.not-after</code> - The latest end time for the scheduled event
     *                     (for example, <code>2014-09-15T17:15:20.000Z</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>event.not-before</code> - The earliest start time for the scheduled
     *                     event (for example, <code>2014-09-15T17:15:20.000Z</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>event.not-before-deadline</code> - The deadline for starting the event
     *                     (for example, <code>2014-09-15T17:15:20.000Z</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-state-code</code> - The code for the instance state, as a
     *                     16-bit unsigned integer. The high byte is used for internal purposes and should
     *                     be ignored. The low byte is set based on the state represented. The valid values
     *                     are 0 (pending), 16 (running), 32 (shutting-down), 48 (terminated), 64
     *                     (stopping), and 80 (stopped).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-state-name</code> - The state of the instance
     *                         (<code>pending</code> | <code>running</code> | <code>shutting-down</code> |
     *                         <code>terminated</code> | <code>stopping</code> |
     *                     <code>stopped</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-status.reachability</code> - Filters on instance status where
     *                     the name is <code>reachability</code> (<code>passed</code> | <code>failed</code>
     *                     | <code>initializing</code> | <code>insufficient-data</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-status.status</code> - The status of the instance
     *                         (<code>ok</code> | <code>impaired</code> | <code>initializing</code> |
     *                         <code>insufficient-data</code> | <code>not-applicable</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>system-status.reachability</code> - Filters on system status where the
     *                     name is <code>reachability</code> (<code>passed</code> | <code>failed</code> |
     *                         <code>initializing</code> | <code>insufficient-data</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>system-status.status</code> - The system status of the instance
     *                         (<code>ok</code> | <code>impaired</code> | <code>initializing</code> |
     *                         <code>insufficient-data</code> | <code>not-applicable</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>When <code>true</code>, includes the health status for all instances. When
     *                 <code>false</code>, includes the health status for running instances only.</p>
     *         <p>Default: <code>false</code>
     *         </p>
     */
    IncludeAllInstances?: boolean;
    /**
     * <p>The instance IDs.</p>
     *         <p>Default: Describes all your instances.</p>
     *         <p>Constraints: Maximum 100 explicitly specified instance IDs.</p>
     */
    InstanceIds?: string[];
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining
     *             results, make another call with the returned <code>NextToken</code> value. This value
     *             can be between 5 and 1000. You cannot specify this parameter and the instance IDs
     *             parameter in the same call.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeInstanceStatusRequest {
    const filterSensitiveLog: (obj: DescribeInstanceStatusRequest) => any;
    const isa: (o: any) => o is DescribeInstanceStatusRequest;
}
export interface DescribeInstanceStatusResult {
    __type?: "DescribeInstanceStatusResult";
    /**
     * <p>Information about the status of the instances.</p>
     */
    InstanceStatuses?: InstanceStatus[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code>
     *             when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeInstanceStatusResult {
    const filterSensitiveLog: (obj: DescribeInstanceStatusResult) => any;
    const isa: (o: any) => o is DescribeInstanceStatusResult;
}
export interface DescribeInstanceTypeOfferingsRequest {
    __type?: "DescribeInstanceTypeOfferingsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *          and provides an error response. If you have the required permissions, the error response is
     *          <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. Filter names and values are case-sensitive.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>location</code> - This depends on the location type. For example, if the location type is
     *       <code>region</code> (default), the location is the Region code (for example, <code>us-east-2</code>.)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-type</code> - The instance type.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The location type.</p>
     */
    LocationType?: LocationType | string;
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results
     *          can be seen by sending another request with the next token value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeInstanceTypeOfferingsRequest {
    const filterSensitiveLog: (obj: DescribeInstanceTypeOfferingsRequest) => any;
    const isa: (o: any) => o is DescribeInstanceTypeOfferingsRequest;
}
export interface DescribeInstanceTypeOfferingsResult {
    __type?: "DescribeInstanceTypeOfferingsResult";
    /**
     * <p>The instance types offered.</p>
     */
    InstanceTypeOfferings?: InstanceTypeOffering[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there
     *          are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeInstanceTypeOfferingsResult {
    const filterSensitiveLog: (obj: DescribeInstanceTypeOfferingsResult) => any;
    const isa: (o: any) => o is DescribeInstanceTypeOfferingsResult;
}
export interface DescribeInstanceTypesRequest {
    __type?: "DescribeInstanceTypesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *          and provides an error response. If you have the required permissions, the error response is
     *          <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. Filter names and values are case-sensitive.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>auto-recovery-supported</code> - Indicates whether auto recovery is supported. (<code>true</code> | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>bare-metal</code> - Indicates whether it is a bare metal instance type. (<code>true</code> | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>burstable-performance-supported</code> - Indicates whether it is a burstable performance instance
     *      type. (<code>true</code> | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>current-generation</code> - Indicates whether this instance type is the latest generation instance type
     *      of an instance family. (<code>true</code> | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>ebs-info.ebs-optimized-support</code> - Indicates whether the instance type is
     *      EBS-optimized. (<code>true</code> | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>ebs-info.encryption-support</code> - Indicates whether EBS encryption is supported. (<code>true</code> | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>free-tier-eligible</code> - Indicates whether the instance type is eligible to use in the free
     *      tier. (<code>true</code> | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>hibernation-supported</code> - Indicates whether On-Demand hibernation is supported. (<code>true</code> | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>hypervisor</code> - The hypervisor used. (<code>nitro</code> | <code>xen</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-storage-info.disk.count</code> - The number of local disks.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-storage-info.disk.size-in-gb</code> - The storage size of each instance storage disk, in
     *      GB.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-storage-info.disk.type</code> - The storage technology for the local instance storage disks.
     *       (<code>hdd</code> | <code>ssd</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-storage-info.total-size-in-gb</code> - The total amount of storage available from all local
     *      instance storage, in GB.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-storage-supported</code> - Indicates whether the instance type has local instance
     *      storage. (<code>true</code> | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>memory-info.size-in-mib</code> - The memory size.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-info.ena-support</code> - Indicates whether Elastic Network Adapter (ENA) is supported or
     *      required. (<code>required</code> | <code>supported</code> | <code>unsupported</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-info.ipv4-addresses-per-interface</code> - The maximum number of private IPv4 addresses per
     *      network interface.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-info.ipv6-addresses-per-interface</code> - The maximum number of private IPv6 addresses per
     *      network interface.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-info.ipv6-supported</code> - Indicates whether the instance type supports IPv6. (<code>true</code> | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-info.maximum-network-interfaces</code> - The maximum number of network interfaces per
     *      instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-info.network-performance</code> - Describes the network performance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>processor-info.sustained-clock-speed-in-ghz</code> - The CPU clock speed, in GHz.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>vcpu-info.default-cores</code> - The default number of cores for the instance type.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>vcpu-info.default-threads-per-core</code> - The default number of threads per core for the instance
     *      type.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>vcpu-info.default-vcpus</code> - The default number of vCPUs for the instance type.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The instance types. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the <i>Amazon Elastic Compute
     *     Cloud User Guide</i>.</p>
     */
    InstanceTypes?: (_InstanceType | string)[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results
     *          can be seen by sending another request with the next token value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeInstanceTypesRequest {
    const filterSensitiveLog: (obj: DescribeInstanceTypesRequest) => any;
    const isa: (o: any) => o is DescribeInstanceTypesRequest;
}
export interface DescribeInstanceTypesResult {
    __type?: "DescribeInstanceTypesResult";
    /**
     * <p>The instance type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the <i>Amazon Elastic Compute
     *     Cloud User Guide</i>.</p>
     */
    InstanceTypes?: InstanceTypeInfo[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there
     *          are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeInstanceTypesResult {
    const filterSensitiveLog: (obj: DescribeInstanceTypesResult) => any;
    const isa: (o: any) => o is DescribeInstanceTypesResult;
}
export interface DescribeInternetGatewaysRequest {
    __type?: "DescribeInternetGatewaysRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>attachment.state</code> - The current state of the attachment between the gateway
     *                     and the VPC (<code>available</code>). Present only if a VPC is attached.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>attachment.vpc-id</code> - The ID of an attached VPC.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>internet-gateway-id</code> - The ID of the Internet gateway.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>owner-id</code> - The ID of the AWS account that owns the internet gateway.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>One or more internet gateway IDs.</p>
     *         <p>Default: Describes all your internet gateways.</p>
     */
    InternetGatewayIds?: string[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeInternetGatewaysRequest {
    const filterSensitiveLog: (obj: DescribeInternetGatewaysRequest) => any;
    const isa: (o: any) => o is DescribeInternetGatewaysRequest;
}
export interface DescribeInternetGatewaysResult {
    __type?: "DescribeInternetGatewaysResult";
    /**
     * <p>Information about one or more internet gateways.</p>
     */
    InternetGateways?: InternetGateway[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeInternetGatewaysResult {
    const filterSensitiveLog: (obj: DescribeInternetGatewaysResult) => any;
    const isa: (o: any) => o is DescribeInternetGatewaysResult;
}
export interface DescribeIpv6PoolsRequest {
    __type?: "DescribeIpv6PoolsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The IDs of the IPv6 address pools.</p>
     */
    PoolIds?: string[];
}
export declare namespace DescribeIpv6PoolsRequest {
    const filterSensitiveLog: (obj: DescribeIpv6PoolsRequest) => any;
    const isa: (o: any) => o is DescribeIpv6PoolsRequest;
}
export interface DescribeIpv6PoolsResult {
    __type?: "DescribeIpv6PoolsResult";
    /**
     * <p>Information about the IPv6 address pools.</p>
     */
    Ipv6Pools?: Ipv6Pool[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeIpv6PoolsResult {
    const filterSensitiveLog: (obj: DescribeIpv6PoolsResult) => any;
    const isa: (o: any) => o is DescribeIpv6PoolsResult;
}
export interface DescribeKeyPairsRequest {
    __type?: "DescribeKeyPairsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>fingerprint</code> - The fingerprint of the key pair.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>key-name</code> - The name of the key pair.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The key pair names.</p>
     *          <p>Default: Describes all your key pairs.</p>
     */
    KeyNames?: string[];
    /**
     * <p>The IDs of the key pairs.</p>
     */
    KeyPairIds?: string[];
}
export declare namespace DescribeKeyPairsRequest {
    const filterSensitiveLog: (obj: DescribeKeyPairsRequest) => any;
    const isa: (o: any) => o is DescribeKeyPairsRequest;
}
export interface DescribeKeyPairsResult {
    __type?: "DescribeKeyPairsResult";
    /**
     * <p>Information about the key pairs.</p>
     */
    KeyPairs?: KeyPairInfo[];
}
export declare namespace DescribeKeyPairsResult {
    const filterSensitiveLog: (obj: DescribeKeyPairsResult) => any;
    const isa: (o: any) => o is DescribeKeyPairsResult;
}
export interface DescribeLaunchTemplatesRequest {
    __type?: "DescribeLaunchTemplatesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually
     *             making the request, and provides an error response. If you have the required
     *             permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is
     *                 <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>create-time</code> - The time the launch template was created.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>launch-template-name</code> - The name of the launch template.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>One or more launch template IDs.</p>
     */
    LaunchTemplateIds?: string[];
    /**
     * <p>One or more launch template names.</p>
     */
    LaunchTemplateNames?: string[];
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining
     *             results, make another call with the returned <code>NextToken</code> value. This value
     *             can be between 1 and 200.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to request the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLaunchTemplatesRequest {
    const filterSensitiveLog: (obj: DescribeLaunchTemplatesRequest) => any;
    const isa: (o: any) => o is DescribeLaunchTemplatesRequest;
}
export interface DescribeLaunchTemplatesResult {
    __type?: "DescribeLaunchTemplatesResult";
    /**
     * <p>Information about the launch templates.</p>
     */
    LaunchTemplates?: LaunchTemplate[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is
     *                 <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLaunchTemplatesResult {
    const filterSensitiveLog: (obj: DescribeLaunchTemplatesResult) => any;
    const isa: (o: any) => o is DescribeLaunchTemplatesResult;
}
export interface DescribeLaunchTemplateVersionsRequest {
    __type?: "DescribeLaunchTemplateVersionsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually
     *             making the request, and provides an error response. If you have the required
     *             permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is
     *                 <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                <p>
     *                   <code>create-time</code> - The time the launch template version was created.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ebs-optimized</code> - A boolean that indicates whether the instance
     *                     is optimized for Amazon EBS I/O.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>iam-instance-profile</code> - The ARN of the IAM instance
     *                     profile.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>image-id</code> - The ID of the AMI.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>instance-type</code> - The instance type.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>is-default-version</code> - A boolean that indicates whether the
     *                     launch template version is the default version.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>kernel-id</code> - The kernel ID.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ram-disk-id</code> - The RAM disk ID.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The ID of the launch template. You must specify either the launch template ID or
     *             launch template name in the request.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template. You must specify either the launch template ID or
     *             launch template name in the request.</p>
     */
    LaunchTemplateName?: string;
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining
     *             results, make another call with the returned <code>NextToken</code> value. This value
     *             can be between 1 and 200.</p>
     */
    MaxResults?: number;
    /**
     * <p>The version number up to which to describe launch template versions.</p>
     */
    MaxVersion?: string;
    /**
     * <p>The version number after which to describe launch template versions.</p>
     */
    MinVersion?: string;
    /**
     * <p>The token to request the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>One or more versions of the launch template.</p>
     */
    Versions?: string[];
}
export declare namespace DescribeLaunchTemplateVersionsRequest {
    const filterSensitiveLog: (obj: DescribeLaunchTemplateVersionsRequest) => any;
    const isa: (o: any) => o is DescribeLaunchTemplateVersionsRequest;
}
export interface DescribeLaunchTemplateVersionsResult {
    __type?: "DescribeLaunchTemplateVersionsResult";
    /**
     * <p>Information about the launch template versions.</p>
     */
    LaunchTemplateVersions?: LaunchTemplateVersion[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is
     *                 <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLaunchTemplateVersionsResult {
    const filterSensitiveLog: (obj: DescribeLaunchTemplateVersionsResult) => any;
    const isa: (o: any) => o is DescribeLaunchTemplateVersionsResult;
}
export interface DescribeLocalGatewayRouteTablesRequest {
    __type?: "DescribeLocalGatewayRouteTablesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     */
    Filters?: Filter[];
    /**
     * <p>The IDs of the local gateway route tables.</p>
     */
    LocalGatewayRouteTableIds?: string[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewayRouteTablesRequest {
    const filterSensitiveLog: (obj: DescribeLocalGatewayRouteTablesRequest) => any;
    const isa: (o: any) => o is DescribeLocalGatewayRouteTablesRequest;
}
export interface DescribeLocalGatewayRouteTablesResult {
    __type?: "DescribeLocalGatewayRouteTablesResult";
    /**
     * <p>Information about the local gateway route tables.</p>
     */
    LocalGatewayRouteTables?: LocalGatewayRouteTable[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewayRouteTablesResult {
    const filterSensitiveLog: (obj: DescribeLocalGatewayRouteTablesResult) => any;
    const isa: (o: any) => o is DescribeLocalGatewayRouteTablesResult;
}
export interface DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsRequest {
    __type?: "DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     */
    Filters?: Filter[];
    /**
     * <p>The IDs of the associations.</p>
     */
    LocalGatewayRouteTableVirtualInterfaceGroupAssociationIds?: string[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsRequest {
    const filterSensitiveLog: (obj: DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsRequest) => any;
    const isa: (o: any) => o is DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsRequest;
}
export interface DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsResult {
    __type?: "DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsResult";
    /**
     * <p>Information about the associations.</p>
     */
    LocalGatewayRouteTableVirtualInterfaceGroupAssociations?: LocalGatewayRouteTableVirtualInterfaceGroupAssociation[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsResult {
    const filterSensitiveLog: (obj: DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsResult) => any;
    const isa: (o: any) => o is DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsResult;
}
export interface DescribeLocalGatewayRouteTableVpcAssociationsRequest {
    __type?: "DescribeLocalGatewayRouteTableVpcAssociationsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     */
    Filters?: Filter[];
    /**
     * <p>The IDs of the associations.</p>
     */
    LocalGatewayRouteTableVpcAssociationIds?: string[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewayRouteTableVpcAssociationsRequest {
    const filterSensitiveLog: (obj: DescribeLocalGatewayRouteTableVpcAssociationsRequest) => any;
    const isa: (o: any) => o is DescribeLocalGatewayRouteTableVpcAssociationsRequest;
}
export interface DescribeLocalGatewayRouteTableVpcAssociationsResult {
    __type?: "DescribeLocalGatewayRouteTableVpcAssociationsResult";
    /**
     * <p>Information about the associations.</p>
     */
    LocalGatewayRouteTableVpcAssociations?: LocalGatewayRouteTableVpcAssociation[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewayRouteTableVpcAssociationsResult {
    const filterSensitiveLog: (obj: DescribeLocalGatewayRouteTableVpcAssociationsResult) => any;
    const isa: (o: any) => o is DescribeLocalGatewayRouteTableVpcAssociationsResult;
}
export interface DescribeLocalGatewaysRequest {
    __type?: "DescribeLocalGatewaysRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     */
    Filters?: Filter[];
    /**
     * <p>The IDs of the local gateways.</p>
     */
    LocalGatewayIds?: string[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewaysRequest {
    const filterSensitiveLog: (obj: DescribeLocalGatewaysRequest) => any;
    const isa: (o: any) => o is DescribeLocalGatewaysRequest;
}
export interface DescribeLocalGatewaysResult {
    __type?: "DescribeLocalGatewaysResult";
    /**
     * <p>Information about the local gateways.</p>
     */
    LocalGateways?: LocalGateway[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewaysResult {
    const filterSensitiveLog: (obj: DescribeLocalGatewaysResult) => any;
    const isa: (o: any) => o is DescribeLocalGatewaysResult;
}
export interface DescribeLocalGatewayVirtualInterfaceGroupsRequest {
    __type?: "DescribeLocalGatewayVirtualInterfaceGroupsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     */
    Filters?: Filter[];
    /**
     * <p>The IDs of the virtual interface groups.</p>
     */
    LocalGatewayVirtualInterfaceGroupIds?: string[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewayVirtualInterfaceGroupsRequest {
    const filterSensitiveLog: (obj: DescribeLocalGatewayVirtualInterfaceGroupsRequest) => any;
    const isa: (o: any) => o is DescribeLocalGatewayVirtualInterfaceGroupsRequest;
}
export interface DescribeLocalGatewayVirtualInterfaceGroupsResult {
    __type?: "DescribeLocalGatewayVirtualInterfaceGroupsResult";
    /**
     * <p>The virtual interface groups.</p>
     */
    LocalGatewayVirtualInterfaceGroups?: LocalGatewayVirtualInterfaceGroup[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewayVirtualInterfaceGroupsResult {
    const filterSensitiveLog: (obj: DescribeLocalGatewayVirtualInterfaceGroupsResult) => any;
    const isa: (o: any) => o is DescribeLocalGatewayVirtualInterfaceGroupsResult;
}
export interface DescribeLocalGatewayVirtualInterfacesRequest {
    __type?: "DescribeLocalGatewayVirtualInterfacesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     */
    Filters?: Filter[];
    /**
     * <p>The IDs of the virtual interfaces.</p>
     */
    LocalGatewayVirtualInterfaceIds?: string[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewayVirtualInterfacesRequest {
    const filterSensitiveLog: (obj: DescribeLocalGatewayVirtualInterfacesRequest) => any;
    const isa: (o: any) => o is DescribeLocalGatewayVirtualInterfacesRequest;
}
export interface DescribeLocalGatewayVirtualInterfacesResult {
    __type?: "DescribeLocalGatewayVirtualInterfacesResult";
    /**
     * <p>Information about the virtual interfaces.</p>
     */
    LocalGatewayVirtualInterfaces?: LocalGatewayVirtualInterface[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeLocalGatewayVirtualInterfacesResult {
    const filterSensitiveLog: (obj: DescribeLocalGatewayVirtualInterfacesResult) => any;
    const isa: (o: any) => o is DescribeLocalGatewayVirtualInterfacesResult;
}
export interface DescribeMovingAddressesRequest {
    __type?: "DescribeMovingAddressesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>moving-status</code> - The status of the Elastic IP address
     *           (<code>MovingToVpc</code> | <code>RestoringToClassic</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining
     *       results of the initial request can be seen by sending another request with the returned
     *       <code>NextToken</code> value. This value can be between 5 and 1000; if
     *       <code>MaxResults</code> is given a value outside of this range, an error is returned.</p>
     *          <p>Default: If no value is provided, the default is 1000.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>One or more Elastic IP addresses.</p>
     */
    PublicIps?: string[];
}
export declare namespace DescribeMovingAddressesRequest {
    const filterSensitiveLog: (obj: DescribeMovingAddressesRequest) => any;
    const isa: (o: any) => o is DescribeMovingAddressesRequest;
}
export interface DescribeMovingAddressesResult {
    __type?: "DescribeMovingAddressesResult";
    /**
     * <p>The status for each Elastic IP address.</p>
     */
    MovingAddressStatuses?: MovingAddressStatus[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeMovingAddressesResult {
    const filterSensitiveLog: (obj: DescribeMovingAddressesResult) => any;
    const isa: (o: any) => o is DescribeMovingAddressesResult;
}
export interface DescribeNatGatewaysRequest {
    __type?: "DescribeNatGatewaysRequest";
    /**
     * <p>One or more filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>nat-gateway-id</code> - The ID of the NAT gateway.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the NAT gateway (<code>pending</code> |
     *               <code>failed</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>subnet-id</code> - The ID of the subnet in which the NAT gateway resides.</p>
     *             </li>
     *             <li>
     * 		             <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     * 		          </li>
     *             <li>
     * 			            <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     * 		          </li>
     *             <li>
     *                <p>
     *                   <code>vpc-id</code> - The ID of the VPC in which the NAT gateway resides.</p>
     *             </li>
     *          </ul>
     */
    Filter?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>One or more NAT gateway IDs.</p>
     */
    NatGatewayIds?: string[];
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeNatGatewaysRequest {
    const filterSensitiveLog: (obj: DescribeNatGatewaysRequest) => any;
    const isa: (o: any) => o is DescribeNatGatewaysRequest;
}
export interface DescribeNatGatewaysResult {
    __type?: "DescribeNatGatewaysResult";
    /**
     * <p>Information about the NAT gateways.</p>
     */
    NatGateways?: NatGateway[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeNatGatewaysResult {
    const filterSensitiveLog: (obj: DescribeNatGatewaysResult) => any;
    const isa: (o: any) => o is DescribeNatGatewaysResult;
}
export interface DescribeNetworkAclsRequest {
    __type?: "DescribeNetworkAclsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     * 		       <ul>
     *             <li>
     * 		             <p>
     *                   <code>association.association-id</code> - The ID of an association ID for the ACL.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>association.network-acl-id</code> - The ID of the network ACL involved in the association.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>association.subnet-id</code> - The ID of the subnet involved in the association.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>default</code> - Indicates whether the ACL is the default network ACL for the VPC.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>entry.cidr</code> - The IPv4 CIDR range specified in the entry.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>entry.icmp.code</code> - The ICMP code specified in the entry, if any.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>entry.icmp.type</code> - The ICMP type specified in the entry, if any.</p>
     * 		          </li>
     *             <li>
     *                   <p>
     *                   <code>entry.ipv6-cidr</code> - The IPv6 CIDR range specified in the entry.</p>
     *                </li>
     *             <li>
     * 		             <p>
     *                   <code>entry.port-range.from</code> - The start of the port range specified in the entry. </p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>entry.port-range.to</code> - The end of the port range specified in the entry. </p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>entry.protocol</code> - The protocol specified in the entry (<code>tcp</code> | <code>udp</code> | <code>icmp</code> or a protocol number).</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>entry.rule-action</code> - Allows or denies the matching traffic (<code>allow</code> | <code>deny</code>).</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>entry.rule-number</code> - The number of an entry (in other words, rule) in
     *                     the set of ACL entries.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>network-acl-id</code> - The ID of the network ACL.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>owner-id</code> - The ID of the AWS account that owns the network ACL.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>vpc-id</code> - The ID of the VPC for the network ACL.</p>
     * 		          </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>One or more network ACL IDs.</p>
     * 		       <p>Default: Describes all your network ACLs.</p>
     */
    NetworkAclIds?: string[];
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeNetworkAclsRequest {
    const filterSensitiveLog: (obj: DescribeNetworkAclsRequest) => any;
    const isa: (o: any) => o is DescribeNetworkAclsRequest;
}
export interface DescribeNetworkAclsResult {
    __type?: "DescribeNetworkAclsResult";
    /**
     * <p>Information about one or more network ACLs.</p>
     */
    NetworkAcls?: NetworkAcl[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeNetworkAclsResult {
    const filterSensitiveLog: (obj: DescribeNetworkAclsResult) => any;
    const isa: (o: any) => o is DescribeNetworkAclsResult;
}
/**
 * <p>Contains the parameters for DescribeNetworkInterfaceAttribute.</p>
 */
export interface DescribeNetworkInterfaceAttributeRequest {
    __type?: "DescribeNetworkInterfaceAttributeRequest";
    /**
     * <p>The attribute of the network interface. This parameter is required.</p>
     */
    Attribute?: NetworkInterfaceAttribute | string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId: string | undefined;
}
export declare namespace DescribeNetworkInterfaceAttributeRequest {
    const filterSensitiveLog: (obj: DescribeNetworkInterfaceAttributeRequest) => any;
    const isa: (o: any) => o is DescribeNetworkInterfaceAttributeRequest;
}
/**
 * <p>Contains the output of DescribeNetworkInterfaceAttribute.</p>
 */
export interface DescribeNetworkInterfaceAttributeResult {
    __type?: "DescribeNetworkInterfaceAttributeResult";
    /**
     * <p>The attachment (if any) of the network interface.</p>
     */
    Attachment?: NetworkInterfaceAttachment;
    /**
     * <p>The description of the network interface.</p>
     */
    Description?: AttributeValue;
    /**
     * <p>The security groups associated with the network interface.</p>
     */
    Groups?: GroupIdentifier[];
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>Indicates whether source/destination checking is enabled.</p>
     */
    SourceDestCheck?: AttributeBooleanValue;
}
export declare namespace DescribeNetworkInterfaceAttributeResult {
    const filterSensitiveLog: (obj: DescribeNetworkInterfaceAttributeResult) => any;
    const isa: (o: any) => o is DescribeNetworkInterfaceAttributeResult;
}
/**
 * <p>Contains the parameters for DescribeNetworkInterfacePermissions.</p>
 */
export interface DescribeNetworkInterfacePermissionsRequest {
    __type?: "DescribeNetworkInterfacePermissionsRequest";
    /**
     * <p>One or more filters.</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>network-interface-permission.network-interface-permission-id</code> - The ID of the
     * 				permission.</p>
     *             </li>
     *             <li>
     * 				           <p>
     *                   <code>network-interface-permission.network-interface-id</code> - The ID of
     * 					the network interface.</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>network-interface-permission.aws-account-id</code> - The AWS account
     * 					ID.</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>network-interface-permission.aws-service</code> - The AWS
     * 					service.</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>network-interface-permission.permission</code> - The type of
     * 					permission (<code>INSTANCE-ATTACH</code> |
     * 					<code>EIP-ASSOCIATE</code>).</p>
     * 			         </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining results,
     * 			make another call with the returned <code>NextToken</code> value. If this parameter is not specified, up to 50 results are returned by default.</p>
     */
    MaxResults?: number;
    /**
     * <p>One or more network interface permission IDs.</p>
     */
    NetworkInterfacePermissionIds?: string[];
    /**
     * <p>The token to request the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeNetworkInterfacePermissionsRequest {
    const filterSensitiveLog: (obj: DescribeNetworkInterfacePermissionsRequest) => any;
    const isa: (o: any) => o is DescribeNetworkInterfacePermissionsRequest;
}
/**
 * <p>Contains the output for DescribeNetworkInterfacePermissions.</p>
 */
export interface DescribeNetworkInterfacePermissionsResult {
    __type?: "DescribeNetworkInterfacePermissionsResult";
    /**
     * <p>The network interface permissions.</p>
     */
    NetworkInterfacePermissions?: NetworkInterfacePermission[];
    /**
     * <p>The token to use to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeNetworkInterfacePermissionsResult {
    const filterSensitiveLog: (obj: DescribeNetworkInterfacePermissionsResult) => any;
    const isa: (o: any) => o is DescribeNetworkInterfacePermissionsResult;
}
/**
 * <p>Contains the parameters for DescribeNetworkInterfaces.</p>
 */
export interface DescribeNetworkInterfacesRequest {
    __type?: "DescribeNetworkInterfacesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     * 		       <ul>
     *             <li>
     * 		             <p>
     *                   <code>addresses.private-ip-address</code> - The private IPv4 addresses
     *                     associated with the network interface.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>addresses.primary</code> - Whether the private IPv4 address is the primary
     *                     IP address associated with the network interface. </p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>addresses.association.public-ip</code> - The association ID returned when
     *                     the network interface was associated with the Elastic IP address
     *                     (IPv4).</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>addresses.association.owner-id</code> - The owner ID of the addresses associated with the network interface.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>association.association-id</code> - The association ID returned when the
     *                     network interface was associated with an IPv4 address.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>association.allocation-id</code> - The allocation ID returned when you
     *                     allocated the Elastic IP address (IPv4) for your network interface.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>association.ip-owner-id</code> - The owner of the Elastic IP address
     *                     (IPv4) associated with the network interface.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>association.public-ip</code> - The address of the Elastic IP address
     *                     (IPv4) bound to the network interface.</p>
     * 		          </li>
     *             <li>
     * 		 		          <p>
     *                   <code>association.public-dns-name</code> - The public DNS name for the network
     *                     interface (IPv4).</p>
     * 		 	        </li>
     *             <li>
     * 		             <p>
     *                   <code>attachment.attachment-id</code> - The ID of the interface attachment.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>attachment.attach-time</code> - The time that the network interface was attached to an instance.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>attachment.delete-on-termination</code> - Indicates whether the attachment is deleted when an instance is terminated.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>attachment.device-index</code> - The device index to which the network interface is attached.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>attachment.instance-id</code> - The ID of the instance to which the network interface is attached.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>attachment.instance-owner-id</code> - The owner ID of the instance to which the network interface is attached.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>attachment.nat-gateway-id</code> - The ID of the NAT gateway to which the network interface is attached.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>attachment.status</code> - The status of the attachment (<code>attaching</code> | <code>attached</code> | <code>detaching</code> | <code>detached</code>).</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>availability-zone</code> - The Availability Zone of the network interface.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>description</code> - The description of the network interface.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>group-id</code> - The ID of a security group associated with the network interface.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>group-name</code> - The name of a security group associated with the network interface.</p>
     * 		          </li>
     *             <li>
     *                   <p>
     *                   <code>ipv6-addresses.ipv6-address</code> - An IPv6 address associated with
     *                     the network interface.</p>
     *                </li>
     *             <li>
     * 		             <p>
     *                   <code>mac-address</code> - The MAC address of the network interface.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>network-interface-id</code> - The ID of the network interface.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>owner-id</code> - The AWS account ID of the network interface owner.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>private-ip-address</code> - The private IPv4 address or addresses of the
     *                     network interface.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>private-dns-name</code> - The private DNS name of the network interface (IPv4).</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>requester-id</code> - The ID of the entity that launched the instance on your behalf (for example, AWS Management Console, Auto Scaling, and so on).</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>requester-managed</code> - Indicates whether the network interface is being managed by an AWS service (for example, AWS Management Console, Auto Scaling, and so on).</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>source-dest-check</code> - Indicates whether the network interface performs source/destination checking.
     * 		            A value of <code>true</code> means checking is enabled, and <code>false</code> means checking is disabled.
     * 		            The value must be <code>false</code> for the network interface to perform network address translation (NAT) in your VPC. </p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>status</code> - The status of the network interface. If the network interface is not attached to an instance, the status is <code>available</code>;
     * 		            if a network interface is attached to an instance the status is <code>in-use</code>.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>subnet-id</code> - The ID of the subnet for the network interface.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>vpc-id</code> - The ID of the VPC for the network interface.</p>
     * 		          </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of items to return for this request. The request returns a token that you
     *             can specify in a subsequent call to get the next set of results. You cannot specify this
     *             parameter and the network interface IDs parameter in the same request.</p>
     */
    MaxResults?: number;
    /**
     * <p>One or more network interface IDs.</p>
     * 		       <p>Default: Describes all your network interfaces.</p>
     */
    NetworkInterfaceIds?: string[];
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeNetworkInterfacesRequest {
    const filterSensitiveLog: (obj: DescribeNetworkInterfacesRequest) => any;
    const isa: (o: any) => o is DescribeNetworkInterfacesRequest;
}
/**
 * <p>Contains the output of DescribeNetworkInterfaces.</p>
 */
export interface DescribeNetworkInterfacesResult {
    __type?: "DescribeNetworkInterfacesResult";
    /**
     * <p>Information about one or more network interfaces.</p>
     */
    NetworkInterfaces?: NetworkInterface[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeNetworkInterfacesResult {
    const filterSensitiveLog: (obj: DescribeNetworkInterfacesResult) => any;
    const isa: (o: any) => o is DescribeNetworkInterfacesResult;
}
export interface DescribePlacementGroupsRequest {
    __type?: "DescribePlacementGroupsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                     <code>group-name</code> - The name of the placement group.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>state</code> - The state of the placement group (<code>pending</code> |
     *                         <code>available</code> | <code>deleting</code> |
     *                     <code>deleted</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>strategy</code> - The strategy of the placement group
     *                         (<code>cluster</code> | <code>spread</code> |
     *                     <code>partition</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The IDs of the placement groups.</p>
     */
    GroupIds?: string[];
    /**
     * <p>The names of the placement groups.</p>
     *         <p>Default: Describes all your placement groups, or only those otherwise
     *             specified.</p>
     */
    GroupNames?: string[];
}
export declare namespace DescribePlacementGroupsRequest {
    const filterSensitiveLog: (obj: DescribePlacementGroupsRequest) => any;
    const isa: (o: any) => o is DescribePlacementGroupsRequest;
}
export interface DescribePlacementGroupsResult {
    __type?: "DescribePlacementGroupsResult";
    /**
     * <p>Information about the placement groups.</p>
     */
    PlacementGroups?: PlacementGroup[];
}
export declare namespace DescribePlacementGroupsResult {
    const filterSensitiveLog: (obj: DescribePlacementGroupsResult) => any;
    const isa: (o: any) => o is DescribePlacementGroupsResult;
}
export interface DescribePrefixListsRequest {
    __type?: "DescribePrefixListsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     * 		       <ul>
     *             <li>
     * 				           <p>
     *                   <code>prefix-list-id</code>: The ID of a prefix list.</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>prefix-list-name</code>: The name of a prefix list.</p>
     * 			         </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>One or more prefix list IDs.</p>
     */
    PrefixListIds?: string[];
}
export declare namespace DescribePrefixListsRequest {
    const filterSensitiveLog: (obj: DescribePrefixListsRequest) => any;
    const isa: (o: any) => o is DescribePrefixListsRequest;
}
export interface DescribePrefixListsResult {
    __type?: "DescribePrefixListsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>All available prefix lists.</p>
     */
    PrefixLists?: PrefixList[];
}
export declare namespace DescribePrefixListsResult {
    const filterSensitiveLog: (obj: DescribePrefixListsResult) => any;
    const isa: (o: any) => o is DescribePrefixListsResult;
}
export interface DescribePrincipalIdFormatRequest {
    __type?: "DescribePrincipalIdFormatRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining
     *             results, make another call with the returned NextToken value. </p>
     */
    MaxResults?: number;
    /**
     * <p>The token to request the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The type of resource: <code>bundle</code> |
     *             <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> |
     *             <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> |
     *             <code>export-task</code> | <code>flow-log</code> | <code>image</code> |
     *             <code>import-task</code> | <code>instance</code> | <code>internet-gateway</code> |
     *             <code>network-acl</code> | <code>network-acl-association</code> |
     *             <code>network-interface</code> | <code>network-interface-attachment</code> |
     *             <code>prefix-list</code> | <code>reservation</code> | <code>route-table</code> |
     *             <code>route-table-association</code> | <code>security-group</code> |
     *             <code>snapshot</code> | <code>subnet</code> |
     *             <code>subnet-cidr-block-association</code> | <code>volume</code> | <code>vpc</code>
     *             | <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> |
     *             <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>
     *          </p>
     */
    Resources?: string[];
}
export declare namespace DescribePrincipalIdFormatRequest {
    const filterSensitiveLog: (obj: DescribePrincipalIdFormatRequest) => any;
    const isa: (o: any) => o is DescribePrincipalIdFormatRequest;
}
export interface DescribePrincipalIdFormatResult {
    __type?: "DescribePrincipalIdFormatResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is null when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the ID format settings for the ARN.</p>
     */
    Principals?: PrincipalIdFormat[];
}
export declare namespace DescribePrincipalIdFormatResult {
    const filterSensitiveLog: (obj: DescribePrincipalIdFormatResult) => any;
    const isa: (o: any) => o is DescribePrincipalIdFormatResult;
}
export interface DescribePublicIpv4PoolsRequest {
    __type?: "DescribePublicIpv4PoolsRequest";
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The IDs of the address pools.</p>
     */
    PoolIds?: string[];
}
export declare namespace DescribePublicIpv4PoolsRequest {
    const filterSensitiveLog: (obj: DescribePublicIpv4PoolsRequest) => any;
    const isa: (o: any) => o is DescribePublicIpv4PoolsRequest;
}
export interface DescribePublicIpv4PoolsResult {
    __type?: "DescribePublicIpv4PoolsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the address pools.</p>
     */
    PublicIpv4Pools?: PublicIpv4Pool[];
}
export declare namespace DescribePublicIpv4PoolsResult {
    const filterSensitiveLog: (obj: DescribePublicIpv4PoolsResult) => any;
    const isa: (o: any) => o is DescribePublicIpv4PoolsResult;
}
export interface DescribeRegionsRequest {
    __type?: "DescribeRegionsRequest";
    /**
     * <p>Indicates whether to display all Regions, including Regions that are disabled for your account.</p>
     */
    AllRegions?: boolean;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>endpoint</code> - The endpoint of the Region (for example, <code>ec2.us-east-1.amazonaws.com</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>opt-in-status</code> - The opt-in status of the Region (<code>opt-in-not-required</code> | <code>opted-in</code> |
     *                  <code>not-opted-in</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>region-name</code> - The name of the Region (for example, <code>us-east-1</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The names of the Regions. You can specify any Regions, whether they are enabled and disabled for your account.</p>
     */
    RegionNames?: string[];
}
export declare namespace DescribeRegionsRequest {
    const filterSensitiveLog: (obj: DescribeRegionsRequest) => any;
    const isa: (o: any) => o is DescribeRegionsRequest;
}
export interface DescribeRegionsResult {
    __type?: "DescribeRegionsResult";
    /**
     * <p>Information about the Regions.</p>
     */
    Regions?: Region[];
}
export declare namespace DescribeRegionsResult {
    const filterSensitiveLog: (obj: DescribeRegionsResult) => any;
    const isa: (o: any) => o is DescribeRegionsResult;
}
/**
 * <p>Contains the parameters for DescribeReservedInstancesListings.</p>
 */
export interface DescribeReservedInstancesListingsRequest {
    __type?: "DescribeReservedInstancesListingsRequest";
    /**
     * <p>One or more filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>reserved-instances-id</code> - The ID of the Reserved Instances.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>reserved-instances-listing-id</code> - The ID of the Reserved Instances listing.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>status</code> - The status of the Reserved Instance listing (<code>pending</code> | <code>active</code> |
     *            <code>cancelled</code> | <code>closed</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>status-message</code> - The reason for the status.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>One or more Reserved Instance IDs.</p>
     */
    ReservedInstancesId?: string;
    /**
     * <p>One or more Reserved Instance listing IDs.</p>
     */
    ReservedInstancesListingId?: string;
}
export declare namespace DescribeReservedInstancesListingsRequest {
    const filterSensitiveLog: (obj: DescribeReservedInstancesListingsRequest) => any;
    const isa: (o: any) => o is DescribeReservedInstancesListingsRequest;
}
/**
 * <p>Contains the output of DescribeReservedInstancesListings.</p>
 */
export interface DescribeReservedInstancesListingsResult {
    __type?: "DescribeReservedInstancesListingsResult";
    /**
     * <p>Information about the Reserved Instance listing.</p>
     */
    ReservedInstancesListings?: ReservedInstancesListing[];
}
export declare namespace DescribeReservedInstancesListingsResult {
    const filterSensitiveLog: (obj: DescribeReservedInstancesListingsResult) => any;
    const isa: (o: any) => o is DescribeReservedInstancesListingsResult;
}
/**
 * <p>Contains the parameters for DescribeReservedInstancesModifications.</p>
 */
export interface DescribeReservedInstancesModificationsRequest {
    __type?: "DescribeReservedInstancesModificationsRequest";
    /**
     * <p>One or more filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>client-token</code> - The idempotency token for the modification request.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>create-date</code> - The time when the modification request was created.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>effective-date</code> - The time when the modification becomes effective.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>modification-result.reserved-instances-id</code> - The ID for the Reserved Instances created as part of the modification request. This ID is only available when the status of the modification is <code>fulfilled</code>.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>modification-result.target-configuration.availability-zone</code> - The Availability Zone for the new Reserved Instances.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>modification-result.target-configuration.instance-count </code> - The number of new Reserved Instances.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>modification-result.target-configuration.instance-type</code> - The instance type of the new Reserved Instances.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>modification-result.target-configuration.platform</code> - The network platform of the new Reserved Instances (<code>EC2-Classic</code> | <code>EC2-VPC</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>reserved-instances-id</code> - The ID of the Reserved Instances modified.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>reserved-instances-modification-id</code> - The ID of the modification request.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>status</code> - The status of the Reserved Instances modification request
     *            (<code>processing</code> | <code>fulfilled</code> | <code>failed</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>status-message</code> - The reason for the status.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>update-date</code> - The time when the modification request was last updated.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>IDs for the submitted modification request.</p>
     */
    ReservedInstancesModificationIds?: string[];
}
export declare namespace DescribeReservedInstancesModificationsRequest {
    const filterSensitiveLog: (obj: DescribeReservedInstancesModificationsRequest) => any;
    const isa: (o: any) => o is DescribeReservedInstancesModificationsRequest;
}
/**
 * <p>Contains the output of DescribeReservedInstancesModifications.</p>
 */
export interface DescribeReservedInstancesModificationsResult {
    __type?: "DescribeReservedInstancesModificationsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when
     * 			there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>The Reserved Instance modification information.</p>
     */
    ReservedInstancesModifications?: ReservedInstancesModification[];
}
export declare namespace DescribeReservedInstancesModificationsResult {
    const filterSensitiveLog: (obj: DescribeReservedInstancesModificationsResult) => any;
    const isa: (o: any) => o is DescribeReservedInstancesModificationsResult;
}
/**
 * <p>Contains the parameters for DescribeReservedInstancesOfferings.</p>
 */
export interface DescribeReservedInstancesOfferingsRequest {
    __type?: "DescribeReservedInstancesOfferingsRequest";
    /**
     * <p>The Availability Zone in which the Reserved Instance can be used.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>availability-zone</code> - The Availability Zone where the Reserved Instance can be
     *           used.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>duration</code> - The duration of the Reserved Instance (for example, one year or
     *           three years), in seconds (<code>31536000</code> | <code>94608000</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>fixed-price</code> - The purchase price of the Reserved Instance (for example,
     *           9800.0).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-type</code> - The instance type that is covered by the
     *           reservation.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>marketplace</code> - Set to <code>true</code> to show only Reserved Instance
     *           Marketplace offerings. When this filter is not used, which is the default behavior, all
     *           offerings from both AWS and the Reserved Instance Marketplace are listed.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>product-description</code> - The Reserved Instance product platform description.
     *           Instances that include <code>(Amazon VPC)</code> in the product platform description will
     *           only be displayed to EC2-Classic account holders and are for use with Amazon VPC.
     *             (<code>Linux/UNIX</code> | <code>Linux/UNIX (Amazon VPC)</code> | <code>SUSE
     *             Linux</code> | <code>SUSE Linux (Amazon VPC)</code> | <code>Red Hat Enterprise
     *             Linux</code> | <code>Red Hat Enterprise Linux (Amazon VPC)</code> | <code>Windows</code>
     *           | <code>Windows (Amazon VPC)</code> | <code>Windows with SQL Server Standard</code> |
     *             <code>Windows with SQL Server Standard (Amazon VPC)</code> | <code>Windows with SQL
     *             Server Web</code> | <code> Windows with SQL Server Web (Amazon VPC)</code> |
     *             <code>Windows with SQL Server Enterprise</code> | <code>Windows with SQL Server
     *             Enterprise (Amazon VPC)</code>) </p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>reserved-instances-offering-id</code> - The Reserved Instances offering
     *           ID.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>scope</code> - The scope of the Reserved Instance (<code>Availability Zone</code> or
     *             <code>Region</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>usage-price</code> - The usage price of the Reserved Instance, per hour (for
     *           example, 0.84).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>Include Reserved Instance Marketplace offerings in the response.</p>
     */
    IncludeMarketplace?: boolean;
    /**
     * <p>The tenancy of the instances covered by the reservation. A Reserved Instance with a tenancy
     *       of <code>dedicated</code> is applied to instances that run in a VPC on single-tenant hardware
     *       (i.e., Dedicated Instances).</p>
     *          <p>
     *             <b>Important:</b> The <code>host</code> value cannot be used with this parameter. Use the <code>default</code> or <code>dedicated</code> values only.</p>
     *          <p>Default: <code>default</code>
     *          </p>
     */
    InstanceTenancy?: Tenancy | string;
    /**
     * <p>The instance type that the reservation will cover (for example, <code>m1.small</code>). For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the
     *       <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The maximum duration (in seconds) to filter when searching for offerings.</p>
     * 		       <p>Default: 94608000 (3 years)</p>
     */
    MaxDuration?: number;
    /**
     * <p>The maximum number of instances to filter when searching for offerings.</p>
     * 		       <p>Default: 20</p>
     */
    MaxInstanceCount?: number;
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining
     * 			results of the initial request can be seen by sending another request with the returned
     * 				<code>NextToken</code> value. The maximum is 100.</p>
     * 		       <p>Default: 100</p>
     */
    MaxResults?: number;
    /**
     * <p>The minimum duration (in seconds) to filter when searching for offerings.</p>
     * 		       <p>Default: 2592000 (1 month)</p>
     */
    MinDuration?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The offering class of the Reserved Instance. Can be <code>standard</code> or <code>convertible</code>.</p>
     */
    OfferingClass?: OfferingClassType | string;
    /**
     * <p>The Reserved Instance offering type. If you are using tools that predate the 2011-11-01 API
     * 			version, you only have access to the <code>Medium Utilization</code> Reserved Instance
     * 			offering type. </p>
     */
    OfferingType?: OfferingTypeValues | string;
    /**
     * <p>The Reserved Instance product platform description. Instances that include <code>(Amazon
     *         VPC)</code> in the description are for use with Amazon VPC.</p>
     */
    ProductDescription?: RIProductDescription | string;
    /**
     * <p>One or more Reserved Instances offering IDs.</p>
     */
    ReservedInstancesOfferingIds?: string[];
}
export declare namespace DescribeReservedInstancesOfferingsRequest {
    const filterSensitiveLog: (obj: DescribeReservedInstancesOfferingsRequest) => any;
    const isa: (o: any) => o is DescribeReservedInstancesOfferingsRequest;
}
/**
 * <p>Contains the output of DescribeReservedInstancesOfferings.</p>
 */
export interface DescribeReservedInstancesOfferingsResult {
    __type?: "DescribeReservedInstancesOfferingsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when
     * 			there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>A list of Reserved Instances offerings.</p>
     */
    ReservedInstancesOfferings?: ReservedInstancesOffering[];
}
export declare namespace DescribeReservedInstancesOfferingsResult {
    const filterSensitiveLog: (obj: DescribeReservedInstancesOfferingsResult) => any;
    const isa: (o: any) => o is DescribeReservedInstancesOfferingsResult;
}
/**
 * <p>Contains the parameters for DescribeReservedInstances.</p>
 */
export interface DescribeReservedInstancesRequest {
    __type?: "DescribeReservedInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>availability-zone</code> - The Availability Zone where the Reserved Instance can be used.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>duration</code> - The duration of the Reserved Instance (one year or three years), in seconds (<code>31536000</code> | <code>94608000</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>end</code> - The time when the Reserved Instance expires (for example, 2015-08-07T11:54:42.000Z).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>fixed-price</code> - The purchase price of the Reserved Instance (for example, 9800.0).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-type</code> - The instance type that is covered by the reservation.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>scope</code> - The scope of the Reserved Instance (<code>Region</code> or <code>Availability Zone</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>product-description</code> - The Reserved Instance product platform description.
     *               Instances that include <code>(Amazon VPC)</code> in the product platform description will
     *               only be displayed to EC2-Classic account holders and are for use with Amazon VPC
     *               (<code>Linux/UNIX</code> | <code>Linux/UNIX (Amazon VPC)</code> | <code>SUSE
     *               Linux</code> | <code>SUSE Linux (Amazon VPC)</code> | <code>Red Hat Enterprise
     *               Linux</code> | <code>Red Hat Enterprise Linux (Amazon VPC)</code> | <code>Windows</code>
     *               | <code>Windows (Amazon VPC)</code> | <code>Windows with SQL Server Standard</code> |
     *               <code>Windows with SQL Server Standard (Amazon VPC)</code> | <code>Windows with SQL
     *               Server Web</code> | <code>Windows with SQL Server Web (Amazon VPC)</code> |
     *               <code>Windows with SQL Server Enterprise</code> | <code>Windows with SQL Server
     *               Enterprise (Amazon VPC)</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>reserved-instances-id</code> - The ID of the Reserved Instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>start</code> - The time at which the Reserved Instance purchase request was placed (for example, 2014-08-07T11:54:42.000Z).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the Reserved Instance (<code>payment-pending</code> | <code>active</code> | <code>payment-failed</code> | <code>retired</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>usage-price</code> - The usage price of the Reserved Instance, per hour (for example, 0.84).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>Describes whether the Reserved Instance is Standard or Convertible.</p>
     */
    OfferingClass?: OfferingClassType | string;
    /**
     * <p>The Reserved Instance offering type. If you are using tools that predate the 2011-11-01 API
     * 			version, you only have access to the <code>Medium Utilization</code> Reserved Instance
     * 			offering type.</p>
     */
    OfferingType?: OfferingTypeValues | string;
    /**
     * <p>One or more Reserved Instance IDs.</p>
     *          <p>Default: Describes all your Reserved Instances, or only those otherwise specified.</p>
     */
    ReservedInstancesIds?: string[];
}
export declare namespace DescribeReservedInstancesRequest {
    const filterSensitiveLog: (obj: DescribeReservedInstancesRequest) => any;
    const isa: (o: any) => o is DescribeReservedInstancesRequest;
}
/**
 * <p>Contains the output for DescribeReservedInstances.</p>
 */
export interface DescribeReservedInstancesResult {
    __type?: "DescribeReservedInstancesResult";
    /**
     * <p>A list of Reserved Instances.</p>
     */
    ReservedInstances?: ReservedInstances[];
}
export declare namespace DescribeReservedInstancesResult {
    const filterSensitiveLog: (obj: DescribeReservedInstancesResult) => any;
    const isa: (o: any) => o is DescribeReservedInstancesResult;
}
export interface DescribeRouteTablesRequest {
    __type?: "DescribeRouteTablesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     * 		       <ul>
     *             <li>
     * 		             <p>
     *                   <code>association.route-table-association-id</code> - The ID of an association
     *                     ID for the route table.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>association.route-table-id</code> - The ID of the route table involved in
     *                     the association.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>association.subnet-id</code> - The ID of the subnet involved in the
     *                     association.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>association.main</code> - Indicates whether the route table is the main
     *                     route table for the VPC (<code>true</code> | <code>false</code>). Route tables
     *                     that do not have an association ID are not returned in the response.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>owner-id</code> - The ID of the AWS account that owns the route table.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>route-table-id</code> - The ID of the route table.</p>
     * 		          </li>
     *             <li>
     * 				           <p>
     *                   <code>route.destination-cidr-block</code> - The IPv4 CIDR range specified in a
     *                     route in the table.</p>
     * 			         </li>
     *             <li>
     *                   <p>
     *                   <code>route.destination-ipv6-cidr-block</code> - The IPv6 CIDR range specified in a route in the route table.</p>
     *                </li>
     *             <li>
     * 				           <p>
     *                   <code>route.destination-prefix-list-id</code> - The ID (prefix) of the AWS
     *                     service specified in a route in the table.</p>
     * 			         </li>
     *             <li>
     *                   <p>
     *                   <code>route.egress-only-internet-gateway-id</code> - The ID of an
     *                     egress-only Internet gateway specified in a route in the route table.</p>
     *                </li>
     *             <li>
     * 		             <p>
     *                   <code>route.gateway-id</code> - The ID of a gateway specified in a route in the table.</p>
     * 		          </li>
     *             <li>
     *                   <p>
     *                   <code>route.instance-id</code> - The ID of an instance specified in a route in the table.</p>
     *                </li>
     *             <li>
     *                   <p>
     *                   <code>route.nat-gateway-id</code> - The ID of a NAT gateway.</p>
     *                </li>
     *             <li>
     * 		             <p>
     *                   <code>route.transit-gateway-id</code> - The ID of a transit gateway.</p>
     * 		          </li>
     *             <li>
     * 		             <p>
     *                   <code>route.origin</code> - Describes how the route was created.
     *                         <code>CreateRouteTable</code> indicates that the route was automatically
     *                     created when the route table was created; <code>CreateRoute</code> indicates
     *                     that the route was manually added to the route table;
     *                         <code>EnableVgwRoutePropagation</code> indicates that the route was
     *                     propagated by route propagation.</p>
     * 		          </li>
     *             <li>
     *                   <p>
     *                   <code>route.state</code> - The state of a route in the route table
     *                         (<code>active</code> | <code>blackhole</code>). The blackhole state
     *                     indicates that the route's target isn't available (for example, the specified
     *                     gateway isn't attached to the VPC, the specified NAT instance has been
     *                     terminated, and so on).</p>
     *                </li>
     *             <li>
     * 		            <p>
     *                   <code>route.vpc-peering-connection-id</code> - The ID of a VPC peering
     * 		                connection specified in a route in the table.</p>
     * 		          </li>
     *             <li>
     * 		            <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     * 		          </li>
     *             <li>
     * 		            <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     * 		          </li>
     *             <li>
     * 		            <p>
     *                   <code>transit-gateway-id</code> - The ID of a transit gateway.</p>
     * 		          </li>
     *             <li>
     * 		            <p>
     *                   <code>vpc-id</code> - The ID of the VPC for the route table.</p>
     * 		          </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>One or more route table IDs.</p>
     * 		       <p>Default: Describes all your route tables.</p>
     */
    RouteTableIds?: string[];
}
export declare namespace DescribeRouteTablesRequest {
    const filterSensitiveLog: (obj: DescribeRouteTablesRequest) => any;
    const isa: (o: any) => o is DescribeRouteTablesRequest;
}
/**
 * <p>Contains the output of DescribeRouteTables.</p>
 */
export interface DescribeRouteTablesResult {
    __type?: "DescribeRouteTablesResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about one or more route tables.</p>
     */
    RouteTables?: RouteTable[];
}
export declare namespace DescribeRouteTablesResult {
    const filterSensitiveLog: (obj: DescribeRouteTablesResult) => any;
    const isa: (o: any) => o is DescribeRouteTablesResult;
}
/**
 * <p>Contains the parameters for DescribeScheduledInstanceAvailability.</p>
 */
export interface DescribeScheduledInstanceAvailabilityRequest {
    __type?: "DescribeScheduledInstanceAvailabilityRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>availability-zone</code> - The Availability Zone (for example, <code>us-west-2a</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-type</code> - The instance type (for example, <code>c4.large</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-platform</code> - The network platform (<code>EC2-Classic</code> or <code>EC2-VPC</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>platform</code> - The platform (<code>Linux/UNIX</code> or <code>Windows</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The time period for the first schedule to start.</p>
     */
    FirstSlotStartTimeRange: SlotDateTimeRangeRequest | undefined;
    /**
     * <p>The maximum number of results to return in a single call.
     *          This value can be between 5 and 300. The default value is 300.
     *          To retrieve the remaining results, make another call with the returned
     *          <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The maximum available duration, in hours. This value must be greater than <code>MinSlotDurationInHours</code>
     *          and less than 1,720.</p>
     */
    MaxSlotDurationInHours?: number;
    /**
     * <p>The minimum available duration, in hours. The minimum required duration is 1,200 hours per year. For example, the minimum daily schedule is 4 hours, the minimum weekly schedule is 24 hours, and the minimum monthly schedule is 100 hours.</p>
     */
    MinSlotDurationInHours?: number;
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The schedule recurrence.</p>
     */
    Recurrence: ScheduledInstanceRecurrenceRequest | undefined;
}
export declare namespace DescribeScheduledInstanceAvailabilityRequest {
    const filterSensitiveLog: (obj: DescribeScheduledInstanceAvailabilityRequest) => any;
    const isa: (o: any) => o is DescribeScheduledInstanceAvailabilityRequest;
}
/**
 * <p>Contains the output of DescribeScheduledInstanceAvailability.</p>
 */
export interface DescribeScheduledInstanceAvailabilityResult {
    __type?: "DescribeScheduledInstanceAvailabilityResult";
    /**
     * <p>The token required to retrieve the next set of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the available Scheduled Instances.</p>
     */
    ScheduledInstanceAvailabilitySet?: ScheduledInstanceAvailability[];
}
export declare namespace DescribeScheduledInstanceAvailabilityResult {
    const filterSensitiveLog: (obj: DescribeScheduledInstanceAvailabilityResult) => any;
    const isa: (o: any) => o is DescribeScheduledInstanceAvailabilityResult;
}
/**
 * <p>Contains the parameters for DescribeScheduledInstances.</p>
 */
export interface DescribeScheduledInstancesRequest {
    __type?: "DescribeScheduledInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>availability-zone</code> - The Availability Zone (for example, <code>us-west-2a</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-type</code> - The instance type (for example, <code>c4.large</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-platform</code> - The network platform (<code>EC2-Classic</code> or <code>EC2-VPC</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>platform</code> - The platform (<code>Linux/UNIX</code> or <code>Windows</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return in a single call.
     *          This value can be between 5 and 300. The default value is 100.
     *          To retrieve the remaining results, make another call with the returned
     *          <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The Scheduled Instance IDs.</p>
     */
    ScheduledInstanceIds?: string[];
    /**
     * <p>The time period for the first schedule to start.</p>
     */
    SlotStartTimeRange?: SlotStartTimeRangeRequest;
}
export declare namespace DescribeScheduledInstancesRequest {
    const filterSensitiveLog: (obj: DescribeScheduledInstancesRequest) => any;
    const isa: (o: any) => o is DescribeScheduledInstancesRequest;
}
/**
 * <p>Contains the output of DescribeScheduledInstances.</p>
 */
export interface DescribeScheduledInstancesResult {
    __type?: "DescribeScheduledInstancesResult";
    /**
     * <p>The token required to retrieve the next set of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the Scheduled Instances.</p>
     */
    ScheduledInstanceSet?: ScheduledInstance[];
}
export declare namespace DescribeScheduledInstancesResult {
    const filterSensitiveLog: (obj: DescribeScheduledInstancesResult) => any;
    const isa: (o: any) => o is DescribeScheduledInstancesResult;
}
export interface DescribeSecurityGroupReferencesRequest {
    __type?: "DescribeSecurityGroupReferencesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the security groups in your account.</p>
     */
    GroupId: string[] | undefined;
}
export declare namespace DescribeSecurityGroupReferencesRequest {
    const filterSensitiveLog: (obj: DescribeSecurityGroupReferencesRequest) => any;
    const isa: (o: any) => o is DescribeSecurityGroupReferencesRequest;
}
export interface DescribeSecurityGroupReferencesResult {
    __type?: "DescribeSecurityGroupReferencesResult";
    /**
     * <p>Information about the VPCs with the referencing security groups.</p>
     */
    SecurityGroupReferenceSet?: SecurityGroupReference[];
}
export declare namespace DescribeSecurityGroupReferencesResult {
    const filterSensitiveLog: (obj: DescribeSecurityGroupReferencesResult) => any;
    const isa: (o: any) => o is DescribeSecurityGroupReferencesResult;
}
export interface DescribeSecurityGroupsRequest {
    __type?: "DescribeSecurityGroupsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters. If using multiple filters for rules, the results include security groups for which any combination of rules - not necessarily a single rule - match all filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>description</code> - The description of the security group.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>egress.ip-permission.cidr</code> - An IPv4 CIDR block for an outbound
     *                     security group rule.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>egress.ip-permission.from-port</code> - For an outbound rule, the
     *                     start of port range for the TCP and UDP protocols, or an ICMP type
     *                     number.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>egress.ip-permission.group-id</code> - The ID of a security group
     *                     that has been referenced in an outbound security group rule.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>egress.ip-permission.group-name</code> - The name of a security group
     *                     that has been referenced in an outbound security group rule.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>egress.ip-permission.ipv6-cidr</code> - An IPv6 CIDR block for an
     *                     outbound security group rule.</p>
     *             </li>
     *             <li>
     *      		        <p>
     *                   <code>egress.ip-permission.prefix-list-id</code> - The ID (prefix) of the AWS
     *                     service to which a security group rule allows outbound access.</p>
     *      	      </li>
     *             <li>
     *                 <p>
     *                   <code>egress.ip-permission.protocol</code> - The IP protocol for an
     *                     outbound security group rule (<code>tcp</code> | <code>udp</code> |
     *                         <code>icmp</code> or a protocol number).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>egress.ip-permission.to-port</code> - For an outbound rule, the end
     *                     of port range for the TCP and UDP protocols, or an ICMP code.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>egress.ip-permission.user-id</code> - The ID of an AWS account that
     *                     has been referenced in an outbound security group rule.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>group-id</code> - The ID of the security group. </p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>group-name</code> - The name of the security group.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>ip-permission.cidr</code> - An IPv4 CIDR block for an inbound security
     *                     group rule.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>ip-permission.from-port</code> - For an inbound rule, the start of port
     *                     range for the TCP and UDP protocols, or an ICMP type number.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>ip-permission.group-id</code> - The ID of a security group that has been
     *                     referenced in an inbound security group rule.</p>
     *             </li>
     *             <li>
     * 				           <p>
     *                     <code>ip-permission.group-name</code> - The name of a security group that has
     *                     been referenced in an inbound security group rule.</p>
     * 			         </li>
     *             <li>
     *                <p>
     *                   <code>ip-permission.ipv6-cidr</code> - An IPv6 CIDR block for an inbound security
     *                     group rule.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ip-permission.prefix-list-id</code> - The ID (prefix) of the AWS
     *                     service from which a security group rule allows inbound access.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>ip-permission.protocol</code> - The IP protocol for an inbound security
     *                     group rule (<code>tcp</code> | <code>udp</code> | <code>icmp</code> or a
     *                     protocol number).</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>ip-permission.to-port</code> - For an inbound rule, the end of port range
     *                     for the TCP and UDP protocols, or an ICMP code.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>ip-permission.user-id</code> - The ID of an AWS account that has been
     *                     referenced in an inbound security group rule.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>owner-id</code> - The AWS account ID of the owner of the security group.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>vpc-id</code> - The ID of the VPC specified when the security group was created.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The IDs of the security groups. Required for security groups in a nondefault VPC.</p>
     *          <p>Default: Describes all your security groups.</p>
     */
    GroupIds?: string[];
    /**
     * <p>[EC2-Classic and default VPC only] The names of the security groups. You can specify either
     * 			the security group name or the security group ID. For security groups in a nondefault VPC, use
     * 			the <code>group-name</code> filter to describe security groups by name.</p>
     *          <p>Default: Describes all your security groups.</p>
     */
    GroupNames?: string[];
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining
     *             results, make another request with the returned <code>NextToken</code> value. This value
     *             can be between 5 and 1000. If this parameter is not specified, then all results are
     *             returned.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to request the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeSecurityGroupsRequest {
    const filterSensitiveLog: (obj: DescribeSecurityGroupsRequest) => any;
    const isa: (o: any) => o is DescribeSecurityGroupsRequest;
}
export interface DescribeSecurityGroupsResult {
    __type?: "DescribeSecurityGroupsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the security groups.</p>
     */
    SecurityGroups?: SecurityGroup[];
}
export declare namespace DescribeSecurityGroupsResult {
    const filterSensitiveLog: (obj: DescribeSecurityGroupsResult) => any;
    const isa: (o: any) => o is DescribeSecurityGroupsResult;
}
export interface DescribeSnapshotAttributeRequest {
    __type?: "DescribeSnapshotAttributeRequest";
    /**
     * <p>The snapshot attribute you would like to view.</p>
     */
    Attribute: SnapshotAttributeName | string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the EBS snapshot.</p>
     */
    SnapshotId: string | undefined;
}
export declare namespace DescribeSnapshotAttributeRequest {
    const filterSensitiveLog: (obj: DescribeSnapshotAttributeRequest) => any;
    const isa: (o: any) => o is DescribeSnapshotAttributeRequest;
}
export interface DescribeSnapshotAttributeResult {
    __type?: "DescribeSnapshotAttributeResult";
    /**
     * <p>The users and groups that have the permissions for creating volumes from the
     *       snapshot.</p>
     */
    CreateVolumePermissions?: CreateVolumePermission[];
    /**
     * <p>The product codes.</p>
     */
    ProductCodes?: ProductCode[];
    /**
     * <p>The ID of the EBS snapshot.</p>
     */
    SnapshotId?: string;
}
export declare namespace DescribeSnapshotAttributeResult {
    const filterSensitiveLog: (obj: DescribeSnapshotAttributeResult) => any;
    const isa: (o: any) => o is DescribeSnapshotAttributeResult;
}
export interface DescribeSnapshotsRequest {
    __type?: "DescribeSnapshotsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>description</code> - A description of the snapshot.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>encrypted</code> - Indicates whether the snapshot is encrypted
     *             (<code>true</code> | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>owner-alias</code> - Value from an Amazon-maintained list (<code>amazon</code> |
     *             <code>self</code> | <code>all</code> | <code>aws-marketplace</code> |
     *             <code>microsoft</code>) of snapshot owners. Not to be confused with the user-configured
     *           AWS account alias, which is set from the IAM console.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>owner-id</code> - The ID of the AWS account that owns the snapshot.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>progress</code> - The progress of the snapshot, as a percentage (for example,
     *           80%).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>snapshot-id</code> - The snapshot ID.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>start-time</code> - The time stamp when the snapshot was initiated.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>status</code> - The status of the snapshot (<code>pending</code> |
     *             <code>completed</code> | <code>error</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>volume-id</code> - The ID of the volume the snapshot is for.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>volume-size</code> - The size of the volume, in GiB.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of snapshot results returned by <code>DescribeSnapshots</code> in
     *       paginated output. When this parameter is used, <code>DescribeSnapshots</code> only returns
     *         <code>MaxResults</code> results in a single page along with a <code>NextToken</code>
     *       response element. The remaining results of the initial request can be seen by sending another
     *         <code>DescribeSnapshots</code> request with the returned <code>NextToken</code> value. This
     *       value can be between 5 and 1000; if <code>MaxResults</code> is given a value larger than 1000,
     *       only 1000 results are returned. If this parameter is not used, then
     *         <code>DescribeSnapshots</code> returns all results. You cannot specify this parameter and
     *       the snapshot IDs parameter in the same request.</p>
     */
    MaxResults?: number;
    /**
     * <p>The <code>NextToken</code> value returned from a previous paginated
     *         <code>DescribeSnapshots</code> request where <code>MaxResults</code> was used and the
     *       results exceeded the value of that parameter. Pagination continues from the end of the
     *       previous results that returned the <code>NextToken</code> value. This value is
     *         <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Describes the snapshots owned by these owners.</p>
     */
    OwnerIds?: string[];
    /**
     * <p>The IDs of the AWS accounts that can create volumes from the snapshot.</p>
     */
    RestorableByUserIds?: string[];
    /**
     * <p>The snapshot IDs.</p>
     *          <p>Default: Describes the snapshots for which you have create volume permissions.</p>
     */
    SnapshotIds?: string[];
}
export declare namespace DescribeSnapshotsRequest {
    const filterSensitiveLog: (obj: DescribeSnapshotsRequest) => any;
    const isa: (o: any) => o is DescribeSnapshotsRequest;
}
export interface DescribeSnapshotsResult {
    __type?: "DescribeSnapshotsResult";
    /**
     * <p>The <code>NextToken</code> value to include in a future <code>DescribeSnapshots</code>
     *       request. When the results of a <code>DescribeSnapshots</code> request exceed
     *         <code>MaxResults</code>, this value can be used to retrieve the next page of results. This
     *       value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the snapshots.</p>
     */
    Snapshots?: Snapshot[];
}
export declare namespace DescribeSnapshotsResult {
    const filterSensitiveLog: (obj: DescribeSnapshotsResult) => any;
    const isa: (o: any) => o is DescribeSnapshotsResult;
}
/**
 * <p>Contains the parameters for DescribeSpotDatafeedSubscription.</p>
 */
export interface DescribeSpotDatafeedSubscriptionRequest {
    __type?: "DescribeSpotDatafeedSubscriptionRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DescribeSpotDatafeedSubscriptionRequest {
    const filterSensitiveLog: (obj: DescribeSpotDatafeedSubscriptionRequest) => any;
    const isa: (o: any) => o is DescribeSpotDatafeedSubscriptionRequest;
}
/**
 * <p>Contains the output of DescribeSpotDatafeedSubscription.</p>
 */
export interface DescribeSpotDatafeedSubscriptionResult {
    __type?: "DescribeSpotDatafeedSubscriptionResult";
    /**
     * <p>The Spot Instance data feed subscription.</p>
     */
    SpotDatafeedSubscription?: SpotDatafeedSubscription;
}
export declare namespace DescribeSpotDatafeedSubscriptionResult {
    const filterSensitiveLog: (obj: DescribeSpotDatafeedSubscriptionResult) => any;
    const isa: (o: any) => o is DescribeSpotDatafeedSubscriptionResult;
}
/**
 * <p>Contains the parameters for DescribeSpotFleetInstances.</p>
 */
export interface DescribeSpotFleetInstancesRequest {
    __type?: "DescribeSpotFleetInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *       and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *       Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The maximum number of results to return in a single call.
     *       Specify a value between 1 and 1000. The default value is 1000.
     *       To retrieve the remaining results, make another call with the returned
     *       <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the Spot Fleet request.</p>
     */
    SpotFleetRequestId: string | undefined;
}
export declare namespace DescribeSpotFleetInstancesRequest {
    const filterSensitiveLog: (obj: DescribeSpotFleetInstancesRequest) => any;
    const isa: (o: any) => o is DescribeSpotFleetInstancesRequest;
}
/**
 * <p>Contains the output of DescribeSpotFleetInstances.</p>
 */
export interface DescribeSpotFleetInstancesResponse {
    __type?: "DescribeSpotFleetInstancesResponse";
    /**
     * <p>The running instances. This list is refreshed periodically and might be out of
     *             date.</p>
     */
    ActiveInstances?: ActiveInstance[];
    /**
     * <p>The token required to retrieve the next set of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the Spot Fleet request.</p>
     */
    SpotFleetRequestId?: string;
}
export declare namespace DescribeSpotFleetInstancesResponse {
    const filterSensitiveLog: (obj: DescribeSpotFleetInstancesResponse) => any;
    const isa: (o: any) => o is DescribeSpotFleetInstancesResponse;
}
/**
 * <p>Contains the parameters for DescribeSpotFleetRequestHistory.</p>
 */
export interface DescribeSpotFleetRequestHistoryRequest {
    __type?: "DescribeSpotFleetRequestHistoryRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *       and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *       Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The type of events to describe. By default, all events are described.</p>
     */
    EventType?: EventType | string;
    /**
     * <p>The maximum number of results to return in a single call.
     *       Specify a value between 1 and 1000. The default value is 1000.
     *       To retrieve the remaining results, make another call with the returned
     *       <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the Spot Fleet request.</p>
     */
    SpotFleetRequestId: string | undefined;
    /**
     * <p>The starting date and time for the events, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    StartTime: Date | undefined;
}
export declare namespace DescribeSpotFleetRequestHistoryRequest {
    const filterSensitiveLog: (obj: DescribeSpotFleetRequestHistoryRequest) => any;
    const isa: (o: any) => o is DescribeSpotFleetRequestHistoryRequest;
}
/**
 * <p>Contains the output of DescribeSpotFleetRequestHistory.</p>
 */
export interface DescribeSpotFleetRequestHistoryResponse {
    __type?: "DescribeSpotFleetRequestHistoryResponse";
    /**
     * <p>Information about the events in the history of the Spot Fleet request.</p>
     */
    HistoryRecords?: HistoryRecord[];
    /**
     * <p>The last date and time for the events, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).
     *       All records up to this time were retrieved.</p>
     *          <p>If <code>nextToken</code> indicates that there are more results, this value is not present.</p>
     */
    LastEvaluatedTime?: Date;
    /**
     * <p>The token required to retrieve the next set of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the Spot Fleet request.</p>
     */
    SpotFleetRequestId?: string;
    /**
     * <p>The starting date and time for the events, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    StartTime?: Date;
}
export declare namespace DescribeSpotFleetRequestHistoryResponse {
    const filterSensitiveLog: (obj: DescribeSpotFleetRequestHistoryResponse) => any;
    const isa: (o: any) => o is DescribeSpotFleetRequestHistoryResponse;
}
/**
 * <p>Contains the parameters for DescribeSpotFleetRequests.</p>
 */
export interface DescribeSpotFleetRequestsRequest {
    __type?: "DescribeSpotFleetRequestsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *       and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *       Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The maximum number of results to return in a single call.
     *       Specify a value between 1 and 1000. The default value is 1000.
     *       To retrieve the remaining results, make another call with the returned
     *       <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The IDs of the Spot Fleet requests.</p>
     */
    SpotFleetRequestIds?: string[];
}
export declare namespace DescribeSpotFleetRequestsRequest {
    const filterSensitiveLog: (obj: DescribeSpotFleetRequestsRequest) => any;
    const isa: (o: any) => o is DescribeSpotFleetRequestsRequest;
}
/**
 * <p>Contains the output of DescribeSpotFleetRequests.</p>
 */
export interface DescribeSpotFleetRequestsResponse {
    __type?: "DescribeSpotFleetRequestsResponse";
    /**
     * <p>The token required to retrieve the next set of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the configuration of your Spot Fleet.</p>
     */
    SpotFleetRequestConfigs?: SpotFleetRequestConfig[];
}
export declare namespace DescribeSpotFleetRequestsResponse {
    const filterSensitiveLog: (obj: DescribeSpotFleetRequestsResponse) => any;
    const isa: (o: any) => o is DescribeSpotFleetRequestsResponse;
}
/**
 * <p>Contains the parameters for DescribeSpotInstanceRequests.</p>
 */
export interface DescribeSpotInstanceRequestsRequest {
    __type?: "DescribeSpotInstanceRequestsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>availability-zone-group</code> - The Availability Zone group.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>create-time</code> - The time stamp when the Spot Instance request was created.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>fault-code</code> - The fault code related to the request.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>fault-message</code> - The fault message related to the request.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-id</code> - The ID of the instance that fulfilled the request.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch-group</code> - The Spot Instance launch group.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.block-device-mapping.delete-on-termination</code> - Indicates whether the EBS volume is deleted on instance termination.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.block-device-mapping.device-name</code> - The device name for the volume in the block device mapping (for example, <code>/dev/sdh</code> or <code>xvdh</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.block-device-mapping.snapshot-id</code> - The ID of the snapshot for the EBS volume.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.block-device-mapping.volume-size</code> - The size of the EBS volume, in GiB.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.block-device-mapping.volume-type</code> - The type of EBS volume:
     *                <code>gp2</code> for General Purpose SSD, <code>io1</code> for Provisioned IOPS SSD, <code>st1</code> for
     *                Throughput Optimized HDD, <code>sc1</code>for Cold HDD, or <code>standard</code> for Magnetic.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.group-id</code> - The ID of the security group for the instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.group-name</code> - The name of the security group for the instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.image-id</code> - The ID of the AMI.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.instance-type</code> - The type of instance (for example, <code>m3.medium</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.kernel-id</code> - The kernel ID.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.key-name</code> - The name of the key pair the instance launched with.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.monitoring-enabled</code> - Whether detailed monitoring is enabled for the Spot Instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launch.ramdisk-id</code> - The RAM disk ID.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>launched-availability-zone</code> - The Availability Zone in which the request is launched.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface.addresses.primary</code> - Indicates whether the IP address is
     *                the primary private IP address.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface.delete-on-termination</code> - Indicates whether the network
     *                interface is deleted when the instance is terminated.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface.description</code> - A description of the network interface.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface.device-index</code> - The index of the device for the network
     *                interface attachment on the instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface.group-id</code> - The ID of the security group associated with the network interface.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface.network-interface-id</code> - The ID of the network interface.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface.private-ip-address</code> - The primary private IP address of
     *                the network interface.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface.subnet-id</code> - The ID of the subnet for the instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>product-description</code> - The product description associated with the instance (<code>Linux/UNIX</code> | <code>Windows</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>spot-instance-request-id</code> - The Spot Instance request ID.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>spot-price</code> - The maximum hourly price for any Spot Instance launched to fulfill the request.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the Spot Instance request (<code>open</code>
     *                     | <code>active</code> | <code>closed</code> | <code>cancelled</code> |
     *                         <code>failed</code>). Spot request status information can help you track
     *                     your Amazon EC2 Spot Instance requests. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-bid-status.html">Spot
     *                         Request Status</a> in the
     *                     <i>Amazon EC2 User Guide for Linux Instances</i>.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>status-code</code> - The short code describing the most recent evaluation of your Spot Instance request.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>status-message</code> - The message explaining the status of the Spot Instance request.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>type</code> - The type of Spot Instance request (<code>one-time</code> | <code>persistent</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>valid-from</code> - The start date of the request.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>valid-until</code> - The end date of the request.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return in a single call. Specify a value between 5
     *             and 1000. To retrieve the remaining results, make another call with the returned
     *                 <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to request the next set of results. This value is <code>null</code> when
     *             there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>One or more Spot Instance request IDs.</p>
     */
    SpotInstanceRequestIds?: string[];
}
export declare namespace DescribeSpotInstanceRequestsRequest {
    const filterSensitiveLog: (obj: DescribeSpotInstanceRequestsRequest) => any;
    const isa: (o: any) => o is DescribeSpotInstanceRequestsRequest;
}
/**
 * <p>Contains the output of DescribeSpotInstanceRequests.</p>
 */
export interface DescribeSpotInstanceRequestsResult {
    __type?: "DescribeSpotInstanceRequestsResult";
    /**
     * <p>The token to use to retrieve the next set of results. This value is <code>null</code>
     *             when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>One or more Spot Instance requests.</p>
     */
    SpotInstanceRequests?: SpotInstanceRequest[];
}
export declare namespace DescribeSpotInstanceRequestsResult {
    const filterSensitiveLog: (obj: DescribeSpotInstanceRequestsResult) => any;
    const isa: (o: any) => o is DescribeSpotInstanceRequestsResult;
}
/**
 * <p>Contains the parameters for DescribeSpotPriceHistory.</p>
 */
export interface DescribeSpotPriceHistoryRequest {
    __type?: "DescribeSpotPriceHistoryRequest";
    /**
     * <p>Filters the results by the specified Availability Zone.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The date and time, up to the current date, from which to stop retrieving the price history data,
     *        in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    EndTime?: Date;
    /**
     * <p>One or more filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>availability-zone</code> - The Availability Zone for which prices should be returned.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instance-type</code> - The type of instance (for example, <code>m3.medium</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>product-description</code> - The product description for the Spot price
     *            (<code>Linux/UNIX</code> | <code>SUSE Linux</code> | <code>Windows</code> | <code>Linux/UNIX (Amazon VPC)</code> | <code>SUSE Linux (Amazon VPC)</code> | <code>Windows (Amazon VPC)</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>spot-price</code> - The Spot price. The value must match exactly (or use wildcards; greater than or less than comparison is not supported).</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>timestamp</code> - The time stamp of the Spot price history, in UTC format
     *                     (for example,
     *                         <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).
     *                     You can use wildcards (* and ?). Greater than or less than comparison is not
     *                     supported.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>Filters the results by the specified instance types.</p>
     */
    InstanceTypes?: (_InstanceType | string)[];
    /**
     * <p>The maximum number of results to return in a single call.
     *        Specify a value between 1 and 1000. The default value is 1000.
     *        To retrieve the remaining results, make another call with the returned
     *        <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next set of results.</p>
     */
    NextToken?: string;
    /**
     * <p>Filters the results by the specified basic product descriptions.</p>
     */
    ProductDescriptions?: string[];
    /**
     * <p>The date and time, up to the past 90 days, from which to start retrieving the price history data,
     *        in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    StartTime?: Date;
}
export declare namespace DescribeSpotPriceHistoryRequest {
    const filterSensitiveLog: (obj: DescribeSpotPriceHistoryRequest) => any;
    const isa: (o: any) => o is DescribeSpotPriceHistoryRequest;
}
/**
 * <p>Contains the output of DescribeSpotPriceHistory.</p>
 */
export interface DescribeSpotPriceHistoryResult {
    __type?: "DescribeSpotPriceHistoryResult";
    /**
     * <p>The token required to retrieve the next set of results. This value is null or an empty
     *             string when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>The historical Spot prices.</p>
     */
    SpotPriceHistory?: SpotPrice[];
}
export declare namespace DescribeSpotPriceHistoryResult {
    const filterSensitiveLog: (obj: DescribeSpotPriceHistoryResult) => any;
    const isa: (o: any) => o is DescribeSpotPriceHistoryResult;
}
export interface DescribeStaleSecurityGroupsRequest {
    __type?: "DescribeStaleSecurityGroupsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The maximum number of items to return for this request. The request returns a token that you can specify in a subsequent call to get the next set of results.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next set of items to return. (You received this token from a prior call.)</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace DescribeStaleSecurityGroupsRequest {
    const filterSensitiveLog: (obj: DescribeStaleSecurityGroupsRequest) => any;
    const isa: (o: any) => o is DescribeStaleSecurityGroupsRequest;
}
export interface DescribeStaleSecurityGroupsResult {
    __type?: "DescribeStaleSecurityGroupsResult";
    /**
     * <p>The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the stale security groups.</p>
     */
    StaleSecurityGroupSet?: StaleSecurityGroup[];
}
export declare namespace DescribeStaleSecurityGroupsResult {
    const filterSensitiveLog: (obj: DescribeStaleSecurityGroupsResult) => any;
    const isa: (o: any) => o is DescribeStaleSecurityGroupsResult;
}
export interface DescribeSubnetsRequest {
    __type?: "DescribeSubnetsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>availability-zone</code> - The Availability Zone for the subnet. You can also use
     *                     <code>availabilityZone</code> as the filter name.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>availability-zone-id</code> - The ID of the Availability Zone for the subnet.
     *                     You can also use <code>availabilityZoneId</code> as the filter name.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>available-ip-address-count</code> - The number of IPv4 addresses in the
     *                     subnet that are available.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>cidr-block</code> - The IPv4 CIDR block of the subnet. The CIDR block you
     *                     specify must exactly match the subnet's CIDR block for information to be
     *                     returned for the subnet. You can also use <code>cidr</code> or
     *                     <code>cidrBlock</code> as the filter names.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>default-for-az</code> - Indicates whether this is the default subnet for the
     *                     Availability Zone. You can also use <code>defaultForAz</code> as the filter name.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ipv6-cidr-block-association.ipv6-cidr-block</code> - An IPv6 CIDR
     *                     block associated with the subnet.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ipv6-cidr-block-association.association-id</code> - An association ID
     *                     for an IPv6 CIDR block associated with the subnet.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ipv6-cidr-block-association.state</code> - The state of an IPv6 CIDR
     *                     block associated with the subnet.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>owner-id</code> - The ID of the AWS account that owns the subnet.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>state</code> - The state of the subnet (<code>pending</code> | <code>available</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>subnet-arn</code> - The Amazon Resource Name (ARN) of the subnet.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>subnet-id</code> - The ID of the subnet.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>vpc-id</code> - The ID of the VPC for the subnet.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>One or more subnet IDs.</p>
     * 		       <p>Default: Describes all your subnets.</p>
     */
    SubnetIds?: string[];
}
export declare namespace DescribeSubnetsRequest {
    const filterSensitiveLog: (obj: DescribeSubnetsRequest) => any;
    const isa: (o: any) => o is DescribeSubnetsRequest;
}
export interface DescribeSubnetsResult {
    __type?: "DescribeSubnetsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about one or more subnets.</p>
     */
    Subnets?: Subnet[];
}
export declare namespace DescribeSubnetsResult {
    const filterSensitiveLog: (obj: DescribeSubnetsResult) => any;
    const isa: (o: any) => o is DescribeSubnetsResult;
}
export interface DescribeTagsRequest {
    __type?: "DescribeTagsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>key</code> - The tag key.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>resource-id</code> - The ID of the resource.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>resource-type</code> - The resource type (<code>customer-gateway</code> |
     *             	  <code>dedicated-host</code> | <code>dhcp-options</code> | <code>elastic-ip</code> | <code>fleet</code> |
     *                   <code>fpga-image</code> | <code>image</code> | <code>instance</code> |
     *                   <code>host-reservation</code> | <code>internet-gateway</code> | <code>launch-template</code> |
     *                   <code>natgateway</code> | <code>network-acl</code> |
     *                   <code>network-interface</code> | <code>reserved-instances</code> |
     *                   <code>route-table</code> | <code>security-group</code> |
     *                   <code>snapshot</code> | <code>spot-instances-request</code> |
     *                   <code>subnet</code> | <code>volume</code> | <code>vpc</code> |
     *                   <code>vpc-peering-connection</code> | <code>vpn-connection</code> |
     *                   <code>vpn-gateway</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag</code>:<key> - The key/value combination of the tag. For example,
     *                 specify "tag:Owner" for the filter name and "TeamA" for the filter value to find
     *                 resources with the tag "Owner=TeamA".</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>value</code> - The tag value.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return in a single call.
     *       This value can be between 5 and 1000.
     * 			To retrieve the remaining results, make another call with the returned <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeTagsRequest {
    const filterSensitiveLog: (obj: DescribeTagsRequest) => any;
    const isa: (o: any) => o is DescribeTagsRequest;
}
export interface DescribeTagsResult {
    __type?: "DescribeTagsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is
     *          <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>The tags.</p>
     */
    Tags?: TagDescription[];
}
export declare namespace DescribeTagsResult {
    const filterSensitiveLog: (obj: DescribeTagsResult) => any;
    const isa: (o: any) => o is DescribeTagsResult;
}
export interface DescribeTrafficMirrorFiltersRequest {
    __type?: "DescribeTrafficMirrorFiltersRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>description</code>: The Traffic Mirror filter description.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>traffic-mirror-filter-id</code>: The ID of the Traffic Mirror filter.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the Traffic Mirror filter.</p>
     */
    TrafficMirrorFilterIds?: string[];
}
export declare namespace DescribeTrafficMirrorFiltersRequest {
    const filterSensitiveLog: (obj: DescribeTrafficMirrorFiltersRequest) => any;
    const isa: (o: any) => o is DescribeTrafficMirrorFiltersRequest;
}
export interface DescribeTrafficMirrorFiltersResult {
    __type?: "DescribeTrafficMirrorFiltersResult";
    /**
     * <p>The token to use to retrieve the next page of results. The value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about one or more Traffic Mirror filters.</p>
     */
    TrafficMirrorFilters?: TrafficMirrorFilter[];
}
export declare namespace DescribeTrafficMirrorFiltersResult {
    const filterSensitiveLog: (obj: DescribeTrafficMirrorFiltersResult) => any;
    const isa: (o: any) => o is DescribeTrafficMirrorFiltersResult;
}
export interface DescribeTrafficMirrorSessionsRequest {
    __type?: "DescribeTrafficMirrorSessionsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>description</code>: The Traffic Mirror session description.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface-id</code>: The ID of the Traffic Mirror session network interface.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>owner-id</code>: The ID of the account that owns the Traffic Mirror session.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>packet-length</code>: The assigned number of packets to mirror. </p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>session-number</code>: The assigned session number. </p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>traffic-mirror-filter-id</code>: The ID of the Traffic Mirror filter.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>traffic-mirror-session-id</code>: The ID of the Traffic Mirror session.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>traffic-mirror-target-id</code>: The ID of the Traffic Mirror target.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>virtual-network-id</code>: The virtual network ID of the Traffic Mirror session.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the Traffic Mirror session.</p>
     */
    TrafficMirrorSessionIds?: string[];
}
export declare namespace DescribeTrafficMirrorSessionsRequest {
    const filterSensitiveLog: (obj: DescribeTrafficMirrorSessionsRequest) => any;
    const isa: (o: any) => o is DescribeTrafficMirrorSessionsRequest;
}
export interface DescribeTrafficMirrorSessionsResult {
    __type?: "DescribeTrafficMirrorSessionsResult";
    /**
     * <p>The token to use to retrieve the next page of results. The value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Describes one or more Traffic Mirror sessions. By default, all Traffic Mirror sessions are described. Alternatively, you can filter the results.</p>
     */
    TrafficMirrorSessions?: TrafficMirrorSession[];
}
export declare namespace DescribeTrafficMirrorSessionsResult {
    const filterSensitiveLog: (obj: DescribeTrafficMirrorSessionsResult) => any;
    const isa: (o: any) => o is DescribeTrafficMirrorSessionsResult;
}
export interface DescribeTrafficMirrorTargetsRequest {
    __type?: "DescribeTrafficMirrorTargetsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>description</code>: The Traffic Mirror target description.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-interface-id</code>: The ID of the Traffic Mirror session network interface.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>network-load-balancer-arn</code>: The Amazon Resource Name (ARN) of the Network Load Balancer that is associated with the session.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>owner-id</code>: The ID of the account that owns the Traffic Mirror session.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>traffic-mirror-target-id</code>: The ID of the Traffic Mirror target.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the Traffic Mirror targets.</p>
     */
    TrafficMirrorTargetIds?: string[];
}
export declare namespace DescribeTrafficMirrorTargetsRequest {
    const filterSensitiveLog: (obj: DescribeTrafficMirrorTargetsRequest) => any;
    const isa: (o: any) => o is DescribeTrafficMirrorTargetsRequest;
}
export interface DescribeTrafficMirrorTargetsResult {
    __type?: "DescribeTrafficMirrorTargetsResult";
    /**
     * <p>The token to use to retrieve the next page of results. The value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about one or more Traffic Mirror targets.</p>
     */
    TrafficMirrorTargets?: TrafficMirrorTarget[];
}
export declare namespace DescribeTrafficMirrorTargetsResult {
    const filterSensitiveLog: (obj: DescribeTrafficMirrorTargetsResult) => any;
    const isa: (o: any) => o is DescribeTrafficMirrorTargetsResult;
}
export interface DescribeTransitGatewayAttachmentsRequest {
    __type?: "DescribeTransitGatewayAttachmentsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>association.state</code> - The state of the association (<code>associating</code> | <code>associated</code> |
     *                <code>disassociating</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>association.transit-gateway-route-table-id</code> - The ID of the route table for the transit gateway.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>resource-id</code> - The ID of the resource.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>resource-owner-id</code> - The ID of the AWS account that owns the resource.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>resource-type</code> - The resource type (<code>vpc</code> | <code>vpn</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the attachment (<code>available</code> | <code>deleted</code> | <code>deleting</code> | <code>failed</code> |  <code>modifying</code> | <code>pendingAcceptance</code> | <code>pending</code> | <code>rollingBack</code> | <code>rejected</code> | <code>rejecting</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-attachment-id</code> - The ID of the attachment.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-id</code> - The ID of the transit gateway.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-owner-id</code> - The ID of the AWS account that owns the transit gateway.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The IDs of the attachments.</p>
     */
    TransitGatewayAttachmentIds?: string[];
}
export declare namespace DescribeTransitGatewayAttachmentsRequest {
    const filterSensitiveLog: (obj: DescribeTransitGatewayAttachmentsRequest) => any;
    const isa: (o: any) => o is DescribeTransitGatewayAttachmentsRequest;
}
export interface DescribeTransitGatewayAttachmentsResult {
    __type?: "DescribeTransitGatewayAttachmentsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the attachments.</p>
     */
    TransitGatewayAttachments?: TransitGatewayAttachment[];
}
export declare namespace DescribeTransitGatewayAttachmentsResult {
    const filterSensitiveLog: (obj: DescribeTransitGatewayAttachmentsResult) => any;
    const isa: (o: any) => o is DescribeTransitGatewayAttachmentsResult;
}
export interface DescribeTransitGatewayMulticastDomainsRequest {
    __type?: "DescribeTransitGatewayMulticastDomainsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>state</code> - The state of the transit gateway multicast domain. Valid values are <code>pending</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>transit-gateway-id</code> - The ID of the transit gateway.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>transit-gateway-multicast-domain-id</code> - The ID of the transit gateway multicast domain.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainIds?: string[];
}
export declare namespace DescribeTransitGatewayMulticastDomainsRequest {
    const filterSensitiveLog: (obj: DescribeTransitGatewayMulticastDomainsRequest) => any;
    const isa: (o: any) => o is DescribeTransitGatewayMulticastDomainsRequest;
}
export interface DescribeTransitGatewayMulticastDomainsResult {
    __type?: "DescribeTransitGatewayMulticastDomainsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the transit gateway multicast domains.</p>
     */
    TransitGatewayMulticastDomains?: TransitGatewayMulticastDomain[];
}
export declare namespace DescribeTransitGatewayMulticastDomainsResult {
    const filterSensitiveLog: (obj: DescribeTransitGatewayMulticastDomainsResult) => any;
    const isa: (o: any) => o is DescribeTransitGatewayMulticastDomainsResult;
}
export interface DescribeTransitGatewayPeeringAttachmentsRequest {
    __type?: "DescribeTransitGatewayPeeringAttachmentsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>One or more IDs of the transit gateway peering attachments.</p>
     */
    TransitGatewayAttachmentIds?: string[];
}
export declare namespace DescribeTransitGatewayPeeringAttachmentsRequest {
    const filterSensitiveLog: (obj: DescribeTransitGatewayPeeringAttachmentsRequest) => any;
    const isa: (o: any) => o is DescribeTransitGatewayPeeringAttachmentsRequest;
}
export interface DescribeTransitGatewayPeeringAttachmentsResult {
    __type?: "DescribeTransitGatewayPeeringAttachmentsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>The transit gateway peering attachments.</p>
     */
    TransitGatewayPeeringAttachments?: TransitGatewayPeeringAttachment[];
}
export declare namespace DescribeTransitGatewayPeeringAttachmentsResult {
    const filterSensitiveLog: (obj: DescribeTransitGatewayPeeringAttachmentsResult) => any;
    const isa: (o: any) => o is DescribeTransitGatewayPeeringAttachmentsResult;
}
export interface DescribeTransitGatewayRouteTablesRequest {
    __type?: "DescribeTransitGatewayRouteTablesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>default-association-route-table</code> - Indicates whether this is the default
     *                 association route table for the transit gateway (<code>true</code> | <code>false</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>default-propagation-route-table</code> - Indicates whether this is the default
     *                propagation route table for the transit gateway (<code>true</code> | <code>false</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the attachment (<code>available</code> | <code>deleted</code> | <code>deleting</code> | <code>failed</code> |  <code>modifying</code> | <code>pendingAcceptance</code> | <code>pending</code> | <code>rollingBack</code> | <code>rejected</code> | <code>rejecting</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-id</code> - The ID of the transit gateway.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-route-table-id</code> - The ID of the transit gateway route table.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The IDs of the transit gateway route tables.</p>
     */
    TransitGatewayRouteTableIds?: string[];
}
export declare namespace DescribeTransitGatewayRouteTablesRequest {
    const filterSensitiveLog: (obj: DescribeTransitGatewayRouteTablesRequest) => any;
    const isa: (o: any) => o is DescribeTransitGatewayRouteTablesRequest;
}
export interface DescribeTransitGatewayRouteTablesResult {
    __type?: "DescribeTransitGatewayRouteTablesResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the transit gateway route tables.</p>
     */
    TransitGatewayRouteTables?: TransitGatewayRouteTable[];
}
export declare namespace DescribeTransitGatewayRouteTablesResult {
    const filterSensitiveLog: (obj: DescribeTransitGatewayRouteTablesResult) => any;
    const isa: (o: any) => o is DescribeTransitGatewayRouteTablesResult;
}
export interface DescribeTransitGatewaysRequest {
    __type?: "DescribeTransitGatewaysRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>options.propagation-default-route-table-id</code> - The ID of the default propagation route table.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>options.amazon-side-asn</code> - The private ASN for the Amazon side of a BGP session.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>options.association-default-route-table-id</code> - The ID of the default association route table.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>options.auto-accept-shared-attachments</code> - Indicates whether there is automatic acceptance of attachment requests (<code>enable</code> | <code>disable</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>options.default-route-table-association</code> - Indicates whether resource attachments are automatically
     *                associated with the default association route table (<code>enable</code> | <code>disable</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>options.default-route-table-propagation</code> - Indicates whether resource attachments automatically propagate
     *                routes to the default propagation route table (<code>enable</code> | <code>disable</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>options.dns-support</code> - Indicates whether DNS support is enabled (<code>enable</code> | <code>disable</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>options.vpn-ecmp-support</code> - Indicates whether Equal Cost Multipath Protocol support is enabled  (<code>enable</code> | <code>disable</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>owner-id</code> - The ID of the AWS account that owns the transit gateway.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the attachment (<code>available</code> | <code>deleted</code> | <code>deleting</code> | <code>failed</code> |  <code>modifying</code> | <code>pendingAcceptance</code> | <code>pending</code> | <code>rollingBack</code> | <code>rejected</code> | <code>rejecting</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-id</code> - The ID of the transit gateway.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The IDs of the transit gateways.</p>
     */
    TransitGatewayIds?: string[];
}
export declare namespace DescribeTransitGatewaysRequest {
    const filterSensitiveLog: (obj: DescribeTransitGatewaysRequest) => any;
    const isa: (o: any) => o is DescribeTransitGatewaysRequest;
}
export interface DescribeTransitGatewaysResult {
    __type?: "DescribeTransitGatewaysResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the transit gateways.</p>
     */
    TransitGateways?: TransitGateway[];
}
export declare namespace DescribeTransitGatewaysResult {
    const filterSensitiveLog: (obj: DescribeTransitGatewaysResult) => any;
    const isa: (o: any) => o is DescribeTransitGatewaysResult;
}
export interface DescribeTransitGatewayVpcAttachmentsRequest {
    __type?: "DescribeTransitGatewayVpcAttachmentsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the attachment (<code>available</code> | <code>deleted</code> | <code>deleting</code> | <code>failed</code> |  <code>modifying</code> | <code>pendingAcceptance</code> | <code>pending</code> | <code>rollingBack</code> | <code>rejected</code> | <code>rejecting</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-attachment-id</code> - The ID of the attachment.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-id</code> - The ID of the transit gateway.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>vpc-id</code> - The ID of the VPC.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The IDs of the attachments.</p>
     */
    TransitGatewayAttachmentIds?: string[];
}
export declare namespace DescribeTransitGatewayVpcAttachmentsRequest {
    const filterSensitiveLog: (obj: DescribeTransitGatewayVpcAttachmentsRequest) => any;
    const isa: (o: any) => o is DescribeTransitGatewayVpcAttachmentsRequest;
}
export interface DescribeTransitGatewayVpcAttachmentsResult {
    __type?: "DescribeTransitGatewayVpcAttachmentsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the VPC attachments.</p>
     */
    TransitGatewayVpcAttachments?: TransitGatewayVpcAttachment[];
}
export declare namespace DescribeTransitGatewayVpcAttachmentsResult {
    const filterSensitiveLog: (obj: DescribeTransitGatewayVpcAttachmentsResult) => any;
    const isa: (o: any) => o is DescribeTransitGatewayVpcAttachmentsResult;
}
export interface DescribeVolumeAttributeRequest {
    __type?: "DescribeVolumeAttributeRequest";
    /**
     * <p>The attribute of the volume. This parameter is required.</p>
     */
    Attribute: VolumeAttributeName | string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the volume.</p>
     */
    VolumeId: string | undefined;
}
export declare namespace DescribeVolumeAttributeRequest {
    const filterSensitiveLog: (obj: DescribeVolumeAttributeRequest) => any;
    const isa: (o: any) => o is DescribeVolumeAttributeRequest;
}
export interface DescribeVolumeAttributeResult {
    __type?: "DescribeVolumeAttributeResult";
    /**
     * <p>The state of <code>autoEnableIO</code> attribute.</p>
     */
    AutoEnableIO?: AttributeBooleanValue;
    /**
     * <p>A list of product codes.</p>
     */
    ProductCodes?: ProductCode[];
    /**
     * <p>The ID of the volume.</p>
     */
    VolumeId?: string;
}
export declare namespace DescribeVolumeAttributeResult {
    const filterSensitiveLog: (obj: DescribeVolumeAttributeResult) => any;
    const isa: (o: any) => o is DescribeVolumeAttributeResult;
}
export interface DescribeVolumesModificationsRequest {
    __type?: "DescribeVolumesModificationsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters. Supported filters: <code>volume-id</code>, <code>modification-state</code>,
     *         <code>target-size</code>, <code>target-iops</code>, <code>target-volume-type</code>,
     *         <code>original-size</code>, <code>original-iops</code>, <code>original-volume-type</code>,
     *         <code>start-time</code>. </p>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results (up to a limit of 500) to be returned in a paginated
     *       request.</p>
     */
    MaxResults?: number;
    /**
     * <p>The <code>nextToken</code> value returned by a previous paginated request.</p>
     */
    NextToken?: string;
    /**
     * <p>The IDs of the volumes for which in-progress modifications will be described.</p>
     */
    VolumeIds?: string[];
}
export declare namespace DescribeVolumesModificationsRequest {
    const filterSensitiveLog: (obj: DescribeVolumesModificationsRequest) => any;
    const isa: (o: any) => o is DescribeVolumesModificationsRequest;
}
export interface DescribeVolumesModificationsResult {
    __type?: "DescribeVolumesModificationsResult";
    /**
     * <p>Token for pagination, null if there are no more results </p>
     */
    NextToken?: string;
    /**
     * <p>Information about the volume modifications.</p>
     */
    VolumesModifications?: VolumeModification[];
}
export declare namespace DescribeVolumesModificationsResult {
    const filterSensitiveLog: (obj: DescribeVolumesModificationsResult) => any;
    const isa: (o: any) => o is DescribeVolumesModificationsResult;
}
export interface DescribeVolumesRequest {
    __type?: "DescribeVolumesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>attachment.attach-time</code> - The time stamp when the attachment
     *           initiated.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>attachment.delete-on-termination</code> - Whether the volume is deleted on
     *           instance termination.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>attachment.device</code> - The device name specified in the block device mapping
     *           (for example, <code>/dev/sda1</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>attachment.instance-id</code> - The ID of the instance the volume is attached
     *           to.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>attachment.status</code> - The attachment state (<code>attaching</code> |
     *             <code>attached</code> | <code>detaching</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>availability-zone</code> - The Availability Zone in which the volume was
     *           created.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>create-time</code> - The time stamp when the volume was created.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>encrypted</code> - Indicates whether the volume is encrypted (<code>true</code>
     *           | <code>false</code>)</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>size</code> - The size of the volume, in GiB.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>snapshot-id</code> - The snapshot from which the volume was created.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>status</code> - The status of the volume (<code>creating</code> |
     *             <code>available</code> | <code>in-use</code> | <code>deleting</code> |
     *             <code>deleted</code> | <code>error</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>volume-id</code> - The volume ID.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>volume-type</code> - The Amazon EBS volume type. This can be <code>gp2</code>
     *           for General Purpose SSD, <code>io1</code> for Provisioned IOPS SSD, <code>st1</code> for Throughput Optimized HDD,
     *             <code>sc1</code> for Cold HDD, or <code>standard</code> for Magnetic volumes.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of volume results returned by <code>DescribeVolumes</code> in paginated
     *       output. When this parameter is used, <code>DescribeVolumes</code> only returns
     *         <code>MaxResults</code> results in a single page along with a <code>NextToken</code>
     *       response element. The remaining results of the initial request can be seen by sending another
     *         <code>DescribeVolumes</code> request with the returned <code>NextToken</code> value. This
     *       value can be between 5 and 500; if <code>MaxResults</code> is given a value larger than 500,
     *       only 500 results are returned. If this parameter is not used, then
     *         <code>DescribeVolumes</code> returns all results. You cannot specify this parameter and the
     *       volume IDs parameter in the same request.</p>
     */
    MaxResults?: number;
    /**
     * <p>The <code>NextToken</code> value returned from a previous paginated
     *         <code>DescribeVolumes</code> request where <code>MaxResults</code> was used and the results
     *       exceeded the value of that parameter. Pagination continues from the end of the previous
     *       results that returned the <code>NextToken</code> value. This value is <code>null</code> when
     *       there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>The volume IDs.</p>
     */
    VolumeIds?: string[];
}
export declare namespace DescribeVolumesRequest {
    const filterSensitiveLog: (obj: DescribeVolumesRequest) => any;
    const isa: (o: any) => o is DescribeVolumesRequest;
}
export interface DescribeVolumesResult {
    __type?: "DescribeVolumesResult";
    /**
     * <p>The <code>NextToken</code> value to include in a future <code>DescribeVolumes</code>
     *       request. When the results of a <code>DescribeVolumes</code> request exceed
     *         <code>MaxResults</code>, this value can be used to retrieve the next page of results. This
     *       value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the volumes.</p>
     */
    Volumes?: Volume[];
}
export declare namespace DescribeVolumesResult {
    const filterSensitiveLog: (obj: DescribeVolumesResult) => any;
    const isa: (o: any) => o is DescribeVolumesResult;
}
export interface DescribeVolumeStatusRequest {
    __type?: "DescribeVolumeStatusRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>action.code</code> - The action code for the event (for example,
     *             <code>enable-volume-io</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>action.description</code> - A description of the action.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>action.event-id</code> - The event ID associated with the action.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>availability-zone</code> - The Availability Zone of the instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>event.description</code> - A description of the event.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>event.event-id</code> - The event ID.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>event.event-type</code> - The event type (for <code>io-enabled</code>:
     *             <code>passed</code> | <code>failed</code>; for <code>io-performance</code>:
     *             <code>io-performance:degraded</code> | <code>io-performance:severely-degraded</code> |
     *             <code>io-performance:stalled</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>event.not-after</code> - The latest end time for the event.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>event.not-before</code> - The earliest start time for the event.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>volume-status.details-name</code> - The cause for
     *             <code>volume-status.status</code> (<code>io-enabled</code> |
     *           <code>io-performance</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>volume-status.details-status</code> - The status of
     *             <code>volume-status.details-name</code> (for <code>io-enabled</code>:
     *             <code>passed</code> | <code>failed</code>; for <code>io-performance</code>:
     *             <code>normal</code> | <code>degraded</code> | <code>severely-degraded</code> |
     *             <code>stalled</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>volume-status.status</code> - The status of the volume (<code>ok</code> |
     *             <code>impaired</code> | <code>warning</code> | <code>insufficient-data</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of volume results returned by <code>DescribeVolumeStatus</code> in
     *       paginated output. When this parameter is used, the request only returns
     *         <code>MaxResults</code> results in a single page along with a <code>NextToken</code>
     *       response element. The remaining results of the initial request can be seen by sending another
     *       request with the returned <code>NextToken</code> value. This value can be between 5 and 1000;
     *       if <code>MaxResults</code> is given a value larger than 1000, only 1000 results are returned.
     *       If this parameter is not used, then <code>DescribeVolumeStatus</code> returns all results. You
     *       cannot specify this parameter and the volume IDs parameter in the same request.</p>
     */
    MaxResults?: number;
    /**
     * <p>The <code>NextToken</code> value to include in a future <code>DescribeVolumeStatus</code>
     *       request. When the results of the request exceed <code>MaxResults</code>, this value can be
     *       used to retrieve the next page of results. This value is <code>null</code> when there are no
     *       more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>The IDs of the volumes.</p>
     *          <p>Default: Describes all your volumes.</p>
     */
    VolumeIds?: string[];
}
export declare namespace DescribeVolumeStatusRequest {
    const filterSensitiveLog: (obj: DescribeVolumeStatusRequest) => any;
    const isa: (o: any) => o is DescribeVolumeStatusRequest;
}
export interface DescribeVolumeStatusResult {
    __type?: "DescribeVolumeStatusResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code>
     *       when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the status of the volumes.</p>
     */
    VolumeStatuses?: VolumeStatusItem[];
}
export declare namespace DescribeVolumeStatusResult {
    const filterSensitiveLog: (obj: DescribeVolumeStatusResult) => any;
    const isa: (o: any) => o is DescribeVolumeStatusResult;
}
export interface DescribeVpcAttributeRequest {
    __type?: "DescribeVpcAttributeRequest";
    /**
     * <p>The VPC attribute.</p>
     */
    Attribute: VpcAttributeName | string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace DescribeVpcAttributeRequest {
    const filterSensitiveLog: (obj: DescribeVpcAttributeRequest) => any;
    const isa: (o: any) => o is DescribeVpcAttributeRequest;
}
export interface DescribeVpcAttributeResult {
    __type?: "DescribeVpcAttributeResult";
    /**
     * <p>Indicates whether the instances launched in the VPC get DNS hostnames.
     * 				If this attribute is <code>true</code>, instances in the VPC get DNS hostnames;
     * 				otherwise, they do not.</p>
     */
    EnableDnsHostnames?: AttributeBooleanValue;
    /**
     * <p>Indicates whether DNS resolution is enabled for
     * 				the VPC. If this attribute is <code>true</code>, the Amazon DNS server
     * 				resolves DNS hostnames for your instances to their corresponding
     * 				IP addresses; otherwise, it does not.</p>
     */
    EnableDnsSupport?: AttributeBooleanValue;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace DescribeVpcAttributeResult {
    const filterSensitiveLog: (obj: DescribeVpcAttributeResult) => any;
    const isa: (o: any) => o is DescribeVpcAttributeResult;
}
export interface DescribeVpcClassicLinkDnsSupportRequest {
    __type?: "DescribeVpcClassicLinkDnsSupportRequest";
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>One or more VPC IDs.</p>
     */
    VpcIds?: string[];
}
export declare namespace DescribeVpcClassicLinkDnsSupportRequest {
    const filterSensitiveLog: (obj: DescribeVpcClassicLinkDnsSupportRequest) => any;
    const isa: (o: any) => o is DescribeVpcClassicLinkDnsSupportRequest;
}
export interface DescribeVpcClassicLinkDnsSupportResult {
    __type?: "DescribeVpcClassicLinkDnsSupportResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the ClassicLink DNS support status of the VPCs.</p>
     */
    Vpcs?: ClassicLinkDnsSupport[];
}
export declare namespace DescribeVpcClassicLinkDnsSupportResult {
    const filterSensitiveLog: (obj: DescribeVpcClassicLinkDnsSupportResult) => any;
    const isa: (o: any) => o is DescribeVpcClassicLinkDnsSupportResult;
}
export interface DescribeVpcClassicLinkRequest {
    __type?: "DescribeVpcClassicLinkRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     * 		       <ul>
     *             <li>
     * 				           <p>
     *                   <code>is-classic-link-enabled</code> - Whether the VPC is enabled for ClassicLink
     * 					   (<code>true</code> | <code>false</code>).</p>
     * 			         </li>
     *             <li>
     * 			            <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     * 			         </li>
     *             <li>
     *     			        <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     * 			         </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>One or more VPCs for which you want to describe the ClassicLink status.</p>
     */
    VpcIds?: string[];
}
export declare namespace DescribeVpcClassicLinkRequest {
    const filterSensitiveLog: (obj: DescribeVpcClassicLinkRequest) => any;
    const isa: (o: any) => o is DescribeVpcClassicLinkRequest;
}
export interface DescribeVpcClassicLinkResult {
    __type?: "DescribeVpcClassicLinkResult";
    /**
     * <p>The ClassicLink status of one or more VPCs.</p>
     */
    Vpcs?: VpcClassicLink[];
}
export declare namespace DescribeVpcClassicLinkResult {
    const filterSensitiveLog: (obj: DescribeVpcClassicLinkResult) => any;
    const isa: (o: any) => o is DescribeVpcClassicLinkResult;
}
export interface DescribeVpcEndpointConnectionNotificationsRequest {
    __type?: "DescribeVpcEndpointConnectionNotificationsRequest";
    /**
     * <p>The ID of the notification.</p>
     */
    ConnectionNotificationId?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>connection-notification-arn</code> - The ARN of the SNS topic for the
     *                     notification.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>connection-notification-id</code> - The ID of the
     *                     notification.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>connection-notification-state</code> - The state of the notification
     *                         (<code>Enabled</code> | <code>Disabled</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>connection-notification-type</code> - The type of notification
     *                         (<code>Topic</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>service-id</code> - The ID of the endpoint service.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>vpc-endpoint-id</code> - The ID of the VPC endpoint.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return in a single call. To retrieve the remaining
     *             results, make another request with the returned <code>NextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to request the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeVpcEndpointConnectionNotificationsRequest {
    const filterSensitiveLog: (obj: DescribeVpcEndpointConnectionNotificationsRequest) => any;
    const isa: (o: any) => o is DescribeVpcEndpointConnectionNotificationsRequest;
}
export interface DescribeVpcEndpointConnectionNotificationsResult {
    __type?: "DescribeVpcEndpointConnectionNotificationsResult";
    /**
     * <p>One or more notifications.</p>
     */
    ConnectionNotificationSet?: ConnectionNotification[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is
     *             <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeVpcEndpointConnectionNotificationsResult {
    const filterSensitiveLog: (obj: DescribeVpcEndpointConnectionNotificationsResult) => any;
    const isa: (o: any) => o is DescribeVpcEndpointConnectionNotificationsResult;
}
export interface DescribeVpcEndpointConnectionsRequest {
    __type?: "DescribeVpcEndpointConnectionsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     * 		       <ul>
     *             <li>
     *                 <p>
     *                   <code>service-id</code> - The ID of the service.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>vpc-endpoint-owner</code> - The AWS account number of the owner of the
     *                     endpoint.</p>
     *             </li>
     *             <li>
     * 			            <p>
     *                   <code>vpc-endpoint-state</code> - The state of the endpoint
     * 			        (<code>pendingAcceptance</code> | <code>pending</code> |
     * 			        <code>available</code> | <code>deleting</code> | <code>deleted</code> |
     * 			        <code>rejected</code> | <code>failed</code>).</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>vpc-endpoint-id</code> - The ID of the endpoint.</p>
     * 			         </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining
     *             results of the initial request can be seen by sending another request with the returned
     *                 <code>NextToken</code> value. This value can be between 5 and 1,000; if
     *                 <code>MaxResults</code> is given a value larger than 1,000, only 1,000 results are
     *             returned.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeVpcEndpointConnectionsRequest {
    const filterSensitiveLog: (obj: DescribeVpcEndpointConnectionsRequest) => any;
    const isa: (o: any) => o is DescribeVpcEndpointConnectionsRequest;
}
export interface DescribeVpcEndpointConnectionsResult {
    __type?: "DescribeVpcEndpointConnectionsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about one or more VPC endpoint connections.</p>
     */
    VpcEndpointConnections?: VpcEndpointConnection[];
}
export declare namespace DescribeVpcEndpointConnectionsResult {
    const filterSensitiveLog: (obj: DescribeVpcEndpointConnectionsResult) => any;
    const isa: (o: any) => o is DescribeVpcEndpointConnectionsResult;
}
export interface DescribeVpcEndpointServiceConfigurationsRequest {
    __type?: "DescribeVpcEndpointServiceConfigurationsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     * 		       <ul>
     *             <li>
     * 				           <p>
     *                   <code>service-name</code> - The name of the service.</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>service-id</code> - The ID of the service.</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>service-state</code> - The state of the service (<code>Pending</code> |
     *                         <code>Available</code> | <code>Deleting</code> | <code>Deleted</code> |
     *                         <code>Failed</code>). </p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     * 			         </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining
     *             results of the initial request can be seen by sending another request with the returned
     *                 <code>NextToken</code> value. This value can be between 5 and 1,000; if
     *                 <code>MaxResults</code> is given a value larger than 1,000, only 1,000 results are
     *             returned.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The IDs of one or more services.</p>
     */
    ServiceIds?: string[];
}
export declare namespace DescribeVpcEndpointServiceConfigurationsRequest {
    const filterSensitiveLog: (obj: DescribeVpcEndpointServiceConfigurationsRequest) => any;
    const isa: (o: any) => o is DescribeVpcEndpointServiceConfigurationsRequest;
}
export interface DescribeVpcEndpointServiceConfigurationsResult {
    __type?: "DescribeVpcEndpointServiceConfigurationsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about one or more services.</p>
     */
    ServiceConfigurations?: ServiceConfiguration[];
}
export declare namespace DescribeVpcEndpointServiceConfigurationsResult {
    const filterSensitiveLog: (obj: DescribeVpcEndpointServiceConfigurationsResult) => any;
    const isa: (o: any) => o is DescribeVpcEndpointServiceConfigurationsResult;
}
export interface DescribeVpcEndpointServicePermissionsRequest {
    __type?: "DescribeVpcEndpointServicePermissionsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     * 		       <ul>
     *             <li>
     * 				           <p>
     *                   <code>principal</code> - The ARN of the principal.</p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>principal-type</code> - The principal type (<code>All</code> |
     * 						<code>Service</code> | <code>OrganizationUnit</code> | <code>Account</code>
     * 					| <code>User</code> | <code>Role</code>).</p>
     * 			         </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining
     *             results of the initial request can be seen by sending another request with the returned
     *                 <code>NextToken</code> value. This value can be between 5 and 1,000; if
     *                 <code>MaxResults</code> is given a value larger than 1,000, only 1,000 results are
     *             returned.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the service.</p>
     */
    ServiceId: string | undefined;
}
export declare namespace DescribeVpcEndpointServicePermissionsRequest {
    const filterSensitiveLog: (obj: DescribeVpcEndpointServicePermissionsRequest) => any;
    const isa: (o: any) => o is DescribeVpcEndpointServicePermissionsRequest;
}
export interface DescribeVpcEndpointServicePermissionsResult {
    __type?: "DescribeVpcEndpointServicePermissionsResult";
    /**
     * <p>Information about one or more allowed principals.</p>
     */
    AllowedPrincipals?: AllowedPrincipal[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace DescribeVpcEndpointServicePermissionsResult {
    const filterSensitiveLog: (obj: DescribeVpcEndpointServicePermissionsResult) => any;
    const isa: (o: any) => o is DescribeVpcEndpointServicePermissionsResult;
}
/**
 * <p>Contains the parameters for DescribeVpcEndpointServices.</p>
 */
export interface DescribeVpcEndpointServicesRequest {
    __type?: "DescribeVpcEndpointServicesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>service-name</code> - The name of the service.</p>
     *             </li>
     *             <li>
     *         		     <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *         	   </li>
     *             <li>
     *         		     <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *         	   </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of items to return for this request. The request returns a token that you can specify in a subsequent call to get the next set of results.</p>
     *         <p>Constraint: If the value is greater than 1,000, we return only 1,000 items.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next set of items to return. (You received this token from a prior call.)</p>
     */
    NextToken?: string;
    /**
     * <p>One or more service names.</p>
     */
    ServiceNames?: string[];
}
export declare namespace DescribeVpcEndpointServicesRequest {
    const filterSensitiveLog: (obj: DescribeVpcEndpointServicesRequest) => any;
    const isa: (o: any) => o is DescribeVpcEndpointServicesRequest;
}
/**
 * <p>Contains the output of DescribeVpcEndpointServices.</p>
 */
export interface DescribeVpcEndpointServicesResult {
    __type?: "DescribeVpcEndpointServicesResult";
    /**
     * <p>The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the service.</p>
     */
    ServiceDetails?: ServiceDetail[];
    /**
     * <p>A list of supported services.</p>
     */
    ServiceNames?: string[];
}
export declare namespace DescribeVpcEndpointServicesResult {
    const filterSensitiveLog: (obj: DescribeVpcEndpointServicesResult) => any;
    const isa: (o: any) => o is DescribeVpcEndpointServicesResult;
}
/**
 * <p>Contains the parameters for DescribeVpcEndpoints.</p>
 */
export interface DescribeVpcEndpointsRequest {
    __type?: "DescribeVpcEndpointsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                     <code>service-name</code> - The name of the service.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>vpc-id</code> - The ID of the VPC in which the endpoint resides.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>vpc-endpoint-id</code> - The ID of the endpoint.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>vpc-endpoint-state</code> - The state of the endpoint
     *                         (<code>pendingAcceptance</code> | <code>pending</code> |
     *                         <code>available</code> | <code>deleting</code> | <code>deleted</code> |
     *                         <code>rejected</code> | <code>failed</code>).</p>
     *             </li>
     *             <li>
     *         		     <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *         	   </li>
     *             <li>
     *         		     <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *         	   </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of items to return for this request. The request returns a token that you can specify in a subsequent call to get the next set of results.</p>
     *         <p>Constraint: If the value is greater than 1,000, we return only 1,000 items.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next set of items to return. (You received this token from a prior call.)</p>
     */
    NextToken?: string;
    /**
     * <p>One or more endpoint IDs.</p>
     */
    VpcEndpointIds?: string[];
}
export declare namespace DescribeVpcEndpointsRequest {
    const filterSensitiveLog: (obj: DescribeVpcEndpointsRequest) => any;
    const isa: (o: any) => o is DescribeVpcEndpointsRequest;
}
/**
 * <p>Contains the output of DescribeVpcEndpoints.</p>
 */
export interface DescribeVpcEndpointsResult {
    __type?: "DescribeVpcEndpointsResult";
    /**
     * <p>The token to use when requesting the next set of items. If there are no additional items to return, the string is empty.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the endpoints.</p>
     */
    VpcEndpoints?: VpcEndpoint[];
}
export declare namespace DescribeVpcEndpointsResult {
    const filterSensitiveLog: (obj: DescribeVpcEndpointsResult) => any;
    const isa: (o: any) => o is DescribeVpcEndpointsResult;
}
export interface DescribeVpcPeeringConnectionsRequest {
    __type?: "DescribeVpcPeeringConnectionsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                     <code>accepter-vpc-info.cidr-block</code> - The IPv4 CIDR block of the accepter
     *                     VPC.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>accepter-vpc-info.owner-id</code> - The AWS account ID of the owner of the
     *                     accepter VPC.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>accepter-vpc-info.vpc-id</code> - The ID of the accepter VPC.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>expiration-time</code> - The expiration date and time for the VPC peering
     *           connection.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>requester-vpc-info.cidr-block</code> - The IPv4 CIDR block of the
     *                     requester's VPC.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>requester-vpc-info.owner-id</code> - The AWS account ID of the owner of the
     *           requester VPC.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>requester-vpc-info.vpc-id</code> - The ID of the requester VPC.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>status-code</code> - The status of the VPC peering connection
     *                         (<code>pending-acceptance</code> | <code>failed</code> |
     *                         <code>expired</code> | <code>provisioning</code> | <code>active</code> |
     *                         <code>deleting</code> | <code>deleted</code> |
     *                     <code>rejected</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>status-message</code> - A message that provides more information about the status
     *           of the VPC peering connection, if applicable.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>vpc-peering-connection-id</code> - The ID of the VPC peering connection.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>One or more VPC peering connection IDs.</p>
     *          <p>Default: Describes all your VPC peering connections.</p>
     */
    VpcPeeringConnectionIds?: string[];
}
export declare namespace DescribeVpcPeeringConnectionsRequest {
    const filterSensitiveLog: (obj: DescribeVpcPeeringConnectionsRequest) => any;
    const isa: (o: any) => o is DescribeVpcPeeringConnectionsRequest;
}
export interface DescribeVpcPeeringConnectionsResult {
    __type?: "DescribeVpcPeeringConnectionsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the VPC peering connections.</p>
     */
    VpcPeeringConnections?: VpcPeeringConnection[];
}
export declare namespace DescribeVpcPeeringConnectionsResult {
    const filterSensitiveLog: (obj: DescribeVpcPeeringConnectionsResult) => any;
    const isa: (o: any) => o is DescribeVpcPeeringConnectionsResult;
}
export interface DescribeVpcsRequest {
    __type?: "DescribeVpcsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>cidr</code> - The primary IPv4 CIDR block of the VPC. The CIDR block you
     *                     specify must exactly match the VPC's CIDR block for information to be returned
     *                     for the VPC. Must contain the slash followed by one or two digits (for example,
     *                     <code>/28</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>cidr-block-association.cidr-block</code> - An IPv4 CIDR block associated with the
     *                     VPC.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>cidr-block-association.association-id</code> - The association ID for
     *                     an IPv4 CIDR block associated with the VPC.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>cidr-block-association.state</code> - The state of an IPv4 CIDR block
     *                     associated with the VPC.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>dhcp-options-id</code> - The ID of a set of DHCP options.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ipv6-cidr-block-association.ipv6-cidr-block</code> - An IPv6 CIDR
     *                     block associated with the VPC.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ipv6-cidr-block-association.ipv6-pool</code> - The ID of the IPv6 address pool from which the IPv6 CIDR block is allocated.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ipv6-cidr-block-association.association-id</code> - The association
     *                     ID for an IPv6 CIDR block associated with the VPC.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>ipv6-cidr-block-association.state</code> - The state of an IPv6 CIDR
     *                     block associated with the VPC.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>isDefault</code> - Indicates whether the VPC is the default VPC.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>owner-id</code> - The ID of the AWS account that owns the VPC.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>state</code> - The state of the VPC (<code>pending</code> | <code>available</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>vpc-id</code> - The ID of the VPC.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>One or more VPC IDs.</p>
     * 		       <p>Default: Describes all your VPCs.</p>
     */
    VpcIds?: string[];
}
export declare namespace DescribeVpcsRequest {
    const filterSensitiveLog: (obj: DescribeVpcsRequest) => any;
    const isa: (o: any) => o is DescribeVpcsRequest;
}
export interface DescribeVpcsResult {
    __type?: "DescribeVpcsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about one or more VPCs.</p>
     */
    Vpcs?: Vpc[];
}
export declare namespace DescribeVpcsResult {
    const filterSensitiveLog: (obj: DescribeVpcsResult) => any;
    const isa: (o: any) => o is DescribeVpcsResult;
}
/**
 * <p>Contains the parameters for DescribeVpnConnections.</p>
 */
export interface DescribeVpnConnectionsRequest {
    __type?: "DescribeVpnConnectionsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>customer-gateway-configuration</code> - The configuration information for the customer gateway.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>customer-gateway-id</code> - The ID of a customer gateway associated with the VPN connection.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>state</code> - The state of the VPN connection (<code>pending</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>option.static-routes-only</code> - Indicates whether the connection has static routes only. Used for devices that do not support Border Gateway Protocol (BGP).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>route.destination-cidr-block</code> - The destination CIDR block. This corresponds to the subnet used in a customer data center.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>bgp-asn</code> - The BGP Autonomous System Number (ASN) associated with a BGP device.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>type</code> - The type of VPN connection. Currently the only supported type is <code>ipsec.1</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>vpn-connection-id</code> - The ID of the VPN connection.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>vpn-gateway-id</code> - The ID of a virtual private gateway associated with the VPN connection.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>transit-gateway-id</code> - The ID of a transit gateway associated with the VPN connection.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>One or more VPN connection IDs.</p>
     *         <p>Default: Describes your VPN connections.</p>
     */
    VpnConnectionIds?: string[];
}
export declare namespace DescribeVpnConnectionsRequest {
    const filterSensitiveLog: (obj: DescribeVpnConnectionsRequest) => any;
    const isa: (o: any) => o is DescribeVpnConnectionsRequest;
}
/**
 * <p>Contains the output of DescribeVpnConnections.</p>
 */
export interface DescribeVpnConnectionsResult {
    __type?: "DescribeVpnConnectionsResult";
    /**
     * <p>Information about one or more VPN connections.</p>
     */
    VpnConnections?: VpnConnection[];
}
export declare namespace DescribeVpnConnectionsResult {
    const filterSensitiveLog: (obj: DescribeVpnConnectionsResult) => any;
    const isa: (o: any) => o is DescribeVpnConnectionsResult;
}
/**
 * <p>Contains the parameters for DescribeVpnGateways.</p>
 */
export interface DescribeVpnGatewaysRequest {
    __type?: "DescribeVpnGatewaysRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     *         <ul>
     *             <li>
     * 				           <p>
     *                   <code>amazon-side-asn</code> - The Autonomous System Number (ASN) for the Amazon side of the gateway.</p>
     * 			         </li>
     *             <li>
     *                 <p>
     *                   <code>attachment.state</code> - The current state of the attachment between the gateway and the VPC (<code>attaching</code> | <code>attached</code> | <code>detaching</code> | <code>detached</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>attachment.vpc-id</code> - The ID of an attached VPC.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>availability-zone</code> - The Availability Zone for the virtual private gateway (if applicable).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>state</code> - The state of the virtual private gateway (<code>pending</code> | <code>available</code> | <code>deleting</code> | <code>deleted</code>).</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag</code>:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value.
     *     For example, to find all resources that have a tag with the key <code>Owner</code> and the value <code>TeamA</code>, specify <code>tag:Owner</code> for the filter name and <code>TeamA</code> for the filter value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>tag-key</code> - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>type</code> - The type of virtual private gateway. Currently the only supported type is <code>ipsec.1</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>vpn-gateway-id</code> - The ID of the virtual private gateway.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>One or more virtual private gateway IDs.</p>
     *         <p>Default: Describes all your virtual private gateways.</p>
     */
    VpnGatewayIds?: string[];
}
export declare namespace DescribeVpnGatewaysRequest {
    const filterSensitiveLog: (obj: DescribeVpnGatewaysRequest) => any;
    const isa: (o: any) => o is DescribeVpnGatewaysRequest;
}
/**
 * <p>Contains the output of DescribeVpnGateways.</p>
 */
export interface DescribeVpnGatewaysResult {
    __type?: "DescribeVpnGatewaysResult";
    /**
     * <p>Information about one or more virtual private gateways.</p>
     */
    VpnGateways?: VpnGateway[];
}
export declare namespace DescribeVpnGatewaysResult {
    const filterSensitiveLog: (obj: DescribeVpnGatewaysResult) => any;
    const isa: (o: any) => o is DescribeVpnGatewaysResult;
}
export interface DetachClassicLinkVpcRequest {
    __type?: "DetachClassicLinkVpcRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance to unlink from the VPC.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>The ID of the VPC to which the instance is linked.</p>
     */
    VpcId: string | undefined;
}
export declare namespace DetachClassicLinkVpcRequest {
    const filterSensitiveLog: (obj: DetachClassicLinkVpcRequest) => any;
    const isa: (o: any) => o is DetachClassicLinkVpcRequest;
}
export interface DetachClassicLinkVpcResult {
    __type?: "DetachClassicLinkVpcResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace DetachClassicLinkVpcResult {
    const filterSensitiveLog: (obj: DetachClassicLinkVpcResult) => any;
    const isa: (o: any) => o is DetachClassicLinkVpcResult;
}
export interface DetachInternetGatewayRequest {
    __type?: "DetachInternetGatewayRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the internet gateway.</p>
     */
    InternetGatewayId: string | undefined;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace DetachInternetGatewayRequest {
    const filterSensitiveLog: (obj: DetachInternetGatewayRequest) => any;
    const isa: (o: any) => o is DetachInternetGatewayRequest;
}
/**
 * <p>Contains the parameters for DetachNetworkInterface.</p>
 */
export interface DetachNetworkInterfaceRequest {
    __type?: "DetachNetworkInterfaceRequest";
    /**
     * <p>The ID of the attachment.</p>
     */
    AttachmentId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Specifies whether to force a detachment.</p>
     *         <note>
     *             <ul>
     *                <li>
     *                     <p>Use the <code>Force</code> parameter only as a last resort to detach a network interface from a failed instance. </p>
     *                 </li>
     *                <li>
     *                     <p>If you use the <code>Force</code> parameter to detach a network interface, you might not be able to attach a different network interface to the same index on the instance without first stopping and starting the instance.</p>
     *                 </li>
     *                <li>
     *                     <p>If you force the detachment of a network interface, the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html">instance metadata</a>
     *                         might not get updated. This means that the attributes associated
     *                         with the detached network interface might still be visible. The
     *                         instance metadata will get updated when you stop and start the
     *                         instance.</p>
     *                 </li>
     *             </ul>
     *         </note>
     */
    Force?: boolean;
}
export declare namespace DetachNetworkInterfaceRequest {
    const filterSensitiveLog: (obj: DetachNetworkInterfaceRequest) => any;
    const isa: (o: any) => o is DetachNetworkInterfaceRequest;
}
export interface DetachVolumeRequest {
    __type?: "DetachVolumeRequest";
    /**
     * <p>The device name.</p>
     */
    Device?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Forces detachment if the previous detachment attempt did not occur cleanly (for example,
     *       logging into an instance, unmounting the volume, and detaching normally). This option can lead
     *       to data loss or a corrupted file system. Use this option only as a last resort to detach a
     *       volume from a failed instance. The instance won't have an opportunity to flush file system
     *       caches or file system metadata. If you use this option, you must perform file system check and
     *       repair procedures.</p>
     */
    Force?: boolean;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The ID of the volume.</p>
     */
    VolumeId: string | undefined;
}
export declare namespace DetachVolumeRequest {
    const filterSensitiveLog: (obj: DetachVolumeRequest) => any;
    const isa: (o: any) => o is DetachVolumeRequest;
}
/**
 * <p>Contains the parameters for DetachVpnGateway.</p>
 */
export interface DetachVpnGatewayRequest {
    __type?: "DetachVpnGatewayRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
    /**
     * <p>The ID of the virtual private gateway.</p>
     */
    VpnGatewayId: string | undefined;
}
export declare namespace DetachVpnGatewayRequest {
    const filterSensitiveLog: (obj: DetachVpnGatewayRequest) => any;
    const isa: (o: any) => o is DetachVpnGatewayRequest;
}
export declare type DeviceType = "ebs" | "instance-store";
/**
 * <p>Describes a DHCP configuration option.</p>
 */
export interface DhcpConfiguration {
    __type?: "DhcpConfiguration";
    /**
     * <p>The name of a DHCP option.</p>
     */
    Key?: string;
    /**
     * <p>One or more values for the DHCP option.</p>
     */
    Values?: AttributeValue[];
}
export declare namespace DhcpConfiguration {
    const filterSensitiveLog: (obj: DhcpConfiguration) => any;
    const isa: (o: any) => o is DhcpConfiguration;
}
/**
 * <p>Describes a set of DHCP options.</p>
 */
export interface DhcpOptions {
    __type?: "DhcpOptions";
    /**
     * <p>One or more DHCP options in the set.</p>
     */
    DhcpConfigurations?: DhcpConfiguration[];
    /**
     * <p>The ID of the set of DHCP options.</p>
     */
    DhcpOptionsId?: string;
    /**
     * <p>The ID of the AWS account that owns the DHCP options set.</p>
     */
    OwnerId?: string;
    /**
     * <p>Any tags assigned to the DHCP options set.</p>
     */
    Tags?: Tag[];
}
export declare namespace DhcpOptions {
    const filterSensitiveLog: (obj: DhcpOptions) => any;
    const isa: (o: any) => o is DhcpOptions;
}
/**
 * <p>Describes an Active Directory.</p>
 */
export interface DirectoryServiceAuthentication {
    __type?: "DirectoryServiceAuthentication";
    /**
     * <p>The ID of the Active Directory used for authentication.</p>
     */
    DirectoryId?: string;
}
export declare namespace DirectoryServiceAuthentication {
    const filterSensitiveLog: (obj: DirectoryServiceAuthentication) => any;
    const isa: (o: any) => o is DirectoryServiceAuthentication;
}
/**
 * <p>Describes the Active Directory to be used for client authentication.</p>
 */
export interface DirectoryServiceAuthenticationRequest {
    __type?: "DirectoryServiceAuthenticationRequest";
    /**
     * <p>The ID of the Active Directory to be used for authentication.</p>
     */
    DirectoryId?: string;
}
export declare namespace DirectoryServiceAuthenticationRequest {
    const filterSensitiveLog: (obj: DirectoryServiceAuthenticationRequest) => any;
    const isa: (o: any) => o is DirectoryServiceAuthenticationRequest;
}
export interface DisableEbsEncryptionByDefaultRequest {
    __type?: "DisableEbsEncryptionByDefaultRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DisableEbsEncryptionByDefaultRequest {
    const filterSensitiveLog: (obj: DisableEbsEncryptionByDefaultRequest) => any;
    const isa: (o: any) => o is DisableEbsEncryptionByDefaultRequest;
}
export interface DisableEbsEncryptionByDefaultResult {
    __type?: "DisableEbsEncryptionByDefaultResult";
    /**
     * <p>The updated status of encryption by default.</p>
     */
    EbsEncryptionByDefault?: boolean;
}
export declare namespace DisableEbsEncryptionByDefaultResult {
    const filterSensitiveLog: (obj: DisableEbsEncryptionByDefaultResult) => any;
    const isa: (o: any) => o is DisableEbsEncryptionByDefaultResult;
}
/**
 * <p>Contains information about the errors that occurred when disabling fast snapshot restores.</p>
 */
export interface DisableFastSnapshotRestoreErrorItem {
    __type?: "DisableFastSnapshotRestoreErrorItem";
    /**
     * <p>The errors.</p>
     */
    FastSnapshotRestoreStateErrors?: DisableFastSnapshotRestoreStateErrorItem[];
    /**
     * <p>The ID of the snapshot.</p>
     */
    SnapshotId?: string;
}
export declare namespace DisableFastSnapshotRestoreErrorItem {
    const filterSensitiveLog: (obj: DisableFastSnapshotRestoreErrorItem) => any;
    const isa: (o: any) => o is DisableFastSnapshotRestoreErrorItem;
}
export interface DisableFastSnapshotRestoresRequest {
    __type?: "DisableFastSnapshotRestoresRequest";
    /**
     * <p>One or more Availability Zones. For example, <code>us-east-2a</code>.</p>
     */
    AvailabilityZones: string[] | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of one or more snapshots. For example, <code>snap-1234567890abcdef0</code>.</p>
     */
    SourceSnapshotIds: string[] | undefined;
}
export declare namespace DisableFastSnapshotRestoresRequest {
    const filterSensitiveLog: (obj: DisableFastSnapshotRestoresRequest) => any;
    const isa: (o: any) => o is DisableFastSnapshotRestoresRequest;
}
export interface DisableFastSnapshotRestoresResult {
    __type?: "DisableFastSnapshotRestoresResult";
    /**
     * <p>Information about the snapshots for which fast snapshot restores were successfully disabled.</p>
     */
    Successful?: DisableFastSnapshotRestoreSuccessItem[];
    /**
     * <p>Information about the snapshots for which fast snapshot restores could not be disabled.</p>
     */
    Unsuccessful?: DisableFastSnapshotRestoreErrorItem[];
}
export declare namespace DisableFastSnapshotRestoresResult {
    const filterSensitiveLog: (obj: DisableFastSnapshotRestoresResult) => any;
    const isa: (o: any) => o is DisableFastSnapshotRestoresResult;
}
/**
 * <p>Describes an error that occurred when disabling fast snapshot restores.</p>
 */
export interface DisableFastSnapshotRestoreStateError {
    __type?: "DisableFastSnapshotRestoreStateError";
    /**
     * <p>The error code.</p>
     */
    Code?: string;
    /**
     * <p>The error message.</p>
     */
    Message?: string;
}
export declare namespace DisableFastSnapshotRestoreStateError {
    const filterSensitiveLog: (obj: DisableFastSnapshotRestoreStateError) => any;
    const isa: (o: any) => o is DisableFastSnapshotRestoreStateError;
}
/**
 * <p>Contains information about an error that occurred when disabling fast snapshot restores.</p>
 */
export interface DisableFastSnapshotRestoreStateErrorItem {
    __type?: "DisableFastSnapshotRestoreStateErrorItem";
    /**
     * <p>The Availability Zone.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The error.</p>
     */
    Error?: DisableFastSnapshotRestoreStateError;
}
export declare namespace DisableFastSnapshotRestoreStateErrorItem {
    const filterSensitiveLog: (obj: DisableFastSnapshotRestoreStateErrorItem) => any;
    const isa: (o: any) => o is DisableFastSnapshotRestoreStateErrorItem;
}
/**
 * <p>Describes fast snapshot restores that were successfully disabled.</p>
 */
export interface DisableFastSnapshotRestoreSuccessItem {
    __type?: "DisableFastSnapshotRestoreSuccessItem";
    /**
     * <p>The Availability Zone.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The time at which fast snapshot restores entered the <code>disabled</code> state.</p>
     */
    DisabledTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>disabling</code> state.</p>
     */
    DisablingTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>enabled</code> state.</p>
     */
    EnabledTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>enabling</code> state.</p>
     */
    EnablingTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>optimizing</code> state.</p>
     */
    OptimizingTime?: Date;
    /**
     * <p>The alias of the snapshot owner.</p>
     */
    OwnerAlias?: string;
    /**
     * <p>The ID of the AWS account that owns the snapshot.</p>
     */
    OwnerId?: string;
    /**
     * <p>The ID of the snapshot.</p>
     */
    SnapshotId?: string;
    /**
     * <p>The state of fast snapshot restores for the snapshot.</p>
     */
    State?: FastSnapshotRestoreStateCode | string;
    /**
     * <p>The reason for the state transition. The possible values are as follows:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>Client.UserInitiated</code> - The state successfully transitioned to <code>enabling</code> or
     *           <code>disabling</code>.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>Client.UserInitiated - Lifecycle state transition</code> - The state successfully transitioned
     *           to <code>optimizing</code>, <code>enabled</code>, or <code>disabled</code>.</p>
     *             </li>
     *          </ul>
     */
    StateTransitionReason?: string;
}
export declare namespace DisableFastSnapshotRestoreSuccessItem {
    const filterSensitiveLog: (obj: DisableFastSnapshotRestoreSuccessItem) => any;
    const isa: (o: any) => o is DisableFastSnapshotRestoreSuccessItem;
}
export interface DisableTransitGatewayRouteTablePropagationRequest {
    __type?: "DisableTransitGatewayRouteTablePropagationRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
    /**
     * <p>The ID of the propagation route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace DisableTransitGatewayRouteTablePropagationRequest {
    const filterSensitiveLog: (obj: DisableTransitGatewayRouteTablePropagationRequest) => any;
    const isa: (o: any) => o is DisableTransitGatewayRouteTablePropagationRequest;
}
export interface DisableTransitGatewayRouteTablePropagationResult {
    __type?: "DisableTransitGatewayRouteTablePropagationResult";
    /**
     * <p>Information about route propagation.</p>
     */
    Propagation?: TransitGatewayPropagation;
}
export declare namespace DisableTransitGatewayRouteTablePropagationResult {
    const filterSensitiveLog: (obj: DisableTransitGatewayRouteTablePropagationResult) => any;
    const isa: (o: any) => o is DisableTransitGatewayRouteTablePropagationResult;
}
/**
 * <p>Contains the parameters for DisableVgwRoutePropagation.</p>
 */
export interface DisableVgwRoutePropagationRequest {
    __type?: "DisableVgwRoutePropagationRequest";
    /**
     * <p>The ID of the virtual private gateway.</p>
     */
    GatewayId: string | undefined;
    /**
     * <p>The ID of the route table.</p>
     */
    RouteTableId: string | undefined;
}
export declare namespace DisableVgwRoutePropagationRequest {
    const filterSensitiveLog: (obj: DisableVgwRoutePropagationRequest) => any;
    const isa: (o: any) => o is DisableVgwRoutePropagationRequest;
}
export interface DisableVpcClassicLinkDnsSupportRequest {
    __type?: "DisableVpcClassicLinkDnsSupportRequest";
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace DisableVpcClassicLinkDnsSupportRequest {
    const filterSensitiveLog: (obj: DisableVpcClassicLinkDnsSupportRequest) => any;
    const isa: (o: any) => o is DisableVpcClassicLinkDnsSupportRequest;
}
export interface DisableVpcClassicLinkDnsSupportResult {
    __type?: "DisableVpcClassicLinkDnsSupportResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace DisableVpcClassicLinkDnsSupportResult {
    const filterSensitiveLog: (obj: DisableVpcClassicLinkDnsSupportResult) => any;
    const isa: (o: any) => o is DisableVpcClassicLinkDnsSupportResult;
}
export interface DisableVpcClassicLinkRequest {
    __type?: "DisableVpcClassicLinkRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace DisableVpcClassicLinkRequest {
    const filterSensitiveLog: (obj: DisableVpcClassicLinkRequest) => any;
    const isa: (o: any) => o is DisableVpcClassicLinkRequest;
}
export interface DisableVpcClassicLinkResult {
    __type?: "DisableVpcClassicLinkResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace DisableVpcClassicLinkResult {
    const filterSensitiveLog: (obj: DisableVpcClassicLinkResult) => any;
    const isa: (o: any) => o is DisableVpcClassicLinkResult;
}
export interface DisassociateAddressRequest {
    __type?: "DisassociateAddressRequest";
    /**
     * <p>[EC2-VPC] The association ID. Required for EC2-VPC.</p>
     */
    AssociationId?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>[EC2-Classic] The Elastic IP address. Required for EC2-Classic.</p>
     */
    PublicIp?: string;
}
export declare namespace DisassociateAddressRequest {
    const filterSensitiveLog: (obj: DisassociateAddressRequest) => any;
    const isa: (o: any) => o is DisassociateAddressRequest;
}
export interface DisassociateClientVpnTargetNetworkRequest {
    __type?: "DisassociateClientVpnTargetNetworkRequest";
    /**
     * <p>The ID of the target network association.</p>
     */
    AssociationId: string | undefined;
    /**
     * <p>The ID of the Client VPN endpoint from which to disassociate the target network.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DisassociateClientVpnTargetNetworkRequest {
    const filterSensitiveLog: (obj: DisassociateClientVpnTargetNetworkRequest) => any;
    const isa: (o: any) => o is DisassociateClientVpnTargetNetworkRequest;
}
export interface DisassociateClientVpnTargetNetworkResult {
    __type?: "DisassociateClientVpnTargetNetworkResult";
    /**
     * <p>The ID of the target network association.</p>
     */
    AssociationId?: string;
    /**
     * <p>The current state of the target network association.</p>
     */
    Status?: AssociationStatus;
}
export declare namespace DisassociateClientVpnTargetNetworkResult {
    const filterSensitiveLog: (obj: DisassociateClientVpnTargetNetworkResult) => any;
    const isa: (o: any) => o is DisassociateClientVpnTargetNetworkResult;
}
export interface DisassociateIamInstanceProfileRequest {
    __type?: "DisassociateIamInstanceProfileRequest";
    /**
     * <p>The ID of the IAM instance profile association.</p>
     */
    AssociationId: string | undefined;
}
export declare namespace DisassociateIamInstanceProfileRequest {
    const filterSensitiveLog: (obj: DisassociateIamInstanceProfileRequest) => any;
    const isa: (o: any) => o is DisassociateIamInstanceProfileRequest;
}
export interface DisassociateIamInstanceProfileResult {
    __type?: "DisassociateIamInstanceProfileResult";
    /**
     * <p>Information about the IAM instance profile association.</p>
     */
    IamInstanceProfileAssociation?: IamInstanceProfileAssociation;
}
export declare namespace DisassociateIamInstanceProfileResult {
    const filterSensitiveLog: (obj: DisassociateIamInstanceProfileResult) => any;
    const isa: (o: any) => o is DisassociateIamInstanceProfileResult;
}
export interface DisassociateRouteTableRequest {
    __type?: "DisassociateRouteTableRequest";
    /**
     * <p>The association ID representing the current association between the route table and subnet.</p>
     */
    AssociationId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace DisassociateRouteTableRequest {
    const filterSensitiveLog: (obj: DisassociateRouteTableRequest) => any;
    const isa: (o: any) => o is DisassociateRouteTableRequest;
}
export interface DisassociateSubnetCidrBlockRequest {
    __type?: "DisassociateSubnetCidrBlockRequest";
    /**
     * <p>The association ID for the CIDR block.</p>
     */
    AssociationId: string | undefined;
}
export declare namespace DisassociateSubnetCidrBlockRequest {
    const filterSensitiveLog: (obj: DisassociateSubnetCidrBlockRequest) => any;
    const isa: (o: any) => o is DisassociateSubnetCidrBlockRequest;
}
export interface DisassociateSubnetCidrBlockResult {
    __type?: "DisassociateSubnetCidrBlockResult";
    /**
     * <p>Information about the IPv6 CIDR block association.</p>
     */
    Ipv6CidrBlockAssociation?: SubnetIpv6CidrBlockAssociation;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId?: string;
}
export declare namespace DisassociateSubnetCidrBlockResult {
    const filterSensitiveLog: (obj: DisassociateSubnetCidrBlockResult) => any;
    const isa: (o: any) => o is DisassociateSubnetCidrBlockResult;
}
export interface DisassociateTransitGatewayMulticastDomainRequest {
    __type?: "DisassociateTransitGatewayMulticastDomainRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the subnets;</p>
     */
    SubnetIds?: string[];
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace DisassociateTransitGatewayMulticastDomainRequest {
    const filterSensitiveLog: (obj: DisassociateTransitGatewayMulticastDomainRequest) => any;
    const isa: (o: any) => o is DisassociateTransitGatewayMulticastDomainRequest;
}
export interface DisassociateTransitGatewayMulticastDomainResult {
    __type?: "DisassociateTransitGatewayMulticastDomainResult";
    /**
     * <p>Information about the association.</p>
     */
    Associations?: TransitGatewayMulticastDomainAssociations;
}
export declare namespace DisassociateTransitGatewayMulticastDomainResult {
    const filterSensitiveLog: (obj: DisassociateTransitGatewayMulticastDomainResult) => any;
    const isa: (o: any) => o is DisassociateTransitGatewayMulticastDomainResult;
}
export interface DisassociateTransitGatewayRouteTableRequest {
    __type?: "DisassociateTransitGatewayRouteTableRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
    /**
     * <p>The ID of the transit gateway route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace DisassociateTransitGatewayRouteTableRequest {
    const filterSensitiveLog: (obj: DisassociateTransitGatewayRouteTableRequest) => any;
    const isa: (o: any) => o is DisassociateTransitGatewayRouteTableRequest;
}
export interface DisassociateTransitGatewayRouteTableResult {
    __type?: "DisassociateTransitGatewayRouteTableResult";
    /**
     * <p>Information about the association.</p>
     */
    Association?: TransitGatewayAssociation;
}
export declare namespace DisassociateTransitGatewayRouteTableResult {
    const filterSensitiveLog: (obj: DisassociateTransitGatewayRouteTableResult) => any;
    const isa: (o: any) => o is DisassociateTransitGatewayRouteTableResult;
}
export interface DisassociateVpcCidrBlockRequest {
    __type?: "DisassociateVpcCidrBlockRequest";
    /**
     * <p>The association ID for the CIDR block.</p>
     */
    AssociationId: string | undefined;
}
export declare namespace DisassociateVpcCidrBlockRequest {
    const filterSensitiveLog: (obj: DisassociateVpcCidrBlockRequest) => any;
    const isa: (o: any) => o is DisassociateVpcCidrBlockRequest;
}
export interface DisassociateVpcCidrBlockResult {
    __type?: "DisassociateVpcCidrBlockResult";
    /**
     * <p>Information about the IPv4 CIDR block association.</p>
     */
    CidrBlockAssociation?: VpcCidrBlockAssociation;
    /**
     * <p>Information about the IPv6 CIDR block association.</p>
     */
    Ipv6CidrBlockAssociation?: VpcIpv6CidrBlockAssociation;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace DisassociateVpcCidrBlockResult {
    const filterSensitiveLog: (obj: DisassociateVpcCidrBlockResult) => any;
    const isa: (o: any) => o is DisassociateVpcCidrBlockResult;
}
/**
 * <p>Describes a disk image.</p>
 */
export interface DiskImage {
    __type?: "DiskImage";
    /**
     * <p>A description of the disk image.</p>
     */
    Description?: string;
    /**
     * <p>Information about the disk image.</p>
     */
    Image?: DiskImageDetail;
    /**
     * <p>Information about the volume.</p>
     */
    Volume?: VolumeDetail;
}
export declare namespace DiskImage {
    const filterSensitiveLog: (obj: DiskImage) => any;
    const isa: (o: any) => o is DiskImage;
}
/**
 * <p>Describes a disk image.</p>
 */
export interface DiskImageDescription {
    __type?: "DiskImageDescription";
    /**
     * <p>The checksum computed for the disk image.</p>
     */
    Checksum?: string;
    /**
     * <p>The disk image format.</p>
     */
    Format?: DiskImageFormat | string;
    /**
     * <p>A presigned URL for the import manifest stored in Amazon S3. For information about creating a presigned URL for
     *    an Amazon S3 object, read the "Query String Request Authentication Alternative" section of the <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/RESTAuthentication.html">Authenticating REST Requests</a> topic in
     *    the <i>Amazon Simple Storage Service Developer Guide</i>.</p>
     *          <p>For information about the import manifest referenced by this API action, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/manifest.html">VM Import Manifest</a>.</p>
     */
    ImportManifestUrl?: string;
    /**
     * <p>The size of the disk image, in GiB.</p>
     */
    Size?: number;
}
export declare namespace DiskImageDescription {
    const filterSensitiveLog: (obj: DiskImageDescription) => any;
    const isa: (o: any) => o is DiskImageDescription;
}
/**
 * <p>Describes a disk image.</p>
 */
export interface DiskImageDetail {
    __type?: "DiskImageDetail";
    /**
     * <p>The size of the disk image, in GiB.</p>
     */
    Bytes: number | undefined;
    /**
     * <p>The disk image format.</p>
     */
    Format: DiskImageFormat | string | undefined;
    /**
     * <p>A presigned URL for the import manifest stored in Amazon S3 and presented here as an Amazon S3 presigned URL.
     *    For information about creating a presigned URL for an Amazon S3 object, read the "Query String Request Authentication
     *    Alternative" section of the <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/RESTAuthentication.html">Authenticating REST Requests</a> topic in the <i>Amazon Simple Storage Service Developer
     *     Guide</i>.</p>
     *          <p>For information about the import manifest referenced by this API action, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/manifest.html">VM Import Manifest</a>.</p>
     */
    ImportManifestUrl: string | undefined;
}
export declare namespace DiskImageDetail {
    const filterSensitiveLog: (obj: DiskImageDetail) => any;
    const isa: (o: any) => o is DiskImageDetail;
}
export declare type DiskImageFormat = "RAW" | "VHD" | "VMDK";
/**
 * <p>Describes a disk image volume.</p>
 */
export interface DiskImageVolumeDescription {
    __type?: "DiskImageVolumeDescription";
    /**
     * <p>The volume identifier.</p>
     */
    Id?: string;
    /**
     * <p>The size of the volume, in GiB.</p>
     */
    Size?: number;
}
export declare namespace DiskImageVolumeDescription {
    const filterSensitiveLog: (obj: DiskImageVolumeDescription) => any;
    const isa: (o: any) => o is DiskImageVolumeDescription;
}
/**
 * <p>Describes the disk.</p>
 */
export interface DiskInfo {
    __type?: "DiskInfo";
    /**
     * <p>The number of disks with this configuration.</p>
     */
    Count?: number;
    /**
     * <p>The size of the disk in GB.</p>
     */
    SizeInGB?: number;
    /**
     * <p>The type of disk.</p>
     */
    Type?: DiskType | string;
}
export declare namespace DiskInfo {
    const filterSensitiveLog: (obj: DiskInfo) => any;
    const isa: (o: any) => o is DiskInfo;
}
export declare type DiskType = "hdd" | "ssd";
/**
 * <p>Describes a DNS entry.</p>
 */
export interface DnsEntry {
    __type?: "DnsEntry";
    /**
     * <p>The DNS name.</p>
     */
    DnsName?: string;
    /**
     * <p>The ID of the private hosted zone.</p>
     */
    HostedZoneId?: string;
}
export declare namespace DnsEntry {
    const filterSensitiveLog: (obj: DnsEntry) => any;
    const isa: (o: any) => o is DnsEntry;
}
export declare enum DnsNameState {
    Failed = "failed",
    PendingVerification = "pendingVerification",
    Verified = "verified"
}
/**
 * <p>Information about the DNS server to be used.</p>
 */
export interface DnsServersOptionsModifyStructure {
    __type?: "DnsServersOptionsModifyStructure";
    /**
     * <p>The IPv4 address range, in CIDR notation, of the DNS servers to be used. You can specify up to
     * 			two DNS servers. Ensure that the DNS servers can be reached by the clients. The specified values
     * 			overwrite the existing values.</p>
     */
    CustomDnsServers?: string[];
    /**
     * <p>Indicates whether DNS servers should be used. Specify <code>False</code> to delete the existing DNS
     * 			servers.</p>
     */
    Enabled?: boolean;
}
export declare namespace DnsServersOptionsModifyStructure {
    const filterSensitiveLog: (obj: DnsServersOptionsModifyStructure) => any;
    const isa: (o: any) => o is DnsServersOptionsModifyStructure;
}
export declare type DnsSupportValue = "disable" | "enable";
export declare type DomainType = "standard" | "vpc";
/**
 * <p>Describes a block device for an EBS volume.</p>
 */
export interface EbsBlockDevice {
    __type?: "EbsBlockDevice";
    /**
     * <p>Indicates whether the EBS volume is deleted on instance termination. For more
     *             information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/terminating-instances.html#preserving-volumes-on-termination">Preserving Amazon EBS Volumes on Instance Termination</a> in the
     *             Amazon Elastic Compute Cloud User Guide.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>Indicates whether the encryption state of an EBS volume is changed while being
     *             restored from a backing snapshot.
     *             The effect of setting the encryption state to <code>true</code> depends on
     * the volume origin (new or from a snapshot), starting encryption state, ownership, and whether encryption by default is enabled.
     *             For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html#encryption-parameters">Amazon EBS Encryption</a>
     *             in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *         <p>In no case can you remove encryption from an encrypted volume.</p>
     *         <p>Encrypted volumes can only be attached to instances that support Amazon EBS
     *             encryption. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html#EBSEncryption_supported_instances">Supported Instance Types</a>.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>The number of I/O operations per second (IOPS) that the volume supports. For
     *             <code>io1</code> volumes, this represents the number of IOPS that are provisioned
     *             for the volume. For <code>gp2</code> volumes, this represents the baseline performance
     *             of the volume and the rate at which the volume accumulates I/O credits for bursting. For
     *             more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html">Amazon EBS Volume Types</a> in the
     *             <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *         <p>Constraints: Range is 100-16,000 IOPS for <code>gp2</code> volumes and
     *             100 to 64,000IOPS for <code>io1</code> volumes in
     *             most Regions. Maximum <code>io1</code> IOPS of 64,000 is guaranteed
     *             only on <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html#ec2-nitro-instances">Nitro-based
     *                 instances</a>. Other instance families guarantee performance up to
     *             32,000 IOPS. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html">Amazon EBS Volume
     *                 Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *         <p>Condition: This parameter is required for requests to create <code>io1</code> volumes;
     *             it is not used in requests to create <code>gp2</code>, <code>st1</code>,
     *                 <code>sc1</code>, or <code>standard</code> volumes.</p>
     */
    Iops?: number;
    /**
     * <p>Identifier (key ID, key alias, ID ARN, or alias ARN) for a customer managed CMK under
     *             which the EBS volume is encrypted.</p>
     *         <p>This parameter is only supported on <code>BlockDeviceMapping</code> objects called by
     *                 <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_RunInstances.html">RunInstances</a>, <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_RequestSpotFleet.html">RequestSpotFleet</a>,
     *             and <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_RequestSpotInstances.html">RequestSpotInstances</a>.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>The ID of the snapshot.</p>
     */
    SnapshotId?: string;
    /**
     * <p>The size of the volume, in GiB.</p>
     *         <p>Default: If you're creating the volume from a snapshot and don't specify a volume
     *             size, the default is the snapshot size.</p>
     *         <p>Constraints: 1-16384 for General Purpose SSD (<code>gp2</code>), 4-16384 for
     *             Provisioned IOPS SSD (<code>io1</code>), 500-16384 for Throughput Optimized HDD
     *                 (<code>st1</code>), 500-16384 for Cold HDD (<code>sc1</code>), and 1-1024 for
     *             Magnetic (<code>standard</code>) volumes. If you specify a snapshot, the volume size
     *             must be equal to or larger than the snapshot size.</p>
     */
    VolumeSize?: number;
    /**
     * <p>The volume type. If you set the type to <code>io1</code>, you must also specify
     *         	the <b>Iops</b> parameter. If you set the type to <code>gp2</code>,
     *         	<code>st1</code>, <code>sc1</code>, or <code>standard</code>, you must omit
     *         	the <b>Iops</b> parameter.</p>
     *         <p>Default: <code>gp2</code>
     *          </p>
     */
    VolumeType?: VolumeType | string;
}
export declare namespace EbsBlockDevice {
    const filterSensitiveLog: (obj: EbsBlockDevice) => any;
    const isa: (o: any) => o is EbsBlockDevice;
}
export declare type EbsEncryptionSupport = "supported" | "unsupported";
/**
 * <p>Describes the Amazon EBS features supported by the instance type.</p>
 */
export interface EbsInfo {
    __type?: "EbsInfo";
    /**
     * <p>Indicates that the instance type is Amazon EBS-optimized. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSOptimized.html">Amazon EBS-Optimized Instances</a> in <i>Amazon EC2 User Guide for
     *     Linux Instances</i>.</p>
     */
    EbsOptimizedSupport?: EbsOptimizedSupport | string;
    /**
     * <p>Indicates whether Amazon EBS encryption is supported.</p>
     */
    EncryptionSupport?: EbsEncryptionSupport | string;
}
export declare namespace EbsInfo {
    const filterSensitiveLog: (obj: EbsInfo) => any;
    const isa: (o: any) => o is EbsInfo;
}
/**
 * <p>Describes a parameter used to set up an EBS volume in a block device mapping.</p>
 */
export interface EbsInstanceBlockDevice {
    __type?: "EbsInstanceBlockDevice";
    /**
     * <p>The time stamp when the attachment initiated.</p>
     */
    AttachTime?: Date;
    /**
     * <p>Indicates whether the volume is deleted on instance termination.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>The attachment state.</p>
     */
    Status?: AttachmentStatus | string;
    /**
     * <p>The ID of the EBS volume.</p>
     */
    VolumeId?: string;
}
export declare namespace EbsInstanceBlockDevice {
    const filterSensitiveLog: (obj: EbsInstanceBlockDevice) => any;
    const isa: (o: any) => o is EbsInstanceBlockDevice;
}
/**
 * <p>Describes information used to set up an EBS volume specified in a block device
 *             mapping.</p>
 */
export interface EbsInstanceBlockDeviceSpecification {
    __type?: "EbsInstanceBlockDeviceSpecification";
    /**
     * <p>Indicates whether the volume is deleted on instance termination.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>The ID of the EBS volume.</p>
     */
    VolumeId?: string;
}
export declare namespace EbsInstanceBlockDeviceSpecification {
    const filterSensitiveLog: (obj: EbsInstanceBlockDeviceSpecification) => any;
    const isa: (o: any) => o is EbsInstanceBlockDeviceSpecification;
}
export declare type EbsOptimizedSupport = "default" | "supported" | "unsupported";
/**
 * <p>Describes an egress-only internet gateway.</p>
 */
export interface EgressOnlyInternetGateway {
    __type?: "EgressOnlyInternetGateway";
    /**
     * <p>Information about the attachment of the egress-only internet gateway.</p>
     */
    Attachments?: InternetGatewayAttachment[];
    /**
     * <p>The ID of the egress-only internet gateway.</p>
     */
    EgressOnlyInternetGatewayId?: string;
    /**
     * <p>The tags assigned to the egress-only internet gateway.</p>
     */
    Tags?: Tag[];
}
export declare namespace EgressOnlyInternetGateway {
    const filterSensitiveLog: (obj: EgressOnlyInternetGateway) => any;
    const isa: (o: any) => o is EgressOnlyInternetGateway;
}
/**
 * <p>Describes the association between an instance and an Elastic Graphics accelerator.</p>
 */
export interface ElasticGpuAssociation {
    __type?: "ElasticGpuAssociation";
    /**
     * <p>The ID of the association.</p>
     */
    ElasticGpuAssociationId?: string;
    /**
     * <p>The state of the association between the instance and the
     *             Elastic Graphics accelerator.</p>
     */
    ElasticGpuAssociationState?: string;
    /**
     * <p>The time the Elastic Graphics accelerator was associated with the instance.</p>
     */
    ElasticGpuAssociationTime?: string;
    /**
     * <p>The ID of the Elastic Graphics accelerator.</p>
     */
    ElasticGpuId?: string;
}
export declare namespace ElasticGpuAssociation {
    const filterSensitiveLog: (obj: ElasticGpuAssociation) => any;
    const isa: (o: any) => o is ElasticGpuAssociation;
}
/**
 * <p>Describes the status of an Elastic Graphics accelerator.</p>
 */
export interface ElasticGpuHealth {
    __type?: "ElasticGpuHealth";
    /**
     * <p>The health status.</p>
     */
    Status?: ElasticGpuStatus | string;
}
export declare namespace ElasticGpuHealth {
    const filterSensitiveLog: (obj: ElasticGpuHealth) => any;
    const isa: (o: any) => o is ElasticGpuHealth;
}
/**
 * <p>Describes an Elastic Graphics accelerator.</p>
 */
export interface ElasticGpus {
    __type?: "ElasticGpus";
    /**
     * <p>The Availability Zone in the which the Elastic Graphics accelerator resides.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The status of the Elastic Graphics accelerator.</p>
     */
    ElasticGpuHealth?: ElasticGpuHealth;
    /**
     * <p>The ID of the Elastic Graphics accelerator.</p>
     */
    ElasticGpuId?: string;
    /**
     * <p>The state of the Elastic Graphics accelerator.</p>
     */
    ElasticGpuState?: ElasticGpuState | string;
    /**
     * <p>The type of Elastic Graphics accelerator.</p>
     */
    ElasticGpuType?: string;
    /**
     * <p>The ID of the instance to which the Elastic Graphics accelerator is attached.</p>
     */
    InstanceId?: string;
    /**
     * <p>The tags assigned to the Elastic Graphics accelerator.</p>
     */
    Tags?: Tag[];
}
export declare namespace ElasticGpus {
    const filterSensitiveLog: (obj: ElasticGpus) => any;
    const isa: (o: any) => o is ElasticGpus;
}
/**
 * <p>A specification for an Elastic Graphics accelerator.</p>
 */
export interface ElasticGpuSpecification {
    __type?: "ElasticGpuSpecification";
    /**
     * <p>The type of Elastic Graphics accelerator. For more information about the values to specify for
     *             <code>Type</code>, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/elastic-graphics.html#elastic-graphics-basics">Elastic Graphics Basics</a>, specifically the Elastic Graphics accelerator column, in the <i>Amazon Elastic Compute Cloud User Guide for Windows
     *                 Instances</i>.</p>
     */
    Type: string | undefined;
}
export declare namespace ElasticGpuSpecification {
    const filterSensitiveLog: (obj: ElasticGpuSpecification) => any;
    const isa: (o: any) => o is ElasticGpuSpecification;
}
/**
 * <p>Describes an elastic GPU.</p>
 */
export interface ElasticGpuSpecificationResponse {
    __type?: "ElasticGpuSpecificationResponse";
    /**
     * <p>The elastic GPU type.</p>
     */
    Type?: string;
}
export declare namespace ElasticGpuSpecificationResponse {
    const filterSensitiveLog: (obj: ElasticGpuSpecificationResponse) => any;
    const isa: (o: any) => o is ElasticGpuSpecificationResponse;
}
export declare enum ElasticGpuState {
    Attached = "ATTACHED"
}
export declare enum ElasticGpuStatus {
    Impaired = "IMPAIRED",
    Ok = "OK"
}
/**
 * <p>
 *            Describes an elastic inference accelerator.
 *         </p>
 */
export interface ElasticInferenceAccelerator {
    __type?: "ElasticInferenceAccelerator";
    /**
     * <p>
     *             The number of elastic inference accelerators to attach to the instance.
     *         </p>
     *          <p>Default: 1</p>
     */
    Count?: number;
    /**
     * <p>
     *         	The type of elastic inference accelerator. The possible values are <code>eia1.medium</code>, <code>eia1.large</code>, and <code>eia1.xlarge</code>.
     *         </p>
     */
    Type: string | undefined;
}
export declare namespace ElasticInferenceAccelerator {
    const filterSensitiveLog: (obj: ElasticInferenceAccelerator) => any;
    const isa: (o: any) => o is ElasticInferenceAccelerator;
}
/**
 * <p>
 *             Describes the association between an instance and an elastic inference accelerator.
 *         </p>
 */
export interface ElasticInferenceAcceleratorAssociation {
    __type?: "ElasticInferenceAcceleratorAssociation";
    /**
     * <p>
     *             The Amazon Resource Name (ARN) of the elastic inference accelerator.
     *         </p>
     */
    ElasticInferenceAcceleratorArn?: string;
    /**
     * <p>
     *             The ID of the association.
     *         </p>
     */
    ElasticInferenceAcceleratorAssociationId?: string;
    /**
     * <p>
     *             The state of the elastic inference accelerator.
     *         </p>
     */
    ElasticInferenceAcceleratorAssociationState?: string;
    /**
     * <p>
     *             The time at which the elastic inference accelerator is associated with an instance.
     *         </p>
     */
    ElasticInferenceAcceleratorAssociationTime?: Date;
}
export declare namespace ElasticInferenceAcceleratorAssociation {
    const filterSensitiveLog: (obj: ElasticInferenceAcceleratorAssociation) => any;
    const isa: (o: any) => o is ElasticInferenceAcceleratorAssociation;
}
export interface EnableEbsEncryptionByDefaultRequest {
    __type?: "EnableEbsEncryptionByDefaultRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace EnableEbsEncryptionByDefaultRequest {
    const filterSensitiveLog: (obj: EnableEbsEncryptionByDefaultRequest) => any;
    const isa: (o: any) => o is EnableEbsEncryptionByDefaultRequest;
}
export interface EnableEbsEncryptionByDefaultResult {
    __type?: "EnableEbsEncryptionByDefaultResult";
    /**
     * <p>The updated status of encryption by default.</p>
     */
    EbsEncryptionByDefault?: boolean;
}
export declare namespace EnableEbsEncryptionByDefaultResult {
    const filterSensitiveLog: (obj: EnableEbsEncryptionByDefaultResult) => any;
    const isa: (o: any) => o is EnableEbsEncryptionByDefaultResult;
}
/**
 * <p>Contains information about the errors that occurred when enabling fast snapshot restores.</p>
 */
export interface EnableFastSnapshotRestoreErrorItem {
    __type?: "EnableFastSnapshotRestoreErrorItem";
    /**
     * <p>The errors.</p>
     */
    FastSnapshotRestoreStateErrors?: EnableFastSnapshotRestoreStateErrorItem[];
    /**
     * <p>The ID of the snapshot.</p>
     */
    SnapshotId?: string;
}
export declare namespace EnableFastSnapshotRestoreErrorItem {
    const filterSensitiveLog: (obj: EnableFastSnapshotRestoreErrorItem) => any;
    const isa: (o: any) => o is EnableFastSnapshotRestoreErrorItem;
}
export interface EnableFastSnapshotRestoresRequest {
    __type?: "EnableFastSnapshotRestoresRequest";
    /**
     * <p>One or more Availability Zones. For example, <code>us-east-2a</code>.</p>
     */
    AvailabilityZones: string[] | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of one or more snapshots. For example, <code>snap-1234567890abcdef0</code>. You can specify
     *       a snapshot that was shared with you from another AWS account.</p>
     */
    SourceSnapshotIds: string[] | undefined;
}
export declare namespace EnableFastSnapshotRestoresRequest {
    const filterSensitiveLog: (obj: EnableFastSnapshotRestoresRequest) => any;
    const isa: (o: any) => o is EnableFastSnapshotRestoresRequest;
}
export interface EnableFastSnapshotRestoresResult {
    __type?: "EnableFastSnapshotRestoresResult";
    /**
     * <p>Information about the snapshots for which fast snapshot restores were successfully enabled.</p>
     */
    Successful?: EnableFastSnapshotRestoreSuccessItem[];
    /**
     * <p>Information about the snapshots for which fast snapshot restores could not be enabled.</p>
     */
    Unsuccessful?: EnableFastSnapshotRestoreErrorItem[];
}
export declare namespace EnableFastSnapshotRestoresResult {
    const filterSensitiveLog: (obj: EnableFastSnapshotRestoresResult) => any;
    const isa: (o: any) => o is EnableFastSnapshotRestoresResult;
}
/**
 * <p>Describes an error that occurred when enabling fast snapshot restores.</p>
 */
export interface EnableFastSnapshotRestoreStateError {
    __type?: "EnableFastSnapshotRestoreStateError";
    /**
     * <p>The error code.</p>
     */
    Code?: string;
    /**
     * <p>The error message.</p>
     */
    Message?: string;
}
export declare namespace EnableFastSnapshotRestoreStateError {
    const filterSensitiveLog: (obj: EnableFastSnapshotRestoreStateError) => any;
    const isa: (o: any) => o is EnableFastSnapshotRestoreStateError;
}
/**
 * <p>Contains information about an error that occurred when enabling fast snapshot restores.</p>
 */
export interface EnableFastSnapshotRestoreStateErrorItem {
    __type?: "EnableFastSnapshotRestoreStateErrorItem";
    /**
     * <p>The Availability Zone.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The error.</p>
     */
    Error?: EnableFastSnapshotRestoreStateError;
}
export declare namespace EnableFastSnapshotRestoreStateErrorItem {
    const filterSensitiveLog: (obj: EnableFastSnapshotRestoreStateErrorItem) => any;
    const isa: (o: any) => o is EnableFastSnapshotRestoreStateErrorItem;
}
/**
 * <p>Describes fast snapshot restores that were successfully enabled.</p>
 */
export interface EnableFastSnapshotRestoreSuccessItem {
    __type?: "EnableFastSnapshotRestoreSuccessItem";
    /**
     * <p>The Availability Zone.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The time at which fast snapshot restores entered the <code>disabled</code> state.</p>
     */
    DisabledTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>disabling</code> state.</p>
     */
    DisablingTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>enabled</code> state.</p>
     */
    EnabledTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>enabling</code> state.</p>
     */
    EnablingTime?: Date;
    /**
     * <p>The time at which fast snapshot restores entered the <code>optimizing</code> state.</p>
     */
    OptimizingTime?: Date;
    /**
     * <p>The alias of the snapshot owner.</p>
     */
    OwnerAlias?: string;
    /**
     * <p>The ID of the AWS account that owns the snapshot.</p>
     */
    OwnerId?: string;
    /**
     * <p>The ID of the snapshot.</p>
     */
    SnapshotId?: string;
    /**
     * <p>The state of fast snapshot restores.</p>
     */
    State?: FastSnapshotRestoreStateCode | string;
    /**
     * <p>The reason for the state transition. The possible values are as follows:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>Client.UserInitiated</code> - The state successfully transitioned to <code>enabling</code> or
     *           <code>disabling</code>.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>Client.UserInitiated - Lifecycle state transition</code> - The state successfully transitioned
     *           to <code>optimizing</code>, <code>enabled</code>, or <code>disabled</code>.</p>
     *             </li>
     *          </ul>
     */
    StateTransitionReason?: string;
}
export declare namespace EnableFastSnapshotRestoreSuccessItem {
    const filterSensitiveLog: (obj: EnableFastSnapshotRestoreSuccessItem) => any;
    const isa: (o: any) => o is EnableFastSnapshotRestoreSuccessItem;
}
export interface EnableTransitGatewayRouteTablePropagationRequest {
    __type?: "EnableTransitGatewayRouteTablePropagationRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
    /**
     * <p>The ID of the propagation route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace EnableTransitGatewayRouteTablePropagationRequest {
    const filterSensitiveLog: (obj: EnableTransitGatewayRouteTablePropagationRequest) => any;
    const isa: (o: any) => o is EnableTransitGatewayRouteTablePropagationRequest;
}
export interface EnableTransitGatewayRouteTablePropagationResult {
    __type?: "EnableTransitGatewayRouteTablePropagationResult";
    /**
     * <p>Information about route propagation.</p>
     */
    Propagation?: TransitGatewayPropagation;
}
export declare namespace EnableTransitGatewayRouteTablePropagationResult {
    const filterSensitiveLog: (obj: EnableTransitGatewayRouteTablePropagationResult) => any;
    const isa: (o: any) => o is EnableTransitGatewayRouteTablePropagationResult;
}
/**
 * <p>Contains the parameters for EnableVgwRoutePropagation.</p>
 */
export interface EnableVgwRoutePropagationRequest {
    __type?: "EnableVgwRoutePropagationRequest";
    /**
     * <p>The ID of the virtual private gateway that is attached to a VPC. The virtual private gateway must be attached to the same VPC that the routing tables are associated with. </p>
     */
    GatewayId: string | undefined;
    /**
     * <p>The ID of the route table. The routing table must be associated with the same VPC that the virtual private gateway is attached to. </p>
     */
    RouteTableId: string | undefined;
}
export declare namespace EnableVgwRoutePropagationRequest {
    const filterSensitiveLog: (obj: EnableVgwRoutePropagationRequest) => any;
    const isa: (o: any) => o is EnableVgwRoutePropagationRequest;
}
export interface EnableVolumeIORequest {
    __type?: "EnableVolumeIORequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the volume.</p>
     */
    VolumeId: string | undefined;
}
export declare namespace EnableVolumeIORequest {
    const filterSensitiveLog: (obj: EnableVolumeIORequest) => any;
    const isa: (o: any) => o is EnableVolumeIORequest;
}
export interface EnableVpcClassicLinkDnsSupportRequest {
    __type?: "EnableVpcClassicLinkDnsSupportRequest";
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace EnableVpcClassicLinkDnsSupportRequest {
    const filterSensitiveLog: (obj: EnableVpcClassicLinkDnsSupportRequest) => any;
    const isa: (o: any) => o is EnableVpcClassicLinkDnsSupportRequest;
}
export interface EnableVpcClassicLinkDnsSupportResult {
    __type?: "EnableVpcClassicLinkDnsSupportResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace EnableVpcClassicLinkDnsSupportResult {
    const filterSensitiveLog: (obj: EnableVpcClassicLinkDnsSupportResult) => any;
    const isa: (o: any) => o is EnableVpcClassicLinkDnsSupportResult;
}
export interface EnableVpcClassicLinkRequest {
    __type?: "EnableVpcClassicLinkRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace EnableVpcClassicLinkRequest {
    const filterSensitiveLog: (obj: EnableVpcClassicLinkRequest) => any;
    const isa: (o: any) => o is EnableVpcClassicLinkRequest;
}
export interface EnableVpcClassicLinkResult {
    __type?: "EnableVpcClassicLinkResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace EnableVpcClassicLinkResult {
    const filterSensitiveLog: (obj: EnableVpcClassicLinkResult) => any;
    const isa: (o: any) => o is EnableVpcClassicLinkResult;
}
export declare type EnaSupport = "required" | "supported" | "unsupported";
export declare type EndDateType = "limited" | "unlimited";
export declare type EventCode = "instance-reboot" | "instance-retirement" | "instance-stop" | "system-maintenance" | "system-reboot";
/**
 * <p>Describes an EC2 Fleet or Spot Fleet event.</p>
 */
export interface EventInformation {
    __type?: "EventInformation";
    /**
     * <p>The description of the event.</p>
     */
    EventDescription?: string;
    /**
     * <p>The event.</p>
     *
     *          <p>The following are the <code>error</code> events:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                     <code>iamFleetRoleInvalid</code> - The EC2 Fleet or Spot Fleet did not have the required
     *                     permissions either to launch or terminate an instance.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>spotFleetRequestConfigurationInvalid</code> - The configuration is not valid. For more information, see the description
     *           of the event.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>spotInstanceCountLimitExceeded</code> - You've reached the limit on the number
     *           of Spot Instances that you can launch.</p>
     *             </li>
     *          </ul>
     *
     *          <p>The following are the <code>fleetRequestChange</code> events:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                     <code>active</code> - The EC2 Fleet or Spot Fleet request has been validated and Amazon EC2 is
     *                     attempting to maintain the target number of running Spot Instances.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>cancelled</code> - The EC2 Fleet or Spot Fleet request is canceled and has no running
     *                     Spot Instances. The EC2 Fleet or Spot Fleet will be deleted two days after its instances
     *                     were terminated.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>cancelled_running</code> - The EC2 Fleet or Spot Fleet request is canceled and does
     *                     not launch additional Spot Instances. Existing Spot Instances continue to run
     *                     until they are interrupted or terminated.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>cancelled_terminating</code> - The EC2 Fleet or Spot Fleet request is canceled and
     *                     its Spot Instances are terminating.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>expired</code> - The EC2 Fleet or Spot Fleet request has expired. A subsequent event
     *                     indicates that the instances were terminated, if the request was created with
     *                         <code>TerminateInstancesWithExpiration</code> set.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>modify_in_progress</code> - A request to modify the EC2 Fleet or Spot Fleet request
     *                     was accepted and is in progress.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>modify_successful</code> - The EC2 Fleet or Spot Fleet request was modified.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>price_update</code> - The price for a launch configuration was adjusted because
     *           it was too high. This change is permanent.</p>
     *             </li>
     *             <li>
     *                <p>
     *                     <code>submitted</code> - The EC2 Fleet or Spot Fleet request is being evaluated and Amazon EC2
     *                     is preparing to launch the target number of Spot Instances.</p>
     *             </li>
     *          </ul>
     *
     *          <p>The following are the <code>instanceChange</code> events:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>launched</code> - A request was fulfilled and a new instance was launched.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>terminated</code> - An instance was terminated by the user.</p>
     *             </li>
     *          </ul>
     *
     *          <p>The following are the <code>Information</code> events:</p>
     *          <ul>
     *             <li>
     *                 <p>
     *                   <code>launchSpecTemporarilyBlacklisted</code> - The configuration is not valid
     *                     and several attempts to launch instances have failed. For more information, see
     *                     the description of the event.</p>
     *             </li>
     *             <li>
     *               <p>
     *                   <code>launchSpecUnusable</code> - The price in a launch specification is not valid because it is
     *                   below the Spot price or the Spot price is above the On-Demand price.</p>
     *             </li>
     *             <li>
     *               <p>
     *                   <code>fleetProgressHalted</code> - The price in every launch specification is not valid. A launch
     *                   specification might become valid if the Spot price changes.</p>
     *             </li>
     *          </ul>
     */
    EventSubType?: string;
    /**
     * <p>The ID of the instance. This information is available only for <code>instanceChange</code> events.</p>
     */
    InstanceId?: string;
}
export declare namespace EventInformation {
    const filterSensitiveLog: (obj: EventInformation) => any;
    const isa: (o: any) => o is EventInformation;
}
export declare enum EventType {
    BATCH_CHANGE = "fleetRequestChange",
    ERROR = "error",
    INFORMATION = "information",
    INSTANCE_CHANGE = "instanceChange"
}
export declare enum ExcessCapacityTerminationPolicy {
    DEFAULT = "default",
    NO_TERMINATION = "noTermination"
}
export interface ExportClientVpnClientCertificateRevocationListRequest {
    __type?: "ExportClientVpnClientCertificateRevocationListRequest";
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace ExportClientVpnClientCertificateRevocationListRequest {
    const filterSensitiveLog: (obj: ExportClientVpnClientCertificateRevocationListRequest) => any;
    const isa: (o: any) => o is ExportClientVpnClientCertificateRevocationListRequest;
}
export interface ExportClientVpnClientCertificateRevocationListResult {
    __type?: "ExportClientVpnClientCertificateRevocationListResult";
    /**
     * <p>Information about the client certificate revocation list.</p>
     */
    CertificateRevocationList?: string;
    /**
     * <p>The current state of the client certificate revocation list.</p>
     */
    Status?: ClientCertificateRevocationListStatus;
}
export declare namespace ExportClientVpnClientCertificateRevocationListResult {
    const filterSensitiveLog: (obj: ExportClientVpnClientCertificateRevocationListResult) => any;
    const isa: (o: any) => o is ExportClientVpnClientCertificateRevocationListResult;
}
export interface ExportClientVpnClientConfigurationRequest {
    __type?: "ExportClientVpnClientConfigurationRequest";
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace ExportClientVpnClientConfigurationRequest {
    const filterSensitiveLog: (obj: ExportClientVpnClientConfigurationRequest) => any;
    const isa: (o: any) => o is ExportClientVpnClientConfigurationRequest;
}
export interface ExportClientVpnClientConfigurationResult {
    __type?: "ExportClientVpnClientConfigurationResult";
    /**
     * <p>The contents of the Client VPN endpoint configuration file.</p>
     */
    ClientConfiguration?: string;
}
export declare namespace ExportClientVpnClientConfigurationResult {
    const filterSensitiveLog: (obj: ExportClientVpnClientConfigurationResult) => any;
    const isa: (o: any) => o is ExportClientVpnClientConfigurationResult;
}
export declare type ExportEnvironment = "citrix" | "microsoft" | "vmware";
export interface ExportImageRequest {
    __type?: "ExportImageRequest";
    /**
     * <p>Token to enable idempotency for export image requests.</p>
     */
    ClientToken?: string;
    /**
     * <p>A description of the image being exported. The maximum length is 255 bytes.</p>
     */
    Description?: string;
    /**
     * <p>The disk image format.</p>
     */
    DiskImageFormat: DiskImageFormat | string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the image.</p>
     */
    ImageId: string | undefined;
    /**
     * <p>The name of the role that grants VM Import/Export permission to export images to your S3 bucket. If this
     *    parameter is not specified, the default role is named 'vmimport'.</p>
     */
    RoleName?: string;
    /**
     * <p>Information about the destination S3 bucket. The bucket must exist and grant WRITE and READ_ACP permissions to
     *    the AWS account vm-import-export@amazon.com.</p>
     */
    S3ExportLocation: ExportTaskS3LocationRequest | undefined;
}
export declare namespace ExportImageRequest {
    const filterSensitiveLog: (obj: ExportImageRequest) => any;
    const isa: (o: any) => o is ExportImageRequest;
}
export interface ExportImageResult {
    __type?: "ExportImageResult";
    /**
     * <p>A description of the image being exported.</p>
     */
    Description?: string;
    /**
     * <p>The disk image format for the exported image.</p>
     */
    DiskImageFormat?: DiskImageFormat | string;
    /**
     * <p>The ID of the export image task.</p>
     */
    ExportImageTaskId?: string;
    /**
     * <p>The ID of the image.</p>
     */
    ImageId?: string;
    /**
     * <p>The percent complete of the export image task.</p>
     */
    Progress?: string;
    /**
     * <p>The name of the role that grants VM Import/Export permission to export images to your S3 bucket.</p>
     */
    RoleName?: string;
    /**
     * <p>Information about the destination S3 bucket.</p>
     */
    S3ExportLocation?: ExportTaskS3Location;
    /**
     * <p>The status of the export image task. The possible values are <code>active</code>, <code>completed</code>,
     *     <code>deleting</code>, and <code>deleted</code>.</p>
     */
    Status?: string;
    /**
     * <p>The status message for the export image task.</p>
     */
    StatusMessage?: string;
}
export declare namespace ExportImageResult {
    const filterSensitiveLog: (obj: ExportImageResult) => any;
    const isa: (o: any) => o is ExportImageResult;
}
/**
 * <p>Describes an export image task.</p>
 */
export interface ExportImageTask {
    __type?: "ExportImageTask";
    /**
     * <p>A description of the image being exported.</p>
     */
    Description?: string;
    /**
     * <p>The ID of the export image task.</p>
     */
    ExportImageTaskId?: string;
    /**
     * <p>The ID of the image.</p>
     */
    ImageId?: string;
    /**
     * <p>The percent complete of the export image task.</p>
     */
    Progress?: string;
    /**
     * <p>Information about the destination S3 bucket.</p>
     */
    S3ExportLocation?: ExportTaskS3Location;
    /**
     * <p>The status of the export image task. The possible values are <code>active</code>, <code>completed</code>,
     *     <code>deleting</code>, and <code>deleted</code>.</p>
     */
    Status?: string;
    /**
     * <p>The status message for the export image task.</p>
     */
    StatusMessage?: string;
}
export declare namespace ExportImageTask {
    const filterSensitiveLog: (obj: ExportImageTask) => any;
    const isa: (o: any) => o is ExportImageTask;
}
/**
 * <p>Describes an instance export task.</p>
 */
export interface ExportTask {
    __type?: "ExportTask";
    /**
     * <p>A description of the resource being exported.</p>
     */
    Description?: string;
    /**
     * <p>The ID of the export task.</p>
     */
    ExportTaskId?: string;
    /**
     * <p>Information about the export task.</p>
     */
    ExportToS3Task?: ExportToS3Task;
    /**
     * <p>Information about the instance to export.</p>
     */
    InstanceExportDetails?: InstanceExportDetails;
    /**
     * <p>The state of the export task.</p>
     */
    State?: ExportTaskState | string;
    /**
     * <p>The status message related to the export task.</p>
     */
    StatusMessage?: string;
    /**
     * <p>The tags for the export task.</p>
     */
    Tags?: Tag[];
}
export declare namespace ExportTask {
    const filterSensitiveLog: (obj: ExportTask) => any;
    const isa: (o: any) => o is ExportTask;
}
/**
 * <p>Describes the destination for an export image task.</p>
 */
export interface ExportTaskS3Location {
    __type?: "ExportTaskS3Location";
    /**
     * <p>The destination S3 bucket.</p>
     */
    S3Bucket?: string;
    /**
     * <p>The prefix (logical hierarchy) in the bucket.</p>
     */
    S3Prefix?: string;
}
export declare namespace ExportTaskS3Location {
    const filterSensitiveLog: (obj: ExportTaskS3Location) => any;
    const isa: (o: any) => o is ExportTaskS3Location;
}
/**
 * <p>Describes the destination for an export image task.</p>
 */
export interface ExportTaskS3LocationRequest {
    __type?: "ExportTaskS3LocationRequest";
    /**
     * <p>The destination S3 bucket.</p>
     */
    S3Bucket: string | undefined;
    /**
     * <p>The prefix (logical hierarchy) in the bucket.</p>
     */
    S3Prefix?: string;
}
export declare namespace ExportTaskS3LocationRequest {
    const filterSensitiveLog: (obj: ExportTaskS3LocationRequest) => any;
    const isa: (o: any) => o is ExportTaskS3LocationRequest;
}
export declare type ExportTaskState = "active" | "cancelled" | "cancelling" | "completed";
/**
 * <p>Describes the format and location for an instance export task.</p>
 */
export interface ExportToS3Task {
    __type?: "ExportToS3Task";
    /**
     * <p>The container format used to combine disk images with metadata (such as OVF). If absent, only the disk image is
     *    exported.</p>
     */
    ContainerFormat?: ContainerFormat | string;
    /**
     * <p>The format for the exported image.</p>
     */
    DiskImageFormat?: DiskImageFormat | string;
    /**
     * <p>The S3 bucket for the destination image. The destination bucket must exist and grant WRITE and READ_ACP
     *    permissions to the AWS account <code>vm-import-export@amazon.com</code>.</p>
     */
    S3Bucket?: string;
    /**
     * <p>The encryption key for your S3 bucket.</p>
     */
    S3Key?: string;
}
export declare namespace ExportToS3Task {
    const filterSensitiveLog: (obj: ExportToS3Task) => any;
    const isa: (o: any) => o is ExportToS3Task;
}
/**
 * <p>Describes an instance export task.</p>
 */
export interface ExportToS3TaskSpecification {
    __type?: "ExportToS3TaskSpecification";
    /**
     * <p>The container format used to combine disk images with metadata (such as OVF). If absent, only the disk image is
     *    exported.</p>
     */
    ContainerFormat?: ContainerFormat | string;
    /**
     * <p>The format for the exported image.</p>
     */
    DiskImageFormat?: DiskImageFormat | string;
    /**
     * <p>The S3 bucket for the destination image. The destination bucket must exist and grant WRITE and READ_ACP
     *    permissions to the AWS account <code>vm-import-export@amazon.com</code>.</p>
     */
    S3Bucket?: string;
    /**
     * <p>The image is written to a single object in the S3 bucket at the S3 key s3prefix + exportTaskId + '.' +
     *    diskImageFormat.</p>
     */
    S3Prefix?: string;
}
export declare namespace ExportToS3TaskSpecification {
    const filterSensitiveLog: (obj: ExportToS3TaskSpecification) => any;
    const isa: (o: any) => o is ExportToS3TaskSpecification;
}
export interface ExportTransitGatewayRoutesRequest {
    __type?: "ExportTransitGatewayRoutesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>attachment.transit-gateway-attachment-id</code> - The id of the transit gateway attachment.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>attachment.resource-id</code> - The resource id of the transit gateway attachment.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>route-search.exact-match</code> - The exact match of the specified filter.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>route-search.longest-prefix-match</code> - The longest prefix that matches the route.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>route-search.subnet-of-match</code> - The routes with a subnet that match the specified CIDR filter.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>route-search.supernet-of-match</code> - The routes with a CIDR that encompass the CIDR filter. For example, if you have 10.0.1.0/29 and 10.0.1.0/31 routes in your route table and you specify supernet-of-match as 10.0.1.0/30, then the result returns 10.0.1.0/29.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the attachment (<code>available</code> | <code>deleted</code> | <code>deleting</code> | <code>failed</code> |  <code>modifying</code> | <code>pendingAcceptance</code> | <code>pending</code> | <code>rollingBack</code> | <code>rejected</code> | <code>rejecting</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-route-destination-cidr-block</code> - The CIDR range.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>type</code> - The type of route (<code>active</code> |
     *                <code>blackhole</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The name of the S3 bucket.</p>
     */
    S3Bucket: string | undefined;
    /**
     * <p>The ID of the route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace ExportTransitGatewayRoutesRequest {
    const filterSensitiveLog: (obj: ExportTransitGatewayRoutesRequest) => any;
    const isa: (o: any) => o is ExportTransitGatewayRoutesRequest;
}
export interface ExportTransitGatewayRoutesResult {
    __type?: "ExportTransitGatewayRoutesResult";
    /**
     * <p>The URL of the exported file in Amazon S3. For example,
     *          s3://<i>bucket_name</i>/VPCTransitGateway/TransitGatewayRouteTables/<i>file_name</i>.</p>
     */
    S3Location?: string;
}
export declare namespace ExportTransitGatewayRoutesResult {
    const filterSensitiveLog: (obj: ExportTransitGatewayRoutesResult) => any;
    const isa: (o: any) => o is ExportTransitGatewayRoutesResult;
}
/**
 * <p>Describes a Reserved Instance whose queued purchase was not deleted.</p>
 */
export interface FailedQueuedPurchaseDeletion {
    __type?: "FailedQueuedPurchaseDeletion";
    /**
     * <p>The error.</p>
     */
    Error?: DeleteQueuedReservedInstancesError;
    /**
     * <p>The ID of the Reserved Instance.</p>
     */
    ReservedInstancesId?: string;
}
export declare namespace FailedQueuedPurchaseDeletion {
    const filterSensitiveLog: (obj: FailedQueuedPurchaseDeletion) => any;
    const isa: (o: any) => o is FailedQueuedPurchaseDeletion;
}
export declare type FastSnapshotRestoreStateCode = "disabled" | "disabling" | "enabled" | "enabling" | "optimizing";
/**
 * <p>A filter name and value pair that is used to return a more specific list of results from a describe operation.
 *          Filters can be used to match a set of resources by specific criteria, such as tags, attributes, or IDs.
 *          The filters supported by a describe operation are documented with the describe operation. For example:</p>
 *          <ul>
 *             <li>
 *                <p>
 *                   <a>DescribeAvailabilityZones</a>
 *                </p>
 *             </li>
 *             <li>
 *                <p>
 *                   <a>DescribeImages</a>
 *                </p>
 *             </li>
 *             <li>
 *                <p>
 *                   <a>DescribeInstances</a>
 *                </p>
 *             </li>
 *             <li>
 *                <p>
 *                   <a>DescribeKeyPairs</a>
 *                </p>
 *             </li>
 *             <li>
 *                <p>
 *                   <a>DescribeSecurityGroups</a>
 *                </p>
 *             </li>
 *             <li>
 *                <p>
 *                   <a>DescribeSnapshots</a>
 *                </p>
 *             </li>
 *             <li>
 *                <p>
 *                   <a>DescribeSubnets</a>
 *                </p>
 *             </li>
 *             <li>
 *                <p>
 *                   <a>DescribeTags</a>
 *                </p>
 *             </li>
 *             <li>
 *                <p>
 *                   <a>DescribeVolumes</a>
 *                </p>
 *             </li>
 *             <li>
 *                <p>
 *                   <a>DescribeVpcs</a>
 *                </p>
 *             </li>
 *          </ul>
 */
export interface Filter {
    __type?: "Filter";
    /**
     * <p>The name of the filter. Filter names are case-sensitive.</p>
     */
    Name?: string;
    /**
     * <p>The filter values. Filter values are case-sensitive.</p>
     */
    Values?: string[];
}
export declare namespace Filter {
    const filterSensitiveLog: (obj: Filter) => any;
    const isa: (o: any) => o is Filter;
}
export declare enum FleetActivityStatus {
    ERROR = "error",
    FULFILLED = "fulfilled",
    PENDING_FULFILLMENT = "pending_fulfillment",
    PENDING_TERMINATION = "pending_termination"
}
export declare enum FleetCapacityReservationUsageStrategy {
    USE_CAPACITY_RESERVATIONS_FIRST = "use-capacity-reservations-first"
}
/**
 * <p>Describes an EC2 Fleet.</p>
 */
export interface FleetData {
    __type?: "FleetData";
    /**
     * <p>The progress of the EC2 Fleet. If there is an error, the status is <code>error</code>. After
     *          all requests are placed, the status is <code>pending_fulfillment</code>. If the size of the
     *          EC2 Fleet is equal to or greater than its target capacity, the status is <code>fulfilled</code>.
     *          If the size of the EC2 Fleet is decreased, the status is <code>pending_termination</code> while
     *          instances are terminating.</p>
     */
    ActivityStatus?: FleetActivityStatus | string;
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the
     *          request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring
     *             Idempotency</a>.</p>
     *          <p>Constraints: Maximum 64 ASCII characters</p>
     */
    ClientToken?: string;
    /**
     * <p>The creation date and time of the EC2 Fleet.</p>
     */
    CreateTime?: Date;
    /**
     * <p>Information about the instances that could not be launched by the fleet. Valid only when
     *             <b>Type</b> is set to <code>instant</code>.</p>
     */
    Errors?: DescribeFleetError[];
    /**
     * <p>Indicates whether running instances should be terminated if the target capacity of the
     *          EC2 Fleet is decreased below the current size of the EC2 Fleet.</p>
     */
    ExcessCapacityTerminationPolicy?: FleetExcessCapacityTerminationPolicy | string;
    /**
     * <p>The ID of the EC2 Fleet.</p>
     */
    FleetId?: string;
    /**
     * <p>The state of the EC2 Fleet.</p>
     */
    FleetState?: FleetStateCode | string;
    /**
     * <p>The number of units fulfilled by this request compared to the set target
     *          capacity.</p>
     */
    FulfilledCapacity?: number;
    /**
     * <p>The number of units fulfilled by this request compared to the set target On-Demand
     *          capacity.</p>
     */
    FulfilledOnDemandCapacity?: number;
    /**
     * <p>Information about the instances that were launched by the fleet. Valid only when
     *             <b>Type</b> is set to <code>instant</code>.</p>
     */
    Instances?: DescribeFleetsInstances[];
    /**
     * <p>The launch template and overrides.</p>
     */
    LaunchTemplateConfigs?: FleetLaunchTemplateConfig[];
    /**
     * <p>The allocation strategy of On-Demand Instances in an EC2 Fleet.</p>
     */
    OnDemandOptions?: OnDemandOptions;
    /**
     * <p>Indicates whether EC2 Fleet should replace unhealthy instances.</p>
     */
    ReplaceUnhealthyInstances?: boolean;
    /**
     * <p>The configuration of Spot Instances in an EC2 Fleet.</p>
     */
    SpotOptions?: SpotOptions;
    /**
     * <p>The tags for an EC2 Fleet resource.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The number of units to request. You can choose to set the target capacity in terms of
     *          instances or a performance characteristic that is important to your application workload,
     *          such as vCPUs, memory, or I/O. If the request type is <code>maintain</code>, you can
     *          specify a target capacity of 0 and add capacity later.</p>
     */
    TargetCapacitySpecification?: TargetCapacitySpecification;
    /**
     * <p>Indicates whether running instances should be terminated when the EC2 Fleet expires. </p>
     */
    TerminateInstancesWithExpiration?: boolean;
    /**
     * <p>The type of request. Indicates whether the EC2 Fleet only <code>requests</code> the target
     *          capacity, or also attempts to <code>maintain</code> it. If you request a certain target
     *          capacity, EC2 Fleet only places the required requests; it does not attempt to replenish
     *          instances if capacity is diminished, and it does not submit requests in alternative
     *          capacity pools if capacity is unavailable. To maintain a certain target capacity, EC2 Fleet
     *          places the required requests to meet this target capacity. It also automatically
     *          replenishes any interrupted Spot Instances. Default: <code>maintain</code>.</p>
     */
    Type?: FleetType | string;
    /**
     * <p>The start date and time of the request, in UTC format (for example,
     *             <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).
     *          The default is to start fulfilling the request immediately. </p>
     */
    ValidFrom?: Date;
    /**
     * <p>The end date and time of the request, in UTC format (for example,
     *             <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).
     *          At this point, no new instance requests are placed or able to fulfill the request. The
     *          default end date is 7 days from the current date. </p>
     */
    ValidUntil?: Date;
}
export declare namespace FleetData {
    const filterSensitiveLog: (obj: FleetData) => any;
    const isa: (o: any) => o is FleetData;
}
export declare enum FleetEventType {
    FLEET_CHANGE = "fleet-change",
    INSTANCE_CHANGE = "instance-change",
    SERVICE_ERROR = "service-error"
}
export declare enum FleetExcessCapacityTerminationPolicy {
    NO_TERMINATION = "no-termination",
    TERMINATION = "termination"
}
/**
 * <p>Describes a launch template and overrides.</p>
 */
export interface FleetLaunchTemplateConfig {
    __type?: "FleetLaunchTemplateConfig";
    /**
     * <p>The launch template.</p>
     */
    LaunchTemplateSpecification?: FleetLaunchTemplateSpecification;
    /**
     * <p>Any parameters that you specify override the same parameters in the launch
     *          template.</p>
     */
    Overrides?: FleetLaunchTemplateOverrides[];
}
export declare namespace FleetLaunchTemplateConfig {
    const filterSensitiveLog: (obj: FleetLaunchTemplateConfig) => any;
    const isa: (o: any) => o is FleetLaunchTemplateConfig;
}
/**
 * <p>Describes a launch template and overrides.</p>
 */
export interface FleetLaunchTemplateConfigRequest {
    __type?: "FleetLaunchTemplateConfigRequest";
    /**
     * <p>The launch template to use. You must specify either the launch template ID or launch
     *          template name in the request. </p>
     */
    LaunchTemplateSpecification?: FleetLaunchTemplateSpecificationRequest;
    /**
     * <p>Any parameters that you specify override the same parameters in the launch
     *          template.</p>
     */
    Overrides?: FleetLaunchTemplateOverridesRequest[];
}
export declare namespace FleetLaunchTemplateConfigRequest {
    const filterSensitiveLog: (obj: FleetLaunchTemplateConfigRequest) => any;
    const isa: (o: any) => o is FleetLaunchTemplateConfigRequest;
}
/**
 * <p>Describes overrides for a launch template.</p>
 */
export interface FleetLaunchTemplateOverrides {
    __type?: "FleetLaunchTemplateOverrides";
    /**
     * <p>The Availability Zone in which to launch the instances.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The maximum price per unit hour that you are willing to pay for a Spot Instance.</p>
     */
    MaxPrice?: string;
    /**
     * <p>The location where the instance launched, if applicable.</p>
     */
    Placement?: PlacementResponse;
    /**
     * <p>The priority for the launch template override. If <b>AllocationStrategy</b> is set to <code>prioritized</code>, EC2 Fleet uses priority to
     *          determine which launch template override to use first in fulfilling On-Demand capacity. The
     *          highest priority is launched first. Valid values are whole numbers starting at
     *             <code>0</code>. The lower the number, the higher the priority. If no number is set, the
     *          override has the lowest priority.</p>
     */
    Priority?: number;
    /**
     * <p>The ID of the subnet in which to launch the instances.</p>
     */
    SubnetId?: string;
    /**
     * <p>The number of units provided by the specified instance type.</p>
     */
    WeightedCapacity?: number;
}
export declare namespace FleetLaunchTemplateOverrides {
    const filterSensitiveLog: (obj: FleetLaunchTemplateOverrides) => any;
    const isa: (o: any) => o is FleetLaunchTemplateOverrides;
}
/**
 * <p>Describes overrides for a launch template.</p>
 */
export interface FleetLaunchTemplateOverridesRequest {
    __type?: "FleetLaunchTemplateOverridesRequest";
    /**
     * <p>The Availability Zone in which to launch the instances.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The maximum price per unit hour that you are willing to pay for a Spot Instance.</p>
     */
    MaxPrice?: string;
    /**
     * <p>The location where the instance launched, if applicable.</p>
     */
    Placement?: Placement;
    /**
     * <p>The priority for the launch template override. If <b>AllocationStrategy</b> is set to <code>prioritized</code>, EC2 Fleet uses priority to
     *          determine which launch template override to use first in fulfilling On-Demand capacity. The
     *          highest priority is launched first. Valid values are whole numbers starting at
     *             <code>0</code>. The lower the number, the higher the priority. If no number is set, the
     *          launch template override has the lowest priority.</p>
     */
    Priority?: number;
    /**
     * <p>The ID of the subnet in which to launch the instances.</p>
     */
    SubnetId?: string;
    /**
     * <p>The number of units provided by the specified instance type.</p>
     */
    WeightedCapacity?: number;
}
export declare namespace FleetLaunchTemplateOverridesRequest {
    const filterSensitiveLog: (obj: FleetLaunchTemplateOverridesRequest) => any;
    const isa: (o: any) => o is FleetLaunchTemplateOverridesRequest;
}
/**
 * <p>Describes a launch template.</p>
 */
export interface FleetLaunchTemplateSpecification {
    __type?: "FleetLaunchTemplateSpecification";
    /**
     * <p>The ID of the launch template. You must specify either a template ID or a template name.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template. You must specify either a template name or a template ID.</p>
     */
    LaunchTemplateName?: string;
    /**
     * <p>The version number of the launch template. You must specify a version number.</p>
     */
    Version?: string;
}
export declare namespace FleetLaunchTemplateSpecification {
    const filterSensitiveLog: (obj: FleetLaunchTemplateSpecification) => any;
    const isa: (o: any) => o is FleetLaunchTemplateSpecification;
}
/**
 * <p>The launch template to use. You must specify either the launch template ID or launch
 *          template name in the request. </p>
 */
export interface FleetLaunchTemplateSpecificationRequest {
    __type?: "FleetLaunchTemplateSpecificationRequest";
    /**
     * <p>The ID of the launch template.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template.</p>
     */
    LaunchTemplateName?: string;
    /**
     * <p>The version number of the launch template. Note: This is a required parameter and will be updated soon. </p>
     */
    Version?: string;
}
export declare namespace FleetLaunchTemplateSpecificationRequest {
    const filterSensitiveLog: (obj: FleetLaunchTemplateSpecificationRequest) => any;
    const isa: (o: any) => o is FleetLaunchTemplateSpecificationRequest;
}
export declare enum FleetOnDemandAllocationStrategy {
    LOWEST_PRICE = "lowest-price",
    PRIORITIZED = "prioritized"
}
export declare enum FleetStateCode {
    ACTIVE = "active",
    DELETED = "deleted",
    DELETED_RUNNING = "deleted_running",
    DELETED_TERMINATING_INSTANCES = "deleted_terminating",
    FAILED = "failed",
    MODIFYING = "modifying",
    SUBMITTED = "submitted"
}
export declare enum FleetType {
    INSTANT = "instant",
    MAINTAIN = "maintain",
    REQUEST = "request"
}
/**
 * <p>Describes a flow log.</p>
 */
export interface FlowLog {
    __type?: "FlowLog";
    /**
     * <p>The date and time the flow log was created.</p>
     */
    CreationTime?: Date;
    /**
     * <p>Information about the error that occurred. <code>Rate limited</code> indicates that
     *             CloudWatch Logs throttling has been applied for one or more network interfaces, or that you've
     *             reached the limit on the number of log groups that you can create. <code>Access
     *                 error</code> indicates that the IAM role associated with the flow log does not have
     *             sufficient permissions to publish to CloudWatch Logs. <code>Unknown error</code> indicates an
     *             internal error.</p>
     */
    DeliverLogsErrorMessage?: string;
    /**
     * <p>The ARN of the IAM role that posts logs to CloudWatch Logs.</p>
     */
    DeliverLogsPermissionArn?: string;
    /**
     * <p>The status of the logs delivery (<code>SUCCESS</code> | <code>FAILED</code>).</p>
     */
    DeliverLogsStatus?: string;
    /**
     * <p>The flow log ID.</p>
     */
    FlowLogId?: string;
    /**
     * <p>The status of the flow log (<code>ACTIVE</code>).</p>
     */
    FlowLogStatus?: string;
    /**
     * <p>Specifies the destination to which the flow log data is published. Flow log data can be
     *             published to an CloudWatch Logs log group or an Amazon S3 bucket. If the flow log publishes to CloudWatch Logs,
     *             this element indicates the Amazon Resource Name (ARN) of the CloudWatch Logs log group to which
     *             the data is published. If the flow log publishes to Amazon S3, this element indicates the ARN
     *             of the Amazon S3 bucket to which the data is published.</p>
     */
    LogDestination?: string;
    /**
     * <p>Specifies the type of destination to which the flow log data is published. Flow log data can be
     *             published to CloudWatch Logs or Amazon S3.</p>
     */
    LogDestinationType?: LogDestinationType | string;
    /**
     * <p>The format of the flow log record.</p>
     */
    LogFormat?: string;
    /**
     * <p>The name of the flow log group.</p>
     */
    LogGroupName?: string;
    /**
     * <p>The ID of the resource on which the flow log was created.</p>
     */
    ResourceId?: string;
    /**
     * <p>The type of traffic captured for the flow log.</p>
     */
    TrafficType?: TrafficType | string;
}
export declare namespace FlowLog {
    const filterSensitiveLog: (obj: FlowLog) => any;
    const isa: (o: any) => o is FlowLog;
}
export declare type FlowLogsResourceType = "NetworkInterface" | "Subnet" | "VPC";
/**
 * <p>Describes the FPGA accelerator for the instance type.</p>
 */
export interface FpgaDeviceInfo {
    __type?: "FpgaDeviceInfo";
    /**
     * <p>The count of FPGA accelerators for the instance type.</p>
     */
    Count?: number;
    /**
     * <p>The manufacturer of the FPGA accelerator.</p>
     */
    Manufacturer?: string;
    /**
     * <p>Describes the memory for the FPGA accelerator for the instance type.</p>
     */
    MemoryInfo?: FpgaDeviceMemoryInfo;
    /**
     * <p>The name of the FPGA accelerator.</p>
     */
    Name?: string;
}
export declare namespace FpgaDeviceInfo {
    const filterSensitiveLog: (obj: FpgaDeviceInfo) => any;
    const isa: (o: any) => o is FpgaDeviceInfo;
}
/**
 * <p>Describes the memory for the FPGA accelerator for the instance type.</p>
 */
export interface FpgaDeviceMemoryInfo {
    __type?: "FpgaDeviceMemoryInfo";
    /**
     * <p>The size (in MiB) for the memory available to the FPGA accelerator.</p>
     */
    SizeInMiB?: number;
}
export declare namespace FpgaDeviceMemoryInfo {
    const filterSensitiveLog: (obj: FpgaDeviceMemoryInfo) => any;
    const isa: (o: any) => o is FpgaDeviceMemoryInfo;
}
/**
 * <p>Describes an Amazon FPGA image (AFI).</p>
 */
export interface FpgaImage {
    __type?: "FpgaImage";
    /**
     * <p>The date and time the AFI was created.</p>
     */
    CreateTime?: Date;
    /**
     * <p>Indicates whether data retention support is enabled for the AFI.</p>
     */
    DataRetentionSupport?: boolean;
    /**
     * <p>The description of the AFI.</p>
     */
    Description?: string;
    /**
     * <p>The global FPGA image identifier (AGFI ID).</p>
     */
    FpgaImageGlobalId?: string;
    /**
     * <p>The FPGA image identifier (AFI ID).</p>
     */
    FpgaImageId?: string;
    /**
     * <p>The name of the AFI.</p>
     */
    Name?: string;
    /**
     * <p>The alias of the AFI owner. Possible values include <code>self</code>, <code>amazon</code>, and <code>aws-marketplace</code>.</p>
     */
    OwnerAlias?: string;
    /**
     * <p>The AWS account ID of the AFI owner.</p>
     */
    OwnerId?: string;
    /**
     * <p>Information about the PCI bus.</p>
     */
    PciId?: PciId;
    /**
     * <p>The product codes for the AFI.</p>
     */
    ProductCodes?: ProductCode[];
    /**
     * <p>Indicates whether the AFI is public.</p>
     */
    Public?: boolean;
    /**
     * <p>The version of the AWS Shell that was used to create the bitstream.</p>
     */
    ShellVersion?: string;
    /**
     * <p>Information about the state of the AFI.</p>
     */
    State?: FpgaImageState;
    /**
     * <p>Any tags assigned to the AFI.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The time of the most recent update to the AFI.</p>
     */
    UpdateTime?: Date;
}
export declare namespace FpgaImage {
    const filterSensitiveLog: (obj: FpgaImage) => any;
    const isa: (o: any) => o is FpgaImage;
}
/**
 * <p>Describes an Amazon FPGA image (AFI) attribute.</p>
 */
export interface FpgaImageAttribute {
    __type?: "FpgaImageAttribute";
    /**
     * <p>The description of the AFI.</p>
     */
    Description?: string;
    /**
     * <p>The ID of the AFI.</p>
     */
    FpgaImageId?: string;
    /**
     * <p>The load permissions.</p>
     */
    LoadPermissions?: LoadPermission[];
    /**
     * <p>The name of the AFI.</p>
     */
    Name?: string;
    /**
     * <p>The product codes.</p>
     */
    ProductCodes?: ProductCode[];
}
export declare namespace FpgaImageAttribute {
    const filterSensitiveLog: (obj: FpgaImageAttribute) => any;
    const isa: (o: any) => o is FpgaImageAttribute;
}
export declare type FpgaImageAttributeName = "description" | "loadPermission" | "name" | "productCodes";
/**
 * <p>Describes the state of the bitstream generation process for an Amazon FPGA image (AFI).</p>
 */
export interface FpgaImageState {
    __type?: "FpgaImageState";
    /**
     * <p>The state. The following are the possible values:</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>pending</code> - AFI bitstream generation is in progress.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>available</code> - The AFI is available for use.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>failed</code> - AFI bitstream generation failed.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>unavailable</code> - The AFI is no longer available for use.</p>
     *             </li>
     *          </ul>
     */
    Code?: FpgaImageStateCode | string;
    /**
     * <p>If the state is <code>failed</code>, this is the error message.</p>
     */
    Message?: string;
}
export declare namespace FpgaImageState {
    const filterSensitiveLog: (obj: FpgaImageState) => any;
    const isa: (o: any) => o is FpgaImageState;
}
export declare type FpgaImageStateCode = "available" | "failed" | "pending" | "unavailable";
/**
 * <p>Describes the FPGAs for the instance type.</p>
 */
export interface FpgaInfo {
    __type?: "FpgaInfo";
    /**
     * <p>Describes the FPGAs for the instance type.</p>
     */
    Fpgas?: FpgaDeviceInfo[];
    /**
     * <p>The total memory of all FPGA accelerators for the instance type.</p>
     */
    TotalFpgaMemoryInMiB?: number;
}
export declare namespace FpgaInfo {
    const filterSensitiveLog: (obj: FpgaInfo) => any;
    const isa: (o: any) => o is FpgaInfo;
}
export declare type GatewayType = "ipsec.1";
export interface GetAssociatedIpv6PoolCidrsRequest {
    __type?: "GetAssociatedIpv6PoolCidrsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the IPv6 address pool.</p>
     */
    PoolId: string | undefined;
}
export declare namespace GetAssociatedIpv6PoolCidrsRequest {
    const filterSensitiveLog: (obj: GetAssociatedIpv6PoolCidrsRequest) => any;
    const isa: (o: any) => o is GetAssociatedIpv6PoolCidrsRequest;
}
export interface GetAssociatedIpv6PoolCidrsResult {
    __type?: "GetAssociatedIpv6PoolCidrsResult";
    /**
     * <p>Information about the IPv6 CIDR block associations.</p>
     */
    Ipv6CidrAssociations?: Ipv6CidrAssociation[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace GetAssociatedIpv6PoolCidrsResult {
    const filterSensitiveLog: (obj: GetAssociatedIpv6PoolCidrsResult) => any;
    const isa: (o: any) => o is GetAssociatedIpv6PoolCidrsResult;
}
export interface GetCapacityReservationUsageRequest {
    __type?: "GetCapacityReservationUsageRequest";
    /**
     * <p>The ID of the Capacity Reservation.</p>
     */
    CapacityReservationId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The maximum number of results to return for the request in a single page. The remaining results can be seen by sending another request with the returned nextToken value.</p>
     * 		       <p>Valid range: Minimum value of 1. Maximum value of 1000.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token to retrieve the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace GetCapacityReservationUsageRequest {
    const filterSensitiveLog: (obj: GetCapacityReservationUsageRequest) => any;
    const isa: (o: any) => o is GetCapacityReservationUsageRequest;
}
export interface GetCapacityReservationUsageResult {
    __type?: "GetCapacityReservationUsageResult";
    /**
     * <p>The remaining capacity. Indicates the number of instances that can be launched in the Capacity Reservation.</p>
     */
    AvailableInstanceCount?: number;
    /**
     * <p>The ID of the Capacity Reservation.</p>
     */
    CapacityReservationId?: string;
    /**
     * <p>The type of instance for which the Capacity Reservation reserves capacity.</p>
     */
    InstanceType?: string;
    /**
     * <p>Information about the Capacity Reservation usage.</p>
     */
    InstanceUsages?: InstanceUsage[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>The current state of the Capacity Reservation. A Capacity Reservation can be in one of the following states:</p>
     * 			      <ul>
     *             <li>
     *                <p>
     *                   <code>active</code> - The Capacity Reservation is active and the capacity is available for your use.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>expired</code> - The Capacity Reservation expired automatically at the date and time specified
     * 					in your request. The reserved capacity is no longer available for your use.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>cancelled</code> - The Capacity Reservation was manually cancelled. The reserved capacity is no
     * 					longer available for your use.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>pending</code> - The Capacity Reservation request was successful but the capacity
     * 					provisioning is still pending.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>failed</code> - The Capacity Reservation request has failed. A request might fail
     * 					due to invalid request parameters, capacity constraints, or instance limit constraints.
     * 					Failed requests are retained for 60 minutes.</p>
     *             </li>
     *          </ul>
     */
    State?: CapacityReservationState | string;
    /**
     * <p>The number of instances for which the Capacity Reservation reserves capacity.</p>
     */
    TotalInstanceCount?: number;
}
export declare namespace GetCapacityReservationUsageResult {
    const filterSensitiveLog: (obj: GetCapacityReservationUsageResult) => any;
    const isa: (o: any) => o is GetCapacityReservationUsageResult;
}
export interface GetCoipPoolUsageRequest {
    __type?: "GetCoipPoolUsageRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The filters. The following are the possible values:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>coip-address-usage.allocation-id</code>
     *                </p>
     *             </li>
     *          </ul>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>coip-address-usage.aws-account-id</code>
     *                </p>
     *             </li>
     *          </ul>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>coip-address-usage.aws-service</code>
     *                </p>
     *             </li>
     *          </ul>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>coip-address-usage.co-ip</code>
     *                </p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the address pool.</p>
     */
    PoolId: string | undefined;
}
export declare namespace GetCoipPoolUsageRequest {
    const filterSensitiveLog: (obj: GetCoipPoolUsageRequest) => any;
    const isa: (o: any) => o is GetCoipPoolUsageRequest;
}
export interface GetCoipPoolUsageResult {
    __type?: "GetCoipPoolUsageResult";
    /**
     * <p>Information about the address usage.</p>
     */
    CoipAddressUsages?: CoipAddressUsage[];
    /**
     * <p>The ID of the customer-owned address pool.</p>
     */
    CoipPoolId?: string;
    /**
     * <p>The ID of the local gateway route table.</p>
     */
    LocalGatewayRouteTableId?: string;
}
export declare namespace GetCoipPoolUsageResult {
    const filterSensitiveLog: (obj: GetCoipPoolUsageResult) => any;
    const isa: (o: any) => o is GetCoipPoolUsageResult;
}
export interface GetConsoleOutputRequest {
    __type?: "GetConsoleOutputRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>When enabled, retrieves the latest console output for the instance.</p>
     *         <p>Default: disabled (<code>false</code>)</p>
     */
    Latest?: boolean;
}
export declare namespace GetConsoleOutputRequest {
    const filterSensitiveLog: (obj: GetConsoleOutputRequest) => any;
    const isa: (o: any) => o is GetConsoleOutputRequest;
}
export interface GetConsoleOutputResult {
    __type?: "GetConsoleOutputResult";
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The console output, base64-encoded. If you are using a command line tool, the tool
     *             decodes the output for you.</p>
     */
    Output?: string;
    /**
     * <p>The time at which the output was last updated.</p>
     */
    Timestamp?: Date;
}
export declare namespace GetConsoleOutputResult {
    const filterSensitiveLog: (obj: GetConsoleOutputResult) => any;
    const isa: (o: any) => o is GetConsoleOutputResult;
}
export interface GetConsoleScreenshotRequest {
    __type?: "GetConsoleScreenshotRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>When set to <code>true</code>, acts as keystroke input and wakes up an instance that's
     *             in standby or "sleep" mode.</p>
     */
    WakeUp?: boolean;
}
export declare namespace GetConsoleScreenshotRequest {
    const filterSensitiveLog: (obj: GetConsoleScreenshotRequest) => any;
    const isa: (o: any) => o is GetConsoleScreenshotRequest;
}
export interface GetConsoleScreenshotResult {
    __type?: "GetConsoleScreenshotResult";
    /**
     * <p>The data that comprises the image.</p>
     */
    ImageData?: string;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
}
export declare namespace GetConsoleScreenshotResult {
    const filterSensitiveLog: (obj: GetConsoleScreenshotResult) => any;
    const isa: (o: any) => o is GetConsoleScreenshotResult;
}
export interface GetDefaultCreditSpecificationRequest {
    __type?: "GetDefaultCreditSpecificationRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The instance family.</p>
     */
    InstanceFamily: UnlimitedSupportedInstanceFamily | string | undefined;
}
export declare namespace GetDefaultCreditSpecificationRequest {
    const filterSensitiveLog: (obj: GetDefaultCreditSpecificationRequest) => any;
    const isa: (o: any) => o is GetDefaultCreditSpecificationRequest;
}
export interface GetDefaultCreditSpecificationResult {
    __type?: "GetDefaultCreditSpecificationResult";
    /**
     * <p>The default credit option for CPU usage of the instance family.</p>
     */
    InstanceFamilyCreditSpecification?: InstanceFamilyCreditSpecification;
}
export declare namespace GetDefaultCreditSpecificationResult {
    const filterSensitiveLog: (obj: GetDefaultCreditSpecificationResult) => any;
    const isa: (o: any) => o is GetDefaultCreditSpecificationResult;
}
export interface GetEbsDefaultKmsKeyIdRequest {
    __type?: "GetEbsDefaultKmsKeyIdRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace GetEbsDefaultKmsKeyIdRequest {
    const filterSensitiveLog: (obj: GetEbsDefaultKmsKeyIdRequest) => any;
    const isa: (o: any) => o is GetEbsDefaultKmsKeyIdRequest;
}
export interface GetEbsDefaultKmsKeyIdResult {
    __type?: "GetEbsDefaultKmsKeyIdResult";
    /**
     * <p>The Amazon Resource Name (ARN) of the default CMK for encryption by default.</p>
     */
    KmsKeyId?: string;
}
export declare namespace GetEbsDefaultKmsKeyIdResult {
    const filterSensitiveLog: (obj: GetEbsDefaultKmsKeyIdResult) => any;
    const isa: (o: any) => o is GetEbsDefaultKmsKeyIdResult;
}
export interface GetEbsEncryptionByDefaultRequest {
    __type?: "GetEbsEncryptionByDefaultRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace GetEbsEncryptionByDefaultRequest {
    const filterSensitiveLog: (obj: GetEbsEncryptionByDefaultRequest) => any;
    const isa: (o: any) => o is GetEbsEncryptionByDefaultRequest;
}
export interface GetEbsEncryptionByDefaultResult {
    __type?: "GetEbsEncryptionByDefaultResult";
    /**
     * <p>Indicates whether encryption by default is enabled.</p>
     */
    EbsEncryptionByDefault?: boolean;
}
export declare namespace GetEbsEncryptionByDefaultResult {
    const filterSensitiveLog: (obj: GetEbsEncryptionByDefaultResult) => any;
    const isa: (o: any) => o is GetEbsEncryptionByDefaultResult;
}
export interface GetHostReservationPurchasePreviewRequest {
    __type?: "GetHostReservationPurchasePreviewRequest";
    /**
     * <p>The IDs of the Dedicated Hosts with which the reservation is associated.</p>
     */
    HostIdSet: string[] | undefined;
    /**
     * <p>The offering ID of the reservation.</p>
     */
    OfferingId: string | undefined;
}
export declare namespace GetHostReservationPurchasePreviewRequest {
    const filterSensitiveLog: (obj: GetHostReservationPurchasePreviewRequest) => any;
    const isa: (o: any) => o is GetHostReservationPurchasePreviewRequest;
}
export interface GetHostReservationPurchasePreviewResult {
    __type?: "GetHostReservationPurchasePreviewResult";
    /**
     * <p>The currency in which the <code>totalUpfrontPrice</code> and
     *                 <code>totalHourlyPrice</code> amounts are specified. At this time, the only
     *             supported currency is <code>USD</code>.</p>
     */
    CurrencyCode?: CurrencyCodeValues | string;
    /**
     * <p>The purchase information of the Dedicated Host reservation and the Dedicated Hosts
     *             associated with it.</p>
     */
    Purchase?: Purchase[];
    /**
     * <p>The potential total hourly price of the reservation per hour.</p>
     */
    TotalHourlyPrice?: string;
    /**
     * <p>The potential total upfront price. This is billed immediately.</p>
     */
    TotalUpfrontPrice?: string;
}
export declare namespace GetHostReservationPurchasePreviewResult {
    const filterSensitiveLog: (obj: GetHostReservationPurchasePreviewResult) => any;
    const isa: (o: any) => o is GetHostReservationPurchasePreviewResult;
}
export interface GetLaunchTemplateDataRequest {
    __type?: "GetLaunchTemplateDataRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually
     *             making the request, and provides an error response. If you have the required
     *             permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is
     *                 <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
}
export declare namespace GetLaunchTemplateDataRequest {
    const filterSensitiveLog: (obj: GetLaunchTemplateDataRequest) => any;
    const isa: (o: any) => o is GetLaunchTemplateDataRequest;
}
export interface GetLaunchTemplateDataResult {
    __type?: "GetLaunchTemplateDataResult";
    /**
     * <p>The instance data.</p>
     */
    LaunchTemplateData?: ResponseLaunchTemplateData;
}
export declare namespace GetLaunchTemplateDataResult {
    const filterSensitiveLog: (obj: GetLaunchTemplateDataResult) => any;
    const isa: (o: any) => o is GetLaunchTemplateDataResult;
}
export interface GetPasswordDataRequest {
    __type?: "GetPasswordDataRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the Windows instance.</p>
     */
    InstanceId: string | undefined;
}
export declare namespace GetPasswordDataRequest {
    const filterSensitiveLog: (obj: GetPasswordDataRequest) => any;
    const isa: (o: any) => o is GetPasswordDataRequest;
}
export interface GetPasswordDataResult {
    __type?: "GetPasswordDataResult";
    /**
     * <p>The ID of the Windows instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The password of the instance. Returns an empty string if the password is not
     *             available.</p>
     */
    PasswordData?: string;
    /**
     * <p>The time the data was last updated.</p>
     */
    Timestamp?: Date;
}
export declare namespace GetPasswordDataResult {
    const filterSensitiveLog: (obj: GetPasswordDataResult) => any;
    const isa: (o: any) => o is GetPasswordDataResult;
}
/**
 * <p>Contains the parameters for GetReservedInstanceExchangeQuote.</p>
 */
export interface GetReservedInstancesExchangeQuoteRequest {
    __type?: "GetReservedInstancesExchangeQuoteRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *       and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *       Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the Convertible Reserved Instances to exchange.</p>
     */
    ReservedInstanceIds: string[] | undefined;
    /**
     * <p>The configuration of the target Convertible Reserved Instance to exchange for your
     *             current Convertible Reserved Instances.</p>
     */
    TargetConfigurations?: TargetConfigurationRequest[];
}
export declare namespace GetReservedInstancesExchangeQuoteRequest {
    const filterSensitiveLog: (obj: GetReservedInstancesExchangeQuoteRequest) => any;
    const isa: (o: any) => o is GetReservedInstancesExchangeQuoteRequest;
}
/**
 * <p>Contains the output of GetReservedInstancesExchangeQuote.</p>
 */
export interface GetReservedInstancesExchangeQuoteResult {
    __type?: "GetReservedInstancesExchangeQuoteResult";
    /**
     * <p>The currency of the transaction.</p>
     */
    CurrencyCode?: string;
    /**
     * <p>If <code>true</code>, the exchange is valid. If <code>false</code>, the exchange cannot be completed.</p>
     */
    IsValidExchange?: boolean;
    /**
     * <p>The new end date of the reservation term.</p>
     */
    OutputReservedInstancesWillExpireAt?: Date;
    /**
     * <p>The total true upfront charge for the exchange.</p>
     */
    PaymentDue?: string;
    /**
     * <p>The cost associated with the Reserved Instance.</p>
     */
    ReservedInstanceValueRollup?: ReservationValue;
    /**
     * <p>The configuration of your Convertible Reserved Instances.</p>
     */
    ReservedInstanceValueSet?: ReservedInstanceReservationValue[];
    /**
     * <p>The cost associated with the Reserved Instance.</p>
     */
    TargetConfigurationValueRollup?: ReservationValue;
    /**
     * <p>The values of the target Convertible Reserved Instances.</p>
     */
    TargetConfigurationValueSet?: TargetReservationValue[];
    /**
     * <p>Describes the reason why the exchange cannot be completed.</p>
     */
    ValidationFailureReason?: string;
}
export declare namespace GetReservedInstancesExchangeQuoteResult {
    const filterSensitiveLog: (obj: GetReservedInstancesExchangeQuoteResult) => any;
    const isa: (o: any) => o is GetReservedInstancesExchangeQuoteResult;
}
export interface GetTransitGatewayAttachmentPropagationsRequest {
    __type?: "GetTransitGatewayAttachmentPropagationsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-route-table-id</code> - The ID of the transit gateway route table.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
}
export declare namespace GetTransitGatewayAttachmentPropagationsRequest {
    const filterSensitiveLog: (obj: GetTransitGatewayAttachmentPropagationsRequest) => any;
    const isa: (o: any) => o is GetTransitGatewayAttachmentPropagationsRequest;
}
export interface GetTransitGatewayAttachmentPropagationsResult {
    __type?: "GetTransitGatewayAttachmentPropagationsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the propagation route tables.</p>
     */
    TransitGatewayAttachmentPropagations?: TransitGatewayAttachmentPropagation[];
}
export declare namespace GetTransitGatewayAttachmentPropagationsResult {
    const filterSensitiveLog: (obj: GetTransitGatewayAttachmentPropagationsResult) => any;
    const isa: (o: any) => o is GetTransitGatewayAttachmentPropagationsResult;
}
export interface GetTransitGatewayMulticastDomainAssociationsRequest {
    __type?: "GetTransitGatewayMulticastDomainAssociationsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>resource-id</code> - The ID of the resource.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>resource-type</code> - The type of resource. The valid value is: <code>vpc</code>.</p>
     *             </li>
     *             <li>
     *
     *                 <p>
     *                   <code>state</code> - The state of the subnet association. Valid values are
     *                         <code>associated</code> |
     *                         <code>associating</code>
     *                     | <code>disassociated</code> | <code>disassociating</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>subnet-id</code> - The ID of the subnet.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>transit-gateway-attachment-id</code> - The id of the transit gateway attachment.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace GetTransitGatewayMulticastDomainAssociationsRequest {
    const filterSensitiveLog: (obj: GetTransitGatewayMulticastDomainAssociationsRequest) => any;
    const isa: (o: any) => o is GetTransitGatewayMulticastDomainAssociationsRequest;
}
export interface GetTransitGatewayMulticastDomainAssociationsResult {
    __type?: "GetTransitGatewayMulticastDomainAssociationsResult";
    /**
     * <p>Information about the multicast domain associations.</p>
     */
    MulticastDomainAssociations?: TransitGatewayMulticastDomainAssociation[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace GetTransitGatewayMulticastDomainAssociationsResult {
    const filterSensitiveLog: (obj: GetTransitGatewayMulticastDomainAssociationsResult) => any;
    const isa: (o: any) => o is GetTransitGatewayMulticastDomainAssociationsResult;
}
export interface GetTransitGatewayRouteTableAssociationsRequest {
    __type?: "GetTransitGatewayRouteTableAssociationsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>resource-id</code> - The ID of the resource.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>resource-type</code> - The resource type (<code>vpc</code> | <code>vpn</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-attachment-id</code> - The ID of the attachment.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the transit gateway route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace GetTransitGatewayRouteTableAssociationsRequest {
    const filterSensitiveLog: (obj: GetTransitGatewayRouteTableAssociationsRequest) => any;
    const isa: (o: any) => o is GetTransitGatewayRouteTableAssociationsRequest;
}
export interface GetTransitGatewayRouteTableAssociationsResult {
    __type?: "GetTransitGatewayRouteTableAssociationsResult";
    /**
     * <p>Information about the associations.</p>
     */
    Associations?: TransitGatewayRouteTableAssociation[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace GetTransitGatewayRouteTableAssociationsResult {
    const filterSensitiveLog: (obj: GetTransitGatewayRouteTableAssociationsResult) => any;
    const isa: (o: any) => o is GetTransitGatewayRouteTableAssociationsResult;
}
export interface GetTransitGatewayRouteTablePropagationsRequest {
    __type?: "GetTransitGatewayRouteTablePropagationsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>resource-id</code> - The ID of the resource.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>resource-type</code> - The resource type (<code>vpc</code> | <code>vpn</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>transit-gateway-attachment-id</code> - The ID of the attachment.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the transit gateway route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace GetTransitGatewayRouteTablePropagationsRequest {
    const filterSensitiveLog: (obj: GetTransitGatewayRouteTablePropagationsRequest) => any;
    const isa: (o: any) => o is GetTransitGatewayRouteTablePropagationsRequest;
}
export interface GetTransitGatewayRouteTablePropagationsResult {
    __type?: "GetTransitGatewayRouteTablePropagationsResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the route table propagations.</p>
     */
    TransitGatewayRouteTablePropagations?: TransitGatewayRouteTablePropagation[];
}
export declare namespace GetTransitGatewayRouteTablePropagationsResult {
    const filterSensitiveLog: (obj: GetTransitGatewayRouteTablePropagationsResult) => any;
    const isa: (o: any) => o is GetTransitGatewayRouteTablePropagationsResult;
}
/**
 * <p>Describes the GPU accelerators for the instance type.</p>
 */
export interface GpuDeviceInfo {
    __type?: "GpuDeviceInfo";
    /**
     * <p>The number of GPUs for the instance type.</p>
     */
    Count?: number;
    /**
     * <p>The manufacturer of the GPU accelerator.</p>
     */
    Manufacturer?: string;
    /**
     * <p>Describes the memory available to the GPU accelerator.</p>
     */
    MemoryInfo?: GpuDeviceMemoryInfo;
    /**
     * <p>The name of the GPU accelerator.</p>
     */
    Name?: string;
}
export declare namespace GpuDeviceInfo {
    const filterSensitiveLog: (obj: GpuDeviceInfo) => any;
    const isa: (o: any) => o is GpuDeviceInfo;
}
/**
 * <p>Describes the memory available to the GPU accelerator.</p>
 */
export interface GpuDeviceMemoryInfo {
    __type?: "GpuDeviceMemoryInfo";
    /**
     * <p>The size (in MiB) for the memory available to the GPU accelerator.</p>
     */
    SizeInMiB?: number;
}
export declare namespace GpuDeviceMemoryInfo {
    const filterSensitiveLog: (obj: GpuDeviceMemoryInfo) => any;
    const isa: (o: any) => o is GpuDeviceMemoryInfo;
}
/**
 * <p>Describes the GPU accelerators for the instance type.</p>
 */
export interface GpuInfo {
    __type?: "GpuInfo";
    /**
     * <p>Describes the GPU accelerators for the instance type.</p>
     */
    Gpus?: GpuDeviceInfo[];
    /**
     * <p>The total size of the memory for the GPU accelerators for the instance type.</p>
     */
    TotalGpuMemoryInMiB?: number;
}
export declare namespace GpuInfo {
    const filterSensitiveLog: (obj: GpuInfo) => any;
    const isa: (o: any) => o is GpuInfo;
}
/**
 * <p>Describes a security group.</p>
 */
export interface GroupIdentifier {
    __type?: "GroupIdentifier";
    /**
     * <p>The ID of the security group.</p>
     */
    GroupId?: string;
    /**
     * <p>The name of the security group.</p>
     */
    GroupName?: string;
}
export declare namespace GroupIdentifier {
    const filterSensitiveLog: (obj: GroupIdentifier) => any;
    const isa: (o: any) => o is GroupIdentifier;
}
/**
 * <p>Indicates whether your instance is configured for hibernation. This parameter is valid
 *             only if the instance meets the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html#hibernating-prerequisites">hibernation
 *                 prerequisites</a>. For
 *             more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the
 *                 <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
 */
export interface HibernationOptions {
    __type?: "HibernationOptions";
    /**
     * <p>If this parameter is set to <code>true</code>, your instance is enabled for
     *             hibernation; otherwise, it is not enabled for hibernation.</p>
     */
    Configured?: boolean;
}
export declare namespace HibernationOptions {
    const filterSensitiveLog: (obj: HibernationOptions) => any;
    const isa: (o: any) => o is HibernationOptions;
}
/**
 * <p>Indicates whether your instance is configured for hibernation. This parameter is valid
 *             only if the instance meets the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html#hibernating-prerequisites">hibernation
 *                 prerequisites</a>. For
 *             more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the
 *                 <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
 */
export interface HibernationOptionsRequest {
    __type?: "HibernationOptionsRequest";
    /**
     * <p>If you set this parameter to <code>true</code>, your instance is enabled for
     *             hibernation.</p>
     *         <p>Default: <code>false</code>
     *          </p>
     */
    Configured?: boolean;
}
export declare namespace HibernationOptionsRequest {
    const filterSensitiveLog: (obj: HibernationOptionsRequest) => any;
    const isa: (o: any) => o is HibernationOptionsRequest;
}
/**
 * <p>Describes an event in the history of the Spot Fleet request.</p>
 */
export interface HistoryRecord {
    __type?: "HistoryRecord";
    /**
     * <p>Information about the event.</p>
     */
    EventInformation?: EventInformation;
    /**
     * <p>The event type.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>error</code> - An error with the Spot Fleet request.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>fleetRequestChange</code> - A change in the status or configuration of the Spot Fleet request.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>instanceChange</code> - An instance was launched or terminated.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>Information</code> - An informational event.</p>
     *             </li>
     *          </ul>
     */
    EventType?: EventType | string;
    /**
     * <p>The date and time of the event, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    Timestamp?: Date;
}
export declare namespace HistoryRecord {
    const filterSensitiveLog: (obj: HistoryRecord) => any;
    const isa: (o: any) => o is HistoryRecord;
}
/**
 * <p>Describes an event in the history of an EC2 Fleet.</p>
 */
export interface HistoryRecordEntry {
    __type?: "HistoryRecordEntry";
    /**
     * <p>Information about the event.</p>
     */
    EventInformation?: EventInformation;
    /**
     * <p>The event type.</p>
     */
    EventType?: FleetEventType | string;
    /**
     * <p>The date and time of the event, in UTC format (for example,
     *             <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    Timestamp?: Date;
}
export declare namespace HistoryRecordEntry {
    const filterSensitiveLog: (obj: HistoryRecordEntry) => any;
    const isa: (o: any) => o is HistoryRecordEntry;
}
/**
 * <p>Describes the properties of the Dedicated Host.</p>
 */
export interface Host {
    __type?: "Host";
    /**
     * <p>The time that the Dedicated Host was allocated.</p>
     */
    AllocationTime?: Date;
    /**
     * <p>Indicates whether the Dedicated Host supports multiple instance types of the same instance family,
     * 			or a specific instance type only. <code>one</code> indicates that the Dedicated Host supports
     * 			multiple instance types in the instance family. <code>off</code> indicates that the Dedicated
     * 			Host supports a single instance type only.</p>
     */
    AllowsMultipleInstanceTypes?: AllowsMultipleInstanceTypes | string;
    /**
     * <p>Whether auto-placement is on or off.</p>
     */
    AutoPlacement?: AutoPlacement | string;
    /**
     * <p>The Availability Zone of the Dedicated Host.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The ID of the Availability Zone in which the Dedicated Host is allocated.</p>
     */
    AvailabilityZoneId?: string;
    /**
     * <p>Information about the instances running on the Dedicated Host.</p>
     */
    AvailableCapacity?: AvailableCapacity;
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The ID of the Dedicated Host.</p>
     */
    HostId?: string;
    /**
     * <p>The hardware specifications of the Dedicated Host.</p>
     */
    HostProperties?: HostProperties;
    /**
     * <p>Indicates whether host recovery is enabled or disabled for the Dedicated Host.</p>
     */
    HostRecovery?: HostRecovery | string;
    /**
     * <p>The reservation ID of the Dedicated Host. This returns a <code>null</code> response
     *             if the Dedicated Host doesn't have an associated reservation.</p>
     */
    HostReservationId?: string;
    /**
     * <p>The IDs and instance type that are currently running on the Dedicated
     *             Host.</p>
     */
    Instances?: HostInstance[];
    /**
     * <p>Indicates whether the Dedicated Host is in a host resource group. If
     * 			<b>memberOfServiceLinkedResourceGroup</b> is
     * 			<code>true</code>, the host is in a host resource group; otherwise, it is not.</p>
     */
    MemberOfServiceLinkedResourceGroup?: boolean;
    /**
     * <p>The ID of the AWS account that owns the Dedicated Host.</p>
     */
    OwnerId?: string;
    /**
     * <p>The time that the Dedicated Host was released.</p>
     */
    ReleaseTime?: Date;
    /**
     * <p>The Dedicated Host's state.</p>
     */
    State?: AllocationState | string;
    /**
     * <p>Any tags assigned to the Dedicated Host.</p>
     */
    Tags?: Tag[];
}
export declare namespace Host {
    const filterSensitiveLog: (obj: Host) => any;
    const isa: (o: any) => o is Host;
}
/**
 * <p>Describes an instance running on a Dedicated Host.</p>
 */
export interface HostInstance {
    __type?: "HostInstance";
    /**
     * <p>The ID of instance that is running on the Dedicated Host.</p>
     */
    InstanceId?: string;
    /**
     * <p>The instance type (for example, <code>m3.medium</code>) of the running instance.</p>
     */
    InstanceType?: string;
    /**
     * <p>The ID of the AWS account that owns the instance.</p>
     */
    OwnerId?: string;
}
export declare namespace HostInstance {
    const filterSensitiveLog: (obj: HostInstance) => any;
    const isa: (o: any) => o is HostInstance;
}
/**
 * <p>Details about the Dedicated Host Reservation offering.</p>
 */
export interface HostOffering {
    __type?: "HostOffering";
    /**
     * <p>The currency of the offering.</p>
     */
    CurrencyCode?: CurrencyCodeValues | string;
    /**
     * <p>The duration of the offering (in seconds).</p>
     */
    Duration?: number;
    /**
     * <p>The hourly price of the offering.</p>
     */
    HourlyPrice?: string;
    /**
     * <p>The instance family of the offering.</p>
     */
    InstanceFamily?: string;
    /**
     * <p>The ID of the offering.</p>
     */
    OfferingId?: string;
    /**
     * <p>The available payment option.</p>
     */
    PaymentOption?: PaymentOption | string;
    /**
     * <p>The upfront price of the offering. Does not apply to No Upfront
     *             offerings.</p>
     */
    UpfrontPrice?: string;
}
export declare namespace HostOffering {
    const filterSensitiveLog: (obj: HostOffering) => any;
    const isa: (o: any) => o is HostOffering;
}
/**
 * <p>Describes the properties of a Dedicated Host.</p>
 */
export interface HostProperties {
    __type?: "HostProperties";
    /**
     * <p>The number of cores on the Dedicated Host.</p>
     */
    Cores?: number;
    /**
     * <p>The instance family supported by the Dedicated Host. For example, <code>m5</code>.</p>
     */
    InstanceFamily?: string;
    /**
     * <p>The instance type supported by the Dedicated Host. For example, <code>m5.large</code>.
     *         	If the host supports multiple instance types, no <b>instanceType</b>
     *         	is returned.</p>
     */
    InstanceType?: string;
    /**
     * <p>The number of sockets on the Dedicated Host.</p>
     */
    Sockets?: number;
    /**
     * <p>The total number of vCPUs on the Dedicated Host.</p>
     */
    TotalVCpus?: number;
}
export declare namespace HostProperties {
    const filterSensitiveLog: (obj: HostProperties) => any;
    const isa: (o: any) => o is HostProperties;
}
export declare type HostRecovery = "off" | "on";
/**
 * <p>Details about the Dedicated Host Reservation and associated Dedicated
 *             Hosts.</p>
 */
export interface HostReservation {
    __type?: "HostReservation";
    /**
     * <p>The number of Dedicated Hosts the reservation is associated with.</p>
     */
    Count?: number;
    /**
     * <p>The currency in which the <code>upfrontPrice</code> and <code>hourlyPrice</code>
     *             amounts are specified. At this time, the only supported currency is
     *             <code>USD</code>.</p>
     */
    CurrencyCode?: CurrencyCodeValues | string;
    /**
     * <p>The length of the reservation's term, specified in seconds. Can be <code>31536000
     *                 (1 year)</code> | <code>94608000 (3 years)</code>.</p>
     */
    Duration?: number;
    /**
     * <p>The date and time that the reservation ends.</p>
     */
    End?: Date;
    /**
     * <p>The IDs of the Dedicated Hosts associated with the reservation.</p>
     */
    HostIdSet?: string[];
    /**
     * <p>The ID of the reservation that specifies the associated Dedicated Hosts.</p>
     */
    HostReservationId?: string;
    /**
     * <p>The hourly price of the reservation.</p>
     */
    HourlyPrice?: string;
    /**
     * <p>The instance family of the Dedicated Host Reservation. The instance family on the
     *             Dedicated Host must be the same in order for it to benefit from the
     *             reservation.</p>
     */
    InstanceFamily?: string;
    /**
     * <p>The ID of the reservation. This remains the same regardless of which Dedicated
     *             Hosts are associated with it.</p>
     */
    OfferingId?: string;
    /**
     * <p>The payment option selected for this reservation.</p>
     */
    PaymentOption?: PaymentOption | string;
    /**
     * <p>The date and time that the reservation started.</p>
     */
    Start?: Date;
    /**
     * <p>The state of the reservation.</p>
     */
    State?: ReservationState | string;
    /**
     * <p>Any tags assigned to the Dedicated Host Reservation.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The upfront price of the reservation.</p>
     */
    UpfrontPrice?: string;
}
export declare namespace HostReservation {
    const filterSensitiveLog: (obj: HostReservation) => any;
    const isa: (o: any) => o is HostReservation;
}
export declare type HostTenancy = "dedicated" | "host";
export declare enum HttpTokensState {
    optional = "optional",
    required = "required"
}
export declare type HypervisorType = "ovm" | "xen";
/**
 * <p>Describes an IAM instance profile.</p>
 */
export interface IamInstanceProfile {
    __type?: "IamInstanceProfile";
    /**
     * <p>The Amazon Resource Name (ARN) of the instance profile.</p>
     */
    Arn?: string;
    /**
     * <p>The ID of the instance profile.</p>
     */
    Id?: string;
}
export declare namespace IamInstanceProfile {
    const filterSensitiveLog: (obj: IamInstanceProfile) => any;
    const isa: (o: any) => o is IamInstanceProfile;
}
/**
 * <p>Describes an association between an IAM instance profile and an instance.</p>
 */
export interface IamInstanceProfileAssociation {
    __type?: "IamInstanceProfileAssociation";
    /**
     * <p>The ID of the association.</p>
     */
    AssociationId?: string;
    /**
     * <p>The IAM instance profile.</p>
     */
    IamInstanceProfile?: IamInstanceProfile;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The state of the association.</p>
     */
    State?: IamInstanceProfileAssociationState | string;
    /**
     * <p>The time the IAM instance profile was associated with the instance.</p>
     */
    Timestamp?: Date;
}
export declare namespace IamInstanceProfileAssociation {
    const filterSensitiveLog: (obj: IamInstanceProfileAssociation) => any;
    const isa: (o: any) => o is IamInstanceProfileAssociation;
}
export declare enum IamInstanceProfileAssociationState {
    ASSOCIATED = "associated",
    ASSOCIATING = "associating",
    DISASSOCIATED = "disassociated",
    DISASSOCIATING = "disassociating"
}
/**
 * <p>Describes an IAM instance profile.</p>
 */
export interface IamInstanceProfileSpecification {
    __type?: "IamInstanceProfileSpecification";
    /**
     * <p>The Amazon Resource Name (ARN) of the instance profile.</p>
     */
    Arn?: string;
    /**
     * <p>The name of the instance profile.</p>
     */
    Name?: string;
}
export declare namespace IamInstanceProfileSpecification {
    const filterSensitiveLog: (obj: IamInstanceProfileSpecification) => any;
    const isa: (o: any) => o is IamInstanceProfileSpecification;
}
/**
 * <p>Describes the ICMP type and code.</p>
 */
export interface IcmpTypeCode {
    __type?: "IcmpTypeCode";
    /**
     * <p>The ICMP code. A value of -1 means all codes for the specified ICMP type.</p>
     */
    Code?: number;
    /**
     * <p>The ICMP type. A value of -1 means all types.</p>
     */
    Type?: number;
}
export declare namespace IcmpTypeCode {
    const filterSensitiveLog: (obj: IcmpTypeCode) => any;
    const isa: (o: any) => o is IcmpTypeCode;
}
/**
 * <p>Describes the ID format for a resource.</p>
 */
export interface IdFormat {
    __type?: "IdFormat";
    /**
     * <p>The date in UTC at which you are permanently switched over to using longer IDs. If a deadline is not yet available for this resource type, this field is not returned.</p>
     */
    Deadline?: Date;
    /**
     * <p>The type of resource.</p>
     */
    Resource?: string;
    /**
     * <p>Indicates whether longer IDs (17-character IDs) are enabled for the resource.</p>
     */
    UseLongIds?: boolean;
}
export declare namespace IdFormat {
    const filterSensitiveLog: (obj: IdFormat) => any;
    const isa: (o: any) => o is IdFormat;
}
/**
 * <p>The internet key exchange (IKE) version permitted for the VPN tunnel.</p>
 */
export interface IKEVersionsListValue {
    __type?: "IKEVersionsListValue";
    /**
     * <p>The IKE version.</p>
     */
    Value?: string;
}
export declare namespace IKEVersionsListValue {
    const filterSensitiveLog: (obj: IKEVersionsListValue) => any;
    const isa: (o: any) => o is IKEVersionsListValue;
}
/**
 * <p>The IKE version that is permitted for the VPN tunnel.</p>
 */
export interface IKEVersionsRequestListValue {
    __type?: "IKEVersionsRequestListValue";
    /**
     * <p>The IKE version.</p>
     */
    Value?: string;
}
export declare namespace IKEVersionsRequestListValue {
    const filterSensitiveLog: (obj: IKEVersionsRequestListValue) => any;
    const isa: (o: any) => o is IKEVersionsRequestListValue;
}
/**
 * <p>Describes an image.</p>
 */
export interface Image {
    __type?: "Image";
    /**
     * <p>The architecture of the image.</p>
     */
    Architecture?: ArchitectureValues | string;
    /**
     * <p>Any block device mapping entries.</p>
     */
    BlockDeviceMappings?: BlockDeviceMapping[];
    /**
     * <p>The date and time the image was created.</p>
     */
    CreationDate?: string;
    /**
     * <p>The description of the AMI that was provided during image creation.</p>
     */
    Description?: string;
    /**
     * <p>Specifies whether enhanced networking with ENA is enabled.</p>
     */
    EnaSupport?: boolean;
    /**
     * <p>The hypervisor type of the image.</p>
     */
    Hypervisor?: HypervisorType | string;
    /**
     * <p>The ID of the AMI.</p>
     */
    ImageId?: string;
    /**
     * <p>The location of the AMI.</p>
     */
    ImageLocation?: string;
    /**
     * <p>The AWS account alias (for example, <code>amazon</code>, <code>self</code>) or
     *         the AWS account ID of the AMI owner.</p>
     */
    ImageOwnerAlias?: string;
    /**
     * <p>The type of image.</p>
     */
    ImageType?: ImageTypeValues | string;
    /**
     * <p>The kernel associated with the image, if any. Only applicable for machine images.</p>
     */
    KernelId?: string;
    /**
     * <p>The name of the AMI that was provided during image creation.</p>
     */
    Name?: string;
    /**
     * <p>The AWS account ID of the image owner.</p>
     */
    OwnerId?: string;
    /**
     * <p>This value is set to <code>windows</code> for Windows AMIs; otherwise, it is blank.</p>
     */
    Platform?: PlatformValues | string;
    /**
     * <p>Any product codes associated with the AMI.</p>
     */
    ProductCodes?: ProductCode[];
    /**
     * <p>Indicates whether the image has public launch permissions. The value is <code>true</code> if
     * 				this image has public launch permissions or <code>false</code>
     * 				if it has only implicit and explicit launch permissions.</p>
     */
    Public?: boolean;
    /**
     * <p>The RAM disk associated with the image, if any. Only applicable for machine images.</p>
     */
    RamdiskId?: string;
    /**
     * <p>The device name of the root device volume (for example, <code>/dev/sda1</code>).</p>
     */
    RootDeviceName?: string;
    /**
     * <p>The type of root device used by the AMI. The AMI can use an EBS volume or an instance store volume.</p>
     */
    RootDeviceType?: DeviceType | string;
    /**
     * <p>Specifies whether enhanced networking with the Intel 82599 Virtual Function interface is enabled.</p>
     */
    SriovNetSupport?: string;
    /**
     * <p>The current state of the AMI. If the state is <code>available</code>, the image is successfully registered and can be used to launch an instance.</p>
     */
    State?: ImageState | string;
    /**
     * <p>The reason for the state change.</p>
     */
    StateReason?: StateReason;
    /**
     * <p>Any tags assigned to the image.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The type of virtualization of the AMI.</p>
     */
    VirtualizationType?: VirtualizationType | string;
}
export declare namespace Image {
    const filterSensitiveLog: (obj: Image) => any;
    const isa: (o: any) => o is Image;
}
/**
 * <p>Describes an image attribute.</p>
 */
export interface ImageAttribute {
    __type?: "ImageAttribute";
    /**
     * <p>The block device mapping entries.</p>
     */
    BlockDeviceMappings?: BlockDeviceMapping[];
    /**
     * <p>A description for the AMI.</p>
     */
    Description?: AttributeValue;
    /**
     * <p>The ID of the AMI.</p>
     */
    ImageId?: string;
    /**
     * <p>The kernel ID.</p>
     */
    KernelId?: AttributeValue;
    /**
     * <p>The launch permissions.</p>
     */
    LaunchPermissions?: LaunchPermission[];
    /**
     * <p>The product codes.</p>
     */
    ProductCodes?: ProductCode[];
    /**
     * <p>The RAM disk ID.</p>
     */
    RamdiskId?: AttributeValue;
    /**
     * <p>Indicates whether enhanced networking with the Intel 82599 Virtual Function interface is enabled.</p>
     */
    SriovNetSupport?: AttributeValue;
}
export declare namespace ImageAttribute {
    const filterSensitiveLog: (obj: ImageAttribute) => any;
    const isa: (o: any) => o is ImageAttribute;
}
export declare type ImageAttributeName = "blockDeviceMapping" | "description" | "kernel" | "launchPermission" | "productCodes" | "ramdisk" | "sriovNetSupport";
/**
 * <p>Describes the disk container object for an import image task.</p>
 */
export interface ImageDiskContainer {
    __type?: "ImageDiskContainer";
    /**
     * <p>The description of the disk image.</p>
     */
    Description?: string;
    /**
     * <p>The block device mapping for the disk.</p>
     */
    DeviceName?: string;
    /**
     * <p>The format of the disk image being imported.</p>
     *          <p>Valid values: <code>VHD</code> | <code>VMDK</code> | <code>OVA</code>
     *          </p>
     */
    Format?: string;
    /**
     * <p>The ID of the EBS snapshot to be used for importing the snapshot.</p>
     */
    SnapshotId?: string;
    /**
     * <p>The URL to the Amazon S3-based disk image being imported. The URL can either be a https URL (https://..) or an
     *    Amazon S3 URL (s3://..)</p>
     */
    Url?: string;
    /**
     * <p>The S3 bucket for the disk image.</p>
     */
    UserBucket?: UserBucket;
}
export declare namespace ImageDiskContainer {
    const filterSensitiveLog: (obj: ImageDiskContainer) => any;
    const isa: (o: any) => o is ImageDiskContainer;
}
export declare type ImageState = "available" | "deregistered" | "error" | "failed" | "invalid" | "pending" | "transient";
export declare type ImageTypeValues = "kernel" | "machine" | "ramdisk";
export interface ImportClientVpnClientCertificateRevocationListRequest {
    __type?: "ImportClientVpnClientCertificateRevocationListRequest";
    /**
     * <p>The client certificate revocation list file. For more information, see <a href="https://docs.aws.amazon.com/vpn/latest/clientvpn-admin/cvpn-working-certificates.html#cvpn-working-certificates-generate">Generate a Client Certificate Revocation List</a> in the
     * 				<i>AWS Client VPN Administrator Guide</i>.</p>
     */
    CertificateRevocationList: string | undefined;
    /**
     * <p>The ID of the Client VPN endpoint to which the client certificate revocation list applies.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace ImportClientVpnClientCertificateRevocationListRequest {
    const filterSensitiveLog: (obj: ImportClientVpnClientCertificateRevocationListRequest) => any;
    const isa: (o: any) => o is ImportClientVpnClientCertificateRevocationListRequest;
}
export interface ImportClientVpnClientCertificateRevocationListResult {
    __type?: "ImportClientVpnClientCertificateRevocationListResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace ImportClientVpnClientCertificateRevocationListResult {
    const filterSensitiveLog: (obj: ImportClientVpnClientCertificateRevocationListResult) => any;
    const isa: (o: any) => o is ImportClientVpnClientCertificateRevocationListResult;
}
/**
 * <p>The request information of license configurations.</p>
 */
export interface ImportImageLicenseConfigurationRequest {
    __type?: "ImportImageLicenseConfigurationRequest";
    /**
     * <p>The ARN of a license configuration.</p>
     */
    LicenseConfigurationArn?: string;
}
export declare namespace ImportImageLicenseConfigurationRequest {
    const filterSensitiveLog: (obj: ImportImageLicenseConfigurationRequest) => any;
    const isa: (o: any) => o is ImportImageLicenseConfigurationRequest;
}
/**
 * <p> The response information for license configurations.</p>
 */
export interface ImportImageLicenseConfigurationResponse {
    __type?: "ImportImageLicenseConfigurationResponse";
    /**
     * <p>The ARN of a license configuration.</p>
     */
    LicenseConfigurationArn?: string;
}
export declare namespace ImportImageLicenseConfigurationResponse {
    const filterSensitiveLog: (obj: ImportImageLicenseConfigurationResponse) => any;
    const isa: (o: any) => o is ImportImageLicenseConfigurationResponse;
}
export interface ImportImageRequest {
    __type?: "ImportImageRequest";
    /**
     * <p>The architecture of the virtual machine.</p>
     *          <p>Valid values: <code>i386</code> | <code>x86_64</code> | <code>arm64</code>
     *          </p>
     */
    Architecture?: string;
    /**
     * <p>The client-specific data.</p>
     */
    ClientData?: ClientData;
    /**
     * <p>The token to enable idempotency for VM import requests.</p>
     */
    ClientToken?: string;
    /**
     * <p>A description string for the import image task.</p>
     */
    Description?: string;
    /**
     * <p>Information about the disk containers.</p>
     */
    DiskContainers?: ImageDiskContainer[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Specifies whether the destination AMI of the imported image should be encrypted. The default CMK for EBS is used
     *    unless you specify a non-default AWS Key Management Service (AWS KMS) CMK using <code>KmsKeyId</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the
     *     <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>The target hypervisor platform.</p>
     *          <p>Valid values: <code>xen</code>
     *          </p>
     */
    Hypervisor?: string;
    /**
     * <p>An identifier for the symmetric AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating the
     *    encrypted AMI. This parameter is only required if you want to use a non-default CMK; if this
     *    parameter is not specified, the default CMK for EBS is used. If a <code>KmsKeyId</code> is
     *    specified, the <code>Encrypted</code> flag must also be set. </p>
     *          <p>The CMK identifier may be provided in any of the following formats: </p>
     *          <ul>
     *             <li>
     *                <p>Key ID</p>
     *             </li>
     *             <li>
     *                <p>Key alias. The alias ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>alias</code> namespace, and then the CMK alias. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>.</p>
     *             </li>
     *             <li>
     *                <p>ARN using key ID. The ID ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>key</code> namespace, and then the CMK ID. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:key/<i>abcd1234-a123-456a-a12b-a123b4cd56ef</i>.</p>
     *             </li>
     *             <li>
     *                <p>ARN using key alias. The alias ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>alias</code> namespace, and then the CMK alias. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>. </p>
     *             </li>
     *          </ul>
     *          <p>AWS parses <code>KmsKeyId</code> asynchronously, meaning that the action you call may appear to complete even
     *    though you provided an invalid identifier. This action will eventually report failure. </p>
     *          <p>The specified CMK must exist in the Region that the AMI is being copied to.</p>
     *          <p>Amazon EBS does not support asymmetric CMKs.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>The ARNs of the license configurations.</p>
     */
    LicenseSpecifications?: ImportImageLicenseConfigurationRequest[];
    /**
     * <p>The license type to be used for the Amazon Machine Image (AMI) after importing.</p>
     *          <p>By default, we detect the source-system operating system (OS) and apply the appropriate license. Specify
     *     <code>AWS</code> to replace the source-system license with an AWS license, if appropriate. Specify <code>BYOL</code>
     *    to retain the source-system license, if appropriate.</p>
     *          <p>To use <code>BYOL</code>, you must have existing licenses with rights to use these licenses in a third party
     *    cloud, such as AWS. For more information, see <a href="https://docs.aws.amazon.com/vm-import/latest/userguide/vmimport-image-import.html#prerequisites-image">Prerequisites</a> in the
     *    VM Import/Export User Guide.</p>
     */
    LicenseType?: string;
    /**
     * <p>The operating system of the virtual machine.</p>
     *          <p>Valid values: <code>Windows</code> | <code>Linux</code>
     *          </p>
     */
    Platform?: string;
    /**
     * <p>The name of the role to use when not using the default role, 'vmimport'.</p>
     */
    RoleName?: string;
}
export declare namespace ImportImageRequest {
    const filterSensitiveLog: (obj: ImportImageRequest) => any;
    const isa: (o: any) => o is ImportImageRequest;
}
export interface ImportImageResult {
    __type?: "ImportImageResult";
    /**
     * <p>The architecture of the virtual machine.</p>
     */
    Architecture?: string;
    /**
     * <p>A description of the import task.</p>
     */
    Description?: string;
    /**
     * <p>Indicates whether the AMI is encypted.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>The target hypervisor of the import task.</p>
     */
    Hypervisor?: string;
    /**
     * <p>The ID of the Amazon Machine Image (AMI) created by the import task.</p>
     */
    ImageId?: string;
    /**
     * <p>The task ID of the import image task.</p>
     */
    ImportTaskId?: string;
    /**
     * <p>The identifier for the symmetric AWS Key Management Service (AWS KMS) customer master key
     *    (CMK) that was used to create the encrypted AMI.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>The ARNs of the license configurations.</p>
     */
    LicenseSpecifications?: ImportImageLicenseConfigurationResponse[];
    /**
     * <p>The license type of the virtual machine.</p>
     */
    LicenseType?: string;
    /**
     * <p>The operating system of the virtual machine.</p>
     */
    Platform?: string;
    /**
     * <p>The progress of the task.</p>
     */
    Progress?: string;
    /**
     * <p>Information about the snapshots.</p>
     */
    SnapshotDetails?: SnapshotDetail[];
    /**
     * <p>A brief status of the task.</p>
     */
    Status?: string;
    /**
     * <p>A detailed status message of the import task.</p>
     */
    StatusMessage?: string;
}
export declare namespace ImportImageResult {
    const filterSensitiveLog: (obj: ImportImageResult) => any;
    const isa: (o: any) => o is ImportImageResult;
}
/**
 * <p>Describes an import image task.</p>
 */
export interface ImportImageTask {
    __type?: "ImportImageTask";
    /**
     * <p>The architecture of the virtual machine.</p>
     *          <p>Valid values: <code>i386</code> | <code>x86_64</code> | <code>arm64</code>
     *          </p>
     */
    Architecture?: string;
    /**
     * <p>A description of the import task.</p>
     */
    Description?: string;
    /**
     * <p>Indicates whether the image is encrypted.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>The target hypervisor for the import task.</p>
     *          <p>Valid values: <code>xen</code>
     *          </p>
     */
    Hypervisor?: string;
    /**
     * <p>The ID of the Amazon Machine Image (AMI) of the imported virtual machine.</p>
     */
    ImageId?: string;
    /**
     * <p>The ID of the import image task.</p>
     */
    ImportTaskId?: string;
    /**
     * <p>The identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) that was used to create
     *    the encrypted image.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>The ARNs of the license configurations that are associated with the import image task.</p>
     */
    LicenseSpecifications?: ImportImageLicenseConfigurationResponse[];
    /**
     * <p>The license type of the virtual machine.</p>
     */
    LicenseType?: string;
    /**
     * <p>The description string for the import image task.</p>
     */
    Platform?: string;
    /**
     * <p>The percentage of progress of the import image task.</p>
     */
    Progress?: string;
    /**
     * <p>Information about the snapshots.</p>
     */
    SnapshotDetails?: SnapshotDetail[];
    /**
     * <p>A brief status for the import image task.</p>
     */
    Status?: string;
    /**
     * <p>A descriptive status message for the import image task.</p>
     */
    StatusMessage?: string;
    /**
     * <p>The tags for the import image task.</p>
     */
    Tags?: Tag[];
}
export declare namespace ImportImageTask {
    const filterSensitiveLog: (obj: ImportImageTask) => any;
    const isa: (o: any) => o is ImportImageTask;
}
/**
 * <p>Describes the launch specification for VM import.</p>
 */
export interface ImportInstanceLaunchSpecification {
    __type?: "ImportInstanceLaunchSpecification";
    /**
     * <p>Reserved.</p>
     */
    AdditionalInfo?: string;
    /**
     * <p>The architecture of the instance.</p>
     */
    Architecture?: ArchitectureValues | string;
    /**
     * <p>The security group IDs.</p>
     */
    GroupIds?: string[];
    /**
     * <p>The security group names.</p>
     */
    GroupNames?: string[];
    /**
     * <p>Indicates whether an instance stops or terminates when you initiate shutdown from the instance (using the
     *    operating system command for system shutdown).</p>
     */
    InstanceInitiatedShutdownBehavior?: ShutdownBehavior | string;
    /**
     * <p>The instance type. For more information about the instance types that you can import, see <a href="https://docs.aws.amazon.com/vm-import/latest/userguide/vmie_prereqs.html#vmimport-instance-types">Instance Types</a> in the
     *    VM Import/Export User Guide.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>Indicates whether monitoring is enabled.</p>
     */
    Monitoring?: boolean;
    /**
     * <p>The placement information for the instance.</p>
     */
    Placement?: Placement;
    /**
     * <p>[EC2-VPC] An available IP address from the IP address range of the subnet.</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>[EC2-VPC] The ID of the subnet in which to launch the instance.</p>
     */
    SubnetId?: string;
    /**
     * <p>The Base64-encoded user data to make available to the instance.</p>
     */
    UserData?: UserData;
}
export declare namespace ImportInstanceLaunchSpecification {
    const filterSensitiveLog: (obj: ImportInstanceLaunchSpecification) => any;
    const isa: (o: any) => o is ImportInstanceLaunchSpecification;
}
export interface ImportInstanceRequest {
    __type?: "ImportInstanceRequest";
    /**
     * <p>A description for the instance being imported.</p>
     */
    Description?: string;
    /**
     * <p>The disk image.</p>
     */
    DiskImages?: DiskImage[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The launch specification.</p>
     */
    LaunchSpecification?: ImportInstanceLaunchSpecification;
    /**
     * <p>The instance operating system.</p>
     */
    Platform: PlatformValues | string | undefined;
}
export declare namespace ImportInstanceRequest {
    const filterSensitiveLog: (obj: ImportInstanceRequest) => any;
    const isa: (o: any) => o is ImportInstanceRequest;
}
export interface ImportInstanceResult {
    __type?: "ImportInstanceResult";
    /**
     * <p>Information about the conversion task.</p>
     */
    ConversionTask?: ConversionTask;
}
export declare namespace ImportInstanceResult {
    const filterSensitiveLog: (obj: ImportInstanceResult) => any;
    const isa: (o: any) => o is ImportInstanceResult;
}
/**
 * <p>Describes an import instance task.</p>
 */
export interface ImportInstanceTaskDetails {
    __type?: "ImportInstanceTaskDetails";
    /**
     * <p>A description of the task.</p>
     */
    Description?: string;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The instance operating system.</p>
     */
    Platform?: PlatformValues | string;
    /**
     * <p>The volumes.</p>
     */
    Volumes?: ImportInstanceVolumeDetailItem[];
}
export declare namespace ImportInstanceTaskDetails {
    const filterSensitiveLog: (obj: ImportInstanceTaskDetails) => any;
    const isa: (o: any) => o is ImportInstanceTaskDetails;
}
/**
 * <p>Describes an import volume task.</p>
 */
export interface ImportInstanceVolumeDetailItem {
    __type?: "ImportInstanceVolumeDetailItem";
    /**
     * <p>The Availability Zone where the resulting instance will reside.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The number of bytes converted so far.</p>
     */
    BytesConverted?: number;
    /**
     * <p>A description of the task.</p>
     */
    Description?: string;
    /**
     * <p>The image.</p>
     */
    Image?: DiskImageDescription;
    /**
     * <p>The status of the import of this particular disk image.</p>
     */
    Status?: string;
    /**
     * <p>The status information or errors related to the disk image.</p>
     */
    StatusMessage?: string;
    /**
     * <p>The volume.</p>
     */
    Volume?: DiskImageVolumeDescription;
}
export declare namespace ImportInstanceVolumeDetailItem {
    const filterSensitiveLog: (obj: ImportInstanceVolumeDetailItem) => any;
    const isa: (o: any) => o is ImportInstanceVolumeDetailItem;
}
export interface ImportKeyPairRequest {
    __type?: "ImportKeyPairRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>A unique name for the key pair.</p>
     */
    KeyName: string | undefined;
    /**
     * <p>The public key. For API calls, the text must be base64-encoded. For command line tools, base64 encoding is performed for you.</p>
     */
    PublicKeyMaterial: Uint8Array | undefined;
}
export declare namespace ImportKeyPairRequest {
    const filterSensitiveLog: (obj: ImportKeyPairRequest) => any;
    const isa: (o: any) => o is ImportKeyPairRequest;
}
export interface ImportKeyPairResult {
    __type?: "ImportKeyPairResult";
    /**
     * <p>The MD5 public key fingerprint as specified in section 4 of RFC 4716.</p>
     */
    KeyFingerprint?: string;
    /**
     * <p>The key pair name you provided.</p>
     */
    KeyName?: string;
}
export declare namespace ImportKeyPairResult {
    const filterSensitiveLog: (obj: ImportKeyPairResult) => any;
    const isa: (o: any) => o is ImportKeyPairResult;
}
export interface ImportSnapshotRequest {
    __type?: "ImportSnapshotRequest";
    /**
     * <p>The client-specific data.</p>
     */
    ClientData?: ClientData;
    /**
     * <p>Token to enable idempotency for VM import requests.</p>
     */
    ClientToken?: string;
    /**
     * <p>The description string for the import snapshot task.</p>
     */
    Description?: string;
    /**
     * <p>Information about the disk container.</p>
     */
    DiskContainer?: SnapshotDiskContainer;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Specifies whether the destination snapshot of the imported image should be encrypted. The default CMK for EBS is
     *    used unless you specify a non-default AWS Key Management Service (AWS KMS) CMK using <code>KmsKeyId</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html">Amazon EBS Encryption</a> in the
     *     <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>An identifier for the symmetric AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating the
     *    encrypted snapshot. This parameter is only required if you want to use a non-default CMK; if this
     *    parameter is not specified, the default CMK for EBS is used. If a <code>KmsKeyId</code> is
     *    specified, the <code>Encrypted</code> flag must also be set. </p>
     *          <p>The CMK identifier may be provided in any of the following formats: </p>
     *          <ul>
     *             <li>
     *                <p>Key ID</p>
     *             </li>
     *             <li>
     *                <p>Key alias. The alias ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>alias</code> namespace, and then the CMK alias. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>.</p>
     *             </li>
     *             <li>
     *                <p>ARN using key ID. The ID ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>key</code> namespace, and then the CMK ID. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:key/<i>abcd1234-a123-456a-a12b-a123b4cd56ef</i>.</p>
     *             </li>
     *             <li>
     *                <p>ARN using key alias. The alias ARN contains the <code>arn:aws:kms</code> namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the <code>alias</code> namespace, and then the CMK alias. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>. </p>
     *             </li>
     *          </ul>
     *          <p>AWS parses <code>KmsKeyId</code> asynchronously, meaning that the action you call may appear to complete even
     *    though you provided an invalid identifier. This action will eventually report failure. </p>
     *          <p>The specified CMK must exist in the Region that the snapshot is being copied to.</p>
     *          <p>Amazon EBS does not support asymmetric CMKs.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>The name of the role to use when not using the default role, 'vmimport'.</p>
     */
    RoleName?: string;
}
export declare namespace ImportSnapshotRequest {
    const filterSensitiveLog: (obj: ImportSnapshotRequest) => any;
    const isa: (o: any) => o is ImportSnapshotRequest;
}
export interface ImportSnapshotResult {
    __type?: "ImportSnapshotResult";
    /**
     * <p>A description of the import snapshot task.</p>
     */
    Description?: string;
    /**
     * <p>The ID of the import snapshot task.</p>
     */
    ImportTaskId?: string;
    /**
     * <p>Information about the import snapshot task.</p>
     */
    SnapshotTaskDetail?: SnapshotTaskDetail;
}
export declare namespace ImportSnapshotResult {
    const filterSensitiveLog: (obj: ImportSnapshotResult) => any;
    const isa: (o: any) => o is ImportSnapshotResult;
}
/**
 * <p>Describes an import snapshot task.</p>
 */
export interface ImportSnapshotTask {
    __type?: "ImportSnapshotTask";
    /**
     * <p>A description of the import snapshot task.</p>
     */
    Description?: string;
    /**
     * <p>The ID of the import snapshot task.</p>
     */
    ImportTaskId?: string;
    /**
     * <p>Describes an import snapshot task.</p>
     */
    SnapshotTaskDetail?: SnapshotTaskDetail;
    /**
     * <p>The tags for the import snapshot task.</p>
     */
    Tags?: Tag[];
}
export declare namespace ImportSnapshotTask {
    const filterSensitiveLog: (obj: ImportSnapshotTask) => any;
    const isa: (o: any) => o is ImportSnapshotTask;
}
export interface ImportVolumeRequest {
    __type?: "ImportVolumeRequest";
    /**
     * <p>The Availability Zone for the resulting EBS volume.</p>
     */
    AvailabilityZone: string | undefined;
    /**
     * <p>A description of the volume.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The disk image.</p>
     */
    Image: DiskImageDetail | undefined;
    /**
     * <p>The volume size.</p>
     */
    Volume: VolumeDetail | undefined;
}
export declare namespace ImportVolumeRequest {
    const filterSensitiveLog: (obj: ImportVolumeRequest) => any;
    const isa: (o: any) => o is ImportVolumeRequest;
}
export interface ImportVolumeResult {
    __type?: "ImportVolumeResult";
    /**
     * <p>Information about the conversion task.</p>
     */
    ConversionTask?: ConversionTask;
}
export declare namespace ImportVolumeResult {
    const filterSensitiveLog: (obj: ImportVolumeResult) => any;
    const isa: (o: any) => o is ImportVolumeResult;
}
/**
 * <p>Describes an import volume task.</p>
 */
export interface ImportVolumeTaskDetails {
    __type?: "ImportVolumeTaskDetails";
    /**
     * <p>The Availability Zone where the resulting volume will reside.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The number of bytes converted so far.</p>
     */
    BytesConverted?: number;
    /**
     * <p>The description you provided when starting the import volume task.</p>
     */
    Description?: string;
    /**
     * <p>The image.</p>
     */
    Image?: DiskImageDescription;
    /**
     * <p>The volume.</p>
     */
    Volume?: DiskImageVolumeDescription;
}
export declare namespace ImportVolumeTaskDetails {
    const filterSensitiveLog: (obj: ImportVolumeTaskDetails) => any;
    const isa: (o: any) => o is ImportVolumeTaskDetails;
}
/**
 * <p>Describes the Inference accelerators for the instance type.</p>
 */
export interface InferenceAcceleratorInfo {
    __type?: "InferenceAcceleratorInfo";
    /**
     * <p>Describes the Inference accelerators for the instance type.</p>
     */
    Accelerators?: InferenceDeviceInfo[];
}
export declare namespace InferenceAcceleratorInfo {
    const filterSensitiveLog: (obj: InferenceAcceleratorInfo) => any;
    const isa: (o: any) => o is InferenceAcceleratorInfo;
}
/**
 * <p>Describes the Inference accelerators for the instance type.</p>
 */
export interface InferenceDeviceInfo {
    __type?: "InferenceDeviceInfo";
    /**
     * <p>The number of Inference accelerators for the instance type.</p>
     */
    Count?: number;
    /**
     * <p>The manufacturer of the Inference accelerator.</p>
     */
    Manufacturer?: string;
    /**
     * <p>The name of the Inference accelerator.</p>
     */
    Name?: string;
}
export declare namespace InferenceDeviceInfo {
    const filterSensitiveLog: (obj: InferenceDeviceInfo) => any;
    const isa: (o: any) => o is InferenceDeviceInfo;
}
/**
 * <p>Describes an instance.</p>
 */
export interface Instance {
    __type?: "Instance";
    /**
     * <p>The AMI launch index, which can be used to find this instance in the launch
     *             group.</p>
     */
    AmiLaunchIndex?: number;
    /**
     * <p>The architecture of the image.</p>
     */
    Architecture?: ArchitectureValues | string;
    /**
     * <p>Any block device mapping entries for the instance.</p>
     */
    BlockDeviceMappings?: InstanceBlockDeviceMapping[];
    /**
     * <p>The ID of the Capacity Reservation.</p>
     */
    CapacityReservationId?: string;
    /**
     * <p>Information about the Capacity Reservation targeting option.</p>
     */
    CapacityReservationSpecification?: CapacityReservationSpecificationResponse;
    /**
     * <p>The idempotency token you provided when you launched the instance, if
     *             applicable.</p>
     */
    ClientToken?: string;
    /**
     * <p>The CPU options for the instance.</p>
     */
    CpuOptions?: CpuOptions;
    /**
     * <p>Indicates whether the instance is optimized for Amazon EBS I/O. This optimization
     *             provides dedicated throughput to Amazon EBS and an optimized configuration stack to
     *             provide optimal I/O performance. This optimization isn't available with all instance
     *             types. Additional usage charges apply when using an EBS Optimized instance.</p>
     */
    EbsOptimized?: boolean;
    /**
     * <p>The Elastic GPU associated with the instance.</p>
     */
    ElasticGpuAssociations?: ElasticGpuAssociation[];
    /**
     * <p> The elastic inference accelerator associated with the instance.</p>
     */
    ElasticInferenceAcceleratorAssociations?: ElasticInferenceAcceleratorAssociation[];
    /**
     * <p>Specifies whether enhanced networking with ENA is enabled.</p>
     */
    EnaSupport?: boolean;
    /**
     * <p>Indicates whether the instance is enabled for hibernation.</p>
     */
    HibernationOptions?: HibernationOptions;
    /**
     * <p>The hypervisor type of the instance.</p>
     */
    Hypervisor?: HypervisorType | string;
    /**
     * <p>The IAM instance profile associated with the instance, if applicable.</p>
     */
    IamInstanceProfile?: IamInstanceProfile;
    /**
     * <p>The ID of the AMI used to launch the instance.</p>
     */
    ImageId?: string;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>Indicates whether this is a Spot Instance or a Scheduled Instance.</p>
     */
    InstanceLifecycle?: InstanceLifecycleType | string;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The kernel associated with this instance, if applicable.</p>
     */
    KernelId?: string;
    /**
     * <p>The name of the key pair, if this instance was launched with an associated key
     *             pair.</p>
     */
    KeyName?: string;
    /**
     * <p>The time the instance was launched.</p>
     */
    LaunchTime?: Date;
    /**
     * <p>The license configurations.</p>
     */
    Licenses?: LicenseConfiguration[];
    /**
     * <p>The metadata options for the instance.</p>
     */
    MetadataOptions?: InstanceMetadataOptionsResponse;
    /**
     * <p>The monitoring for the instance.</p>
     */
    Monitoring?: Monitoring;
    /**
     * <p>[EC2-VPC] The network interfaces for the instance.</p>
     */
    NetworkInterfaces?: InstanceNetworkInterface[];
    /**
     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
     */
    OutpostArn?: string;
    /**
     * <p>The location where the instance launched, if applicable.</p>
     */
    Placement?: Placement;
    /**
     * <p>The value is <code>Windows</code> for Windows instances; otherwise blank.</p>
     */
    Platform?: PlatformValues | string;
    /**
     * <p>(IPv4 only) The private DNS hostname name assigned to the instance. This DNS hostname
     *             can only be used inside the Amazon EC2 network. This name is not available until the
     *             instance enters the <code>running</code> state. </p>
     *         <p>[EC2-VPC] The Amazon-provided DNS server resolves Amazon-provided private DNS
     *             hostnames if you've enabled DNS resolution and DNS hostnames in your VPC. If you are not
     *             using the Amazon-provided DNS server in your VPC, your custom domain name servers must
     *             resolve the hostname as appropriate.</p>
     */
    PrivateDnsName?: string;
    /**
     * <p>The private IPv4 address assigned to the instance.</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>The product codes attached to this instance, if applicable.</p>
     */
    ProductCodes?: ProductCode[];
    /**
     * <p>(IPv4 only) The public DNS name assigned to the instance. This name is not available
     *             until the instance enters the <code>running</code> state. For EC2-VPC, this name is only
     *             available if you've enabled DNS hostnames for your VPC.</p>
     */
    PublicDnsName?: string;
    /**
     * <p>The public IPv4 address assigned to the instance, if applicable.</p>
     */
    PublicIpAddress?: string;
    /**
     * <p>The RAM disk associated with this instance, if applicable.</p>
     */
    RamdiskId?: string;
    /**
     * <p>The device name of the root device volume (for example,
     *             <code>/dev/sda1</code>).</p>
     */
    RootDeviceName?: string;
    /**
     * <p>The root device type used by the AMI. The AMI can use an EBS volume or an instance
     *             store volume.</p>
     */
    RootDeviceType?: DeviceType | string;
    /**
     * <p>The security groups for the instance.</p>
     */
    SecurityGroups?: GroupIdentifier[];
    /**
     * <p>Specifies whether to enable an instance launched in a VPC to perform NAT. This
     *             controls whether source/destination checking is enabled on the instance. A value of
     *                 <code>true</code> means that checking is enabled, and <code>false</code> means that
     *             checking is disabled. The value must be <code>false</code> for the instance to perform
     *             NAT. For more information, see <a href="https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_NAT_Instance.html">NAT Instances</a> in the
     *                 <i>Amazon Virtual Private Cloud User Guide</i>.</p>
     */
    SourceDestCheck?: boolean;
    /**
     * <p>If the request is a Spot Instance request, the ID of the request.</p>
     */
    SpotInstanceRequestId?: string;
    /**
     * <p>Specifies whether enhanced networking with the Intel 82599 Virtual Function interface
     *             is enabled.</p>
     */
    SriovNetSupport?: string;
    /**
     * <p>The current state of the instance.</p>
     */
    State?: InstanceState;
    /**
     * <p>The reason for the most recent state transition.</p>
     */
    StateReason?: StateReason;
    /**
     * <p>The reason for the most recent state transition. This might be an empty string.</p>
     */
    StateTransitionReason?: string;
    /**
     * <p>[EC2-VPC] The ID of the subnet in which the instance is running.</p>
     */
    SubnetId?: string;
    /**
     * <p>Any tags assigned to the instance.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The virtualization type of the instance.</p>
     */
    VirtualizationType?: VirtualizationType | string;
    /**
     * <p>[EC2-VPC] The ID of the VPC in which the instance is running.</p>
     */
    VpcId?: string;
}
export declare namespace Instance {
    const filterSensitiveLog: (obj: Instance) => any;
    const isa: (o: any) => o is Instance;
}
/**
 * <p>Describes an instance attribute.</p>
 */
export interface InstanceAttribute {
    __type?: "InstanceAttribute";
    /**
     * <p>The block device mapping of the instance.</p>
     */
    BlockDeviceMappings?: InstanceBlockDeviceMapping[];
    /**
     * <p>If the value is <code>true</code>, you can't terminate the instance through the Amazon
     *             EC2 console, CLI, or API; otherwise, you can.</p>
     */
    DisableApiTermination?: AttributeBooleanValue;
    /**
     * <p>Indicates whether the instance is optimized for Amazon EBS I/O.</p>
     */
    EbsOptimized?: AttributeBooleanValue;
    /**
     * <p>Indicates whether enhanced networking with ENA is enabled.</p>
     */
    EnaSupport?: AttributeBooleanValue;
    /**
     * <p>The security groups associated with the instance.</p>
     */
    Groups?: GroupIdentifier[];
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>Indicates whether an instance stops or terminates when you initiate shutdown from the
     *             instance (using the operating system command for system shutdown).</p>
     */
    InstanceInitiatedShutdownBehavior?: AttributeValue;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: AttributeValue;
    /**
     * <p>The kernel ID.</p>
     */
    KernelId?: AttributeValue;
    /**
     * <p>A list of product codes.</p>
     */
    ProductCodes?: ProductCode[];
    /**
     * <p>The RAM disk ID.</p>
     */
    RamdiskId?: AttributeValue;
    /**
     * <p>The device name of the root device volume (for example,
     *             <code>/dev/sda1</code>).</p>
     */
    RootDeviceName?: AttributeValue;
    /**
     * <p>Indicates whether source/destination checking is enabled. A value of <code>true</code>
     *             means that checking is enabled, and <code>false</code> means that checking is disabled.
     *             This value must be <code>false</code> for a NAT instance to perform NAT.</p>
     */
    SourceDestCheck?: AttributeBooleanValue;
    /**
     * <p>Indicates whether enhanced networking with the Intel 82599 Virtual Function interface
     *             is enabled.</p>
     */
    SriovNetSupport?: AttributeValue;
    /**
     * <p>The user data.</p>
     */
    UserData?: AttributeValue;
}
export declare namespace InstanceAttribute {
    const filterSensitiveLog: (obj: InstanceAttribute) => any;
    const isa: (o: any) => o is InstanceAttribute;
}
export declare type InstanceAttributeName = "blockDeviceMapping" | "disableApiTermination" | "ebsOptimized" | "enaSupport" | "groupSet" | "instanceInitiatedShutdownBehavior" | "instanceType" | "kernel" | "productCodes" | "ramdisk" | "rootDeviceName" | "sourceDestCheck" | "sriovNetSupport" | "userData";
/**
 * <p>Describes a block device mapping.</p>
 */
export interface InstanceBlockDeviceMapping {
    __type?: "InstanceBlockDeviceMapping";
    /**
     * <p>The device name (for example, <code>/dev/sdh</code> or <code>xvdh</code>).</p>
     */
    DeviceName?: string;
    /**
     * <p>Parameters used to automatically set up EBS volumes when the instance is
     *             launched.</p>
     */
    Ebs?: EbsInstanceBlockDevice;
}
export declare namespace InstanceBlockDeviceMapping {
    const filterSensitiveLog: (obj: InstanceBlockDeviceMapping) => any;
    const isa: (o: any) => o is InstanceBlockDeviceMapping;
}
/**
 * <p>Describes a block device mapping entry.</p>
 */
export interface InstanceBlockDeviceMappingSpecification {
    __type?: "InstanceBlockDeviceMappingSpecification";
    /**
     * <p>The device name (for example, <code>/dev/sdh</code> or <code>xvdh</code>).</p>
     */
    DeviceName?: string;
    /**
     * <p>Parameters used to automatically set up EBS volumes when the instance is
     *             launched.</p>
     */
    Ebs?: EbsInstanceBlockDeviceSpecification;
    /**
     * <p>suppress the specified device included in the block device mapping.</p>
     */
    NoDevice?: string;
    /**
     * <p>The virtual device name.</p>
     */
    VirtualName?: string;
}
export declare namespace InstanceBlockDeviceMappingSpecification {
    const filterSensitiveLog: (obj: InstanceBlockDeviceMappingSpecification) => any;
    const isa: (o: any) => o is InstanceBlockDeviceMappingSpecification;
}
/**
 * <p>Information about the number of instances that can be launched onto the Dedicated
 *     		Host.</p>
 */
export interface InstanceCapacity {
    __type?: "InstanceCapacity";
    /**
     * <p>The number of instances that can be launched onto the Dedicated Host based on the
     *     		host's available capacity.</p>
     */
    AvailableCapacity?: number;
    /**
     * <p>The instance type supported by the Dedicated Host.</p>
     */
    InstanceType?: string;
    /**
     * <p>The total number of instances that can be launched onto the Dedicated Host if there
     *     		are no instances running on it.</p>
     */
    TotalCapacity?: number;
}
export declare namespace InstanceCapacity {
    const filterSensitiveLog: (obj: InstanceCapacity) => any;
    const isa: (o: any) => o is InstanceCapacity;
}
/**
 * <p>Describes a Reserved Instance listing state.</p>
 */
export interface InstanceCount {
    __type?: "InstanceCount";
    /**
     * <p>The number of listed Reserved Instances in the state specified by the <code>state</code>.</p>
     */
    InstanceCount?: number;
    /**
     * <p>The states of the listed Reserved Instances.</p>
     */
    State?: ListingState | string;
}
export declare namespace InstanceCount {
    const filterSensitiveLog: (obj: InstanceCount) => any;
    const isa: (o: any) => o is InstanceCount;
}
/**
 * <p>Describes the credit option for CPU usage of a burstable performance instance. </p>
 */
export interface InstanceCreditSpecification {
    __type?: "InstanceCreditSpecification";
    /**
     * <p>The credit option for CPU usage of the instance. Valid values are
     *                 <code>standard</code> and <code>unlimited</code>.</p>
     */
    CpuCredits?: string;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
}
export declare namespace InstanceCreditSpecification {
    const filterSensitiveLog: (obj: InstanceCreditSpecification) => any;
    const isa: (o: any) => o is InstanceCreditSpecification;
}
/**
 * <p>Describes the credit option for CPU usage of a burstable performance instance.</p>
 */
export interface InstanceCreditSpecificationRequest {
    __type?: "InstanceCreditSpecificationRequest";
    /**
     * <p>The credit option for CPU usage of the instance. Valid values are
     *                 <code>standard</code> and <code>unlimited</code>.</p>
     */
    CpuCredits?: string;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
}
export declare namespace InstanceCreditSpecificationRequest {
    const filterSensitiveLog: (obj: InstanceCreditSpecificationRequest) => any;
    const isa: (o: any) => o is InstanceCreditSpecificationRequest;
}
/**
 * <p>Describes an instance to export.</p>
 */
export interface InstanceExportDetails {
    __type?: "InstanceExportDetails";
    /**
     * <p>The ID of the resource being exported.</p>
     */
    InstanceId?: string;
    /**
     * <p>The target virtualization environment.</p>
     */
    TargetEnvironment?: ExportEnvironment | string;
}
export declare namespace InstanceExportDetails {
    const filterSensitiveLog: (obj: InstanceExportDetails) => any;
    const isa: (o: any) => o is InstanceExportDetails;
}
/**
 * <p>Describes the default credit option for CPU usage of a burstable performance instance family.</p>
 */
export interface InstanceFamilyCreditSpecification {
    __type?: "InstanceFamilyCreditSpecification";
    /**
     * <p>The default credit option for CPU usage of the instance family. Valid values are <code>standard</code> and <code>unlimited</code>.</p>
     */
    CpuCredits?: string;
    /**
     * <p>The instance family.</p>
     */
    InstanceFamily?: UnlimitedSupportedInstanceFamily | string;
}
export declare namespace InstanceFamilyCreditSpecification {
    const filterSensitiveLog: (obj: InstanceFamilyCreditSpecification) => any;
    const isa: (o: any) => o is InstanceFamilyCreditSpecification;
}
export declare enum InstanceHealthStatus {
    HEALTHY_STATUS = "healthy",
    UNHEALTHY_STATUS = "unhealthy"
}
export declare type InstanceInterruptionBehavior = "hibernate" | "stop" | "terminate";
/**
 * <p>Describes an IPv6 address.</p>
 */
export interface InstanceIpv6Address {
    __type?: "InstanceIpv6Address";
    /**
     * <p>The IPv6 address.</p>
     */
    Ipv6Address?: string;
}
export declare namespace InstanceIpv6Address {
    const filterSensitiveLog: (obj: InstanceIpv6Address) => any;
    const isa: (o: any) => o is InstanceIpv6Address;
}
/**
 * <p>Describes an IPv6 address.</p>
 */
export interface InstanceIpv6AddressRequest {
    __type?: "InstanceIpv6AddressRequest";
    /**
     * <p>The IPv6 address.</p>
     */
    Ipv6Address?: string;
}
export declare namespace InstanceIpv6AddressRequest {
    const filterSensitiveLog: (obj: InstanceIpv6AddressRequest) => any;
    const isa: (o: any) => o is InstanceIpv6AddressRequest;
}
export declare enum InstanceLifecycle {
    ON_DEMAND = "on-demand",
    SPOT = "spot"
}
export declare type InstanceLifecycleType = "scheduled" | "spot";
/**
 * <p>Describes the market (purchasing) option for the instances.</p>
 */
export interface InstanceMarketOptionsRequest {
    __type?: "InstanceMarketOptionsRequest";
    /**
     * <p>The market type.</p>
     */
    MarketType?: MarketType | string;
    /**
     * <p>The options for Spot Instances.</p>
     */
    SpotOptions?: SpotMarketOptions;
}
export declare namespace InstanceMarketOptionsRequest {
    const filterSensitiveLog: (obj: InstanceMarketOptionsRequest) => any;
    const isa: (o: any) => o is InstanceMarketOptionsRequest;
}
export declare type InstanceMatchCriteria = "open" | "targeted";
export declare type InstanceMetadataEndpointState = "disabled" | "enabled";
/**
 * <p>The metadata options for the instance.</p>
 */
export interface InstanceMetadataOptionsRequest {
    __type?: "InstanceMetadataOptionsRequest";
    /**
     * <p>This parameter enables or disables the HTTP metadata endpoint on your instances. If
     *             the parameter is not specified, the default state is <code>enabled</code>.</p>
     *         <note>
     *             <p>If you specify a value of <code>disabled</code>, you will not be able to access your
     *                 instance metadata.</p>
     *         </note>
     */
    HttpEndpoint?: InstanceMetadataEndpointState | string;
    /**
     * <p>The desired HTTP PUT response hop limit for instance metadata requests. The larger the
     *             number, the further instance metadata requests can travel.</p>
     *         <p>Default: 1</p>
     *         <p>Possible values: Integers from 1 to 64</p>
     */
    HttpPutResponseHopLimit?: number;
    /**
     * <p>The state of token usage for your instance metadata requests. If the parameter is not
     *             specified in the request, the default state is <code>optional</code>.</p>
     *         <p>If the state is <code>optional</code>, you can choose to retrieve instance metadata
     *             with or without a signed token header on your request. If you retrieve the IAM role
     *             credentials without a token, the version 1.0 role credentials are returned. If you
     *             retrieve the IAM role credentials using a valid signed token, the version 2.0 role
     *             credentials are returned.</p>
     *         <p>If the state is <code>required</code>, you must send a signed token header with any
     *             instance metadata retrieval requests. In this state, retrieving the IAM role credentials
     *             always returns the version 2.0 credentials; the version 1.0 credentials are not
     *             available.</p>
     */
    HttpTokens?: HttpTokensState | string;
}
export declare namespace InstanceMetadataOptionsRequest {
    const filterSensitiveLog: (obj: InstanceMetadataOptionsRequest) => any;
    const isa: (o: any) => o is InstanceMetadataOptionsRequest;
}
/**
 * <p>The metadata options for the instance.</p>
 */
export interface InstanceMetadataOptionsResponse {
    __type?: "InstanceMetadataOptionsResponse";
    /**
     * <p>This parameter enables or disables the HTTP metadata endpoint on your instances. If
     *             the parameter is not specified, the default state is <code>enabled</code>.</p>
     *         <note>
     *             <p>If you specify a value of <code>disabled</code>, you will not be able to access your
     *                 instance metadata.</p>
     *         </note>
     */
    HttpEndpoint?: InstanceMetadataEndpointState | string;
    /**
     * <p>The desired HTTP PUT response hop limit for instance metadata requests. The larger the
     *             number, the further instance metadata requests can travel.</p>
     *         <p>Default: 1</p>
     *         <p>Possible values: Integers from 1 to 64</p>
     */
    HttpPutResponseHopLimit?: number;
    /**
     * <p>The state of token usage for your instance metadata requests. If the parameter is not
     *             specified in the request, the default state is <code>optional</code>.</p>
     *         <p>If the state is <code>optional</code>, you can choose to retrieve instance metadata
     *             with or without a signed token header on your request. If you retrieve the IAM role
     *             credentials without a token, the version 1.0 role credentials are returned. If you
     *             retrieve the IAM role credentials using a valid signed token, the version 2.0 role
     *             credentials are returned.</p>
     *         <p>If the state is <code>required</code>, you must send a signed token header with any
     *             instance metadata retrieval requests. In this state, retrieving the IAM role credential
     *             always returns the version 2.0 credentials; the version 1.0 credentials are not
     *             available.</p>
     */
    HttpTokens?: HttpTokensState | string;
    /**
     * <p>The state of the metadata option changes.</p>
     *         <p>
     *             <code>pending</code> - The metadata options are being updated and the instance is not
     *             ready to process metadata traffic with the new selection.</p>
     *         <p>
     *             <code>applied</code> - The metadata options have been successfully applied on the
     *             instance.</p>
     */
    State?: InstanceMetadataOptionsState | string;
}
export declare namespace InstanceMetadataOptionsResponse {
    const filterSensitiveLog: (obj: InstanceMetadataOptionsResponse) => any;
    const isa: (o: any) => o is InstanceMetadataOptionsResponse;
}
export declare type InstanceMetadataOptionsState = "applied" | "pending";
/**
 * <p>Describes the monitoring of an instance.</p>
 */
export interface InstanceMonitoring {
    __type?: "InstanceMonitoring";
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The monitoring for the instance.</p>
     */
    Monitoring?: Monitoring;
}
export declare namespace InstanceMonitoring {
    const filterSensitiveLog: (obj: InstanceMonitoring) => any;
    const isa: (o: any) => o is InstanceMonitoring;
}
/**
 * <p>Describes a network interface.</p>
 */
export interface InstanceNetworkInterface {
    __type?: "InstanceNetworkInterface";
    /**
     * <p>The association information for an Elastic IPv4 associated with the network
     *             interface.</p>
     */
    Association?: InstanceNetworkInterfaceAssociation;
    /**
     * <p>The network interface attachment.</p>
     */
    Attachment?: InstanceNetworkInterfaceAttachment;
    /**
     * <p>The description.</p>
     */
    Description?: string;
    /**
     * <p>One or more security groups.</p>
     */
    Groups?: GroupIdentifier[];
    /**
     * <p>Describes the type of network interface.</p>
     * 		       <p>Valid values: <code>interface</code> | <code>efa</code>
     *          </p>
     */
    InterfaceType?: string;
    /**
     * <p>One or more IPv6 addresses associated with the network interface.</p>
     */
    Ipv6Addresses?: InstanceIpv6Address[];
    /**
     * <p>The MAC address.</p>
     */
    MacAddress?: string;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The ID of the AWS account that created the network interface.</p>
     */
    OwnerId?: string;
    /**
     * <p>The private DNS name.</p>
     */
    PrivateDnsName?: string;
    /**
     * <p>The IPv4 address of the network interface within the subnet.</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>One or more private IPv4 addresses associated with the network interface.</p>
     */
    PrivateIpAddresses?: InstancePrivateIpAddress[];
    /**
     * <p>Indicates whether to validate network traffic to or from this network interface.</p>
     */
    SourceDestCheck?: boolean;
    /**
     * <p>The status of the network interface.</p>
     */
    Status?: NetworkInterfaceStatus | string;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId?: string;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace InstanceNetworkInterface {
    const filterSensitiveLog: (obj: InstanceNetworkInterface) => any;
    const isa: (o: any) => o is InstanceNetworkInterface;
}
/**
 * <p>Describes association information for an Elastic IP address (IPv4).</p>
 */
export interface InstanceNetworkInterfaceAssociation {
    __type?: "InstanceNetworkInterfaceAssociation";
    /**
     * <p>The ID of the owner of the Elastic IP address.</p>
     */
    IpOwnerId?: string;
    /**
     * <p>The public DNS name.</p>
     */
    PublicDnsName?: string;
    /**
     * <p>The public IP address or Elastic IP address bound to the network interface.</p>
     */
    PublicIp?: string;
}
export declare namespace InstanceNetworkInterfaceAssociation {
    const filterSensitiveLog: (obj: InstanceNetworkInterfaceAssociation) => any;
    const isa: (o: any) => o is InstanceNetworkInterfaceAssociation;
}
/**
 * <p>Describes a network interface attachment.</p>
 */
export interface InstanceNetworkInterfaceAttachment {
    __type?: "InstanceNetworkInterfaceAttachment";
    /**
     * <p>The time stamp when the attachment initiated.</p>
     */
    AttachTime?: Date;
    /**
     * <p>The ID of the network interface attachment.</p>
     */
    AttachmentId?: string;
    /**
     * <p>Indicates whether the network interface is deleted when the instance is terminated.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>The index of the device on the instance for the network interface attachment.</p>
     */
    DeviceIndex?: number;
    /**
     * <p>The attachment state.</p>
     */
    Status?: AttachmentStatus | string;
}
export declare namespace InstanceNetworkInterfaceAttachment {
    const filterSensitiveLog: (obj: InstanceNetworkInterfaceAttachment) => any;
    const isa: (o: any) => o is InstanceNetworkInterfaceAttachment;
}
/**
 * <p>Describes a network interface.</p>
 */
export interface InstanceNetworkInterfaceSpecification {
    __type?: "InstanceNetworkInterfaceSpecification";
    /**
     * <p>Indicates whether to assign a public IPv4 address to an instance you launch in a VPC. The
     *             public IP address can only be assigned to a network interface for eth0, and can only be
     *             assigned to a new network interface, not an existing one. You cannot specify more than one
     *             network interface in the request. If launching into a default subnet, the default value is
     *             <code>true</code>.</p>
     */
    AssociatePublicIpAddress?: boolean;
    /**
     * <p>If set to <code>true</code>, the interface is deleted when the instance is terminated. You can
     *             specify <code>true</code> only if creating a new network interface when launching an
     *             instance.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>The description of the network interface. Applies only if creating a network interface when launching an instance.</p>
     */
    Description?: string;
    /**
     * <p>The position of the network interface in the attachment order.
     *           A primary network interface has a device index of 0.</p>
     *         <p>If you specify a network interface when launching an instance,
     *           you must specify the device index.</p>
     */
    DeviceIndex?: number;
    /**
     * <p>The IDs of the security groups for the network interface. Applies only if creating a network interface when launching an instance.</p>
     */
    Groups?: string[];
    /**
     * <p>The type of network interface. To create an Elastic Fabric Adapter (EFA), specify
     * 			<code>efa</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/efa.html">Elastic Fabric Adapter</a> in the
     * 			<i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     * 		       <p>If you are not creating an EFA, specify <code>interface</code> or omit this parameter.</p>
     * 		       <p>Valid values: <code>interface</code> | <code>efa</code>
     *          </p>
     */
    InterfaceType?: string;
    /**
     * <p>A number of IPv6 addresses to assign to the network interface. Amazon EC2 chooses
     *             the IPv6 addresses from the range of the subnet. You cannot specify this option and the
     *             option to assign specific IPv6 addresses in the same request. You can specify this
     *             option if you've specified a minimum number of instances to launch.</p>
     */
    Ipv6AddressCount?: number;
    /**
     * <p>One or more IPv6 addresses to assign to the network interface. You cannot specify
     *             this option and the option to assign a number of IPv6 addresses in the same request. You
     *             cannot specify this option if you've specified a minimum number of instances to
     *             launch.</p>
     */
    Ipv6Addresses?: InstanceIpv6Address[];
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The private IPv4 address of the network interface. Applies only if creating a network interface when launching an instance. You cannot specify this option if you're launching
     *         	more than one instance in a <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_RunInstances.html">RunInstances</a> request.</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>One or more private IPv4 addresses to assign to the network interface. Only one private IPv4 address can be designated as primary. You cannot specify this option if you're
     *         	launching more than one instance in a <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_RunInstances.html">RunInstances</a> request.</p>
     */
    PrivateIpAddresses?: PrivateIpAddressSpecification[];
    /**
     * <p>The number of secondary private IPv4 addresses. You can't specify this option and specify more than one private IP address using the private IP addresses option. You cannot specify this option if you're
     *         	launching more than one instance in a <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_RunInstances.html">RunInstances</a> request.</p>
     */
    SecondaryPrivateIpAddressCount?: number;
    /**
     * <p>The ID of the subnet associated with the network interface. Applies only if creating a network interface when launching an instance.</p>
     */
    SubnetId?: string;
}
export declare namespace InstanceNetworkInterfaceSpecification {
    const filterSensitiveLog: (obj: InstanceNetworkInterfaceSpecification) => any;
    const isa: (o: any) => o is InstanceNetworkInterfaceSpecification;
}
/**
 * <p>Describes a private IPv4 address.</p>
 */
export interface InstancePrivateIpAddress {
    __type?: "InstancePrivateIpAddress";
    /**
     * <p>The association information for an Elastic IP address for the network interface.</p>
     */
    Association?: InstanceNetworkInterfaceAssociation;
    /**
     * <p>Indicates whether this IPv4 address is the primary private IP address of the network interface.</p>
     */
    Primary?: boolean;
    /**
     * <p>The private IPv4 DNS name.</p>
     */
    PrivateDnsName?: string;
    /**
     * <p>The private IPv4 address of the network interface.</p>
     */
    PrivateIpAddress?: string;
}
export declare namespace InstancePrivateIpAddress {
    const filterSensitiveLog: (obj: InstancePrivateIpAddress) => any;
    const isa: (o: any) => o is InstancePrivateIpAddress;
}
/**
 * <p>The instance details to specify which volumes should be snapshotted.</p>
 */
export interface InstanceSpecification {
    __type?: "InstanceSpecification";
    /**
     * <p>Excludes the root volume from being snapshotted.</p>
     */
    ExcludeBootVolume?: boolean;
    /**
     * <p>The instance to specify which volumes should be snapshotted.</p>
     */
    InstanceId?: string;
}
export declare namespace InstanceSpecification {
    const filterSensitiveLog: (obj: InstanceSpecification) => any;
    const isa: (o: any) => o is InstanceSpecification;
}
/**
 * <p>Describes the current state of an instance.</p>
 */
export interface InstanceState {
    __type?: "InstanceState";
    /**
     * <p>The state of the instance as a 16-bit unsigned integer. </p>
     *         <p>The high byte is all of the bits between 2^8 and (2^16)-1, which equals decimal values
     *             between 256 and 65,535. These numerical values are used for internal purposes and should
     *             be ignored.</p>
     *         <p>The low byte is all of the bits between 2^0 and (2^8)-1, which equals decimal values
     *             between 0 and 255. </p>
     *         <p>The valid values for instance-state-code will all be in the range of the low byte and
     *             they are:</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                     <code>0</code> : <code>pending</code>
     *                 </p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>16</code> : <code>running</code>
     *                 </p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>32</code> : <code>shutting-down</code>
     *                 </p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>48</code> : <code>terminated</code>
     *                 </p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>64</code> : <code>stopping</code>
     *                 </p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>80</code> : <code>stopped</code>
     *                 </p>
     *             </li>
     *          </ul>
     *         <p>You can ignore the high byte value by zeroing out all of the bits above 2^8 or 256 in
     *             decimal.</p>
     */
    Code?: number;
    /**
     * <p>The current state of the instance.</p>
     */
    Name?: InstanceStateName | string;
}
export declare namespace InstanceState {
    const filterSensitiveLog: (obj: InstanceState) => any;
    const isa: (o: any) => o is InstanceState;
}
/**
 * <p>Describes an instance state change.</p>
 */
export interface InstanceStateChange {
    __type?: "InstanceStateChange";
    /**
     * <p>The current state of the instance.</p>
     */
    CurrentState?: InstanceState;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The previous state of the instance.</p>
     */
    PreviousState?: InstanceState;
}
export declare namespace InstanceStateChange {
    const filterSensitiveLog: (obj: InstanceStateChange) => any;
    const isa: (o: any) => o is InstanceStateChange;
}
export declare type InstanceStateName = "pending" | "running" | "shutting-down" | "stopped" | "stopping" | "terminated";
/**
 * <p>Describes the status of an instance.</p>
 */
export interface InstanceStatus {
    __type?: "InstanceStatus";
    /**
     * <p>The Availability Zone of the instance.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>Any scheduled events associated with the instance.</p>
     */
    Events?: InstanceStatusEvent[];
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The intended state of the instance. <a>DescribeInstanceStatus</a> requires
     *             that an instance be in the <code>running</code> state.</p>
     */
    InstanceState?: InstanceState;
    /**
     * <p>Reports impaired functionality that stems from issues internal to the instance, such
     *             as impaired reachability.</p>
     */
    InstanceStatus?: InstanceStatusSummary;
    /**
     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
     */
    OutpostArn?: string;
    /**
     * <p>Reports impaired functionality that stems from issues related to the systems that
     *             support an instance, such as hardware failures and network connectivity problems.</p>
     */
    SystemStatus?: InstanceStatusSummary;
}
export declare namespace InstanceStatus {
    const filterSensitiveLog: (obj: InstanceStatus) => any;
    const isa: (o: any) => o is InstanceStatus;
}
/**
 * <p>Describes the instance status.</p>
 */
export interface InstanceStatusDetails {
    __type?: "InstanceStatusDetails";
    /**
     * <p>The time when a status check failed. For an instance that was launched and impaired,
     *             this is the time when the instance was launched.</p>
     */
    ImpairedSince?: Date;
    /**
     * <p>The type of instance status.</p>
     */
    Name?: StatusName | string;
    /**
     * <p>The status.</p>
     */
    Status?: StatusType | string;
}
export declare namespace InstanceStatusDetails {
    const filterSensitiveLog: (obj: InstanceStatusDetails) => any;
    const isa: (o: any) => o is InstanceStatusDetails;
}
/**
 * <p>Describes a scheduled event for an instance.</p>
 */
export interface InstanceStatusEvent {
    __type?: "InstanceStatusEvent";
    /**
     * <p>The event code.</p>
     */
    Code?: EventCode | string;
    /**
     * <p>A description of the event.</p>
     *         <p>After a scheduled event is completed, it can still be described for up to a week. If
     *             the event has been completed, this description starts with the following text:
     *             [Completed].</p>
     */
    Description?: string;
    /**
     * <p>The ID of the event.</p>
     */
    InstanceEventId?: string;
    /**
     * <p>The latest scheduled end time for the event.</p>
     */
    NotAfter?: Date;
    /**
     * <p>The earliest scheduled start time for the event.</p>
     */
    NotBefore?: Date;
    /**
     * <p>The deadline for starting the event.</p>
     */
    NotBeforeDeadline?: Date;
}
export declare namespace InstanceStatusEvent {
    const filterSensitiveLog: (obj: InstanceStatusEvent) => any;
    const isa: (o: any) => o is InstanceStatusEvent;
}
/**
 * <p>Describes the status of an instance.</p>
 */
export interface InstanceStatusSummary {
    __type?: "InstanceStatusSummary";
    /**
     * <p>The system instance health or application instance health.</p>
     */
    Details?: InstanceStatusDetails[];
    /**
     * <p>The status.</p>
     */
    Status?: SummaryStatus | string;
}
export declare namespace InstanceStatusSummary {
    const filterSensitiveLog: (obj: InstanceStatusSummary) => any;
    const isa: (o: any) => o is InstanceStatusSummary;
}
/**
 * <p>Describes the disks that are available for the instance type.</p>
 */
export interface InstanceStorageInfo {
    __type?: "InstanceStorageInfo";
    /**
     * <p>Array describing the disks that are available for the instance type.</p>
     */
    Disks?: DiskInfo[];
    /**
     * <p>The total size of the disks, in GB.</p>
     */
    TotalSizeInGB?: number;
}
export declare namespace InstanceStorageInfo {
    const filterSensitiveLog: (obj: InstanceStorageInfo) => any;
    const isa: (o: any) => o is InstanceStorageInfo;
}
export declare type _InstanceType = "a1.2xlarge" | "a1.4xlarge" | "a1.large" | "a1.medium" | "a1.metal" | "a1.xlarge" | "c1.medium" | "c1.xlarge" | "c3.2xlarge" | "c3.4xlarge" | "c3.8xlarge" | "c3.large" | "c3.xlarge" | "c4.2xlarge" | "c4.4xlarge" | "c4.8xlarge" | "c4.large" | "c4.xlarge" | "c5.12xlarge" | "c5.18xlarge" | "c5.24xlarge" | "c5.2xlarge" | "c5.4xlarge" | "c5.9xlarge" | "c5.large" | "c5.metal" | "c5.xlarge" | "c5d.12xlarge" | "c5d.18xlarge" | "c5d.24xlarge" | "c5d.2xlarge" | "c5d.4xlarge" | "c5d.9xlarge" | "c5d.large" | "c5d.metal" | "c5d.xlarge" | "c5n.18xlarge" | "c5n.2xlarge" | "c5n.4xlarge" | "c5n.9xlarge" | "c5n.large" | "c5n.xlarge" | "cc1.4xlarge" | "cc2.8xlarge" | "cg1.4xlarge" | "cr1.8xlarge" | "d2.2xlarge" | "d2.4xlarge" | "d2.8xlarge" | "d2.xlarge" | "f1.16xlarge" | "f1.2xlarge" | "f1.4xlarge" | "g2.2xlarge" | "g2.8xlarge" | "g3.16xlarge" | "g3.4xlarge" | "g3.8xlarge" | "g3s.xlarge" | "g4dn.12xlarge" | "g4dn.16xlarge" | "g4dn.2xlarge" | "g4dn.4xlarge" | "g4dn.8xlarge" | "g4dn.xlarge" | "h1.16xlarge" | "h1.2xlarge" | "h1.4xlarge" | "h1.8xlarge" | "hi1.4xlarge" | "hs1.8xlarge" | "i2.2xlarge" | "i2.4xlarge" | "i2.8xlarge" | "i2.xlarge" | "i3.16xlarge" | "i3.2xlarge" | "i3.4xlarge" | "i3.8xlarge" | "i3.large" | "i3.metal" | "i3.xlarge" | "i3en.12xlarge" | "i3en.24xlarge" | "i3en.2xlarge" | "i3en.3xlarge" | "i3en.6xlarge" | "i3en.large" | "i3en.metal" | "i3en.xlarge" | "inf1.24xlarge" | "inf1.2xlarge" | "inf1.6xlarge" | "inf1.xlarge" | "m1.large" | "m1.medium" | "m1.small" | "m1.xlarge" | "m2.2xlarge" | "m2.4xlarge" | "m2.xlarge" | "m3.2xlarge" | "m3.large" | "m3.medium" | "m3.xlarge" | "m4.10xlarge" | "m4.16xlarge" | "m4.2xlarge" | "m4.4xlarge" | "m4.large" | "m4.xlarge" | "m5.12xlarge" | "m5.16xlarge" | "m5.24xlarge" | "m5.2xlarge" | "m5.4xlarge" | "m5.8xlarge" | "m5.large" | "m5.metal" | "m5.xlarge" | "m5a.12xlarge" | "m5a.16xlarge" | "m5a.24xlarge" | "m5a.2xlarge" | "m5a.4xlarge" | "m5a.8xlarge" | "m5a.large" | "m5a.xlarge" | "m5ad.12xlarge" | "m5ad.16xlarge" | "m5ad.24xlarge" | "m5ad.2xlarge" | "m5ad.4xlarge" | "m5ad.8xlarge" | "m5ad.large" | "m5ad.xlarge" | "m5d.12xlarge" | "m5d.16xlarge" | "m5d.24xlarge" | "m5d.2xlarge" | "m5d.4xlarge" | "m5d.8xlarge" | "m5d.large" | "m5d.metal" | "m5d.xlarge" | "m5dn.12xlarge" | "m5dn.16xlarge" | "m5dn.24xlarge" | "m5dn.2xlarge" | "m5dn.4xlarge" | "m5dn.8xlarge" | "m5dn.large" | "m5dn.xlarge" | "m5n.12xlarge" | "m5n.16xlarge" | "m5n.24xlarge" | "m5n.2xlarge" | "m5n.4xlarge" | "m5n.8xlarge" | "m5n.large" | "m5n.xlarge" | "p2.16xlarge" | "p2.8xlarge" | "p2.xlarge" | "p3.16xlarge" | "p3.2xlarge" | "p3.8xlarge" | "p3dn.24xlarge" | "r3.2xlarge" | "r3.4xlarge" | "r3.8xlarge" | "r3.large" | "r3.xlarge" | "r4.16xlarge" | "r4.2xlarge" | "r4.4xlarge" | "r4.8xlarge" | "r4.large" | "r4.xlarge" | "r5.12xlarge" | "r5.16xlarge" | "r5.24xlarge" | "r5.2xlarge" | "r5.4xlarge" | "r5.8xlarge" | "r5.large" | "r5.metal" | "r5.xlarge" | "r5a.12xlarge" | "r5a.16xlarge" | "r5a.24xlarge" | "r5a.2xlarge" | "r5a.4xlarge" | "r5a.8xlarge" | "r5a.large" | "r5a.xlarge" | "r5ad.12xlarge" | "r5ad.16xlarge" | "r5ad.24xlarge" | "r5ad.2xlarge" | "r5ad.4xlarge" | "r5ad.8xlarge" | "r5ad.large" | "r5ad.xlarge" | "r5d.12xlarge" | "r5d.16xlarge" | "r5d.24xlarge" | "r5d.2xlarge" | "r5d.4xlarge" | "r5d.8xlarge" | "r5d.large" | "r5d.metal" | "r5d.xlarge" | "r5dn.12xlarge" | "r5dn.16xlarge" | "r5dn.24xlarge" | "r5dn.2xlarge" | "r5dn.4xlarge" | "r5dn.8xlarge" | "r5dn.large" | "r5dn.xlarge" | "r5n.12xlarge" | "r5n.16xlarge" | "r5n.24xlarge" | "r5n.2xlarge" | "r5n.4xlarge" | "r5n.8xlarge" | "r5n.large" | "r5n.xlarge" | "t1.micro" | "t2.2xlarge" | "t2.large" | "t2.medium" | "t2.micro" | "t2.nano" | "t2.small" | "t2.xlarge" | "t3.2xlarge" | "t3.large" | "t3.medium" | "t3.micro" | "t3.nano" | "t3.small" | "t3.xlarge" | "t3a.2xlarge" | "t3a.large" | "t3a.medium" | "t3a.micro" | "t3a.nano" | "t3a.small" | "t3a.xlarge" | "u-12tb1.metal" | "u-18tb1.metal" | "u-24tb1.metal" | "u-6tb1.metal" | "u-9tb1.metal" | "x1.16xlarge" | "x1.32xlarge" | "x1e.16xlarge" | "x1e.2xlarge" | "x1e.32xlarge" | "x1e.4xlarge" | "x1e.8xlarge" | "x1e.xlarge" | "z1d.12xlarge" | "z1d.2xlarge" | "z1d.3xlarge" | "z1d.6xlarge" | "z1d.large" | "z1d.metal" | "z1d.xlarge";
export declare enum InstanceTypeHypervisor {
    NITRO = "nitro",
    XEN = "xen"
}
/**
 * <p>Describes the instance type.</p>
 */
export interface InstanceTypeInfo {
    __type?: "InstanceTypeInfo";
    /**
     * <p>Indicates whether auto recovery is supported.</p>
     */
    AutoRecoverySupported?: boolean;
    /**
     * <p>Indicates whether the instance is bare metal.</p>
     */
    BareMetal?: boolean;
    /**
     * <p>Indicates whether the instance type is a burstable performance instance type.</p>
     */
    BurstablePerformanceSupported?: boolean;
    /**
     * <p>Indicates whether the instance type is a current generation.</p>
     */
    CurrentGeneration?: boolean;
    /**
     * <p>Indicates whether Dedicated Hosts are supported on the instance type.</p>
     */
    DedicatedHostsSupported?: boolean;
    /**
     * <p>Describes the Amazon EBS settings for the instance type.</p>
     */
    EbsInfo?: EbsInfo;
    /**
     * <p>Describes the FPGA accelerator settings for the instance type.</p>
     */
    FpgaInfo?: FpgaInfo;
    /**
     * <p>Indicates whether the instance type is eligible for the free tier.</p>
     */
    FreeTierEligible?: boolean;
    /**
     * <p>Describes the GPU accelerator settings for the instance type.</p>
     */
    GpuInfo?: GpuInfo;
    /**
     * <p>Indicates whether On-Demand hibernation is supported.</p>
     */
    HibernationSupported?: boolean;
    /**
     * <p>Indicates the hypervisor used for the instance type.</p>
     */
    Hypervisor?: InstanceTypeHypervisor | string;
    /**
     * <p>Describes the Inference accelerator settings for the instance type.</p>
     */
    InferenceAcceleratorInfo?: InferenceAcceleratorInfo;
    /**
     * <p>Describes the disks for the instance type.</p>
     */
    InstanceStorageInfo?: InstanceStorageInfo;
    /**
     * <p>Indicates whether instance storage is supported.</p>
     */
    InstanceStorageSupported?: boolean;
    /**
     * <p>The instance type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the <i>Amazon Elastic Compute
     *     Cloud User Guide</i>.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>Describes the memory for the instance type.</p>
     */
    MemoryInfo?: MemoryInfo;
    /**
     * <p>Describes the network settings for the instance type.</p>
     */
    NetworkInfo?: NetworkInfo;
    /**
     * <p>Describes the placement group settings for the instance type.</p>
     */
    PlacementGroupInfo?: PlacementGroupInfo;
    /**
     * <p>Describes the processor.</p>
     */
    ProcessorInfo?: ProcessorInfo;
    /**
     * <p>Indicates the supported root device types.</p>
     */
    SupportedRootDeviceTypes?: (RootDeviceType | string)[];
    /**
     * <p>Indicates whether the instance type is offered for spot or On-Demand.</p>
     */
    SupportedUsageClasses?: (UsageClassType | string)[];
    /**
     * <p>Describes the vCPU configurations for the instance type.</p>
     */
    VCpuInfo?: VCpuInfo;
}
export declare namespace InstanceTypeInfo {
    const filterSensitiveLog: (obj: InstanceTypeInfo) => any;
    const isa: (o: any) => o is InstanceTypeInfo;
}
/**
 * <p>The instance types offered.</p>
 */
export interface InstanceTypeOffering {
    __type?: "InstanceTypeOffering";
    /**
     * <p>The instance type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the <i>Amazon Elastic Compute
     *     Cloud User Guide</i>.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The identifier for the location. This depends on the location type. For example, if the location type is
     *     <code>region</code>, the location is the Region code (for example, <code>us-east-2</code>.)</p>
     */
    Location?: string;
    /**
     * <p>The location type.</p>
     */
    LocationType?: LocationType | string;
}
export declare namespace InstanceTypeOffering {
    const filterSensitiveLog: (obj: InstanceTypeOffering) => any;
    const isa: (o: any) => o is InstanceTypeOffering;
}
/**
 * <p>Information about the Capacity Reservation usage.</p>
 */
export interface InstanceUsage {
    __type?: "InstanceUsage";
    /**
     * <p>The ID of the AWS account that is making use of the Capacity Reservation.</p>
     */
    AccountId?: string;
    /**
     * <p>The number of instances the AWS account currently has in the Capacity Reservation.</p>
     */
    UsedInstanceCount?: number;
}
export declare namespace InstanceUsage {
    const filterSensitiveLog: (obj: InstanceUsage) => any;
    const isa: (o: any) => o is InstanceUsage;
}
export declare type InterfacePermissionType = "EIP-ASSOCIATE" | "INSTANCE-ATTACH";
/**
 * <p>Describes an internet gateway.</p>
 */
export interface InternetGateway {
    __type?: "InternetGateway";
    /**
     * <p>Any VPCs attached to the internet gateway.</p>
     */
    Attachments?: InternetGatewayAttachment[];
    /**
     * <p>The ID of the internet gateway.</p>
     */
    InternetGatewayId?: string;
    /**
     * <p>The ID of the AWS account that owns the internet gateway.</p>
     */
    OwnerId?: string;
    /**
     * <p>Any tags assigned to the internet gateway.</p>
     */
    Tags?: Tag[];
}
export declare namespace InternetGateway {
    const filterSensitiveLog: (obj: InternetGateway) => any;
    const isa: (o: any) => o is InternetGateway;
}
/**
 * <p>Describes the attachment of a VPC to an internet gateway or an egress-only internet
 * 			gateway.</p>
 */
export interface InternetGatewayAttachment {
    __type?: "InternetGatewayAttachment";
    /**
     * <p>The current state of the attachment. For an internet gateway, the state is
     * 				<code>available</code> when attached to a VPC; otherwise, this value is not
     * 			returned.</p>
     */
    State?: AttachmentStatus | string;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace InternetGatewayAttachment {
    const filterSensitiveLog: (obj: InternetGatewayAttachment) => any;
    const isa: (o: any) => o is InternetGatewayAttachment;
}
/**
 * <p>Describes a set of permissions for a security group rule.</p>
 */
export interface IpPermission {
    __type?: "IpPermission";
    /**
     * <p>The start of port range for the TCP and UDP protocols, or an ICMP/ICMPv6 type number.
     *         A value of <code>-1</code> indicates all ICMP/ICMPv6 types. If you specify all
     * 		ICMP/ICMPv6 types, you must specify all codes.</p>
     */
    FromPort?: number;
    /**
     * <p>The IP protocol name (<code>tcp</code>, <code>udp</code>, <code>icmp</code>, <code>icmpv6</code>)
     *         or number (see <a href="http://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml">Protocol Numbers</a>).</p>
     *          <p>[VPC only] Use <code>-1</code> to specify all protocols. When authorizing
     *         security group rules, specifying <code>-1</code> or a protocol number other than
     *         <code>tcp</code>, <code>udp</code>, <code>icmp</code>, or <code>icmpv6</code> allows
     *         traffic on all ports, regardless of any port range you specify. For <code>tcp</code>,
     *         <code>udp</code>, and <code>icmp</code>, you must specify a port range. For <code>icmpv6</code>,
     *         the port range is optional; if you omit the port range, traffic for all types and codes is allowed.</p>
     */
    IpProtocol?: string;
    /**
     * <p>The IPv4 ranges.</p>
     */
    IpRanges?: IpRange[];
    /**
     * <p>[VPC only] The IPv6 ranges.</p>
     */
    Ipv6Ranges?: Ipv6Range[];
    /**
     * <p>[VPC only] The prefix list IDs for an AWS service. With outbound rules, this is the
     *         AWS service to access through a VPC endpoint from instances associated with the security
     *         group.</p>
     */
    PrefixListIds?: PrefixListId[];
    /**
     * <p>The end of port range for the TCP and UDP protocols, or an ICMP/ICMPv6 code. A value
     * 		of <code>-1</code> indicates all ICMP/ICMPv6 codes. If you specify all ICMP/ICMPv6 types,
     *         you must specify all codes.</p>
     */
    ToPort?: number;
    /**
     * <p>The security group and AWS account ID pairs.</p>
     */
    UserIdGroupPairs?: UserIdGroupPair[];
}
export declare namespace IpPermission {
    const filterSensitiveLog: (obj: IpPermission) => any;
    const isa: (o: any) => o is IpPermission;
}
/**
 * <p>Describes an IPv4 range.</p>
 */
export interface IpRange {
    __type?: "IpRange";
    /**
     * <p>The IPv4 CIDR range. You can either specify a CIDR range or a source security group,
     *             not both. To specify a single IPv4 address, use the /32 prefix length.</p>
     */
    CidrIp?: string;
    /**
     * <p>A description for the security group rule that references this IPv4 address range.</p>
     *          <p>Constraints: Up to 255 characters in length. Allowed characters are a-z, A-Z, 0-9,
     *         spaces, and ._-:/()#,@[]+=&;{}!$*</p>
     */
    Description?: string;
}
export declare namespace IpRange {
    const filterSensitiveLog: (obj: IpRange) => any;
    const isa: (o: any) => o is IpRange;
}
/**
 * <p>Describes an IPv6 CIDR block association.</p>
 */
export interface Ipv6CidrAssociation {
    __type?: "Ipv6CidrAssociation";
    /**
     * <p>The resource that's associated with the IPv6 CIDR block.</p>
     */
    AssociatedResource?: string;
    /**
     * <p>The IPv6 CIDR block.</p>
     */
    Ipv6Cidr?: string;
}
export declare namespace Ipv6CidrAssociation {
    const filterSensitiveLog: (obj: Ipv6CidrAssociation) => any;
    const isa: (o: any) => o is Ipv6CidrAssociation;
}
/**
 * <p>Describes an IPv6 CIDR block.</p>
 */
export interface Ipv6CidrBlock {
    __type?: "Ipv6CidrBlock";
    /**
     * <p>The IPv6 CIDR block.</p>
     */
    Ipv6CidrBlock?: string;
}
export declare namespace Ipv6CidrBlock {
    const filterSensitiveLog: (obj: Ipv6CidrBlock) => any;
    const isa: (o: any) => o is Ipv6CidrBlock;
}
/**
 * <p>Describes an IPv6 address pool.</p>
 */
export interface Ipv6Pool {
    __type?: "Ipv6Pool";
    /**
     * <p>The description for the address pool.</p>
     */
    Description?: string;
    /**
     * <p>The CIDR blocks for the address pool.</p>
     */
    PoolCidrBlocks?: PoolCidrBlock[];
    /**
     * <p>The ID of the address pool.</p>
     */
    PoolId?: string;
    /**
     * <p>Any tags for the address pool.</p>
     */
    Tags?: Tag[];
}
export declare namespace Ipv6Pool {
    const filterSensitiveLog: (obj: Ipv6Pool) => any;
    const isa: (o: any) => o is Ipv6Pool;
}
/**
 * <p>[EC2-VPC only] Describes an IPv6 range.</p>
 */
export interface Ipv6Range {
    __type?: "Ipv6Range";
    /**
     * <p>The IPv6 CIDR range. You can either specify a CIDR range or a source security group,
     *         not both. To specify a single IPv6 address, use the /128 prefix length.</p>
     */
    CidrIpv6?: string;
    /**
     * <p>A description for the security group rule that references this IPv6 address range.</p>
     *          <p>Constraints: Up to 255 characters in length. Allowed characters are a-z, A-Z, 0-9,
     *         spaces, and ._-:/()#,@[]+=&;{}!$*</p>
     */
    Description?: string;
}
export declare namespace Ipv6Range {
    const filterSensitiveLog: (obj: Ipv6Range) => any;
    const isa: (o: any) => o is Ipv6Range;
}
export declare type Ipv6SupportValue = "disable" | "enable";
/**
 * <p>Describes a key pair.</p>
 */
export interface KeyPair {
    __type?: "KeyPair";
    /**
     * <p>The SHA-1 digest of the DER encoded private key.</p>
     */
    KeyFingerprint?: string;
    /**
     * <p>An unencrypted PEM encoded RSA private key.</p>
     */
    KeyMaterial?: string;
    /**
     * <p>The name of the key pair.</p>
     */
    KeyName?: string;
    /**
     * <p>The ID of the key pair.</p>
     */
    KeyPairId?: string;
}
export declare namespace KeyPair {
    const filterSensitiveLog: (obj: KeyPair) => any;
    const isa: (o: any) => o is KeyPair;
}
/**
 * <p>Describes a key pair.</p>
 */
export interface KeyPairInfo {
    __type?: "KeyPairInfo";
    /**
     * <p>If you used <a>CreateKeyPair</a> to create the key pair, this is the SHA-1 digest of the DER encoded private key.
     *         If you used <a>ImportKeyPair</a> to provide AWS the public key, this is the MD5 public key fingerprint as specified in section 4 of RFC4716.</p>
     */
    KeyFingerprint?: string;
    /**
     * <p>The name of the key pair.</p>
     */
    KeyName?: string;
    /**
     * <p>The ID of the key pair.</p>
     */
    KeyPairId?: string;
    /**
     * <p>Any tags applied to the key pair.</p>
     */
    Tags?: Tag[];
}
export declare namespace KeyPairInfo {
    const filterSensitiveLog: (obj: KeyPairInfo) => any;
    const isa: (o: any) => o is KeyPairInfo;
}
/**
 * <p>The last error that occurred for a VPC endpoint.</p>
 */
export interface LastError {
    __type?: "LastError";
    /**
     * <p>The error code for the VPC endpoint error.</p>
     */
    Code?: string;
    /**
     * <p>The error message for the VPC endpoint error.</p>
     */
    Message?: string;
}
export declare namespace LastError {
    const filterSensitiveLog: (obj: LastError) => any;
    const isa: (o: any) => o is LastError;
}
/**
 * <p>Describes a launch permission.</p>
 */
export interface LaunchPermission {
    __type?: "LaunchPermission";
    /**
     * <p>The name of the group.</p>
     */
    Group?: PermissionGroup | string;
    /**
     * <p>The AWS account ID.</p>
     */
    UserId?: string;
}
export declare namespace LaunchPermission {
    const filterSensitiveLog: (obj: LaunchPermission) => any;
    const isa: (o: any) => o is LaunchPermission;
}
/**
 * <p>Describes a launch permission modification.</p>
 */
export interface LaunchPermissionModifications {
    __type?: "LaunchPermissionModifications";
    /**
     * <p>The AWS account ID to add to the list of launch permissions for the AMI.</p>
     */
    Add?: LaunchPermission[];
    /**
     * <p>The AWS account ID to remove from the list of launch permissions for the AMI.</p>
     */
    Remove?: LaunchPermission[];
}
export declare namespace LaunchPermissionModifications {
    const filterSensitiveLog: (obj: LaunchPermissionModifications) => any;
    const isa: (o: any) => o is LaunchPermissionModifications;
}
/**
 * <p>Describes the launch specification for an instance.</p>
 */
export interface LaunchSpecification {
    __type?: "LaunchSpecification";
    /**
     * <p>Deprecated.</p>
     */
    AddressingType?: string;
    /**
     * <p>One or more block device mapping entries.</p>
     */
    BlockDeviceMappings?: BlockDeviceMapping[];
    /**
     * <p>Indicates whether the instance is optimized for EBS I/O. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal EBS I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS Optimized instance.</p>
     *          <p>Default: <code>false</code>
     *          </p>
     */
    EbsOptimized?: boolean;
    /**
     * <p>The IAM instance profile.</p>
     */
    IamInstanceProfile?: IamInstanceProfileSpecification;
    /**
     * <p>The ID of the AMI.</p>
     */
    ImageId?: string;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The ID of the kernel.</p>
     */
    KernelId?: string;
    /**
     * <p>The name of the key pair.</p>
     */
    KeyName?: string;
    /**
     * <p>Describes the monitoring of an instance.</p>
     */
    Monitoring?: RunInstancesMonitoringEnabled;
    /**
     * <p>One or more network interfaces. If you specify a network interface, you must specify
     *            subnet IDs and security group IDs using the network interface.</p>
     */
    NetworkInterfaces?: InstanceNetworkInterfaceSpecification[];
    /**
     * <p>The placement information for the instance.</p>
     */
    Placement?: SpotPlacement;
    /**
     * <p>The ID of the RAM disk.</p>
     */
    RamdiskId?: string;
    /**
     * <p>One or more security groups. When requesting instances in a VPC, you must specify the IDs of the security groups. When requesting instances in EC2-Classic, you can specify the names or the IDs of the security groups.</p>
     */
    SecurityGroups?: GroupIdentifier[];
    /**
     * <p>The ID of the subnet in which to launch the instance.</p>
     */
    SubnetId?: string;
    /**
     * <p>The Base64-encoded user data for the instance.</p>
     */
    UserData?: string;
}
export declare namespace LaunchSpecification {
    const filterSensitiveLog: (obj: LaunchSpecification) => any;
    const isa: (o: any) => o is LaunchSpecification;
}
/**
 * <p>Describes a launch template.</p>
 */
export interface LaunchTemplate {
    __type?: "LaunchTemplate";
    /**
     * <p>The time launch template was created.</p>
     */
    CreateTime?: Date;
    /**
     * <p>The principal that created the launch template. </p>
     */
    CreatedBy?: string;
    /**
     * <p>The version number of the default version of the launch template.</p>
     */
    DefaultVersionNumber?: number;
    /**
     * <p>The version number of the latest version of the launch template.</p>
     */
    LatestVersionNumber?: number;
    /**
     * <p>The ID of the launch template.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template.</p>
     */
    LaunchTemplateName?: string;
    /**
     * <p>The tags for the launch template.</p>
     */
    Tags?: Tag[];
}
export declare namespace LaunchTemplate {
    const filterSensitiveLog: (obj: LaunchTemplate) => any;
    const isa: (o: any) => o is LaunchTemplate;
}
/**
 * <p>Describes a launch template and overrides.</p>
 */
export interface LaunchTemplateAndOverridesResponse {
    __type?: "LaunchTemplateAndOverridesResponse";
    /**
     * <p>The launch template.</p>
     */
    LaunchTemplateSpecification?: FleetLaunchTemplateSpecification;
    /**
     * <p>Any parameters that you specify override the same parameters in the launch
     *          template.</p>
     */
    Overrides?: FleetLaunchTemplateOverrides;
}
export declare namespace LaunchTemplateAndOverridesResponse {
    const filterSensitiveLog: (obj: LaunchTemplateAndOverridesResponse) => any;
    const isa: (o: any) => o is LaunchTemplateAndOverridesResponse;
}
/**
 * <p>Describes a block device mapping.</p>
 */
export interface LaunchTemplateBlockDeviceMapping {
    __type?: "LaunchTemplateBlockDeviceMapping";
    /**
     * <p>The device name.</p>
     */
    DeviceName?: string;
    /**
     * <p>Information about the block device for an EBS volume.</p>
     */
    Ebs?: LaunchTemplateEbsBlockDevice;
    /**
     * <p>Suppresses the specified device included in the block device mapping of the AMI.</p>
     */
    NoDevice?: string;
    /**
     * <p>The virtual device name (ephemeralN).</p>
     */
    VirtualName?: string;
}
export declare namespace LaunchTemplateBlockDeviceMapping {
    const filterSensitiveLog: (obj: LaunchTemplateBlockDeviceMapping) => any;
    const isa: (o: any) => o is LaunchTemplateBlockDeviceMapping;
}
/**
 * <p>Describes a block device mapping.</p>
 */
export interface LaunchTemplateBlockDeviceMappingRequest {
    __type?: "LaunchTemplateBlockDeviceMappingRequest";
    /**
     * <p>The device name (for example, /dev/sdh or xvdh).</p>
     */
    DeviceName?: string;
    /**
     * <p>Parameters used to automatically set up EBS volumes when the instance is launched.</p>
     */
    Ebs?: LaunchTemplateEbsBlockDeviceRequest;
    /**
     * <p>Suppresses the specified device included in the block device mapping of the AMI.</p>
     */
    NoDevice?: string;
    /**
     * <p>The virtual device name (ephemeralN). Instance store volumes are numbered starting from 0. An instance type with 2 available instance store volumes can specify mappings for ephemeral0 and ephemeral1. The number of available instance store volumes depends on the instance type. After you connect to the instance, you must mount the volume.</p>
     */
    VirtualName?: string;
}
export declare namespace LaunchTemplateBlockDeviceMappingRequest {
    const filterSensitiveLog: (obj: LaunchTemplateBlockDeviceMappingRequest) => any;
    const isa: (o: any) => o is LaunchTemplateBlockDeviceMappingRequest;
}
/**
 * <p>Describes an instance's Capacity Reservation targeting option. You can specify only one option at a time. Use the
 * 			<code>CapacityReservationPreference</code> parameter to configure the instance to run in On-Demand capacity or
 * 			to run in any <code>open</code> Capacity Reservation that has matching attributes (instance type, platform, Availability Zone).
 * 			Use the <code>CapacityReservationTarget</code> parameter to explicitly target a specific Capacity Reservation.</p>
 */
export interface LaunchTemplateCapacityReservationSpecificationRequest {
    __type?: "LaunchTemplateCapacityReservationSpecificationRequest";
    /**
     * <p>Indicates the instance's Capacity Reservation preferences. Possible preferences include:</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>open</code> - The instance can run in any <code>open</code> Capacity Reservation that has matching attributes
     * 				(instance type, platform, Availability Zone).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>none</code> - The instance avoids running in a Capacity Reservation even if one is available. The instance
     * 				runs in On-Demand capacity.</p>
     *             </li>
     *          </ul>
     */
    CapacityReservationPreference?: CapacityReservationPreference | string;
    /**
     * <p>Information about the target Capacity Reservation.</p>
     */
    CapacityReservationTarget?: CapacityReservationTarget;
}
export declare namespace LaunchTemplateCapacityReservationSpecificationRequest {
    const filterSensitiveLog: (obj: LaunchTemplateCapacityReservationSpecificationRequest) => any;
    const isa: (o: any) => o is LaunchTemplateCapacityReservationSpecificationRequest;
}
/**
 * <p>Information about the Capacity Reservation targeting option.</p>
 */
export interface LaunchTemplateCapacityReservationSpecificationResponse {
    __type?: "LaunchTemplateCapacityReservationSpecificationResponse";
    /**
     * <p>Indicates the instance's Capacity Reservation preferences. Possible preferences include:</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>open</code> - The instance can run in any <code>open</code> Capacity Reservation that has matching attributes
     * 				(instance type, platform, Availability Zone).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>none</code> - The instance avoids running in a Capacity Reservation even if one is available. The instance
     * 				runs in On-Demand capacity.</p>
     *             </li>
     *          </ul>
     */
    CapacityReservationPreference?: CapacityReservationPreference | string;
    /**
     * <p>Information about the target Capacity Reservation.</p>
     */
    CapacityReservationTarget?: CapacityReservationTargetResponse;
}
export declare namespace LaunchTemplateCapacityReservationSpecificationResponse {
    const filterSensitiveLog: (obj: LaunchTemplateCapacityReservationSpecificationResponse) => any;
    const isa: (o: any) => o is LaunchTemplateCapacityReservationSpecificationResponse;
}
/**
 * <p>Describes a launch template and overrides.</p>
 */
export interface LaunchTemplateConfig {
    __type?: "LaunchTemplateConfig";
    /**
     * <p>The launch template.</p>
     */
    LaunchTemplateSpecification?: FleetLaunchTemplateSpecification;
    /**
     * <p>Any parameters that you specify override the same parameters in the launch template.</p>
     */
    Overrides?: LaunchTemplateOverrides[];
}
export declare namespace LaunchTemplateConfig {
    const filterSensitiveLog: (obj: LaunchTemplateConfig) => any;
    const isa: (o: any) => o is LaunchTemplateConfig;
}
/**
 * <p>The CPU options for the instance.</p>
 */
export interface LaunchTemplateCpuOptions {
    __type?: "LaunchTemplateCpuOptions";
    /**
     * <p>The number of CPU cores for the instance.</p>
     */
    CoreCount?: number;
    /**
     * <p>The number of threads per CPU core.</p>
     */
    ThreadsPerCore?: number;
}
export declare namespace LaunchTemplateCpuOptions {
    const filterSensitiveLog: (obj: LaunchTemplateCpuOptions) => any;
    const isa: (o: any) => o is LaunchTemplateCpuOptions;
}
/**
 * <p>The CPU options for the instance. Both the core count and threads per core
 * 			must be specified in the request.</p>
 */
export interface LaunchTemplateCpuOptionsRequest {
    __type?: "LaunchTemplateCpuOptionsRequest";
    /**
     * <p>The number of CPU cores for the instance.</p>
     */
    CoreCount?: number;
    /**
     * <p>The number of threads per CPU core. To disable multithreading
     * 			for the instance, specify a value of 1. Otherwise, specify the default value of 2.</p>
     */
    ThreadsPerCore?: number;
}
export declare namespace LaunchTemplateCpuOptionsRequest {
    const filterSensitiveLog: (obj: LaunchTemplateCpuOptionsRequest) => any;
    const isa: (o: any) => o is LaunchTemplateCpuOptionsRequest;
}
/**
 * <p>Describes a block device for an EBS volume.</p>
 */
export interface LaunchTemplateEbsBlockDevice {
    __type?: "LaunchTemplateEbsBlockDevice";
    /**
     * <p>Indicates whether the EBS volume is deleted on instance termination.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>Indicates whether the EBS volume is encrypted.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>The number of I/O operations per second (IOPS) that the volume supports. </p>
     */
    Iops?: number;
    /**
     * <p>The ARN of the AWS Key Management Service (AWS KMS) CMK used for encryption.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>The ID of the snapshot.</p>
     */
    SnapshotId?: string;
    /**
     * <p>The size of the volume, in GiB.</p>
     */
    VolumeSize?: number;
    /**
     * <p>The volume type.</p>
     */
    VolumeType?: VolumeType | string;
}
export declare namespace LaunchTemplateEbsBlockDevice {
    const filterSensitiveLog: (obj: LaunchTemplateEbsBlockDevice) => any;
    const isa: (o: any) => o is LaunchTemplateEbsBlockDevice;
}
/**
 * <p>The parameters for a block device for an EBS volume.</p>
 */
export interface LaunchTemplateEbsBlockDeviceRequest {
    __type?: "LaunchTemplateEbsBlockDeviceRequest";
    /**
     * <p>Indicates whether the EBS volume is deleted on instance termination.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>Indicates whether the EBS volume is encrypted. Encrypted volumes can only be attached to instances that support Amazon EBS encryption. If you are creating a volume from a snapshot, you can't specify an encryption value.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>The number of I/O operations per second (IOPS) that the volume supports. For io1, this represents the number of IOPS that are provisioned for the volume. For gp2, this represents the baseline performance of the volume and the rate at which the volume accumulates I/O credits for bursting. For more information about General Purpose SSD baseline performance, I/O credits, and bursting, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html">Amazon EBS Volume Types</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *         <p>Condition: This parameter is required for requests to create io1 volumes; it is not used in requests to create gp2, st1, sc1, or standard volumes.</p>
     */
    Iops?: number;
    /**
     * <p>The ARN of the symmetric AWS Key Management Service (AWS KMS) CMK used for
     *             encryption.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>The ID of the snapshot.</p>
     */
    SnapshotId?: string;
    /**
     * <p>The size of the volume, in GiB.</p>
     *         <p>Default: If you're creating the volume from a snapshot and don't specify a volume size, the default is the snapshot size.</p>
     */
    VolumeSize?: number;
    /**
     * <p>The volume type.</p>
     */
    VolumeType?: VolumeType | string;
}
export declare namespace LaunchTemplateEbsBlockDeviceRequest {
    const filterSensitiveLog: (obj: LaunchTemplateEbsBlockDeviceRequest) => any;
    const isa: (o: any) => o is LaunchTemplateEbsBlockDeviceRequest;
}
/**
 * <p>
 *             Describes an elastic inference accelerator.
 *         </p>
 */
export interface LaunchTemplateElasticInferenceAccelerator {
    __type?: "LaunchTemplateElasticInferenceAccelerator";
    /**
     * <p>
     *     		The number of elastic inference accelerators to attach to the instance.
     *     	</p>
     *     	    <p>Default: 1</p>
     */
    Count?: number;
    /**
     * <p>
     *             The type of elastic inference accelerator. The possible values are eia1.medium, eia1.large, and eia1.xlarge.
     *         </p>
     */
    Type: string | undefined;
}
export declare namespace LaunchTemplateElasticInferenceAccelerator {
    const filterSensitiveLog: (obj: LaunchTemplateElasticInferenceAccelerator) => any;
    const isa: (o: any) => o is LaunchTemplateElasticInferenceAccelerator;
}
/**
 * <p>
 *             Describes an elastic inference accelerator.
 *         </p>
 */
export interface LaunchTemplateElasticInferenceAcceleratorResponse {
    __type?: "LaunchTemplateElasticInferenceAcceleratorResponse";
    /**
     * <p>
     *     		The number of elastic inference accelerators to attach to the instance.
     *     	</p>
     *     	    <p>Default: 1</p>
     */
    Count?: number;
    /**
     * <p>
     *             The type of elastic inference accelerator. The possible values are eia1.medium, eia1.large, and eia1.xlarge.
     *         </p>
     */
    Type?: string;
}
export declare namespace LaunchTemplateElasticInferenceAcceleratorResponse {
    const filterSensitiveLog: (obj: LaunchTemplateElasticInferenceAcceleratorResponse) => any;
    const isa: (o: any) => o is LaunchTemplateElasticInferenceAcceleratorResponse;
}
export declare enum LaunchTemplateErrorCode {
    LAUNCH_TEMPLATE_ID_DOES_NOT_EXIST = "launchTemplateIdDoesNotExist",
    LAUNCH_TEMPLATE_ID_MALFORMED = "launchTemplateIdMalformed",
    LAUNCH_TEMPLATE_NAME_DOES_NOT_EXIST = "launchTemplateNameDoesNotExist",
    LAUNCH_TEMPLATE_NAME_MALFORMED = "launchTemplateNameMalformed",
    LAUNCH_TEMPLATE_VERSION_DOES_NOT_EXIST = "launchTemplateVersionDoesNotExist",
    UNEXPECTED_ERROR = "unexpectedError"
}
/**
 * <p>Indicates whether an instance is configured for hibernation.</p>
 */
export interface LaunchTemplateHibernationOptions {
    __type?: "LaunchTemplateHibernationOptions";
    /**
     * <p>If this parameter is set to <code>true</code>, the instance is enabled for hibernation; otherwise, it is not enabled for hibernation.</p>
     */
    Configured?: boolean;
}
export declare namespace LaunchTemplateHibernationOptions {
    const filterSensitiveLog: (obj: LaunchTemplateHibernationOptions) => any;
    const isa: (o: any) => o is LaunchTemplateHibernationOptions;
}
/**
 * <p>Indicates whether the instance is configured for hibernation. This parameter is valid only
 *             if the instance meets the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html#hibernating-prerequisites">hibernation
 *                 prerequisites</a>.</p>
 */
export interface LaunchTemplateHibernationOptionsRequest {
    __type?: "LaunchTemplateHibernationOptionsRequest";
    /**
     * <p>If you set this parameter to <code>true</code>, the instance is enabled for hibernation.</p>
     *          <p>Default: <code>false</code>
     *          </p>
     */
    Configured?: boolean;
}
export declare namespace LaunchTemplateHibernationOptionsRequest {
    const filterSensitiveLog: (obj: LaunchTemplateHibernationOptionsRequest) => any;
    const isa: (o: any) => o is LaunchTemplateHibernationOptionsRequest;
}
export declare enum LaunchTemplateHttpTokensState {
    optional = "optional",
    required = "required"
}
/**
 * <p>Describes an IAM instance profile.</p>
 */
export interface LaunchTemplateIamInstanceProfileSpecification {
    __type?: "LaunchTemplateIamInstanceProfileSpecification";
    /**
     * <p>The Amazon Resource Name (ARN) of the instance profile.</p>
     */
    Arn?: string;
    /**
     * <p>The name of the instance profile.</p>
     */
    Name?: string;
}
export declare namespace LaunchTemplateIamInstanceProfileSpecification {
    const filterSensitiveLog: (obj: LaunchTemplateIamInstanceProfileSpecification) => any;
    const isa: (o: any) => o is LaunchTemplateIamInstanceProfileSpecification;
}
/**
 * <p>An IAM instance profile.</p>
 */
export interface LaunchTemplateIamInstanceProfileSpecificationRequest {
    __type?: "LaunchTemplateIamInstanceProfileSpecificationRequest";
    /**
     * <p>The Amazon Resource Name (ARN) of the instance profile.</p>
     */
    Arn?: string;
    /**
     * <p>The name of the instance profile.</p>
     */
    Name?: string;
}
export declare namespace LaunchTemplateIamInstanceProfileSpecificationRequest {
    const filterSensitiveLog: (obj: LaunchTemplateIamInstanceProfileSpecificationRequest) => any;
    const isa: (o: any) => o is LaunchTemplateIamInstanceProfileSpecificationRequest;
}
/**
 * <p>The market (purchasing) option for the instances.</p>
 */
export interface LaunchTemplateInstanceMarketOptions {
    __type?: "LaunchTemplateInstanceMarketOptions";
    /**
     * <p>The market type.</p>
     */
    MarketType?: MarketType | string;
    /**
     * <p>The options for Spot Instances.</p>
     */
    SpotOptions?: LaunchTemplateSpotMarketOptions;
}
export declare namespace LaunchTemplateInstanceMarketOptions {
    const filterSensitiveLog: (obj: LaunchTemplateInstanceMarketOptions) => any;
    const isa: (o: any) => o is LaunchTemplateInstanceMarketOptions;
}
/**
 * <p>The market (purchasing) option for the instances.</p>
 */
export interface LaunchTemplateInstanceMarketOptionsRequest {
    __type?: "LaunchTemplateInstanceMarketOptionsRequest";
    /**
     * <p>The market type.</p>
     */
    MarketType?: MarketType | string;
    /**
     * <p>The options for Spot Instances.</p>
     */
    SpotOptions?: LaunchTemplateSpotMarketOptionsRequest;
}
export declare namespace LaunchTemplateInstanceMarketOptionsRequest {
    const filterSensitiveLog: (obj: LaunchTemplateInstanceMarketOptionsRequest) => any;
    const isa: (o: any) => o is LaunchTemplateInstanceMarketOptionsRequest;
}
export declare type LaunchTemplateInstanceMetadataEndpointState = "disabled" | "enabled";
/**
 * <p>The metadata options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html">Instance Metadata and User Data</a> in the
 *                 <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
 */
export interface LaunchTemplateInstanceMetadataOptions {
    __type?: "LaunchTemplateInstanceMetadataOptions";
    /**
     * <p>This parameter enables or disables the HTTP metadata endpoint on your instances. If the parameter is not specified, the default state is <code>enabled</code>.</p>
     *         <note>
     *             <p>If you specify a value of <code>disabled</code>, you will not be able to access your instance metadata.
     *             </p>
     *         </note>
     */
    HttpEndpoint?: LaunchTemplateInstanceMetadataEndpointState | string;
    /**
     * <p>The desired HTTP PUT response hop limit for instance metadata requests. The larger the number, the further instance metadata requests can travel.</p>
     *          <p>Default: 1</p>
     *          <p>Possible values: Integers from 1 to 64</p>
     */
    HttpPutResponseHopLimit?: number;
    /**
     * <p>The state of token usage for your instance metadata requests. If the parameter is not specified in the request, the default state is <code>optional</code>.</p>
     *         <p>If the state is <code>optional</code>, you can choose to retrieve instance metadata with or without a signed token header on your request. If you retrieve the IAM role credentials without a token, the version 1.0 role credentials are returned. If you retrieve the IAM role credentials using a valid signed token, the version 2.0 role credentials are returned.</p>
     *         <p>If the state is <code>required</code>, you must send a signed token header with any instance metadata retrieval requests. In this state, retrieving the IAM role credentials always returns the version 2.0 credentials; the version 1.0 credentials are not available.</p>
     */
    HttpTokens?: LaunchTemplateHttpTokensState | string;
    /**
     * <p>The state of the metadata option changes.</p>
     *          <p>
     *             <code>pending</code> - The metadata options are being updated and the instance is not ready to process metadata traffic with the new selection.</p>
     *          <p>
     *             <code>applied</code> - The metadata options have been successfully applied on the instance.</p>
     */
    State?: LaunchTemplateInstanceMetadataOptionsState | string;
}
export declare namespace LaunchTemplateInstanceMetadataOptions {
    const filterSensitiveLog: (obj: LaunchTemplateInstanceMetadataOptions) => any;
    const isa: (o: any) => o is LaunchTemplateInstanceMetadataOptions;
}
/**
 * <p>The metadata options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html">Instance Metadata and User Data</a> in the
 *             <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
 */
export interface LaunchTemplateInstanceMetadataOptionsRequest {
    __type?: "LaunchTemplateInstanceMetadataOptionsRequest";
    /**
     * <p>This parameter enables or disables the HTTP metadata endpoint on your instances. If the parameter is not specified, the default state is <code>enabled</code>.</p>
     *          <note>
     *             <p>If you specify a value of <code>disabled</code>, you will not be able to access your instance metadata.
     * </p>
     *          </note>
     */
    HttpEndpoint?: LaunchTemplateInstanceMetadataEndpointState | string;
    /**
     * <p>The desired HTTP PUT response hop limit for instance metadata requests. The larger the number, the further instance metadata requests can travel.</p>
     *         <p>Default: 1</p>
     *         <p>Possible values: Integers from 1 to 64</p>
     */
    HttpPutResponseHopLimit?: number;
    /**
     * <p>The state of token usage for your instance metadata requests. If the parameter is not specified in the request, the default state is <code>optional</code>.</p>
     *         <p>If the state is <code>optional</code>, you can choose to retrieve instance metadata with or without a signed token header on your request. If you retrieve the IAM role credentials without a token, the version 1.0 role credentials are returned. If you retrieve the IAM role credentials using a valid signed token, the version 2.0 role credentials are returned.</p>
     *             <p>If the state is <code>required</code>, you must send a signed token header with any instance metadata retrieval requests. In this state, retrieving the IAM role credentials always returns the version 2.0 credentials; the version 1.0 credentials are not available.</p>
     */
    HttpTokens?: LaunchTemplateHttpTokensState | string;
}
export declare namespace LaunchTemplateInstanceMetadataOptionsRequest {
    const filterSensitiveLog: (obj: LaunchTemplateInstanceMetadataOptionsRequest) => any;
    const isa: (o: any) => o is LaunchTemplateInstanceMetadataOptionsRequest;
}
export declare type LaunchTemplateInstanceMetadataOptionsState = "applied" | "pending";
/**
 * <p>Describes a network interface.</p>
 */
export interface LaunchTemplateInstanceNetworkInterfaceSpecification {
    __type?: "LaunchTemplateInstanceNetworkInterfaceSpecification";
    /**
     * <p>Indicates whether to associate a public IPv4 address with eth0 for a new network interface.</p>
     */
    AssociatePublicIpAddress?: boolean;
    /**
     * <p>Indicates whether the network interface is deleted when the instance is terminated.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>A description for the network interface.</p>
     */
    Description?: string;
    /**
     * <p>The device index for the network interface attachment.</p>
     */
    DeviceIndex?: number;
    /**
     * <p>The IDs of one or more security groups.</p>
     */
    Groups?: string[];
    /**
     * <p>The type of network interface.</p>
     */
    InterfaceType?: string;
    /**
     * <p>The number of IPv6 addresses for the network interface.</p>
     */
    Ipv6AddressCount?: number;
    /**
     * <p>The IPv6 addresses for the network interface.</p>
     */
    Ipv6Addresses?: InstanceIpv6Address[];
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The primary private IPv4 address of the network interface.</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>One or more private IPv4 addresses.</p>
     */
    PrivateIpAddresses?: PrivateIpAddressSpecification[];
    /**
     * <p>The number of secondary private IPv4 addresses for the network interface.</p>
     */
    SecondaryPrivateIpAddressCount?: number;
    /**
     * <p>The ID of the subnet for the network interface.</p>
     */
    SubnetId?: string;
}
export declare namespace LaunchTemplateInstanceNetworkInterfaceSpecification {
    const filterSensitiveLog: (obj: LaunchTemplateInstanceNetworkInterfaceSpecification) => any;
    const isa: (o: any) => o is LaunchTemplateInstanceNetworkInterfaceSpecification;
}
/**
 * <p>The parameters for a network interface.</p>
 */
export interface LaunchTemplateInstanceNetworkInterfaceSpecificationRequest {
    __type?: "LaunchTemplateInstanceNetworkInterfaceSpecificationRequest";
    /**
     * <p>Associates a public IPv4 address with eth0 for a new network interface.</p>
     */
    AssociatePublicIpAddress?: boolean;
    /**
     * <p>Indicates whether the network interface is deleted when the instance is terminated.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>A description for the network interface.</p>
     */
    Description?: string;
    /**
     * <p>The device index for the network interface attachment.</p>
     */
    DeviceIndex?: number;
    /**
     * <p>The IDs of one or more security groups.</p>
     */
    Groups?: string[];
    /**
     * <p>The type of network interface. To create an Elastic Fabric Adapter (EFA), specify
     * 			<code>efa</code>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/efa.html">Elastic Fabric Adapter</a> in the
     * 			<i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     * 		       <p>If you are not creating an EFA, specify <code>interface</code> or omit this parameter.</p>
     * 		       <p>Valid values: <code>interface</code> | <code>efa</code>
     *          </p>
     */
    InterfaceType?: string;
    /**
     * <p>The number of IPv6 addresses to assign to a network interface. Amazon EC2 automatically selects the IPv6 addresses from the subnet range. You can't use this option if specifying specific IPv6 addresses.</p>
     */
    Ipv6AddressCount?: number;
    /**
     * <p>One or more specific IPv6 addresses from the IPv6 CIDR block range of your subnet. You can't use this option if you're specifying a number of IPv6 addresses.</p>
     */
    Ipv6Addresses?: InstanceIpv6AddressRequest[];
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The primary private IPv4 address of the network interface.</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>One or more private IPv4 addresses.</p>
     */
    PrivateIpAddresses?: PrivateIpAddressSpecification[];
    /**
     * <p>The number of secondary private IPv4 addresses to assign to a network interface.</p>
     */
    SecondaryPrivateIpAddressCount?: number;
    /**
     * <p>The ID of the subnet for the network interface.</p>
     */
    SubnetId?: string;
}
export declare namespace LaunchTemplateInstanceNetworkInterfaceSpecificationRequest {
    const filterSensitiveLog: (obj: LaunchTemplateInstanceNetworkInterfaceSpecificationRequest) => any;
    const isa: (o: any) => o is LaunchTemplateInstanceNetworkInterfaceSpecificationRequest;
}
/**
 * <p>Describes a license configuration.</p>
 */
export interface LaunchTemplateLicenseConfiguration {
    __type?: "LaunchTemplateLicenseConfiguration";
    /**
     * <p>The Amazon Resource Name (ARN) of the license configuration.</p>
     */
    LicenseConfigurationArn?: string;
}
export declare namespace LaunchTemplateLicenseConfiguration {
    const filterSensitiveLog: (obj: LaunchTemplateLicenseConfiguration) => any;
    const isa: (o: any) => o is LaunchTemplateLicenseConfiguration;
}
/**
 * <p>Describes a license configuration.</p>
 */
export interface LaunchTemplateLicenseConfigurationRequest {
    __type?: "LaunchTemplateLicenseConfigurationRequest";
    /**
     * <p>The Amazon Resource Name (ARN) of the license configuration.</p>
     */
    LicenseConfigurationArn?: string;
}
export declare namespace LaunchTemplateLicenseConfigurationRequest {
    const filterSensitiveLog: (obj: LaunchTemplateLicenseConfigurationRequest) => any;
    const isa: (o: any) => o is LaunchTemplateLicenseConfigurationRequest;
}
/**
 * <p>Describes overrides for a launch template.</p>
 */
export interface LaunchTemplateOverrides {
    __type?: "LaunchTemplateOverrides";
    /**
     * <p>The Availability Zone in which to launch the instances.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The priority for the launch template override. If <b>OnDemandAllocationStrategy</b> is set to <code>prioritized</code>, Spot Fleet uses
     *             priority to determine which launch template override to use first in fulfilling
     *             On-Demand capacity. The highest priority is launched first. Valid values are whole numbers starting at <code>0</code>. The lower
     *             the number, the higher the priority. If no number is set, the launch template override
     *             has the lowest priority.</p>
     */
    Priority?: number;
    /**
     * <p>The maximum price per unit hour that you are willing to pay for a Spot Instance.</p>
     */
    SpotPrice?: string;
    /**
     * <p>The ID of the subnet in which to launch the instances.</p>
     */
    SubnetId?: string;
    /**
     * <p>The number of units provided by the specified instance type.</p>
     */
    WeightedCapacity?: number;
}
export declare namespace LaunchTemplateOverrides {
    const filterSensitiveLog: (obj: LaunchTemplateOverrides) => any;
    const isa: (o: any) => o is LaunchTemplateOverrides;
}
/**
 * <p>Describes the placement of an instance.</p>
 */
export interface LaunchTemplatePlacement {
    __type?: "LaunchTemplatePlacement";
    /**
     * <p>The affinity setting for the instance on the Dedicated Host.</p>
     */
    Affinity?: string;
    /**
     * <p>The Availability Zone of the instance.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The name of the placement group for the instance.</p>
     */
    GroupName?: string;
    /**
     * <p>The ID of the Dedicated Host for the instance.</p>
     */
    HostId?: string;
    /**
     * <p>The ARN of the host resource group in which to launch the instances.
     *     	</p>
     */
    HostResourceGroupArn?: string;
    /**
     * <p>The number of the partition the instance should launch in. Valid only if the placement group strategy is set to <code>partition</code>.</p>
     */
    PartitionNumber?: number;
    /**
     * <p>Reserved for future use.</p>
     */
    SpreadDomain?: string;
    /**
     * <p>The tenancy of the instance (if the instance is running in a VPC). An instance with a tenancy of <code>dedicated</code> runs on single-tenant hardware. </p>
     */
    Tenancy?: Tenancy | string;
}
export declare namespace LaunchTemplatePlacement {
    const filterSensitiveLog: (obj: LaunchTemplatePlacement) => any;
    const isa: (o: any) => o is LaunchTemplatePlacement;
}
/**
 * <p>Describes the placement of an instance.</p>
 */
export interface LaunchTemplatePlacementRequest {
    __type?: "LaunchTemplatePlacementRequest";
    /**
     * <p>The affinity setting for an instance on a Dedicated Host.</p>
     */
    Affinity?: string;
    /**
     * <p>The Availability Zone for the instance.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The name of the placement group for the instance.</p>
     */
    GroupName?: string;
    /**
     * <p>The ID of the Dedicated Host for the instance.</p>
     */
    HostId?: string;
    /**
     * <p>The ARN of the host resource group in which to launch the instances. If you specify a host
     * 		resource group ARN, omit the <b>Tenancy</b> parameter
     * 		or set it to <code>host</code>.</p>
     */
    HostResourceGroupArn?: string;
    /**
     * <p>The number of the partition the instance should launch in. Valid only if the placement group strategy is set to <code>partition</code>.</p>
     */
    PartitionNumber?: number;
    /**
     * <p>Reserved for future use.</p>
     */
    SpreadDomain?: string;
    /**
     * <p>The tenancy of the instance (if the instance is running in a VPC). An instance with a tenancy
     *         	of dedicated runs on single-tenant hardware.</p>
     */
    Tenancy?: Tenancy | string;
}
export declare namespace LaunchTemplatePlacementRequest {
    const filterSensitiveLog: (obj: LaunchTemplatePlacementRequest) => any;
    const isa: (o: any) => o is LaunchTemplatePlacementRequest;
}
/**
 * <p>Describes the monitoring for the instance.</p>
 */
export interface LaunchTemplatesMonitoring {
    __type?: "LaunchTemplatesMonitoring";
    /**
     * <p>Indicates whether detailed monitoring is enabled. Otherwise, basic monitoring is
     *             enabled.</p>
     */
    Enabled?: boolean;
}
export declare namespace LaunchTemplatesMonitoring {
    const filterSensitiveLog: (obj: LaunchTemplatesMonitoring) => any;
    const isa: (o: any) => o is LaunchTemplatesMonitoring;
}
/**
 * <p>Describes the monitoring for the instance.</p>
 */
export interface LaunchTemplatesMonitoringRequest {
    __type?: "LaunchTemplatesMonitoringRequest";
    /**
     * <p>Specify <code>true</code> to enable detailed monitoring. Otherwise, basic monitoring is enabled.</p>
     */
    Enabled?: boolean;
}
export declare namespace LaunchTemplatesMonitoringRequest {
    const filterSensitiveLog: (obj: LaunchTemplatesMonitoringRequest) => any;
    const isa: (o: any) => o is LaunchTemplatesMonitoringRequest;
}
/**
 * <p>The launch template to use. You must specify either the launch template ID or
 *             launch template name in the request, but not both.</p>
 */
export interface LaunchTemplateSpecification {
    __type?: "LaunchTemplateSpecification";
    /**
     * <p>The ID of the launch template.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template.</p>
     */
    LaunchTemplateName?: string;
    /**
     * <p>The version number of the launch template.</p>
     *         <p>Default: The default version for the launch template.</p>
     */
    Version?: string;
}
export declare namespace LaunchTemplateSpecification {
    const filterSensitiveLog: (obj: LaunchTemplateSpecification) => any;
    const isa: (o: any) => o is LaunchTemplateSpecification;
}
/**
 * <p>The options for Spot Instances.</p>
 */
export interface LaunchTemplateSpotMarketOptions {
    __type?: "LaunchTemplateSpotMarketOptions";
    /**
     * <p>The required duration for the Spot Instances (also known as Spot blocks), in minutes. This value must be a multiple of 60 (60, 120, 180, 240, 300, or 360).</p>
     */
    BlockDurationMinutes?: number;
    /**
     * <p>The behavior when a Spot Instance is interrupted.</p>
     */
    InstanceInterruptionBehavior?: InstanceInterruptionBehavior | string;
    /**
     * <p>The maximum hourly price you're willing to pay for the Spot Instances.</p>
     */
    MaxPrice?: string;
    /**
     * <p>The Spot Instance request type.</p>
     */
    SpotInstanceType?: SpotInstanceType | string;
    /**
     * <p>The end date of the request. For a one-time request, the request remains active until all instances launch, the request is canceled, or this date is reached. If the request is persistent, it remains active until it is canceled or this date and time is reached.</p>
     */
    ValidUntil?: Date;
}
export declare namespace LaunchTemplateSpotMarketOptions {
    const filterSensitiveLog: (obj: LaunchTemplateSpotMarketOptions) => any;
    const isa: (o: any) => o is LaunchTemplateSpotMarketOptions;
}
/**
 * <p>The options for Spot Instances.</p>
 */
export interface LaunchTemplateSpotMarketOptionsRequest {
    __type?: "LaunchTemplateSpotMarketOptionsRequest";
    /**
     * <p>The required duration for the Spot Instances (also known as Spot blocks), in minutes. This value must be a multiple of 60 (60, 120, 180, 240, 300, or 360).</p>
     */
    BlockDurationMinutes?: number;
    /**
     * <p>The behavior when a Spot Instance is interrupted. The default is <code>terminate</code>.</p>
     */
    InstanceInterruptionBehavior?: InstanceInterruptionBehavior | string;
    /**
     * <p>The maximum hourly price you're willing to pay for the Spot Instances.</p>
     */
    MaxPrice?: string;
    /**
     * <p>The Spot Instance request type.</p>
     */
    SpotInstanceType?: SpotInstanceType | string;
    /**
     * <p>The end date of the request.
     *             For a one-time request, the request remains active until all instances launch, the request is canceled, or this date is reached.
     *             If the request is persistent, it remains active until it is canceled or this date and time is reached.
     *             The default end date is 7 days from the current date.</p>
     */
    ValidUntil?: Date;
}
export declare namespace LaunchTemplateSpotMarketOptionsRequest {
    const filterSensitiveLog: (obj: LaunchTemplateSpotMarketOptionsRequest) => any;
    const isa: (o: any) => o is LaunchTemplateSpotMarketOptionsRequest;
}
/**
 * <p>The tag specification for the launch template.</p>
 */
export interface LaunchTemplateTagSpecification {
    __type?: "LaunchTemplateTagSpecification";
    /**
     * <p>The type of resource.</p>
     */
    ResourceType?: ResourceType | string;
    /**
     * <p>The tags for the resource.</p>
     */
    Tags?: Tag[];
}
export declare namespace LaunchTemplateTagSpecification {
    const filterSensitiveLog: (obj: LaunchTemplateTagSpecification) => any;
    const isa: (o: any) => o is LaunchTemplateTagSpecification;
}
/**
 * <p>The tags specification for the launch template.</p>
 */
export interface LaunchTemplateTagSpecificationRequest {
    __type?: "LaunchTemplateTagSpecificationRequest";
    /**
     * <p>The type of resource to tag. Currently, the resource types that support tagging on
     *             creation are <code>instance</code> and <code>volume</code>. To tag a resource after it
     *         	has been created, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html">CreateTags</a>.</p>
     */
    ResourceType?: ResourceType | string;
    /**
     * <p>The tags to apply to the resource.</p>
     */
    Tags?: Tag[];
}
export declare namespace LaunchTemplateTagSpecificationRequest {
    const filterSensitiveLog: (obj: LaunchTemplateTagSpecificationRequest) => any;
    const isa: (o: any) => o is LaunchTemplateTagSpecificationRequest;
}
/**
 * <p>Describes a launch template version.</p>
 */
export interface LaunchTemplateVersion {
    __type?: "LaunchTemplateVersion";
    /**
     * <p>The time the version was created.</p>
     */
    CreateTime?: Date;
    /**
     * <p>The principal that created the version.</p>
     */
    CreatedBy?: string;
    /**
     * <p>Indicates whether the version is the default version.</p>
     */
    DefaultVersion?: boolean;
    /**
     * <p>Information about the launch template.</p>
     */
    LaunchTemplateData?: ResponseLaunchTemplateData;
    /**
     * <p>The ID of the launch template.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template.</p>
     */
    LaunchTemplateName?: string;
    /**
     * <p>The description for the version.</p>
     */
    VersionDescription?: string;
    /**
     * <p>The version number.</p>
     */
    VersionNumber?: number;
}
export declare namespace LaunchTemplateVersion {
    const filterSensitiveLog: (obj: LaunchTemplateVersion) => any;
    const isa: (o: any) => o is LaunchTemplateVersion;
}
/**
 * <p>Describes a license configuration.</p>
 */
export interface LicenseConfiguration {
    __type?: "LicenseConfiguration";
    /**
     * <p>The Amazon Resource Name (ARN) of the license configuration.</p>
     */
    LicenseConfigurationArn?: string;
}
export declare namespace LicenseConfiguration {
    const filterSensitiveLog: (obj: LicenseConfiguration) => any;
    const isa: (o: any) => o is LicenseConfiguration;
}
/**
 * <p>Describes a license configuration.</p>
 */
export interface LicenseConfigurationRequest {
    __type?: "LicenseConfigurationRequest";
    /**
     * <p>The Amazon Resource Name (ARN) of the license configuration.</p>
     */
    LicenseConfigurationArn?: string;
}
export declare namespace LicenseConfigurationRequest {
    const filterSensitiveLog: (obj: LicenseConfigurationRequest) => any;
    const isa: (o: any) => o is LicenseConfigurationRequest;
}
export declare type ListingState = "available" | "cancelled" | "pending" | "sold";
export declare type ListingStatus = "active" | "cancelled" | "closed" | "pending";
/**
 * <p>Describes the Classic Load Balancers and target groups to attach to a Spot Fleet request.</p>
 */
export interface LoadBalancersConfig {
    __type?: "LoadBalancersConfig";
    /**
     * <p>The Classic Load Balancers.</p>
     */
    ClassicLoadBalancersConfig?: ClassicLoadBalancersConfig;
    /**
     * <p>The target groups.</p>
     */
    TargetGroupsConfig?: TargetGroupsConfig;
}
export declare namespace LoadBalancersConfig {
    const filterSensitiveLog: (obj: LoadBalancersConfig) => any;
    const isa: (o: any) => o is LoadBalancersConfig;
}
/**
 * <p>Describes a load permission.</p>
 */
export interface LoadPermission {
    __type?: "LoadPermission";
    /**
     * <p>The name of the group.</p>
     */
    Group?: PermissionGroup | string;
    /**
     * <p>The AWS account ID.</p>
     */
    UserId?: string;
}
export declare namespace LoadPermission {
    const filterSensitiveLog: (obj: LoadPermission) => any;
    const isa: (o: any) => o is LoadPermission;
}
/**
 * <p>Describes modifications to the load permissions of an Amazon FPGA image (AFI).</p>
 */
export interface LoadPermissionModifications {
    __type?: "LoadPermissionModifications";
    /**
     * <p>The load permissions to add.</p>
     */
    Add?: LoadPermissionRequest[];
    /**
     * <p>The load permissions to remove.</p>
     */
    Remove?: LoadPermissionRequest[];
}
export declare namespace LoadPermissionModifications {
    const filterSensitiveLog: (obj: LoadPermissionModifications) => any;
    const isa: (o: any) => o is LoadPermissionModifications;
}
/**
 * <p>Describes a load permission.</p>
 */
export interface LoadPermissionRequest {
    __type?: "LoadPermissionRequest";
    /**
     * <p>The name of the group.</p>
     */
    Group?: PermissionGroup | string;
    /**
     * <p>The AWS account ID.</p>
     */
    UserId?: string;
}
export declare namespace LoadPermissionRequest {
    const filterSensitiveLog: (obj: LoadPermissionRequest) => any;
    const isa: (o: any) => o is LoadPermissionRequest;
}
/**
 * <p>Describes a local gateway.</p>
 */
export interface LocalGateway {
    __type?: "LocalGateway";
    /**
     * <p>The ID of the local gateway.</p>
     */
    LocalGatewayId?: string;
    /**
     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
     */
    OutpostArn?: string;
    /**
     * <p>The ID of the AWS account ID that owns the local gateway.</p>
     */
    OwnerId?: string;
    /**
     * <p>The state of the local gateway.</p>
     */
    State?: string;
    /**
     * <p>The tags assigned to the local gateway.</p>
     */
    Tags?: Tag[];
}
export declare namespace LocalGateway {
    const filterSensitiveLog: (obj: LocalGateway) => any;
    const isa: (o: any) => o is LocalGateway;
}
/**
 * <p>Describes a route for a local gateway route table.</p>
 */
export interface LocalGatewayRoute {
    __type?: "LocalGatewayRoute";
    /**
     * <p>The CIDR block used for destination matches.</p>
     */
    DestinationCidrBlock?: string;
    /**
     * <p>The ID of the local gateway route table.</p>
     */
    LocalGatewayRouteTableId?: string;
    /**
     * <p>The ID of the virtual interface group.</p>
     */
    LocalGatewayVirtualInterfaceGroupId?: string;
    /**
     * <p>The state of the route.</p>
     */
    State?: LocalGatewayRouteState | string;
    /**
     * <p>The route type.</p>
     */
    Type?: LocalGatewayRouteType | string;
}
export declare namespace LocalGatewayRoute {
    const filterSensitiveLog: (obj: LocalGatewayRoute) => any;
    const isa: (o: any) => o is LocalGatewayRoute;
}
export declare type LocalGatewayRouteState = "active" | "blackhole" | "deleted" | "deleting" | "pending";
/**
 * <p>Describes a local gateway route table.</p>
 */
export interface LocalGatewayRouteTable {
    __type?: "LocalGatewayRouteTable";
    /**
     * <p>The ID of the local gateway.</p>
     */
    LocalGatewayId?: string;
    /**
     * <p>The ID of the local gateway route table.</p>
     */
    LocalGatewayRouteTableId?: string;
    /**
     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
     */
    OutpostArn?: string;
    /**
     * <p>The state of the local gateway route table.</p>
     */
    State?: string;
    /**
     * <p>The tags assigned to the local gateway route table.</p>
     */
    Tags?: Tag[];
}
export declare namespace LocalGatewayRouteTable {
    const filterSensitiveLog: (obj: LocalGatewayRouteTable) => any;
    const isa: (o: any) => o is LocalGatewayRouteTable;
}
/**
 * <p>Describes an association between a local gateway route table and a virtual interface group.</p>
 */
export interface LocalGatewayRouteTableVirtualInterfaceGroupAssociation {
    __type?: "LocalGatewayRouteTableVirtualInterfaceGroupAssociation";
    /**
     * <p>The ID of the local gateway.</p>
     */
    LocalGatewayId?: string;
    /**
     * <p>The ID of the local gateway route table.</p>
     */
    LocalGatewayRouteTableId?: string;
    /**
     * <p>The ID of the association.</p>
     */
    LocalGatewayRouteTableVirtualInterfaceGroupAssociationId?: string;
    /**
     * <p>The ID of the virtual interface group.</p>
     */
    LocalGatewayVirtualInterfaceGroupId?: string;
    /**
     * <p>The state of the association.</p>
     */
    State?: string;
    /**
     * <p>The tags assigned to the association.</p>
     */
    Tags?: Tag[];
}
export declare namespace LocalGatewayRouteTableVirtualInterfaceGroupAssociation {
    const filterSensitiveLog: (obj: LocalGatewayRouteTableVirtualInterfaceGroupAssociation) => any;
    const isa: (o: any) => o is LocalGatewayRouteTableVirtualInterfaceGroupAssociation;
}
/**
 * <p>Describes an association between a local gateway route table and a VPC.</p>
 */
export interface LocalGatewayRouteTableVpcAssociation {
    __type?: "LocalGatewayRouteTableVpcAssociation";
    /**
     * <p>The ID of the local gateway.</p>
     */
    LocalGatewayId?: string;
    /**
     * <p>The ID of the local gateway route table.</p>
     */
    LocalGatewayRouteTableId?: string;
    /**
     * <p>The ID of the association.</p>
     */
    LocalGatewayRouteTableVpcAssociationId?: string;
    /**
     * <p>The state of the association.</p>
     */
    State?: string;
    /**
     * <p>The tags assigned to the association.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace LocalGatewayRouteTableVpcAssociation {
    const filterSensitiveLog: (obj: LocalGatewayRouteTableVpcAssociation) => any;
    const isa: (o: any) => o is LocalGatewayRouteTableVpcAssociation;
}
export declare type LocalGatewayRouteType = "propagated" | "static";
/**
 * <p>Describes a local gateway virtual interface.</p>
 */
export interface LocalGatewayVirtualInterface {
    __type?: "LocalGatewayVirtualInterface";
    /**
     * <p>The local address.</p>
     */
    LocalAddress?: string;
    /**
     * <p>The Border Gateway Protocol (BGP) Autonomous System Number (ASN) of the local gateway.</p>
     */
    LocalBgpAsn?: number;
    /**
     * <p>The ID of the local gateway.</p>
     */
    LocalGatewayId?: string;
    /**
     * <p>The ID of the virtual interface.</p>
     */
    LocalGatewayVirtualInterfaceId?: string;
    /**
     * <p>The peer address.</p>
     */
    PeerAddress?: string;
    /**
     * <p>The peer BGP ASN.</p>
     */
    PeerBgpAsn?: number;
    /**
     * <p>The tags assigned to the virtual interface.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the VLAN.</p>
     */
    Vlan?: number;
}
export declare namespace LocalGatewayVirtualInterface {
    const filterSensitiveLog: (obj: LocalGatewayVirtualInterface) => any;
    const isa: (o: any) => o is LocalGatewayVirtualInterface;
}
/**
 * <p>Describes a local gateway virtual interface group.</p>
 */
export interface LocalGatewayVirtualInterfaceGroup {
    __type?: "LocalGatewayVirtualInterfaceGroup";
    /**
     * <p>The ID of the local gateway.</p>
     */
    LocalGatewayId?: string;
    /**
     * <p>The ID of the virtual interface group.</p>
     */
    LocalGatewayVirtualInterfaceGroupId?: string;
    /**
     * <p>The IDs of the virtual interfaces.</p>
     */
    LocalGatewayVirtualInterfaceIds?: string[];
    /**
     * <p>The tags assigned to the virtual interface group.</p>
     */
    Tags?: Tag[];
}
export declare namespace LocalGatewayVirtualInterfaceGroup {
    const filterSensitiveLog: (obj: LocalGatewayVirtualInterfaceGroup) => any;
    const isa: (o: any) => o is LocalGatewayVirtualInterfaceGroup;
}
export declare type LocationType = "availability-zone" | "availability-zone-id" | "region";
export declare type LogDestinationType = "cloud-watch-logs" | "s3";
export declare type MarketType = "spot";
export declare type MembershipType = "igmp" | "static";
/**
 * <p>Describes the memory for the instance type.</p>
 */
export interface MemoryInfo {
    __type?: "MemoryInfo";
    /**
     * <p>Size of the memory, in MiB.</p>
     */
    SizeInMiB?: number;
}
export declare namespace MemoryInfo {
    const filterSensitiveLog: (obj: MemoryInfo) => any;
    const isa: (o: any) => o is MemoryInfo;
}
export interface ModifyCapacityReservationRequest {
    __type?: "ModifyCapacityReservationRequest";
    /**
     * <p>The ID of the Capacity Reservation.</p>
     */
    CapacityReservationId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The date and time at which the Capacity Reservation expires. When a Capacity Reservation expires, the reserved capacity
     * 			is released and you can no longer launch instances into it. The Capacity Reservation's state changes to
     * 				<code>expired</code> when it reaches its end date and time.</p>
     * 		       <p>The Capacity Reservation is cancelled within an hour from the specified time. For example, if you specify
     * 			5/31/2019, 13:30:55, the Capacity Reservation is guaranteed to end between 13:30:55 and 14:30:55 on 5/31/2019.</p>
     * 		       <p>You must provide an <code>EndDate</code> value if <code>EndDateType</code> is
     * 				<code>limited</code>. Omit <code>EndDate</code> if <code>EndDateType</code> is
     * 				<code>unlimited</code>.</p>
     */
    EndDate?: Date;
    /**
     * <p>Indicates the way in which the Capacity Reservation ends. A Capacity Reservation can have one of the following end
     * 			types:</p>
     * 		       <ul>
     *             <li>
     *                <p>
     *                   <code>unlimited</code> - The Capacity Reservation remains active until you explicitly cancel it. Do not
     * 					provide an <code>EndDate</code> value if <code>EndDateType</code> is
     * 						<code>unlimited</code>.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>limited</code> - The Capacity Reservation expires automatically at a specified date and time. You must
     * 					provide an <code>EndDate</code> value if <code>EndDateType</code> is
     * 						<code>limited</code>.</p>
     *             </li>
     *          </ul>
     */
    EndDateType?: EndDateType | string;
    /**
     * <p>The number of instances for which to reserve capacity.</p>
     */
    InstanceCount?: number;
}
export declare namespace ModifyCapacityReservationRequest {
    const filterSensitiveLog: (obj: ModifyCapacityReservationRequest) => any;
    const isa: (o: any) => o is ModifyCapacityReservationRequest;
}
export interface ModifyCapacityReservationResult {
    __type?: "ModifyCapacityReservationResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace ModifyCapacityReservationResult {
    const filterSensitiveLog: (obj: ModifyCapacityReservationResult) => any;
    const isa: (o: any) => o is ModifyCapacityReservationResult;
}
export interface ModifyClientVpnEndpointRequest {
    __type?: "ModifyClientVpnEndpointRequest";
    /**
     * <p>The ID of the Client VPN endpoint to modify.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Information about the client connection logging options.</p>
     * 		       <p>If you enable client connection logging, data about client connections is sent to a
     * 			Cloudwatch Logs log stream. The following information is logged:</p>
     * 		       <ul>
     *             <li>
     *                <p>Client connection requests</p>
     *             </li>
     *             <li>
     *                <p>Client connection results (successful and unsuccessful)</p>
     *             </li>
     *             <li>
     *                <p>Reasons for unsuccessful client connection requests</p>
     *             </li>
     *             <li>
     *                <p>Client connection termination time</p>
     *             </li>
     *          </ul>
     */
    ConnectionLogOptions?: ConnectionLogOptions;
    /**
     * <p>A brief description of the Client VPN endpoint.</p>
     */
    Description?: string;
    /**
     * <p>Information about the DNS servers to be used by Client VPN connections. A Client VPN endpoint can have
     * 			up to two DNS servers.</p>
     */
    DnsServers?: DnsServersOptionsModifyStructure;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ARN of the server certificate to be used. The server certificate must be provisioned in AWS Certificate Manager (ACM).</p>
     */
    ServerCertificateArn?: string;
    /**
     * <p>Indicates whether the VPN is split-tunnel.</p>
     *         <p>For information about split-tunnel VPN endpoints, see <a href="https://docs.aws.amazon.com/vpn/latest/clientvpn-admin/split-tunnel-vpn.html">Split-Tunnel AWS Client VPN Endpoint</a> in the <i>AWS
     *                 Client VPN Administrator Guide</i>.</p>
     */
    SplitTunnel?: boolean;
    /**
     * <p>The port number to assign to the Client VPN endpoint for TCP and UDP traffic.</p>
     * 	        <p>Valid Values: <code>443</code> | <code>1194</code>
     *          </p>
     * 	        <p>Default Value: <code>443</code>
     *          </p>
     */
    VpnPort?: number;
}
export declare namespace ModifyClientVpnEndpointRequest {
    const filterSensitiveLog: (obj: ModifyClientVpnEndpointRequest) => any;
    const isa: (o: any) => o is ModifyClientVpnEndpointRequest;
}
export interface ModifyClientVpnEndpointResult {
    __type?: "ModifyClientVpnEndpointResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace ModifyClientVpnEndpointResult {
    const filterSensitiveLog: (obj: ModifyClientVpnEndpointResult) => any;
    const isa: (o: any) => o is ModifyClientVpnEndpointResult;
}
export interface ModifyDefaultCreditSpecificationRequest {
    __type?: "ModifyDefaultCreditSpecificationRequest";
    /**
     * <p>The credit option for CPU usage of the instance family.</p>
     *         <p>Valid Values: <code>standard</code> | <code>unlimited</code>
     *          </p>
     */
    CpuCredits: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The instance family.</p>
     */
    InstanceFamily: UnlimitedSupportedInstanceFamily | string | undefined;
}
export declare namespace ModifyDefaultCreditSpecificationRequest {
    const filterSensitiveLog: (obj: ModifyDefaultCreditSpecificationRequest) => any;
    const isa: (o: any) => o is ModifyDefaultCreditSpecificationRequest;
}
export interface ModifyDefaultCreditSpecificationResult {
    __type?: "ModifyDefaultCreditSpecificationResult";
    /**
     * <p>The default credit option for CPU usage of the instance family.</p>
     */
    InstanceFamilyCreditSpecification?: InstanceFamilyCreditSpecification;
}
export declare namespace ModifyDefaultCreditSpecificationResult {
    const filterSensitiveLog: (obj: ModifyDefaultCreditSpecificationResult) => any;
    const isa: (o: any) => o is ModifyDefaultCreditSpecificationResult;
}
export interface ModifyEbsDefaultKmsKeyIdRequest {
    __type?: "ModifyEbsDefaultKmsKeyIdRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The identifier of the AWS Key Management Service (AWS KMS) customer master key (CMK) to use for Amazon EBS encryption.
     *       If this parameter is not specified, your AWS managed CMK for EBS is used. If <code>KmsKeyId</code> is
     *       specified, the encrypted state must be <code>true</code>.</p>
     *          <p>You can specify the CMK using any of the following:</p>
     *          <ul>
     *             <li>
     *                <p>Key ID. For example, key/1234abcd-12ab-34cd-56ef-1234567890ab.</p>
     *             </li>
     *             <li>
     *                <p>Key alias. For example, alias/ExampleAlias.</p>
     *             </li>
     *             <li>
     *                <p>Key ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:key/<i>abcd1234-a123-456a-a12b-a123b4cd56ef</i>.</p>
     *             </li>
     *             <li>
     *                <p>Alias ARN. For example, arn:aws:kms:<i>us-east-1</i>:<i>012345678910</i>:alias/<i>ExampleAlias</i>.</p>
     *             </li>
     *          </ul>
     *          <p>AWS authenticates the CMK asynchronously. Therefore, if you specify an ID, alias, or ARN that is not valid,
     *       the action can appear to complete, but eventually fails.</p>
     *          <p>Amazon EBS does not support asymmetric CMKs.</p>
     */
    KmsKeyId: string | undefined;
}
export declare namespace ModifyEbsDefaultKmsKeyIdRequest {
    const filterSensitiveLog: (obj: ModifyEbsDefaultKmsKeyIdRequest) => any;
    const isa: (o: any) => o is ModifyEbsDefaultKmsKeyIdRequest;
}
export interface ModifyEbsDefaultKmsKeyIdResult {
    __type?: "ModifyEbsDefaultKmsKeyIdResult";
    /**
     * <p>The Amazon Resource Name (ARN) of the default CMK for encryption by default.</p>
     */
    KmsKeyId?: string;
}
export declare namespace ModifyEbsDefaultKmsKeyIdResult {
    const filterSensitiveLog: (obj: ModifyEbsDefaultKmsKeyIdResult) => any;
    const isa: (o: any) => o is ModifyEbsDefaultKmsKeyIdResult;
}
export interface ModifyFleetRequest {
    __type?: "ModifyFleetRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Indicates whether running instances should be terminated if the total target capacity of
     *          the EC2 Fleet is decreased below the current size of the EC2 Fleet.</p>
     */
    ExcessCapacityTerminationPolicy?: FleetExcessCapacityTerminationPolicy | string;
    /**
     * <p>The ID of the EC2 Fleet.</p>
     */
    FleetId: string | undefined;
    /**
     * <p>The size of the EC2 Fleet.</p>
     */
    TargetCapacitySpecification: TargetCapacitySpecificationRequest | undefined;
}
export declare namespace ModifyFleetRequest {
    const filterSensitiveLog: (obj: ModifyFleetRequest) => any;
    const isa: (o: any) => o is ModifyFleetRequest;
}
export interface ModifyFleetResult {
    __type?: "ModifyFleetResult";
    /**
     * <p>Is <code>true</code> if the request succeeds, and an error otherwise.</p>
     */
    Return?: boolean;
}
export declare namespace ModifyFleetResult {
    const filterSensitiveLog: (obj: ModifyFleetResult) => any;
    const isa: (o: any) => o is ModifyFleetResult;
}
export interface ModifyFpgaImageAttributeRequest {
    __type?: "ModifyFpgaImageAttributeRequest";
    /**
     * <p>The name of the attribute.</p>
     */
    Attribute?: FpgaImageAttributeName | string;
    /**
     * <p>A description for the AFI.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the AFI.</p>
     */
    FpgaImageId: string | undefined;
    /**
     * <p>The load permission for the AFI.</p>
     */
    LoadPermission?: LoadPermissionModifications;
    /**
     * <p>A name for the AFI.</p>
     */
    Name?: string;
    /**
     * <p>The operation type.</p>
     */
    OperationType?: OperationType | string;
    /**
     * <p>The product codes. After you add a product code to an AFI, it can't be removed.
     * 		    This parameter is valid only when modifying the <code>productCodes</code> attribute.</p>
     */
    ProductCodes?: string[];
    /**
     * <p>The user groups. This parameter is valid only when modifying the <code>loadPermission</code> attribute.</p>
     */
    UserGroups?: string[];
    /**
     * <p>The AWS account IDs. This parameter is valid only when modifying the <code>loadPermission</code> attribute.</p>
     */
    UserIds?: string[];
}
export declare namespace ModifyFpgaImageAttributeRequest {
    const filterSensitiveLog: (obj: ModifyFpgaImageAttributeRequest) => any;
    const isa: (o: any) => o is ModifyFpgaImageAttributeRequest;
}
export interface ModifyFpgaImageAttributeResult {
    __type?: "ModifyFpgaImageAttributeResult";
    /**
     * <p>Information about the attribute.</p>
     */
    FpgaImageAttribute?: FpgaImageAttribute;
}
export declare namespace ModifyFpgaImageAttributeResult {
    const filterSensitiveLog: (obj: ModifyFpgaImageAttributeResult) => any;
    const isa: (o: any) => o is ModifyFpgaImageAttributeResult;
}
export interface ModifyHostsRequest {
    __type?: "ModifyHostsRequest";
    /**
     * <p>Specify whether to enable or disable auto-placement.</p>
     */
    AutoPlacement?: AutoPlacement | string;
    /**
     * <p>The IDs of the Dedicated Hosts to modify.</p>
     */
    HostIds: string[] | undefined;
    /**
     * <p>Indicates whether to enable or disable host recovery for the Dedicated Host. For more information,
     * 			see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-recovery.html">
     * 				Host Recovery</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    HostRecovery?: HostRecovery | string;
    /**
     * <p>Specifies the instance family to be supported by the Dedicated Host. Specify this parameter
     * 			to modify a Dedicated Host to support multiple instance types within its current
     * 			instance family.</p>
     *
     * 		       <p>If you want to modify a Dedicated Host to support a specific instance type only, omit this parameter
     * 			and specify <b>InstanceType</b> instead. You cannot specify
     * 			<b>InstanceFamily</b> and <b>InstanceType</b>
     * 			in the same request.</p>
     */
    InstanceFamily?: string;
    /**
     * <p>Specifies the instance type to be supported by the Dedicated Host. Specify this parameter to
     * 			modify a Dedicated Host to support only a specific instance type.</p>
     *
     * 		       <p>If you want to modify a Dedicated Host to support multiple instance types in its current instance
     * 			family, omit this parameter and specify <b>InstanceFamily</b>
     * 			instead. You cannot specify <b>InstanceType</b> and
     * 			<b>InstanceFamily</b> in the same request.</p>
     */
    InstanceType?: string;
}
export declare namespace ModifyHostsRequest {
    const filterSensitiveLog: (obj: ModifyHostsRequest) => any;
    const isa: (o: any) => o is ModifyHostsRequest;
}
export interface ModifyHostsResult {
    __type?: "ModifyHostsResult";
    /**
     * <p>The IDs of the Dedicated Hosts that were successfully modified.</p>
     */
    Successful?: string[];
    /**
     * <p>The IDs of the Dedicated Hosts that could not be modified. Check whether the
     *             setting you requested can be used.</p>
     */
    Unsuccessful?: UnsuccessfulItem[];
}
export declare namespace ModifyHostsResult {
    const filterSensitiveLog: (obj: ModifyHostsResult) => any;
    const isa: (o: any) => o is ModifyHostsResult;
}
export interface ModifyIdentityIdFormatRequest {
    __type?: "ModifyIdentityIdFormatRequest";
    /**
     * <p>The ARN of the principal, which can be an IAM user, IAM role, or the root user. Specify
     *        <code>all</code> to modify the ID format for all IAM users, IAM roles, and the root user of
     *        the account.</p>
     */
    PrincipalArn: string | undefined;
    /**
     * <p>The type of resource: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> |
     *           <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> |
     *           <code>export-task</code> | <code>flow-log</code> | <code>image</code> |
     *           <code>import-task</code> | <code>internet-gateway</code> | <code>network-acl</code>
     *           | <code>network-acl-association</code> | <code>network-interface</code> |
     *           <code>network-interface-attachment</code> | <code>prefix-list</code> |
     *           <code>route-table</code> | <code>route-table-association</code> |
     *           <code>security-group</code> | <code>subnet</code> |
     *           <code>subnet-cidr-block-association</code> | <code>vpc</code> |
     *           <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>.</p>
     *          <p>Alternatively, use the <code>all-current</code> option to include all resource types that are
     *           currently within their opt-in period for longer IDs.</p>
     */
    Resource: string | undefined;
    /**
     * <p>Indicates whether the resource should use longer IDs (17-character IDs)</p>
     */
    UseLongIds: boolean | undefined;
}
export declare namespace ModifyIdentityIdFormatRequest {
    const filterSensitiveLog: (obj: ModifyIdentityIdFormatRequest) => any;
    const isa: (o: any) => o is ModifyIdentityIdFormatRequest;
}
export interface ModifyIdFormatRequest {
    __type?: "ModifyIdFormatRequest";
    /**
     * <p>The type of resource: <code>bundle</code> | <code>conversion-task</code> | <code>customer-gateway</code> | <code>dhcp-options</code> |
     *            <code>elastic-ip-allocation</code> | <code>elastic-ip-association</code> |
     *            <code>export-task</code> | <code>flow-log</code> | <code>image</code> |
     *            <code>import-task</code> | <code>internet-gateway</code> | <code>network-acl</code>
     *            | <code>network-acl-association</code> | <code>network-interface</code> |
     *            <code>network-interface-attachment</code> | <code>prefix-list</code> |
     *            <code>route-table</code> | <code>route-table-association</code> |
     *            <code>security-group</code> | <code>subnet</code> |
     *            <code>subnet-cidr-block-association</code> | <code>vpc</code> |
     *            <code>vpc-cidr-block-association</code> | <code>vpc-endpoint</code> | <code>vpc-peering-connection</code> | <code>vpn-connection</code> | <code>vpn-gateway</code>.</p>
     *          <p>Alternatively, use the <code>all-current</code> option to include all resource types that are
     *        currently within their opt-in period for longer IDs.</p>
     */
    Resource: string | undefined;
    /**
     * <p>Indicate whether the resource should use longer IDs (17-character IDs).</p>
     */
    UseLongIds: boolean | undefined;
}
export declare namespace ModifyIdFormatRequest {
    const filterSensitiveLog: (obj: ModifyIdFormatRequest) => any;
    const isa: (o: any) => o is ModifyIdFormatRequest;
}
/**
 * <p>Contains the parameters for ModifyImageAttribute.</p>
 */
export interface ModifyImageAttributeRequest {
    __type?: "ModifyImageAttributeRequest";
    /**
     * <p>The name of the attribute to modify.
     *        The valid values are <code>description</code>, <code>launchPermission</code>, and <code>productCodes</code>.</p>
     */
    Attribute?: string;
    /**
     * <p>A new description for the AMI.</p>
     */
    Description?: AttributeValue;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the AMI.</p>
     */
    ImageId: string | undefined;
    /**
     * <p>A new launch permission for the AMI.</p>
     */
    LaunchPermission?: LaunchPermissionModifications;
    /**
     * <p>The operation type.
     *        This parameter can be used only when the <code>Attribute</code> parameter is <code>launchPermission</code>.</p>
     */
    OperationType?: OperationType | string;
    /**
     * <p>The DevPay product codes. After you add a product code to an AMI, it can't be removed.</p>
     */
    ProductCodes?: string[];
    /**
     * <p>The user groups.
     *        This parameter can be used only when the <code>Attribute</code> parameter is <code>launchPermission</code>.</p>
     */
    UserGroups?: string[];
    /**
     * <p>The AWS account IDs.
     *        This parameter can be used only when the <code>Attribute</code> parameter is <code>launchPermission</code>.</p>
     */
    UserIds?: string[];
    /**
     * <p>The value of the attribute being modified.
     *        This parameter can be used only when the <code>Attribute</code> parameter is <code>description</code> or <code>productCodes</code>.</p>
     */
    Value?: string;
}
export declare namespace ModifyImageAttributeRequest {
    const filterSensitiveLog: (obj: ModifyImageAttributeRequest) => any;
    const isa: (o: any) => o is ModifyImageAttributeRequest;
}
export interface ModifyInstanceAttributeRequest {
    __type?: "ModifyInstanceAttributeRequest";
    /**
     * <p>The name of the attribute.</p>
     */
    Attribute?: InstanceAttributeName | string;
    /**
     * <p>Modifies the <code>DeleteOnTermination</code> attribute for volumes that are currently
     *             attached. The volume must be owned by the caller. If no value is specified for
     *                 <code>DeleteOnTermination</code>, the default is <code>true</code> and the volume is
     *             deleted when the instance is terminated.</p>
     *         <p>To add instance store volumes to an Amazon EBS-backed instance, you must add them when
     *             you launch the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/block-device-mapping-concepts.html#Using_OverridingAMIBDM">Updating the Block Device Mapping when Launching an Instance</a> in the
     *                 <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    BlockDeviceMappings?: InstanceBlockDeviceMappingSpecification[];
    /**
     * <p>If the value is <code>true</code>, you can't terminate the instance using the Amazon
     *             EC2 console, CLI, or API; otherwise, you can. You cannot use this parameter for Spot
     *             Instances.</p>
     */
    DisableApiTermination?: AttributeBooleanValue;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Specifies whether the instance is optimized for Amazon EBS I/O. This optimization
     *             provides dedicated throughput to Amazon EBS and an optimized configuration stack to
     *             provide optimal EBS I/O performance. This optimization isn't available with all instance
     *             types. Additional usage charges apply when using an EBS Optimized instance.</p>
     */
    EbsOptimized?: AttributeBooleanValue;
    /**
     * <p>Set to <code>true</code> to enable enhanced networking with ENA for the
     *             instance.</p>
     *         <p>This option is supported only for HVM instances. Specifying this option with a PV
     *             instance can make it unreachable.</p>
     */
    EnaSupport?: AttributeBooleanValue;
    /**
     * <p>[EC2-VPC] Changes the security groups of the instance. You must specify at least one
     *             security group, even if it's just the default security group for the VPC. You must
     *             specify the security group ID, not the security group name.</p>
     */
    Groups?: string[];
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>Specifies whether an instance stops or terminates when you initiate shutdown from the
     *             instance (using the operating system command for system shutdown).</p>
     */
    InstanceInitiatedShutdownBehavior?: AttributeValue;
    /**
     * <p>Changes the instance type to the specified value. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance
     *                 Types</a>. If the instance type is not valid, the error returned is
     *                 <code>InvalidInstanceAttributeValue</code>.</p>
     */
    InstanceType?: AttributeValue;
    /**
     * <p>Changes the instance's kernel to the specified value. We recommend that you use
     *             PV-GRUB instead of kernels and RAM disks. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedKernels.html">PV-GRUB</a>.</p>
     */
    Kernel?: AttributeValue;
    /**
     * <p>Changes the instance's RAM disk to the specified value. We recommend that you use
     *             PV-GRUB instead of kernels and RAM disks. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedKernels.html">PV-GRUB</a>.</p>
     */
    Ramdisk?: AttributeValue;
    /**
     * <p>Specifies whether source/destination checking is enabled. A value of <code>true</code>
     *             means that checking is enabled, and <code>false</code> means that checking is disabled.
     *             This value must be <code>false</code> for a NAT instance to perform NAT.</p>
     */
    SourceDestCheck?: AttributeBooleanValue;
    /**
     * <p>Set to <code>simple</code> to enable enhanced networking with the Intel 82599 Virtual
     *             Function interface for the instance.</p>
     *         <p>There is no way to disable enhanced networking with the Intel 82599 Virtual Function
     *             interface at this time.</p>
     *         <p>This option is supported only for HVM instances. Specifying this option with a PV
     *             instance can make it unreachable.</p>
     */
    SriovNetSupport?: AttributeValue;
    /**
     * <p>Changes the instance's user data to the specified value. If you are using an AWS SDK
     *             or command line tool, base64-encoding is performed for you, and you can load the text
     *             from a file. Otherwise, you must provide base64-encoded text.</p>
     */
    UserData?: BlobAttributeValue;
    /**
     * <p>A new value for the attribute. Use only with the <code>kernel</code>,
     *                 <code>ramdisk</code>, <code>userData</code>, <code>disableApiTermination</code>, or
     *                 <code>instanceInitiatedShutdownBehavior</code> attribute.</p>
     */
    Value?: string;
}
export declare namespace ModifyInstanceAttributeRequest {
    const filterSensitiveLog: (obj: ModifyInstanceAttributeRequest) => any;
    const isa: (o: any) => o is ModifyInstanceAttributeRequest;
}
export interface ModifyInstanceCapacityReservationAttributesRequest {
    __type?: "ModifyInstanceCapacityReservationAttributesRequest";
    /**
     * <p>Information about the Capacity Reservation targeting option.</p>
     */
    CapacityReservationSpecification: CapacityReservationSpecification | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance to be modified.</p>
     */
    InstanceId: string | undefined;
}
export declare namespace ModifyInstanceCapacityReservationAttributesRequest {
    const filterSensitiveLog: (obj: ModifyInstanceCapacityReservationAttributesRequest) => any;
    const isa: (o: any) => o is ModifyInstanceCapacityReservationAttributesRequest;
}
export interface ModifyInstanceCapacityReservationAttributesResult {
    __type?: "ModifyInstanceCapacityReservationAttributesResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace ModifyInstanceCapacityReservationAttributesResult {
    const filterSensitiveLog: (obj: ModifyInstanceCapacityReservationAttributesResult) => any;
    const isa: (o: any) => o is ModifyInstanceCapacityReservationAttributesResult;
}
export interface ModifyInstanceCreditSpecificationRequest {
    __type?: "ModifyInstanceCreditSpecificationRequest";
    /**
     * <p>A unique, case-sensitive token that you provide to ensure idempotency of your
     *             modification request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring
     *                 Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Information about the credit option for CPU usage.</p>
     */
    InstanceCreditSpecifications: InstanceCreditSpecificationRequest[] | undefined;
}
export declare namespace ModifyInstanceCreditSpecificationRequest {
    const filterSensitiveLog: (obj: ModifyInstanceCreditSpecificationRequest) => any;
    const isa: (o: any) => o is ModifyInstanceCreditSpecificationRequest;
}
export interface ModifyInstanceCreditSpecificationResult {
    __type?: "ModifyInstanceCreditSpecificationResult";
    /**
     * <p>Information about the instances whose credit option for CPU usage was successfully
     *             modified.</p>
     */
    SuccessfulInstanceCreditSpecifications?: SuccessfulInstanceCreditSpecificationItem[];
    /**
     * <p>Information about the instances whose credit option for CPU usage was not
     *             modified.</p>
     */
    UnsuccessfulInstanceCreditSpecifications?: UnsuccessfulInstanceCreditSpecificationItem[];
}
export declare namespace ModifyInstanceCreditSpecificationResult {
    const filterSensitiveLog: (obj: ModifyInstanceCreditSpecificationResult) => any;
    const isa: (o: any) => o is ModifyInstanceCreditSpecificationResult;
}
export interface ModifyInstanceEventStartTimeRequest {
    __type?: "ModifyInstanceEventStartTimeRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the event whose date and time you are modifying.</p>
     */
    InstanceEventId: string | undefined;
    /**
     * <p>The ID of the instance with the scheduled event.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>The new date and time when the event will take place.</p>
     */
    NotBefore: Date | undefined;
}
export declare namespace ModifyInstanceEventStartTimeRequest {
    const filterSensitiveLog: (obj: ModifyInstanceEventStartTimeRequest) => any;
    const isa: (o: any) => o is ModifyInstanceEventStartTimeRequest;
}
export interface ModifyInstanceEventStartTimeResult {
    __type?: "ModifyInstanceEventStartTimeResult";
    /**
     * <p>Describes a scheduled event for an instance.</p>
     */
    Event?: InstanceStatusEvent;
}
export declare namespace ModifyInstanceEventStartTimeResult {
    const filterSensitiveLog: (obj: ModifyInstanceEventStartTimeResult) => any;
    const isa: (o: any) => o is ModifyInstanceEventStartTimeResult;
}
export interface ModifyInstanceMetadataOptionsRequest {
    __type?: "ModifyInstanceMetadataOptionsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>This parameter enables or disables the HTTP metadata endpoint on your instances. If
     *             the parameter is not specified, the existing state is maintained.</p>
     *         <note>
     *             <p>If you specify a value of <code>disabled</code>, you will not be able to access your
     *                 instance metadata.</p>
     *         </note>
     */
    HttpEndpoint?: InstanceMetadataEndpointState | string;
    /**
     * <p>The desired HTTP PUT response hop limit for instance metadata requests. The larger the
     *             number, the further instance metadata requests can travel. If no parameter is specified, the existing state is maintained.</p>
     *         <p>Possible values: Integers from 1 to 64</p>
     */
    HttpPutResponseHopLimit?: number;
    /**
     * <p>The state of token usage for your instance metadata requests. If the parameter is not
     *             specified in the request, the default state is <code>optional</code>.</p>
     *         <p>If the state is <code>optional</code>, you can choose to retrieve instance metadata
     *             with or without a signed token header on your request. If you retrieve the IAM role
     *             credentials without a token, the version 1.0 role credentials are returned. If you
     *             retrieve the IAM role credentials using a valid signed token, the version 2.0 role
     *             credentials are returned.</p>
     *         <p>If the state is <code>required</code>, you must send a signed token header with any
     *             instance metadata retrieval requests. In this state, retrieving the IAM role credential
     *             always returns the version 2.0 credentials; the version 1.0 credentials are not
     *             available.</p>
     */
    HttpTokens?: HttpTokensState | string;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
}
export declare namespace ModifyInstanceMetadataOptionsRequest {
    const filterSensitiveLog: (obj: ModifyInstanceMetadataOptionsRequest) => any;
    const isa: (o: any) => o is ModifyInstanceMetadataOptionsRequest;
}
export interface ModifyInstanceMetadataOptionsResult {
    __type?: "ModifyInstanceMetadataOptionsResult";
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The metadata options for the instance.</p>
     */
    InstanceMetadataOptions?: InstanceMetadataOptionsResponse;
}
export declare namespace ModifyInstanceMetadataOptionsResult {
    const filterSensitiveLog: (obj: ModifyInstanceMetadataOptionsResult) => any;
    const isa: (o: any) => o is ModifyInstanceMetadataOptionsResult;
}
export interface ModifyInstancePlacementRequest {
    __type?: "ModifyInstancePlacementRequest";
    /**
     * <p>The affinity setting for the instance.</p>
     */
    Affinity?: Affinity | string;
    /**
     * <p>The name of the placement group in which to place the instance. For spread placement
     *             groups, the instance must have a tenancy of <code>default</code>. For cluster and
     *             partition placement groups, the instance must have a tenancy of <code>default</code> or
     *                 <code>dedicated</code>.</p>
     *         <p>To remove an instance from a placement group, specify an empty string
     *             ("").</p>
     */
    GroupName?: string;
    /**
     * <p>The ID of the Dedicated Host with which to associate the instance.</p>
     */
    HostId?: string;
    /**
     * <p>The ARN of the host resource group in which to place the instance.</p>
     */
    HostResourceGroupArn?: string;
    /**
     * <p>The ID of the instance that you are modifying.</p>
     */
    InstanceId: string | undefined;
    /**
     * <p>Reserved for future use.</p>
     */
    PartitionNumber?: number;
    /**
     * <p>The tenancy for the instance.</p>
     */
    Tenancy?: HostTenancy | string;
}
export declare namespace ModifyInstancePlacementRequest {
    const filterSensitiveLog: (obj: ModifyInstancePlacementRequest) => any;
    const isa: (o: any) => o is ModifyInstancePlacementRequest;
}
export interface ModifyInstancePlacementResult {
    __type?: "ModifyInstancePlacementResult";
    /**
     * <p>Is <code>true</code> if the request succeeds, and an error otherwise.</p>
     */
    Return?: boolean;
}
export declare namespace ModifyInstancePlacementResult {
    const filterSensitiveLog: (obj: ModifyInstancePlacementResult) => any;
    const isa: (o: any) => o is ModifyInstancePlacementResult;
}
export interface ModifyLaunchTemplateRequest {
    __type?: "ModifyLaunchTemplateRequest";
    /**
     * <p>Unique, case-sensitive identifier you provide to ensure the idempotency of the
     *             request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring
     *                 Idempotency</a>.</p>
     *         <p>Constraint: Maximum 128 ASCII characters.</p>
     */
    ClientToken?: string;
    /**
     * <p>The version number of the launch template to set as the default version.</p>
     */
    DefaultVersion?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually
     *             making the request, and provides an error response. If you have the required
     *             permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is
     *                 <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the launch template. You must specify either the launch template ID or
     *             launch template name in the request.</p>
     */
    LaunchTemplateId?: string;
    /**
     * <p>The name of the launch template. You must specify either the launch template ID or
     *             launch template name in the request.</p>
     */
    LaunchTemplateName?: string;
}
export declare namespace ModifyLaunchTemplateRequest {
    const filterSensitiveLog: (obj: ModifyLaunchTemplateRequest) => any;
    const isa: (o: any) => o is ModifyLaunchTemplateRequest;
}
export interface ModifyLaunchTemplateResult {
    __type?: "ModifyLaunchTemplateResult";
    /**
     * <p>Information about the launch template.</p>
     */
    LaunchTemplate?: LaunchTemplate;
}
export declare namespace ModifyLaunchTemplateResult {
    const filterSensitiveLog: (obj: ModifyLaunchTemplateResult) => any;
    const isa: (o: any) => o is ModifyLaunchTemplateResult;
}
/**
 * <p>Contains the parameters for ModifyNetworkInterfaceAttribute.</p>
 */
export interface ModifyNetworkInterfaceAttributeRequest {
    __type?: "ModifyNetworkInterfaceAttributeRequest";
    /**
     * <p>Information about the interface attachment. If modifying the 'delete on termination' attribute, you must specify the ID of the interface attachment.</p>
     */
    Attachment?: NetworkInterfaceAttachmentChanges;
    /**
     * <p>A description for the network interface.</p>
     */
    Description?: AttributeValue;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Changes the security groups for the network interface. The new set of groups you specify replaces the current set. You must specify at least one group, even if it's just the default security group in the VPC. You must specify the ID of the security group, not the name.</p>
     */
    Groups?: string[];
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId: string | undefined;
    /**
     * <p>Indicates whether source/destination checking is enabled.
     *             A value of <code>true</code> means checking
     *             is enabled, and <code>false</code> means checking is disabled. This value
     *             must be <code>false</code> for a NAT instance to perform NAT. For more
     *             information, see <a href="https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_NAT_Instance.html">NAT
     *                 Instances</a> in the <i>Amazon Virtual Private Cloud User Guide</i>.</p>
     */
    SourceDestCheck?: AttributeBooleanValue;
}
export declare namespace ModifyNetworkInterfaceAttributeRequest {
    const filterSensitiveLog: (obj: ModifyNetworkInterfaceAttributeRequest) => any;
    const isa: (o: any) => o is ModifyNetworkInterfaceAttributeRequest;
}
/**
 * <p>Contains the parameters for ModifyReservedInstances.</p>
 */
export interface ModifyReservedInstancesRequest {
    __type?: "ModifyReservedInstancesRequest";
    /**
     * <p>A unique, case-sensitive token you provide to ensure idempotency of your modification request. For more information, see
     *    		<a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The IDs of the Reserved Instances to modify.</p>
     */
    ReservedInstancesIds: string[] | undefined;
    /**
     * <p>The configuration settings for the Reserved Instances to modify.</p>
     */
    TargetConfigurations: ReservedInstancesConfiguration[] | undefined;
}
export declare namespace ModifyReservedInstancesRequest {
    const filterSensitiveLog: (obj: ModifyReservedInstancesRequest) => any;
    const isa: (o: any) => o is ModifyReservedInstancesRequest;
}
/**
 * <p>Contains the output of ModifyReservedInstances.</p>
 */
export interface ModifyReservedInstancesResult {
    __type?: "ModifyReservedInstancesResult";
    /**
     * <p>The ID for the modification.</p>
     */
    ReservedInstancesModificationId?: string;
}
export declare namespace ModifyReservedInstancesResult {
    const filterSensitiveLog: (obj: ModifyReservedInstancesResult) => any;
    const isa: (o: any) => o is ModifyReservedInstancesResult;
}
export interface ModifySnapshotAttributeRequest {
    __type?: "ModifySnapshotAttributeRequest";
    /**
     * <p>The snapshot attribute to modify. Only volume creation permissions can be modified.</p>
     */
    Attribute?: SnapshotAttributeName | string;
    /**
     * <p>A JSON representation of the snapshot attribute modification.</p>
     */
    CreateVolumePermission?: CreateVolumePermissionModifications;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The group to modify for the snapshot.</p>
     */
    GroupNames?: string[];
    /**
     * <p>The type of operation to perform to the attribute.</p>
     */
    OperationType?: OperationType | string;
    /**
     * <p>The ID of the snapshot.</p>
     */
    SnapshotId: string | undefined;
    /**
     * <p>The account ID to modify for the snapshot.</p>
     */
    UserIds?: string[];
}
export declare namespace ModifySnapshotAttributeRequest {
    const filterSensitiveLog: (obj: ModifySnapshotAttributeRequest) => any;
    const isa: (o: any) => o is ModifySnapshotAttributeRequest;
}
/**
 * <p>Contains the parameters for ModifySpotFleetRequest.</p>
 */
export interface ModifySpotFleetRequestRequest {
    __type?: "ModifySpotFleetRequestRequest";
    /**
     * <p>Indicates whether running Spot Instances should be terminated if the target capacity of the Spot Fleet request is decreased below the current size of the Spot Fleet.</p>
     */
    ExcessCapacityTerminationPolicy?: ExcessCapacityTerminationPolicy | string;
    /**
     * <p>The number of On-Demand Instances in the fleet.</p>
     */
    OnDemandTargetCapacity?: number;
    /**
     * <p>The ID of the Spot Fleet request.</p>
     */
    SpotFleetRequestId: string | undefined;
    /**
     * <p>The size of the fleet.</p>
     */
    TargetCapacity?: number;
}
export declare namespace ModifySpotFleetRequestRequest {
    const filterSensitiveLog: (obj: ModifySpotFleetRequestRequest) => any;
    const isa: (o: any) => o is ModifySpotFleetRequestRequest;
}
/**
 * <p>Contains the output of ModifySpotFleetRequest.</p>
 */
export interface ModifySpotFleetRequestResponse {
    __type?: "ModifySpotFleetRequestResponse";
    /**
     * <p>Is <code>true</code> if the request succeeds, and an error otherwise.</p>
     */
    Return?: boolean;
}
export declare namespace ModifySpotFleetRequestResponse {
    const filterSensitiveLog: (obj: ModifySpotFleetRequestResponse) => any;
    const isa: (o: any) => o is ModifySpotFleetRequestResponse;
}
export interface ModifySubnetAttributeRequest {
    __type?: "ModifySubnetAttributeRequest";
    /**
     * <p>Specify <code>true</code> to indicate that network interfaces created in the
     *             specified subnet should be assigned an IPv6 address. This includes a network interface
     *             that's created when launching an instance into the subnet (the instance therefore
     *             receives an IPv6 address). </p>
     *         <p>If you enable the IPv6 addressing feature for your subnet, your network interface
     *             or instance only receives an IPv6 address if it's created using version
     *                 <code>2016-11-15</code> or later of the Amazon EC2 API.</p>
     */
    AssignIpv6AddressOnCreation?: AttributeBooleanValue;
    /**
     * <p>Specify <code>true</code> to indicate that ENIs attached to instances created in the
     *             specified subnet should be assigned a public IPv4 address.</p>
     */
    MapPublicIpOnLaunch?: AttributeBooleanValue;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId: string | undefined;
}
export declare namespace ModifySubnetAttributeRequest {
    const filterSensitiveLog: (obj: ModifySubnetAttributeRequest) => any;
    const isa: (o: any) => o is ModifySubnetAttributeRequest;
}
export interface ModifyTrafficMirrorFilterNetworkServicesRequest {
    __type?: "ModifyTrafficMirrorFilterNetworkServicesRequest";
    /**
     * <p>The network service, for example Amazon DNS, that you want to mirror.</p>
     */
    AddNetworkServices?: (TrafficMirrorNetworkService | string)[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The network service, for example Amazon DNS, that you no longer want to mirror.</p>
     */
    RemoveNetworkServices?: (TrafficMirrorNetworkService | string)[];
    /**
     * <p>The ID of the Traffic Mirror filter.</p>
     */
    TrafficMirrorFilterId: string | undefined;
}
export declare namespace ModifyTrafficMirrorFilterNetworkServicesRequest {
    const filterSensitiveLog: (obj: ModifyTrafficMirrorFilterNetworkServicesRequest) => any;
    const isa: (o: any) => o is ModifyTrafficMirrorFilterNetworkServicesRequest;
}
export interface ModifyTrafficMirrorFilterNetworkServicesResult {
    __type?: "ModifyTrafficMirrorFilterNetworkServicesResult";
    /**
     * <p>The Traffic Mirror filter that the network service is associated with.</p>
     */
    TrafficMirrorFilter?: TrafficMirrorFilter;
}
export declare namespace ModifyTrafficMirrorFilterNetworkServicesResult {
    const filterSensitiveLog: (obj: ModifyTrafficMirrorFilterNetworkServicesResult) => any;
    const isa: (o: any) => o is ModifyTrafficMirrorFilterNetworkServicesResult;
}
export interface ModifyTrafficMirrorFilterRuleRequest {
    __type?: "ModifyTrafficMirrorFilterRuleRequest";
    /**
     * <p>The description to assign to the Traffic Mirror rule.</p>
     */
    Description?: string;
    /**
     * <p>The destination CIDR block to assign to the Traffic Mirror rule.</p>
     */
    DestinationCidrBlock?: string;
    /**
     * <p>The destination ports that are associated with the Traffic Mirror rule.</p>
     */
    DestinationPortRange?: TrafficMirrorPortRangeRequest;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The protocol, for example TCP, to assign to the Traffic Mirror rule.</p>
     */
    Protocol?: number;
    /**
     * <p>The properties that you want to remove from the Traffic Mirror filter rule.</p>
     *          <p>When you remove a property from a Traffic Mirror filter rule, the property is set to the default.</p>
     */
    RemoveFields?: (TrafficMirrorFilterRuleField | string)[];
    /**
     * <p>The action to assign to the rule.</p>
     */
    RuleAction?: TrafficMirrorRuleAction | string;
    /**
     * <p>The number of the Traffic Mirror rule. This number must be unique for each Traffic Mirror rule in a given
     *          direction. The rules are processed in ascending order by rule number.</p>
     */
    RuleNumber?: number;
    /**
     * <p>The source CIDR block to assign to the Traffic Mirror rule.</p>
     */
    SourceCidrBlock?: string;
    /**
     * <p>The port range to assign to the Traffic Mirror rule.</p>
     */
    SourcePortRange?: TrafficMirrorPortRangeRequest;
    /**
     * <p>The type of traffic (<code>ingress</code> | <code>egress</code>) to assign to the rule.</p>
     */
    TrafficDirection?: TrafficDirection | string;
    /**
     * <p>The ID of the Traffic Mirror rule.</p>
     */
    TrafficMirrorFilterRuleId: string | undefined;
}
export declare namespace ModifyTrafficMirrorFilterRuleRequest {
    const filterSensitiveLog: (obj: ModifyTrafficMirrorFilterRuleRequest) => any;
    const isa: (o: any) => o is ModifyTrafficMirrorFilterRuleRequest;
}
export interface ModifyTrafficMirrorFilterRuleResult {
    __type?: "ModifyTrafficMirrorFilterRuleResult";
    /**
     * <p>Modifies a Traffic Mirror rule.</p>
     */
    TrafficMirrorFilterRule?: TrafficMirrorFilterRule;
}
export declare namespace ModifyTrafficMirrorFilterRuleResult {
    const filterSensitiveLog: (obj: ModifyTrafficMirrorFilterRuleResult) => any;
    const isa: (o: any) => o is ModifyTrafficMirrorFilterRuleResult;
}
export interface ModifyTrafficMirrorSessionRequest {
    __type?: "ModifyTrafficMirrorSessionRequest";
    /**
     * <p>The description to assign to the Traffic Mirror session.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The number of bytes in each packet to mirror. These are bytes after the VXLAN header. To mirror a subset, set this to the length (in bytes) to mirror. For example, if you set this value to 100, then the first 100 bytes that meet the filter criteria are copied to the target. Do not specify this parameter when you want to mirror the entire packet.</p>
     */
    PacketLength?: number;
    /**
     * <p>The properties that you want to remove from the Traffic Mirror session.</p>
     *          <p>When you remove a property from a Traffic Mirror session, the property is set to the default.</p>
     */
    RemoveFields?: (TrafficMirrorSessionField | string)[];
    /**
     * <p>The session number determines the order in which sessions are evaluated when an interface is used by multiple sessions. The first session with a matching filter is the one that mirrors the packets.</p>
     *          <p>Valid values are 1-32766.</p>
     */
    SessionNumber?: number;
    /**
     * <p>The ID of the Traffic Mirror filter.</p>
     */
    TrafficMirrorFilterId?: string;
    /**
     * <p>The ID of the Traffic Mirror session.</p>
     */
    TrafficMirrorSessionId: string | undefined;
    /**
     * <p>The Traffic Mirror target. The target must be in the same VPC as the source, or have a VPC peering connection with the source.</p>
     */
    TrafficMirrorTargetId?: string;
    /**
     * <p>The virtual network ID of the Traffic Mirror session.</p>
     */
    VirtualNetworkId?: number;
}
export declare namespace ModifyTrafficMirrorSessionRequest {
    const filterSensitiveLog: (obj: ModifyTrafficMirrorSessionRequest) => any;
    const isa: (o: any) => o is ModifyTrafficMirrorSessionRequest;
}
export interface ModifyTrafficMirrorSessionResult {
    __type?: "ModifyTrafficMirrorSessionResult";
    /**
     * <p>Information about the Traffic Mirror session.</p>
     */
    TrafficMirrorSession?: TrafficMirrorSession;
}
export declare namespace ModifyTrafficMirrorSessionResult {
    const filterSensitiveLog: (obj: ModifyTrafficMirrorSessionResult) => any;
    const isa: (o: any) => o is ModifyTrafficMirrorSessionResult;
}
export interface ModifyTransitGatewayVpcAttachmentRequest {
    __type?: "ModifyTransitGatewayVpcAttachmentRequest";
    /**
     * <p>The IDs of one or more subnets to add. You can specify at most one subnet per Availability Zone.</p>
     */
    AddSubnetIds?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The new VPC attachment options.</p>
     *          <note>
     *             <p>You cannot modify the IPv6 options.</p>
     *          </note>
     */
    Options?: ModifyTransitGatewayVpcAttachmentRequestOptions;
    /**
     * <p>The IDs of one or more subnets to remove.</p>
     */
    RemoveSubnetIds?: string[];
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
}
export declare namespace ModifyTransitGatewayVpcAttachmentRequest {
    const filterSensitiveLog: (obj: ModifyTransitGatewayVpcAttachmentRequest) => any;
    const isa: (o: any) => o is ModifyTransitGatewayVpcAttachmentRequest;
}
/**
 * <p>Describes the options for a VPC attachment.</p>
 */
export interface ModifyTransitGatewayVpcAttachmentRequestOptions {
    __type?: "ModifyTransitGatewayVpcAttachmentRequestOptions";
    /**
     * <p>Enable or disable DNS support. The default is <code>enable</code>.</p>
     */
    DnsSupport?: DnsSupportValue | string;
    /**
     * <p>Enable or disable IPv6 support. The default is <code>enable</code>.</p>
     */
    Ipv6Support?: Ipv6SupportValue | string;
}
export declare namespace ModifyTransitGatewayVpcAttachmentRequestOptions {
    const filterSensitiveLog: (obj: ModifyTransitGatewayVpcAttachmentRequestOptions) => any;
    const isa: (o: any) => o is ModifyTransitGatewayVpcAttachmentRequestOptions;
}
export interface ModifyTransitGatewayVpcAttachmentResult {
    __type?: "ModifyTransitGatewayVpcAttachmentResult";
    /**
     * <p>Information about the modified attachment.</p>
     */
    TransitGatewayVpcAttachment?: TransitGatewayVpcAttachment;
}
export declare namespace ModifyTransitGatewayVpcAttachmentResult {
    const filterSensitiveLog: (obj: ModifyTransitGatewayVpcAttachmentResult) => any;
    const isa: (o: any) => o is ModifyTransitGatewayVpcAttachmentResult;
}
export interface ModifyVolumeAttributeRequest {
    __type?: "ModifyVolumeAttributeRequest";
    /**
     * <p>Indicates whether the volume should be auto-enabled for I/O operations.</p>
     */
    AutoEnableIO?: AttributeBooleanValue;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the volume.</p>
     */
    VolumeId: string | undefined;
}
export declare namespace ModifyVolumeAttributeRequest {
    const filterSensitiveLog: (obj: ModifyVolumeAttributeRequest) => any;
    const isa: (o: any) => o is ModifyVolumeAttributeRequest;
}
export interface ModifyVolumeRequest {
    __type?: "ModifyVolumeRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The target IOPS rate of the volume.</p>
     *          <p>This is only valid for Provisioned IOPS SSD (<code>io1</code>) volumes. For more
     *       information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html#EBSVolumeTypes_piops">Provisioned IOPS SSD (io1)
     *         Volumes</a>.</p>
     *          <p>Default: If no IOPS value is specified, the existing value is retained.</p>
     */
    Iops?: number;
    /**
     * <p>The target size of the volume, in GiB. The target volume size must be greater than or
     *       equal to than the existing size of the volume. For information about available EBS volume
     *       sizes, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html">Amazon EBS Volume
     *         Types</a>.</p>
     *          <p>Default: If no size is specified, the existing size is retained.</p>
     */
    Size?: number;
    /**
     * <p>The ID of the volume.</p>
     */
    VolumeId: string | undefined;
    /**
     * <p>The target EBS volume type of the volume.</p>
     *          <p>Default: If no type is specified, the existing type is retained.</p>
     */
    VolumeType?: VolumeType | string;
}
export declare namespace ModifyVolumeRequest {
    const filterSensitiveLog: (obj: ModifyVolumeRequest) => any;
    const isa: (o: any) => o is ModifyVolumeRequest;
}
export interface ModifyVolumeResult {
    __type?: "ModifyVolumeResult";
    /**
     * <p>Information about the volume modification.</p>
     */
    VolumeModification?: VolumeModification;
}
export declare namespace ModifyVolumeResult {
    const filterSensitiveLog: (obj: ModifyVolumeResult) => any;
    const isa: (o: any) => o is ModifyVolumeResult;
}
export interface ModifyVpcAttributeRequest {
    __type?: "ModifyVpcAttributeRequest";
    /**
     * <p>Indicates whether the instances launched in the VPC get DNS hostnames. If enabled, instances in the VPC get DNS hostnames; otherwise, they do not.</p>
     *          <p>You cannot modify the DNS resolution and DNS hostnames attributes in the same request. Use separate requests for each attribute. You can only enable DNS hostnames if you've enabled DNS support.</p>
     */
    EnableDnsHostnames?: AttributeBooleanValue;
    /**
     * <p>Indicates whether the DNS resolution is supported for the VPC. If enabled, queries to
     * 			the Amazon provided DNS server at the 169.254.169.253 IP address, or the reserved IP
     * 			address at the base of the VPC network range "plus two" succeed. If disabled, the Amazon
     * 			provided DNS service in the VPC that resolves public DNS hostnames to IP addresses is
     * 			not enabled.</p>
     *          <p>You cannot modify the DNS resolution and DNS hostnames attributes in the same request. Use separate requests for each attribute.</p>
     */
    EnableDnsSupport?: AttributeBooleanValue;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace ModifyVpcAttributeRequest {
    const filterSensitiveLog: (obj: ModifyVpcAttributeRequest) => any;
    const isa: (o: any) => o is ModifyVpcAttributeRequest;
}
export interface ModifyVpcEndpointConnectionNotificationRequest {
    __type?: "ModifyVpcEndpointConnectionNotificationRequest";
    /**
     * <p>One or more events for the endpoint. Valid values are <code>Accept</code>,
     *                 <code>Connect</code>, <code>Delete</code>, and <code>Reject</code>.</p>
     */
    ConnectionEvents?: string[];
    /**
     * <p>The ARN for the SNS topic for the notification.</p>
     */
    ConnectionNotificationArn?: string;
    /**
     * <p>The ID of the notification.</p>
     */
    ConnectionNotificationId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace ModifyVpcEndpointConnectionNotificationRequest {
    const filterSensitiveLog: (obj: ModifyVpcEndpointConnectionNotificationRequest) => any;
    const isa: (o: any) => o is ModifyVpcEndpointConnectionNotificationRequest;
}
export interface ModifyVpcEndpointConnectionNotificationResult {
    __type?: "ModifyVpcEndpointConnectionNotificationResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    ReturnValue?: boolean;
}
export declare namespace ModifyVpcEndpointConnectionNotificationResult {
    const filterSensitiveLog: (obj: ModifyVpcEndpointConnectionNotificationResult) => any;
    const isa: (o: any) => o is ModifyVpcEndpointConnectionNotificationResult;
}
/**
 * <p>Contains the parameters for ModifyVpcEndpoint.</p>
 */
export interface ModifyVpcEndpointRequest {
    __type?: "ModifyVpcEndpointRequest";
    /**
     * <p>(Gateway endpoint) One or more route tables IDs to associate with the endpoint.</p>
     */
    AddRouteTableIds?: string[];
    /**
     * <p>(Interface endpoint) One or more security group IDs to associate with the network interface.</p>
     */
    AddSecurityGroupIds?: string[];
    /**
     * <p>(Interface endpoint) One or more subnet IDs in which to serve the endpoint.</p>
     */
    AddSubnetIds?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>A policy to attach to the endpoint that controls access to the service. The policy must
     *             be in valid JSON format.</p>
     */
    PolicyDocument?: string;
    /**
     * <p>(Interface endpoint) Indicates whether a private hosted zone is associated with the
     *             VPC.</p>
     */
    PrivateDnsEnabled?: boolean;
    /**
     * <p>(Gateway endpoint) One or more route table IDs to disassociate from the endpoint.</p>
     */
    RemoveRouteTableIds?: string[];
    /**
     * <p>(Interface endpoint) One or more security group IDs to disassociate from the network interface.</p>
     */
    RemoveSecurityGroupIds?: string[];
    /**
     * <p>(Interface endpoint) One or more subnets IDs in which to remove the endpoint.</p>
     */
    RemoveSubnetIds?: string[];
    /**
     * <p>(Gateway endpoint) Specify <code>true</code> to reset the policy document to the
     *             default policy. The default policy allows full access to the service.</p>
     */
    ResetPolicy?: boolean;
    /**
     * <p>The ID of the endpoint.</p>
     */
    VpcEndpointId: string | undefined;
}
export declare namespace ModifyVpcEndpointRequest {
    const filterSensitiveLog: (obj: ModifyVpcEndpointRequest) => any;
    const isa: (o: any) => o is ModifyVpcEndpointRequest;
}
export interface ModifyVpcEndpointResult {
    __type?: "ModifyVpcEndpointResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace ModifyVpcEndpointResult {
    const filterSensitiveLog: (obj: ModifyVpcEndpointResult) => any;
    const isa: (o: any) => o is ModifyVpcEndpointResult;
}
export interface ModifyVpcEndpointServiceConfigurationRequest {
    __type?: "ModifyVpcEndpointServiceConfigurationRequest";
    /**
     * <p>Indicates whether requests to create an endpoint to your service must be accepted.</p>
     */
    AcceptanceRequired?: boolean;
    /**
     * <p>The Amazon Resource Names (ARNs) of Network Load Balancers to add to your service
     *             configuration.</p>
     */
    AddNetworkLoadBalancerArns?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The private DNS name to assign to the endpoint service.</p>
     */
    PrivateDnsName?: string;
    /**
     * <p>The Amazon Resource Names (ARNs) of Network Load Balancers to remove from your service
     *             configuration.</p>
     */
    RemoveNetworkLoadBalancerArns?: string[];
    /**
     * <p>Removes the private DNS name of the endpoint service.</p>
     */
    RemovePrivateDnsName?: boolean;
    /**
     * <p>The ID of the service.</p>
     */
    ServiceId: string | undefined;
}
export declare namespace ModifyVpcEndpointServiceConfigurationRequest {
    const filterSensitiveLog: (obj: ModifyVpcEndpointServiceConfigurationRequest) => any;
    const isa: (o: any) => o is ModifyVpcEndpointServiceConfigurationRequest;
}
export interface ModifyVpcEndpointServiceConfigurationResult {
    __type?: "ModifyVpcEndpointServiceConfigurationResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace ModifyVpcEndpointServiceConfigurationResult {
    const filterSensitiveLog: (obj: ModifyVpcEndpointServiceConfigurationResult) => any;
    const isa: (o: any) => o is ModifyVpcEndpointServiceConfigurationResult;
}
export interface ModifyVpcEndpointServicePermissionsRequest {
    __type?: "ModifyVpcEndpointServicePermissionsRequest";
    /**
     * <p>The Amazon Resource Names (ARN) of one or more principals.
     * 	        Permissions are granted to the principals in this list.
     * 	        To grant permissions to all principals, specify an asterisk (*).</p>
     */
    AddAllowedPrincipals?: string[];
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The Amazon Resource Names (ARN) of one or more principals.
     * 	        Permissions are revoked for principals in this list.</p>
     */
    RemoveAllowedPrincipals?: string[];
    /**
     * <p>The ID of the service.</p>
     */
    ServiceId: string | undefined;
}
export declare namespace ModifyVpcEndpointServicePermissionsRequest {
    const filterSensitiveLog: (obj: ModifyVpcEndpointServicePermissionsRequest) => any;
    const isa: (o: any) => o is ModifyVpcEndpointServicePermissionsRequest;
}
export interface ModifyVpcEndpointServicePermissionsResult {
    __type?: "ModifyVpcEndpointServicePermissionsResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    ReturnValue?: boolean;
}
export declare namespace ModifyVpcEndpointServicePermissionsResult {
    const filterSensitiveLog: (obj: ModifyVpcEndpointServicePermissionsResult) => any;
    const isa: (o: any) => o is ModifyVpcEndpointServicePermissionsResult;
}
export interface ModifyVpcPeeringConnectionOptionsRequest {
    __type?: "ModifyVpcPeeringConnectionOptionsRequest";
    /**
     * <p>The VPC peering connection options for the accepter VPC.</p>
     */
    AccepterPeeringConnectionOptions?: PeeringConnectionOptionsRequest;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The VPC peering connection options for the requester VPC.</p>
     */
    RequesterPeeringConnectionOptions?: PeeringConnectionOptionsRequest;
    /**
     * <p>The ID of the VPC peering connection.</p>
     */
    VpcPeeringConnectionId: string | undefined;
}
export declare namespace ModifyVpcPeeringConnectionOptionsRequest {
    const filterSensitiveLog: (obj: ModifyVpcPeeringConnectionOptionsRequest) => any;
    const isa: (o: any) => o is ModifyVpcPeeringConnectionOptionsRequest;
}
export interface ModifyVpcPeeringConnectionOptionsResult {
    __type?: "ModifyVpcPeeringConnectionOptionsResult";
    /**
     * <p>Information about the VPC peering connection options for the accepter VPC.</p>
     */
    AccepterPeeringConnectionOptions?: PeeringConnectionOptions;
    /**
     * <p>Information about the VPC peering connection options for the requester VPC.</p>
     */
    RequesterPeeringConnectionOptions?: PeeringConnectionOptions;
}
export declare namespace ModifyVpcPeeringConnectionOptionsResult {
    const filterSensitiveLog: (obj: ModifyVpcPeeringConnectionOptionsResult) => any;
    const isa: (o: any) => o is ModifyVpcPeeringConnectionOptionsResult;
}
export interface ModifyVpcTenancyRequest {
    __type?: "ModifyVpcTenancyRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The instance tenancy attribute for the VPC. </p>
     */
    InstanceTenancy: VpcTenancy | string | undefined;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId: string | undefined;
}
export declare namespace ModifyVpcTenancyRequest {
    const filterSensitiveLog: (obj: ModifyVpcTenancyRequest) => any;
    const isa: (o: any) => o is ModifyVpcTenancyRequest;
}
export interface ModifyVpcTenancyResult {
    __type?: "ModifyVpcTenancyResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, returns an
     *             error.</p>
     */
    ReturnValue?: boolean;
}
export declare namespace ModifyVpcTenancyResult {
    const filterSensitiveLog: (obj: ModifyVpcTenancyResult) => any;
    const isa: (o: any) => o is ModifyVpcTenancyResult;
}
export interface ModifyVpnConnectionRequest {
    __type?: "ModifyVpnConnectionRequest";
    /**
     * <p>The ID of the customer gateway at your end of the VPN connection.</p>
     */
    CustomerGatewayId?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId?: string;
    /**
     * <p>The ID of the VPN connection.</p>
     */
    VpnConnectionId: string | undefined;
    /**
     * <p>The ID of the virtual private gateway at the AWS side of the VPN connection.</p>
     */
    VpnGatewayId?: string;
}
export declare namespace ModifyVpnConnectionRequest {
    const filterSensitiveLog: (obj: ModifyVpnConnectionRequest) => any;
    const isa: (o: any) => o is ModifyVpnConnectionRequest;
}
export interface ModifyVpnConnectionResult {
    __type?: "ModifyVpnConnectionResult";
    /**
     * <p>Describes a VPN connection.</p>
     */
    VpnConnection?: VpnConnection;
}
export declare namespace ModifyVpnConnectionResult {
    const filterSensitiveLog: (obj: ModifyVpnConnectionResult) => any;
    const isa: (o: any) => o is ModifyVpnConnectionResult;
}
export interface ModifyVpnTunnelCertificateRequest {
    __type?: "ModifyVpnTunnelCertificateRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the AWS Site-to-Site VPN connection.</p>
     */
    VpnConnectionId: string | undefined;
    /**
     * <p>The external IP address of the VPN tunnel.</p>
     */
    VpnTunnelOutsideIpAddress: string | undefined;
}
export declare namespace ModifyVpnTunnelCertificateRequest {
    const filterSensitiveLog: (obj: ModifyVpnTunnelCertificateRequest) => any;
    const isa: (o: any) => o is ModifyVpnTunnelCertificateRequest;
}
export interface ModifyVpnTunnelCertificateResult {
    __type?: "ModifyVpnTunnelCertificateResult";
    /**
     * <p>Describes a VPN connection.</p>
     */
    VpnConnection?: VpnConnection;
}
export declare namespace ModifyVpnTunnelCertificateResult {
    const filterSensitiveLog: (obj: ModifyVpnTunnelCertificateResult) => any;
    const isa: (o: any) => o is ModifyVpnTunnelCertificateResult;
}
export interface ModifyVpnTunnelOptionsRequest {
    __type?: "ModifyVpnTunnelOptionsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The tunnel options to modify.</p>
     */
    TunnelOptions: ModifyVpnTunnelOptionsSpecification | undefined;
    /**
     * <p>The ID of the AWS Site-to-Site VPN connection.</p>
     */
    VpnConnectionId: string | undefined;
    /**
     * <p>The external IP address of the VPN tunnel.</p>
     */
    VpnTunnelOutsideIpAddress: string | undefined;
}
export declare namespace ModifyVpnTunnelOptionsRequest {
    const filterSensitiveLog: (obj: ModifyVpnTunnelOptionsRequest) => any;
    const isa: (o: any) => o is ModifyVpnTunnelOptionsRequest;
}
export interface ModifyVpnTunnelOptionsResult {
    __type?: "ModifyVpnTunnelOptionsResult";
    /**
     * <p>Describes a VPN connection.</p>
     */
    VpnConnection?: VpnConnection;
}
export declare namespace ModifyVpnTunnelOptionsResult {
    const filterSensitiveLog: (obj: ModifyVpnTunnelOptionsResult) => any;
    const isa: (o: any) => o is ModifyVpnTunnelOptionsResult;
}
/**
 * <p>The AWS Site-to-Site VPN tunnel options to modify.</p>
 */
export interface ModifyVpnTunnelOptionsSpecification {
    __type?: "ModifyVpnTunnelOptionsSpecification";
    /**
     * <p>The number of seconds after which a DPD timeout occurs.</p>
     *         <p>Constraints: A value between 0 and 30.</p>
     *         <p>Default: <code>30</code>
     *          </p>
     */
    DPDTimeoutSeconds?: number;
    /**
     * <p>The IKE versions that are permitted for the VPN tunnel.</p>
     *         <p>Valid values: <code>ikev1</code> | <code>ikev2</code>
     *          </p>
     */
    IKEVersions?: IKEVersionsRequestListValue[];
    /**
     * <p>One or more Diffie-Hellman group numbers that are permitted for the VPN tunnel for phase 1 IKE negotiations.</p>
     *         <p>Valid values: <code>2</code> | <code>14</code> | <code>15</code> | <code>16</code> | <code>17</code> | <code>18</code> | <code>22</code> | <code>23</code> | <code>24</code>
     *          </p>
     */
    Phase1DHGroupNumbers?: Phase1DHGroupNumbersRequestListValue[];
    /**
     * <p>One or more encryption algorithms that are permitted for the VPN tunnel for phase 1 IKE negotiations.</p>
     *         <p>Valid values: <code>AES128</code> | <code>AES256</code>
     *          </p>
     */
    Phase1EncryptionAlgorithms?: Phase1EncryptionAlgorithmsRequestListValue[];
    /**
     * <p>One or more integrity algorithms that are permitted for the VPN tunnel for phase 1 IKE negotiations.</p>
     *         <p>Valid values: <code>SHA1</code> | <code>SHA2-256</code>
     *          </p>
     */
    Phase1IntegrityAlgorithms?: Phase1IntegrityAlgorithmsRequestListValue[];
    /**
     * <p>The lifetime for phase 1 of the IKE negotiation, in seconds.</p>
     *         <p>Constraints: A value between 900 and 28,800.</p>
     *         <p>Default: <code>28800</code>
     *          </p>
     */
    Phase1LifetimeSeconds?: number;
    /**
     * <p>One or more Diffie-Hellman group numbers that are permitted for the VPN tunnel for phase 2 IKE negotiations.</p>
     *         <p>Valid values: <code>2</code> | <code>5</code> | <code>14</code> | <code>15</code> | <code>16</code> | <code>17</code> | <code>18</code> | <code>22</code> | <code>23</code> | <code>24</code>
     *          </p>
     */
    Phase2DHGroupNumbers?: Phase2DHGroupNumbersRequestListValue[];
    /**
     * <p>One or more encryption algorithms that are permitted for the VPN tunnel for phase 2 IKE negotiations.</p>
     *         <p>Valid values: <code>AES128</code> | <code>AES256</code>
     *          </p>
     */
    Phase2EncryptionAlgorithms?: Phase2EncryptionAlgorithmsRequestListValue[];
    /**
     * <p>One or more integrity algorithms that are permitted for the VPN tunnel for phase 2 IKE negotiations.</p>
     *         <p>Valid values: <code>SHA1</code> | <code>SHA2-256</code>
     *          </p>
     */
    Phase2IntegrityAlgorithms?: Phase2IntegrityAlgorithmsRequestListValue[];
    /**
     * <p>The lifetime for phase 2 of the IKE negotiation, in seconds.</p>
     *         <p>Constraints: A value between 900 and 3,600. The value must be less than the value for <code>Phase1LifetimeSeconds</code>.</p>
     *         <p>Default: <code>3600</code>
     *          </p>
     */
    Phase2LifetimeSeconds?: number;
    /**
     * <p>The pre-shared key (PSK) to establish initial authentication between the virtual
     *             private gateway and the customer gateway.</p>
     *         <p>Constraints: Allowed characters are alphanumeric characters, periods (.), and
     *             underscores (_). Must be between 8 and 64 characters in length and cannot start with
     *             zero (0).</p>
     */
    PreSharedKey?: string;
    /**
     * <p>The percentage of the rekey window (determined by <code>RekeyMarginTimeSeconds</code>) during which the rekey time is randomly selected.</p>
     *         <p>Constraints: A value between 0 and 100.</p>
     *         <p>Default: <code>100</code>
     *          </p>
     */
    RekeyFuzzPercentage?: number;
    /**
     * <p>The margin time, in seconds, before the phase 2 lifetime expires, during which the AWS side of the VPN connection performs an IKE rekey. The exact time of the rekey is randomly selected based on the value for <code>RekeyFuzzPercentage</code>.</p>
     *         <p>Constraints: A value between 60 and half of <code>Phase2LifetimeSeconds</code>.</p>
     *         <p>Default: <code>540</code>
     *          </p>
     */
    RekeyMarginTimeSeconds?: number;
    /**
     * <p>The number of packets in an IKE replay window.</p>
     *         <p>Constraints: A value between 64 and 2048.</p>
     *         <p>Default: <code>1024</code>
     *          </p>
     */
    ReplayWindowSize?: number;
    /**
     * <p>The range of inside IP addresses for the tunnel. Any specified CIDR blocks must be unique
     *             across all VPN connections that use the same virtual private gateway. </p>
     *         <p>Constraints: A size /30 CIDR block from the <code>169.254.0.0/16</code> range. The
     *             following CIDR blocks are reserved and cannot be used:</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>169.254.0.0/30</code>
     *                </p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>169.254.1.0/30</code>
     *                </p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>169.254.2.0/30</code>
     *                </p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>169.254.3.0/30</code>
     *                </p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>169.254.4.0/30</code>
     *                </p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>169.254.5.0/30</code>
     *                </p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>169.254.169.252/30</code>
     *                </p>
     *             </li>
     *          </ul>
     */
    TunnelInsideCidr?: string;
}
export declare namespace ModifyVpnTunnelOptionsSpecification {
    const filterSensitiveLog: (obj: ModifyVpnTunnelOptionsSpecification) => any;
    const isa: (o: any) => o is ModifyVpnTunnelOptionsSpecification;
}
/**
 * <p>Describes the monitoring of an instance.</p>
 */
export interface Monitoring {
    __type?: "Monitoring";
    /**
     * <p>Indicates whether detailed monitoring is enabled. Otherwise, basic monitoring is
     *             enabled.</p>
     */
    State?: MonitoringState | string;
}
export declare namespace Monitoring {
    const filterSensitiveLog: (obj: Monitoring) => any;
    const isa: (o: any) => o is Monitoring;
}
export declare type MonitoringState = "disabled" | "disabling" | "enabled" | "pending";
export interface MonitorInstancesRequest {
    __type?: "MonitorInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the instances.</p>
     */
    InstanceIds: string[] | undefined;
}
export declare namespace MonitorInstancesRequest {
    const filterSensitiveLog: (obj: MonitorInstancesRequest) => any;
    const isa: (o: any) => o is MonitorInstancesRequest;
}
export interface MonitorInstancesResult {
    __type?: "MonitorInstancesResult";
    /**
     * <p>The monitoring information.</p>
     */
    InstanceMonitorings?: InstanceMonitoring[];
}
export declare namespace MonitorInstancesResult {
    const filterSensitiveLog: (obj: MonitorInstancesResult) => any;
    const isa: (o: any) => o is MonitorInstancesResult;
}
export interface MoveAddressToVpcRequest {
    __type?: "MoveAddressToVpcRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The Elastic IP address.</p>
     */
    PublicIp: string | undefined;
}
export declare namespace MoveAddressToVpcRequest {
    const filterSensitiveLog: (obj: MoveAddressToVpcRequest) => any;
    const isa: (o: any) => o is MoveAddressToVpcRequest;
}
export interface MoveAddressToVpcResult {
    __type?: "MoveAddressToVpcResult";
    /**
     * <p>The allocation ID for the Elastic IP address.</p>
     */
    AllocationId?: string;
    /**
     * <p>The status of the move of the IP address.</p>
     */
    Status?: Status | string;
}
export declare namespace MoveAddressToVpcResult {
    const filterSensitiveLog: (obj: MoveAddressToVpcResult) => any;
    const isa: (o: any) => o is MoveAddressToVpcResult;
}
export declare type MoveStatus = "movingToVpc" | "restoringToClassic";
/**
 * <p>Describes the status of a moving Elastic IP address.</p>
 */
export interface MovingAddressStatus {
    __type?: "MovingAddressStatus";
    /**
     * <p>The status of the Elastic IP address that's being moved to the EC2-VPC platform, or restored to the EC2-Classic platform.</p>
     */
    MoveStatus?: MoveStatus | string;
    /**
     * <p>The Elastic IP address.</p>
     */
    PublicIp?: string;
}
export declare namespace MovingAddressStatus {
    const filterSensitiveLog: (obj: MovingAddressStatus) => any;
    const isa: (o: any) => o is MovingAddressStatus;
}
export declare type MulticastSupportValue = "disable" | "enable";
/**
 * <p>Describes a NAT gateway.</p>
 */
export interface NatGateway {
    __type?: "NatGateway";
    /**
     * <p>The date and time the NAT gateway was created.</p>
     */
    CreateTime?: Date;
    /**
     * <p>The date and time the NAT gateway was deleted, if applicable.</p>
     */
    DeleteTime?: Date;
    /**
     * <p>If the NAT gateway could not be created, specifies the error code for the failure.
     *         (<code>InsufficientFreeAddressesInSubnet</code> | <code>Gateway.NotAttached</code> |
     *          <code>InvalidAllocationID.NotFound</code> | <code>Resource.AlreadyAssociated</code> |
     *          <code>InternalError</code> | <code>InvalidSubnetID.NotFound</code>)</p>
     */
    FailureCode?: string;
    /**
     * <p>If the NAT gateway could not be created, specifies the error message for the failure, that corresponds to the error code.</p>
     *          <ul>
     *             <li>
     *                <p>For InsufficientFreeAddressesInSubnet: "Subnet has insufficient free addresses to create this NAT gateway"</p>
     *             </li>
     *             <li>
     *                <p>For Gateway.NotAttached: "Network vpc-xxxxxxxx has no Internet gateway attached"</p>
     *             </li>
     *             <li>
     *                <p>For InvalidAllocationID.NotFound: "Elastic IP address eipalloc-xxxxxxxx could not be associated with this NAT gateway"</p>
     *             </li>
     *             <li>
     *                <p>For Resource.AlreadyAssociated: "Elastic IP address eipalloc-xxxxxxxx is already associated"</p>
     *             </li>
     *             <li>
     *                <p>For InternalError: "Network interface eni-xxxxxxxx, created and used internally by this NAT gateway is in an invalid state. Please try again."</p>
     *             </li>
     *             <li>
     *                <p>For InvalidSubnetID.NotFound: "The specified subnet subnet-xxxxxxxx does not exist or could not be found."</p>
     *             </li>
     *          </ul>
     */
    FailureMessage?: string;
    /**
     * <p>Information about the IP addresses and network interface associated with the NAT gateway.</p>
     */
    NatGatewayAddresses?: NatGatewayAddress[];
    /**
     * <p>The ID of the NAT gateway.</p>
     */
    NatGatewayId?: string;
    /**
     * <p>Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.</p>
     */
    ProvisionedBandwidth?: ProvisionedBandwidth;
    /**
     * <p>The state of the NAT gateway.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>pending</code>: The NAT gateway is being created and is not ready to process
     *           traffic.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>failed</code>: The NAT gateway could not be created. Check the
     *             <code>failureCode</code> and <code>failureMessage</code> fields for the reason.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>available</code>: The NAT gateway is able to process traffic. This status remains
     *           until you delete the NAT gateway, and does not indicate the health of the NAT gateway.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>deleting</code>: The NAT gateway is in the process of being terminated and may
     *           still be processing traffic.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>deleted</code>: The NAT gateway has been terminated and is no longer processing
     *           traffic.</p>
     *             </li>
     *          </ul>
     */
    State?: NatGatewayState | string;
    /**
     * <p>The ID of the subnet in which the NAT gateway is located.</p>
     */
    SubnetId?: string;
    /**
     * <p>The tags for the NAT gateway.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the VPC in which the NAT gateway is located.</p>
     */
    VpcId?: string;
}
export declare namespace NatGateway {
    const filterSensitiveLog: (obj: NatGateway) => any;
    const isa: (o: any) => o is NatGateway;
}
/**
 * <p>Describes the IP addresses and network interface associated with a NAT gateway.</p>
 */
export interface NatGatewayAddress {
    __type?: "NatGatewayAddress";
    /**
     * <p>The allocation ID of the Elastic IP address that's associated with the NAT gateway.</p>
     */
    AllocationId?: string;
    /**
     * <p>The ID of the network interface associated with the NAT gateway.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The private IP address associated with the Elastic IP address.</p>
     */
    PrivateIp?: string;
    /**
     * <p>The Elastic IP address associated with the NAT gateway.</p>
     */
    PublicIp?: string;
}
export declare namespace NatGatewayAddress {
    const filterSensitiveLog: (obj: NatGatewayAddress) => any;
    const isa: (o: any) => o is NatGatewayAddress;
}
export declare enum NatGatewayState {
    AVAILABLE = "available",
    DELETED = "deleted",
    DELETING = "deleting",
    FAILED = "failed",
    PENDING = "pending"
}
/**
 * <p>Describes a network ACL.</p>
 */
export interface NetworkAcl {
    __type?: "NetworkAcl";
    /**
     * <p>Any associations between the network ACL and one or more subnets</p>
     */
    Associations?: NetworkAclAssociation[];
    /**
     * <p>One or more entries (rules) in the network ACL.</p>
     */
    Entries?: NetworkAclEntry[];
    /**
     * <p>Indicates whether this is the default network ACL for the VPC.</p>
     */
    IsDefault?: boolean;
    /**
     * <p>The ID of the network ACL.</p>
     */
    NetworkAclId?: string;
    /**
     * <p>The ID of the AWS account that owns the network ACL.</p>
     */
    OwnerId?: string;
    /**
     * <p>Any tags assigned to the network ACL.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the VPC for the network ACL.</p>
     */
    VpcId?: string;
}
export declare namespace NetworkAcl {
    const filterSensitiveLog: (obj: NetworkAcl) => any;
    const isa: (o: any) => o is NetworkAcl;
}
/**
 * <p>Describes an association between a network ACL and a subnet.</p>
 */
export interface NetworkAclAssociation {
    __type?: "NetworkAclAssociation";
    /**
     * <p>The ID of the association between a network ACL and a subnet.</p>
     */
    NetworkAclAssociationId?: string;
    /**
     * <p>The ID of the network ACL.</p>
     */
    NetworkAclId?: string;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId?: string;
}
export declare namespace NetworkAclAssociation {
    const filterSensitiveLog: (obj: NetworkAclAssociation) => any;
    const isa: (o: any) => o is NetworkAclAssociation;
}
/**
 * <p>Describes an entry in a network ACL.</p>
 */
export interface NetworkAclEntry {
    __type?: "NetworkAclEntry";
    /**
     * <p>The IPv4 network range to allow or deny, in CIDR notation.</p>
     */
    CidrBlock?: string;
    /**
     * <p>Indicates whether the rule is an egress rule (applied to traffic leaving the subnet).</p>
     */
    Egress?: boolean;
    /**
     * <p>ICMP protocol: The ICMP type and code.</p>
     */
    IcmpTypeCode?: IcmpTypeCode;
    /**
     * <p>The IPv6 network range to allow or deny, in CIDR notation.</p>
     */
    Ipv6CidrBlock?: string;
    /**
     * <p>TCP or UDP protocols: The range of ports the rule applies to.</p>
     */
    PortRange?: PortRange;
    /**
     * <p>The protocol number. A value of "-1" means all protocols.</p>
     */
    Protocol?: string;
    /**
     * <p>Indicates whether to allow or deny the traffic that matches the rule.</p>
     */
    RuleAction?: RuleAction | string;
    /**
     * <p>The rule number for the entry. ACL entries are processed in ascending order by rule number.</p>
     */
    RuleNumber?: number;
}
export declare namespace NetworkAclEntry {
    const filterSensitiveLog: (obj: NetworkAclEntry) => any;
    const isa: (o: any) => o is NetworkAclEntry;
}
/**
 * <p>Describes the networking features of the instance type.</p>
 */
export interface NetworkInfo {
    __type?: "NetworkInfo";
    /**
     * <p>Indicates whether Elastic Network Adapter (ENA) is supported.</p>
     */
    EnaSupport?: EnaSupport | string;
    /**
     * <p>The maximum number of IPv4 addresses per network interface.</p>
     */
    Ipv4AddressesPerInterface?: number;
    /**
     * <p>The maximum number of IPv6 addresses per network interface.</p>
     */
    Ipv6AddressesPerInterface?: number;
    /**
     * <p>Indicates whether IPv6 is supported.</p>
     */
    Ipv6Supported?: boolean;
    /**
     * <p>The maximum number of network interfaces for the instance type.</p>
     */
    MaximumNetworkInterfaces?: number;
    /**
     * <p>Describes the network performance.</p>
     */
    NetworkPerformance?: string;
}
export declare namespace NetworkInfo {
    const filterSensitiveLog: (obj: NetworkInfo) => any;
    const isa: (o: any) => o is NetworkInfo;
}
/**
 * <p>Describes a network interface.</p>
 */
export interface NetworkInterface {
    __type?: "NetworkInterface";
    /**
     * <p>The association information for an Elastic IP address (IPv4) associated with the network interface.</p>
     */
    Association?: NetworkInterfaceAssociation;
    /**
     * <p>The network interface attachment.</p>
     */
    Attachment?: NetworkInterfaceAttachment;
    /**
     * <p>The Availability Zone.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>A description.</p>
     */
    Description?: string;
    /**
     * <p>Any security groups for the network interface.</p>
     */
    Groups?: GroupIdentifier[];
    /**
     * <p>The type of network interface.</p>
     */
    InterfaceType?: NetworkInterfaceType | string;
    /**
     * <p>The IPv6 addresses associated with the network interface.</p>
     */
    Ipv6Addresses?: NetworkInterfaceIpv6Address[];
    /**
     * <p>The MAC address.</p>
     */
    MacAddress?: string;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
     */
    OutpostArn?: string;
    /**
     * <p>The AWS account ID of the owner of the network interface.</p>
     */
    OwnerId?: string;
    /**
     * <p>The private DNS name.</p>
     */
    PrivateDnsName?: string;
    /**
     * <p>The IPv4 address of the network interface within the subnet.</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>The private IPv4 addresses associated with the network interface.</p>
     */
    PrivateIpAddresses?: NetworkInterfacePrivateIpAddress[];
    /**
     * <p>The ID of the entity that launched the instance on your behalf (for example, AWS Management Console or Auto Scaling).</p>
     */
    RequesterId?: string;
    /**
     * <p>Indicates whether the network interface is being managed by AWS.</p>
     */
    RequesterManaged?: boolean;
    /**
     * <p>Indicates whether traffic to or from the instance is validated.</p>
     */
    SourceDestCheck?: boolean;
    /**
     * <p>The status of the network interface.</p>
     */
    Status?: NetworkInterfaceStatus | string;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId?: string;
    /**
     * <p>Any tags assigned to the network interface.</p>
     */
    TagSet?: Tag[];
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace NetworkInterface {
    const filterSensitiveLog: (obj: NetworkInterface) => any;
    const isa: (o: any) => o is NetworkInterface;
}
/**
 * <p>Describes association information for an Elastic IP address (IPv4 only).</p>
 */
export interface NetworkInterfaceAssociation {
    __type?: "NetworkInterfaceAssociation";
    /**
     * <p>The allocation ID.</p>
     */
    AllocationId?: string;
    /**
     * <p>The association ID.</p>
     */
    AssociationId?: string;
    /**
     * <p>The ID of the Elastic IP address owner.</p>
     */
    IpOwnerId?: string;
    /**
     * <p>The public DNS name.</p>
     */
    PublicDnsName?: string;
    /**
     * <p>The address of the Elastic IP address bound to the network interface.</p>
     */
    PublicIp?: string;
}
export declare namespace NetworkInterfaceAssociation {
    const filterSensitiveLog: (obj: NetworkInterfaceAssociation) => any;
    const isa: (o: any) => o is NetworkInterfaceAssociation;
}
/**
 * <p>Describes a network interface attachment.</p>
 */
export interface NetworkInterfaceAttachment {
    __type?: "NetworkInterfaceAttachment";
    /**
     * <p>The timestamp indicating when the attachment initiated.</p>
     */
    AttachTime?: Date;
    /**
     * <p>The ID of the network interface attachment.</p>
     */
    AttachmentId?: string;
    /**
     * <p>Indicates whether the network interface is deleted when the instance is terminated.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>The device index of the network interface attachment on the instance.</p>
     */
    DeviceIndex?: number;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The AWS account ID of the owner of the instance.</p>
     */
    InstanceOwnerId?: string;
    /**
     * <p>The attachment state.</p>
     */
    Status?: AttachmentStatus | string;
}
export declare namespace NetworkInterfaceAttachment {
    const filterSensitiveLog: (obj: NetworkInterfaceAttachment) => any;
    const isa: (o: any) => o is NetworkInterfaceAttachment;
}
/**
 * <p>Describes an attachment change.</p>
 */
export interface NetworkInterfaceAttachmentChanges {
    __type?: "NetworkInterfaceAttachmentChanges";
    /**
     * <p>The ID of the network interface attachment.</p>
     */
    AttachmentId?: string;
    /**
     * <p>Indicates whether the network interface is deleted when the instance is terminated.</p>
     */
    DeleteOnTermination?: boolean;
}
export declare namespace NetworkInterfaceAttachmentChanges {
    const filterSensitiveLog: (obj: NetworkInterfaceAttachmentChanges) => any;
    const isa: (o: any) => o is NetworkInterfaceAttachmentChanges;
}
export declare type NetworkInterfaceAttribute = "attachment" | "description" | "groupSet" | "sourceDestCheck";
export declare type NetworkInterfaceCreationType = "efa";
/**
 * <p>Describes an IPv6 address associated with a network interface.</p>
 */
export interface NetworkInterfaceIpv6Address {
    __type?: "NetworkInterfaceIpv6Address";
    /**
     * <p>The IPv6 address.</p>
     */
    Ipv6Address?: string;
}
export declare namespace NetworkInterfaceIpv6Address {
    const filterSensitiveLog: (obj: NetworkInterfaceIpv6Address) => any;
    const isa: (o: any) => o is NetworkInterfaceIpv6Address;
}
/**
 * <p>Describes a permission for a network interface.</p>
 */
export interface NetworkInterfacePermission {
    __type?: "NetworkInterfacePermission";
    /**
     * <p>The AWS account ID.</p>
     */
    AwsAccountId?: string;
    /**
     * <p>The AWS service.</p>
     */
    AwsService?: string;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The ID of the network interface permission.</p>
     */
    NetworkInterfacePermissionId?: string;
    /**
     * <p>The type of permission.</p>
     */
    Permission?: InterfacePermissionType | string;
    /**
     * <p>Information about the state of the permission.</p>
     */
    PermissionState?: NetworkInterfacePermissionState;
}
export declare namespace NetworkInterfacePermission {
    const filterSensitiveLog: (obj: NetworkInterfacePermission) => any;
    const isa: (o: any) => o is NetworkInterfacePermission;
}
/**
 * <p>Describes the state of a network interface permission.</p>
 */
export interface NetworkInterfacePermissionState {
    __type?: "NetworkInterfacePermissionState";
    /**
     * <p>The state of the permission.</p>
     */
    State?: NetworkInterfacePermissionStateCode | string;
    /**
     * <p>A status message, if applicable.</p>
     */
    StatusMessage?: string;
}
export declare namespace NetworkInterfacePermissionState {
    const filterSensitiveLog: (obj: NetworkInterfacePermissionState) => any;
    const isa: (o: any) => o is NetworkInterfacePermissionState;
}
export declare type NetworkInterfacePermissionStateCode = "granted" | "pending" | "revoked" | "revoking";
/**
 * <p>Describes the private IPv4 address of a network interface.</p>
 */
export interface NetworkInterfacePrivateIpAddress {
    __type?: "NetworkInterfacePrivateIpAddress";
    /**
     * <p>The association information for an Elastic IP address (IPv4) associated with the network interface.</p>
     */
    Association?: NetworkInterfaceAssociation;
    /**
     * <p>Indicates whether this IPv4 address is the primary private IPv4 address of the network interface.</p>
     */
    Primary?: boolean;
    /**
     * <p>The private DNS name.</p>
     */
    PrivateDnsName?: string;
    /**
     * <p>The private IPv4 address.</p>
     */
    PrivateIpAddress?: string;
}
export declare namespace NetworkInterfacePrivateIpAddress {
    const filterSensitiveLog: (obj: NetworkInterfacePrivateIpAddress) => any;
    const isa: (o: any) => o is NetworkInterfacePrivateIpAddress;
}
export declare type NetworkInterfaceStatus = "associated" | "attaching" | "available" | "detaching" | "in-use";
export declare type NetworkInterfaceType = "efa" | "interface" | "natGateway";
/**
 * <p>Describes a DHCP configuration option.</p>
 */
export interface NewDhcpConfiguration {
    __type?: "NewDhcpConfiguration";
    /**
     * <p>The name of a DHCP option.</p>
     */
    Key?: string;
    /**
     * <p>One or more values for the DHCP option.</p>
     */
    Values?: string[];
}
export declare namespace NewDhcpConfiguration {
    const filterSensitiveLog: (obj: NewDhcpConfiguration) => any;
    const isa: (o: any) => o is NewDhcpConfiguration;
}
export declare enum OfferingClassType {
    CONVERTIBLE = "convertible",
    STANDARD = "standard"
}
export declare type OfferingTypeValues = "All Upfront" | "Heavy Utilization" | "Light Utilization" | "Medium Utilization" | "No Upfront" | "Partial Upfront";
export declare enum OnDemandAllocationStrategy {
    LOWEST_PRICE = "lowestPrice",
    PRIORITIZED = "prioritized"
}
/**
 * <p>Describes the configuration of On-Demand Instances in an EC2 Fleet.</p>
 */
export interface OnDemandOptions {
    __type?: "OnDemandOptions";
    /**
     * <p>The order of the launch template overrides to use in fulfilling On-Demand capacity. If
     *          you specify <code>lowest-price</code>, EC2 Fleet uses price to determine the order, launching
     *          the lowest price first. If you specify <code>prioritized</code>, EC2 Fleet uses the priority
     *          that you assigned to each launch template override, launching the highest priority first.
     *          If you do not specify a value, EC2 Fleet defaults to <code>lowest-price</code>.</p>
     */
    AllocationStrategy?: FleetOnDemandAllocationStrategy | string;
    /**
     * <p>The strategy for using unused Capacity Reservations for fulfilling On-Demand capacity.
     *          Supported only for fleets of type <code>instant</code>.</p>
     */
    CapacityReservationOptions?: CapacityReservationOptions;
    /**
     * <p>The maximum amount per hour for On-Demand Instances that you're willing to pay.</p>
     */
    MaxTotalPrice?: string;
    /**
     * <p>The minimum target capacity for On-Demand Instances in the fleet. If the minimum target capacity is
     *          not reached, the fleet launches no instances.</p>
     */
    MinTargetCapacity?: number;
    /**
     * <p>Indicates that the fleet launches all On-Demand Instances into a single Availability Zone. Supported
     *          only for fleets of type <code>instant</code>.</p>
     */
    SingleAvailabilityZone?: boolean;
    /**
     * <p>Indicates that the fleet uses a single instance type to launch all On-Demand Instances in the fleet.
     *          Supported only for fleets of type <code>instant</code>.</p>
     */
    SingleInstanceType?: boolean;
}
export declare namespace OnDemandOptions {
    const filterSensitiveLog: (obj: OnDemandOptions) => any;
    const isa: (o: any) => o is OnDemandOptions;
}
/**
 * <p>Describes the configuration of On-Demand Instances in an EC2 Fleet.</p>
 */
export interface OnDemandOptionsRequest {
    __type?: "OnDemandOptionsRequest";
    /**
     * <p>The order of the launch template overrides to use in fulfilling On-Demand capacity. If
     *          you specify <code>lowest-price</code>, EC2 Fleet uses price to determine the order, launching
     *          the lowest price first. If you specify <code>prioritized</code>, EC2 Fleet uses the priority
     *          that you assigned to each launch template override, launching the highest priority first.
     *          If you do not specify a value, EC2 Fleet defaults to <code>lowest-price</code>.</p>
     */
    AllocationStrategy?: FleetOnDemandAllocationStrategy | string;
    /**
     * <p>The strategy for using unused Capacity Reservations for fulfilling On-Demand capacity.
     *          Supported only for fleets of type <code>instant</code>.</p>
     */
    CapacityReservationOptions?: CapacityReservationOptionsRequest;
    /**
     * <p>The maximum amount per hour for On-Demand Instances that you're willing to pay.</p>
     */
    MaxTotalPrice?: string;
    /**
     * <p>The minimum target capacity for On-Demand Instances in the fleet. If the minimum target capacity is
     *          not reached, the fleet launches no instances.</p>
     */
    MinTargetCapacity?: number;
    /**
     * <p>Indicates that the fleet launches all On-Demand Instances into a single Availability Zone. Supported
     *          only for fleets of type <code>instant</code>.</p>
     */
    SingleAvailabilityZone?: boolean;
    /**
     * <p>Indicates that the fleet uses a single instance type to launch all On-Demand Instances in the fleet.
     *          Supported only for fleets of type <code>instant</code>.</p>
     */
    SingleInstanceType?: boolean;
}
export declare namespace OnDemandOptionsRequest {
    const filterSensitiveLog: (obj: OnDemandOptionsRequest) => any;
    const isa: (o: any) => o is OnDemandOptionsRequest;
}
export declare type OperationType = "add" | "remove";
export declare enum PaymentOption {
    ALL_UPFRONT = "AllUpfront",
    NO_UPFRONT = "NoUpfront",
    PARTIAL_UPFRONT = "PartialUpfront"
}
/**
 * <p>Describes the data that identifies an Amazon FPGA image (AFI) on the PCI bus.</p>
 */
export interface PciId {
    __type?: "PciId";
    /**
     * <p>The ID of the device.</p>
     */
    DeviceId?: string;
    /**
     * <p>The ID of the subsystem.</p>
     */
    SubsystemId?: string;
    /**
     * <p>The ID of the vendor for the subsystem.</p>
     */
    SubsystemVendorId?: string;
    /**
     * <p>The ID of the vendor.</p>
     */
    VendorId?: string;
}
export declare namespace PciId {
    const filterSensitiveLog: (obj: PciId) => any;
    const isa: (o: any) => o is PciId;
}
/**
 * <p>The status of the transit gateway peering attachment.</p>
 */
export interface PeeringAttachmentStatus {
    __type?: "PeeringAttachmentStatus";
    /**
     * <p>The status code.</p>
     */
    Code?: string;
    /**
     * <p>The status message, if applicable.</p>
     */
    Message?: string;
}
export declare namespace PeeringAttachmentStatus {
    const filterSensitiveLog: (obj: PeeringAttachmentStatus) => any;
    const isa: (o: any) => o is PeeringAttachmentStatus;
}
/**
 * <p>Describes the VPC peering connection options.</p>
 */
export interface PeeringConnectionOptions {
    __type?: "PeeringConnectionOptions";
    /**
     * <p>If true, the public DNS hostnames of instances in the specified VPC resolve to private
     *             IP addresses when queried from instances in the peer VPC.</p>
     */
    AllowDnsResolutionFromRemoteVpc?: boolean;
    /**
     * <p>If true, enables outbound communication from an EC2-Classic instance that's linked to
     *             a local VPC using ClassicLink to instances in a peer VPC.</p>
     */
    AllowEgressFromLocalClassicLinkToRemoteVpc?: boolean;
    /**
     * <p>If true, enables outbound communication from instances in a local VPC to an
     *             EC2-Classic instance that's linked to a peer VPC using ClassicLink.</p>
     */
    AllowEgressFromLocalVpcToRemoteClassicLink?: boolean;
}
export declare namespace PeeringConnectionOptions {
    const filterSensitiveLog: (obj: PeeringConnectionOptions) => any;
    const isa: (o: any) => o is PeeringConnectionOptions;
}
/**
 * <p>The VPC peering connection options.</p>
 */
export interface PeeringConnectionOptionsRequest {
    __type?: "PeeringConnectionOptionsRequest";
    /**
     * <p>If true, enables a local VPC to resolve public DNS hostnames to private IP addresses when queried from instances in the peer VPC.</p>
     */
    AllowDnsResolutionFromRemoteVpc?: boolean;
    /**
     * <p>If true, enables outbound communication from an EC2-Classic instance that's linked to
     *             a local VPC using ClassicLink to instances in a peer VPC.</p>
     */
    AllowEgressFromLocalClassicLinkToRemoteVpc?: boolean;
    /**
     * <p>If true, enables outbound communication from instances in a local VPC to an
     *             EC2-Classic instance that's linked to a peer VPC using ClassicLink.</p>
     */
    AllowEgressFromLocalVpcToRemoteClassicLink?: boolean;
}
export declare namespace PeeringConnectionOptionsRequest {
    const filterSensitiveLog: (obj: PeeringConnectionOptionsRequest) => any;
    const isa: (o: any) => o is PeeringConnectionOptionsRequest;
}
/**
 * <p>Information about the transit gateway in the peering attachment.</p>
 */
export interface PeeringTgwInfo {
    __type?: "PeeringTgwInfo";
    /**
     * <p>The AWS account ID of the owner of the transit gateway.</p>
     */
    OwnerId?: string;
    /**
     * <p>The Region of the transit gateway.</p>
     */
    Region?: string;
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId?: string;
}
export declare namespace PeeringTgwInfo {
    const filterSensitiveLog: (obj: PeeringTgwInfo) => any;
    const isa: (o: any) => o is PeeringTgwInfo;
}
export declare type PermissionGroup = "all";
/**
 * <p>The Diffie-Hellmann group number for phase 1 IKE negotiations.</p>
 */
export interface Phase1DHGroupNumbersListValue {
    __type?: "Phase1DHGroupNumbersListValue";
    /**
     * <p>The Diffie-Hellmann group number.</p>
     */
    Value?: number;
}
export declare namespace Phase1DHGroupNumbersListValue {
    const filterSensitiveLog: (obj: Phase1DHGroupNumbersListValue) => any;
    const isa: (o: any) => o is Phase1DHGroupNumbersListValue;
}
/**
 * <p>Specifies a Diffie-Hellman group number for the VPN tunnel for phase 1 IKE negotiations.</p>
 */
export interface Phase1DHGroupNumbersRequestListValue {
    __type?: "Phase1DHGroupNumbersRequestListValue";
    /**
     * <p>The Diffie-Hellmann group number.</p>
     */
    Value?: number;
}
export declare namespace Phase1DHGroupNumbersRequestListValue {
    const filterSensitiveLog: (obj: Phase1DHGroupNumbersRequestListValue) => any;
    const isa: (o: any) => o is Phase1DHGroupNumbersRequestListValue;
}
/**
 * <p>The encryption algorithm for phase 1 IKE negotiations.</p>
 */
export interface Phase1EncryptionAlgorithmsListValue {
    __type?: "Phase1EncryptionAlgorithmsListValue";
    /**
     * <p>The value for the encryption algorithm.</p>
     */
    Value?: string;
}
export declare namespace Phase1EncryptionAlgorithmsListValue {
    const filterSensitiveLog: (obj: Phase1EncryptionAlgorithmsListValue) => any;
    const isa: (o: any) => o is Phase1EncryptionAlgorithmsListValue;
}
/**
 * <p>Specifies the encryption algorithm for the VPN tunnel for phase 1 IKE negotiations.</p>
 */
export interface Phase1EncryptionAlgorithmsRequestListValue {
    __type?: "Phase1EncryptionAlgorithmsRequestListValue";
    /**
     * <p>The value for the encryption algorithm.</p>
     */
    Value?: string;
}
export declare namespace Phase1EncryptionAlgorithmsRequestListValue {
    const filterSensitiveLog: (obj: Phase1EncryptionAlgorithmsRequestListValue) => any;
    const isa: (o: any) => o is Phase1EncryptionAlgorithmsRequestListValue;
}
/**
 * <p>The integrity algorithm for phase 1 IKE negotiations.</p>
 */
export interface Phase1IntegrityAlgorithmsListValue {
    __type?: "Phase1IntegrityAlgorithmsListValue";
    /**
     * <p>The value for the integrity algorithm.</p>
     */
    Value?: string;
}
export declare namespace Phase1IntegrityAlgorithmsListValue {
    const filterSensitiveLog: (obj: Phase1IntegrityAlgorithmsListValue) => any;
    const isa: (o: any) => o is Phase1IntegrityAlgorithmsListValue;
}
/**
 * <p>Specifies the integrity algorithm for the VPN tunnel for phase 1 IKE negotiations.</p>
 */
export interface Phase1IntegrityAlgorithmsRequestListValue {
    __type?: "Phase1IntegrityAlgorithmsRequestListValue";
    /**
     * <p>The value for the integrity algorithm.</p>
     */
    Value?: string;
}
export declare namespace Phase1IntegrityAlgorithmsRequestListValue {
    const filterSensitiveLog: (obj: Phase1IntegrityAlgorithmsRequestListValue) => any;
    const isa: (o: any) => o is Phase1IntegrityAlgorithmsRequestListValue;
}
/**
 * <p>The Diffie-Hellmann group number for phase 2 IKE negotiations.</p>
 */
export interface Phase2DHGroupNumbersListValue {
    __type?: "Phase2DHGroupNumbersListValue";
    /**
     * <p>The Diffie-Hellmann group number.</p>
     */
    Value?: number;
}
export declare namespace Phase2DHGroupNumbersListValue {
    const filterSensitiveLog: (obj: Phase2DHGroupNumbersListValue) => any;
    const isa: (o: any) => o is Phase2DHGroupNumbersListValue;
}
/**
 * <p>Specifies a Diffie-Hellman group number for the VPN tunnel for phase 2 IKE negotiations.</p>
 */
export interface Phase2DHGroupNumbersRequestListValue {
    __type?: "Phase2DHGroupNumbersRequestListValue";
    /**
     * <p>The Diffie-Hellmann group number.</p>
     */
    Value?: number;
}
export declare namespace Phase2DHGroupNumbersRequestListValue {
    const filterSensitiveLog: (obj: Phase2DHGroupNumbersRequestListValue) => any;
    const isa: (o: any) => o is Phase2DHGroupNumbersRequestListValue;
}
/**
 * <p>The encryption algorithm for phase 2 IKE negotiations.</p>
 */
export interface Phase2EncryptionAlgorithmsListValue {
    __type?: "Phase2EncryptionAlgorithmsListValue";
    /**
     * <p>The encryption algorithm.</p>
     */
    Value?: string;
}
export declare namespace Phase2EncryptionAlgorithmsListValue {
    const filterSensitiveLog: (obj: Phase2EncryptionAlgorithmsListValue) => any;
    const isa: (o: any) => o is Phase2EncryptionAlgorithmsListValue;
}
/**
 * <p>Specifies the encryption algorithm for the VPN tunnel for phase 2 IKE negotiations.</p>
 */
export interface Phase2EncryptionAlgorithmsRequestListValue {
    __type?: "Phase2EncryptionAlgorithmsRequestListValue";
    /**
     * <p>The encryption algorithm.</p>
     */
    Value?: string;
}
export declare namespace Phase2EncryptionAlgorithmsRequestListValue {
    const filterSensitiveLog: (obj: Phase2EncryptionAlgorithmsRequestListValue) => any;
    const isa: (o: any) => o is Phase2EncryptionAlgorithmsRequestListValue;
}
/**
 * <p>The integrity algorithm for phase 2 IKE negotiations.</p>
 */
export interface Phase2IntegrityAlgorithmsListValue {
    __type?: "Phase2IntegrityAlgorithmsListValue";
    /**
     * <p>The integrity algorithm.</p>
     */
    Value?: string;
}
export declare namespace Phase2IntegrityAlgorithmsListValue {
    const filterSensitiveLog: (obj: Phase2IntegrityAlgorithmsListValue) => any;
    const isa: (o: any) => o is Phase2IntegrityAlgorithmsListValue;
}
/**
 * <p>Specifies the integrity algorithm for the VPN tunnel for phase 2 IKE negotiations.</p>
 */
export interface Phase2IntegrityAlgorithmsRequestListValue {
    __type?: "Phase2IntegrityAlgorithmsRequestListValue";
    /**
     * <p>The integrity algorithm.</p>
     */
    Value?: string;
}
export declare namespace Phase2IntegrityAlgorithmsRequestListValue {
    const filterSensitiveLog: (obj: Phase2IntegrityAlgorithmsRequestListValue) => any;
    const isa: (o: any) => o is Phase2IntegrityAlgorithmsRequestListValue;
}
/**
 * <p>Describes the placement of an instance.</p>
 */
export interface Placement {
    __type?: "Placement";
    /**
     * <p>The affinity setting for the instance on the Dedicated Host. This parameter is not
     *             supported for the <a>ImportInstance</a> command.</p>
     */
    Affinity?: string;
    /**
     * <p>The Availability Zone of the instance.</p>
     *         <p>If not specified, an Availability Zone will be automatically chosen for you based on
     *             the load balancing criteria for the Region.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The name of the placement group the instance is in.</p>
     */
    GroupName?: string;
    /**
     * <p>The ID of the Dedicated Host on which the instance resides. This parameter is not
     *             supported for the <a>ImportInstance</a> command.</p>
     */
    HostId?: string;
    /**
     * <p>The ARN of the host resource group in which to launch the instances. If you specify
     *     		a host resource group ARN, omit the <b>Tenancy</b>
     *     		parameter or set it to <code>host</code>.</p>
     */
    HostResourceGroupArn?: string;
    /**
     * <p>The number of the partition the instance is in. Valid only if the placement group
     *             strategy is set to <code>partition</code>.</p>
     */
    PartitionNumber?: number;
    /**
     * <p>Reserved for future use.</p>
     */
    SpreadDomain?: string;
    /**
     * <p>The tenancy of the instance (if the instance is running in a VPC). An instance with a
     *             tenancy of <code>dedicated</code> runs on single-tenant hardware. The <code>host</code>
     *             tenancy is not supported for the <a>ImportInstance</a> command.</p>
     */
    Tenancy?: Tenancy | string;
}
export declare namespace Placement {
    const filterSensitiveLog: (obj: Placement) => any;
    const isa: (o: any) => o is Placement;
}
/**
 * <p>Describes a placement group.</p>
 */
export interface PlacementGroup {
    __type?: "PlacementGroup";
    /**
     * <p>The ID of the placement group.</p>
     */
    GroupId?: string;
    /**
     * <p>The name of the placement group.</p>
     */
    GroupName?: string;
    /**
     * <p>The number of partitions. Valid only if <b>strategy</b> is
     *             set to <code>partition</code>.</p>
     */
    PartitionCount?: number;
    /**
     * <p>The state of the placement group.</p>
     */
    State?: PlacementGroupState | string;
    /**
     * <p>The placement strategy.</p>
     */
    Strategy?: PlacementStrategy | string;
    /**
     * <p>Any tags applied to the placement group.</p>
     */
    Tags?: Tag[];
}
export declare namespace PlacementGroup {
    const filterSensitiveLog: (obj: PlacementGroup) => any;
    const isa: (o: any) => o is PlacementGroup;
}
/**
 * <p>Describes the placement group support of the instance type.</p>
 */
export interface PlacementGroupInfo {
    __type?: "PlacementGroupInfo";
    /**
     * <p>A list of supported placement groups types.</p>
     */
    SupportedStrategies?: (PlacementGroupStrategy | string)[];
}
export declare namespace PlacementGroupInfo {
    const filterSensitiveLog: (obj: PlacementGroupInfo) => any;
    const isa: (o: any) => o is PlacementGroupInfo;
}
export declare type PlacementGroupState = "available" | "deleted" | "deleting" | "pending";
export declare type PlacementGroupStrategy = "cluster" | "partition" | "spread";
/**
 * <p>Describes the placement of an instance.</p>
 */
export interface PlacementResponse {
    __type?: "PlacementResponse";
    /**
     * <p>The name of the placement group that the instance is in.</p>
     */
    GroupName?: string;
}
export declare namespace PlacementResponse {
    const filterSensitiveLog: (obj: PlacementResponse) => any;
    const isa: (o: any) => o is PlacementResponse;
}
export declare type PlacementStrategy = "cluster" | "partition" | "spread";
export declare type PlatformValues = "Windows";
/**
 * <p>Describes a CIDR block for an address pool.</p>
 */
export interface PoolCidrBlock {
    __type?: "PoolCidrBlock";
    /**
     * <p>The CIDR block.</p>
     */
    Cidr?: string;
}
export declare namespace PoolCidrBlock {
    const filterSensitiveLog: (obj: PoolCidrBlock) => any;
    const isa: (o: any) => o is PoolCidrBlock;
}
/**
 * <p>Describes a range of ports.</p>
 */
export interface PortRange {
    __type?: "PortRange";
    /**
     * <p>The first port in the range.</p>
     */
    From?: number;
    /**
     * <p>The last port in the range.</p>
     */
    To?: number;
}
export declare namespace PortRange {
    const filterSensitiveLog: (obj: PortRange) => any;
    const isa: (o: any) => o is PortRange;
}
/**
 * <p>Describes prefixes for AWS services.</p>
 */
export interface PrefixList {
    __type?: "PrefixList";
    /**
     * <p>The IP address range of the AWS service.</p>
     */
    Cidrs?: string[];
    /**
     * <p>The ID of the prefix.</p>
     */
    PrefixListId?: string;
    /**
     * <p>The name of the prefix.</p>
     */
    PrefixListName?: string;
}
export declare namespace PrefixList {
    const filterSensitiveLog: (obj: PrefixList) => any;
    const isa: (o: any) => o is PrefixList;
}
/**
 * <p>Describes a prefix list ID.</p>
 */
export interface PrefixListId {
    __type?: "PrefixListId";
    /**
     * <p>A description for the security group rule that references this prefix list ID.</p>
     *          <p>Constraints: Up to 255 characters in length. Allowed characters are a-z, A-Z, 0-9,
     *       spaces, and ._-:/()#,@[]+=;{}!$*</p>
     */
    Description?: string;
    /**
     * <p>The ID of the prefix.</p>
     */
    PrefixListId?: string;
}
export declare namespace PrefixListId {
    const filterSensitiveLog: (obj: PrefixListId) => any;
    const isa: (o: any) => o is PrefixListId;
}
/**
 * <p>Describes the price for a Reserved Instance.</p>
 */
export interface PriceSchedule {
    __type?: "PriceSchedule";
    /**
     * <p>The current price schedule, as determined by the term remaining for the Reserved Instance in the listing.</p>
     *          <p>A specific price schedule is always in effect, but only one price schedule can be active at any time. Take, for example, a Reserved Instance listing that has five months remaining in its term. When you specify price schedules for five months and two months, this means that schedule 1, covering the first three months of the remaining term, will be active during months 5, 4, and 3. Then schedule 2, covering the last two months of the term, will be active for months 2 and 1.</p>
     */
    Active?: boolean;
    /**
     * <p>The currency for transacting the Reserved Instance resale.
     * 				At this time, the only supported currency is <code>USD</code>.</p>
     */
    CurrencyCode?: CurrencyCodeValues | string;
    /**
     * <p>The fixed price for the term.</p>
     */
    Price?: number;
    /**
     * <p>The number of months remaining in the reservation. For example, 2 is the second to the last month before the capacity reservation expires.</p>
     */
    Term?: number;
}
export declare namespace PriceSchedule {
    const filterSensitiveLog: (obj: PriceSchedule) => any;
    const isa: (o: any) => o is PriceSchedule;
}
/**
 * <p>Describes the price for a Reserved Instance.</p>
 */
export interface PriceScheduleSpecification {
    __type?: "PriceScheduleSpecification";
    /**
     * <p>The currency for transacting the Reserved Instance resale.
     * 				At this time, the only supported currency is <code>USD</code>.</p>
     */
    CurrencyCode?: CurrencyCodeValues | string;
    /**
     * <p>The fixed price for the term.</p>
     */
    Price?: number;
    /**
     * <p>The number of months remaining in the reservation. For example, 2 is the second to the last month before the capacity reservation expires.</p>
     */
    Term?: number;
}
export declare namespace PriceScheduleSpecification {
    const filterSensitiveLog: (obj: PriceScheduleSpecification) => any;
    const isa: (o: any) => o is PriceScheduleSpecification;
}
/**
 * <p>Describes a Reserved Instance offering.</p>
 */
export interface PricingDetail {
    __type?: "PricingDetail";
    /**
     * <p>The number of reservations available for the price.</p>
     */
    Count?: number;
    /**
     * <p>The price per instance.</p>
     */
    Price?: number;
}
export declare namespace PricingDetail {
    const filterSensitiveLog: (obj: PricingDetail) => any;
    const isa: (o: any) => o is PricingDetail;
}
/**
 * <p>PrincipalIdFormat description</p>
 */
export interface PrincipalIdFormat {
    __type?: "PrincipalIdFormat";
    /**
     * <p>PrincipalIdFormatARN description</p>
     */
    Arn?: string;
    /**
     * <p>PrincipalIdFormatStatuses description</p>
     */
    Statuses?: IdFormat[];
}
export declare namespace PrincipalIdFormat {
    const filterSensitiveLog: (obj: PrincipalIdFormat) => any;
    const isa: (o: any) => o is PrincipalIdFormat;
}
export declare enum PrincipalType {
    Account = "Account",
    All = "All",
    OrganizationUnit = "OrganizationUnit",
    Role = "Role",
    Service = "Service",
    User = "User"
}
/**
 * <p>Information about the private DNS name for the service endpoint. For more information
 *             about these parameters, see <a href="https://docs.aws.amazon.com/vpc/latest/userguide/ndpoint-services-dns-validation.html">VPC Endpoint Service
 *                 Private DNS Name Verification</a> in the
 *             <i>Amazon Virtual Private Cloud User Guide</i>.</p>
 */
export interface PrivateDnsNameConfiguration {
    __type?: "PrivateDnsNameConfiguration";
    /**
     * <p>The name of the record subdomain the service provider needs to create. The service provider adds the <code>value</code> text to the <code>name</code>.</p>
     */
    Name?: string;
    /**
     * <p>The verification state of the VPC endpoint service.</p>
     *         <p>>Consumers
     *             of the endpoint service can use the private name only when the state is
     *                 <code>verified</code>.</p>
     */
    State?: DnsNameState | string;
    /**
     * <p>The endpoint service verification type, for example TXT.</p>
     */
    Type?: string;
    /**
     * <p>The value the service provider adds to the private DNS name domain record before verification.</p>
     */
    Value?: string;
}
export declare namespace PrivateDnsNameConfiguration {
    const filterSensitiveLog: (obj: PrivateDnsNameConfiguration) => any;
    const isa: (o: any) => o is PrivateDnsNameConfiguration;
}
/**
 * <p>Describes a secondary private IPv4 address for a network interface.</p>
 */
export interface PrivateIpAddressSpecification {
    __type?: "PrivateIpAddressSpecification";
    /**
     * <p>Indicates whether the private IPv4 address is the primary private IPv4 address. Only
     *             one IPv4 address can be designated as primary.</p>
     */
    Primary?: boolean;
    /**
     * <p>The private IPv4 addresses.</p>
     */
    PrivateIpAddress?: string;
}
export declare namespace PrivateIpAddressSpecification {
    const filterSensitiveLog: (obj: PrivateIpAddressSpecification) => any;
    const isa: (o: any) => o is PrivateIpAddressSpecification;
}
/**
 * <p>Describes the processor used by the instance type.</p>
 */
export interface ProcessorInfo {
    __type?: "ProcessorInfo";
    /**
     * <p>A list of architectures supported by the instance type.</p>
     */
    SupportedArchitectures?: (ArchitectureType | string)[];
    /**
     * <p>The speed of the processor, in GHz.</p>
     */
    SustainedClockSpeedInGhz?: number;
}
export declare namespace ProcessorInfo {
    const filterSensitiveLog: (obj: ProcessorInfo) => any;
    const isa: (o: any) => o is ProcessorInfo;
}
/**
 * <p>Describes a product code.</p>
 */
export interface ProductCode {
    __type?: "ProductCode";
    /**
     * <p>The product code.</p>
     */
    ProductCodeId?: string;
    /**
     * <p>The type of product code.</p>
     */
    ProductCodeType?: ProductCodeValues | string;
}
export declare namespace ProductCode {
    const filterSensitiveLog: (obj: ProductCode) => any;
    const isa: (o: any) => o is ProductCode;
}
export declare type ProductCodeValues = "devpay" | "marketplace";
/**
 * <p>Describes a virtual private gateway propagating route.</p>
 */
export interface PropagatingVgw {
    __type?: "PropagatingVgw";
    /**
     * <p>The ID of the virtual private gateway.</p>
     */
    GatewayId?: string;
}
export declare namespace PropagatingVgw {
    const filterSensitiveLog: (obj: PropagatingVgw) => any;
    const isa: (o: any) => o is PropagatingVgw;
}
export interface ProvisionByoipCidrRequest {
    __type?: "ProvisionByoipCidrRequest";
    /**
     * <p>The public IPv4 or IPv6 address range, in CIDR notation. The most specific IPv4 prefix that you can
     *          specify is /24. The most specific IPv6 prefix you can specify is /56. The address range cannot overlap with another address range that you've
     *          brought to this or another Region.</p>
     */
    Cidr: string | undefined;
    /**
     * <p>A signed document that proves that you are authorized to bring the specified IP address
     *          range to Amazon using BYOIP.</p>
     */
    CidrAuthorizationContext?: CidrAuthorizationContext;
    /**
     * <p>A description for the address range and the address pool.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>(IPv6 only) Indicate whether the address range will be publicly advertised to the
     *             internet.</p>
     *         <p>Default: true</p>
     */
    PubliclyAdvertisable?: boolean;
}
export declare namespace ProvisionByoipCidrRequest {
    const filterSensitiveLog: (obj: ProvisionByoipCidrRequest) => any;
    const isa: (o: any) => o is ProvisionByoipCidrRequest;
}
export interface ProvisionByoipCidrResult {
    __type?: "ProvisionByoipCidrResult";
    /**
     * <p>Information about the address range.</p>
     */
    ByoipCidr?: ByoipCidr;
}
export declare namespace ProvisionByoipCidrResult {
    const filterSensitiveLog: (obj: ProvisionByoipCidrResult) => any;
    const isa: (o: any) => o is ProvisionByoipCidrResult;
}
/**
 * <p>Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.</p>
 */
export interface ProvisionedBandwidth {
    __type?: "ProvisionedBandwidth";
    /**
     * <p>Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.</p>
     */
    ProvisionTime?: Date;
    /**
     * <p>Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.</p>
     */
    Provisioned?: string;
    /**
     * <p>Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.</p>
     */
    RequestTime?: Date;
    /**
     * <p>Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.</p>
     */
    Requested?: string;
    /**
     * <p>Reserved. If you need to sustain traffic greater than the <a href="https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html">documented limits</a>, contact us through the <a href="https://console.aws.amazon.com/support/home?">Support Center</a>.</p>
     */
    Status?: string;
}
export declare namespace ProvisionedBandwidth {
    const filterSensitiveLog: (obj: ProvisionedBandwidth) => any;
    const isa: (o: any) => o is ProvisionedBandwidth;
}
/**
 * <p>Describes an IPv4 address pool.</p>
 */
export interface PublicIpv4Pool {
    __type?: "PublicIpv4Pool";
    /**
     * <p>A description of the address pool.</p>
     */
    Description?: string;
    /**
     * <p>The address ranges.</p>
     */
    PoolAddressRanges?: PublicIpv4PoolRange[];
    /**
     * <p>The ID of the address pool.</p>
     */
    PoolId?: string;
    /**
     * <p>The total number of addresses.</p>
     */
    TotalAddressCount?: number;
    /**
     * <p>The total number of available addresses.</p>
     */
    TotalAvailableAddressCount?: number;
}
export declare namespace PublicIpv4Pool {
    const filterSensitiveLog: (obj: PublicIpv4Pool) => any;
    const isa: (o: any) => o is PublicIpv4Pool;
}
/**
 * <p>Describes an address range of an IPv4 address pool.</p>
 */
export interface PublicIpv4PoolRange {
    __type?: "PublicIpv4PoolRange";
    /**
     * <p>The number of addresses in the range.</p>
     */
    AddressCount?: number;
    /**
     * <p>The number of available addresses in the range.</p>
     */
    AvailableAddressCount?: number;
    /**
     * <p>The first IP address in the range.</p>
     */
    FirstAddress?: string;
    /**
     * <p>The last IP address in the range.</p>
     */
    LastAddress?: string;
}
export declare namespace PublicIpv4PoolRange {
    const filterSensitiveLog: (obj: PublicIpv4PoolRange) => any;
    const isa: (o: any) => o is PublicIpv4PoolRange;
}
/**
 * <p>Describes the result of the purchase.</p>
 */
export interface Purchase {
    __type?: "Purchase";
    /**
     * <p>The currency in which the <code>UpfrontPrice</code> and <code>HourlyPrice</code>
     *             amounts are specified. At this time, the only supported currency is
     *             <code>USD</code>.</p>
     */
    CurrencyCode?: CurrencyCodeValues | string;
    /**
     * <p>The duration of the reservation's term in seconds.</p>
     */
    Duration?: number;
    /**
     * <p>The IDs of the Dedicated Hosts associated with the reservation.</p>
     */
    HostIdSet?: string[];
    /**
     * <p>The ID of the reservation.</p>
     */
    HostReservationId?: string;
    /**
     * <p>The hourly price of the reservation per hour.</p>
     */
    HourlyPrice?: string;
    /**
     * <p>The instance family on the Dedicated Host that the reservation can be associated
     *             with.</p>
     */
    InstanceFamily?: string;
    /**
     * <p>The payment option for the reservation.</p>
     */
    PaymentOption?: PaymentOption | string;
    /**
     * <p>The upfront price of the reservation.</p>
     */
    UpfrontPrice?: string;
}
export declare namespace Purchase {
    const filterSensitiveLog: (obj: Purchase) => any;
    const isa: (o: any) => o is Purchase;
}
export interface PurchaseHostReservationRequest {
    __type?: "PurchaseHostReservationRequest";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The currency in which the <code>totalUpfrontPrice</code>, <code>LimitPrice</code>,
     *             and <code>totalHourlyPrice</code> amounts are specified. At this time, the only
     *             supported currency is <code>USD</code>.</p>
     */
    CurrencyCode?: CurrencyCodeValues | string;
    /**
     * <p>The IDs of the Dedicated Hosts with which the reservation will be associated.</p>
     */
    HostIdSet: string[] | undefined;
    /**
     * <p>The specified limit is checked against the total upfront cost of the reservation
     *             (calculated as the offering's upfront cost multiplied by the host count). If the total
     *             upfront cost is greater than the specified price limit, the request fails. This is used
     *             to ensure that the purchase does not exceed the expected upfront cost of the purchase.
     *             At this time, the only supported currency is <code>USD</code>. For example, to indicate
     *             a limit price of USD 100, specify 100.00.</p>
     */
    LimitPrice?: string;
    /**
     * <p>The ID of the offering.</p>
     */
    OfferingId: string | undefined;
}
export declare namespace PurchaseHostReservationRequest {
    const filterSensitiveLog: (obj: PurchaseHostReservationRequest) => any;
    const isa: (o: any) => o is PurchaseHostReservationRequest;
}
export interface PurchaseHostReservationResult {
    __type?: "PurchaseHostReservationResult";
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The currency in which the <code>totalUpfrontPrice</code> and
     *                 <code>totalHourlyPrice</code> amounts are specified. At this time, the only
     *             supported currency is <code>USD</code>.</p>
     */
    CurrencyCode?: CurrencyCodeValues | string;
    /**
     * <p>Describes the details of the purchase.</p>
     */
    Purchase?: Purchase[];
    /**
     * <p>The total hourly price of the reservation calculated per hour.</p>
     */
    TotalHourlyPrice?: string;
    /**
     * <p>The total amount charged to your account when you purchase the reservation.</p>
     */
    TotalUpfrontPrice?: string;
}
export declare namespace PurchaseHostReservationResult {
    const filterSensitiveLog: (obj: PurchaseHostReservationResult) => any;
    const isa: (o: any) => o is PurchaseHostReservationResult;
}
/**
 * <p>Describes a request to purchase Scheduled Instances.</p>
 */
export interface PurchaseRequest {
    __type?: "PurchaseRequest";
    /**
     * <p>The number of instances.</p>
     */
    InstanceCount: number | undefined;
    /**
     * <p>The purchase token.</p>
     */
    PurchaseToken: string | undefined;
}
export declare namespace PurchaseRequest {
    const filterSensitiveLog: (obj: PurchaseRequest) => any;
    const isa: (o: any) => o is PurchaseRequest;
}
/**
 * <p>Contains the parameters for PurchaseReservedInstancesOffering.</p>
 */
export interface PurchaseReservedInstancesOfferingRequest {
    __type?: "PurchaseReservedInstancesOfferingRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The number of Reserved Instances to purchase.</p>
     */
    InstanceCount: number | undefined;
    /**
     * <p>Specified for Reserved Instance Marketplace offerings to limit the total order and ensure that the Reserved Instances are not purchased at unexpected prices.</p>
     */
    LimitPrice?: ReservedInstanceLimitPrice;
    /**
     * <p>The time at which to purchase the Reserved Instance, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    PurchaseTime?: Date;
    /**
     * <p>The ID of the Reserved Instance offering to purchase.</p>
     */
    ReservedInstancesOfferingId: string | undefined;
}
export declare namespace PurchaseReservedInstancesOfferingRequest {
    const filterSensitiveLog: (obj: PurchaseReservedInstancesOfferingRequest) => any;
    const isa: (o: any) => o is PurchaseReservedInstancesOfferingRequest;
}
/**
 * <p>Contains the output of PurchaseReservedInstancesOffering.</p>
 */
export interface PurchaseReservedInstancesOfferingResult {
    __type?: "PurchaseReservedInstancesOfferingResult";
    /**
     * <p>The IDs of the purchased Reserved Instances.</p>
     */
    ReservedInstancesId?: string;
}
export declare namespace PurchaseReservedInstancesOfferingResult {
    const filterSensitiveLog: (obj: PurchaseReservedInstancesOfferingResult) => any;
    const isa: (o: any) => o is PurchaseReservedInstancesOfferingResult;
}
/**
 * <p>Contains the parameters for PurchaseScheduledInstances.</p>
 */
export interface PurchaseScheduledInstancesRequest {
    __type?: "PurchaseScheduledInstancesRequest";
    /**
     * <p>Unique, case-sensitive identifier that ensures the idempotency of the request.
     *          For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The purchase requests.</p>
     */
    PurchaseRequests: PurchaseRequest[] | undefined;
}
export declare namespace PurchaseScheduledInstancesRequest {
    const filterSensitiveLog: (obj: PurchaseScheduledInstancesRequest) => any;
    const isa: (o: any) => o is PurchaseScheduledInstancesRequest;
}
/**
 * <p>Contains the output of PurchaseScheduledInstances.</p>
 */
export interface PurchaseScheduledInstancesResult {
    __type?: "PurchaseScheduledInstancesResult";
    /**
     * <p>Information about the Scheduled Instances.</p>
     */
    ScheduledInstanceSet?: ScheduledInstance[];
}
export declare namespace PurchaseScheduledInstancesResult {
    const filterSensitiveLog: (obj: PurchaseScheduledInstancesResult) => any;
    const isa: (o: any) => o is PurchaseScheduledInstancesResult;
}
export interface RebootInstancesRequest {
    __type?: "RebootInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The instance IDs.</p>
     */
    InstanceIds: string[] | undefined;
}
export declare namespace RebootInstancesRequest {
    const filterSensitiveLog: (obj: RebootInstancesRequest) => any;
    const isa: (o: any) => o is RebootInstancesRequest;
}
/**
 * <p>Describes a recurring charge.</p>
 */
export interface RecurringCharge {
    __type?: "RecurringCharge";
    /**
     * <p>The amount of the recurring charge.</p>
     */
    Amount?: number;
    /**
     * <p>The frequency of the recurring charge.</p>
     */
    Frequency?: RecurringChargeFrequency | string;
}
export declare namespace RecurringCharge {
    const filterSensitiveLog: (obj: RecurringCharge) => any;
    const isa: (o: any) => o is RecurringCharge;
}
export declare type RecurringChargeFrequency = "Hourly";
/**
 * <p>Describes a Region.</p>
 */
export interface Region {
    __type?: "Region";
    /**
     * <p>The Region service endpoint.</p>
     */
    Endpoint?: string;
    /**
     * <p>The Region opt-in status. The possible values are <code>opt-in-not-required</code>, <code>opted-in</code>, and
     *         <code>not-opted-in</code>.</p>
     */
    OptInStatus?: string;
    /**
     * <p>The name of the Region.</p>
     */
    RegionName?: string;
}
export declare namespace Region {
    const filterSensitiveLog: (obj: Region) => any;
    const isa: (o: any) => o is Region;
}
/**
 * <p>Contains the parameters for RegisterImage.</p>
 */
export interface RegisterImageRequest {
    __type?: "RegisterImageRequest";
    /**
     * <p>The architecture of the AMI.</p>
     *          <p>Default: For Amazon EBS-backed AMIs, <code>i386</code>.
     *         For instance store-backed AMIs, the architecture specified in the manifest file.</p>
     */
    Architecture?: ArchitectureValues | string;
    /**
     * <p>The billing product codes. Your account must be authorized to specify billing product codes. Otherwise,
     *          you can use the AWS Marketplace to bill for the use of an AMI.</p>
     */
    BillingProducts?: string[];
    /**
     * <p>The block device mapping entries.</p>
     */
    BlockDeviceMappings?: BlockDeviceMapping[];
    /**
     * <p>A description for your AMI.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Set to <code>true</code> to enable enhanced networking with ENA for the AMI and any instances that you launch from the AMI.</p>
     *          <p>This option is supported only for HVM AMIs. Specifying this option with a PV AMI can make instances launched from the AMI unreachable.</p>
     */
    EnaSupport?: boolean;
    /**
     * <p>The full path to your AMI manifest in Amazon S3 storage. The specified bucket must have the
     *    		<code>aws-exec-read</code> canned access control list (ACL) to ensure that it can be accessed
     *    		by Amazon EC2. For more information, see <a href="https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#canned-acl">Canned ACLs</a> in the
     *    		<i>Amazon S3 Service Developer Guide</i>.</p>
     */
    ImageLocation?: string;
    /**
     * <p>The ID of the kernel.</p>
     */
    KernelId?: string;
    /**
     * <p>A name for your AMI.</p>
     *          <p>Constraints: 3-128 alphanumeric characters, parentheses (()), square brackets ([]), spaces ( ), periods (.), slashes (/), dashes (-), single quotes ('), at-signs (@), or underscores(_)</p>
     */
    Name: string | undefined;
    /**
     * <p>The ID of the RAM disk.</p>
     */
    RamdiskId?: string;
    /**
     * <p>The device name of the root device volume (for example, <code>/dev/sda1</code>).</p>
     */
    RootDeviceName?: string;
    /**
     * <p>Set to <code>simple</code> to enable enhanced networking with the Intel 82599 Virtual Function interface for the AMI and any instances that you launch from the AMI.</p>
     *          <p>There is no way to disable <code>sriovNetSupport</code> at this time.</p>
     *          <p>This option is supported only for HVM AMIs. Specifying this option with a PV AMI can make instances launched from the AMI unreachable.</p>
     */
    SriovNetSupport?: string;
    /**
     * <p>The type of virtualization (<code>hvm</code> | <code>paravirtual</code>).</p>
     *          <p>Default: <code>paravirtual</code>
     *          </p>
     */
    VirtualizationType?: string;
}
export declare namespace RegisterImageRequest {
    const filterSensitiveLog: (obj: RegisterImageRequest) => any;
    const isa: (o: any) => o is RegisterImageRequest;
}
/**
 * <p>Contains the output of RegisterImage.</p>
 */
export interface RegisterImageResult {
    __type?: "RegisterImageResult";
    /**
     * <p>The ID of the newly registered AMI.</p>
     */
    ImageId?: string;
}
export declare namespace RegisterImageResult {
    const filterSensitiveLog: (obj: RegisterImageResult) => any;
    const isa: (o: any) => o is RegisterImageResult;
}
export interface RegisterTransitGatewayMulticastGroupMembersRequest {
    __type?: "RegisterTransitGatewayMulticastGroupMembersRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IP address assigned to the  transit gateway multicast group.</p>
     */
    GroupIpAddress?: string;
    /**
     * <p>The group members' network interface IDs to register with the  transit gateway multicast group.</p>
     */
    NetworkInterfaceIds?: string[];
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace RegisterTransitGatewayMulticastGroupMembersRequest {
    const filterSensitiveLog: (obj: RegisterTransitGatewayMulticastGroupMembersRequest) => any;
    const isa: (o: any) => o is RegisterTransitGatewayMulticastGroupMembersRequest;
}
export interface RegisterTransitGatewayMulticastGroupMembersResult {
    __type?: "RegisterTransitGatewayMulticastGroupMembersResult";
    /**
     * <p>Information about the registered  transit gateway multicast group members.</p>
     */
    RegisteredMulticastGroupMembers?: TransitGatewayMulticastRegisteredGroupMembers;
}
export declare namespace RegisterTransitGatewayMulticastGroupMembersResult {
    const filterSensitiveLog: (obj: RegisterTransitGatewayMulticastGroupMembersResult) => any;
    const isa: (o: any) => o is RegisterTransitGatewayMulticastGroupMembersResult;
}
export interface RegisterTransitGatewayMulticastGroupSourcesRequest {
    __type?: "RegisterTransitGatewayMulticastGroupSourcesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IP address assigned to the  transit gateway multicast group.</p>
     */
    GroupIpAddress?: string;
    /**
     * <p>The group sources' network interface IDs to register with the  transit gateway multicast group.</p>
     */
    NetworkInterfaceIds?: string[];
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace RegisterTransitGatewayMulticastGroupSourcesRequest {
    const filterSensitiveLog: (obj: RegisterTransitGatewayMulticastGroupSourcesRequest) => any;
    const isa: (o: any) => o is RegisterTransitGatewayMulticastGroupSourcesRequest;
}
export interface RegisterTransitGatewayMulticastGroupSourcesResult {
    __type?: "RegisterTransitGatewayMulticastGroupSourcesResult";
    /**
     * <p>Information about the  transit gateway multicast group sources.</p>
     */
    RegisteredMulticastGroupSources?: TransitGatewayMulticastRegisteredGroupSources;
}
export declare namespace RegisterTransitGatewayMulticastGroupSourcesResult {
    const filterSensitiveLog: (obj: RegisterTransitGatewayMulticastGroupSourcesResult) => any;
    const isa: (o: any) => o is RegisterTransitGatewayMulticastGroupSourcesResult;
}
export interface RejectTransitGatewayPeeringAttachmentRequest {
    __type?: "RejectTransitGatewayPeeringAttachmentRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the transit gateway peering attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
}
export declare namespace RejectTransitGatewayPeeringAttachmentRequest {
    const filterSensitiveLog: (obj: RejectTransitGatewayPeeringAttachmentRequest) => any;
    const isa: (o: any) => o is RejectTransitGatewayPeeringAttachmentRequest;
}
export interface RejectTransitGatewayPeeringAttachmentResult {
    __type?: "RejectTransitGatewayPeeringAttachmentResult";
    /**
     * <p>The transit gateway peering attachment.</p>
     */
    TransitGatewayPeeringAttachment?: TransitGatewayPeeringAttachment;
}
export declare namespace RejectTransitGatewayPeeringAttachmentResult {
    const filterSensitiveLog: (obj: RejectTransitGatewayPeeringAttachmentResult) => any;
    const isa: (o: any) => o is RejectTransitGatewayPeeringAttachmentResult;
}
export interface RejectTransitGatewayVpcAttachmentRequest {
    __type?: "RejectTransitGatewayVpcAttachmentRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId: string | undefined;
}
export declare namespace RejectTransitGatewayVpcAttachmentRequest {
    const filterSensitiveLog: (obj: RejectTransitGatewayVpcAttachmentRequest) => any;
    const isa: (o: any) => o is RejectTransitGatewayVpcAttachmentRequest;
}
export interface RejectTransitGatewayVpcAttachmentResult {
    __type?: "RejectTransitGatewayVpcAttachmentResult";
    /**
     * <p>Information about the attachment.</p>
     */
    TransitGatewayVpcAttachment?: TransitGatewayVpcAttachment;
}
export declare namespace RejectTransitGatewayVpcAttachmentResult {
    const filterSensitiveLog: (obj: RejectTransitGatewayVpcAttachmentResult) => any;
    const isa: (o: any) => o is RejectTransitGatewayVpcAttachmentResult;
}
export interface RejectVpcEndpointConnectionsRequest {
    __type?: "RejectVpcEndpointConnectionsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the service.</p>
     */
    ServiceId: string | undefined;
    /**
     * <p>The IDs of one or more VPC endpoints.</p>
     */
    VpcEndpointIds: string[] | undefined;
}
export declare namespace RejectVpcEndpointConnectionsRequest {
    const filterSensitiveLog: (obj: RejectVpcEndpointConnectionsRequest) => any;
    const isa: (o: any) => o is RejectVpcEndpointConnectionsRequest;
}
export interface RejectVpcEndpointConnectionsResult {
    __type?: "RejectVpcEndpointConnectionsResult";
    /**
     * <p>Information about the endpoints that were not rejected, if applicable.</p>
     */
    Unsuccessful?: UnsuccessfulItem[];
}
export declare namespace RejectVpcEndpointConnectionsResult {
    const filterSensitiveLog: (obj: RejectVpcEndpointConnectionsResult) => any;
    const isa: (o: any) => o is RejectVpcEndpointConnectionsResult;
}
export interface RejectVpcPeeringConnectionRequest {
    __type?: "RejectVpcPeeringConnectionRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the VPC peering connection.</p>
     */
    VpcPeeringConnectionId: string | undefined;
}
export declare namespace RejectVpcPeeringConnectionRequest {
    const filterSensitiveLog: (obj: RejectVpcPeeringConnectionRequest) => any;
    const isa: (o: any) => o is RejectVpcPeeringConnectionRequest;
}
export interface RejectVpcPeeringConnectionResult {
    __type?: "RejectVpcPeeringConnectionResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace RejectVpcPeeringConnectionResult {
    const filterSensitiveLog: (obj: RejectVpcPeeringConnectionResult) => any;
    const isa: (o: any) => o is RejectVpcPeeringConnectionResult;
}
export interface ReleaseAddressRequest {
    __type?: "ReleaseAddressRequest";
    /**
     * <p>[EC2-VPC] The allocation ID. Required for EC2-VPC.</p>
     */
    AllocationId?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The location that the IP address is released from.</p>
     *          <p>If you provide an incorrect network border group, you will receive an <code>InvalidAddress.NotFound</code> error. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/errors-overview.html">Error Codes</a>.</p>
     *          <note>
     *             <p>You cannot use a network border group with EC2 Classic. If you attempt this operation on EC2 classic, you will receive an <code>InvalidParameterCombination</code> error. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/errors-overview.html">Error Codes</a>.</p>
     *          </note>
     */
    NetworkBorderGroup?: string;
    /**
     * <p>[EC2-Classic] The Elastic IP address. Required for EC2-Classic.</p>
     */
    PublicIp?: string;
}
export declare namespace ReleaseAddressRequest {
    const filterSensitiveLog: (obj: ReleaseAddressRequest) => any;
    const isa: (o: any) => o is ReleaseAddressRequest;
}
export interface ReleaseHostsRequest {
    __type?: "ReleaseHostsRequest";
    /**
     * <p>The IDs of the Dedicated Hosts to release.</p>
     */
    HostIds: string[] | undefined;
}
export declare namespace ReleaseHostsRequest {
    const filterSensitiveLog: (obj: ReleaseHostsRequest) => any;
    const isa: (o: any) => o is ReleaseHostsRequest;
}
export interface ReleaseHostsResult {
    __type?: "ReleaseHostsResult";
    /**
     * <p>The IDs of the Dedicated Hosts that were successfully released.</p>
     */
    Successful?: string[];
    /**
     * <p>The IDs of the Dedicated Hosts that could not be released, including an error
     *             message.</p>
     */
    Unsuccessful?: UnsuccessfulItem[];
}
export declare namespace ReleaseHostsResult {
    const filterSensitiveLog: (obj: ReleaseHostsResult) => any;
    const isa: (o: any) => o is ReleaseHostsResult;
}
export interface ReplaceIamInstanceProfileAssociationRequest {
    __type?: "ReplaceIamInstanceProfileAssociationRequest";
    /**
     * <p>The ID of the existing IAM instance profile association.</p>
     */
    AssociationId: string | undefined;
    /**
     * <p>The IAM instance profile.</p>
     */
    IamInstanceProfile: IamInstanceProfileSpecification | undefined;
}
export declare namespace ReplaceIamInstanceProfileAssociationRequest {
    const filterSensitiveLog: (obj: ReplaceIamInstanceProfileAssociationRequest) => any;
    const isa: (o: any) => o is ReplaceIamInstanceProfileAssociationRequest;
}
export interface ReplaceIamInstanceProfileAssociationResult {
    __type?: "ReplaceIamInstanceProfileAssociationResult";
    /**
     * <p>Information about the IAM instance profile association.</p>
     */
    IamInstanceProfileAssociation?: IamInstanceProfileAssociation;
}
export declare namespace ReplaceIamInstanceProfileAssociationResult {
    const filterSensitiveLog: (obj: ReplaceIamInstanceProfileAssociationResult) => any;
    const isa: (o: any) => o is ReplaceIamInstanceProfileAssociationResult;
}
export interface ReplaceNetworkAclAssociationRequest {
    __type?: "ReplaceNetworkAclAssociationRequest";
    /**
     * <p>The ID of the current association between the original network ACL and the subnet.</p>
     */
    AssociationId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the new network ACL to associate with the subnet.</p>
     */
    NetworkAclId: string | undefined;
}
export declare namespace ReplaceNetworkAclAssociationRequest {
    const filterSensitiveLog: (obj: ReplaceNetworkAclAssociationRequest) => any;
    const isa: (o: any) => o is ReplaceNetworkAclAssociationRequest;
}
export interface ReplaceNetworkAclAssociationResult {
    __type?: "ReplaceNetworkAclAssociationResult";
    /**
     * <p>The ID of the new association.</p>
     */
    NewAssociationId?: string;
}
export declare namespace ReplaceNetworkAclAssociationResult {
    const filterSensitiveLog: (obj: ReplaceNetworkAclAssociationResult) => any;
    const isa: (o: any) => o is ReplaceNetworkAclAssociationResult;
}
export interface ReplaceNetworkAclEntryRequest {
    __type?: "ReplaceNetworkAclEntryRequest";
    /**
     * <p>The IPv4 network range to allow or deny, in CIDR notation (for example
     *                 <code>172.16.0.0/24</code>).</p>
     */
    CidrBlock?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Indicates whether to replace the egress rule.</p>
     * 		       <p>Default: If no value is specified, we replace the ingress rule.</p>
     */
    Egress: boolean | undefined;
    /**
     * <p>ICMP protocol: The ICMP or ICMPv6 type and code. Required if specifying protocol
     * 		        1 (ICMP) or protocol 58 (ICMPv6) with an IPv6 CIDR block.</p>
     */
    IcmpTypeCode?: IcmpTypeCode;
    /**
     * <p>The IPv6 network range to allow or deny, in CIDR notation (for example
     *                 <code>2001:bd8:1234:1a00::/64</code>).</p>
     */
    Ipv6CidrBlock?: string;
    /**
     * <p>The ID of the ACL.</p>
     */
    NetworkAclId: string | undefined;
    /**
     * <p>TCP or UDP protocols: The range of ports the rule applies to.
     * 		        Required if specifying protocol 6 (TCP) or 17 (UDP).</p>
     */
    PortRange?: PortRange;
    /**
     * <p>The protocol number. A value of "-1" means all protocols. If you specify "-1" or a
     *            protocol number other than "6" (TCP), "17" (UDP), or "1" (ICMP), traffic on all ports is
     *            allowed, regardless of any ports or ICMP types or codes that you specify. If you specify
     *            protocol "58" (ICMPv6) and specify an IPv4 CIDR block, traffic for all ICMP types and
     *            codes allowed, regardless of any that you specify. If you specify protocol "58" (ICMPv6)
     *            and specify an IPv6 CIDR block, you must specify an ICMP type and code.</p>
     */
    Protocol: string | undefined;
    /**
     * <p>Indicates whether to allow or deny the traffic that matches the rule.</p>
     */
    RuleAction: RuleAction | string | undefined;
    /**
     * <p>The rule number of the entry to replace.</p>
     */
    RuleNumber: number | undefined;
}
export declare namespace ReplaceNetworkAclEntryRequest {
    const filterSensitiveLog: (obj: ReplaceNetworkAclEntryRequest) => any;
    const isa: (o: any) => o is ReplaceNetworkAclEntryRequest;
}
export interface ReplaceRouteRequest {
    __type?: "ReplaceRouteRequest";
    /**
     * <p>The IPv4 CIDR address block used for the destination match. The value that you
     * 			provide must match the CIDR of an existing route in the table.</p>
     */
    DestinationCidrBlock?: string;
    /**
     * <p>The IPv6 CIDR address block used for the destination match. The value that you
     * 			provide must match the CIDR of an existing route in the table.</p>
     */
    DestinationIpv6CidrBlock?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>[IPv6 traffic only] The ID of an egress-only internet gateway.</p>
     */
    EgressOnlyInternetGatewayId?: string;
    /**
     * <p>The ID of an internet gateway or virtual private gateway.</p>
     */
    GatewayId?: string;
    /**
     * <p>The ID of a NAT instance in your VPC.</p>
     */
    InstanceId?: string;
    /**
     * <p>The ID of the local gateway.</p>
     */
    LocalGatewayId?: string;
    /**
     * <p>Specifies whether to reset the local route to its default target (<code>local</code>).</p>
     */
    LocalTarget?: boolean;
    /**
     * <p>[IPv4 traffic only] The ID of a NAT gateway.</p>
     */
    NatGatewayId?: string;
    /**
     * <p>The ID of a network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The ID of the route table.</p>
     */
    RouteTableId: string | undefined;
    /**
     * <p>The ID of a transit gateway.</p>
     */
    TransitGatewayId?: string;
    /**
     * <p>The ID of a VPC peering connection.</p>
     */
    VpcPeeringConnectionId?: string;
}
export declare namespace ReplaceRouteRequest {
    const filterSensitiveLog: (obj: ReplaceRouteRequest) => any;
    const isa: (o: any) => o is ReplaceRouteRequest;
}
export interface ReplaceRouteTableAssociationRequest {
    __type?: "ReplaceRouteTableAssociationRequest";
    /**
     * <p>The association ID.</p>
     */
    AssociationId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the new route table to associate with the subnet.</p>
     */
    RouteTableId: string | undefined;
}
export declare namespace ReplaceRouteTableAssociationRequest {
    const filterSensitiveLog: (obj: ReplaceRouteTableAssociationRequest) => any;
    const isa: (o: any) => o is ReplaceRouteTableAssociationRequest;
}
export interface ReplaceRouteTableAssociationResult {
    __type?: "ReplaceRouteTableAssociationResult";
    /**
     * <p>The state of the association.</p>
     */
    AssociationState?: RouteTableAssociationState;
    /**
     * <p>The ID of the new association.</p>
     */
    NewAssociationId?: string;
}
export declare namespace ReplaceRouteTableAssociationResult {
    const filterSensitiveLog: (obj: ReplaceRouteTableAssociationResult) => any;
    const isa: (o: any) => o is ReplaceRouteTableAssociationResult;
}
export interface ReplaceTransitGatewayRouteRequest {
    __type?: "ReplaceTransitGatewayRouteRequest";
    /**
     * <p>Indicates whether traffic matching this route is to be dropped.</p>
     */
    Blackhole?: boolean;
    /**
     * <p>The CIDR range used for the destination match. Routing decisions are based on the most specific match.</p>
     */
    DestinationCidrBlock: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
    /**
     * <p>The ID of the route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace ReplaceTransitGatewayRouteRequest {
    const filterSensitiveLog: (obj: ReplaceTransitGatewayRouteRequest) => any;
    const isa: (o: any) => o is ReplaceTransitGatewayRouteRequest;
}
export interface ReplaceTransitGatewayRouteResult {
    __type?: "ReplaceTransitGatewayRouteResult";
    /**
     * <p>Information about the modified route.</p>
     */
    Route?: TransitGatewayRoute;
}
export declare namespace ReplaceTransitGatewayRouteResult {
    const filterSensitiveLog: (obj: ReplaceTransitGatewayRouteResult) => any;
    const isa: (o: any) => o is ReplaceTransitGatewayRouteResult;
}
export declare type ReportInstanceReasonCodes = "instance-stuck-in-state" | "not-accepting-credentials" | "other" | "password-not-available" | "performance-ebs-volume" | "performance-instance-store" | "performance-network" | "performance-other" | "unresponsive";
export interface ReportInstanceStatusRequest {
    __type?: "ReportInstanceStatusRequest";
    /**
     * <p>Descriptive text about the health state of your instance.</p>
     */
    Description?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The time at which the reported instance health state ended.</p>
     */
    EndTime?: Date;
    /**
     * <p>The instances.</p>
     */
    Instances: string[] | undefined;
    /**
     * <p>The reason codes that describe the health state of your instance.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                     <code>instance-stuck-in-state</code>: My instance is stuck in a state.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>unresponsive</code>: My instance is unresponsive.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>not-accepting-credentials</code>: My instance is not accepting my
     *                     credentials.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>password-not-available</code>: A password is not available for my
     *                     instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>performance-network</code>: My instance is experiencing performance
     *                     problems that I believe are network related.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>performance-instance-store</code>: My instance is experiencing performance
     *                     problems that I believe are related to the instance stores.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>performance-ebs-volume</code>: My instance is experiencing performance
     *                     problems that I believe are related to an EBS volume.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>performance-other</code>: My instance is experiencing performance
     *                     problems.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                     <code>other</code>: [explain using the description parameter]</p>
     *             </li>
     *          </ul>
     */
    ReasonCodes: (ReportInstanceReasonCodes | string)[] | undefined;
    /**
     * <p>The time at which the reported instance health state began.</p>
     */
    StartTime?: Date;
    /**
     * <p>The status of all instances listed.</p>
     */
    Status: ReportStatusType | string | undefined;
}
export declare namespace ReportInstanceStatusRequest {
    const filterSensitiveLog: (obj: ReportInstanceStatusRequest) => any;
    const isa: (o: any) => o is ReportInstanceStatusRequest;
}
export declare type ReportStatusType = "impaired" | "ok";
/**
 * <p>The information to include in the launch template.</p>
 */
export interface RequestLaunchTemplateData {
    __type?: "RequestLaunchTemplateData";
    /**
     * <p>The block device mapping.</p>
     */
    BlockDeviceMappings?: LaunchTemplateBlockDeviceMappingRequest[];
    /**
     * <p>The Capacity Reservation targeting option. If you do not specify this parameter, the
     * 			instance's Capacity Reservation preference defaults to <code>open</code>, which enables it
     * 			to run in any open Capacity Reservation that has matching attributes (instance type,
     * 			platform, Availability Zone).</p>
     */
    CapacityReservationSpecification?: LaunchTemplateCapacityReservationSpecificationRequest;
    /**
     * <p>The CPU options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html">Optimizing CPU Options</a> in the <i>Amazon Elastic Compute Cloud User
     * 				Guide</i>.</p>
     */
    CpuOptions?: LaunchTemplateCpuOptionsRequest;
    /**
     * <p>The credit option for CPU usage of the instance. Valid for T2 or T3 instances
     *             only.</p>
     */
    CreditSpecification?: CreditSpecificationRequest;
    /**
     * <p>If you set this parameter to <code>true</code>, you can't terminate the instance using
     *     		the Amazon EC2 console, CLI, or API; otherwise, you can. To change this attribute after launch,
     *     		use <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ModifyInstanceAttribute.html">ModifyInstanceAttribute</a>.
     *     		Alternatively, if you set <code>InstanceInitiatedShutdownBehavior</code> to
     *     		<code>terminate</code>, you can terminate the instance by running the shutdown
     *     		command from the instance.</p>
     */
    DisableApiTermination?: boolean;
    /**
     * <p>Indicates whether the instance is optimized for Amazon EBS I/O. This optimization
     *             provides dedicated throughput to Amazon EBS and an optimized configuration stack to
     *             provide optimal Amazon EBS I/O performance. This optimization isn't available with all
     *             instance types. Additional usage charges apply when using an EBS-optimized
     *             instance.</p>
     */
    EbsOptimized?: boolean;
    /**
     * <p>An elastic GPU to associate with the instance.</p>
     */
    ElasticGpuSpecifications?: ElasticGpuSpecification[];
    /**
     * <p>
     *             The elastic inference accelerator for the instance.
     *         </p>
     */
    ElasticInferenceAccelerators?: LaunchTemplateElasticInferenceAccelerator[];
    /**
     * <p>Indicates whether an instance is enabled for hibernation. This parameter is valid only
     *             if the instance meets the <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html#hibernating-prerequisites">hibernation
     *                 prerequisites</a>. For
     *             more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate Your Instance</a> in the
     *                 <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    HibernationOptions?: LaunchTemplateHibernationOptionsRequest;
    /**
     * <p>The IAM instance profile.</p>
     */
    IamInstanceProfile?: LaunchTemplateIamInstanceProfileSpecificationRequest;
    /**
     * <p>The ID of the AMI.</p>
     */
    ImageId?: string;
    /**
     * <p>Indicates whether an instance stops or terminates when you initiate shutdown from
     *             the instance (using the operating system command for system shutdown).</p>
     *         <p>Default: <code>stop</code>
     *         </p>
     */
    InstanceInitiatedShutdownBehavior?: ShutdownBehavior | string;
    /**
     * <p>The market (purchasing) option for the instances.</p>
     */
    InstanceMarketOptions?: LaunchTemplateInstanceMarketOptionsRequest;
    /**
     * <p>The instance type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the
     *                 <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The ID of the kernel.</p>
     *         <important>
     *             <p>We recommend that you use PV-GRUB instead of kernels and RAM disks. For more
     *                 information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html">User Provided
     *                     Kernels</a> in the <i>Amazon Elastic Compute Cloud User
     *                     Guide</i>.</p>
     *         </important>
     */
    KernelId?: string;
    /**
     * <p>The name of the key pair. You can create a key pair using <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateKeyPair.html">CreateKeyPair</a> or <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ImportKeyPair.html">ImportKeyPair</a>.</p>
     *         <important>
     *             <p>If you do not specify a key pair, you can't connect to the instance unless you
     *                 choose an AMI that is configured to allow users another way to log in.</p>
     *         </important>
     */
    KeyName?: string;
    /**
     * <p>The license configurations.</p>
     */
    LicenseSpecifications?: LaunchTemplateLicenseConfigurationRequest[];
    /**
     * <p>The metadata options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html">Instance Metadata and User Data</a> in the
     *             <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    MetadataOptions?: LaunchTemplateInstanceMetadataOptionsRequest;
    /**
     * <p>The monitoring for the instance.</p>
     */
    Monitoring?: LaunchTemplatesMonitoringRequest;
    /**
     * <p>One or more network interfaces. If you specify a network interface, you must specify
     *             any security groups and subnets as part of the network interface.</p>
     */
    NetworkInterfaces?: LaunchTemplateInstanceNetworkInterfaceSpecificationRequest[];
    /**
     * <p>The placement for the instance.</p>
     */
    Placement?: LaunchTemplatePlacementRequest;
    /**
     * <p>The ID of the RAM disk.</p>
     *         <important>
     *             <p>We recommend that you use PV-GRUB instead of kernels and RAM disks. For more
     *                 information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html">User Provided
     *                     Kernels</a> in the <i>Amazon Elastic Compute Cloud User
     *                     Guide</i>.</p>
     *         </important>
     */
    RamDiskId?: string;
    /**
     * <p>One or more security group IDs. You can create a security group using <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateSecurityGroup.html">CreateSecurityGroup</a>. You cannot specify both a security group ID and
     *             security name in the same request.</p>
     */
    SecurityGroupIds?: string[];
    /**
     * <p>[EC2-Classic, default VPC] One or more security group names. For a nondefault VPC,
     *             you must use security group IDs instead. You cannot specify both a security group ID and
     *             security name in the same request.</p>
     */
    SecurityGroups?: string[];
    /**
     * <p>The tags to apply to the resources during launch. You can only tag instances and
     *             volumes on launch. The specified tags are applied to all instances or volumes that are
     *             created during launch. To tag a resource after it has been created, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html">CreateTags</a>.</p>
     */
    TagSpecifications?: LaunchTemplateTagSpecificationRequest[];
    /**
     * <p>The Base64-encoded user data to make available to the instance. For more
     *             information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/user-data.html">Running Commands on Your Linux Instance
     *                 at Launch</a> (Linux) and <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/ec2-instance-metadata.html#instancedata-add-user-data">Adding User Data</a> (Windows).</p>
     */
    UserData?: string;
}
export declare namespace RequestLaunchTemplateData {
    const filterSensitiveLog: (obj: RequestLaunchTemplateData) => any;
    const isa: (o: any) => o is RequestLaunchTemplateData;
}
/**
 * <p>Contains the parameters for RequestSpotFleet.</p>
 */
export interface RequestSpotFleetRequest {
    __type?: "RequestSpotFleetRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *       and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *       Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The configuration for the Spot Fleet request.</p>
     */
    SpotFleetRequestConfig: SpotFleetRequestConfigData | undefined;
}
export declare namespace RequestSpotFleetRequest {
    const filterSensitiveLog: (obj: RequestSpotFleetRequest) => any;
    const isa: (o: any) => o is RequestSpotFleetRequest;
}
/**
 * <p>Contains the output of RequestSpotFleet.</p>
 */
export interface RequestSpotFleetResponse {
    __type?: "RequestSpotFleetResponse";
    /**
     * <p>The ID of the Spot Fleet request.</p>
     */
    SpotFleetRequestId?: string;
}
export declare namespace RequestSpotFleetResponse {
    const filterSensitiveLog: (obj: RequestSpotFleetResponse) => any;
    const isa: (o: any) => o is RequestSpotFleetResponse;
}
/**
 * <p>Contains the parameters for RequestSpotInstances.</p>
 */
export interface RequestSpotInstancesRequest {
    __type?: "RequestSpotInstancesRequest";
    /**
     * <p>The user-specified name for a logical grouping of requests.</p>
     *          <p>When you specify an Availability Zone group in a Spot Instance request, all Spot Instances in the request are launched in the same Availability Zone. Instance proximity is maintained with this parameter, but the choice of Availability Zone is not. The group applies only to requests for Spot Instances of the same instance type. Any additional Spot Instance requests that are specified with the same Availability Zone group name are launched in that same Availability Zone, as long as at least one instance from the group is still active.</p>
     *          <p>If there is no active instance running in the Availability Zone group that you specify for a new Spot Instance request (all instances are terminated, the request is expired, or the maximum price you specified falls below current Spot price), then Amazon EC2 launches the instance in any Availability Zone where the constraint can be met. Consequently, the subsequent set of Spot Instances could be placed in a different zone from the original request, even if you specified the same Availability Zone group.</p>
     *          <p>Default: Instances are launched in any available Availability Zone.</p>
     */
    AvailabilityZoneGroup?: string;
    /**
     * <p>The required duration for the Spot Instances (also known as Spot blocks), in minutes. This value must be a multiple of 60 (60, 120, 180, 240, 300, or 360).</p>
     *          <p>The duration period starts as soon as your Spot Instance receives its instance ID. At
     *             the end of the duration period, Amazon EC2 marks the Spot Instance for termination and
     *             provides a Spot Instance termination notice, which gives the instance a two-minute
     *             warning before it terminates.</p>
     *          <p>You can't specify an Availability Zone group or a launch group if you specify a
     *             duration.</p>
     */
    BlockDurationMinutes?: number;
    /**
     * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.
     *        For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html">How to Ensure Idempotency</a>
     *        in the <i>Amazon EC2 User Guide for Linux Instances</i>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The maximum number of Spot Instances to launch.</p>
     *          <p>Default: 1</p>
     */
    InstanceCount?: number;
    /**
     * <p>The behavior when a Spot Instance is interrupted. The default is <code>terminate</code>.</p>
     */
    InstanceInterruptionBehavior?: InstanceInterruptionBehavior | string;
    /**
     * <p>The instance launch group. Launch groups are Spot Instances that launch together and terminate together.</p>
     *          <p>Default: Instances are launched and terminated individually</p>
     */
    LaunchGroup?: string;
    /**
     * <p>The launch specification.</p>
     */
    LaunchSpecification?: RequestSpotLaunchSpecification;
    /**
     * <p>The maximum price per hour that you are willing to pay for a Spot Instance. The default is the On-Demand price.</p>
     */
    SpotPrice?: string;
    /**
     * <p>The Spot Instance request type.</p>
     *          <p>Default: <code>one-time</code>
     *          </p>
     */
    Type?: SpotInstanceType | string;
    /**
     * <p>The start date of the request. If this is a one-time request, the request becomes active at this date and time and remains active until all instances launch, the request expires, or the request is canceled. If the request is persistent, the request becomes active at this date and time and remains active until it expires or is canceled.</p>
     *    	     <p>The specified start date and time cannot be equal to the current date and time. You must specify a start date and time that occurs after the current date and time.</p>
     */
    ValidFrom?: Date;
    /**
     * <p>The end date of the request. If this is a one-time request, the request remains active until all instances launch, the request is canceled, or this date is reached.
     *          If the request is persistent, it remains active until it is canceled or this date is reached. The default end date is 7 days from the current date.</p>
     */
    ValidUntil?: Date;
}
export declare namespace RequestSpotInstancesRequest {
    const filterSensitiveLog: (obj: RequestSpotInstancesRequest) => any;
    const isa: (o: any) => o is RequestSpotInstancesRequest;
}
/**
 * <p>Contains the output of RequestSpotInstances.</p>
 */
export interface RequestSpotInstancesResult {
    __type?: "RequestSpotInstancesResult";
    /**
     * <p>One or more Spot Instance requests.</p>
     */
    SpotInstanceRequests?: SpotInstanceRequest[];
}
export declare namespace RequestSpotInstancesResult {
    const filterSensitiveLog: (obj: RequestSpotInstancesResult) => any;
    const isa: (o: any) => o is RequestSpotInstancesResult;
}
/**
 * <p>Describes the launch specification for an instance.</p>
 */
export interface RequestSpotLaunchSpecification {
    __type?: "RequestSpotLaunchSpecification";
    /**
     * <p>Deprecated.</p>
     */
    AddressingType?: string;
    /**
     * <p>One or more block device mapping entries. You can't specify both a snapshot ID and an encryption value.
     *            This is because only blank volumes can be encrypted on creation. If a snapshot is the basis for a volume,
     *            it is not blank and its encryption status is used for the volume encryption status.</p>
     */
    BlockDeviceMappings?: BlockDeviceMapping[];
    /**
     * <p>Indicates whether the instance is optimized for EBS I/O. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal EBS I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS Optimized instance.</p>
     *          <p>Default: <code>false</code>
     *          </p>
     */
    EbsOptimized?: boolean;
    /**
     * <p>The IAM instance profile.</p>
     */
    IamInstanceProfile?: IamInstanceProfileSpecification;
    /**
     * <p>The ID of the AMI.</p>
     */
    ImageId?: string;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The ID of the kernel.</p>
     */
    KernelId?: string;
    /**
     * <p>The name of the key pair.</p>
     */
    KeyName?: string;
    /**
     * <p>Indicates whether basic or detailed monitoring is enabled for the instance.</p>
     *          <p>Default: Disabled</p>
     */
    Monitoring?: RunInstancesMonitoringEnabled;
    /**
     * <p>One or more network interfaces. If you specify a network interface, you must specify
     *            subnet IDs and security group IDs using the network interface.</p>
     */
    NetworkInterfaces?: InstanceNetworkInterfaceSpecification[];
    /**
     * <p>The placement information for the instance.</p>
     */
    Placement?: SpotPlacement;
    /**
     * <p>The ID of the RAM disk.</p>
     */
    RamdiskId?: string;
    /**
     * <p>One or more security group IDs.</p>
     */
    SecurityGroupIds?: string[];
    /**
     * <p>One or more security groups. When requesting instances in a VPC, you must specify the IDs of the security groups. When requesting instances in EC2-Classic, you can specify the names or the IDs of the security groups.</p>
     */
    SecurityGroups?: string[];
    /**
     * <p>The IDs of the subnets in which to launch the instance. To specify multiple subnets, separate
     *            them using commas; for example, "subnet-1234abcdeexample1, subnet-0987cdef6example2".</p>
     */
    SubnetId?: string;
    /**
     * <p>The Base64-encoded user data for the instance. User data is limited to 16 KB.</p>
     */
    UserData?: string;
}
export declare namespace RequestSpotLaunchSpecification {
    const filterSensitiveLog: (obj: RequestSpotLaunchSpecification) => any;
    const isa: (o: any) => o is RequestSpotLaunchSpecification;
}
/**
 * <p>Describes a reservation.</p>
 */
export interface Reservation {
    __type?: "Reservation";
    /**
     * <p>[EC2-Classic only] The security groups.</p>
     */
    Groups?: GroupIdentifier[];
    /**
     * <p>The instances.</p>
     */
    Instances?: Instance[];
    /**
     * <p>The ID of the AWS account that owns the reservation.</p>
     */
    OwnerId?: string;
    /**
     * <p>The ID of the requester that launched the instances on your behalf (for example, AWS
     *             Management Console or Auto Scaling).</p>
     */
    RequesterId?: string;
    /**
     * <p>The ID of the reservation.</p>
     */
    ReservationId?: string;
}
export declare namespace Reservation {
    const filterSensitiveLog: (obj: Reservation) => any;
    const isa: (o: any) => o is Reservation;
}
export declare enum ReservationState {
    ACTIVE = "active",
    PAYMENT_FAILED = "payment-failed",
    PAYMENT_PENDING = "payment-pending",
    RETIRED = "retired"
}
/**
 * <p>The cost associated with the Reserved Instance.</p>
 */
export interface ReservationValue {
    __type?: "ReservationValue";
    /**
     * <p>The hourly rate of the reservation.</p>
     */
    HourlyPrice?: string;
    /**
     * <p>The balance of the total value (the sum of remainingUpfrontValue + hourlyPrice * number of hours remaining).</p>
     */
    RemainingTotalValue?: string;
    /**
     * <p>The remaining upfront cost of the reservation.</p>
     */
    RemainingUpfrontValue?: string;
}
export declare namespace ReservationValue {
    const filterSensitiveLog: (obj: ReservationValue) => any;
    const isa: (o: any) => o is ReservationValue;
}
/**
 * <p>Describes the limit price of a Reserved Instance offering.</p>
 */
export interface ReservedInstanceLimitPrice {
    __type?: "ReservedInstanceLimitPrice";
    /**
     * <p>Used for Reserved Instance Marketplace offerings. Specifies the limit price on the total order (instanceCount * price).</p>
     */
    Amount?: number;
    /**
     * <p>The currency in which the <code>limitPrice</code> amount is specified.
     * 				At this time, the only supported currency is <code>USD</code>.</p>
     */
    CurrencyCode?: CurrencyCodeValues | string;
}
export declare namespace ReservedInstanceLimitPrice {
    const filterSensitiveLog: (obj: ReservedInstanceLimitPrice) => any;
    const isa: (o: any) => o is ReservedInstanceLimitPrice;
}
/**
 * <p>The total value of the Convertible Reserved Instance.</p>
 */
export interface ReservedInstanceReservationValue {
    __type?: "ReservedInstanceReservationValue";
    /**
     * <p>The total value of the Convertible Reserved Instance that you are exchanging.</p>
     */
    ReservationValue?: ReservationValue;
    /**
     * <p>The ID of the Convertible Reserved Instance that you are exchanging.</p>
     */
    ReservedInstanceId?: string;
}
export declare namespace ReservedInstanceReservationValue {
    const filterSensitiveLog: (obj: ReservedInstanceReservationValue) => any;
    const isa: (o: any) => o is ReservedInstanceReservationValue;
}
/**
 * <p>Describes a Reserved Instance.</p>
 */
export interface ReservedInstances {
    __type?: "ReservedInstances";
    /**
     * <p>The Availability Zone in which the Reserved Instance can be used.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The currency of the Reserved Instance. It's specified using ISO 4217 standard currency codes.
     * 				At this time, the only supported currency is <code>USD</code>.</p>
     */
    CurrencyCode?: CurrencyCodeValues | string;
    /**
     * <p>The duration of the Reserved Instance, in seconds.</p>
     */
    Duration?: number;
    /**
     * <p>The time when the Reserved Instance expires.</p>
     */
    End?: Date;
    /**
     * <p>The purchase price of the Reserved Instance.</p>
     */
    FixedPrice?: number;
    /**
     * <p>The number of reservations purchased.</p>
     */
    InstanceCount?: number;
    /**
     * <p>The tenancy of the instance.</p>
     */
    InstanceTenancy?: Tenancy | string;
    /**
     * <p>The instance type on which the Reserved Instance can be used.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The offering class of the Reserved Instance.</p>
     */
    OfferingClass?: OfferingClassType | string;
    /**
     * <p>The Reserved Instance offering type.</p>
     */
    OfferingType?: OfferingTypeValues | string;
    /**
     * <p>The Reserved Instance product platform description.</p>
     */
    ProductDescription?: RIProductDescription | string;
    /**
     * <p>The recurring charge tag assigned to the resource.</p>
     */
    RecurringCharges?: RecurringCharge[];
    /**
     * <p>The ID of the Reserved Instance.</p>
     */
    ReservedInstancesId?: string;
    /**
     * <p>The scope of the Reserved Instance.</p>
     */
    Scope?: Scope | string;
    /**
     * <p>The date and time the Reserved Instance started.</p>
     */
    Start?: Date;
    /**
     * <p>The state of the Reserved Instance purchase.</p>
     */
    State?: ReservedInstanceState | string;
    /**
     * <p>Any tags assigned to the resource.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The usage price of the Reserved Instance, per hour.</p>
     */
    UsagePrice?: number;
}
export declare namespace ReservedInstances {
    const filterSensitiveLog: (obj: ReservedInstances) => any;
    const isa: (o: any) => o is ReservedInstances;
}
/**
 * <p>Describes the configuration settings for the modified Reserved Instances.</p>
 */
export interface ReservedInstancesConfiguration {
    __type?: "ReservedInstancesConfiguration";
    /**
     * <p>The Availability Zone for the modified Reserved Instances.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The number of modified Reserved Instances.</p>
     *          <note>
     *             <p>This is a required field for a request.</p>
     *          </note>
     */
    InstanceCount?: number;
    /**
     * <p>The instance type for the modified Reserved Instances.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The network platform of the modified Reserved Instances, which is either EC2-Classic or EC2-VPC.</p>
     */
    Platform?: string;
    /**
     * <p>Whether the Reserved Instance is applied to instances in a Region or instances in a specific Availability Zone.</p>
     */
    Scope?: Scope | string;
}
export declare namespace ReservedInstancesConfiguration {
    const filterSensitiveLog: (obj: ReservedInstancesConfiguration) => any;
    const isa: (o: any) => o is ReservedInstancesConfiguration;
}
/**
 * <p>Describes the ID of a Reserved Instance.</p>
 */
export interface ReservedInstancesId {
    __type?: "ReservedInstancesId";
    /**
     * <p>The ID of the Reserved Instance.</p>
     */
    ReservedInstancesId?: string;
}
export declare namespace ReservedInstancesId {
    const filterSensitiveLog: (obj: ReservedInstancesId) => any;
    const isa: (o: any) => o is ReservedInstancesId;
}
/**
 * <p>Describes a Reserved Instance listing.</p>
 */
export interface ReservedInstancesListing {
    __type?: "ReservedInstancesListing";
    /**
     * <p>A unique, case-sensitive key supplied by the client to ensure that the request is
     * 			idempotent. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The time the listing was created.</p>
     */
    CreateDate?: Date;
    /**
     * <p>The number of instances in this state.</p>
     */
    InstanceCounts?: InstanceCount[];
    /**
     * <p>The price of the Reserved Instance listing.</p>
     */
    PriceSchedules?: PriceSchedule[];
    /**
     * <p>The ID of the Reserved Instance.</p>
     */
    ReservedInstancesId?: string;
    /**
     * <p>The ID of the Reserved Instance listing.</p>
     */
    ReservedInstancesListingId?: string;
    /**
     * <p>The status of the Reserved Instance listing.</p>
     */
    Status?: ListingStatus | string;
    /**
     * <p>The reason for the current status of the Reserved Instance listing. The response can be blank.</p>
     */
    StatusMessage?: string;
    /**
     * <p>Any tags assigned to the resource.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The last modified timestamp of the listing.</p>
     */
    UpdateDate?: Date;
}
export declare namespace ReservedInstancesListing {
    const filterSensitiveLog: (obj: ReservedInstancesListing) => any;
    const isa: (o: any) => o is ReservedInstancesListing;
}
/**
 * <p>Describes a Reserved Instance modification.</p>
 */
export interface ReservedInstancesModification {
    __type?: "ReservedInstancesModification";
    /**
     * <p>A unique, case-sensitive key supplied by the client to ensure that the request is idempotent.
     * 			For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring
     * 				Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>The time when the modification request was created.</p>
     */
    CreateDate?: Date;
    /**
     * <p>The time for the modification to become effective.</p>
     */
    EffectiveDate?: Date;
    /**
     * <p>Contains target configurations along with their corresponding new Reserved Instance IDs.</p>
     */
    ModificationResults?: ReservedInstancesModificationResult[];
    /**
     * <p>The IDs of one or more Reserved Instances.</p>
     */
    ReservedInstancesIds?: ReservedInstancesId[];
    /**
     * <p>A unique ID for the Reserved Instance modification.</p>
     */
    ReservedInstancesModificationId?: string;
    /**
     * <p>The status of the Reserved Instances modification request.</p>
     */
    Status?: string;
    /**
     * <p>The reason for the status.</p>
     */
    StatusMessage?: string;
    /**
     * <p>The time when the modification request was last updated.</p>
     */
    UpdateDate?: Date;
}
export declare namespace ReservedInstancesModification {
    const filterSensitiveLog: (obj: ReservedInstancesModification) => any;
    const isa: (o: any) => o is ReservedInstancesModification;
}
/**
 * <p>Describes the modification request/s.</p>
 */
export interface ReservedInstancesModificationResult {
    __type?: "ReservedInstancesModificationResult";
    /**
     * <p>The ID for the Reserved Instances that were created as part of the modification request. This field is only available when the modification is fulfilled.</p>
     */
    ReservedInstancesId?: string;
    /**
     * <p>The target Reserved Instances configurations supplied as part of the modification request.</p>
     */
    TargetConfiguration?: ReservedInstancesConfiguration;
}
export declare namespace ReservedInstancesModificationResult {
    const filterSensitiveLog: (obj: ReservedInstancesModificationResult) => any;
    const isa: (o: any) => o is ReservedInstancesModificationResult;
}
/**
 * <p>Describes a Reserved Instance offering.</p>
 */
export interface ReservedInstancesOffering {
    __type?: "ReservedInstancesOffering";
    /**
     * <p>The Availability Zone in which the Reserved Instance can be used.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The currency of the Reserved Instance offering you are purchasing. It's
     * 				specified using ISO 4217 standard currency codes. At this time,
     * 				the only supported currency is <code>USD</code>.</p>
     */
    CurrencyCode?: CurrencyCodeValues | string;
    /**
     * <p>The duration of the Reserved Instance, in seconds.</p>
     */
    Duration?: number;
    /**
     * <p>The purchase price of the Reserved Instance.</p>
     */
    FixedPrice?: number;
    /**
     * <p>The tenancy of the instance.</p>
     */
    InstanceTenancy?: Tenancy | string;
    /**
     * <p>The instance type on which the Reserved Instance can be used.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>Indicates whether the offering is available through the Reserved Instance Marketplace (resale) or AWS.
     *         If it's a Reserved Instance Marketplace offering, this is <code>true</code>.</p>
     */
    Marketplace?: boolean;
    /**
     * <p>If <code>convertible</code> it can be exchanged for Reserved Instances of
     *       the same or higher monetary value, with different configurations. If <code>standard</code>, it is not
     *       possible to perform an exchange.</p>
     */
    OfferingClass?: OfferingClassType | string;
    /**
     * <p>The Reserved Instance offering type.</p>
     */
    OfferingType?: OfferingTypeValues | string;
    /**
     * <p>The pricing details of the Reserved Instance offering.</p>
     */
    PricingDetails?: PricingDetail[];
    /**
     * <p>The Reserved Instance product platform description.</p>
     */
    ProductDescription?: RIProductDescription | string;
    /**
     * <p>The recurring charge tag assigned to the resource.</p>
     */
    RecurringCharges?: RecurringCharge[];
    /**
     * <p>The ID of the Reserved Instance offering. This is the offering ID used in <a>GetReservedInstancesExchangeQuote</a>
     *      to confirm that an exchange can be made.</p>
     */
    ReservedInstancesOfferingId?: string;
    /**
     * <p>Whether the Reserved Instance is applied to instances in a Region or an Availability Zone.</p>
     */
    Scope?: Scope | string;
    /**
     * <p>The usage price of the Reserved Instance, per hour.</p>
     */
    UsagePrice?: number;
}
export declare namespace ReservedInstancesOffering {
    const filterSensitiveLog: (obj: ReservedInstancesOffering) => any;
    const isa: (o: any) => o is ReservedInstancesOffering;
}
export declare type ReservedInstanceState = "active" | "payment-failed" | "payment-pending" | "queued" | "queued-deleted" | "retired";
export interface ResetEbsDefaultKmsKeyIdRequest {
    __type?: "ResetEbsDefaultKmsKeyIdRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace ResetEbsDefaultKmsKeyIdRequest {
    const filterSensitiveLog: (obj: ResetEbsDefaultKmsKeyIdRequest) => any;
    const isa: (o: any) => o is ResetEbsDefaultKmsKeyIdRequest;
}
export interface ResetEbsDefaultKmsKeyIdResult {
    __type?: "ResetEbsDefaultKmsKeyIdResult";
    /**
     * <p>The Amazon Resource Name (ARN) of the default CMK for EBS encryption by default.</p>
     */
    KmsKeyId?: string;
}
export declare namespace ResetEbsDefaultKmsKeyIdResult {
    const filterSensitiveLog: (obj: ResetEbsDefaultKmsKeyIdResult) => any;
    const isa: (o: any) => o is ResetEbsDefaultKmsKeyIdResult;
}
export declare type ResetFpgaImageAttributeName = "loadPermission";
export interface ResetFpgaImageAttributeRequest {
    __type?: "ResetFpgaImageAttributeRequest";
    /**
     * <p>The attribute.</p>
     */
    Attribute?: ResetFpgaImageAttributeName | string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the AFI.</p>
     */
    FpgaImageId: string | undefined;
}
export declare namespace ResetFpgaImageAttributeRequest {
    const filterSensitiveLog: (obj: ResetFpgaImageAttributeRequest) => any;
    const isa: (o: any) => o is ResetFpgaImageAttributeRequest;
}
export interface ResetFpgaImageAttributeResult {
    __type?: "ResetFpgaImageAttributeResult";
    /**
     * <p>Is <code>true</code> if the request succeeds, and an error otherwise.</p>
     */
    Return?: boolean;
}
export declare namespace ResetFpgaImageAttributeResult {
    const filterSensitiveLog: (obj: ResetFpgaImageAttributeResult) => any;
    const isa: (o: any) => o is ResetFpgaImageAttributeResult;
}
export declare type ResetImageAttributeName = "launchPermission";
/**
 * <p>Contains the parameters for ResetImageAttribute.</p>
 */
export interface ResetImageAttributeRequest {
    __type?: "ResetImageAttributeRequest";
    /**
     * <p>The attribute to reset (currently you can only reset the launch permission attribute).</p>
     */
    Attribute: ResetImageAttributeName | string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *        and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *        Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the AMI.</p>
     */
    ImageId: string | undefined;
}
export declare namespace ResetImageAttributeRequest {
    const filterSensitiveLog: (obj: ResetImageAttributeRequest) => any;
    const isa: (o: any) => o is ResetImageAttributeRequest;
}
export interface ResetInstanceAttributeRequest {
    __type?: "ResetInstanceAttributeRequest";
    /**
     * <p>The attribute to reset.</p>
     *         <important>
     *             <p>You can only reset the following attributes: <code>kernel</code> |
     *                     <code>ramdisk</code> | <code>sourceDestCheck</code>. To change an instance
     *                 attribute, use <a>ModifyInstanceAttribute</a>.</p>
     *         </important>
     */
    Attribute: InstanceAttributeName | string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
}
export declare namespace ResetInstanceAttributeRequest {
    const filterSensitiveLog: (obj: ResetInstanceAttributeRequest) => any;
    const isa: (o: any) => o is ResetInstanceAttributeRequest;
}
/**
 * <p>Contains the parameters for ResetNetworkInterfaceAttribute.</p>
 */
export interface ResetNetworkInterfaceAttributeRequest {
    __type?: "ResetNetworkInterfaceAttributeRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *             and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *             Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId: string | undefined;
    /**
     * <p>The source/destination checking attribute. Resets the value to <code>true</code>.</p>
     */
    SourceDestCheck?: string;
}
export declare namespace ResetNetworkInterfaceAttributeRequest {
    const filterSensitiveLog: (obj: ResetNetworkInterfaceAttributeRequest) => any;
    const isa: (o: any) => o is ResetNetworkInterfaceAttributeRequest;
}
export interface ResetSnapshotAttributeRequest {
    __type?: "ResetSnapshotAttributeRequest";
    /**
     * <p>The attribute to reset. Currently, only the attribute for permission to create volumes can
     *       be reset.</p>
     */
    Attribute: SnapshotAttributeName | string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the snapshot.</p>
     */
    SnapshotId: string | undefined;
}
export declare namespace ResetSnapshotAttributeRequest {
    const filterSensitiveLog: (obj: ResetSnapshotAttributeRequest) => any;
    const isa: (o: any) => o is ResetSnapshotAttributeRequest;
}
export declare type ResourceType = "client-vpn-endpoint" | "customer-gateway" | "dedicated-host" | "dhcp-options" | "elastic-ip" | "fleet" | "fpga-image" | "host-reservation" | "image" | "instance" | "internet-gateway" | "key-pair" | "launch-template" | "natgateway" | "network-acl" | "network-interface" | "placement-group" | "reserved-instances" | "route-table" | "security-group" | "snapshot" | "spot-fleet-request" | "spot-instances-request" | "subnet" | "traffic-mirror-filter" | "traffic-mirror-session" | "traffic-mirror-target" | "transit-gateway" | "transit-gateway-attachment" | "transit-gateway-multicast-domain" | "transit-gateway-route-table" | "volume" | "vpc" | "vpc-peering-connection" | "vpn-connection" | "vpn-gateway";
/**
 * <p>Describes the error that's returned when you cannot delete a launch template
 *             version.</p>
 */
export interface ResponseError {
    __type?: "ResponseError";
    /**
     * <p>The error code.</p>
     */
    Code?: LaunchTemplateErrorCode | string;
    /**
     * <p>The error message, if applicable.</p>
     */
    Message?: string;
}
export declare namespace ResponseError {
    const filterSensitiveLog: (obj: ResponseError) => any;
    const isa: (o: any) => o is ResponseError;
}
/**
 * <p>The information for a launch template. </p>
 */
export interface ResponseLaunchTemplateData {
    __type?: "ResponseLaunchTemplateData";
    /**
     * <p>The block device mappings.</p>
     */
    BlockDeviceMappings?: LaunchTemplateBlockDeviceMapping[];
    /**
     * <p>Information about the Capacity Reservation targeting option.</p>
     */
    CapacityReservationSpecification?: LaunchTemplateCapacityReservationSpecificationResponse;
    /**
     * <p>The CPU options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html">Optimizing CPU Options</a> in the <i>Amazon Elastic Compute Cloud User
     * 				Guide</i>.</p>
     */
    CpuOptions?: LaunchTemplateCpuOptions;
    /**
     * <p>The credit option for CPU usage of the instance.</p>
     */
    CreditSpecification?: CreditSpecification;
    /**
     * <p>If set to <code>true</code>, indicates that the instance cannot be terminated using
     *             the Amazon EC2 console, command line tool, or API.</p>
     */
    DisableApiTermination?: boolean;
    /**
     * <p>Indicates whether the instance is optimized for Amazon EBS I/O. </p>
     */
    EbsOptimized?: boolean;
    /**
     * <p>The elastic GPU specification.</p>
     */
    ElasticGpuSpecifications?: ElasticGpuSpecificationResponse[];
    /**
     * <p>
     *             The elastic inference accelerator for the instance.
     *         </p>
     */
    ElasticInferenceAccelerators?: LaunchTemplateElasticInferenceAcceleratorResponse[];
    /**
     * <p>Indicates whether an instance is configured for hibernation. For more information, see
     *                 <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate
     *                 Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    HibernationOptions?: LaunchTemplateHibernationOptions;
    /**
     * <p>The IAM instance profile.</p>
     */
    IamInstanceProfile?: LaunchTemplateIamInstanceProfileSpecification;
    /**
     * <p>The ID of the AMI that was used to launch the instance.</p>
     */
    ImageId?: string;
    /**
     * <p>Indicates whether an instance stops or terminates when you initiate shutdown from
     *             the instance (using the operating system command for system shutdown).</p>
     */
    InstanceInitiatedShutdownBehavior?: ShutdownBehavior | string;
    /**
     * <p>The market (purchasing) option for the instances.</p>
     */
    InstanceMarketOptions?: LaunchTemplateInstanceMarketOptions;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The ID of the kernel, if applicable.</p>
     */
    KernelId?: string;
    /**
     * <p>The name of the key pair.</p>
     */
    KeyName?: string;
    /**
     * <p>The license configurations.</p>
     */
    LicenseSpecifications?: LaunchTemplateLicenseConfiguration[];
    /**
     * <p>The metadata options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html">Instance Metadata and User Data</a> in the
     *             <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    MetadataOptions?: LaunchTemplateInstanceMetadataOptions;
    /**
     * <p>The monitoring for the instance.</p>
     */
    Monitoring?: LaunchTemplatesMonitoring;
    /**
     * <p>The network interfaces.</p>
     */
    NetworkInterfaces?: LaunchTemplateInstanceNetworkInterfaceSpecification[];
    /**
     * <p>The placement of the instance.</p>
     */
    Placement?: LaunchTemplatePlacement;
    /**
     * <p>The ID of the RAM disk, if applicable.</p>
     */
    RamDiskId?: string;
    /**
     * <p>The security group IDs.</p>
     */
    SecurityGroupIds?: string[];
    /**
     * <p>The security group names.</p>
     */
    SecurityGroups?: string[];
    /**
     * <p>The tags.</p>
     */
    TagSpecifications?: LaunchTemplateTagSpecification[];
    /**
     * <p>The user data for the instance. </p>
     */
    UserData?: string;
}
export declare namespace ResponseLaunchTemplateData {
    const filterSensitiveLog: (obj: ResponseLaunchTemplateData) => any;
    const isa: (o: any) => o is ResponseLaunchTemplateData;
}
export interface RestoreAddressToClassicRequest {
    __type?: "RestoreAddressToClassicRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The Elastic IP address.</p>
     */
    PublicIp: string | undefined;
}
export declare namespace RestoreAddressToClassicRequest {
    const filterSensitiveLog: (obj: RestoreAddressToClassicRequest) => any;
    const isa: (o: any) => o is RestoreAddressToClassicRequest;
}
export interface RestoreAddressToClassicResult {
    __type?: "RestoreAddressToClassicResult";
    /**
     * <p>The Elastic IP address.</p>
     */
    PublicIp?: string;
    /**
     * <p>The move status for the IP address.</p>
     */
    Status?: Status | string;
}
export declare namespace RestoreAddressToClassicResult {
    const filterSensitiveLog: (obj: RestoreAddressToClassicResult) => any;
    const isa: (o: any) => o is RestoreAddressToClassicResult;
}
export interface RevokeClientVpnIngressRequest {
    __type?: "RevokeClientVpnIngressRequest";
    /**
     * <p>The ID of the Active Directory group for which to revoke access. </p>
     */
    AccessGroupId?: string;
    /**
     * <p>The ID of the Client VPN endpoint with which the authorization rule is associated.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Indicates whether access should be revoked for all clients.</p>
     */
    RevokeAllGroups?: boolean;
    /**
     * <p>The IPv4 address range, in CIDR notation, of the network for which access is being removed.</p>
     */
    TargetNetworkCidr: string | undefined;
}
export declare namespace RevokeClientVpnIngressRequest {
    const filterSensitiveLog: (obj: RevokeClientVpnIngressRequest) => any;
    const isa: (o: any) => o is RevokeClientVpnIngressRequest;
}
export interface RevokeClientVpnIngressResult {
    __type?: "RevokeClientVpnIngressResult";
    /**
     * <p>The current state of the authorization rule.</p>
     */
    Status?: ClientVpnAuthorizationRuleStatus;
}
export declare namespace RevokeClientVpnIngressResult {
    const filterSensitiveLog: (obj: RevokeClientVpnIngressResult) => any;
    const isa: (o: any) => o is RevokeClientVpnIngressResult;
}
export interface RevokeSecurityGroupEgressRequest {
    __type?: "RevokeSecurityGroupEgressRequest";
    /**
     * <p>Not supported. Use a set of IP permissions to specify the CIDR.</p>
     */
    CidrIp?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Not supported. Use a set of IP permissions to specify the port.</p>
     */
    FromPort?: number;
    /**
     * <p>The ID of the security group.</p>
     */
    GroupId: string | undefined;
    /**
     * <p>The sets of IP permissions. You can't specify a destination security group and a CIDR IP address range in the same set of permissions.</p>
     */
    IpPermissions?: IpPermission[];
    /**
     * <p>Not supported. Use a set of IP permissions to specify the protocol name or
     *             number.</p>
     */
    IpProtocol?: string;
    /**
     * <p>Not supported. Use a set of IP permissions to specify a
     *            destination security group.</p>
     */
    SourceSecurityGroupName?: string;
    /**
     * <p>Not supported. Use a set of IP permissions to specify a destination security
     *             group.</p>
     */
    SourceSecurityGroupOwnerId?: string;
    /**
     * <p>Not supported. Use a set of IP permissions to specify the port.</p>
     */
    ToPort?: number;
}
export declare namespace RevokeSecurityGroupEgressRequest {
    const filterSensitiveLog: (obj: RevokeSecurityGroupEgressRequest) => any;
    const isa: (o: any) => o is RevokeSecurityGroupEgressRequest;
}
export interface RevokeSecurityGroupIngressRequest {
    __type?: "RevokeSecurityGroupIngressRequest";
    /**
     * <p>The CIDR IP address range. You can't specify this parameter when specifying a source security group.</p>
     */
    CidrIp?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The start of port range for the TCP and UDP protocols, or an ICMP type number. For the ICMP type number,
     *         use <code>-1</code> to specify all ICMP types.</p>
     */
    FromPort?: number;
    /**
     * <p>The ID of the security group. You must specify either the security group ID or the
     *            security group name in the request. For security groups in a nondefault VPC, you must
     *            specify the security group ID.</p>
     */
    GroupId?: string;
    /**
     * <p>[EC2-Classic, default VPC] The name of the security group. You must specify either the
     *            security group ID or the security group name in the request.</p>
     */
    GroupName?: string;
    /**
     * <p>The sets of IP permissions. You can't specify a source security group and a CIDR IP address range in the same set of permissions.</p>
     */
    IpPermissions?: IpPermission[];
    /**
     * <p>The IP protocol name (<code>tcp</code>, <code>udp</code>, <code>icmp</code>) or number
     *         (see <a href="http://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml">Protocol Numbers</a>).
     *         Use <code>-1</code> to specify all.</p>
     */
    IpProtocol?: string;
    /**
     * <p>[EC2-Classic, default VPC] The name of the source security group. You can't specify this parameter in combination with the following parameters: the CIDR IP address range, the start of the port range, the IP protocol, and the end of the port range. For EC2-VPC, the source security group must be in the same VPC. To revoke a specific rule for an IP protocol and port range, use a set of IP permissions instead.</p>
     */
    SourceSecurityGroupName?: string;
    /**
     * <p>[EC2-Classic] The AWS account ID of the source security group, if the source security group is in a different account. You can't specify this parameter in combination with the following parameters: the CIDR IP address range, the IP protocol, the start of the port range, and the end of the port range. To revoke a specific rule for an IP protocol and port range, use a set of IP permissions instead.</p>
     */
    SourceSecurityGroupOwnerId?: string;
    /**
     * <p>The end of port range for the TCP and UDP protocols, or an ICMP code number. For the ICMP code number,
     *         use <code>-1</code> to specify all ICMP codes for the ICMP type.</p>
     */
    ToPort?: number;
}
export declare namespace RevokeSecurityGroupIngressRequest {
    const filterSensitiveLog: (obj: RevokeSecurityGroupIngressRequest) => any;
    const isa: (o: any) => o is RevokeSecurityGroupIngressRequest;
}
export declare type RIProductDescription = "Linux/UNIX" | "Linux/UNIX (Amazon VPC)" | "Windows" | "Windows (Amazon VPC)";
export declare type RootDeviceType = "ebs" | "instance-store";
/**
 * <p>Describes a route in a route table.</p>
 */
export interface Route {
    __type?: "Route";
    /**
     * <p>The IPv4 CIDR block used for the destination match.</p>
     */
    DestinationCidrBlock?: string;
    /**
     * <p>The IPv6 CIDR block used for the destination match.</p>
     */
    DestinationIpv6CidrBlock?: string;
    /**
     * <p>The prefix of the AWS service.</p>
     */
    DestinationPrefixListId?: string;
    /**
     * <p>The ID of the egress-only internet gateway.</p>
     */
    EgressOnlyInternetGatewayId?: string;
    /**
     * <p>The ID of a gateway attached to your VPC.</p>
     */
    GatewayId?: string;
    /**
     * <p>The ID of a NAT instance in your VPC.</p>
     */
    InstanceId?: string;
    /**
     * <p>The AWS account ID of the owner of the instance.</p>
     */
    InstanceOwnerId?: string;
    /**
     * <p>The ID of the local gateway.</p>
     */
    LocalGatewayId?: string;
    /**
     * <p>The ID of a NAT gateway.</p>
     */
    NatGatewayId?: string;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>Describes how the route was created.</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>CreateRouteTable</code> - The route was automatically created when the route table was created.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>CreateRoute</code> - The route was manually added to the route table.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>EnableVgwRoutePropagation</code> - The route was propagated by route propagation.</p>
     *             </li>
     *          </ul>
     */
    Origin?: RouteOrigin | string;
    /**
     * <p>The state of the route. The <code>blackhole</code> state indicates that the
     * 				route's target isn't available (for example, the specified gateway isn't attached to the
     * 				VPC, or the specified NAT instance has been terminated).</p>
     */
    State?: RouteState | string;
    /**
     * <p>The ID of a transit gateway.</p>
     */
    TransitGatewayId?: string;
    /**
     * <p>The ID of a VPC peering connection.</p>
     */
    VpcPeeringConnectionId?: string;
}
export declare namespace Route {
    const filterSensitiveLog: (obj: Route) => any;
    const isa: (o: any) => o is Route;
}
export declare type RouteOrigin = "CreateRoute" | "CreateRouteTable" | "EnableVgwRoutePropagation";
export declare type RouteState = "active" | "blackhole";
/**
 * <p>Describes a route table.</p>
 */
export interface RouteTable {
    __type?: "RouteTable";
    /**
     * <p>The associations between the route table and one or more subnets or a gateway.</p>
     */
    Associations?: RouteTableAssociation[];
    /**
     * <p>The ID of the AWS account that owns the route table.</p>
     */
    OwnerId?: string;
    /**
     * <p>Any virtual private gateway (VGW) propagating routes.</p>
     */
    PropagatingVgws?: PropagatingVgw[];
    /**
     * <p>The ID of the route table.</p>
     */
    RouteTableId?: string;
    /**
     * <p>The routes in the route table.</p>
     */
    Routes?: Route[];
    /**
     * <p>Any tags assigned to the route table.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace RouteTable {
    const filterSensitiveLog: (obj: RouteTable) => any;
    const isa: (o: any) => o is RouteTable;
}
/**
 * <p>Describes an association between a route table and a subnet or gateway.</p>
 */
export interface RouteTableAssociation {
    __type?: "RouteTableAssociation";
    /**
     * <p>The state of the association.</p>
     */
    AssociationState?: RouteTableAssociationState;
    /**
     * <p>The ID of the internet gateway or virtual private gateway.</p>
     */
    GatewayId?: string;
    /**
     * <p>Indicates whether this is the main route table.</p>
     */
    Main?: boolean;
    /**
     * <p>The ID of the association.</p>
     */
    RouteTableAssociationId?: string;
    /**
     * <p>The ID of the route table.</p>
     */
    RouteTableId?: string;
    /**
     * <p>The ID of the subnet. A subnet ID is not returned for an implicit association.</p>
     */
    SubnetId?: string;
}
export declare namespace RouteTableAssociation {
    const filterSensitiveLog: (obj: RouteTableAssociation) => any;
    const isa: (o: any) => o is RouteTableAssociation;
}
/**
 * <p>Describes the state of an association between a route table and a subnet or gateway.</p>
 */
export interface RouteTableAssociationState {
    __type?: "RouteTableAssociationState";
    /**
     * <p>The state of the association.</p>
     */
    State?: RouteTableAssociationStateCode | string;
    /**
     * <p>The status message, if applicable.</p>
     */
    StatusMessage?: string;
}
export declare namespace RouteTableAssociationState {
    const filterSensitiveLog: (obj: RouteTableAssociationState) => any;
    const isa: (o: any) => o is RouteTableAssociationState;
}
export declare type RouteTableAssociationStateCode = "associated" | "associating" | "disassociated" | "disassociating" | "failed";
export declare type RuleAction = "allow" | "deny";
/**
 * <p>Describes the monitoring of an instance.</p>
 */
export interface RunInstancesMonitoringEnabled {
    __type?: "RunInstancesMonitoringEnabled";
    /**
     * <p>Indicates whether detailed monitoring is enabled. Otherwise, basic monitoring is
     *             enabled.</p>
     */
    Enabled: boolean | undefined;
}
export declare namespace RunInstancesMonitoringEnabled {
    const filterSensitiveLog: (obj: RunInstancesMonitoringEnabled) => any;
    const isa: (o: any) => o is RunInstancesMonitoringEnabled;
}
export interface RunInstancesRequest {
    __type?: "RunInstancesRequest";
    /**
     * <p>Reserved.</p>
     */
    AdditionalInfo?: string;
    /**
     * <p>The block device mapping entries.</p>
     */
    BlockDeviceMappings?: BlockDeviceMapping[];
    /**
     * <p>Information about the Capacity Reservation targeting option. If you do not specify this parameter, the
     *             instance's Capacity Reservation preference defaults to <code>open</code>, which enables
     *             it to run in any open Capacity Reservation that has matching attributes (instance type,
     *             platform, Availability Zone).</p>
     */
    CapacityReservationSpecification?: CapacityReservationSpecification;
    /**
     * <p>Unique, case-sensitive identifier you provide to ensure the idempotency of the
     *             request. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring
     *                 Idempotency</a>.</p>
     *         <p>Constraints: Maximum 64 ASCII characters</p>
     */
    ClientToken?: string;
    /**
     * <p>The CPU options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html">Optimizing CPU Options</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    CpuOptions?: CpuOptionsRequest;
    /**
     * <p>The credit option for CPU usage of the burstable performance instance. Valid values
     *             are <code>standard</code> and <code>unlimited</code>. To change this attribute after
     *             launch, use <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ModifyInstanceCreditSpecification.html">
     *                 ModifyInstanceCreditSpecification</a>. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/burstable-performance-instances.html">Burstable
     *                 Performance Instances</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *         <p>Default: <code>standard</code> (T2 instances) or <code>unlimited</code> (T3/T3a
     *             instances)</p>
     */
    CreditSpecification?: CreditSpecificationRequest;
    /**
     * <p>If you set this parameter to <code>true</code>, you can't terminate the instance using
     *             the Amazon EC2 console, CLI, or API; otherwise, you can. To change this attribute after launch,
     *             use <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ModifyInstanceAttribute.html">ModifyInstanceAttribute</a>.
     *             Alternatively, if you set <code>InstanceInitiatedShutdownBehavior</code> to
     *                 <code>terminate</code>, you can terminate the instance by running the shutdown
     *             command from the instance.</p>
     *         <p>Default: <code>false</code>
     *         </p>
     */
    DisableApiTermination?: boolean;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Indicates whether the instance is optimized for Amazon EBS I/O. This optimization
     *             provides dedicated throughput to Amazon EBS and an optimized configuration stack to
     *             provide optimal Amazon EBS I/O performance. This optimization isn't available with all
     *             instance types. Additional usage charges apply when using an EBS-optimized
     *             instance.</p>
     *         <p>Default: <code>false</code>
     *         </p>
     */
    EbsOptimized?: boolean;
    /**
     * <p>An elastic GPU to associate with the instance. An Elastic GPU is a GPU resource that
     *             you can attach to your Windows instance to accelerate the graphics performance of your
     *             applications. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/elastic-graphics.html"> Amazon EC2 Elastic
     *                 GPUs</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    ElasticGpuSpecification?: ElasticGpuSpecification[];
    /**
     * <p>An elastic inference accelerator to associate with the instance. Elastic inference
     *             accelerators are a resource you can attach to your Amazon EC2 instances to accelerate
     *             your Deep Learning (DL) inference workloads.</p>
     */
    ElasticInferenceAccelerators?: ElasticInferenceAccelerator[];
    /**
     * <p>Indicates whether an instance is enabled for hibernation. For more information, see
     *                 <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate
     *                 Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     */
    HibernationOptions?: HibernationOptionsRequest;
    /**
     * <p>The IAM instance profile.</p>
     */
    IamInstanceProfile?: IamInstanceProfileSpecification;
    /**
     * <p>The ID of the AMI. An AMI ID is required to launch an instance and must be
     *         	specified here or in a launch template.</p>
     */
    ImageId?: string;
    /**
     * <p>Indicates whether an instance stops or terminates when you initiate shutdown from the
     *             instance (using the operating system command for system shutdown).</p>
     *         <p>Default: <code>stop</code>
     *         </p>
     */
    InstanceInitiatedShutdownBehavior?: ShutdownBehavior | string;
    /**
     * <p>The market (purchasing) option for the instances.</p>
     *         <p>For <a>RunInstances</a>, persistent Spot Instance requests are
     *             only supported when <b>InstanceInterruptionBehavior</b> is set
     *             to either <code>hibernate</code> or <code>stop</code>.</p>
     */
    InstanceMarketOptions?: InstanceMarketOptionsRequest;
    /**
     * <p>The instance type. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html">Instance Types</a> in the
     *                 <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *         <p>Default: <code>m1.small</code>
     *          </p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>[EC2-VPC] The number of IPv6 addresses to associate with the primary network
     *             interface. Amazon EC2 chooses the IPv6 addresses from the range of your subnet. You
     *             cannot specify this option and the option to assign specific IPv6 addresses in the same
     *             request. You can specify this option if you've specified a minimum number of instances
     *             to launch.</p>
     *         <p>You cannot specify this option and the network interfaces option in the same
     *             request.</p>
     */
    Ipv6AddressCount?: number;
    /**
     * <p>[EC2-VPC] The IPv6 addresses from the range of the subnet to associate with the
     *             primary network interface. You cannot specify this option and the option to assign a
     *             number of IPv6 addresses in the same request. You cannot specify this option if you've
     *             specified a minimum number of instances to launch.</p>
     *         <p>You cannot specify this option and the network interfaces option in the same
     *             request.</p>
     */
    Ipv6Addresses?: InstanceIpv6Address[];
    /**
     * <p>The ID of the kernel.</p>
     *         <important>
     *             <p>We recommend that you use PV-GRUB instead of kernels and RAM disks. For more
     *                 information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html"> PV-GRUB</a> in the
     *                     <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *         </important>
     */
    KernelId?: string;
    /**
     * <p>The name of the key pair. You can create a key pair using <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateKeyPair.html">CreateKeyPair</a> or
     *                 <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ImportKeyPair.html">ImportKeyPair</a>.</p>
     *         <important>
     *             <p>If you do not specify a key pair, you can't connect to the instance unless you
     *                 choose an AMI that is configured to allow users another way to log in.</p>
     *         </important>
     */
    KeyName?: string;
    /**
     * <p>The launch template to use to launch the instances. Any parameters that you specify in
     *                 <a>RunInstances</a> override the same parameters in the launch template.
     *             You can specify either the name or ID of a launch template, but not both.</p>
     */
    LaunchTemplate?: LaunchTemplateSpecification;
    /**
     * <p>The license configurations.</p>
     */
    LicenseSpecifications?: LicenseConfigurationRequest[];
    /**
     * <p>The maximum number of instances to launch. If you specify more instances than Amazon
     *             EC2 can launch in the target Availability Zone, Amazon EC2 launches the largest possible
     *             number of instances above <code>MinCount</code>.</p>
     *         <p>Constraints: Between 1 and the maximum number you're allowed for the specified
     *             instance type. For more information about the default limits, and how to request an
     *             increase, see <a href="http://aws.amazon.com/ec2/faqs/#How_many_instances_can_I_run_in_Amazon_EC2">How many instances can I
     *                 run in Amazon EC2</a> in the Amazon EC2 FAQ.</p>
     */
    MaxCount: number | undefined;
    /**
     * <p>The metadata options for the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html">Instance Metadata and User Data</a>.</p>
     */
    MetadataOptions?: InstanceMetadataOptionsRequest;
    /**
     * <p>The minimum number of instances to launch. If you specify a minimum that is more
     *             instances than Amazon EC2 can launch in the target Availability Zone, Amazon EC2
     *             launches no instances.</p>
     *         <p>Constraints: Between 1 and the maximum number you're allowed for the specified
     *             instance type. For more information about the default limits, and how to request an
     *             increase, see <a href="http://aws.amazon.com/ec2/faqs/#How_many_instances_can_I_run_in_Amazon_EC2">How many instances can I
     *                 run in Amazon EC2</a> in the Amazon EC2 General FAQ.</p>
     */
    MinCount: number | undefined;
    /**
     * <p>Specifies whether detailed monitoring is enabled for the instance.</p>
     */
    Monitoring?: RunInstancesMonitoringEnabled;
    /**
     * <p>The network interfaces to associate with the instance. If you specify a network interface,
     *             you must specify any security groups and subnets as part of the network interface.</p>
     */
    NetworkInterfaces?: InstanceNetworkInterfaceSpecification[];
    /**
     * <p>The placement for the instance.</p>
     */
    Placement?: Placement;
    /**
     * <p>[EC2-VPC] The primary IPv4 address. You must specify a value from the IPv4 address
     *             range of the subnet.</p>
     *         <p>Only one private IP address can be designated as primary. You can't specify this
     *             option if you've specified the option to designate a private IP address as the primary
     *             IP address in a network interface specification. You cannot specify this option if
     *             you're launching more than one instance in the request.</p>
     *         <p>You cannot specify this option and the network interfaces option in the same
     *             request.</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>The ID of the RAM disk to select. Some kernels require additional drivers at launch.
     *             Check the kernel requirements for information about whether you need to specify a RAM
     *             disk. To find kernel requirements, go to the AWS Resource Center and search for the
     *             kernel ID.</p>
     *         <important>
     *             <p>We recommend that you use PV-GRUB instead of kernels and RAM disks. For more
     *                 information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html"> PV-GRUB</a> in the
     *                     <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *         </important>
     */
    RamdiskId?: string;
    /**
     * <p>The IDs of the security groups. You can create a security group using
     * 	<a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateSecurityGroup.html">CreateSecurityGroup</a>.</p>
     *         <p>If you specify a network interface, you must specify any security groups as part of the network interface.</p>
     */
    SecurityGroupIds?: string[];
    /**
     * <p>[EC2-Classic, default VPC] The names of the security groups. For a nondefault VPC, you
     *             must use security group IDs instead.</p>
     *         <p>If you specify a network interface, you must specify any security groups as part of
     *             the network interface.</p>
     *         <p>Default: Amazon EC2 uses the default security group.</p>
     */
    SecurityGroups?: string[];
    /**
     * <p>[EC2-VPC] The ID of the subnet to launch the instance into.</p>
     *         <p>If you specify a network interface, you must specify any subnets as part of the network interface.</p>
     */
    SubnetId?: string;
    /**
     * <p>The tags to apply to the resources during launch. You can only tag instances and
     *             volumes on launch. The specified tags are applied to all instances or volumes that are
     *             created during launch. To tag a resource after it has been created, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html">CreateTags</a>.</p>
     */
    TagSpecifications?: TagSpecification[];
    /**
     * <p>The user data to make available to the instance. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/user-data.html">Running Commands
     *                 on Your Linux Instance at Launch</a> (Linux) and <a href="https://docs.aws.amazon.com/AWSEC2/latest/WindowsGuide/ec2-instance-metadata.html#instancedata-add-user-data">Adding User Data</a> (Windows). If you are using a command line tool,
     *             base64-encoding is performed for you, and you can load the text from a file. Otherwise,
     *             you must provide base64-encoded text. User data is limited to 16 KB.</p>
     */
    UserData?: string;
}
export declare namespace RunInstancesRequest {
    const filterSensitiveLog: (obj: RunInstancesRequest) => any;
    const isa: (o: any) => o is RunInstancesRequest;
}
/**
 * <p>Contains the parameters for RunScheduledInstances.</p>
 */
export interface RunScheduledInstancesRequest {
    __type?: "RunScheduledInstancesRequest";
    /**
     * <p>Unique, case-sensitive identifier that ensures the idempotency of the request.
     *          For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The number of instances.</p>
     *          <p>Default: 1</p>
     */
    InstanceCount?: number;
    /**
     * <p>The launch specification. You must match the instance type, Availability Zone,
     *          network, and platform of the schedule that you purchased.</p>
     */
    LaunchSpecification: ScheduledInstancesLaunchSpecification | undefined;
    /**
     * <p>The Scheduled Instance ID.</p>
     */
    ScheduledInstanceId: string | undefined;
}
export declare namespace RunScheduledInstancesRequest {
    const filterSensitiveLog: (obj: RunScheduledInstancesRequest) => any;
    const isa: (o: any) => o is RunScheduledInstancesRequest;
}
/**
 * <p>Contains the output of RunScheduledInstances.</p>
 */
export interface RunScheduledInstancesResult {
    __type?: "RunScheduledInstancesResult";
    /**
     * <p>The IDs of the newly launched instances.</p>
     */
    InstanceIdSet?: string[];
}
export declare namespace RunScheduledInstancesResult {
    const filterSensitiveLog: (obj: RunScheduledInstancesResult) => any;
    const isa: (o: any) => o is RunScheduledInstancesResult;
}
/**
 * <p>Describes the storage parameters for S3 and S3 buckets for an instance store-backed AMI.</p>
 */
export interface S3Storage {
    __type?: "S3Storage";
    /**
     * <p>The access key ID of the owner of the bucket. Before you specify a value for your access key ID, review and follow the guidance
     *        in <a href="https://docs.aws.amazon.com/general/latest/gr/aws-access-keys-best-practices.html">Best Practices for Managing AWS Access Keys</a>.</p>
     */
    AWSAccessKeyId?: string;
    /**
     * <p>The bucket in which to store the AMI. You can specify a bucket that you already own or a new bucket that Amazon EC2 creates on your behalf. If you specify a bucket that belongs to someone else, Amazon EC2 returns an error.</p>
     */
    Bucket?: string;
    /**
     * <p>The beginning of the file name of the AMI.</p>
     */
    Prefix?: string;
    /**
     * <p>An Amazon S3 upload policy that gives Amazon EC2 permission to upload items into Amazon S3 on your behalf.</p>
     */
    UploadPolicy?: Uint8Array;
    /**
     * <p>The signature of the JSON document.</p>
     */
    UploadPolicySignature?: string;
}
export declare namespace S3Storage {
    const filterSensitiveLog: (obj: S3Storage) => any;
    const isa: (o: any) => o is S3Storage;
}
/**
 * <p>Describes a Scheduled Instance.</p>
 */
export interface ScheduledInstance {
    __type?: "ScheduledInstance";
    /**
     * <p>The Availability Zone.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The date when the Scheduled Instance was purchased.</p>
     */
    CreateDate?: Date;
    /**
     * <p>The hourly price for a single instance.</p>
     */
    HourlyPrice?: string;
    /**
     * <p>The number of instances.</p>
     */
    InstanceCount?: number;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: string;
    /**
     * <p>The network platform (<code>EC2-Classic</code> or <code>EC2-VPC</code>).</p>
     */
    NetworkPlatform?: string;
    /**
     * <p>The time for the next schedule to start.</p>
     */
    NextSlotStartTime?: Date;
    /**
     * <p>The platform (<code>Linux/UNIX</code> or <code>Windows</code>).</p>
     */
    Platform?: string;
    /**
     * <p>The time that the previous schedule ended or will end.</p>
     */
    PreviousSlotEndTime?: Date;
    /**
     * <p>The schedule recurrence.</p>
     */
    Recurrence?: ScheduledInstanceRecurrence;
    /**
     * <p>The Scheduled Instance ID.</p>
     */
    ScheduledInstanceId?: string;
    /**
     * <p>The number of hours in the schedule.</p>
     */
    SlotDurationInHours?: number;
    /**
     * <p>The end date for the Scheduled Instance.</p>
     */
    TermEndDate?: Date;
    /**
     * <p>The start date for the Scheduled Instance.</p>
     */
    TermStartDate?: Date;
    /**
     * <p>The total number of hours for a single instance for the entire term.</p>
     */
    TotalScheduledInstanceHours?: number;
}
export declare namespace ScheduledInstance {
    const filterSensitiveLog: (obj: ScheduledInstance) => any;
    const isa: (o: any) => o is ScheduledInstance;
}
/**
 * <p>Describes a schedule that is available for your Scheduled Instances.</p>
 */
export interface ScheduledInstanceAvailability {
    __type?: "ScheduledInstanceAvailability";
    /**
     * <p>The Availability Zone.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The number of available instances.</p>
     */
    AvailableInstanceCount?: number;
    /**
     * <p>The time period for the first schedule to start.</p>
     */
    FirstSlotStartTime?: Date;
    /**
     * <p>The hourly price for a single instance.</p>
     */
    HourlyPrice?: string;
    /**
     * <p>The instance type. You can specify one of the C3, C4, M4, or R3 instance types.</p>
     */
    InstanceType?: string;
    /**
     * <p>The maximum term. The only possible value is 365 days.</p>
     */
    MaxTermDurationInDays?: number;
    /**
     * <p>The minimum term. The only possible value is 365 days.</p>
     */
    MinTermDurationInDays?: number;
    /**
     * <p>The network platform (<code>EC2-Classic</code> or <code>EC2-VPC</code>).</p>
     */
    NetworkPlatform?: string;
    /**
     * <p>The platform (<code>Linux/UNIX</code> or <code>Windows</code>).</p>
     */
    Platform?: string;
    /**
     * <p>The purchase token. This token expires in two hours.</p>
     */
    PurchaseToken?: string;
    /**
     * <p>The schedule recurrence.</p>
     */
    Recurrence?: ScheduledInstanceRecurrence;
    /**
     * <p>The number of hours in the schedule.</p>
     */
    SlotDurationInHours?: number;
    /**
     * <p>The total number of hours for a single instance for the entire term.</p>
     */
    TotalScheduledInstanceHours?: number;
}
export declare namespace ScheduledInstanceAvailability {
    const filterSensitiveLog: (obj: ScheduledInstanceAvailability) => any;
    const isa: (o: any) => o is ScheduledInstanceAvailability;
}
/**
 * <p>Describes the recurring schedule for a Scheduled Instance.</p>
 */
export interface ScheduledInstanceRecurrence {
    __type?: "ScheduledInstanceRecurrence";
    /**
     * <p>The frequency (<code>Daily</code>, <code>Weekly</code>, or <code>Monthly</code>).</p>
     */
    Frequency?: string;
    /**
     * <p>The interval quantity. The interval unit depends on the value of <code>frequency</code>. For example, every 2
     *          weeks or every 2 months.</p>
     */
    Interval?: number;
    /**
     * <p>The days. For a monthly schedule, this is one or more days of the month (1-31). For a weekly schedule, this is one or more days of the week (1-7, where 1 is Sunday).</p>
     */
    OccurrenceDaySet?: number[];
    /**
     * <p>Indicates whether the occurrence is relative to the end of the specified week or month.</p>
     */
    OccurrenceRelativeToEnd?: boolean;
    /**
     * <p>The unit for <code>occurrenceDaySet</code> (<code>DayOfWeek</code> or <code>DayOfMonth</code>).</p>
     */
    OccurrenceUnit?: string;
}
export declare namespace ScheduledInstanceRecurrence {
    const filterSensitiveLog: (obj: ScheduledInstanceRecurrence) => any;
    const isa: (o: any) => o is ScheduledInstanceRecurrence;
}
/**
 * <p>Describes the recurring schedule for a Scheduled Instance.</p>
 */
export interface ScheduledInstanceRecurrenceRequest {
    __type?: "ScheduledInstanceRecurrenceRequest";
    /**
     * <p>The frequency (<code>Daily</code>, <code>Weekly</code>, or <code>Monthly</code>).</p>
     */
    Frequency?: string;
    /**
     * <p>The interval quantity. The interval unit depends on the value of <code>Frequency</code>. For example, every 2
     *          weeks or every 2 months.</p>
     */
    Interval?: number;
    /**
     * <p>The days. For a monthly schedule, this is one or more days of the month (1-31). For a weekly schedule, this is one or more days of the week (1-7, where 1 is Sunday). You can't specify this value with a daily schedule. If the occurrence is relative to the end of the month, you can specify only a single day.</p>
     */
    OccurrenceDays?: number[];
    /**
     * <p>Indicates whether the occurrence is relative to the end of the specified week or month. You can't specify this value with a daily schedule.</p>
     */
    OccurrenceRelativeToEnd?: boolean;
    /**
     * <p>The unit for <code>OccurrenceDays</code> (<code>DayOfWeek</code> or <code>DayOfMonth</code>).
     *         This value is required for a monthly schedule.
     *         You can't specify <code>DayOfWeek</code> with a weekly schedule.
     *         You can't specify this value with a daily schedule.</p>
     */
    OccurrenceUnit?: string;
}
export declare namespace ScheduledInstanceRecurrenceRequest {
    const filterSensitiveLog: (obj: ScheduledInstanceRecurrenceRequest) => any;
    const isa: (o: any) => o is ScheduledInstanceRecurrenceRequest;
}
/**
 * <p>Describes a block device mapping for a Scheduled Instance.</p>
 */
export interface ScheduledInstancesBlockDeviceMapping {
    __type?: "ScheduledInstancesBlockDeviceMapping";
    /**
     * <p>The device name (for example, <code>/dev/sdh</code> or <code>xvdh</code>).</p>
     */
    DeviceName?: string;
    /**
     * <p>Parameters used to set up EBS volumes automatically when the instance is launched.</p>
     */
    Ebs?: ScheduledInstancesEbs;
    /**
     * <p>Suppresses the specified device included in the block device mapping of the AMI.</p>
     */
    NoDevice?: string;
    /**
     * <p>The virtual device name (<code>ephemeral</code>N). Instance store volumes are numbered
     *          starting from 0. An instance type with two available instance store volumes can specify mappings
     *          for <code>ephemeral0</code> and <code>ephemeral1</code>. The number of available instance store
     *          volumes depends on the instance type. After you connect to the instance, you must mount the
     *          volume.</p>
     *          <p>Constraints: For M3 instances, you must specify instance store volumes in the block device mapping for the instance. When you launch an M3 instance, we ignore any instance store volumes specified in the block device mapping for the AMI.</p>
     */
    VirtualName?: string;
}
export declare namespace ScheduledInstancesBlockDeviceMapping {
    const filterSensitiveLog: (obj: ScheduledInstancesBlockDeviceMapping) => any;
    const isa: (o: any) => o is ScheduledInstancesBlockDeviceMapping;
}
/**
 * <p>Describes an EBS volume for a Scheduled Instance.</p>
 */
export interface ScheduledInstancesEbs {
    __type?: "ScheduledInstancesEbs";
    /**
     * <p>Indicates whether the volume is deleted on instance termination.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>Indicates whether the volume is encrypted. You can attached encrypted volumes only to instances that support them.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>The number of I/O operations per second (IOPS) that the volume supports. For io1 volumes,
     *          this represents the number of IOPS that are provisioned for the volume. For
     *             <code>gp2</code> volumes, this represents the baseline performance of the volume and
     *          the rate at which the volume accumulates I/O credits for bursting. For more information
     *          about <code>gp2</code> baseline performance, I/O credits, and bursting, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html">Amazon EBS Volume Types</a> in the
     *             <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *          <p>Constraint: Range is 100-20000 IOPS for <code>io1</code> volumes and
     *          100-10000 IOPS for <code>gp2</code> volumes.</p>
     *          <p>Condition: This parameter is required for requests to create <code>io1</code>volumes; it
     *          is not used in requests to create <code>gp2</code>, <code>st1</code>,
     *             <code>sc1</code>, or <code>standard</code> volumes.</p>
     */
    Iops?: number;
    /**
     * <p>The ID of the snapshot.</p>
     */
    SnapshotId?: string;
    /**
     * <p>The size of the volume, in GiB.</p>
     *          <p>Default: If you're creating the volume from a snapshot and don't specify a volume size, the default is the snapshot size.</p>
     */
    VolumeSize?: number;
    /**
     * <p>The volume type. <code>gp2</code> for General Purpose SSD, <code>io1</code> for Provisioned IOPS SSD, Throughput Optimized HDD
     *          for <code>st1</code>, Cold HDD for <code>sc1</code>, or <code>standard</code> for
     *          Magnetic.</p>
     *          <p>Default: <code>gp2</code>
     *          </p>
     */
    VolumeType?: string;
}
export declare namespace ScheduledInstancesEbs {
    const filterSensitiveLog: (obj: ScheduledInstancesEbs) => any;
    const isa: (o: any) => o is ScheduledInstancesEbs;
}
/**
 * <p>Describes an IAM instance profile for a Scheduled Instance.</p>
 */
export interface ScheduledInstancesIamInstanceProfile {
    __type?: "ScheduledInstancesIamInstanceProfile";
    /**
     * <p>The Amazon Resource Name (ARN).</p>
     */
    Arn?: string;
    /**
     * <p>The name.</p>
     */
    Name?: string;
}
export declare namespace ScheduledInstancesIamInstanceProfile {
    const filterSensitiveLog: (obj: ScheduledInstancesIamInstanceProfile) => any;
    const isa: (o: any) => o is ScheduledInstancesIamInstanceProfile;
}
/**
 * <p>Describes an IPv6 address.</p>
 */
export interface ScheduledInstancesIpv6Address {
    __type?: "ScheduledInstancesIpv6Address";
    /**
     * <p>The IPv6 address.</p>
     */
    Ipv6Address?: string;
}
export declare namespace ScheduledInstancesIpv6Address {
    const filterSensitiveLog: (obj: ScheduledInstancesIpv6Address) => any;
    const isa: (o: any) => o is ScheduledInstancesIpv6Address;
}
/**
 * <p>Describes the launch specification for a Scheduled Instance.</p>
 *          <p>If you are launching the Scheduled Instance in EC2-VPC, you must specify the ID of the subnet.
 *           You can specify the subnet using either <code>SubnetId</code> or <code>NetworkInterface</code>.</p>
 */
export interface ScheduledInstancesLaunchSpecification {
    __type?: "ScheduledInstancesLaunchSpecification";
    /**
     * <p>The block device mapping entries.</p>
     */
    BlockDeviceMappings?: ScheduledInstancesBlockDeviceMapping[];
    /**
     * <p>Indicates whether the instances are optimized for EBS I/O. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal EBS I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS-optimized instance.</p>
     *          <p>Default: <code>false</code>
     *          </p>
     */
    EbsOptimized?: boolean;
    /**
     * <p>The IAM instance profile.</p>
     */
    IamInstanceProfile?: ScheduledInstancesIamInstanceProfile;
    /**
     * <p>The ID of the Amazon Machine Image (AMI).</p>
     */
    ImageId: string | undefined;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: string;
    /**
     * <p>The ID of the kernel.</p>
     */
    KernelId?: string;
    /**
     * <p>The name of the key pair.</p>
     */
    KeyName?: string;
    /**
     * <p>Enable or disable monitoring for the instances.</p>
     */
    Monitoring?: ScheduledInstancesMonitoring;
    /**
     * <p>The network interfaces.</p>
     */
    NetworkInterfaces?: ScheduledInstancesNetworkInterface[];
    /**
     * <p>The placement information.</p>
     */
    Placement?: ScheduledInstancesPlacement;
    /**
     * <p>The ID of the RAM disk.</p>
     */
    RamdiskId?: string;
    /**
     * <p>The IDs of the security groups.</p>
     */
    SecurityGroupIds?: string[];
    /**
     * <p>The ID of the subnet in which to launch the instances.</p>
     */
    SubnetId?: string;
    /**
     * <p>The base64-encoded MIME user data.</p>
     */
    UserData?: string;
}
export declare namespace ScheduledInstancesLaunchSpecification {
    const filterSensitiveLog: (obj: ScheduledInstancesLaunchSpecification) => any;
    const isa: (o: any) => o is ScheduledInstancesLaunchSpecification;
}
/**
 * <p>Describes whether monitoring is enabled for a Scheduled Instance.</p>
 */
export interface ScheduledInstancesMonitoring {
    __type?: "ScheduledInstancesMonitoring";
    /**
     * <p>Indicates whether monitoring is enabled.</p>
     */
    Enabled?: boolean;
}
export declare namespace ScheduledInstancesMonitoring {
    const filterSensitiveLog: (obj: ScheduledInstancesMonitoring) => any;
    const isa: (o: any) => o is ScheduledInstancesMonitoring;
}
/**
 * <p>Describes a network interface for a Scheduled Instance.</p>
 */
export interface ScheduledInstancesNetworkInterface {
    __type?: "ScheduledInstancesNetworkInterface";
    /**
     * <p>Indicates whether to assign a public IPv4 address to instances launched in a VPC. The
     *          public IPv4 address can only be assigned to a network interface for eth0, and can only be
     *          assigned to a new network interface, not an existing one. You cannot specify more than one
     *          network interface in the request. If launching into a default subnet, the default value is
     *          <code>true</code>.</p>
     */
    AssociatePublicIpAddress?: boolean;
    /**
     * <p>Indicates whether to delete the interface when the instance is terminated.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>The description.</p>
     */
    Description?: string;
    /**
     * <p>The index of the device for the network interface attachment.</p>
     */
    DeviceIndex?: number;
    /**
     * <p>The IDs of the security groups.</p>
     */
    Groups?: string[];
    /**
     * <p>The number of IPv6 addresses to assign to the network interface. The IPv6 addresses are automatically selected from the subnet range.</p>
     */
    Ipv6AddressCount?: number;
    /**
     * <p>The specific IPv6 addresses from the subnet range.</p>
     */
    Ipv6Addresses?: ScheduledInstancesIpv6Address[];
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The IPv4 address of the network interface within the subnet.</p>
     */
    PrivateIpAddress?: string;
    /**
     * <p>The private IPv4 addresses.</p>
     */
    PrivateIpAddressConfigs?: ScheduledInstancesPrivateIpAddressConfig[];
    /**
     * <p>The number of secondary private IPv4 addresses.</p>
     */
    SecondaryPrivateIpAddressCount?: number;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId?: string;
}
export declare namespace ScheduledInstancesNetworkInterface {
    const filterSensitiveLog: (obj: ScheduledInstancesNetworkInterface) => any;
    const isa: (o: any) => o is ScheduledInstancesNetworkInterface;
}
/**
 * <p>Describes the placement for a Scheduled Instance.</p>
 */
export interface ScheduledInstancesPlacement {
    __type?: "ScheduledInstancesPlacement";
    /**
     * <p>The Availability Zone.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The name of the placement group.</p>
     */
    GroupName?: string;
}
export declare namespace ScheduledInstancesPlacement {
    const filterSensitiveLog: (obj: ScheduledInstancesPlacement) => any;
    const isa: (o: any) => o is ScheduledInstancesPlacement;
}
/**
 * <p>Describes a private IPv4 address for a Scheduled Instance.</p>
 */
export interface ScheduledInstancesPrivateIpAddressConfig {
    __type?: "ScheduledInstancesPrivateIpAddressConfig";
    /**
     * <p>Indicates whether this is a primary IPv4 address. Otherwise, this is a secondary IPv4 address.</p>
     */
    Primary?: boolean;
    /**
     * <p>The IPv4 address.</p>
     */
    PrivateIpAddress?: string;
}
export declare namespace ScheduledInstancesPrivateIpAddressConfig {
    const filterSensitiveLog: (obj: ScheduledInstancesPrivateIpAddressConfig) => any;
    const isa: (o: any) => o is ScheduledInstancesPrivateIpAddressConfig;
}
export declare enum Scope {
    AVAILABILITY_ZONE = "Availability Zone",
    REGIONAL = "Region"
}
export interface SearchLocalGatewayRoutesRequest {
    __type?: "SearchLocalGatewayRoutesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters.</p>
     */
    Filters: Filter[] | undefined;
    /**
     * <p>The ID of the local gateway route table.</p>
     */
    LocalGatewayRouteTableId: string | undefined;
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
}
export declare namespace SearchLocalGatewayRoutesRequest {
    const filterSensitiveLog: (obj: SearchLocalGatewayRoutesRequest) => any;
    const isa: (o: any) => o is SearchLocalGatewayRoutesRequest;
}
export interface SearchLocalGatewayRoutesResult {
    __type?: "SearchLocalGatewayRoutesResult";
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
    /**
     * <p>Information about the routes.</p>
     */
    Routes?: LocalGatewayRoute[];
}
export declare namespace SearchLocalGatewayRoutesResult {
    const filterSensitiveLog: (obj: SearchLocalGatewayRoutesResult) => any;
    const isa: (o: any) => o is SearchLocalGatewayRoutesResult;
}
export interface SearchTransitGatewayMulticastGroupsRequest {
    __type?: "SearchTransitGatewayMulticastGroupsRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>group-ip-address</code> - The IP address of the  transit gateway multicast group.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>is-group-member</code> - The resource is a group member. Valid values are <code>true</code> | <code>false</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>is-group-source</code> - The resource is a group source. Valid values are <code>true</code> | <code>false</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>member-type</code> - The member type. Valid values are <code>igmp</code> | <code>static</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>resource-id</code> - The ID of the resource.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>resource-type</code> - The type of resource. Valid values are <code>vpc</code> | <code>vpn</code> | <code>direct-connect-gateway</code> | <code>tgw-peering</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>source-type</code> - The source type. Valid values are <code>igmp</code> | <code>static</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>state</code> - The state of the subnet association. Valid values are <code>associated</code> | <code>associated</code> | <code>disassociated</code> | <code>disassociating</code>.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>subnet-id</code> - The ID of the subnet.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>transit-gateway-attachment-id</code> - The id of the transit gateway attachment.</p>
     *             </li>
     *          </ul>
     */
    Filters?: Filter[];
    /**
     * <p>The maximum number of results to return with a single call.
     * 	To retrieve the remaining results, make another call with the returned <code>nextToken</code> value.</p>
     */
    MaxResults?: number;
    /**
     * <p>The token for the next page of results.</p>
     */
    NextToken?: string;
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace SearchTransitGatewayMulticastGroupsRequest {
    const filterSensitiveLog: (obj: SearchTransitGatewayMulticastGroupsRequest) => any;
    const isa: (o: any) => o is SearchTransitGatewayMulticastGroupsRequest;
}
export interface SearchTransitGatewayMulticastGroupsResult {
    __type?: "SearchTransitGatewayMulticastGroupsResult";
    /**
     * <p>Information about the  transit gateway multicast group.</p>
     */
    MulticastGroups?: TransitGatewayMulticastGroup[];
    /**
     * <p>The token to use to retrieve the next page of results. This value is <code>null</code> when there are no more results to return.</p>
     */
    NextToken?: string;
}
export declare namespace SearchTransitGatewayMulticastGroupsResult {
    const filterSensitiveLog: (obj: SearchTransitGatewayMulticastGroupsResult) => any;
    const isa: (o: any) => o is SearchTransitGatewayMulticastGroupsResult;
}
export interface SearchTransitGatewayRoutesRequest {
    __type?: "SearchTransitGatewayRoutesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>One or more filters. The possible values are:</p>
     *          <ul>
     *             <li>
     *                <p>
     *                   <code>attachment.transit-gateway-attachment-id</code>- The id of the transit gateway attachment.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>attachment.resource-id</code> - The resource id of the transit gateway attachment.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>attachment.resource-type</code> - The attachment resource type (<code>vpc</code> | <code>vpn</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>route-search.exact-match</code> - The exact match of the specified filter.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>route-search.longest-prefix-match</code> - The longest prefix that matches the route.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>route-search.subnet-of-match</code> - The routes with a subnet that match the specified CIDR filter.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>route-search.supernet-of-match</code> - The routes with a CIDR that encompass the CIDR filter. For example, if you have 10.0.1.0/29 and 10.0.1.0/31 routes in your route table and you specify supernet-of-match as 10.0.1.0/30, then the result returns 10.0.1.0/29.</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>state</code> - The state of the route (<code>active</code> | <code>blackhole</code>).</p>
     *             </li>
     *             <li>
     *                <p>
     *                   <code>type</code> - The type of route (<code>propagated</code> |
     *                   <code>static</code>).</p>
     *             </li>
     *          </ul>
     */
    Filters: Filter[] | undefined;
    /**
     * <p>The maximum number of routes to return.</p>
     */
    MaxResults?: number;
    /**
     * <p>The ID of the transit gateway route table.</p>
     */
    TransitGatewayRouteTableId: string | undefined;
}
export declare namespace SearchTransitGatewayRoutesRequest {
    const filterSensitiveLog: (obj: SearchTransitGatewayRoutesRequest) => any;
    const isa: (o: any) => o is SearchTransitGatewayRoutesRequest;
}
export interface SearchTransitGatewayRoutesResult {
    __type?: "SearchTransitGatewayRoutesResult";
    /**
     * <p>Indicates whether there are additional routes available.</p>
     */
    AdditionalRoutesAvailable?: boolean;
    /**
     * <p>Information about the routes.</p>
     */
    Routes?: TransitGatewayRoute[];
}
export declare namespace SearchTransitGatewayRoutesResult {
    const filterSensitiveLog: (obj: SearchTransitGatewayRoutesResult) => any;
    const isa: (o: any) => o is SearchTransitGatewayRoutesResult;
}
/**
 * <p>Describes a security group</p>
 */
export interface SecurityGroup {
    __type?: "SecurityGroup";
    /**
     * <p>A description of the security group.</p>
     */
    Description?: string;
    /**
     * <p>The ID of the security group.</p>
     */
    GroupId?: string;
    /**
     * <p>The name of the security group.</p>
     */
    GroupName?: string;
    /**
     * <p>The inbound rules associated with the security group.</p>
     */
    IpPermissions?: IpPermission[];
    /**
     * <p>[VPC only] The outbound rules associated with the security group.</p>
     */
    IpPermissionsEgress?: IpPermission[];
    /**
     * <p>The AWS account ID of the owner of the security group.</p>
     */
    OwnerId?: string;
    /**
     * <p>Any tags assigned to the security group.</p>
     */
    Tags?: Tag[];
    /**
     * <p>[VPC only] The ID of the VPC for the security group.</p>
     */
    VpcId?: string;
}
export declare namespace SecurityGroup {
    const filterSensitiveLog: (obj: SecurityGroup) => any;
    const isa: (o: any) => o is SecurityGroup;
}
/**
 * <p>Describes a security group.</p>
 */
export interface SecurityGroupIdentifier {
    __type?: "SecurityGroupIdentifier";
    /**
     * <p>The ID of the security group.</p>
     */
    GroupId?: string;
    /**
     * <p>The name of the security group.</p>
     */
    GroupName?: string;
}
export declare namespace SecurityGroupIdentifier {
    const filterSensitiveLog: (obj: SecurityGroupIdentifier) => any;
    const isa: (o: any) => o is SecurityGroupIdentifier;
}
/**
 * <p>Describes a VPC with a security group that references your security group.</p>
 */
export interface SecurityGroupReference {
    __type?: "SecurityGroupReference";
    /**
     * <p>The ID of your security group.</p>
     */
    GroupId?: string;
    /**
     * <p>The ID of the VPC with the referencing security group.</p>
     */
    ReferencingVpcId?: string;
    /**
     * <p>The ID of the VPC peering connection.</p>
     */
    VpcPeeringConnectionId?: string;
}
export declare namespace SecurityGroupReference {
    const filterSensitiveLog: (obj: SecurityGroupReference) => any;
    const isa: (o: any) => o is SecurityGroupReference;
}
export interface SendDiagnosticInterruptRequest {
    __type?: "SendDiagnosticInterruptRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId: string | undefined;
}
export declare namespace SendDiagnosticInterruptRequest {
    const filterSensitiveLog: (obj: SendDiagnosticInterruptRequest) => any;
    const isa: (o: any) => o is SendDiagnosticInterruptRequest;
}
/**
 * <p>Describes a service configuration for a VPC endpoint service.</p>
 */
export interface ServiceConfiguration {
    __type?: "ServiceConfiguration";
    /**
     * <p>Indicates whether requests from other AWS accounts to create an endpoint to the service must first be accepted.</p>
     */
    AcceptanceRequired?: boolean;
    /**
     * <p>The Availability Zones in which the service is available.</p>
     */
    AvailabilityZones?: string[];
    /**
     * <p>The DNS names for the service.</p>
     */
    BaseEndpointDnsNames?: string[];
    /**
     * <p>Indicates whether the service manages its VPC endpoints. Management of the service VPC
     *             endpoints using the VPC endpoint API is restricted.</p>
     */
    ManagesVpcEndpoints?: boolean;
    /**
     * <p>The Amazon Resource Names (ARNs) of the Network Load Balancers for the service.</p>
     */
    NetworkLoadBalancerArns?: string[];
    /**
     * <p>The private DNS name for the service.</p>
     */
    PrivateDnsName?: string;
    /**
     * <p>Information about the endpoint service private DNS name configuration.</p>
     */
    PrivateDnsNameConfiguration?: PrivateDnsNameConfiguration;
    /**
     * <p>The ID of the service.</p>
     */
    ServiceId?: string;
    /**
     * <p>The name of the service.</p>
     */
    ServiceName?: string;
    /**
     * <p>The service state.</p>
     */
    ServiceState?: ServiceState | string;
    /**
     * <p>The type of service.</p>
     */
    ServiceType?: ServiceTypeDetail[];
    /**
     * <p>Any tags assigned to the service.</p>
     */
    Tags?: Tag[];
}
export declare namespace ServiceConfiguration {
    const filterSensitiveLog: (obj: ServiceConfiguration) => any;
    const isa: (o: any) => o is ServiceConfiguration;
}
/**
 * <p>Describes a VPC endpoint service.</p>
 */
export interface ServiceDetail {
    __type?: "ServiceDetail";
    /**
     * <p>Indicates whether VPC endpoint connection requests to the service must be accepted by the service owner.</p>
     */
    AcceptanceRequired?: boolean;
    /**
     * <p>The Availability Zones in which the service is available.</p>
     */
    AvailabilityZones?: string[];
    /**
     * <p>The DNS names for the service.</p>
     */
    BaseEndpointDnsNames?: string[];
    /**
     * <p>Indicates whether the service manages its VPC endpoints. Management of the service VPC
     *             endpoints using the VPC endpoint API is restricted.</p>
     */
    ManagesVpcEndpoints?: boolean;
    /**
     * <p>The AWS account ID of the service owner.</p>
     */
    Owner?: string;
    /**
     * <p>The private DNS name for the service.</p>
     */
    PrivateDnsName?: string;
    /**
     * <p>The verification state of the VPC endpoint service.</p>
     *         <p>Consumers of the endpoint service cannot use the private name when the state is not <code>verified</code>.</p>
     */
    PrivateDnsNameVerificationState?: DnsNameState | string;
    /**
     * <p>The ID of the endpoint service.</p>
     */
    ServiceId?: string;
    /**
     * <p>The Amazon Resource Name (ARN) of the service.</p>
     */
    ServiceName?: string;
    /**
     * <p>The type of service.</p>
     */
    ServiceType?: ServiceTypeDetail[];
    /**
     * <p>Any tags assigned to the service.</p>
     */
    Tags?: Tag[];
    /**
     * <p>Indicates whether the service supports endpoint policies.</p>
     */
    VpcEndpointPolicySupported?: boolean;
}
export declare namespace ServiceDetail {
    const filterSensitiveLog: (obj: ServiceDetail) => any;
    const isa: (o: any) => o is ServiceDetail;
}
export declare enum ServiceState {
    Available = "Available",
    Deleted = "Deleted",
    Deleting = "Deleting",
    Failed = "Failed",
    Pending = "Pending"
}
export declare enum ServiceType {
    Gateway = "Gateway",
    Interface = "Interface"
}
/**
 * <p>Describes the type of service for a VPC endpoint.</p>
 */
export interface ServiceTypeDetail {
    __type?: "ServiceTypeDetail";
    /**
     * <p>The type of service.</p>
     */
    ServiceType?: ServiceType | string;
}
export declare namespace ServiceTypeDetail {
    const filterSensitiveLog: (obj: ServiceTypeDetail) => any;
    const isa: (o: any) => o is ServiceTypeDetail;
}
export declare type ShutdownBehavior = "stop" | "terminate";
/**
 * <p>Describes the time period for a Scheduled Instance to start its first schedule. The time period must span less than one day.</p>
 */
export interface SlotDateTimeRangeRequest {
    __type?: "SlotDateTimeRangeRequest";
    /**
     * <p>The earliest date and time, in UTC, for the Scheduled Instance to start.</p>
     */
    EarliestTime: Date | undefined;
    /**
     * <p>The latest date and time, in UTC, for the Scheduled Instance to start. This value must be later than or equal to the earliest date and at most three months in the future.</p>
     */
    LatestTime: Date | undefined;
}
export declare namespace SlotDateTimeRangeRequest {
    const filterSensitiveLog: (obj: SlotDateTimeRangeRequest) => any;
    const isa: (o: any) => o is SlotDateTimeRangeRequest;
}
/**
 * <p>Describes the time period for a Scheduled Instance to start its first schedule.</p>
 */
export interface SlotStartTimeRangeRequest {
    __type?: "SlotStartTimeRangeRequest";
    /**
     * <p>The earliest date and time, in UTC, for the Scheduled Instance to start.</p>
     */
    EarliestTime?: Date;
    /**
     * <p>The latest date and time, in UTC, for the Scheduled Instance to start.</p>
     */
    LatestTime?: Date;
}
export declare namespace SlotStartTimeRangeRequest {
    const filterSensitiveLog: (obj: SlotStartTimeRangeRequest) => any;
    const isa: (o: any) => o is SlotStartTimeRangeRequest;
}
/**
 * <p>Describes a snapshot.</p>
 */
export interface Snapshot {
    __type?: "Snapshot";
    /**
     * <p>The data encryption key identifier for the snapshot. This value is a unique identifier
     *       that corresponds to the data encryption key that was used to encrypt the original volume or
     *       snapshot copy. Because data encryption keys are inherited by volumes created from snapshots,
     *       and vice versa, if snapshots share the same data encryption key identifier, then they belong
     *       to the same volume/snapshot lineage. This parameter is only returned by <a>DescribeSnapshots</a>.</p>
     */
    DataEncryptionKeyId?: string;
    /**
     * <p>The description for the snapshot.</p>
     */
    Description?: string;
    /**
     * <p>Indicates whether the snapshot is encrypted.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>The Amazon Resource Name (ARN) of the AWS Key Management Service (AWS KMS) customer master key (CMK) that was used to protect the
     *       volume encryption key for the parent volume.</p>
     */
    KmsKeyId?: string;
    /**
     * <p> Value from an Amazon-maintained list (<code>amazon</code> | <code>self</code> |
     *         <code>all</code> | <code>aws-marketplace</code> | <code>microsoft</code>) of snapshot
     *       owners. Not to be confused with the user-configured AWS account alias, which is set from the
     *       IAM console. </p>
     */
    OwnerAlias?: string;
    /**
     * <p>The AWS account ID of the EBS snapshot owner.</p>
     */
    OwnerId?: string;
    /**
     * <p>The progress of the snapshot, as a percentage.</p>
     */
    Progress?: string;
    /**
     * <p>The ID of the snapshot. Each snapshot receives a unique identifier when it is
     *       created.</p>
     */
    SnapshotId?: string;
    /**
     * <p>The time stamp when the snapshot was initiated.</p>
     */
    StartTime?: Date;
    /**
     * <p>The snapshot state.</p>
     */
    State?: SnapshotState | string;
    /**
     * <p>Encrypted Amazon EBS snapshots are copied asynchronously. If a snapshot copy operation fails
     *       (for example, if the proper AWS Key Management Service (AWS KMS) permissions are not obtained) this field displays error
     *       state details to help you diagnose why the error occurred. This parameter is only returned by
     *       <a>DescribeSnapshots</a>.</p>
     */
    StateMessage?: string;
    /**
     * <p>Any tags assigned to the snapshot.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the volume that was used to create the snapshot. Snapshots created by the <a>CopySnapshot</a> action have an arbitrary volume ID that should not be used for any
     *       purpose.</p>
     */
    VolumeId?: string;
    /**
     * <p>The size of the volume, in GiB.</p>
     */
    VolumeSize?: number;
}
export declare namespace Snapshot {
    const filterSensitiveLog: (obj: Snapshot) => any;
    const isa: (o: any) => o is Snapshot;
}
export declare type SnapshotAttributeName = "createVolumePermission" | "productCodes";
/**
 * <p>Describes the snapshot created from the imported disk.</p>
 */
export interface SnapshotDetail {
    __type?: "SnapshotDetail";
    /**
     * <p>A description for the snapshot.</p>
     */
    Description?: string;
    /**
     * <p>The block device mapping for the snapshot.</p>
     */
    DeviceName?: string;
    /**
     * <p>The size of the disk in the snapshot, in GiB.</p>
     */
    DiskImageSize?: number;
    /**
     * <p>The format of the disk image from which the snapshot is created.</p>
     */
    Format?: string;
    /**
     * <p>The percentage of progress for the task.</p>
     */
    Progress?: string;
    /**
     * <p>The snapshot ID of the disk being imported.</p>
     */
    SnapshotId?: string;
    /**
     * <p>A brief status of the snapshot creation.</p>
     */
    Status?: string;
    /**
     * <p>A detailed status message for the snapshot creation.</p>
     */
    StatusMessage?: string;
    /**
     * <p>The URL used to access the disk image.</p>
     */
    Url?: string;
    /**
     * <p>The S3 bucket for the disk image.</p>
     */
    UserBucket?: UserBucketDetails;
}
export declare namespace SnapshotDetail {
    const filterSensitiveLog: (obj: SnapshotDetail) => any;
    const isa: (o: any) => o is SnapshotDetail;
}
/**
 * <p>The disk container object for the import snapshot request.</p>
 */
export interface SnapshotDiskContainer {
    __type?: "SnapshotDiskContainer";
    /**
     * <p>The description of the disk image being imported.</p>
     */
    Description?: string;
    /**
     * <p>The format of the disk image being imported.</p>
     *          <p>Valid values: <code>VHD</code> | <code>VMDK</code>
     *          </p>
     */
    Format?: string;
    /**
     * <p>The URL to the Amazon S3-based disk image being imported. It can either be a https URL (https://..) or an Amazon
     *    S3 URL (s3://..).</p>
     */
    Url?: string;
    /**
     * <p>The S3 bucket for the disk image.</p>
     */
    UserBucket?: UserBucket;
}
export declare namespace SnapshotDiskContainer {
    const filterSensitiveLog: (obj: SnapshotDiskContainer) => any;
    const isa: (o: any) => o is SnapshotDiskContainer;
}
/**
 * <p>Information about a snapshot.</p>
 */
export interface SnapshotInfo {
    __type?: "SnapshotInfo";
    /**
     * <p>Description specified by the CreateSnapshotRequest that has been applied to all
     *     snapshots.</p>
     */
    Description?: string;
    /**
     * <p>Indicates whether the snapshot is encrypted.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>Account id used when creating this snapshot.</p>
     */
    OwnerId?: string;
    /**
     * <p>Progress this snapshot has made towards completing.</p>
     */
    Progress?: string;
    /**
     * <p>Snapshot id that can be used to describe this snapshot.</p>
     */
    SnapshotId?: string;
    /**
     * <p>Time this snapshot was started. This is the same for all snapshots initiated by the
     *     same request.</p>
     */
    StartTime?: Date;
    /**
     * <p>Current state of the snapshot.</p>
     */
    State?: SnapshotState | string;
    /**
     * <p>Tags associated with this snapshot.</p>
     */
    Tags?: Tag[];
    /**
     * <p>Source volume from which this snapshot was created.</p>
     */
    VolumeId?: string;
    /**
     * <p>Size of the volume from which this snapshot was created.</p>
     */
    VolumeSize?: number;
}
export declare namespace SnapshotInfo {
    const filterSensitiveLog: (obj: SnapshotInfo) => any;
    const isa: (o: any) => o is SnapshotInfo;
}
export declare type SnapshotState = "completed" | "error" | "pending";
/**
 * <p>Details about the import snapshot task.</p>
 */
export interface SnapshotTaskDetail {
    __type?: "SnapshotTaskDetail";
    /**
     * <p>The description of the snapshot.</p>
     */
    Description?: string;
    /**
     * <p>The size of the disk in the snapshot, in GiB.</p>
     */
    DiskImageSize?: number;
    /**
     * <p>Indicates whether the snapshot is encrypted.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>The format of the disk image from which the snapshot is created.</p>
     */
    Format?: string;
    /**
     * <p>The identifier for the AWS Key Management Service (AWS KMS) customer master key (CMK) that was used to create
     *    the encrypted snapshot.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>The percentage of completion for the import snapshot task.</p>
     */
    Progress?: string;
    /**
     * <p>The snapshot ID of the disk being imported.</p>
     */
    SnapshotId?: string;
    /**
     * <p>A brief status for the import snapshot task.</p>
     */
    Status?: string;
    /**
     * <p>A detailed status message for the import snapshot task.</p>
     */
    StatusMessage?: string;
    /**
     * <p>The URL of the disk image from which the snapshot is created.</p>
     */
    Url?: string;
    /**
     * <p>The S3 bucket for the disk image.</p>
     */
    UserBucket?: UserBucketDetails;
}
export declare namespace SnapshotTaskDetail {
    const filterSensitiveLog: (obj: SnapshotTaskDetail) => any;
    const isa: (o: any) => o is SnapshotTaskDetail;
}
export declare enum SpotAllocationStrategy {
    CAPACITY_OPTIMIZED = "capacity-optimized",
    DIVERSIFIED = "diversified",
    LOWEST_PRICE = "lowest-price"
}
/**
 * <p>Describes the data feed for a Spot Instance.</p>
 */
export interface SpotDatafeedSubscription {
    __type?: "SpotDatafeedSubscription";
    /**
     * <p>The Amazon S3 bucket where the Spot Instance data feed is located.</p>
     */
    Bucket?: string;
    /**
     * <p>The fault codes for the Spot Instance request, if any.</p>
     */
    Fault?: SpotInstanceStateFault;
    /**
     * <p>The AWS account ID of the account.</p>
     */
    OwnerId?: string;
    /**
     * <p>The prefix that is prepended to data feed files.</p>
     */
    Prefix?: string;
    /**
     * <p>The state of the Spot Instance data feed subscription.</p>
     */
    State?: DatafeedSubscriptionState | string;
}
export declare namespace SpotDatafeedSubscription {
    const filterSensitiveLog: (obj: SpotDatafeedSubscription) => any;
    const isa: (o: any) => o is SpotDatafeedSubscription;
}
/**
 * <p>Describes the launch specification for one or more Spot Instances. If you include
 *             On-Demand capacity in your fleet request, you can't use
 *                 <code>SpotFleetLaunchSpecification</code>; you must use <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_LaunchTemplateConfig.html">LaunchTemplateConfig</a>.</p>
 */
export interface SpotFleetLaunchSpecification {
    __type?: "SpotFleetLaunchSpecification";
    /**
     * <p>Deprecated.</p>
     */
    AddressingType?: string;
    /**
     * <p>One or more block devices that are mapped to the Spot Instances. You can't specify both
     *             a snapshot ID and an encryption value. This is because only blank volumes can be
     *             encrypted on creation. If a snapshot is the basis for a volume, it is not blank and its
     *             encryption status is used for the volume encryption status.</p>
     */
    BlockDeviceMappings?: BlockDeviceMapping[];
    /**
     * <p>Indicates whether the instances are optimized for EBS I/O. This optimization provides dedicated throughput to Amazon EBS and an optimized configuration stack to provide optimal EBS I/O performance. This optimization isn't available with all instance types. Additional usage charges apply when using an EBS Optimized instance.</p>
     *          <p>Default: <code>false</code>
     *          </p>
     */
    EbsOptimized?: boolean;
    /**
     * <p>The IAM instance profile.</p>
     */
    IamInstanceProfile?: IamInstanceProfileSpecification;
    /**
     * <p>The ID of the AMI.</p>
     */
    ImageId?: string;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>The ID of the kernel.</p>
     */
    KernelId?: string;
    /**
     * <p>The name of the key pair.</p>
     */
    KeyName?: string;
    /**
     * <p>Enable or disable monitoring for the instances.</p>
     */
    Monitoring?: SpotFleetMonitoring;
    /**
     * <p>One or more network interfaces. If you specify a network interface, you must specify
     *           subnet IDs and security group IDs using the network interface.</p>
     */
    NetworkInterfaces?: InstanceNetworkInterfaceSpecification[];
    /**
     * <p>The placement information.</p>
     */
    Placement?: SpotPlacement;
    /**
     * <p>The ID of the RAM disk. Some kernels require additional drivers at launch. Check the kernel requirements for information about whether you need to specify a RAM disk. To find kernel requirements, refer to the AWS Resource Center and search for the kernel ID.</p>
     */
    RamdiskId?: string;
    /**
     * <p>One or more security groups. When requesting instances in a VPC, you must specify the IDs of the security groups. When requesting instances in EC2-Classic, you can specify the names or the IDs of the security groups.</p>
     */
    SecurityGroups?: GroupIdentifier[];
    /**
     * <p>The maximum price per unit hour that you are willing to pay for a Spot Instance.
     *           If this value is not specified, the default is the Spot price specified for the fleet.
     *           To determine the Spot price per unit hour, divide the Spot price by the
     *           value of <code>WeightedCapacity</code>.</p>
     */
    SpotPrice?: string;
    /**
     * <p>The IDs of the subnets in which to launch the instances. To specify multiple subnets, separate
     *           them using commas; for example, "subnet-1234abcdeexample1, subnet-0987cdef6example2".</p>
     */
    SubnetId?: string;
    /**
     * <p>The tags to apply during creation.</p>
     */
    TagSpecifications?: SpotFleetTagSpecification[];
    /**
     * <p>The Base64-encoded user data that instances use when starting up.</p>
     */
    UserData?: string;
    /**
     * <p>The number of units provided by the specified instance type. These are the same units that you chose to set the target capacity in terms of instances, or a performance characteristic such as vCPUs, memory, or I/O.</p>
     *          <p>If the target capacity divided by this value is not a whole number, Amazon EC2 rounds the number of instances to the next whole number. If this value is not specified, the default is 1.</p>
     */
    WeightedCapacity?: number;
}
export declare namespace SpotFleetLaunchSpecification {
    const filterSensitiveLog: (obj: SpotFleetLaunchSpecification) => any;
    const isa: (o: any) => o is SpotFleetLaunchSpecification;
}
/**
 * <p>Describes whether monitoring is enabled.</p>
 */
export interface SpotFleetMonitoring {
    __type?: "SpotFleetMonitoring";
    /**
     * <p>Enables monitoring for the instance.</p>
     *          <p>Default: <code>false</code>
     *          </p>
     */
    Enabled?: boolean;
}
export declare namespace SpotFleetMonitoring {
    const filterSensitiveLog: (obj: SpotFleetMonitoring) => any;
    const isa: (o: any) => o is SpotFleetMonitoring;
}
/**
 * <p>Describes a Spot Fleet request.</p>
 */
export interface SpotFleetRequestConfig {
    __type?: "SpotFleetRequestConfig";
    /**
     * <p>The progress of the Spot Fleet request.
     *           If there is an error, the status is <code>error</code>.
     *           After all requests are placed, the status is <code>pending_fulfillment</code>.
     *           If the size of the fleet is equal to or greater than its target capacity, the status is <code>fulfilled</code>.
     *           If the size of the fleet is decreased, the status is <code>pending_termination</code>
     *           while Spot Instances are terminating.</p>
     */
    ActivityStatus?: ActivityStatus | string;
    /**
     * <p>The creation date and time of the request.</p>
     */
    CreateTime?: Date;
    /**
     * <p>The configuration of the Spot Fleet request.</p>
     */
    SpotFleetRequestConfig?: SpotFleetRequestConfigData;
    /**
     * <p>The ID of the Spot Fleet request.</p>
     */
    SpotFleetRequestId?: string;
    /**
     * <p>The state of the Spot Fleet request.</p>
     */
    SpotFleetRequestState?: BatchState | string;
}
export declare namespace SpotFleetRequestConfig {
    const filterSensitiveLog: (obj: SpotFleetRequestConfig) => any;
    const isa: (o: any) => o is SpotFleetRequestConfig;
}
/**
 * <p>Describes the configuration of a Spot Fleet request.</p>
 */
export interface SpotFleetRequestConfigData {
    __type?: "SpotFleetRequestConfigData";
    /**
     * <p>Indicates how to allocate the target Spot Instance capacity across the Spot Instance pools specified by
     *             the Spot Fleet request.</p>
     *         <p>If the allocation strategy is <code>lowestPrice</code>, Spot Fleet launches instances from
     *             the Spot Instance pools with the lowest price. This is the default allocation strategy.</p>
     *         <p>If the allocation strategy is <code>diversified</code>, Spot Fleet launches instances from
     *             all the Spot Instance pools that you specify.</p>
     *         <p>If the allocation strategy is <code>capacityOptimized</code>, Spot Fleet launches instances
     *             from Spot Instance pools with optimal capacity for the number of instances that are
     *             launching.</p>
     */
    AllocationStrategy?: AllocationStrategy | string;
    /**
     * <p>A unique, case-sensitive identifier that you provide to ensure the idempotency of
     *             your listings. This helps to avoid duplicate listings. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html">Ensuring Idempotency</a>.</p>
     */
    ClientToken?: string;
    /**
     * <p>Indicates whether running Spot Instances should be terminated if you decrease the target
     *             capacity of the Spot Fleet request below the current size of the Spot Fleet.</p>
     */
    ExcessCapacityTerminationPolicy?: ExcessCapacityTerminationPolicy | string;
    /**
     * <p>The number of units fulfilled by this request compared to the set target capacity. You cannot set this value.</p>
     */
    FulfilledCapacity?: number;
    /**
     * <p>The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role that
     *             grants the Spot Fleet the permission to request, launch, terminate, and tag instances on
     *             your behalf. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-fleet-requests.html#spot-fleet-prerequisites">Spot
     *                 Fleet Prerequisites</a> in the <i>Amazon EC2 User Guide for Linux Instances</i>. Spot Fleet
     *             can terminate Spot Instances on your behalf when you cancel its Spot Fleet request using
     *                 <a>CancelSpotFleetRequests</a> or when the Spot Fleet request expires, if
     *             you set <code>TerminateInstancesWithExpiration</code>.</p>
     */
    IamFleetRole: string | undefined;
    /**
     * <p>The behavior when a Spot Instance is interrupted. The default is <code>terminate</code>.</p>
     */
    InstanceInterruptionBehavior?: InstanceInterruptionBehavior | string;
    /**
     * <p>The number of Spot pools across which to allocate your target Spot capacity. Valid
     *             only when Spot <b>AllocationStrategy</b> is set to
     *                 <code>lowest-price</code>. Spot Fleet selects the cheapest Spot pools and evenly
     *             allocates your target Spot capacity across the number of Spot pools that you
     *             specify.</p>
     */
    InstancePoolsToUseCount?: number;
    /**
     * <p>The launch specifications for the Spot Fleet request. If you specify
     *                 <code>LaunchSpecifications</code>, you can't specify
     *                 <code>LaunchTemplateConfigs</code>. If you include On-Demand capacity in your
     *             request, you must use <code>LaunchTemplateConfigs</code>.</p>
     */
    LaunchSpecifications?: SpotFleetLaunchSpecification[];
    /**
     * <p>The launch template and overrides. If you specify <code>LaunchTemplateConfigs</code>,
     *             you can't specify <code>LaunchSpecifications</code>. If you include On-Demand capacity
     *             in your request, you must use <code>LaunchTemplateConfigs</code>.</p>
     */
    LaunchTemplateConfigs?: LaunchTemplateConfig[];
    /**
     * <p>One or more Classic Load Balancers and target groups to attach to the Spot Fleet request.
     *           Spot Fleet registers the running Spot Instances with the specified Classic Load Balancers and target groups.</p>
     *          <p>With Network Load Balancers, Spot Fleet cannot register instances that have the following instance types:
     *           C1, CC1, CC2, CG1, CG2, CR1, CS1, G1, G2, HI1, HS1, M1, M2, M3, and T1.</p>
     */
    LoadBalancersConfig?: LoadBalancersConfig;
    /**
     * <p>The order of the launch template overrides to use in fulfilling On-Demand capacity.
     *             If you specify <code>lowestPrice</code>, Spot Fleet uses price to determine the order,
     *             launching the lowest price first. If you specify <code>prioritized</code>, Spot Fleet uses the
     *             priority that you assign to each Spot Fleet launch template override, launching the highest
     *             priority first. If you do not specify a value, Spot Fleet defaults to
     *             <code>lowestPrice</code>.</p>
     */
    OnDemandAllocationStrategy?: OnDemandAllocationStrategy | string;
    /**
     * <p>The number of On-Demand units fulfilled by this request compared to the set target On-Demand capacity.</p>
     */
    OnDemandFulfilledCapacity?: number;
    /**
     * <p>The maximum amount per hour for On-Demand Instances that you're willing to pay.
     *             You can use the <code>onDemandMaxTotalPrice</code> parameter, the <code>spotMaxTotalPrice</code>
     *             parameter, or both parameters to ensure that your fleet cost does not exceed
     *             your budget. If you set a maximum price per hour for the On-Demand Instances and
     *             Spot Instances in your request, Spot Fleet will launch instances until it reaches
     *             the maximum amount you're willing to pay. When the maximum amount you're willing to pay is reached,
     *             the fleet stops launching instances even if it hasn’t met the target capacity.</p>
     */
    OnDemandMaxTotalPrice?: string;
    /**
     * <p>The number of On-Demand units to request. You can choose to set the target capacity
     *             in terms of instances or a performance characteristic that is important to your
     *             application workload, such as vCPUs, memory, or I/O. If the request type is
     *                 <code>maintain</code>, you can specify a target capacity of 0 and add capacity
     *             later.</p>
     */
    OnDemandTargetCapacity?: number;
    /**
     * <p>Indicates whether Spot Fleet should replace unhealthy instances.</p>
     */
    ReplaceUnhealthyInstances?: boolean;
    /**
     * <p>The maximum amount per hour for Spot Instances that you're willing to pay.
     *             You can use the <code>spotdMaxTotalPrice</code> parameter, the <code>onDemandMaxTotalPrice</code>
     *             parameter, or both parameters to ensure that your fleet cost does not exceed
     *             your budget. If you set a maximum price per hour for the On-Demand Instances and
     *             Spot Instances in your request, Spot Fleet will launch instances until it reaches
     *             the maximum amount you're willing to pay. When the maximum amount you're willing to pay is reached,
     *             the fleet stops launching instances even if it hasn’t met the target capacity.</p>
     */
    SpotMaxTotalPrice?: string;
    /**
     * <p>The maximum price per unit hour that you are willing to pay for a Spot Instance. The default is the On-Demand price.</p>
     */
    SpotPrice?: string;
    /**
     * <p>The number of units to request for the Spot Fleet. You can choose to set the target
     *             capacity in terms of instances or a performance characteristic that is important to your
     *             application workload, such as vCPUs, memory, or I/O. If the request type is
     *                 <code>maintain</code>, you can specify a target capacity of 0 and add capacity
     *             later.</p>
     */
    TargetCapacity: number | undefined;
    /**
     * <p>Indicates whether running Spot Instances are terminated when the Spot Fleet request
     *             expires.</p>
     */
    TerminateInstancesWithExpiration?: boolean;
    /**
     * <p>The type of request. Indicates whether the Spot Fleet only requests the target capacity
     *             or also attempts to maintain it. When this value is <code>request</code>, the Spot Fleet
     *             only places the required requests. It does not attempt to replenish Spot Instances if
     *             capacity is diminished, nor does it submit requests in alternative Spot pools if
     *             capacity is not available. When this value is <code>maintain</code>, the Spot Fleet
     *             maintains the target capacity. The Spot Fleet places the required requests to meet
     *             capacity and automatically replenishes any interrupted instances. Default:
     *                 <code>maintain</code>. <code>instant</code> is listed but is not used by Spot
     *             Fleet.</p>
     */
    Type?: FleetType | string;
    /**
     * <p>The start date and time of the request, in UTC format
     *                 (<i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).
     *             By default, Amazon EC2 starts fulfilling the request immediately.</p>
     */
    ValidFrom?: Date;
    /**
     * <p>The end date and time of the request, in UTC format
     *                 (<i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).
     *             After the end date and time, no new Spot Instance requests are placed or able to fulfill the request.
     *             If no value is specified, the Spot Fleet request remains until you cancel it.</p>
     */
    ValidUntil?: Date;
}
export declare namespace SpotFleetRequestConfigData {
    const filterSensitiveLog: (obj: SpotFleetRequestConfigData) => any;
    const isa: (o: any) => o is SpotFleetRequestConfigData;
}
/**
 * <p>The tags for a Spot Fleet resource.</p>
 */
export interface SpotFleetTagSpecification {
    __type?: "SpotFleetTagSpecification";
    /**
     * <p>The type of resource. Currently, the only resource type that is supported is <code>instance</code>.</p>
     */
    ResourceType?: ResourceType | string;
    /**
     * <p>The tags.</p>
     */
    Tags?: Tag[];
}
export declare namespace SpotFleetTagSpecification {
    const filterSensitiveLog: (obj: SpotFleetTagSpecification) => any;
    const isa: (o: any) => o is SpotFleetTagSpecification;
}
export declare type SpotInstanceInterruptionBehavior = "hibernate" | "stop" | "terminate";
/**
 * <p>Describes a Spot Instance request.</p>
 */
export interface SpotInstanceRequest {
    __type?: "SpotInstanceRequest";
    /**
     * <p>If you specified a duration and your Spot Instance request was fulfilled, this is the fixed hourly price in effect for the Spot Instance while it runs.</p>
     */
    ActualBlockHourlyPrice?: string;
    /**
     * <p>The Availability Zone group. If you specify the same Availability Zone group for all Spot Instance requests, all Spot Instances are launched in the same Availability Zone.</p>
     */
    AvailabilityZoneGroup?: string;
    /**
     * <p>The duration for the Spot Instance, in minutes.</p>
     */
    BlockDurationMinutes?: number;
    /**
     * <p>The date and time when the Spot Instance request was created, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    CreateTime?: Date;
    /**
     * <p>The fault codes for the Spot Instance request, if any.</p>
     */
    Fault?: SpotInstanceStateFault;
    /**
     * <p>The instance ID, if an instance has been launched to fulfill the Spot Instance request.</p>
     */
    InstanceId?: string;
    /**
     * <p>The behavior when a Spot Instance is interrupted.</p>
     */
    InstanceInterruptionBehavior?: InstanceInterruptionBehavior | string;
    /**
     * <p>The instance launch group. Launch groups are Spot Instances that launch together and terminate together.</p>
     */
    LaunchGroup?: string;
    /**
     * <p>Additional information for launching instances.</p>
     */
    LaunchSpecification?: LaunchSpecification;
    /**
     * <p>The Availability Zone in which the request is launched.</p>
     */
    LaunchedAvailabilityZone?: string;
    /**
     * <p>The product description associated with the Spot Instance.</p>
     */
    ProductDescription?: RIProductDescription | string;
    /**
     * <p>The ID of the Spot Instance request.</p>
     */
    SpotInstanceRequestId?: string;
    /**
     * <p>The maximum price per hour that you are willing to pay for a Spot Instance.</p>
     */
    SpotPrice?: string;
    /**
     * <p>The state of the Spot Instance request. Spot status information helps track your Spot
     *             Instance requests. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-bid-status.html">Spot Status</a> in the
     *                 <i>Amazon EC2 User Guide for Linux Instances</i>.</p>
     */
    State?: SpotInstanceState | string;
    /**
     * <p>The status code and status message describing the Spot Instance request.</p>
     */
    Status?: SpotInstanceStatus;
    /**
     * <p>Any tags assigned to the resource.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The Spot Instance request type.</p>
     */
    Type?: SpotInstanceType | string;
    /**
     * <p>The start date of the request, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).
     *         The request becomes active at this date and time.</p>
     */
    ValidFrom?: Date;
    /**
     * <p>The end date of the request, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).
     *         If this is a one-time request, it remains active until all instances launch, the request is canceled, or this date is reached.
     * 		If the request is persistent, it remains active until it is canceled or this date is reached.
     *         The default end date is 7 days from the current date.</p>
     */
    ValidUntil?: Date;
}
export declare namespace SpotInstanceRequest {
    const filterSensitiveLog: (obj: SpotInstanceRequest) => any;
    const isa: (o: any) => o is SpotInstanceRequest;
}
export declare type SpotInstanceState = "active" | "cancelled" | "closed" | "failed" | "open";
/**
 * <p>Describes a Spot Instance state change.</p>
 */
export interface SpotInstanceStateFault {
    __type?: "SpotInstanceStateFault";
    /**
     * <p>The reason code for the Spot Instance state change.</p>
     */
    Code?: string;
    /**
     * <p>The message for the Spot Instance state change.</p>
     */
    Message?: string;
}
export declare namespace SpotInstanceStateFault {
    const filterSensitiveLog: (obj: SpotInstanceStateFault) => any;
    const isa: (o: any) => o is SpotInstanceStateFault;
}
/**
 * <p>Describes the status of a Spot Instance request.</p>
 */
export interface SpotInstanceStatus {
    __type?: "SpotInstanceStatus";
    /**
     * <p>The status code. For a list of status codes, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-bid-status.html#spot-instance-bid-status-understand">Spot Status Codes</a>
     *         in the <i>Amazon EC2 User Guide for Linux Instances</i>.</p>
     */
    Code?: string;
    /**
     * <p>The description for the status code.</p>
     */
    Message?: string;
    /**
     * <p>The date and time of the most recent status update, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    UpdateTime?: Date;
}
export declare namespace SpotInstanceStatus {
    const filterSensitiveLog: (obj: SpotInstanceStatus) => any;
    const isa: (o: any) => o is SpotInstanceStatus;
}
export declare type SpotInstanceType = "one-time" | "persistent";
/**
 * <p>The options for Spot Instances.</p>
 */
export interface SpotMarketOptions {
    __type?: "SpotMarketOptions";
    /**
     * <p>The required duration for the Spot Instances (also known as Spot blocks), in minutes.
     *             This value must be a multiple of 60 (60, 120, 180, 240, 300, or 360).</p>
     */
    BlockDurationMinutes?: number;
    /**
     * <p>The behavior when a Spot Instance is interrupted. The default is
     *                 <code>terminate</code>.</p>
     */
    InstanceInterruptionBehavior?: InstanceInterruptionBehavior | string;
    /**
     * <p>The maximum hourly price you're willing to pay for the Spot Instances. The default is
     *             the On-Demand price.</p>
     */
    MaxPrice?: string;
    /**
     * <p>The Spot Instance request type. For <a>RunInstances</a>, persistent Spot
     *             Instance requests are only supported when <b>InstanceInterruptionBehavior</b> is set to either <code>hibernate</code> or
     *                 <code>stop</code>.</p>
     */
    SpotInstanceType?: SpotInstanceType | string;
    /**
     * <p>The end date of the request. For a one-time request, the request remains active until
     *             all instances launch, the request is canceled, or this date is reached. If the request
     *             is persistent, it remains active until it is canceled or this date and time is reached.
     *             The default end date is 7 days from the current date.</p>
     */
    ValidUntil?: Date;
}
export declare namespace SpotMarketOptions {
    const filterSensitiveLog: (obj: SpotMarketOptions) => any;
    const isa: (o: any) => o is SpotMarketOptions;
}
/**
 * <p>Describes the configuration of Spot Instances in an EC2 Fleet.</p>
 */
export interface SpotOptions {
    __type?: "SpotOptions";
    /**
     * <p>Indicates how to allocate the target Spot Instance capacity across the Spot Instance pools specified by
     *          the EC2 Fleet.</p>
     *          <p>If the allocation strategy is <code>lowest-price</code>, EC2 Fleet launches instances from
     *          the Spot Instance pools with the lowest price. This is the default allocation strategy.</p>
     *          <p>If the allocation strategy is <code>diversified</code>, EC2 Fleet launches instances from all
     *          of the Spot Instance pools that you specify.</p>
     *          <p>If the allocation strategy is <code>capacity-optimized</code>, EC2 Fleet launches instances
     *          from Spot Instance pools with optimal capacity for the number of instances that are
     *          launching.</p>
     */
    AllocationStrategy?: SpotAllocationStrategy | string;
    /**
     * <p>The behavior when a Spot Instance is interrupted. The default is <code>terminate</code>.</p>
     */
    InstanceInterruptionBehavior?: SpotInstanceInterruptionBehavior | string;
    /**
     * <p>The number of Spot pools across which to allocate your target Spot capacity. Valid only
     *          when <b>AllocationStrategy</b> is set to
     *             <code>lowest-price</code>. EC2 Fleet selects the cheapest Spot pools and evenly allocates
     *          your target Spot capacity across the number of Spot pools that you specify.</p>
     */
    InstancePoolsToUseCount?: number;
    /**
     * <p>The maximum amount per hour for Spot Instances that you're willing to pay.</p>
     */
    MaxTotalPrice?: string;
    /**
     * <p>The minimum target capacity for Spot Instances in the fleet. If the minimum target capacity is
     *          not reached, the fleet launches no instances.</p>
     */
    MinTargetCapacity?: number;
    /**
     * <p>Indicates that the fleet launches all Spot Instances into a single Availability Zone. Supported
     *          only for fleets of type <code>instant</code>.</p>
     */
    SingleAvailabilityZone?: boolean;
    /**
     * <p>Indicates that the fleet uses a single instance type to launch all Spot Instances in the fleet.
     *          Supported only for fleets of type <code>instant</code>.</p>
     */
    SingleInstanceType?: boolean;
}
export declare namespace SpotOptions {
    const filterSensitiveLog: (obj: SpotOptions) => any;
    const isa: (o: any) => o is SpotOptions;
}
/**
 * <p>Describes the configuration of Spot Instances in an EC2 Fleet request.</p>
 */
export interface SpotOptionsRequest {
    __type?: "SpotOptionsRequest";
    /**
     * <p>Indicates how to allocate the target Spot Instance capacity across the Spot Instance pools specified by
     *          the EC2 Fleet.</p>
     *          <p>If the allocation strategy is <code>lowest-price</code>, EC2 Fleet launches instances from
     *          the Spot Instance pools with the lowest price. This is the default allocation strategy.</p>
     *          <p>If the allocation strategy is <code>diversified</code>, EC2 Fleet launches instances from all
     *          of the Spot Instance pools that you specify.</p>
     *          <p>If the allocation strategy is <code>capacity-optimized</code>, EC2 Fleet launches instances
     *          from Spot Instance pools with optimal capacity for the number of instances that are
     *          launching.</p>
     */
    AllocationStrategy?: SpotAllocationStrategy | string;
    /**
     * <p>The behavior when a Spot Instance is interrupted. The default is <code>terminate</code>.</p>
     */
    InstanceInterruptionBehavior?: SpotInstanceInterruptionBehavior | string;
    /**
     * <p>The number of Spot pools across which to allocate your target Spot capacity. Valid only
     *          when Spot <b>AllocationStrategy</b> is set to
     *             <code>lowest-price</code>. EC2 Fleet selects the cheapest Spot pools and evenly allocates
     *          your target Spot capacity across the number of Spot pools that you specify.</p>
     */
    InstancePoolsToUseCount?: number;
    /**
     * <p>The maximum amount per hour for Spot Instances that you're willing to pay.</p>
     */
    MaxTotalPrice?: string;
    /**
     * <p>The minimum target capacity for Spot Instances in the fleet. If the minimum target capacity is
     *          not reached, the fleet launches no instances.</p>
     */
    MinTargetCapacity?: number;
    /**
     * <p>Indicates that the fleet launches all Spot Instances into a single Availability Zone. Supported
     *          only for fleets of type <code>instant</code>.</p>
     */
    SingleAvailabilityZone?: boolean;
    /**
     * <p>Indicates that the fleet uses a single instance type to launch all Spot Instances in the fleet.
     *          Supported only for fleets of type <code>instant</code>.</p>
     */
    SingleInstanceType?: boolean;
}
export declare namespace SpotOptionsRequest {
    const filterSensitiveLog: (obj: SpotOptionsRequest) => any;
    const isa: (o: any) => o is SpotOptionsRequest;
}
/**
 * <p>Describes Spot Instance placement.</p>
 */
export interface SpotPlacement {
    __type?: "SpotPlacement";
    /**
     * <p>The Availability Zone.</p>
     *          <p>[Spot Fleet only] To specify multiple Availability Zones, separate them using commas; for example, "us-west-2a, us-west-2b".</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The name of the placement group.</p>
     */
    GroupName?: string;
    /**
     * <p>The tenancy of the instance (if the instance is running in a VPC). An instance with a tenancy
     *        of <code>dedicated</code> runs on single-tenant hardware. The <code>host</code> tenancy is not
     *        supported for Spot Instances.</p>
     */
    Tenancy?: Tenancy | string;
}
export declare namespace SpotPlacement {
    const filterSensitiveLog: (obj: SpotPlacement) => any;
    const isa: (o: any) => o is SpotPlacement;
}
/**
 * <p>Describes the maximum price per hour that you are willing to pay for a Spot Instance.</p>
 */
export interface SpotPrice {
    __type?: "SpotPrice";
    /**
     * <p>The Availability Zone.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The instance type.</p>
     */
    InstanceType?: _InstanceType | string;
    /**
     * <p>A general description of the AMI.</p>
     */
    ProductDescription?: RIProductDescription | string;
    /**
     * <p>The maximum price per hour that you are willing to pay for a Spot Instance.</p>
     */
    SpotPrice?: string;
    /**
     * <p>The date and time the request was created, in UTC format (for example, <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
     */
    Timestamp?: Date;
}
export declare namespace SpotPrice {
    const filterSensitiveLog: (obj: SpotPrice) => any;
    const isa: (o: any) => o is SpotPrice;
}
/**
 * <p>Describes a stale rule in a security group.</p>
 */
export interface StaleIpPermission {
    __type?: "StaleIpPermission";
    /**
     * <p>The start of the port range for the TCP and UDP protocols, or an ICMP type number. A value of
     *         <code>-1</code> indicates all ICMP types. </p>
     */
    FromPort?: number;
    /**
     * <p>The IP protocol name (for <code>tcp</code>, <code>udp</code>, and <code>icmp</code>) or number  (see <a href="http://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml">Protocol Numbers)</a>.</p>
     */
    IpProtocol?: string;
    /**
     * <p>The IP ranges. Not applicable for stale security group rules.</p>
     */
    IpRanges?: string[];
    /**
     * <p>The prefix list IDs for an AWS service. Not applicable for stale security group rules.</p>
     */
    PrefixListIds?: string[];
    /**
     * <p>The end of the port range for the TCP and UDP protocols, or an ICMP type number. A value of
     *         <code>-1</code> indicates all ICMP types. </p>
     */
    ToPort?: number;
    /**
     * <p>The security group pairs. Returns the ID of the referenced security group and VPC, and the ID and status of the VPC peering connection.</p>
     */
    UserIdGroupPairs?: UserIdGroupPair[];
}
export declare namespace StaleIpPermission {
    const filterSensitiveLog: (obj: StaleIpPermission) => any;
    const isa: (o: any) => o is StaleIpPermission;
}
/**
 * <p>Describes a stale security group (a security group that contains stale rules).</p>
 */
export interface StaleSecurityGroup {
    __type?: "StaleSecurityGroup";
    /**
     * <p>The description of the security group.</p>
     */
    Description?: string;
    /**
     * <p>The ID of the security group.</p>
     */
    GroupId?: string;
    /**
     * <p>The name of the security group.</p>
     */
    GroupName?: string;
    /**
     * <p>Information about the stale inbound rules in the security group.</p>
     */
    StaleIpPermissions?: StaleIpPermission[];
    /**
     * <p>Information about the stale outbound rules in the security group.</p>
     */
    StaleIpPermissionsEgress?: StaleIpPermission[];
    /**
     * <p>The ID of the VPC for the security group.</p>
     */
    VpcId?: string;
}
export declare namespace StaleSecurityGroup {
    const filterSensitiveLog: (obj: StaleSecurityGroup) => any;
    const isa: (o: any) => o is StaleSecurityGroup;
}
export interface StartInstancesRequest {
    __type?: "StartInstancesRequest";
    /**
     * <p>Reserved.</p>
     */
    AdditionalInfo?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the instances.</p>
     */
    InstanceIds: string[] | undefined;
}
export declare namespace StartInstancesRequest {
    const filterSensitiveLog: (obj: StartInstancesRequest) => any;
    const isa: (o: any) => o is StartInstancesRequest;
}
export interface StartInstancesResult {
    __type?: "StartInstancesResult";
    /**
     * <p>Information about the started instances.</p>
     */
    StartingInstances?: InstanceStateChange[];
}
export declare namespace StartInstancesResult {
    const filterSensitiveLog: (obj: StartInstancesResult) => any;
    const isa: (o: any) => o is StartInstancesResult;
}
export interface StartVpcEndpointServicePrivateDnsVerificationRequest {
    __type?: "StartVpcEndpointServicePrivateDnsVerificationRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the endpoint service.</p>
     */
    ServiceId: string | undefined;
}
export declare namespace StartVpcEndpointServicePrivateDnsVerificationRequest {
    const filterSensitiveLog: (obj: StartVpcEndpointServicePrivateDnsVerificationRequest) => any;
    const isa: (o: any) => o is StartVpcEndpointServicePrivateDnsVerificationRequest;
}
export interface StartVpcEndpointServicePrivateDnsVerificationResult {
    __type?: "StartVpcEndpointServicePrivateDnsVerificationResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, it returns an error.</p>
     */
    ReturnValue?: boolean;
}
export declare namespace StartVpcEndpointServicePrivateDnsVerificationResult {
    const filterSensitiveLog: (obj: StartVpcEndpointServicePrivateDnsVerificationResult) => any;
    const isa: (o: any) => o is StartVpcEndpointServicePrivateDnsVerificationResult;
}
export declare enum State {
    Available = "Available",
    Deleted = "Deleted",
    Deleting = "Deleting",
    Expired = "Expired",
    Failed = "Failed",
    Pending = "Pending",
    PendingAcceptance = "PendingAcceptance",
    Rejected = "Rejected"
}
/**
 * <p>Describes a state change.</p>
 */
export interface StateReason {
    __type?: "StateReason";
    /**
     * <p>The reason code for the state change.</p>
     */
    Code?: string;
    /**
     * <p>The message for the state change.</p>
     *         <ul>
     *             <li>
     *                 <p>
     *                   <code>Server.InsufficientInstanceCapacity</code>: There was insufficient
     *                     capacity available to satisfy the launch request.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>Server.InternalError</code>: An internal error caused the instance to
     *                     terminate during launch.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>Server.ScheduledStop</code>: The instance was stopped due to a scheduled
     *                     retirement.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>Server.SpotInstanceShutdown</code>: The instance was stopped because the
     *                     number of Spot requests with a maximum price equal to or higher than the Spot
     *                     price exceeded available capacity or because of an increase in the Spot
     *                     price.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>Server.SpotInstanceTermination</code>: The instance was terminated
     *                     because the number of Spot requests with a maximum price equal to or higher than
     *                     the Spot price exceeded available capacity or because of an increase in the Spot
     *                     price.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>Client.InstanceInitiatedShutdown</code>: The instance was shut down
     *                     using the <code>shutdown -h</code> command from the instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>Client.InstanceTerminated</code>: The instance was terminated or
     *                     rebooted during AMI creation.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>Client.InternalError</code>: A client error caused the instance to
     *                     terminate during launch.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>Client.InvalidSnapshot.NotFound</code>: The specified snapshot was not
     *                     found.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>Client.UserInitiatedHibernate</code>: Hibernation was initiated on the
     *                     instance.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>Client.UserInitiatedShutdown</code>: The instance was shut down using
     *                     the Amazon EC2 API.</p>
     *             </li>
     *             <li>
     *                 <p>
     *                   <code>Client.VolumeLimitExceeded</code>: The limit on the number of EBS
     *                     volumes or total storage was exceeded. Decrease usage or request an increase in
     *                     your account limits.</p>
     *             </li>
     *          </ul>
     */
    Message?: string;
}
export declare namespace StateReason {
    const filterSensitiveLog: (obj: StateReason) => any;
    const isa: (o: any) => o is StateReason;
}
export declare enum Status {
    inClassic = "InClassic",
    inVpc = "InVpc",
    moveInProgress = "MoveInProgress"
}
export declare type StatusName = "reachability";
export declare type StatusType = "failed" | "initializing" | "insufficient-data" | "passed";
export interface StopInstancesRequest {
    __type?: "StopInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>Forces the instances to stop. The instances do not have an opportunity to flush file
     *             system caches or file system metadata. If you use this option, you must perform file
     *             system check and repair procedures. This option is not recommended for Windows
     *             instances.</p>
     *         <p>Default: <code>false</code>
     *         </p>
     */
    Force?: boolean;
    /**
     * <p>Hibernates the instance if the instance was enabled for hibernation at launch. If the
     *             instance cannot hibernate successfully, a normal shutdown occurs. For more information,
     *             see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html">Hibernate
     *                 Your Instance</a> in the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *         <p> Default: <code>false</code>
     *          </p>
     */
    Hibernate?: boolean;
    /**
     * <p>The IDs of the instances.</p>
     */
    InstanceIds: string[] | undefined;
}
export declare namespace StopInstancesRequest {
    const filterSensitiveLog: (obj: StopInstancesRequest) => any;
    const isa: (o: any) => o is StopInstancesRequest;
}
export interface StopInstancesResult {
    __type?: "StopInstancesResult";
    /**
     * <p>Information about the stopped instances.</p>
     */
    StoppingInstances?: InstanceStateChange[];
}
export declare namespace StopInstancesResult {
    const filterSensitiveLog: (obj: StopInstancesResult) => any;
    const isa: (o: any) => o is StopInstancesResult;
}
/**
 * <p>Describes the storage location for an instance store-backed AMI.</p>
 */
export interface Storage {
    __type?: "Storage";
    /**
     * <p>An Amazon S3 storage location.</p>
     */
    S3?: S3Storage;
}
export declare namespace Storage {
    const filterSensitiveLog: (obj: Storage) => any;
    const isa: (o: any) => o is Storage;
}
/**
 * <p>Describes a storage location in Amazon S3.</p>
 */
export interface StorageLocation {
    __type?: "StorageLocation";
    /**
     * <p>The name of the S3 bucket.</p>
     */
    Bucket?: string;
    /**
     * <p>The key.</p>
     */
    Key?: string;
}
export declare namespace StorageLocation {
    const filterSensitiveLog: (obj: StorageLocation) => any;
    const isa: (o: any) => o is StorageLocation;
}
/**
 * <p>Describes a subnet.</p>
 */
export interface Subnet {
    __type?: "Subnet";
    /**
     * <p>Indicates whether a network interface created in this subnet (including a network
     *             interface created by <a>RunInstances</a>) receives an IPv6 address.</p>
     */
    AssignIpv6AddressOnCreation?: boolean;
    /**
     * <p>The Availability Zone of the subnet.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The AZ ID of the subnet.</p>
     */
    AvailabilityZoneId?: string;
    /**
     * <p>The number of unused private IPv4 addresses in the subnet. The IPv4 addresses for any
     * 			stopped instances are considered unavailable.</p>
     */
    AvailableIpAddressCount?: number;
    /**
     * <p>The IPv4 CIDR block assigned to the subnet.</p>
     */
    CidrBlock?: string;
    /**
     * <p>Indicates whether this is the default subnet for the Availability Zone.</p>
     */
    DefaultForAz?: boolean;
    /**
     * <p>Information about the IPv6 CIDR blocks associated with the subnet.</p>
     */
    Ipv6CidrBlockAssociationSet?: SubnetIpv6CidrBlockAssociation[];
    /**
     * <p>Indicates whether instances launched in this subnet receive a public IPv4 address.</p>
     */
    MapPublicIpOnLaunch?: boolean;
    /**
     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
     */
    OutpostArn?: string;
    /**
     * <p>The ID of the AWS account that owns the subnet.</p>
     */
    OwnerId?: string;
    /**
     * <p>The current state of the subnet.</p>
     */
    State?: SubnetState | string;
    /**
     * <p>The Amazon Resource Name (ARN) of the subnet.</p>
     */
    SubnetArn?: string;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId?: string;
    /**
     * <p>Any tags assigned to the subnet.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the VPC the subnet is in.</p>
     */
    VpcId?: string;
}
export declare namespace Subnet {
    const filterSensitiveLog: (obj: Subnet) => any;
    const isa: (o: any) => o is Subnet;
}
/**
 * <p>Describes the subnet association with the transit gateway multicast domain.</p>
 */
export interface SubnetAssociation {
    __type?: "SubnetAssociation";
    /**
     * <p>The state of the subnet association.</p>
     */
    State?: TransitGatewayMulitcastDomainAssociationState | string;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId?: string;
}
export declare namespace SubnetAssociation {
    const filterSensitiveLog: (obj: SubnetAssociation) => any;
    const isa: (o: any) => o is SubnetAssociation;
}
/**
 * <p>Describes the state of a CIDR block.</p>
 */
export interface SubnetCidrBlockState {
    __type?: "SubnetCidrBlockState";
    /**
     * <p>The state of a CIDR block.</p>
     */
    State?: SubnetCidrBlockStateCode | string;
    /**
     * <p>A message about the status of the CIDR block, if applicable.</p>
     */
    StatusMessage?: string;
}
export declare namespace SubnetCidrBlockState {
    const filterSensitiveLog: (obj: SubnetCidrBlockState) => any;
    const isa: (o: any) => o is SubnetCidrBlockState;
}
export declare type SubnetCidrBlockStateCode = "associated" | "associating" | "disassociated" | "disassociating" | "failed" | "failing";
/**
 * <p>Describes an IPv6 CIDR block associated with a subnet.</p>
 */
export interface SubnetIpv6CidrBlockAssociation {
    __type?: "SubnetIpv6CidrBlockAssociation";
    /**
     * <p>The association ID for the CIDR block.</p>
     */
    AssociationId?: string;
    /**
     * <p>The IPv6 CIDR block.</p>
     */
    Ipv6CidrBlock?: string;
    /**
     * <p>Information about the state of the CIDR block.</p>
     */
    Ipv6CidrBlockState?: SubnetCidrBlockState;
}
export declare namespace SubnetIpv6CidrBlockAssociation {
    const filterSensitiveLog: (obj: SubnetIpv6CidrBlockAssociation) => any;
    const isa: (o: any) => o is SubnetIpv6CidrBlockAssociation;
}
export declare type SubnetState = "available" | "pending";
/**
 * <p>Describes the burstable performance instance whose credit option for CPU usage was
 *             successfully modified.</p>
 */
export interface SuccessfulInstanceCreditSpecificationItem {
    __type?: "SuccessfulInstanceCreditSpecificationItem";
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
}
export declare namespace SuccessfulInstanceCreditSpecificationItem {
    const filterSensitiveLog: (obj: SuccessfulInstanceCreditSpecificationItem) => any;
    const isa: (o: any) => o is SuccessfulInstanceCreditSpecificationItem;
}
/**
 * <p>Describes a Reserved Instance whose queued purchase was successfully deleted.</p>
 */
export interface SuccessfulQueuedPurchaseDeletion {
    __type?: "SuccessfulQueuedPurchaseDeletion";
    /**
     * <p>The ID of the Reserved Instance.</p>
     */
    ReservedInstancesId?: string;
}
export declare namespace SuccessfulQueuedPurchaseDeletion {
    const filterSensitiveLog: (obj: SuccessfulQueuedPurchaseDeletion) => any;
    const isa: (o: any) => o is SuccessfulQueuedPurchaseDeletion;
}
export declare type SummaryStatus = "impaired" | "initializing" | "insufficient-data" | "not-applicable" | "ok";
/**
 * <p>Describes a tag.</p>
 */
export interface Tag {
    __type?: "Tag";
    /**
     * <p>The key of the tag.</p>
     *          <p>Constraints: Tag keys are case-sensitive and accept a maximum of 127 Unicode characters.
     *          May not begin with <code>aws:</code>.</p>
     */
    Key?: string;
    /**
     * <p>The value of the tag.</p>
     *          <p>Constraints: Tag values are case-sensitive and accept a maximum of 255 Unicode characters.</p>
     */
    Value?: string;
}
export declare namespace Tag {
    const filterSensitiveLog: (obj: Tag) => any;
    const isa: (o: any) => o is Tag;
}
/**
 * <p>Describes a tag.</p>
 */
export interface TagDescription {
    __type?: "TagDescription";
    /**
     * <p>The tag key.</p>
     */
    Key?: string;
    /**
     * <p>The ID of the resource.</p>
     */
    ResourceId?: string;
    /**
     * <p>The resource type.</p>
     */
    ResourceType?: ResourceType | string;
    /**
     * <p>The tag value.</p>
     */
    Value?: string;
}
export declare namespace TagDescription {
    const filterSensitiveLog: (obj: TagDescription) => any;
    const isa: (o: any) => o is TagDescription;
}
/**
 * <p>The tags to apply to a resource when the resource is being created.</p>
 */
export interface TagSpecification {
    __type?: "TagSpecification";
    /**
     * <p>The type of resource to tag. Currently, the resource types that support tagging on creation are:
     *    	   <code>capacity-reservation</code> | <code>client-vpn-endpoint</code> | <code>dedicated-host</code> |
     *    	   <code>fleet</code> | <code>fpga-image</code> | <code>instance</code> | <code>key-pair</code> |
     *     	  <code>launch-template</code> | <code>placement-group</code> |
     *    	   <code>snapshot</code> | <code>traffic-mirror-filter</code> | <code>traffic-mirror-session</code> |
     *          <code>traffic-mirror-target</code> | <code>transit-gateway</code> | <code>transit-gateway-attachment</code> |
     *    	   <code>transit-gateway-route-table</code> | <code>volume</code>.</p>
     *          <p>To tag a resource after it has been created, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html">CreateTags</a>.</p>
     */
    ResourceType?: ResourceType | string;
    /**
     * <p>The tags to apply to the resource.</p>
     */
    Tags?: Tag[];
}
export declare namespace TagSpecification {
    const filterSensitiveLog: (obj: TagSpecification) => any;
    const isa: (o: any) => o is TagSpecification;
}
/**
 * <p>The number of units to request. You can choose to set the target capacity in terms of
 *          instances or a performance characteristic that is important to your application workload,
 *          such as vCPUs, memory, or I/O. If the request type is <code>maintain</code>, you can
 *          specify a target capacity of 0 and add capacity later.</p>
 *          <p>You can use the On-Demand Instance <code>MaxTotalPrice</code> parameter, the Spot Instance
 *             <code>MaxTotalPrice</code>, or both to ensure that your fleet cost does not exceed your
 *          budget. If you set a maximum price per hour for the On-Demand Instances and Spot Instances in your request, EC2 Fleet
 *          will launch instances until it reaches the maximum amount that you're willing to pay. When
 *          the maximum amount you're willing to pay is reached, the fleet stops launching instances
 *          even if it hasn’t met the target capacity. The <code>MaxTotalPrice</code> parameters are
 *          located in  and
 *       </p>
 */
export interface TargetCapacitySpecification {
    __type?: "TargetCapacitySpecification";
    /**
     * <p>The default <code>TotalTargetCapacity</code>, which is either <code>Spot</code> or
     *             <code>On-Demand</code>.</p>
     */
    DefaultTargetCapacityType?: DefaultTargetCapacityType | string;
    /**
     * <p>The number of On-Demand units to request. If you specify a target capacity for Spot units, you cannot specify a target capacity for On-Demand units.</p>
     */
    OnDemandTargetCapacity?: number;
    /**
     * <p>The maximum number of Spot units to launch. If you specify a target capacity for On-Demand units, you cannot specify a target capacity for Spot units.</p>
     */
    SpotTargetCapacity?: number;
    /**
     * <p>The number of units to request, filled using
     *          <code>DefaultTargetCapacityType</code>.</p>
     */
    TotalTargetCapacity?: number;
}
export declare namespace TargetCapacitySpecification {
    const filterSensitiveLog: (obj: TargetCapacitySpecification) => any;
    const isa: (o: any) => o is TargetCapacitySpecification;
}
/**
 * <p>The number of units to request. You can choose to set the target capacity as the number of
 *          instances. Or you can set the target capacity to a performance characteristic that is important to your application workload,
 *          such as vCPUs, memory, or I/O. If the request type is <code>maintain</code>, you can
 *          specify a target capacity of 0 and add capacity later.</p>
 *          <p>You can use the On-Demand Instance <code>MaxTotalPrice</code> parameter, the Spot Instance
 *             <code>MaxTotalPrice</code> parameter, or both parameters to ensure that your fleet cost
 *          does not exceed your budget. If you set a maximum price per hour for the On-Demand Instances and Spot Instances
 *          in your request, EC2 Fleet will launch instances until it reaches the maximum amount that you're
 *          willing to pay. When the maximum amount you're willing to pay is reached, the fleet stops
 *          launching instances even if it hasn’t met the target capacity. The
 *             <code>MaxTotalPrice</code> parameters are located in  and .</p>
 */
export interface TargetCapacitySpecificationRequest {
    __type?: "TargetCapacitySpecificationRequest";
    /**
     * <p>The default <code>TotalTargetCapacity</code>, which is either <code>Spot</code> or
     *             <code>On-Demand</code>.</p>
     */
    DefaultTargetCapacityType?: DefaultTargetCapacityType | string;
    /**
     * <p>The number of On-Demand units to request.</p>
     */
    OnDemandTargetCapacity?: number;
    /**
     * <p>The number of Spot units to request.</p>
     */
    SpotTargetCapacity?: number;
    /**
     * <p>The number of units to request, filled using
     *          <code>DefaultTargetCapacityType</code>.</p>
     */
    TotalTargetCapacity: number | undefined;
}
export declare namespace TargetCapacitySpecificationRequest {
    const filterSensitiveLog: (obj: TargetCapacitySpecificationRequest) => any;
    const isa: (o: any) => o is TargetCapacitySpecificationRequest;
}
/**
 * <p>Information about the Convertible Reserved Instance offering.</p>
 */
export interface TargetConfiguration {
    __type?: "TargetConfiguration";
    /**
     * <p>The number of instances the Convertible Reserved Instance offering can be applied to. This parameter is
     *       reserved and cannot be specified in a request</p>
     */
    InstanceCount?: number;
    /**
     * <p>The ID of the Convertible Reserved Instance offering.</p>
     */
    OfferingId?: string;
}
export declare namespace TargetConfiguration {
    const filterSensitiveLog: (obj: TargetConfiguration) => any;
    const isa: (o: any) => o is TargetConfiguration;
}
/**
 * <p>Details about the target configuration.</p>
 */
export interface TargetConfigurationRequest {
    __type?: "TargetConfigurationRequest";
    /**
     * <p>The number of instances the Covertible Reserved Instance offering can be applied to. This parameter is reserved and cannot
     *       be specified in a request</p>
     */
    InstanceCount?: number;
    /**
     * <p>The Convertible Reserved Instance offering ID.</p>
     */
    OfferingId: string | undefined;
}
export declare namespace TargetConfigurationRequest {
    const filterSensitiveLog: (obj: TargetConfigurationRequest) => any;
    const isa: (o: any) => o is TargetConfigurationRequest;
}
/**
 * <p>Describes a load balancer target group.</p>
 */
export interface TargetGroup {
    __type?: "TargetGroup";
    /**
     * <p>The Amazon Resource Name (ARN) of the target group.</p>
     */
    Arn?: string;
}
export declare namespace TargetGroup {
    const filterSensitiveLog: (obj: TargetGroup) => any;
    const isa: (o: any) => o is TargetGroup;
}
/**
 * <p>Describes the target groups to attach to a Spot Fleet. Spot Fleet registers the running Spot Instances
 *             with these target groups.</p>
 */
export interface TargetGroupsConfig {
    __type?: "TargetGroupsConfig";
    /**
     * <p>One or more target groups.</p>
     */
    TargetGroups?: TargetGroup[];
}
export declare namespace TargetGroupsConfig {
    const filterSensitiveLog: (obj: TargetGroupsConfig) => any;
    const isa: (o: any) => o is TargetGroupsConfig;
}
/**
 * <p>Describes a target network associated with a Client VPN endpoint.</p>
 */
export interface TargetNetwork {
    __type?: "TargetNetwork";
    /**
     * <p>The ID of the association.</p>
     */
    AssociationId?: string;
    /**
     * <p>The ID of the Client VPN endpoint with which the target network is associated.</p>
     */
    ClientVpnEndpointId?: string;
    /**
     * <p>The IDs of the security groups applied to the target network association.</p>
     */
    SecurityGroups?: string[];
    /**
     * <p>The current state of the target network association.</p>
     */
    Status?: AssociationStatus;
    /**
     * <p>The ID of the subnet specified as the target network.</p>
     */
    TargetNetworkId?: string;
    /**
     * <p>The ID of the VPC in which the target network (subnet) is located.</p>
     */
    VpcId?: string;
}
export declare namespace TargetNetwork {
    const filterSensitiveLog: (obj: TargetNetwork) => any;
    const isa: (o: any) => o is TargetNetwork;
}
/**
 * <p>The total value of the new Convertible Reserved Instances.</p>
 */
export interface TargetReservationValue {
    __type?: "TargetReservationValue";
    /**
     * <p>The total value of the Convertible Reserved Instances that make up the exchange. This is the sum of
     *       the list value, remaining upfront price, and additional upfront cost of the exchange.</p>
     */
    ReservationValue?: ReservationValue;
    /**
     * <p>The configuration of the Convertible Reserved Instances that make up the exchange.</p>
     */
    TargetConfiguration?: TargetConfiguration;
}
export declare namespace TargetReservationValue {
    const filterSensitiveLog: (obj: TargetReservationValue) => any;
    const isa: (o: any) => o is TargetReservationValue;
}
export declare type TelemetryStatus = "DOWN" | "UP";
export declare type Tenancy = "dedicated" | "default" | "host";
export interface TerminateClientVpnConnectionsRequest {
    __type?: "TerminateClientVpnConnectionsRequest";
    /**
     * <p>The ID of the Client VPN endpoint to which the client is connected.</p>
     */
    ClientVpnEndpointId: string | undefined;
    /**
     * <p>The ID of the client connection to be terminated.</p>
     */
    ConnectionId?: string;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>. Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The name of the user who initiated the connection. Use this option to terminate all active connections for
     * 			the specified user. This option can only be used if the user has established up to five connections.</p>
     */
    Username?: string;
}
export declare namespace TerminateClientVpnConnectionsRequest {
    const filterSensitiveLog: (obj: TerminateClientVpnConnectionsRequest) => any;
    const isa: (o: any) => o is TerminateClientVpnConnectionsRequest;
}
export interface TerminateClientVpnConnectionsResult {
    __type?: "TerminateClientVpnConnectionsResult";
    /**
     * <p>The ID of the Client VPN endpoint.</p>
     */
    ClientVpnEndpointId?: string;
    /**
     * <p>The current state of the client connections.</p>
     */
    ConnectionStatuses?: TerminateConnectionStatus[];
    /**
     * <p>The user who established the terminated client connections.</p>
     */
    Username?: string;
}
export declare namespace TerminateClientVpnConnectionsResult {
    const filterSensitiveLog: (obj: TerminateClientVpnConnectionsResult) => any;
    const isa: (o: any) => o is TerminateClientVpnConnectionsResult;
}
/**
 * <p>Information about a terminated Client VPN endpoint client connection.</p>
 */
export interface TerminateConnectionStatus {
    __type?: "TerminateConnectionStatus";
    /**
     * <p>The ID of the client connection.</p>
     */
    ConnectionId?: string;
    /**
     * <p>A message about the status of the client connection, if applicable.</p>
     */
    CurrentStatus?: ClientVpnConnectionStatus;
    /**
     * <p>The state of the client connection.</p>
     */
    PreviousStatus?: ClientVpnConnectionStatus;
}
export declare namespace TerminateConnectionStatus {
    const filterSensitiveLog: (obj: TerminateConnectionStatus) => any;
    const isa: (o: any) => o is TerminateConnectionStatus;
}
export interface TerminateInstancesRequest {
    __type?: "TerminateInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the instances.</p>
     *         <p>Constraints: Up to 1000 instance IDs. We recommend breaking up this request into
     *             smaller batches.</p>
     */
    InstanceIds: string[] | undefined;
}
export declare namespace TerminateInstancesRequest {
    const filterSensitiveLog: (obj: TerminateInstancesRequest) => any;
    const isa: (o: any) => o is TerminateInstancesRequest;
}
export interface TerminateInstancesResult {
    __type?: "TerminateInstancesResult";
    /**
     * <p>Information about the terminated instances.</p>
     */
    TerminatingInstances?: InstanceStateChange[];
}
export declare namespace TerminateInstancesResult {
    const filterSensitiveLog: (obj: TerminateInstancesResult) => any;
    const isa: (o: any) => o is TerminateInstancesResult;
}
export declare type TrafficDirection = "egress" | "ingress";
/**
 * <p>Describes the Traffic Mirror filter.</p>
 */
export interface TrafficMirrorFilter {
    __type?: "TrafficMirrorFilter";
    /**
     * <p>The description of the Traffic Mirror filter.</p>
     */
    Description?: string;
    /**
     * <p>Information about the egress rules that are associated with the Traffic Mirror filter.</p>
     */
    EgressFilterRules?: TrafficMirrorFilterRule[];
    /**
     * <p>Information about the ingress rules that are associated with the Traffic Mirror filter.</p>
     */
    IngressFilterRules?: TrafficMirrorFilterRule[];
    /**
     * <p>The network service traffic that is associated with the Traffic Mirror filter.</p>
     */
    NetworkServices?: (TrafficMirrorNetworkService | string)[];
    /**
     * <p>The tags assigned to the Traffic Mirror filter.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the Traffic Mirror filter.</p>
     */
    TrafficMirrorFilterId?: string;
}
export declare namespace TrafficMirrorFilter {
    const filterSensitiveLog: (obj: TrafficMirrorFilter) => any;
    const isa: (o: any) => o is TrafficMirrorFilter;
}
/**
 * <p>Describes the Traffic Mirror rule.</p>
 */
export interface TrafficMirrorFilterRule {
    __type?: "TrafficMirrorFilterRule";
    /**
     * <p>The description of the Traffic Mirror rule.</p>
     */
    Description?: string;
    /**
     * <p>The destination CIDR block assigned to the Traffic Mirror rule.</p>
     */
    DestinationCidrBlock?: string;
    /**
     * <p>The destination port range assigned to the Traffic Mirror rule.</p>
     */
    DestinationPortRange?: TrafficMirrorPortRange;
    /**
     * <p>The protocol assigned to the Traffic Mirror rule.</p>
     */
    Protocol?: number;
    /**
     * <p>The action assigned to the Traffic Mirror rule.</p>
     */
    RuleAction?: TrafficMirrorRuleAction | string;
    /**
     * <p>The rule number of the Traffic Mirror rule.</p>
     */
    RuleNumber?: number;
    /**
     * <p>The source CIDR block assigned to the Traffic Mirror rule.</p>
     */
    SourceCidrBlock?: string;
    /**
     * <p>The source port range assigned to the Traffic Mirror rule.</p>
     */
    SourcePortRange?: TrafficMirrorPortRange;
    /**
     * <p>The traffic direction assigned to the Traffic Mirror rule.</p>
     */
    TrafficDirection?: TrafficDirection | string;
    /**
     * <p>The ID of the Traffic Mirror filter that the rule is associated with.</p>
     */
    TrafficMirrorFilterId?: string;
    /**
     * <p>The ID of the Traffic Mirror rule.</p>
     */
    TrafficMirrorFilterRuleId?: string;
}
export declare namespace TrafficMirrorFilterRule {
    const filterSensitiveLog: (obj: TrafficMirrorFilterRule) => any;
    const isa: (o: any) => o is TrafficMirrorFilterRule;
}
export declare type TrafficMirrorFilterRuleField = "description" | "destination-port-range" | "protocol" | "source-port-range";
export declare type TrafficMirrorNetworkService = "amazon-dns";
/**
 * <p>Describes the Traffic Mirror port range.</p>
 */
export interface TrafficMirrorPortRange {
    __type?: "TrafficMirrorPortRange";
    /**
     * <p>The start of the Traffic Mirror port range. This applies to the TCP and UDP protocols.</p>
     */
    FromPort?: number;
    /**
     * <p>The end of the Traffic Mirror port range. This applies to the TCP and UDP protocols.</p>
     */
    ToPort?: number;
}
export declare namespace TrafficMirrorPortRange {
    const filterSensitiveLog: (obj: TrafficMirrorPortRange) => any;
    const isa: (o: any) => o is TrafficMirrorPortRange;
}
/**
 * <p>Information about the Traffic Mirror filter rule port range.</p>
 */
export interface TrafficMirrorPortRangeRequest {
    __type?: "TrafficMirrorPortRangeRequest";
    /**
     * <p>The first port in the Traffic Mirror port range. This applies to the TCP and UDP protocols.</p>
     */
    FromPort?: number;
    /**
     * <p>The last port in the Traffic Mirror port range. This applies to the TCP and UDP protocols.</p>
     */
    ToPort?: number;
}
export declare namespace TrafficMirrorPortRangeRequest {
    const filterSensitiveLog: (obj: TrafficMirrorPortRangeRequest) => any;
    const isa: (o: any) => o is TrafficMirrorPortRangeRequest;
}
export declare type TrafficMirrorRuleAction = "accept" | "reject";
/**
 * <p>Describes a Traffic Mirror session.</p>
 */
export interface TrafficMirrorSession {
    __type?: "TrafficMirrorSession";
    /**
     * <p>The description of the Traffic Mirror session.</p>
     */
    Description?: string;
    /**
     * <p>The ID of the Traffic Mirror session's network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The ID of the account that owns the Traffic Mirror session.</p>
     */
    OwnerId?: string;
    /**
     * <p>The number of bytes in each packet to mirror. These are the bytes after the VXLAN header. To mirror a subset, set this to the length (in bytes) to mirror. For example, if you set this value to 100, then the first 100 bytes that meet the filter criteria are copied to the target. Do not specify this parameter when you want to mirror the entire packet</p>
     */
    PacketLength?: number;
    /**
     * <p>The session number determines the order in which sessions are evaluated when an interface is used by multiple sessions. The first session with a matching filter is the one that mirrors the packets.</p>
     *          <p>Valid values are 1-32766.</p>
     */
    SessionNumber?: number;
    /**
     * <p>The tags assigned to the Traffic Mirror session.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the Traffic Mirror filter.</p>
     */
    TrafficMirrorFilterId?: string;
    /**
     * <p>The ID for the Traffic Mirror session.</p>
     */
    TrafficMirrorSessionId?: string;
    /**
     * <p>The ID of the Traffic Mirror target.</p>
     */
    TrafficMirrorTargetId?: string;
    /**
     * <p>The virtual network ID associated with the Traffic Mirror session.</p>
     */
    VirtualNetworkId?: number;
}
export declare namespace TrafficMirrorSession {
    const filterSensitiveLog: (obj: TrafficMirrorSession) => any;
    const isa: (o: any) => o is TrafficMirrorSession;
}
export declare type TrafficMirrorSessionField = "description" | "packet-length" | "virtual-network-id";
/**
 * <p>Describes a Traffic Mirror target.</p>
 */
export interface TrafficMirrorTarget {
    __type?: "TrafficMirrorTarget";
    /**
     * <p>Information about the Traffic Mirror target.</p>
     */
    Description?: string;
    /**
     * <p>The network interface ID that is attached to the target.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The Amazon Resource Name (ARN) of the Network Load Balancer.</p>
     */
    NetworkLoadBalancerArn?: string;
    /**
     * <p>The ID of the account that owns the Traffic Mirror target.</p>
     */
    OwnerId?: string;
    /**
     * <p>The tags assigned to the Traffic Mirror target.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the Traffic Mirror target.</p>
     */
    TrafficMirrorTargetId?: string;
    /**
     * <p>The type of Traffic Mirror target.</p>
     */
    Type?: TrafficMirrorTargetType | string;
}
export declare namespace TrafficMirrorTarget {
    const filterSensitiveLog: (obj: TrafficMirrorTarget) => any;
    const isa: (o: any) => o is TrafficMirrorTarget;
}
export declare type TrafficMirrorTargetType = "network-interface" | "network-load-balancer";
export declare type TrafficType = "ACCEPT" | "ALL" | "REJECT";
/**
 * <p>Describes a transit gateway.</p>
 */
export interface TransitGateway {
    __type?: "TransitGateway";
    /**
     * <p>The creation time.</p>
     */
    CreationTime?: Date;
    /**
     * <p>The description of the transit gateway.</p>
     */
    Description?: string;
    /**
     * <p>The transit gateway options.</p>
     */
    Options?: TransitGatewayOptions;
    /**
     * <p>The ID of the AWS account ID that owns the transit gateway.</p>
     */
    OwnerId?: string;
    /**
     * <p>The state of the transit gateway.</p>
     */
    State?: TransitGatewayState | string;
    /**
     * <p>The tags for the transit gateway.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The Amazon Resource Name (ARN) of the transit gateway.</p>
     */
    TransitGatewayArn?: string;
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId?: string;
}
export declare namespace TransitGateway {
    const filterSensitiveLog: (obj: TransitGateway) => any;
    const isa: (o: any) => o is TransitGateway;
}
/**
 * <p>Describes an association between a resource attachment and a transit gateway route table.</p>
 */
export interface TransitGatewayAssociation {
    __type?: "TransitGatewayAssociation";
    /**
     * <p>The ID of the resource.</p>
     */
    ResourceId?: string;
    /**
     * <p>The resource type.</p>
     */
    ResourceType?: TransitGatewayAttachmentResourceType | string;
    /**
     * <p>The state of the association.</p>
     */
    State?: TransitGatewayAssociationState | string;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
    /**
     * <p>The ID of the transit gateway route table.</p>
     */
    TransitGatewayRouteTableId?: string;
}
export declare namespace TransitGatewayAssociation {
    const filterSensitiveLog: (obj: TransitGatewayAssociation) => any;
    const isa: (o: any) => o is TransitGatewayAssociation;
}
export declare type TransitGatewayAssociationState = "associated" | "associating" | "disassociated" | "disassociating";
/**
 * <p>Describes an attachment between a resource and a transit gateway.</p>
 */
export interface TransitGatewayAttachment {
    __type?: "TransitGatewayAttachment";
    /**
     * <p>The association.</p>
     */
    Association?: TransitGatewayAttachmentAssociation;
    /**
     * <p>The creation time.</p>
     */
    CreationTime?: Date;
    /**
     * <p>The ID of the resource.</p>
     */
    ResourceId?: string;
    /**
     * <p>The ID of the AWS account that owns the resource.</p>
     */
    ResourceOwnerId?: string;
    /**
     * <p>The resource type.</p>
     */
    ResourceType?: TransitGatewayAttachmentResourceType | string;
    /**
     * <p>The attachment state.</p>
     */
    State?: TransitGatewayAttachmentState | string;
    /**
     * <p>The tags for the attachment.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId?: string;
    /**
     * <p>The ID of the AWS account that owns the transit gateway.</p>
     */
    TransitGatewayOwnerId?: string;
}
export declare namespace TransitGatewayAttachment {
    const filterSensitiveLog: (obj: TransitGatewayAttachment) => any;
    const isa: (o: any) => o is TransitGatewayAttachment;
}
/**
 * <p>Describes an association.</p>
 */
export interface TransitGatewayAttachmentAssociation {
    __type?: "TransitGatewayAttachmentAssociation";
    /**
     * <p>The state of the association.</p>
     */
    State?: TransitGatewayAssociationState | string;
    /**
     * <p>The ID of the route table for the transit gateway.</p>
     */
    TransitGatewayRouteTableId?: string;
}
export declare namespace TransitGatewayAttachmentAssociation {
    const filterSensitiveLog: (obj: TransitGatewayAttachmentAssociation) => any;
    const isa: (o: any) => o is TransitGatewayAttachmentAssociation;
}
/**
 * <p>Describes a propagation route table.</p>
 */
export interface TransitGatewayAttachmentPropagation {
    __type?: "TransitGatewayAttachmentPropagation";
    /**
     * <p>The state of the propagation route table.</p>
     */
    State?: TransitGatewayPropagationState | string;
    /**
     * <p>The ID of the propagation route table.</p>
     */
    TransitGatewayRouteTableId?: string;
}
export declare namespace TransitGatewayAttachmentPropagation {
    const filterSensitiveLog: (obj: TransitGatewayAttachmentPropagation) => any;
    const isa: (o: any) => o is TransitGatewayAttachmentPropagation;
}
export declare type TransitGatewayAttachmentResourceType = "direct-connect-gateway" | "tgw-peering" | "vpc" | "vpn";
export declare type TransitGatewayAttachmentState = "available" | "deleted" | "deleting" | "failed" | "failing" | "initiating" | "modifying" | "pending" | "pendingAcceptance" | "rejected" | "rejecting" | "rollingBack";
export declare type TransitGatewayMulitcastDomainAssociationState = "associated" | "associating" | "disassociated" | "disassociating";
/**
 * <p>Describes the deregistered  transit gateway multicast group members.</p>
 */
export interface TransitGatewayMulticastDeregisteredGroupMembers {
    __type?: "TransitGatewayMulticastDeregisteredGroupMembers";
    /**
     * <p>The network interface IDs of the deregistered members.</p>
     */
    DeregisteredNetworkInterfaceIds?: string[];
    /**
     * <p>The IP address assigned to the  transit gateway multicast group.</p>
     */
    GroupIpAddress?: string;
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace TransitGatewayMulticastDeregisteredGroupMembers {
    const filterSensitiveLog: (obj: TransitGatewayMulticastDeregisteredGroupMembers) => any;
    const isa: (o: any) => o is TransitGatewayMulticastDeregisteredGroupMembers;
}
/**
 * <p>Describes the deregistered  transit gateway multicast group sources.</p>
 */
export interface TransitGatewayMulticastDeregisteredGroupSources {
    __type?: "TransitGatewayMulticastDeregisteredGroupSources";
    /**
     * <p>The network interface IDs of the non-registered members.</p>
     */
    DeregisteredNetworkInterfaceIds?: string[];
    /**
     * <p>The IP address assigned to the  transit gateway multicast group.</p>
     */
    GroupIpAddress?: string;
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace TransitGatewayMulticastDeregisteredGroupSources {
    const filterSensitiveLog: (obj: TransitGatewayMulticastDeregisteredGroupSources) => any;
    const isa: (o: any) => o is TransitGatewayMulticastDeregisteredGroupSources;
}
/**
 * <p>Describes the transit gateway multicast domain.</p>
 */
export interface TransitGatewayMulticastDomain {
    __type?: "TransitGatewayMulticastDomain";
    /**
     * <p>The time the transit gateway multicast domain was created.</p>
     */
    CreationTime?: Date;
    /**
     * <p>The state of the transit gateway multicast domain.</p>
     */
    State?: TransitGatewayMulticastDomainState | string;
    /**
     * <p>The tags for the transit gateway multicast domain.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId?: string;
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace TransitGatewayMulticastDomain {
    const filterSensitiveLog: (obj: TransitGatewayMulticastDomain) => any;
    const isa: (o: any) => o is TransitGatewayMulticastDomain;
}
/**
 * <p>Describes the resources associated with the transit gateway multicast domain.</p>
 */
export interface TransitGatewayMulticastDomainAssociation {
    __type?: "TransitGatewayMulticastDomainAssociation";
    /**
     * <p>The ID of the resource.</p>
     */
    ResourceId?: string;
    /**
     * <p>The type of resource, for example a VPC attachment.</p>
     */
    ResourceType?: TransitGatewayAttachmentResourceType | string;
    /**
     * <p>The subnet associated with the transit gateway multicast domain.</p>
     */
    Subnet?: SubnetAssociation;
    /**
     * <p>The ID of the transit gateway attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
}
export declare namespace TransitGatewayMulticastDomainAssociation {
    const filterSensitiveLog: (obj: TransitGatewayMulticastDomainAssociation) => any;
    const isa: (o: any) => o is TransitGatewayMulticastDomainAssociation;
}
/**
 * <p>Describes the multicast domain associations.</p>
 */
export interface TransitGatewayMulticastDomainAssociations {
    __type?: "TransitGatewayMulticastDomainAssociations";
    /**
     * <p>The ID of the resource.</p>
     */
    ResourceId?: string;
    /**
     * <p>The type of resource, for example a VPC attachment.</p>
     */
    ResourceType?: TransitGatewayAttachmentResourceType | string;
    /**
     * <p>The subnets associated with the multicast domain.</p>
     */
    Subnets?: SubnetAssociation[];
    /**
     * <p>The ID of the transit gateway attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace TransitGatewayMulticastDomainAssociations {
    const filterSensitiveLog: (obj: TransitGatewayMulticastDomainAssociations) => any;
    const isa: (o: any) => o is TransitGatewayMulticastDomainAssociations;
}
export declare type TransitGatewayMulticastDomainState = "available" | "deleted" | "deleting" | "pending";
/**
 * <p>Describes the  transit gateway multicast group resources.</p>
 */
export interface TransitGatewayMulticastGroup {
    __type?: "TransitGatewayMulticastGroup";
    /**
     * <p>The IP address assigned to the  transit gateway multicast group.</p>
     */
    GroupIpAddress?: string;
    /**
     * <p>Indicates that the resource is a  transit gateway multicast group member.</p>
     */
    GroupMember?: boolean;
    /**
     * <p>Indicates that the resource is a  transit gateway multicast group member.</p>
     */
    GroupSource?: boolean;
    /**
     * <p>The member type (for example, <code>static</code>).</p>
     */
    MemberType?: MembershipType | string;
    /**
     * <p>The ID of the transit gateway attachment.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The ID of the resource.</p>
     */
    ResourceId?: string;
    /**
     * <p>The type of resource, for example a VPC attachment.</p>
     */
    ResourceType?: TransitGatewayAttachmentResourceType | string;
    /**
     * <p>The source type.</p>
     */
    SourceType?: MembershipType | string;
    /**
     * <p>The ID of the subnet.</p>
     */
    SubnetId?: string;
    /**
     * <p>The ID of the transit gateway attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
}
export declare namespace TransitGatewayMulticastGroup {
    const filterSensitiveLog: (obj: TransitGatewayMulticastGroup) => any;
    const isa: (o: any) => o is TransitGatewayMulticastGroup;
}
/**
 * <p>Describes the registered  transit gateway multicast group members.</p>
 */
export interface TransitGatewayMulticastRegisteredGroupMembers {
    __type?: "TransitGatewayMulticastRegisteredGroupMembers";
    /**
     * <p>The IP address assigned to the  transit gateway multicast group.</p>
     */
    GroupIpAddress?: string;
    /**
     * <p>The ID of the registered network interfaces.</p>
     */
    RegisteredNetworkInterfaceIds?: string[];
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace TransitGatewayMulticastRegisteredGroupMembers {
    const filterSensitiveLog: (obj: TransitGatewayMulticastRegisteredGroupMembers) => any;
    const isa: (o: any) => o is TransitGatewayMulticastRegisteredGroupMembers;
}
/**
 * <p>Describes the members registered with the  transit gateway multicast group.</p>
 */
export interface TransitGatewayMulticastRegisteredGroupSources {
    __type?: "TransitGatewayMulticastRegisteredGroupSources";
    /**
     * <p>The IP address assigned to the  transit gateway multicast group.</p>
     */
    GroupIpAddress?: string;
    /**
     * <p>The IDs of the network interfaces members registered with the  transit gateway multicast group.</p>
     */
    RegisteredNetworkInterfaceIds?: string[];
    /**
     * <p>The ID of the transit gateway multicast domain.</p>
     */
    TransitGatewayMulticastDomainId?: string;
}
export declare namespace TransitGatewayMulticastRegisteredGroupSources {
    const filterSensitiveLog: (obj: TransitGatewayMulticastRegisteredGroupSources) => any;
    const isa: (o: any) => o is TransitGatewayMulticastRegisteredGroupSources;
}
/**
 * <p>Describes the options for a transit gateway.</p>
 */
export interface TransitGatewayOptions {
    __type?: "TransitGatewayOptions";
    /**
     * <p>A private Autonomous System Number (ASN) for the Amazon side of a BGP session.
     *          The range is 64512 to 65534 for 16-bit ASNs and 4200000000 to 4294967294 for 32-bit ASNs.</p>
     */
    AmazonSideAsn?: number;
    /**
     * <p>The ID of the default association route table.</p>
     */
    AssociationDefaultRouteTableId?: string;
    /**
     * <p>Indicates whether attachment requests are automatically accepted.</p>
     */
    AutoAcceptSharedAttachments?: AutoAcceptSharedAttachmentsValue | string;
    /**
     * <p>Indicates whether resource attachments are automatically associated with the default association route table.</p>
     */
    DefaultRouteTableAssociation?: DefaultRouteTableAssociationValue | string;
    /**
     * <p>Indicates whether resource attachments automatically propagate routes to the default propagation route table.</p>
     */
    DefaultRouteTablePropagation?: DefaultRouteTablePropagationValue | string;
    /**
     * <p>Indicates whether DNS support is enabled.</p>
     */
    DnsSupport?: DnsSupportValue | string;
    /**
     * <p>Indicates whether multicast is enabled on the transit gateway</p>
     */
    MulticastSupport?: MulticastSupportValue | string;
    /**
     * <p>The ID of the default propagation route table.</p>
     */
    PropagationDefaultRouteTableId?: string;
    /**
     * <p>Indicates whether Equal Cost Multipath Protocol support is enabled.</p>
     */
    VpnEcmpSupport?: VpnEcmpSupportValue | string;
}
export declare namespace TransitGatewayOptions {
    const filterSensitiveLog: (obj: TransitGatewayOptions) => any;
    const isa: (o: any) => o is TransitGatewayOptions;
}
/**
 * <p>Describes the transit gateway peering attachment.</p>
 */
export interface TransitGatewayPeeringAttachment {
    __type?: "TransitGatewayPeeringAttachment";
    /**
     * <p>Information about the accepter transit gateway.</p>
     */
    AccepterTgwInfo?: PeeringTgwInfo;
    /**
     * <p>The time the transit gateway peering attachment was created.</p>
     */
    CreationTime?: Date;
    /**
     * <p>Information about the requester transit gateway.</p>
     */
    RequesterTgwInfo?: PeeringTgwInfo;
    /**
     * <p>The state of the transit gateway peering attachment.</p>
     */
    State?: TransitGatewayAttachmentState | string;
    /**
     * <p>The status of the transit gateway peering attachment.</p>
     */
    Status?: PeeringAttachmentStatus;
    /**
     * <p>The tags for the transit gateway peering attachment.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the transit gateway peering attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
}
export declare namespace TransitGatewayPeeringAttachment {
    const filterSensitiveLog: (obj: TransitGatewayPeeringAttachment) => any;
    const isa: (o: any) => o is TransitGatewayPeeringAttachment;
}
/**
 * <p>Describes route propagation.</p>
 */
export interface TransitGatewayPropagation {
    __type?: "TransitGatewayPropagation";
    /**
     * <p>The ID of the resource.</p>
     */
    ResourceId?: string;
    /**
     * <p>The resource type.</p>
     */
    ResourceType?: TransitGatewayAttachmentResourceType | string;
    /**
     * <p>The state.</p>
     */
    State?: TransitGatewayPropagationState | string;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
    /**
     * <p>The ID of the transit gateway route table.</p>
     */
    TransitGatewayRouteTableId?: string;
}
export declare namespace TransitGatewayPropagation {
    const filterSensitiveLog: (obj: TransitGatewayPropagation) => any;
    const isa: (o: any) => o is TransitGatewayPropagation;
}
export declare type TransitGatewayPropagationState = "disabled" | "disabling" | "enabled" | "enabling";
/**
 * <p>Describes the options for a transit gateway.</p>
 */
export interface TransitGatewayRequestOptions {
    __type?: "TransitGatewayRequestOptions";
    /**
     * <p>A private Autonomous System Number (ASN) for the Amazon side of a BGP session.
     *          The range is 64512 to 65534 for 16-bit ASNs and 4200000000 to 4294967294 for 32-bit ASNs.</p>
     */
    AmazonSideAsn?: number;
    /**
     * <p>Enable or disable automatic acceptance of attachment requests. The default is <code>disable</code>.</p>
     */
    AutoAcceptSharedAttachments?: AutoAcceptSharedAttachmentsValue | string;
    /**
     * <p>Enable or disable automatic association with the default association route table. The default is <code>enable</code>.</p>
     */
    DefaultRouteTableAssociation?: DefaultRouteTableAssociationValue | string;
    /**
     * <p>Enable or disable automatic propagation of routes to the default propagation route table. The default is <code>enable</code>.</p>
     */
    DefaultRouteTablePropagation?: DefaultRouteTablePropagationValue | string;
    /**
     * <p>Enable or disable DNS support.</p>
     */
    DnsSupport?: DnsSupportValue | string;
    /**
     * <p>Indicates whether multicast is enabled on the transit gateway</p>
     */
    MulticastSupport?: MulticastSupportValue | string;
    /**
     * <p>Enable or disable Equal Cost Multipath Protocol support.</p>
     */
    VpnEcmpSupport?: VpnEcmpSupportValue | string;
}
export declare namespace TransitGatewayRequestOptions {
    const filterSensitiveLog: (obj: TransitGatewayRequestOptions) => any;
    const isa: (o: any) => o is TransitGatewayRequestOptions;
}
/**
 * <p>Describes a route for a transit gateway route table.</p>
 */
export interface TransitGatewayRoute {
    __type?: "TransitGatewayRoute";
    /**
     * <p>The CIDR block used for destination matches.</p>
     */
    DestinationCidrBlock?: string;
    /**
     * <p>The state of the route.</p>
     */
    State?: TransitGatewayRouteState | string;
    /**
     * <p>The attachments.</p>
     */
    TransitGatewayAttachments?: TransitGatewayRouteAttachment[];
    /**
     * <p>The route type.</p>
     */
    Type?: TransitGatewayRouteType | string;
}
export declare namespace TransitGatewayRoute {
    const filterSensitiveLog: (obj: TransitGatewayRoute) => any;
    const isa: (o: any) => o is TransitGatewayRoute;
}
/**
 * <p>Describes a route attachment.</p>
 */
export interface TransitGatewayRouteAttachment {
    __type?: "TransitGatewayRouteAttachment";
    /**
     * <p>The ID of the resource.</p>
     */
    ResourceId?: string;
    /**
     * <p>The resource type.</p>
     */
    ResourceType?: TransitGatewayAttachmentResourceType | string;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
}
export declare namespace TransitGatewayRouteAttachment {
    const filterSensitiveLog: (obj: TransitGatewayRouteAttachment) => any;
    const isa: (o: any) => o is TransitGatewayRouteAttachment;
}
export declare type TransitGatewayRouteState = "active" | "blackhole" | "deleted" | "deleting" | "pending";
/**
 * <p>Describes a transit gateway route table.</p>
 */
export interface TransitGatewayRouteTable {
    __type?: "TransitGatewayRouteTable";
    /**
     * <p>The creation time.</p>
     */
    CreationTime?: Date;
    /**
     * <p>Indicates whether this is the default association route table for the transit gateway.</p>
     */
    DefaultAssociationRouteTable?: boolean;
    /**
     * <p>Indicates whether this is the default propagation route table for the transit gateway.</p>
     */
    DefaultPropagationRouteTable?: boolean;
    /**
     * <p>The state of the transit gateway route table.</p>
     */
    State?: TransitGatewayRouteTableState | string;
    /**
     * <p>Any tags assigned to the route table.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId?: string;
    /**
     * <p>The ID of the transit gateway route table.</p>
     */
    TransitGatewayRouteTableId?: string;
}
export declare namespace TransitGatewayRouteTable {
    const filterSensitiveLog: (obj: TransitGatewayRouteTable) => any;
    const isa: (o: any) => o is TransitGatewayRouteTable;
}
/**
 * <p>Describes an association between a route table and a resource attachment.</p>
 */
export interface TransitGatewayRouteTableAssociation {
    __type?: "TransitGatewayRouteTableAssociation";
    /**
     * <p>The ID of the resource.</p>
     */
    ResourceId?: string;
    /**
     * <p>The resource type.</p>
     */
    ResourceType?: TransitGatewayAttachmentResourceType | string;
    /**
     * <p>The state of the association.</p>
     */
    State?: TransitGatewayAssociationState | string;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
}
export declare namespace TransitGatewayRouteTableAssociation {
    const filterSensitiveLog: (obj: TransitGatewayRouteTableAssociation) => any;
    const isa: (o: any) => o is TransitGatewayRouteTableAssociation;
}
/**
 * <p>Describes a route table propagation.</p>
 */
export interface TransitGatewayRouteTablePropagation {
    __type?: "TransitGatewayRouteTablePropagation";
    /**
     * <p>The ID of the resource.</p>
     */
    ResourceId?: string;
    /**
     * <p>The type of resource.</p>
     */
    ResourceType?: TransitGatewayAttachmentResourceType | string;
    /**
     * <p>The state of the resource.</p>
     */
    State?: TransitGatewayPropagationState | string;
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
}
export declare namespace TransitGatewayRouteTablePropagation {
    const filterSensitiveLog: (obj: TransitGatewayRouteTablePropagation) => any;
    const isa: (o: any) => o is TransitGatewayRouteTablePropagation;
}
export declare type TransitGatewayRouteTableState = "available" | "deleted" | "deleting" | "pending";
export declare type TransitGatewayRouteType = "propagated" | "static";
export declare type TransitGatewayState = "available" | "deleted" | "deleting" | "modifying" | "pending";
/**
 * <p>Describes a VPC attachment.</p>
 */
export interface TransitGatewayVpcAttachment {
    __type?: "TransitGatewayVpcAttachment";
    /**
     * <p>The creation time.</p>
     */
    CreationTime?: Date;
    /**
     * <p>The VPC attachment options.</p>
     */
    Options?: TransitGatewayVpcAttachmentOptions;
    /**
     * <p>The state of the VPC attachment.</p>
     */
    State?: TransitGatewayAttachmentState | string;
    /**
     * <p>The IDs of the subnets.</p>
     */
    SubnetIds?: string[];
    /**
     * <p>The tags for the VPC attachment.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the attachment.</p>
     */
    TransitGatewayAttachmentId?: string;
    /**
     * <p>The ID of the transit gateway.</p>
     */
    TransitGatewayId?: string;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
    /**
     * <p>The ID of the AWS account that owns the VPC.</p>
     */
    VpcOwnerId?: string;
}
export declare namespace TransitGatewayVpcAttachment {
    const filterSensitiveLog: (obj: TransitGatewayVpcAttachment) => any;
    const isa: (o: any) => o is TransitGatewayVpcAttachment;
}
/**
 * <p>Describes the VPC attachment options.</p>
 */
export interface TransitGatewayVpcAttachmentOptions {
    __type?: "TransitGatewayVpcAttachmentOptions";
    /**
     * <p>Indicates whether DNS support is enabled.</p>
     */
    DnsSupport?: DnsSupportValue | string;
    /**
     * <p>Indicates whether IPv6 support is enabled.</p>
     */
    Ipv6Support?: Ipv6SupportValue | string;
}
export declare namespace TransitGatewayVpcAttachmentOptions {
    const filterSensitiveLog: (obj: TransitGatewayVpcAttachmentOptions) => any;
    const isa: (o: any) => o is TransitGatewayVpcAttachmentOptions;
}
export declare type TransportProtocol = "tcp" | "udp";
/**
 * <p>The VPN tunnel options.</p>
 */
export interface TunnelOption {
    __type?: "TunnelOption";
    /**
     * <p>The number of seconds after which a DPD timeout occurs.</p>
     */
    DpdTimeoutSeconds?: number;
    /**
     * <p>The IKE versions that are permitted for the VPN tunnel.</p>
     */
    IkeVersions?: IKEVersionsListValue[];
    /**
     * <p>The external IP address of the VPN tunnel.</p>
     */
    OutsideIpAddress?: string;
    /**
     * <p>The permitted Diffie-Hellman group numbers for the VPN tunnel for phase 1 IKE negotiations.</p>
     */
    Phase1DHGroupNumbers?: Phase1DHGroupNumbersListValue[];
    /**
     * <p>The permitted encryption algorithms for the VPN tunnel for phase 1 IKE negotiations.</p>
     */
    Phase1EncryptionAlgorithms?: Phase1EncryptionAlgorithmsListValue[];
    /**
     * <p>The permitted integrity algorithms for the VPN tunnel for phase 1 IKE negotiations.</p>
     */
    Phase1IntegrityAlgorithms?: Phase1IntegrityAlgorithmsListValue[];
    /**
     * <p>The lifetime for phase 1 of the IKE negotiation, in seconds.</p>
     */
    Phase1LifetimeSeconds?: number;
    /**
     * <p>The permitted Diffie-Hellman group numbers for the VPN tunnel for phase 2 IKE negotiations.</p>
     */
    Phase2DHGroupNumbers?: Phase2DHGroupNumbersListValue[];
    /**
     * <p>The permitted encryption algorithms for the VPN tunnel for phase 2 IKE negotiations.</p>
     */
    Phase2EncryptionAlgorithms?: Phase2EncryptionAlgorithmsListValue[];
    /**
     * <p>The permitted integrity algorithms for the VPN tunnel for phase 2 IKE negotiations.</p>
     */
    Phase2IntegrityAlgorithms?: Phase2IntegrityAlgorithmsListValue[];
    /**
     * <p>The lifetime for phase 2 of the IKE negotiation, in seconds.</p>
     */
    Phase2LifetimeSeconds?: number;
    /**
     * <p>The pre-shared key (PSK) to establish initial authentication between the virtual
     *             private gateway and the customer gateway.</p>
     */
    PreSharedKey?: string;
    /**
     * <p>The percentage of the rekey window determined by <code>RekeyMarginTimeSeconds</code> during which the rekey time is randomly selected.</p>
     */
    RekeyFuzzPercentage?: number;
    /**
     * <p>The margin time, in seconds, before the phase 2 lifetime expires, during which the AWS side of the VPN connection performs an IKE rekey.</p>
     */
    RekeyMarginTimeSeconds?: number;
    /**
     * <p>The number of packets in an IKE replay window.</p>
     */
    ReplayWindowSize?: number;
    /**
     * <p>The range of inside IP addresses for the tunnel.</p>
     */
    TunnelInsideCidr?: string;
}
export declare namespace TunnelOption {
    const filterSensitiveLog: (obj: TunnelOption) => any;
    const isa: (o: any) => o is TunnelOption;
}
export interface UnassignIpv6AddressesRequest {
    __type?: "UnassignIpv6AddressesRequest";
    /**
     * <p>The IPv6 addresses to unassign from the network interface.</p>
     */
    Ipv6Addresses: string[] | undefined;
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId: string | undefined;
}
export declare namespace UnassignIpv6AddressesRequest {
    const filterSensitiveLog: (obj: UnassignIpv6AddressesRequest) => any;
    const isa: (o: any) => o is UnassignIpv6AddressesRequest;
}
export interface UnassignIpv6AddressesResult {
    __type?: "UnassignIpv6AddressesResult";
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId?: string;
    /**
     * <p>The IPv6 addresses that have been unassigned from the network interface.</p>
     */
    UnassignedIpv6Addresses?: string[];
}
export declare namespace UnassignIpv6AddressesResult {
    const filterSensitiveLog: (obj: UnassignIpv6AddressesResult) => any;
    const isa: (o: any) => o is UnassignIpv6AddressesResult;
}
/**
 * <p>Contains the parameters for UnassignPrivateIpAddresses.</p>
 */
export interface UnassignPrivateIpAddressesRequest {
    __type?: "UnassignPrivateIpAddressesRequest";
    /**
     * <p>The ID of the network interface.</p>
     */
    NetworkInterfaceId: string | undefined;
    /**
     * <p>The secondary private IP addresses to unassign from the network interface. You can specify this option multiple times to unassign more than one IP address.</p>
     */
    PrivateIpAddresses: string[] | undefined;
}
export declare namespace UnassignPrivateIpAddressesRequest {
    const filterSensitiveLog: (obj: UnassignPrivateIpAddressesRequest) => any;
    const isa: (o: any) => o is UnassignPrivateIpAddressesRequest;
}
export declare type UnlimitedSupportedInstanceFamily = "t2" | "t3" | "t3a";
export interface UnmonitorInstancesRequest {
    __type?: "UnmonitorInstancesRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The IDs of the instances.</p>
     */
    InstanceIds: string[] | undefined;
}
export declare namespace UnmonitorInstancesRequest {
    const filterSensitiveLog: (obj: UnmonitorInstancesRequest) => any;
    const isa: (o: any) => o is UnmonitorInstancesRequest;
}
export interface UnmonitorInstancesResult {
    __type?: "UnmonitorInstancesResult";
    /**
     * <p>The monitoring information.</p>
     */
    InstanceMonitorings?: InstanceMonitoring[];
}
export declare namespace UnmonitorInstancesResult {
    const filterSensitiveLog: (obj: UnmonitorInstancesResult) => any;
    const isa: (o: any) => o is UnmonitorInstancesResult;
}
export declare enum UnsuccessfulInstanceCreditSpecificationErrorCode {
    INCORRECT_INSTANCE_STATE = "IncorrectInstanceState",
    INSTANCE_CREDIT_SPECIFICATION_NOT_SUPPORTED = "InstanceCreditSpecification.NotSupported",
    INSTANCE_NOT_FOUND = "InvalidInstanceID.NotFound",
    INVALID_INSTANCE_ID = "InvalidInstanceID.Malformed"
}
/**
 * <p>Describes the burstable performance instance whose credit option for CPU usage was not
 *             modified.</p>
 */
export interface UnsuccessfulInstanceCreditSpecificationItem {
    __type?: "UnsuccessfulInstanceCreditSpecificationItem";
    /**
     * <p>The applicable error for the burstable performance instance whose credit option for
     *             CPU usage was not modified.</p>
     */
    Error?: UnsuccessfulInstanceCreditSpecificationItemError;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
}
export declare namespace UnsuccessfulInstanceCreditSpecificationItem {
    const filterSensitiveLog: (obj: UnsuccessfulInstanceCreditSpecificationItem) => any;
    const isa: (o: any) => o is UnsuccessfulInstanceCreditSpecificationItem;
}
/**
 * <p>Information about the error for the burstable performance instance whose credit option
 *             for CPU usage was not modified.</p>
 */
export interface UnsuccessfulInstanceCreditSpecificationItemError {
    __type?: "UnsuccessfulInstanceCreditSpecificationItemError";
    /**
     * <p>The error code.</p>
     */
    Code?: UnsuccessfulInstanceCreditSpecificationErrorCode | string;
    /**
     * <p>The applicable error message.</p>
     */
    Message?: string;
}
export declare namespace UnsuccessfulInstanceCreditSpecificationItemError {
    const filterSensitiveLog: (obj: UnsuccessfulInstanceCreditSpecificationItemError) => any;
    const isa: (o: any) => o is UnsuccessfulInstanceCreditSpecificationItemError;
}
/**
 * <p>Information about items that were not successfully processed in a batch call.</p>
 */
export interface UnsuccessfulItem {
    __type?: "UnsuccessfulItem";
    /**
     * <p>Information about the error.</p>
     */
    Error?: UnsuccessfulItemError;
    /**
     * <p>The ID of the resource.</p>
     */
    ResourceId?: string;
}
export declare namespace UnsuccessfulItem {
    const filterSensitiveLog: (obj: UnsuccessfulItem) => any;
    const isa: (o: any) => o is UnsuccessfulItem;
}
/**
 * <p>Information about the error that occurred. For more information about errors, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/errors-overview.html">Error Codes</a>.</p>
 */
export interface UnsuccessfulItemError {
    __type?: "UnsuccessfulItemError";
    /**
     * <p>The error code.</p>
     */
    Code?: string;
    /**
     * <p>The error message accompanying the error code.</p>
     */
    Message?: string;
}
export declare namespace UnsuccessfulItemError {
    const filterSensitiveLog: (obj: UnsuccessfulItemError) => any;
    const isa: (o: any) => o is UnsuccessfulItemError;
}
export interface UpdateSecurityGroupRuleDescriptionsEgressRequest {
    __type?: "UpdateSecurityGroupRuleDescriptionsEgressRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the security group. You must specify either the security group ID or the
     * 			security group name in the request. For security groups in a nondefault VPC, you must
     * 			specify the security group ID.</p>
     */
    GroupId?: string;
    /**
     * <p>[Default VPC] The name of the security group. You must specify either the security group
     * 			ID or the security group name in the request.</p>
     */
    GroupName?: string;
    /**
     * <p>The IP permissions for the security group rule.</p>
     */
    IpPermissions: IpPermission[] | undefined;
}
export declare namespace UpdateSecurityGroupRuleDescriptionsEgressRequest {
    const filterSensitiveLog: (obj: UpdateSecurityGroupRuleDescriptionsEgressRequest) => any;
    const isa: (o: any) => o is UpdateSecurityGroupRuleDescriptionsEgressRequest;
}
export interface UpdateSecurityGroupRuleDescriptionsEgressResult {
    __type?: "UpdateSecurityGroupRuleDescriptionsEgressResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace UpdateSecurityGroupRuleDescriptionsEgressResult {
    const filterSensitiveLog: (obj: UpdateSecurityGroupRuleDescriptionsEgressResult) => any;
    const isa: (o: any) => o is UpdateSecurityGroupRuleDescriptionsEgressResult;
}
export interface UpdateSecurityGroupRuleDescriptionsIngressRequest {
    __type?: "UpdateSecurityGroupRuleDescriptionsIngressRequest";
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
    /**
     * <p>The ID of the security group. You must specify either the security group ID or the
     * 			security group name in the request. For security groups in a nondefault VPC, you must
     * 			specify the security group ID.</p>
     */
    GroupId?: string;
    /**
     * <p>[EC2-Classic, default VPC] The name of the security group. You must specify either the
     * 			security group ID or the security group name in the request.</p>
     */
    GroupName?: string;
    /**
     * <p>The IP permissions for the security group rule. </p>
     */
    IpPermissions: IpPermission[] | undefined;
}
export declare namespace UpdateSecurityGroupRuleDescriptionsIngressRequest {
    const filterSensitiveLog: (obj: UpdateSecurityGroupRuleDescriptionsIngressRequest) => any;
    const isa: (o: any) => o is UpdateSecurityGroupRuleDescriptionsIngressRequest;
}
export interface UpdateSecurityGroupRuleDescriptionsIngressResult {
    __type?: "UpdateSecurityGroupRuleDescriptionsIngressResult";
    /**
     * <p>Returns <code>true</code> if the request succeeds; otherwise, returns an error.</p>
     */
    Return?: boolean;
}
export declare namespace UpdateSecurityGroupRuleDescriptionsIngressResult {
    const filterSensitiveLog: (obj: UpdateSecurityGroupRuleDescriptionsIngressResult) => any;
    const isa: (o: any) => o is UpdateSecurityGroupRuleDescriptionsIngressResult;
}
export declare type UsageClassType = "on-demand" | "spot";
/**
 * <p>Describes the S3 bucket for the disk image.</p>
 */
export interface UserBucket {
    __type?: "UserBucket";
    /**
     * <p>The name of the S3 bucket where the disk image is located.</p>
     */
    S3Bucket?: string;
    /**
     * <p>The file name of the disk image.</p>
     */
    S3Key?: string;
}
export declare namespace UserBucket {
    const filterSensitiveLog: (obj: UserBucket) => any;
    const isa: (o: any) => o is UserBucket;
}
/**
 * <p>Describes the S3 bucket for the disk image.</p>
 */
export interface UserBucketDetails {
    __type?: "UserBucketDetails";
    /**
     * <p>The S3 bucket from which the disk image was created.</p>
     */
    S3Bucket?: string;
    /**
     * <p>The file name of the disk image.</p>
     */
    S3Key?: string;
}
export declare namespace UserBucketDetails {
    const filterSensitiveLog: (obj: UserBucketDetails) => any;
    const isa: (o: any) => o is UserBucketDetails;
}
/**
 * <p>Describes the user data for an instance.</p>
 */
export interface UserData {
    __type?: "UserData";
    /**
     * <p>The user data. If you are using an AWS SDK or command line tool, Base64-encoding is performed for you, and you
     *    can load the text from a file. Otherwise, you must provide Base64-encoded text.</p>
     */
    Data?: string;
}
export declare namespace UserData {
    const filterSensitiveLog: (obj: UserData) => any;
    const isa: (o: any) => o is UserData;
}
/**
 * <p>Describes a security group and AWS account ID pair.</p>
 */
export interface UserIdGroupPair {
    __type?: "UserIdGroupPair";
    /**
     * <p>A description for the security group rule that references this user ID group
     * 			pair.</p>
     *          <p>Constraints: Up to 255 characters in length. Allowed characters are a-z, A-Z, 0-9,
     *       spaces, and ._-:/()#,@[]+=;{}!$*</p>
     */
    Description?: string;
    /**
     * <p>The ID of the security group.</p>
     */
    GroupId?: string;
    /**
     * <p>The name of the security group. In a request, use this parameter for a security group
     *             in EC2-Classic or a default VPC only. For a security group in a nondefault VPC, use the
     *             security group ID. </p>
     *         <p>For a referenced security group in another VPC, this value is not returned if the
     *             referenced security group is deleted.</p>
     */
    GroupName?: string;
    /**
     * <p>The status of a VPC peering connection, if applicable.</p>
     */
    PeeringStatus?: string;
    /**
     * <p>The ID of an AWS account.</p>
     *         <p>For a referenced security group in another VPC, the account ID of the referenced
     *             security group is returned in the response. If the referenced security group is deleted,
     *             this value is not returned.</p>
     *          <p>[EC2-Classic] Required when adding or removing rules that reference a security group
     *             in another AWS account.</p>
     */
    UserId?: string;
    /**
     * <p>The ID of the VPC for the referenced security group, if applicable.</p>
     */
    VpcId?: string;
    /**
     * <p>The ID of the VPC peering connection, if applicable.</p>
     */
    VpcPeeringConnectionId?: string;
}
export declare namespace UserIdGroupPair {
    const filterSensitiveLog: (obj: UserIdGroupPair) => any;
    const isa: (o: any) => o is UserIdGroupPair;
}
/**
 * <p>Describes the vCPU configurations for the instance type.</p>
 */
export interface VCpuInfo {
    __type?: "VCpuInfo";
    /**
     * <p>The default number of cores for the instance type.</p>
     */
    DefaultCores?: number;
    /**
     * <p>The default number of threads per core for the instance type.</p>
     */
    DefaultThreadsPerCore?: number;
    /**
     * <p>The default number of vCPUs for the instance type.</p>
     */
    DefaultVCpus?: number;
    /**
     * <p>List of the valid number of cores that can be configured for the instance type.</p>
     */
    ValidCores?: number[];
    /**
     * <p>List of the valid number of threads per core that can be configured for the instance type. </p>
     */
    ValidThreadsPerCore?: number[];
}
export declare namespace VCpuInfo {
    const filterSensitiveLog: (obj: VCpuInfo) => any;
    const isa: (o: any) => o is VCpuInfo;
}
/**
 * <p>Describes telemetry for a VPN tunnel.</p>
 */
export interface VgwTelemetry {
    __type?: "VgwTelemetry";
    /**
     * <p>The number of accepted routes.</p>
     */
    AcceptedRouteCount?: number;
    /**
     * <p>The Amazon Resource Name (ARN) of the VPN tunnel endpoint certificate.</p>
     */
    CertificateArn?: string;
    /**
     * <p>The date and time of the last change in status.</p>
     */
    LastStatusChange?: Date;
    /**
     * <p>The Internet-routable IP address of the virtual private gateway's outside interface.</p>
     */
    OutsideIpAddress?: string;
    /**
     * <p>The status of the VPN tunnel.</p>
     */
    Status?: TelemetryStatus | string;
    /**
     * <p>If an error occurs, a description of the error.</p>
     */
    StatusMessage?: string;
}
export declare namespace VgwTelemetry {
    const filterSensitiveLog: (obj: VgwTelemetry) => any;
    const isa: (o: any) => o is VgwTelemetry;
}
export declare type VirtualizationType = "hvm" | "paravirtual";
/**
 * <p>Describes a volume.</p>
 */
export interface Volume {
    __type?: "Volume";
    /**
     * <p>Information about the volume attachments.</p>
     */
    Attachments?: VolumeAttachment[];
    /**
     * <p>The Availability Zone for the volume.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The time stamp when volume creation was initiated.</p>
     */
    CreateTime?: Date;
    /**
     * <p>Indicates whether the volume is encrypted.</p>
     */
    Encrypted?: boolean;
    /**
     * <p>Indicates whether the volume was created using fast snapshot restore.</p>
     */
    FastRestored?: boolean;
    /**
     * <p>The number of I/O operations per second (IOPS) that the volume supports. For Provisioned IOPS SSD
     *       volumes, this represents the number of IOPS that are provisioned for the volume. For General Purpose SSD
     *       volumes, this represents the baseline performance of the volume and the rate at which the
     *       volume accumulates I/O credits for bursting. For more information, see <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html">Amazon EBS Volume Types</a> in
     *       the <i>Amazon Elastic Compute Cloud User Guide</i>.</p>
     *          <p>Constraints: Range is 100-16,000 IOPS for <code>gp2</code> volumes and 100
     *       to 64,000IOPS for <code>io1</code> volumes, in most Regions. The maximum
     *       IOPS for <code>io1</code> of 64,000 is guaranteed only on <a href="https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html#ec2-nitro-instances">Nitro-based
     *         instances</a>. Other instance families guarantee performance up to
     *       32,000 IOPS.</p>
     *          <p>Condition: This parameter is required for requests to create <code>io1</code> volumes;
     *       it is not used in requests to create <code>gp2</code>, <code>st1</code>,
     *         <code>sc1</code>, or <code>standard</code> volumes.</p>
     */
    Iops?: number;
    /**
     * <p>The Amazon Resource Name (ARN) of the AWS Key Management Service (AWS KMS) customer master key (CMK) that was used to protect the
     *       volume encryption key for the volume.</p>
     */
    KmsKeyId?: string;
    /**
     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
     */
    OutpostArn?: string;
    /**
     * <p>The size of the volume, in GiBs.</p>
     */
    Size?: number;
    /**
     * <p>The snapshot from which the volume was created, if applicable.</p>
     */
    SnapshotId?: string;
    /**
     * <p>The volume state.</p>
     */
    State?: VolumeState | string;
    /**
     * <p>Any tags assigned to the volume.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the volume.</p>
     */
    VolumeId?: string;
    /**
     * <p>The volume type. This can be <code>gp2</code> for General Purpose SSD, <code>io1</code> for Provisioned IOPS SSD,
     *         <code>st1</code> for Throughput Optimized HDD, <code>sc1</code> for Cold HDD, or
     *         <code>standard</code> for Magnetic volumes.</p>
     */
    VolumeType?: VolumeType | string;
}
export declare namespace Volume {
    const filterSensitiveLog: (obj: Volume) => any;
    const isa: (o: any) => o is Volume;
}
/**
 * <p>Describes volume attachment details.</p>
 */
export interface VolumeAttachment {
    __type?: "VolumeAttachment";
    /**
     * <p>The time stamp when the attachment initiated.</p>
     */
    AttachTime?: Date;
    /**
     * <p>Indicates whether the EBS volume is deleted on instance termination.</p>
     */
    DeleteOnTermination?: boolean;
    /**
     * <p>The device name.</p>
     */
    Device?: string;
    /**
     * <p>The ID of the instance.</p>
     */
    InstanceId?: string;
    /**
     * <p>The attachment state of the volume.</p>
     */
    State?: VolumeAttachmentState | string;
    /**
     * <p>The ID of the volume.</p>
     */
    VolumeId?: string;
}
export declare namespace VolumeAttachment {
    const filterSensitiveLog: (obj: VolumeAttachment) => any;
    const isa: (o: any) => o is VolumeAttachment;
}
export declare type VolumeAttachmentState = "attached" | "attaching" | "busy" | "detached" | "detaching";
export declare type VolumeAttributeName = "autoEnableIO" | "productCodes";
/**
 * <p>Describes an EBS volume.</p>
 */
export interface VolumeDetail {
    __type?: "VolumeDetail";
    /**
     * <p>The size of the volume, in GiB.</p>
     */
    Size: number | undefined;
}
export declare namespace VolumeDetail {
    const filterSensitiveLog: (obj: VolumeDetail) => any;
    const isa: (o: any) => o is VolumeDetail;
}
/**
 * <p>Describes the modification status of an EBS volume.</p>
 *          <p>If the volume has never been modified, some element values will be null.</p>
 */
export interface VolumeModification {
    __type?: "VolumeModification";
    /**
     * <p>The modification completion or failure time.</p>
     */
    EndTime?: Date;
    /**
     * <p>The current modification state. The modification state is null for unmodified
     *       volumes.</p>
     */
    ModificationState?: VolumeModificationState | string;
    /**
     * <p>The original IOPS rate of the volume.</p>
     */
    OriginalIops?: number;
    /**
     * <p>The original size of the volume.</p>
     */
    OriginalSize?: number;
    /**
     * <p>The original EBS volume type of the volume.</p>
     */
    OriginalVolumeType?: VolumeType | string;
    /**
     * <p>The modification progress, from 0 to 100 percent complete.</p>
     */
    Progress?: number;
    /**
     * <p>The modification start time.</p>
     */
    StartTime?: Date;
    /**
     * <p>A status message about the modification progress or failure.</p>
     */
    StatusMessage?: string;
    /**
     * <p>The target IOPS rate of the volume.</p>
     */
    TargetIops?: number;
    /**
     * <p>The target size of the volume, in GiB.</p>
     */
    TargetSize?: number;
    /**
     * <p>The target EBS volume type of the volume.</p>
     */
    TargetVolumeType?: VolumeType | string;
    /**
     * <p>The ID of the volume.</p>
     */
    VolumeId?: string;
}
export declare namespace VolumeModification {
    const filterSensitiveLog: (obj: VolumeModification) => any;
    const isa: (o: any) => o is VolumeModification;
}
export declare type VolumeModificationState = "completed" | "failed" | "modifying" | "optimizing";
export declare type VolumeState = "available" | "creating" | "deleted" | "deleting" | "error" | "in-use";
/**
 * <p>Describes a volume status operation code.</p>
 */
export interface VolumeStatusAction {
    __type?: "VolumeStatusAction";
    /**
     * <p>The code identifying the operation, for example, <code>enable-volume-io</code>.</p>
     */
    Code?: string;
    /**
     * <p>A description of the operation.</p>
     */
    Description?: string;
    /**
     * <p>The ID of the event associated with this operation.</p>
     */
    EventId?: string;
    /**
     * <p>The event type associated with this operation.</p>
     */
    EventType?: string;
}
export declare namespace VolumeStatusAction {
    const filterSensitiveLog: (obj: VolumeStatusAction) => any;
    const isa: (o: any) => o is VolumeStatusAction;
}
/**
 * <p>Describes a volume status.</p>
 */
export interface VolumeStatusDetails {
    __type?: "VolumeStatusDetails";
    /**
     * <p>The name of the volume status.</p>
     */
    Name?: VolumeStatusName | string;
    /**
     * <p>The intended status of the volume status.</p>
     */
    Status?: string;
}
export declare namespace VolumeStatusDetails {
    const filterSensitiveLog: (obj: VolumeStatusDetails) => any;
    const isa: (o: any) => o is VolumeStatusDetails;
}
/**
 * <p>Describes a volume status event.</p>
 */
export interface VolumeStatusEvent {
    __type?: "VolumeStatusEvent";
    /**
     * <p>A description of the event.</p>
     */
    Description?: string;
    /**
     * <p>The ID of this event.</p>
     */
    EventId?: string;
    /**
     * <p>The type of this event.</p>
     */
    EventType?: string;
    /**
     * <p>The latest end time of the event.</p>
     */
    NotAfter?: Date;
    /**
     * <p>The earliest start time of the event.</p>
     */
    NotBefore?: Date;
}
export declare namespace VolumeStatusEvent {
    const filterSensitiveLog: (obj: VolumeStatusEvent) => any;
    const isa: (o: any) => o is VolumeStatusEvent;
}
/**
 * <p>Describes the status of a volume.</p>
 */
export interface VolumeStatusInfo {
    __type?: "VolumeStatusInfo";
    /**
     * <p>The details of the volume status.</p>
     */
    Details?: VolumeStatusDetails[];
    /**
     * <p>The status of the volume.</p>
     */
    Status?: VolumeStatusInfoStatus | string;
}
export declare namespace VolumeStatusInfo {
    const filterSensitiveLog: (obj: VolumeStatusInfo) => any;
    const isa: (o: any) => o is VolumeStatusInfo;
}
export declare type VolumeStatusInfoStatus = "impaired" | "insufficient-data" | "ok";
/**
 * <p>Describes the volume status.</p>
 */
export interface VolumeStatusItem {
    __type?: "VolumeStatusItem";
    /**
     * <p>The details of the operation.</p>
     */
    Actions?: VolumeStatusAction[];
    /**
     * <p>The Availability Zone of the volume.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>A list of events associated with the volume.</p>
     */
    Events?: VolumeStatusEvent[];
    /**
     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
     */
    OutpostArn?: string;
    /**
     * <p>The volume ID.</p>
     */
    VolumeId?: string;
    /**
     * <p>The volume status.</p>
     */
    VolumeStatus?: VolumeStatusInfo;
}
export declare namespace VolumeStatusItem {
    const filterSensitiveLog: (obj: VolumeStatusItem) => any;
    const isa: (o: any) => o is VolumeStatusItem;
}
export declare type VolumeStatusName = "io-enabled" | "io-performance";
export declare type VolumeType = "gp2" | "io1" | "sc1" | "st1" | "standard";
/**
 * <p>Describes a VPC.</p>
 */
export interface Vpc {
    __type?: "Vpc";
    /**
     * <p>The primary IPv4 CIDR block for the VPC.</p>
     */
    CidrBlock?: string;
    /**
     * <p>Information about the IPv4 CIDR blocks associated with the VPC.</p>
     */
    CidrBlockAssociationSet?: VpcCidrBlockAssociation[];
    /**
     * <p>The ID of the set of DHCP options you've associated with the VPC (or <code>default</code>
     * 				if the default options are associated with the VPC).</p>
     */
    DhcpOptionsId?: string;
    /**
     * <p>The allowed tenancy of instances launched into the VPC.</p>
     */
    InstanceTenancy?: Tenancy | string;
    /**
     * <p>Information about the IPv6 CIDR blocks associated with the VPC.</p>
     */
    Ipv6CidrBlockAssociationSet?: VpcIpv6CidrBlockAssociation[];
    /**
     * <p>Indicates whether the VPC is the default VPC.</p>
     */
    IsDefault?: boolean;
    /**
     * <p>The ID of the AWS account that owns the VPC.</p>
     */
    OwnerId?: string;
    /**
     * <p>The current state of the VPC.</p>
     */
    State?: VpcState | string;
    /**
     * <p>Any tags assigned to the VPC.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace Vpc {
    const filterSensitiveLog: (obj: Vpc) => any;
    const isa: (o: any) => o is Vpc;
}
/**
 * <p>Describes an attachment between a virtual private gateway and a VPC.</p>
 */
export interface VpcAttachment {
    __type?: "VpcAttachment";
    /**
     * <p>The current state of the attachment.</p>
     */
    State?: AttachmentStatus | string;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace VpcAttachment {
    const filterSensitiveLog: (obj: VpcAttachment) => any;
    const isa: (o: any) => o is VpcAttachment;
}
export declare type VpcAttributeName = "enableDnsHostnames" | "enableDnsSupport";
/**
 * <p>Describes an IPv4 CIDR block associated with a VPC.</p>
 */
export interface VpcCidrBlockAssociation {
    __type?: "VpcCidrBlockAssociation";
    /**
     * <p>The association ID for the IPv4 CIDR block.</p>
     */
    AssociationId?: string;
    /**
     * <p>The IPv4 CIDR block.</p>
     */
    CidrBlock?: string;
    /**
     * <p>Information about the state of the CIDR block.</p>
     */
    CidrBlockState?: VpcCidrBlockState;
}
export declare namespace VpcCidrBlockAssociation {
    const filterSensitiveLog: (obj: VpcCidrBlockAssociation) => any;
    const isa: (o: any) => o is VpcCidrBlockAssociation;
}
/**
 * <p>Describes the state of a CIDR block.</p>
 */
export interface VpcCidrBlockState {
    __type?: "VpcCidrBlockState";
    /**
     * <p>The state of the CIDR block.</p>
     */
    State?: VpcCidrBlockStateCode | string;
    /**
     * <p>A message about the status of the CIDR block, if applicable.</p>
     */
    StatusMessage?: string;
}
export declare namespace VpcCidrBlockState {
    const filterSensitiveLog: (obj: VpcCidrBlockState) => any;
    const isa: (o: any) => o is VpcCidrBlockState;
}
export declare type VpcCidrBlockStateCode = "associated" | "associating" | "disassociated" | "disassociating" | "failed" | "failing";
/**
 * <p>Describes whether a VPC is enabled for ClassicLink.</p>
 */
export interface VpcClassicLink {
    __type?: "VpcClassicLink";
    /**
     * <p>Indicates whether the VPC is enabled for ClassicLink.</p>
     */
    ClassicLinkEnabled?: boolean;
    /**
     * <p>Any tags assigned to the VPC.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace VpcClassicLink {
    const filterSensitiveLog: (obj: VpcClassicLink) => any;
    const isa: (o: any) => o is VpcClassicLink;
}
/**
 * <p>Describes a VPC endpoint.</p>
 */
export interface VpcEndpoint {
    __type?: "VpcEndpoint";
    /**
     * <p>The date and time that the VPC endpoint was created.</p>
     */
    CreationTimestamp?: Date;
    /**
     * <p>(Interface endpoint) The DNS entries for the endpoint.</p>
     */
    DnsEntries?: DnsEntry[];
    /**
     * <p>(Interface endpoint) Information about the security groups that are associated with
     *             the network interface.</p>
     */
    Groups?: SecurityGroupIdentifier[];
    /**
     * <p>The last error that occurred for VPC endpoint.</p>
     */
    LastError?: LastError;
    /**
     * <p>(Interface endpoint) One or more network interfaces for the endpoint.</p>
     */
    NetworkInterfaceIds?: string[];
    /**
     * <p>The ID of the AWS account that owns the VPC endpoint.</p>
     */
    OwnerId?: string;
    /**
     * <p>The policy document associated with the endpoint, if applicable.</p>
     */
    PolicyDocument?: string;
    /**
     * <p>(Interface endpoint) Indicates whether the VPC is associated with a private hosted zone.</p>
     */
    PrivateDnsEnabled?: boolean;
    /**
     * <p>Indicates whether the VPC endpoint is being managed by its service.</p>
     */
    RequesterManaged?: boolean;
    /**
     * <p>(Gateway endpoint) One or more route tables associated with the endpoint.</p>
     */
    RouteTableIds?: string[];
    /**
     * <p>The name of the service to which the endpoint is associated.</p>
     */
    ServiceName?: string;
    /**
     * <p>The state of the VPC endpoint.</p>
     */
    State?: State | string;
    /**
     * <p>(Interface endpoint) One or more subnets in which the endpoint is located.</p>
     */
    SubnetIds?: string[];
    /**
     * <p>Any tags assigned to the VPC endpoint.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the VPC endpoint.</p>
     */
    VpcEndpointId?: string;
    /**
     * <p>The type of endpoint.</p>
     */
    VpcEndpointType?: VpcEndpointType | string;
    /**
     * <p>The ID of the VPC to which the endpoint is associated.</p>
     */
    VpcId?: string;
}
export declare namespace VpcEndpoint {
    const filterSensitiveLog: (obj: VpcEndpoint) => any;
    const isa: (o: any) => o is VpcEndpoint;
}
/**
 * <p>Describes a VPC endpoint connection to a service.</p>
 */
export interface VpcEndpointConnection {
    __type?: "VpcEndpointConnection";
    /**
     * <p>The date and time that the VPC endpoint was created.</p>
     */
    CreationTimestamp?: Date;
    /**
     * <p>The DNS entries for the VPC endpoint.</p>
     */
    DnsEntries?: DnsEntry[];
    /**
     * <p>The Amazon Resource Names (ARNs) of the network load balancers for the service.</p>
     */
    NetworkLoadBalancerArns?: string[];
    /**
     * <p>The ID of the service to which the endpoint is connected.</p>
     */
    ServiceId?: string;
    /**
     * <p>The ID of the VPC endpoint.</p>
     */
    VpcEndpointId?: string;
    /**
     * <p>The AWS account ID of the owner of the VPC endpoint.</p>
     */
    VpcEndpointOwner?: string;
    /**
     * <p>The state of the VPC endpoint.</p>
     */
    VpcEndpointState?: State | string;
}
export declare namespace VpcEndpointConnection {
    const filterSensitiveLog: (obj: VpcEndpointConnection) => any;
    const isa: (o: any) => o is VpcEndpointConnection;
}
export declare enum VpcEndpointType {
    Gateway = "Gateway",
    Interface = "Interface"
}
/**
 * <p>Describes an IPv6 CIDR block associated with a VPC.</p>
 */
export interface VpcIpv6CidrBlockAssociation {
    __type?: "VpcIpv6CidrBlockAssociation";
    /**
     * <p>The association ID for the IPv6 CIDR block.</p>
     */
    AssociationId?: string;
    /**
     * <p>The IPv6 CIDR block.</p>
     */
    Ipv6CidrBlock?: string;
    /**
     * <p>Information about the state of the CIDR block.</p>
     */
    Ipv6CidrBlockState?: VpcCidrBlockState;
    /**
     * <p>The ID of the IPv6 address pool from which the IPv6 CIDR block is allocated.</p>
     */
    Ipv6Pool?: string;
    /**
     * <p>The name of the location from which we advertise the IPV6 CIDR block.</p>
     */
    NetworkBorderGroup?: string;
}
export declare namespace VpcIpv6CidrBlockAssociation {
    const filterSensitiveLog: (obj: VpcIpv6CidrBlockAssociation) => any;
    const isa: (o: any) => o is VpcIpv6CidrBlockAssociation;
}
/**
 * <p>Describes a VPC peering connection.</p>
 */
export interface VpcPeeringConnection {
    __type?: "VpcPeeringConnection";
    /**
     * <p>Information about the accepter VPC. CIDR block information is only returned when describing an active VPC peering connection.</p>
     */
    AccepterVpcInfo?: VpcPeeringConnectionVpcInfo;
    /**
     * <p>The time that an unaccepted VPC peering connection will expire.</p>
     */
    ExpirationTime?: Date;
    /**
     * <p>Information about the requester VPC. CIDR block information is only returned when describing an active VPC peering connection.</p>
     */
    RequesterVpcInfo?: VpcPeeringConnectionVpcInfo;
    /**
     * <p>The status of the VPC peering connection.</p>
     */
    Status?: VpcPeeringConnectionStateReason;
    /**
     * <p>Any tags assigned to the resource.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the VPC peering connection.</p>
     */
    VpcPeeringConnectionId?: string;
}
export declare namespace VpcPeeringConnection {
    const filterSensitiveLog: (obj: VpcPeeringConnection) => any;
    const isa: (o: any) => o is VpcPeeringConnection;
}
/**
 * <p>Describes the VPC peering connection options.</p>
 */
export interface VpcPeeringConnectionOptionsDescription {
    __type?: "VpcPeeringConnectionOptionsDescription";
    /**
     * <p>Indicates whether a local VPC can resolve public DNS hostnames to private IP addresses when queried from instances in a peer VPC.</p>
     */
    AllowDnsResolutionFromRemoteVpc?: boolean;
    /**
     * <p>Indicates whether a local ClassicLink connection can communicate with the peer VPC over the VPC peering connection.</p>
     */
    AllowEgressFromLocalClassicLinkToRemoteVpc?: boolean;
    /**
     * <p>Indicates whether a local VPC can communicate with a ClassicLink connection in the peer VPC over the VPC peering connection.</p>
     */
    AllowEgressFromLocalVpcToRemoteClassicLink?: boolean;
}
export declare namespace VpcPeeringConnectionOptionsDescription {
    const filterSensitiveLog: (obj: VpcPeeringConnectionOptionsDescription) => any;
    const isa: (o: any) => o is VpcPeeringConnectionOptionsDescription;
}
/**
 * <p>Describes the status of a VPC peering connection.</p>
 */
export interface VpcPeeringConnectionStateReason {
    __type?: "VpcPeeringConnectionStateReason";
    /**
     * <p>The status of the VPC peering connection.</p>
     */
    Code?: VpcPeeringConnectionStateReasonCode | string;
    /**
     * <p>A message that provides more information about the status, if applicable.</p>
     */
    Message?: string;
}
export declare namespace VpcPeeringConnectionStateReason {
    const filterSensitiveLog: (obj: VpcPeeringConnectionStateReason) => any;
    const isa: (o: any) => o is VpcPeeringConnectionStateReason;
}
export declare type VpcPeeringConnectionStateReasonCode = "active" | "deleted" | "deleting" | "expired" | "failed" | "initiating-request" | "pending-acceptance" | "provisioning" | "rejected";
/**
 * <p>Describes a VPC in a VPC peering connection.</p>
 */
export interface VpcPeeringConnectionVpcInfo {
    __type?: "VpcPeeringConnectionVpcInfo";
    /**
     * <p>The IPv4 CIDR block for the VPC.</p>
     */
    CidrBlock?: string;
    /**
     * <p>Information about the IPv4 CIDR blocks for the VPC.</p>
     */
    CidrBlockSet?: CidrBlock[];
    /**
     * <p>The IPv6 CIDR block for the VPC.</p>
     */
    Ipv6CidrBlockSet?: Ipv6CidrBlock[];
    /**
     * <p>The AWS account ID of the VPC owner.</p>
     */
    OwnerId?: string;
    /**
     * <p>Information about the VPC peering connection options for the accepter or requester VPC.</p>
     */
    PeeringOptions?: VpcPeeringConnectionOptionsDescription;
    /**
     * <p>The Region in which the VPC is located.</p>
     */
    Region?: string;
    /**
     * <p>The ID of the VPC.</p>
     */
    VpcId?: string;
}
export declare namespace VpcPeeringConnectionVpcInfo {
    const filterSensitiveLog: (obj: VpcPeeringConnectionVpcInfo) => any;
    const isa: (o: any) => o is VpcPeeringConnectionVpcInfo;
}
export declare type VpcState = "available" | "pending";
export declare type VpcTenancy = "default";
/**
 * <p>Describes a VPN connection.</p>
 */
export interface VpnConnection {
    __type?: "VpnConnection";
    /**
     * <p>The category of the VPN connection. A value of <code>VPN</code> indicates an AWS VPN
     * 			connection. A value of <code>VPN-Classic</code> indicates an AWS Classic VPN connection.</p>
     */
    Category?: string;
    /**
     * <p>The configuration information for the VPN connection's customer gateway (in the native XML
     *             format). This element is always present in the <a>CreateVpnConnection</a> response; however,
     *             it's present in the <a>DescribeVpnConnections</a> response only if the VPN connection is in
     *             the <code>pending</code> or <code>available</code> state.</p>
     */
    CustomerGatewayConfiguration?: string;
    /**
     * <p>The ID of the customer gateway at your end of the VPN connection.</p>
     */
    CustomerGatewayId?: string;
    /**
     * <p>The VPN connection options.</p>
     */
    Options?: VpnConnectionOptions;
    /**
     * <p>The static routes associated with the VPN connection.</p>
     */
    Routes?: VpnStaticRoute[];
    /**
     * <p>The current state of the VPN connection.</p>
     */
    State?: VpnState | string;
    /**
     * <p>Any tags assigned to the VPN connection.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The ID of the transit gateway associated with the VPN connection.</p>
     */
    TransitGatewayId?: string;
    /**
     * <p>The type of VPN connection.</p>
     */
    Type?: GatewayType | string;
    /**
     * <p>Information about the VPN tunnel.</p>
     */
    VgwTelemetry?: VgwTelemetry[];
    /**
     * <p>The ID of the VPN connection.</p>
     */
    VpnConnectionId?: string;
    /**
     * <p>The ID of the virtual private gateway at the AWS side of the VPN connection.</p>
     */
    VpnGatewayId?: string;
}
export declare namespace VpnConnection {
    const filterSensitiveLog: (obj: VpnConnection) => any;
    const isa: (o: any) => o is VpnConnection;
}
/**
 * <p>Describes VPN connection options.</p>
 */
export interface VpnConnectionOptions {
    __type?: "VpnConnectionOptions";
    /**
     * <p>Indicates whether acceleration is enabled for the VPN connection.</p>
     */
    EnableAcceleration?: boolean;
    /**
     * <p>Indicates whether the VPN connection uses static routes only. Static routes must be used for devices that don't support BGP.</p>
     */
    StaticRoutesOnly?: boolean;
    /**
     * <p>Indicates the VPN tunnel options.</p>
     */
    TunnelOptions?: TunnelOption[];
}
export declare namespace VpnConnectionOptions {
    const filterSensitiveLog: (obj: VpnConnectionOptions) => any;
    const isa: (o: any) => o is VpnConnectionOptions;
}
/**
 * <p>Describes VPN connection options.</p>
 */
export interface VpnConnectionOptionsSpecification {
    __type?: "VpnConnectionOptionsSpecification";
    /**
     * <p>Indicate whether to enable acceleration for the VPN connection.</p>
     *         <p>Default: <code>false</code>
     *          </p>
     */
    EnableAcceleration?: boolean;
    /**
     * <p>Indicate whether the VPN connection uses static routes only. If you are creating a
     *             VPN connection for a device that does not support BGP, you must specify
     *                 <code>true</code>. Use <a>CreateVpnConnectionRoute</a> to create a static route.</p>
     * 		       <p>Default: <code>false</code>
     *          </p>
     */
    StaticRoutesOnly?: boolean;
    /**
     * <p>The tunnel options for the VPN connection.</p>
     */
    TunnelOptions?: VpnTunnelOptionsSpecification[];
}
export declare namespace VpnConnectionOptionsSpecification {
    const filterSensitiveLog: (obj: VpnConnectionOptionsSpecification) => any;
    const isa: (o: any) => o is VpnConnectionOptionsSpecification;
}
export declare type VpnEcmpSupportValue = "disable" | "enable";
/**
 * <p>Describes a virtual private gateway.</p>
 */
export interface VpnGateway {
    __type?: "VpnGateway";
    /**
     * <p>The private Autonomous System Number (ASN) for the Amazon side of a BGP session.</p>
     */
    AmazonSideAsn?: number;
    /**
     * <p>The Availability Zone where the virtual private gateway was created, if applicable. This field may be empty or not returned.</p>
     */
    AvailabilityZone?: string;
    /**
     * <p>The current state of the virtual private gateway.</p>
     */
    State?: VpnState | string;
    /**
     * <p>Any tags assigned to the virtual private gateway.</p>
     */
    Tags?: Tag[];
    /**
     * <p>The type of VPN connection the virtual private gateway supports.</p>
     */
    Type?: GatewayType | string;
    /**
     * <p>Any VPCs attached to the virtual private gateway.</p>
     */
    VpcAttachments?: VpcAttachment[];
    /**
     * <p>The ID of the virtual private gateway.</p>
     */
    VpnGatewayId?: string;
}
export declare namespace VpnGateway {
    const filterSensitiveLog: (obj: VpnGateway) => any;
    const isa: (o: any) => o is VpnGateway;
}
export declare type VpnProtocol = "openvpn";
export declare type VpnState = "available" | "deleted" | "deleting" | "pending";
/**
 * <p>Describes a static route for a VPN connection.</p>
 */
export interface VpnStaticRoute {
    __type?: "VpnStaticRoute";
    /**
     * <p>The CIDR block associated with the local subnet of the customer data center.</p>
     */
    DestinationCidrBlock?: string;
    /**
     * <p>Indicates how the routes were provided.</p>
     */
    Source?: VpnStaticRouteSource | string;
    /**
     * <p>The current state of the static route.</p>
     */
    State?: VpnState | string;
}
export declare namespace VpnStaticRoute {
    const filterSensitiveLog: (obj: VpnStaticRoute) => any;
    const isa: (o: any) => o is VpnStaticRoute;
}
export declare type VpnStaticRouteSource = "Static";
/**
 * <p>The tunnel options for a VPN connection.</p>
 */
export interface VpnTunnelOptionsSpecification {
    __type?: "VpnTunnelOptionsSpecification";
    /**
     * <p>The number of seconds after which a DPD timeout occurs.</p>
     *         <p>Constraints: A value between 0 and 30.</p>
     *         <p>Default: <code>30</code>
     *          </p>
     */
    DPDTimeoutSeconds?: number;
    /**
     * <p>The IKE versions that are permitted for the VPN tunnel.</p>
     *         <p>Valid values: <code>ikev1</code> | <code>ikev2</code>
     *          </p>
     */
    IKEVersions?: IKEVersionsRequestListValue[];
    /**
     * <p>One or more Diffie-Hellman group numbers that are permitted for the VPN tunnel for phase 1 IKE negotiations.</p>
     *         <p>Valid values: <code>2</code> | <code>14</code> | <code>15</code> | <code>16</code> | <code>17</code> | <code>18</code> | <code>22</code> | <code>23</code> | <code>24</code>
     *          </p>
     */
    Phase1DHGroupNumbers?: Phase1DHGroupNumbersRequestListValue[];
    /**
     * <p>One or more encryption algorithms that are permitted for the VPN tunnel for phase 1 IKE negotiations.</p>
     *         <p>Valid values: <code>AES128</code> | <code>AES256</code>
     *          </p>
     */
    Phase1EncryptionAlgorithms?: Phase1EncryptionAlgorithmsRequestListValue[];
    /**
     * <p>One or more integrity algorithms that are permitted for the VPN tunnel for phase 1 IKE negotiations.</p>
     *         <p>Valid values: <code>SHA1</code> | <code>SHA2-256</code>
     *          </p>
     */
    Phase1IntegrityAlgorithms?: Phase1IntegrityAlgorithmsRequestListValue[];
    /**
     * <p>The lifetime for phase 1 of the IKE negotiation, in seconds.</p>
     *         <p>Constraints: A value between 900 and 28,800.</p>
     *         <p>Default: <code>28800</code>
     *          </p>
     */
    Phase1LifetimeSeconds?: number;
    /**
     * <p>One or more Diffie-Hellman group numbers that are permitted for the VPN tunnel for phase 2 IKE negotiations.</p>
     *         <p>Valid values: <code>2</code> | <code>5</code> | <code>14</code> | <code>15</code> | <code>16</code> | <code>17</code> | <code>18</code> | <code>22</code> | <code>23</code> | <code>24</code>
     *          </p>
     */
    Phase2DHGroupNumbers?: Phase2DHGroupNumbersRequestListValue[];
    /**
     * <p>One or more encryption algorithms that are permitted for the VPN tunnel for phase 2 IKE negotiations.</p>
     *         <p>Valid values: <code>AES128</code> | <code>AES256</code>
     *          </p>
     */
    Phase2EncryptionAlgorithms?: Phase2EncryptionAlgorithmsRequestListValue[];
    /**
     * <p>One or more integrity algorithms that are permitted for the VPN tunnel for phase 2 IKE negotiations.</p>
     *         <p>Valid values: <code>SHA1</code> | <code>SHA2-256</code>
     *          </p>
     */
    Phase2IntegrityAlgorithms?: Phase2IntegrityAlgorithmsRequestListValue[];
    /**
     * <p>The lifetime for phase 2 of the IKE negotiation, in seconds.</p>
     *         <p>Constraints: A value between 900 and 3,600. The value must be less than the value for <code>Phase1LifetimeSeconds</code>.</p>
     *         <p>Default: <code>3600</code>
     *          </p>
     */
    Phase2LifetimeSeconds?: number;
    /**
     * <p>The pre-shared key (PSK) to establish initial authentication between the virtual private
     * 			gateway and customer gateway.</p>
     * 		       <p>Constraints: Allowed characters are alphanumeric characters, periods (.), and underscores
     *             (_). Must be between 8 and 64 characters in length and cannot start with zero
     *             (0).</p>
     */
    PreSharedKey?: string;
    /**
     * <p>The percentage of the rekey window (determined by <code>RekeyMarginTimeSeconds</code>) during which the rekey time is randomly selected.</p>
     *         <p>Constraints: A value between 0 and 100.</p>
     *         <p>Default: <code>100</code>
     *          </p>
     */
    RekeyFuzzPercentage?: number;
    /**
     * <p>The margin time, in seconds, before the phase 2 lifetime expires, during which the AWS side of the VPN connection performs an IKE rekey. The exact time of the rekey is randomly selected based on the value for <code>RekeyFuzzPercentage</code>.</p>
     *         <p>Constraints: A value between 60 and half of <code>Phase2LifetimeSeconds</code>.</p>
     *         <p>Default: <code>540</code>
     *          </p>
     */
    RekeyMarginTimeSeconds?: number;
    /**
     * <p>The number of packets in an IKE replay window.</p>
     *         <p>Constraints: A value between 64 and 2048.</p>
     *         <p>Default: <code>1024</code>
     *          </p>
     */
    ReplayWindowSize?: number;
    /**
     * <p>The range of inside IP addresses for the tunnel. Any specified CIDR blocks must be unique
     * 			across all VPN connections that use the same virtual private gateway. </p>
     * 		       <p>Constraints: A size /30 CIDR block from the <code>169.254.0.0/16</code> range. The
     * 			following CIDR blocks are reserved and cannot be used:</p>
     * 		       <ul>
     *             <li>
     * 				           <p>
     *                   <code>169.254.0.0/30</code>
     *                </p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>169.254.1.0/30</code>
     *                </p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>169.254.2.0/30</code>
     *                </p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>169.254.3.0/30</code>
     *                </p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>169.254.4.0/30</code>
     *                </p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>169.254.5.0/30</code>
     *                </p>
     * 			         </li>
     *             <li>
     * 				           <p>
     *                   <code>169.254.169.252/30</code>
     *                </p>
     * 			         </li>
     *          </ul>
     */
    TunnelInsideCidr?: string;
}
export declare namespace VpnTunnelOptionsSpecification {
    const filterSensitiveLog: (obj: VpnTunnelOptionsSpecification) => any;
    const isa: (o: any) => o is VpnTunnelOptionsSpecification;
}
export interface WithdrawByoipCidrRequest {
    __type?: "WithdrawByoipCidrRequest";
    /**
     * <p>The address range, in CIDR notation.</p>
     */
    Cidr: string | undefined;
    /**
     * <p>Checks whether you have the required permissions for the action, without actually making the request,
     *    and provides an error response. If you have the required permissions, the error response is <code>DryRunOperation</code>.
     *    Otherwise, it is <code>UnauthorizedOperation</code>.</p>
     */
    DryRun?: boolean;
}
export declare namespace WithdrawByoipCidrRequest {
    const filterSensitiveLog: (obj: WithdrawByoipCidrRequest) => any;
    const isa: (o: any) => o is WithdrawByoipCidrRequest;
}
export interface WithdrawByoipCidrResult {
    __type?: "WithdrawByoipCidrResult";
    /**
     * <p>Information about the address pool.</p>
     */
    ByoipCidr?: ByoipCidr;
}
export declare namespace WithdrawByoipCidrResult {
    const filterSensitiveLog: (obj: WithdrawByoipCidrResult) => any;
    const isa: (o: any) => o is WithdrawByoipCidrResult;
}
